#!/bin/sh
# Copyright (C) 2014, The Meme Factory, Inc.  http://www.meme.com/
#
#    This file is part of Babase.
#
#    Babase is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
#
# Syntax: compare_pg_backups bk1 bk2
#
# Remarks:
#   Compares backups made by pg_dump (made with other than -Fp).
#
# Bugs:
#   Does not work when dump file paths, db schema names, or db table names
# have spaces in them.
#
#   Does not compare content of a table named "DATA" (in all caps).
#
# Karl O. Pinc <kop@meme.com>
#

export tfile1=/tmp/compare_pg_backups1.$$
export tfile2=/tmp/compare_pg_backups2.$$

# Parse
export backup1=$1
export backup2=$2

cleanup () {
  rm -f $tfile1 $tfile2
}

get_schema_info () {
  backup=$1

  pg_restore -l $backup \
    | awk -F ' ' '{if ($4 == "TABLE" && $5 != "DATA") print $5, $6;}' \
    | sort
}

do_cmp () {
  msg="$1"
  if ! cmp $tfile1 $tfile2; then
    echo "$msg" >&2
    exit 1
  fi
}

dump_table () {
  backup=$1
  schema=$2
  table=$3

  pg_restore -n $schema -t $table $backup | sort
}

cmp_table () {
  schema=$1
  table=$2

  dump_table $backup1 $schema $table > $tfile1
  dump_table $backup2 $schema $table > $tfile2

  do_cmp "Failed compare of schema $schema, table $table"
}

trap cleanup EXIT,HUP,INT,TERM

# Compare the schemas
pg_restore --schema-only $backup1 > $tfile1
pg_restore --schema-only $backup2 > $tfile2

do_cmp "Schemas differ"

# Compare the files

# Get a list of file/schema pairs
get_schema_info $backup1 > $tfile1
get_schema_info $backup2 > $tfile2

do_cmp "List of tables differ"

get_schema_info $backup1 \
  | while read schema table ; do
      cmp_table $schema $table
    done
