# Copyright (C) 2014, 2015 The Meme Factory, Inc.  http://www.meme.com/
#
#    This file is part of Babase.
#
#    Babase is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
#
# Karl O. Pinc <kop@meme.com>
#
# Remarks:
#   Utility library.
#
# Bugs:

import os, sys
from sqlalchemy.exc import IntegrityError
from babase.lib.orm import Analysis

def std_error(msg):
    '''Write to stderr.'''
    sys.stderr.write(msg)

def data_error(msg):
    '''Exit with an error regards data'''
    sys.exit('{0}: {1}'.format(os.path.basename(sys.argv[0]), msg))

def report_keyboard_interrupt():
    '''Exit on keyboard interrupt.'''
    data_error('Aborted: Received keyboard interrupt')

def make_file_error(fname):
    '''Return a function that given a message, raises an error reporting
    ``fname`` as the name of the file associated with the error.
    '''
    def file_error(msg):
        '''Report file name and exit with error'''
        data_error('{0}: {1}'.format(fname, msg))
    return file_error

def analysis_name(args, path, suffix=None):
    '''Return an analysis.name value, defaulting to the
    basename of the ``path`` without the ``suffix``.

    Parameters:
    
    ``args``
    An argparse parsed argument object, with a analysis_name attribute.

    ``path``
    The pathname supplied to the program.

    ``suffix``
    A suffix to remove from the basename of the ``path``.
    ``suffix`` must be in lower case.  Suffix matching is case-insensitive.
    '''
    if args.analysis_name == None:
        fname = os.path.basename(path)
        parts = fname.split('.')
        if suffix != None and  parts[-1].lower() == suffix:
            return '.'.join(parts[:-1])
        else:
            return fname
    else:
        return args.analysis_name

def make_analysis(args, name, term_obj):
    '''Return an SQLAlchemy analysis instance.'''
    if term_obj == None:
        term = None
    else:
        term = term_obj.cvterm_id

    analysis = Analysis()
    analysis.name = name
    analysis.description    = args.analysis_description
    analysis.program        = args.analysis_program
    analysis.programversion = args.analysis_programversion
    analysis.algorithm      = args.analysis_algorithm
    analysis.sourcename     = args.analysis_sourcename
    analysis.sourceversion  = args.analysis_sourceversion
    analysis.sourceuri      = args.analysis_sourceuri
    analysis.timeexecuted   = args.analysis_timeexecuted
    analysis.type_id        = term

    return analysis

def save_analysis(err_func, session, analysis):
    '''Save the sqlalchemy Analysis object, ``analysis`` in the db.'''
    session.add(analysis)
    try:
        session.flush()
    except IntegrityError as err:
        err_func(('Invalid ANALYSIS data:\n{0}\n'
                  'Hint: Is the ANALYSIS.Name value already in the database?')
                 .format(err))
