dnl Copyright (C) 2012, 2015 The Meme Factory, Inc.  http://www.meme.com/
dnl Copyright (C) 2004-2006, 2008, 2009-2011 Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl
include(`copyright.m4')
include(`constants.m4')

-- Run a bunch of queries reporting problems in the database.
--
-- Copyright (C) 2012 The Meme Factory, Inc.  http://www.meme.com/
-- GPL_notice(`  --', `2004-2006, 2008-2010', `Karl O. Pinc <kop@meme.com>')

--
-- BIOGRAPH
--

-- Error condition
--
-- Can't be born before the last time the mother was seen.
--
SELECT mom.bioid AS mom_bioid
     , mom.sname AS mom
     , mom.statdate AS mom_statdate
     , kid.pid
     , kid.bioid AS kid_bioid
     , kid.sname AS kid
     , kid.birth AS kid_birth
     , 'Error: Kid born after last observervation of mother'
    FROM biograph AS kid
         JOIN pregs ON (pregs.pid = kid.pid)
         JOIN cycpoints ON (cycpoints.cpid = pregs.conceive)
         JOIN cycles ON (cycles.cid = cycpoints.cid)
         JOIN biograph AS mom ON (mom.sname = cycles.sname)
    WHERE mom.statdate < kid.birth
    ORDER BY mom.statdate, mom.sname;


--
-- MATUREDATES
--

-- Warning condition
-- It's unusual to have a mature date within 3 years of the birthdate,
-- although 2 lodge females have managed it.
--SELECT biograph.sname, birth, matured
--       , 'Warning: Matdate within 3 years of birthdate.'
--  FROM biograph, maturedates
--  WHERE biograph.sname = maturedates.sname
--        AND birth >= matured - CAST('3 years' AS interval)
--  ORDER BY biograph.sname;
--
-- Re-written for the warning system:
SELECT '(' || biograph.sname || ') Matured (' || matured
       || ') within 3 years of birthdate (' || birth || ')'
       AS id
     , 'Matdate within 3 years of birthdate: Sname (' || biograph.sname
       || '): MATUREDATES.Matured (' || matured
       || '): Birth (' || birth || ')'
       AS msg
  FROM biograph, maturedates
  WHERE biograph.sname = maturedates.sname
        AND birth >= matured - CAST('3 years' AS interval);

-- Warning condition
-- It's unusual to have a mature date more than 7 years after the birthdate,
-- although ALE the immigrant anubis male has managed it.
SELECT biograph.sname, birth, matured
       , 'Warning: Matdate 7 years or more after birthdate.'
  FROM biograph, maturedates
  WHERE biograph.sname = maturedates.sname
        AND birth <= matured - CAST ('7 years' AS interval)
  ORDER BY biograph.sname;

--
-- CONSORTDATES
---
SELECT consortdates.sname, consortdates.consorted, biograph.birth
       , 'Warning: CONSORTDATES.Consorted is bb_max_rankdate_age or more years after birthdate'
  FROM biograph, consortdates
  WHERE biograph.sname = consortdates.sname
        AND biograph.birth <= consortdates.consorted 
                              - CAST ('bb_max_rankdate_age years' AS interval)
  ORDER BY biograph.sname;

--
-- DISPERSEDATES
--

-- Warning condition
-- Dispersal before maturity.
-- Rewritten for warning system:
--SELECT dispersedates.sname, dispersedates.dispersed, maturedates.matured
--       , 'Warning: Dispersed before maturity'
--  FROM dispersedates LEFT OUTER JOIN maturedates
--       ON (maturedates.sname = dispersedates.sname)
--  WHERE maturedates.matured IS NULL
--        OR dispersedates.dispersed < maturedates.matured
--  ORDER BY dispersedates.sname;
SELECT '(' || dispersedates.sname
       || ') Dispersed (' || dispersedates.dispersed
       || ') before maturity ('
       || COALESCE(maturedates.matured::TEXT, 'NULL') || ')'
       AS id
     , 'Dispersed before maturity: Sname (' || dispersedates.sname
       || ': Dispersed (' || dispersedates.dispersed
       || ': Matured (' || COALESCE(maturedates.matured::TEXT, 'NULL') || ')'
       AS msg
  FROM dispersedates LEFT OUTER JOIN maturedates
       ON (maturedates.sname = dispersedates.sname)
  WHERE maturedates.matured IS NULL
        OR dispersedates.dispersed < maturedates.matured;


--
-- Before we get into the sexual cycle stuff, check that the
-- cycles table has Seq and Series that start with 1 and contain
-- no gaps.
--

-- Error condition
-- CYCLES.Seq must begin with 1
SELECT cycles.sname, MIN(cycles.seq)
       , 'Error: CYCLES.Seq must begin with 1' AS "Error"
  FROM cycles
  WHERE cycles.sname NOT IN (SELECT c.sname
                               FROM cycles AS c
                               WHERE c.sname = cycles.sname
                                     AND c.seq = 1)
  GROUP BY cycles.sname
  ORDER BY cycles.sname;

-- Error condition
-- CYCLES.Series must begin with 1
SELECT cycles.sname, MIN(cycles.series)
       , 'Error: CYCLES.Series must begin with 1' AS "Error"
  FROM cycles
  WHERE cycles.sname NOT IN (SELECT c.sname
                               FROM cycles AS c
                               WHERE c.sname = cycles.sname
                                     AND c.series = 1)
  GROUP BY cycles.sname
  ORDER BY cycles.sname;

-- Error condition
-- CYCLES.Seq must not contain gaps in it's sequence progression
SELECT cycles.sname, cycles.seq
       , 'Error: This CYCLES.Seq is preceded by a "gap" in sequence progression'
  FROM cycles
  WHERE NOT EXISTS (SELECT 1
                      FROM cycles AS c
                      WHERE c.sname = cycles.sname
                            AND c.seq = cycles.seq - 1
                      LIMIT 1)
        AND cycles.seq > 1
  ORDER BY cycles.sname, cycles.seq;

-- Error condition
-- CYCLES.Series must not contain gaps in it's sequence progression
SELECT cycles.sname, cycles.series
       , 'Error: This CYCLES.Series is preceded by a "gap" in sequence progression'
  FROM cycles
  WHERE NOT EXISTS (SELECT 1
                      FROM cycles AS c
                      WHERE c.sname = cycles.sname
                            AND c.series = cycles.series - 1
                      LIMIT 1)
        AND cycles.series > 1
  ORDER BY cycles.sname, cycles.series;

--
-- PREGS
--

-- Not needed because we now generate the parity
-- -- Error condition
-- -- Parity, modulo 100, must be contiguous.
-- SELECT pid, parity
--        , 'Error: Parity out of order'
--   FROM pregs outerpregs
--      , cycpoints as outercycpoints
--      , cycles as outercycles
--   WHERE parity % 100 <> 1
--         AND outerpregs.conceive = outercycpoints.cpid
--         AND outercycles.cid = outercycpoints.cid
--         AND NOT EXISTS
--           (SELECT 1
--              FROM pregs AS innerpregs
--                 , cycpoints as innercycpoints
--                 , cycles AS innercycles
--              WHERE innercycles.sname = outercycles.sname
--                    AND innercycpoints.cid = innercycles.cid
--                    AND innerpregs.conceive = innercycpoints.cpid
--                    AND outerpregs.parity % 100 = innerpregs.parity % 100 + 1
--              LIMIT 1)
--   ORDER BY pid, parity;

-- Not needed, checked for in the triggers.
-- -- Warning condition
-- -- resumption of menses cycle should not have a mdate.
-- SELECT devent.sname, pregs.parity, pregs.pid
--        , mevent.date, mevent.edate, mevent.ldate
--        , 'Warning: Mdate on first cycle after pregnancy'
--   FROM pregs, cycpoints devent, cycpoints mevent
--   WHERE pregs.resume = devent.cpid
--         AND devent.cid = mevent.cid
--         AND mevent.code = 'bb_menses'
--   ORDER BY devent.sname, pregs.parity;

-- We check below that you can't cycle while pregnant.  
-- -- Warning condition
-- -- More than one pregs/female with no resume.
-- SELECT conception.sname, bad_pregs.pid, bad_pregs.parity
--       , 'Warning: more than one PREGS row/female with no resume cycle'
--   FROM pregs AS bad_pregs, pregs AS other_pregs
--        , cycpoints AS conception, cycpoints AS bad_conception
--   WHERE other_pregs.conceive = conception.cpid
--         AND bad_pregs.conceive = bad_conception.cpid
--         AND bad_conception.sname = conception.sname
--         AND bad_pregs.resume IS NULL
--         AND other_pregs.resume IS NULL
--         AND bad_pregs.oid <> other_pregs.oid
--   ORDER BY conception.sname, bad_pregs.pid, bad_pregs.parity;
-- 
-- -- Warning condition
-- -- pregs with no resume must have largest Tdate for that female.
-- SELECT conception.sname, bad_pregs.pid, bad_pregs.parity, bad_pregs.date
--       , 'Warning: PREGS row with no resume cycle that is not the last pregnancy (by Ddate)'
--   FROM pregs AS bad_pregs, pregs AS other_pregs
--        , cycpoints AS conception, cycpoints AS bad_conception
--   WHERE other_pregs.conceive = conception.cpid
--         AND bad_pregs.conceive = bad_conception.cpid
--         AND bad_conception.sname = conception.sname
--         AND bad_pregs.resume IS NULL
--         AND bad_conception.date <= conception.date
--         AND bad_pregs.oid <> other_pregs.oid
--   ORDER BY conception.sname, bad_pregs.pid, bad_pregs.parity;
-- 
-- -- pregs with no resume must have largest parity for that female.
-- SELECT conception.sname, bad_pregs.pid, bad_pregs.parity, bad_pregs.date
--       , 'Warning: PREGS row with no resume cycle that is not the last pregnancy (by parity)'
--   FROM pregs AS bad_pregs, pregs AS other_pregs
--        , cycpoints AS conception, cycpoints AS bad_conception
--   WHERE other_pregs.conceive = conception.cpid
--         AND bad_pregs.conceive = bad_conception.cpid
--         AND bad_conception.sname = conception.sname
--         AND bad_pregs.resume IS NULL
--         AND bad_pregs.parity <= other_pregs.parity
--         AND bad_pregs.oid <> other_pregs.oid
--   ORDER BY conception.sname, bad_pregs.pid, bad_pregs.parity;

-- Warning condition
-- Pregs with no T date in the conception cycle will not
-- have the conception cycle referenced in the repstats rows
-- for the cycling period preceding the pregnancy.
-- (This query might benefit from use of the cycgapdays table.)
--       , 'Warning: PREGS conception cycle has no T date (but there is no gap) so REPSTATS rows recording the cycling state before the pregnancy do not reference the conception cycle'
SELECT dcycle.sname, pregs.pid, pregs.conceive
       , 'Warning: PREGS conception cycle has no T date, and is not preceded by a gap'
  FROM pregs, cycpoints AS devent, cycles AS dcycle
  WHERE pregs.conceive = devent.cpid
        AND devent.source <> 'bb_estimated'
        AND dcycle.cid = devent.cid
        AND NOT EXISTS 
          (SELECT 1 FROM cycpoints AS tevent
                  WHERE tevent.cid = devent.cid
                        AND tevent.code = 'bb_turgesence')
        AND NOT EXISTS
          (SELECT 1 FROM cycgaps
                    WHERE cycgaps.sname = dcycle.sname
                          AND cycgaps.code = 'bb_cg_end'
                          AND cycgaps.date >= (SELECT prior.date
                                         FROM cycpoints AS prior
                                              , cycles AS pcycles
                                         WHERE pcycles.sname = dcycle.sname
                                               AND prior.cid = pcycles.cid
                                               AND prior.date < devent.date
                                         ORDER BY prior.date DESC
                                         LIMIT 1)
                          AND cycgaps.date < devent.date)
        AND NOT EXISTS
          (SELECT 1 FROM cycgaps
                    WHERE cycgaps.sname = dcycle.sname
                          AND cycgaps.code = 'bb_cg_point'
                          AND cycgaps.date = (SELECT prior.date
                                         FROM cycpoints AS prior
                                             , cycles AS pcycles
                                         WHERE pcycles.sname = dcycle.sname
                                               AND prior.cid = pcycles.cid
                                               AND prior.date < devent.date
                                         ORDER BY prior.date DESC
                                         LIMIT 1))
  ORDER BY dcycle.sname, pregs.pid;

-- Warning condition
-- Pregnancy exceeds bb_preg_len_warn days.
-- (This query could probably benefit by using the cycgapdays table.)
SELECT pregs.pid
       , 'Warning: Pregnancy exceeds bb_preg_len_warn days (and there are no gaps in observation)'
  FROM pregs, biograph AS mom, cycpoints, cycles
  WHERE pregs.resume IS NULL
        AND NOT EXISTS (SELECT 1
                          FROM biograph AS child
                          WHERE child.pid = pregs.pid)
        AND cycpoints.cpid = pregs.conceive
        AND cycles.cid = cycpoints.cid
        AND mom.sname = cycles.sname
        AND mom.statdate >= cycpoints.date + CAST('bb_preg_len_warn days' AS interval)
        AND NOT EXISTS (SELECT 1
                          FROM cycgaps
                          WHERE cycgaps.sname = cycles.sname
                                AND cycgaps.state = 'bb_cg_start'
                                AND cycgaps.date > cycpoints.date
                                AND cycgaps.date <= cycpoints.date
                                            + CAST('bb_preg_len_warn days' AS interval))
        AND NOT EXISTS (SELECT 1
                          FROM cycgaps
                          WHERE cycgaps.sname = cycles.sname
                                AND cycgaps.state = 'bb_cg_end'
                                AND cycgaps.date >= cycpoints.date
                                AND cycgaps.date < cycpoints.date
                                           + CAST('bb_preg_len_warn days' AS interval))
  ORDER BY pregs.pid;

--
-- Error condition
-- You can't cycle while pregnant
--
-- No need for this test, triggers now take care of it.
--SELECT pregs.pid, later.date, later.code
--       , 'Error: Female cycling while pregnant'
--  FROM pregs, cycpoints AS c, cycpoints AS later
--       , cycles AS ccycle, cycles AS lcycle
--  WHERE pregs.resume IS NULL
--        AND c.cpid = pregs.conceive
--        AND ccycle.cid = c.cid
--        AND lcycle.sname = ccycle.sname
--        AND later.cid = ccycle.cid
--        AND later.date > c.date
--        AND later.code = 'bb_turgesence'
--        AND NOT EXISTS
--         (SELECT 1
--            FROM cycgaps
--            WHERE cycgaps.sname = ccycle.sname
--                  AND cycgaps.code = 'bb_cg_end'
--                  AND cycgaps.date >= c.date)
--        AND NOT EXISTS
--         (SELECT 1
--            FROM cycgaps
--            WHERE cycgaps.sname = ccycle.sname
--                  AND cycgaps.code = 'bb_cg_point'
--                  AND cycgaps.date = c.date)
--  ORDER BY pregs.pid, later.date, later.code;

--
-- Error condition, can't have no observation between
-- conception and resumption cycle.
--
SELECT pregs.pid, c.date AS zdate, r.date AS resume
     , 'Error: Gap in observation between conception and resumption cycles' AS Error
  FROM pregs, cycpoints AS c, cycpoints AS r, cycles
  WHERE c.cpid = pregs.conceive
        AND r.cpid = pregs.resume
        AND cycles.cid = c.cid
        AND EXISTS (SELECT 1
                      FROM cycgapdays
                      WHERE cycgapdays.sname = cycles.sname
                            AND cycgapdays.date > c.date
                            AND cycgapdays.date < r.date);


--
-- Error condition, conception cycle must be one less than
-- resumption cycle.
-- This is pretty much the same test as above, but why not?
-- Note that the PREGS triggers check this but the
-- CYCLES triggers cannot, so sometimes the user gets
-- the error right away but sometimes they must wait for 
-- the batch error checking.
--
SELECT pregs.pid, ccycle.seq AS cseq, rcycle.seq AS rseq
     , 'Error: Conception cycle Seq must be one less than the resumption cycle Seq'
  FROM pregs
     , cycpoints AS conceive
     , cycpoints AS resume
     , cycles AS ccycle
     , cycles AS rcycle
  WHERE conceive.cpid = pregs.conceive
        AND ccycle.cid = conceive.cid
        AND resume.cpid = pregs.resume
        AND rcycle.cid = resume.cid
        AND rcycle.seq <> ccycle.seq + 1;

--
-- CYCPOINTS
--

--
-- Must have Mdate, Tdate, Ddate order, unless there's a gap.
--
-- Test prior cycpoint (on this side of any gaps.)
-- (The freaky NOT() constructs are to deal with the 3 valued
-- logic we get when NULLs are returned from the outer join.)
--
-- Bug: This does not account for pregnancies, or maturity date,
-- and the SQL seems broken anyhow.   Probably needs a complete rewrite.
SELECT this_cycles.sname
     , prior_cycpoints.date AS "Earlier date"
     , prior_cycpoints.code AS "Earlier code"
     , this_cycpoints.date AS "Later date"
     , this_cycpoints.code AS "Later code"
     , 'Error: CYCPOINTS must be in Mdate, Tdate, Ddate order unless there is a gap'
  FROM cycpoints AS this_cycpoint
    JOIN cycles AS this_cycle
      ON (this_cycle.cid = this_cycpoint.cid)
    CROSS JOIN (SELECT sub.date, sub.code
                  FROM cycles AS prior_cycles
                    LEFT OUTER JOIN cycgaps
                      ON (cycgaps.sname = this_cycle.sname
                          AND cycgaps.code = 'bb_cg_end'
                          AND cycgaps.date < this_cycpoint.date)
                    JOIN cycpoints AS sub
                      ON (sub.cid = prior_cycles.cid
                          AND sub.code <> 'bb_bleeding'
                          AND ((this_cycpoint.code = 'bb_turgesence'
                                -- M and Tdates can be on the same date
                                AND sub.date <= this_cycpoint.date
                                AND sub.cpid <> this_cycpoint.cpid)
                               OR (this_cycpoint.code <> 'bb_turgesence'
                                   AND sub.date < this_cycpoint.date))
                          AND NOT(sub.date <= cycgaps.date))
                  WHERE prior_cycles.sname = this_cycle.sname
                  ORDER BY sub.date DESC
                  LIMIT 1) AS prior_cycpoints
  WHERE this_cycpoint.code <> 'bb_bleeding'
        AND ((this_cycpoint.code = 'bb_menses'
              AND prior_cycpoints.code <> 'bb_deturgesence')
             OR (this_cycpoint.code = 'bb_turgesence'
                 AND prior_cycpoints.code <> 'bb_menses')
             OR (this_cycpoint.code = 'bb_deturgesence'
                 AND prior_cycpoints.code <> 'bb_turgesence'))
  ORDER BY this_cycle.sname, this_cycpoint.date;

-- Warning condition
-- No cycpoints on a mature female
SELECT maturedates.sname, maturedates.matured
       , 'Warning: Mature female has no Mdate, Tdate, or Ddate CYCPOINTS rows'
  FROM maturedates, biograph
  WHERE biograph.sex = 'bb_female'
        AND maturedates.sname = biograph.sname
        AND NOT EXISTS (SELECT 1
                    FROM cycpoints, cycles
                    WHERE cycles.sname = biograph.sname
                          AND cycpoints.cid = cycles.cid
                          AND cycpoints.code <> 'bb_bleeding'
                    LIMIT 1)
  ORDER BY maturedates.sname;

-- Error condition
-- When Tdates are the first sexual cycle event that occurs after
-- (but not on) the beginning of observation, then the cycgaps
-- state at the beginning of observation must be M, P, or L.
-- (This one is important to debug if it's not working.)
SELECT cycgaps.sname, cycgaps.date, cycgaps.state
     , cycpoints.code, cycpoints.date
     , 'Error: The CYCGAPS.State does not correspond with having a Tdate as the first CYCPOINTS in the series'
  FROM cycgaps, cycles, cycpoints
  WHERE cycgaps.code = 'bb_cg_start'
        AND cycgaps.state <> 'bb_cg_menses'
        AND cycgaps.state <> 'bb_cg_pregnant'
        AND cycgaps.state <> 'bb_cg_lactating'
        AND cycles.sname = cycgaps.sname
        AND cycpoints.cid = cycles.cid
        AND cycpoints.code = 'bb_turgesence'
        AND cycpoints.date > cycgaps.date
        AND cycpoints.date = (SELECT cp.date
                                FROM cycles AS cy, cycpoints AS cp
                                WHERE cy.sname = cycgaps.sname
                                      AND cy.series = cycles.series
                                      AND cp.cid = cy.cid
                                      AND cp.date >= cycgaps.date
                                ORDER BY cp.date
                                LIMIT 1)
  ORDER BY cycgaps.sname, cycgaps.date;

-- Error condition
-- An estimated Ddate that's not a conception event.
SELECT cycles.sname, cycpoints.date, cycpoints.code, cycpoints.source
     , 'Error: An estimated Ddate that is not a conception event'
  FROM cycles, cycpoints
  WHERE cycpoints.code = 'bb_deturgesence'
        AND cycpoints.source = 'bb_estimated'
        AND cycpoints.cpid NOT IN (SELECT conceive FROM pregs)
        AND cycles.cid = cycpoints.cid
  ORDER BY cycles.sname, cycpoints.date;

-- 
-- CYCLES
--
-- Note that early-on above, before Pregs, we test for Seq and Series
-- validity.  It may be that some of the checks between here and there
-- need valid Seq or Series.
--

-- -- Warning condition
-- -- Female's first sexual cycle should not have a mdate.
-- SELECT sname, cid, mdate, ddate, 'Warning: mdate on first sexual cycle'
--   FROM cycles
--   WHERE seq = 1
--         AND mdate IS NOT NULL
--   ORDER BY sname;

-- -- The update and delete triggers do not check for sequentiality of
-- -- the sequence number.
-- SELECT cid, seq, sname, 'CYCLES.Seq out of order'
--   FROM cycles AS outercycles
--   WHERE seq <> 1
--         AND NOT EXISTS (SELECT innercycles.seq
--                           FROM cycles AS innercycles
--                           WHERE outercycles.sname = innercycles.sname
--                                 AND outercycles.seq = innercycles.seq + 1)
--   ORDER BY sname, seq;

-- Error condition
-- There should be no cycles without cycpoints.
SELECT cid, sname, seq, series
     , 'Error: CYCLES row has no CYCPOINTS'
  FROM CYCLES
  WHERE NOT EXISTS (SELECT 1 FROM cycpoints WHERE cycpoints.cid = cycles.cid)
  ORDER BY sname, seq;

-- Error condition
-- The combination of sname and seq should be unique.
SELECT ca.sname, ca.seq, ca.cid AS "one cid", cb.cid AS "another cid"
     , 'Error: the combination of Sname and Seq on CYCLES should be unique'
  FROM cycles AS ca, cycles AS cb
  WHERE ca.sname = cb.sname
        AND ca.seq = cb.seq
        AND ca.cid <> cb.cid
  ORDER BY ca.sname, ca.seq;

-- Error condition
-- There should be no NULL seqs.
SELECT cid, sname
     , 'Error: CYCLES should not have NULL Seq values'
  FROM cycles
  WHERE seq IS NULL
  ORDER BY sname, cid;

-- Error condition
-- There should be no NULL series.
SELECT cid, sname
     , 'Error: CYCLES should not have NULL Series values'
  FROM cycles
  WHERE series IS NULL
  ORDER BY sname, cid;

--
-- SEXSKINS (and cycles)
--

-- Starting here these checks are done in the sexskins triggers but
-- not elsewhere.  (To "ending here" comment.)

-- Error condition
-- There are sexskins but no Tdate in the cycle.
SELECT cycles.sname, sexskins.date, sexskins.size
     , 'Error: Cannot have SEXSKINS in a cycle with no Tdate'
  FROM cycles, sexskins
  WHERE cycles.cid = sexskins.cid
        AND NOT EXISTS (SELECT 1
                          FROM cycpoints
                          WHERE cycpoints.cid = sexskins.cid
                                AND cycpoints.code = 'bb_turgesence'
                          LIMIT 1)
  ORDER BY cycles.sname, sexskins.date;

-- Error condition
-- There are sexskins but no Mdate in the cycle.
SELECT cycles.sname, sexskins.date, sexskins.size
     , 'Error: Cannot have SEXSKINS in a cycle with no Mdate unless the cycle resumes a pregnancy or is at menarche or there is a gap (CYCGAPS)'
  FROM cycles, sexskins
  WHERE cycles.cid = sexskins.cid
        AND NOT EXISTS (SELECT 1
                          FROM cycpoints
                          WHERE cycpoints.cid = sexskins.cid
                                AND cycpoints.code = 'bb_menses'
                          LIMIT 1)
        AND NOT EXISTS (SELECT 1
                          FROM pregs
                          WHERE pregs.resume = sexskins.cid
                          LIMIT 1)
        AND NOT EXISTS (SELECT 1
                          FROM cycpoints, maturedates
                          WHERE cycpoints.cid = sexskins.cid
                                AND cycpoints.code = 'bb_turgesence'
                                AND maturedates.sname = cycles.sname
                                AND maturedates.matured = cycpoints.date
                                AND maturedates.mstatus = 'bb_on'
                          LIMIT 1)
        AND NOT EXISTS (SELECT 1
                          FROM cycpoints as cp
                               , cycles as cy
                               , cycpoints as thiscp
                          WHERE cy.sname = cycles.sname
                                AND cp.cid = cy.cid
                                AND thiscp.cid = sexskins.cid
                                AND cp.date < thiscp.date
                                AND cp.cid <> sexskins.cid
                                AND cp.date > (SELECT cycgapdays.date
                                        FROM cycgapdays
                                        WHERE cycgapdays.sname = cycles.sname
                                              AND cycgapdays.date < thiscp.date
                                        ORDER BY cycgapdays.date DESC
                                        LIMIT 1))
  ORDER BY cycles.sname, sexskins.date;

-- Error condition
-- There are sexskins with sizes greater than bb_max_skin_before_t
-- before the cycle's Tdate.
SELECT cycles.sname, sexskins.date, sexskins.size, cycpoints.date AS "Tdate"
     , 'Error: Cannot have a SEXSKINS row with a size bigger than bb_max_skin_before_t before the cycle''s Tdate'
  FROM cycles, sexskins, cycpoints
  WHERE cycles.cid = sexskins.cid
        AND cycpoints.cid = sexskins.cid
        AND cycpoints.code = 'bb_turgesence'
        AND sexskins.size > bb_max_skin_before_t
        AND sexskins.date < cycpoints.date
  ORDER BY cycles.sname, sexskins.date;

-- Error condition
-- There are zero sexskin sizes after the Tdate.
SELECT cycles.sname, sexskins.date, sexskins.size, cycpoints.date AS "Tdate"
     , 'Error: Cannot have zero sexskin sizes after the Tdate'
  FROM cycles, cycpoints, sexskins
  WHERE cycles.cid = sexskins.cid
        AND cycpoints.cid = cycles.cid
        AND cycpoints.code = 'bb_turgesence'
        AND sexskins.date >= cycpoints.date
        AND sexskins.size = 0
  ORDER BY cycles.sname, sexskins.date;

-- Error condition
-- There are sexskins with sizes greater than zero
-- before the cycle's Mdate.
SELECT cycles.sname, sexskins.date, sexskins.size, cycpoints.date AS "Mdate"
     , 'Error: Cannot have a SEXSKINS row with a size bigger than 0 before the cycle''s Mdate'
  FROM cycles, sexskins, cycpoints
  WHERE cycles.cid = sexskins.cid
        AND cycpoints.cid = sexskins.cid
        AND cycpoints.code = 'bb_menses'
        AND sexskins.size > 0
        AND cycpoints.date < sexskins.date
  ORDER BY cycles.sname, sexskins.date;

-- Warning condition
-- Sexskin measurements larger than 0 and there is no Mdate.
SELECT cycles.cid, cycles.sname, sexskins.date, sexskins.size
       , 'Warning: Sexskin measurement larger than 0 and there is no Mdate in the cycle and the Tdate is not the maturity date and the cycle is not a resumption of pregnancy'
  FROM cycles, sexskins
  WHERE sexskins.cid = cycles.cid
        AND sexskins.size > 0
        AND NOT EXISTS (SELECT 1
                          FROM cycpoints
                          WHERE cycpoints.cid = sexskins.cid
                                AND cycpoints.code = 'bb_menses'
                          LIMIT 1)
        AND NOT EXISTS (SELECT 1
                          FROM pregs
                          WHERE pregs.resume = sexskins.cid
                          LIMIT 1)
        AND NOT EXISTS (SELECT 1
                          FROM cycpoints, maturedates
                          WHERE cycpoints.cid = sexskins.cid
                                AND cycpoints.code = 'bb_turgesence'
                                AND maturedates.sname = cycles.sname
                                AND maturedates.matured = cycpoints.date
                                AND maturedates.mstatus = 'bb_on'
                          LIMIT 1)
  ORDER BY cycles.sname, sexskins.date;

-- Starting here these checks are done in the sexskins triggers but
-- not elsewhere.  (To "ending here" comment.)

-- Warning condition
-- Sexskin measurement larger than bb_max_skin_before_t before the Tdate.
SELECT cycles.sname, sexskins.date, sexskins.size, cycpoints.date as Tdate
       , 'Warning: Sexskin measurement larger than bb_max_skin_before_t before the Tdate'
  FROM cycpoints, cycles, sexskins
  WHERE cycpoints.code = 'bb_turgesence'
        AND cycles.cid = cycpoints.cid
        AND sexskins.cid = cycpoints.cid
        AND sexskins.date < cycpoints.date
        AND sexskins.size > bb_max_skin_before_t
  ORDER BY cycles.sname, sexskins.date;

-- Warning condition
-- Sexskin measurement larger than 0 on the Mdate.
SELECT cycles.sname, sexskins.date, sexskins.size
       , 'Warning: Sexskin measurement larger than bb_normal_max_skin_on_m on the Mdate'
  FROM cycles, cycpoints, sexskins
  WHERE cycpoints.code = 'bb_menses'
        AND cycles.cid = cycpoints.cid
        AND sexskins.cid = cycpoints.cid
        AND sexskins.date = cycpoints.date
        AND sexskins.size > bb_normal_max_skin_on_m
  ORDER BY cycles.sname, sexskins.date;


-- Warning condition
-- D dates are not normally after the first measurement after
-- the largest measurement of the cycle.
-- SELECT cycles.sname, cycles.cid, cycles.ddate
--        , outersexskins.date, outersexskins.size
--        , 'Warning: D date after the first measurement following the largest sexskin measurement of the cycle'
--     FROM sexskins AS outersexskins, cycles
--     WHERE outersexskins.cid = cycles.cid
--           AND date = (SELECT MIN(middlesexskins.date)
--                         FROM sexskins AS middlesexskins
--                         WHERE middlesexskins.cid = cycles.cid
--                               AND middlesexskins.date >
--                                  (SELECT MAX(innersexskins.date)
--                                     FROM sexskins as innersexskins
--                                     WHERE innersexskins.cid = cycles.cid
--                                           AND innersexskins.size =
--                                               (SELECT MAX(maxsize.size)
--                                                  FROM sexskins AS maxsize
--                                                  WHERE maxsize.cid
--                                                        = cycles.cid)))
--           AND (outersexskins.date < cycles.ddate
--                OR outersexskins.date < cycles.eddate)
--     ORDER BY cycles.sname, cycles.cid;
SELECT cycles.sname, cycles.cid, cycles.ddate
       , outersexskins.date, outersexskins.size
       , 'Warning: D date after the first measurement following the largest sexskin measurement of the cycle'
    FROM sexskins AS outersexskins, cycles
    WHERE outersexskins.cid = cycles.cid
          AND date = (SELECT middlesexskins.date
                        FROM sexskins AS middlesexskins
                        WHERE middlesexskins.cid = cycles.cid
                              AND middlesexskins.date >
                                 (SELECT innersexskins.date
                                    FROM sexskins as innersexskins
                                    WHERE innersexskins.cid = cycles.cid
                                          AND innersexskins.size =
                                              (SELECT maxsize.size
                                                 FROM sexskins AS maxsize
                                                 WHERE maxsize.cid
                                                       = cycles.cid
                                                 ORDER BY maxsize.size DESC
                                                 LIMIT 1)
                                    ORDER BY innersexskins.date DESC
                                    LIMIT 1)
                        ORDER BY middlesexskins.date
                        LIMIT 1)
          AND (outersexskins.date < cycles.ddate
               OR outersexskins.date < cycles.eddate)
    ORDER BY cycles.sname, cycles.cid;

--
-- CENSUS
--

-- Warning condition
-- The first censused group is not the individual's matgrp.
-- SELECT biograph.sname, biograph.birth, biograph.matgrp
--        , date AS "census date", grp AS censusgrp
--        , 'Warning: First non-absent census is not in Matgrp'
--     FROM biograph, census
--     WHERE biograph.sname = census.sname
--           AND census.status <> 'bb_absent'
--           AND date = (SELECT MIN(inn.date)
--                                     FROM census AS inn
--                                     WHERE inn.sname = biograph.sname
--                                           AND inn.status <> 'bb_absent')
--           AND grp <> biograph.matgrp
--     ORDER BY biograph.sname;
SELECT biograph.sname, biograph.birth, biograph.matgrp
       , date AS "census date", grp AS censusgrp
       , 'Warning: First non-absent census is not in (non bb_unk_grp) Matgrp'
    FROM biograph, census
    WHERE biograph.sname = census.sname
          AND biograph.matgrp <> 'bb_unk_grp'
          AND census.status <> 'bb_absent'
          AND date = (SELECT inn.date
                                    FROM census AS inn
                                    WHERE inn.sname = biograph.sname
                                          AND inn.status <> 'bb_absent'
                                    0ORDER BY inn.date
                                    LIMIT 1)
          AND grp <> biograph.matgrp
    ORDER BY biograph.sname;


-- Error condition
-- ''D'' census rows should have a demography note, unless the note
-- is in the process of being entered.
-- Note that this _could_ be accomplished with a DEFERRED CONSTRAINT
-- in the table definition.  I'm not doing it that way partly
-- because this is already done and partly to give people
-- flexibility on the front end so they can enter demography notes without
-- transactions in their front-end.
SELECT census.cenid, census.sname, census.date, census.status
       , 'Error: CENSUS rows with a Status of bb_demog must have an associated DEMOG row'
    FROM census
    WHERE status = 'bb_demog'
          AND cenid NOT IN (SELECT cenid FROM demog)
    ORDER BY census.sname, census.date, census.cenid;

-- Warning condition
-- No census row
SELECT biograph.sname, 'Warning: Individuals with a Sname but no (non absent) CENSUS row'
    FROM biograph
    WHERE sname IS NOT NULL
          AND sname NOT IN (SELECT census.sname
                                   FROM census
                                   WHERE census.sname = biograph.sname
                                         AND census.status <> 'bb_absent')
    ORDER BY sname;

--
-- INTERACT & PARTS
--

-- Error condition
-- Each Interact row should have exactly two corresponding Parts rows
-- unless the to the focal individual of an ad-lib
-- focal point sample is the lone participant.
SELECT iid, act, date, 'Error: INTERACT rows must have exactly 2 corresponding rows on PARTS unless they are collected via ad-lib focal point sampling'
  FROM interact
  WHERE (SELECT count(*)
           FROM parts
           WHERE interact.iid = parts.iid) <> 2
        AND interact.sid IS NULL
  ORDER BY interact.date, interact.act, interact.iid;

SELECT interact_data.iid
     , interact_data.act
     , interact_data.date
     , samples.sname
     , 'Error: INTERACT_DATA rows must have exactly 2 corresponding rows on PARTS unless there is a single participant who is the focal of a related point sample'
  FROM interact_data, samples
  WHERE samples.sid = interact_data.sid
        AND (((SELECT count(*)
                 FROM parts
                 WHERE interact_data.iid = parts.iid) <> 2
              AND (SELECT count(*)
                     FROM parts
                     WHERE interact_data.iid = parts.iid) <> 1)
             OR ((SELECT count(*)
                    FROM parts
                    WHERE interact_data.iid = parts.iid) = 1
                 AND NOT EXISTS (SELECT parts.sname
                                  FROM parts
                                  WHERE parts.sname = samples.sname)))
  ORDER BY interact_data.date
         , interact_data.act
         , samples.sname
         , interact_data.iid;

-- Warning condition
-- Interactions should be between individuals in the same supergroup.
SELECT interact.iid, interact.act, interact.date
       , actor.sname, supergroup(actorgrp.gid, interact.date) as actorsupergroup
       , actee.sname, supergroup(acteegrp.gid, interact.date) as acteesupergroup
       , 'Warning: Interactions should be between individuals in the same supergroup'
  FROM interact, parts AS actor, parts AS actee
       , members AS acteembr, members AS actormbr
       , groups AS acteegrp, groups AS actorgrp
  WHERE actor.iid = interact.iid
        AND actor.role = 'bb_actor'
        AND actormbr.date = interact.date
        AND actormbr.sname = actor.sname
        AND actorgrp.gid = actormbr.grp
        AND actee.iid = interact.iid
        AND actee.role = 'bb_actee'
        AND acteembr.date = interact.date
        AND acteembr.sname = actee.sname
        AND acteegrp.gid = acteembr.grp
        AND supergroup(actorgrp.gid, interact.date)
            <> supergroup(acteegrp.gid, interact.date)
  ORDER BY interact.date, interact.act, interact.iid, actor.sname, actee.sname;


--
-- Samples and Points
--

-- Error condition
-- Samples.minsis should equal the number of related points data rows.
SELECT samples.sid, samples.date, samples.grp, samples.sname
       , samples.minsis, COUNT(points.pid)
       , 'Error: SAMPLES.Minsis must equal the total number of related POINTS data rows'
  FROM samples, points
  WHERE samples.sid = points.sid
  GROUP BY samples.sid
  HAVING samples.minsis <> count(points.pid)
  ORDER BY samples.sid;

-- Error condition
-- The female point samples must all have fpoints.
SELECT samples.sid, samples.date, samples.grp, samples.stype, samples.sname
       , points.pid, points.time
       , 'Error: Female point samples must have related FPOINTS rows'
  FROM samples, points
  WHERE samples.stype = 'bb_female_point'
        AND points.sid = samples.sid
        AND points.pntid NOT IN (SELECT pntid FROM fpoints)
  ORDER BY samples.sid, points.pntid;

-- Warning condition
-- The juvenile point samples should be taken on individuals
-- less than 4 years old.
SELECT samples.sid, samples.date, samples.grp, samples.stype, samples.sname
       , biograph.birth
       , 'Warning: Juvenile point samples should be for individuals < 4 years of age'
  FROM samples, biograph
  WHERE samples.sname = biograph.sname
        AND samples.stype = 'bb_juvenile_point'
        AND samples.date - CAST ('4 years' AS Interval) >= biograph.birth;

-- Warning condition
-- The juvenile point samples should not be taken on individuals
-- more than 90 days past their maturity date.
SELECT samples.sid, samples.date, samples.grp, samples.stype, samples.sname
       , maturedates.matured
       , 'Warning: Juvenile point sample taken more than 90 days past maturity date'
  FROM samples, maturedates
  WHERE samples.sname = maturedates.sname
        AND samples.stype = 'bb_juvenile_point'
        AND samples.date - CAST ('90 days' AS Interval) > maturedates.matured;


---
--- CYCSTATS
---

-- Warning condition
-- REPSTATS say an individual is cycling but CYCSTATS has no rows.
SELECT sname, date
       , 'Warning: REPSTATS indicates cycling but there is not a CYCSTATS row'
  FROM repstats
  WHERE NOT EXISTS (SELECT 1
                      FROM cycstats
                      WHERE cycstats.sname = repstats.sname
                            AND cycstats.date = repstats.date)
        AND repstats.state = 'bb_cycling'
  ORDER BY sname, date;


--
-- RANKS
--

-- Warning condition
-- Check that every member of a group is given a rank.
SELECT 'Warning: The program (stored procedure) has not been written that finds every Rnktype and Rnkdate on RANKS and runs the queries in RNKTYPES to ensure that every member of every group is included in the ranks.';

--
-- MPI
--
-- Warning condition.
-- Warn when help was given according to MPIACTS but no help was
-- given according to MPI_DATA.Helped of prior Seq numbers.
-- (This is probably inefficient.  It'd be better to start from
-- the row where help was requested but not received.
-- This hot minute I don't care.)
-- This is the "we say we got no help but later we did get help" warning.
SELECT 'Warning: According to the MPI_DATA.MPIActs help was given, and help was requested, but according to earlier events in the multi-party interaction no help was received (MPI_DATA.Helped may be wrong)'
  , mpis.mpiid
  , mpis.date
  , mpi_data.mpidid
  , mpi_data.mpiact
  , mpi_data.seq
  , mpi_data.active
  , mpi_data.helped
  , received.sname
  , received.unksname
  , received.role
  FROM mpis
       JOIN mpi_data ON (mpi_data.mpiid = mpis.mpiid)
       JOIN mpiacts ON (mpiacts.mpiact = mpi_data.mpiact)
       JOIN mpi_parts AS received ON (received.mpidid = mpi_data.mpidid)
  WHERE mpiacts.kind = 'bb_help_given_mact'
        AND received.role = 'bb_actee'
        -- Help was requested
        AND EXISTS (
          SELECT 1
            FROM mpi_data AS earlier
                 JOIN mpi_parts AS requestor
                   ON (requestor.mpidid = earlier.mpidid)
            WHERE earlier.mpiid = mpi_data.mpiid
                  AND earlier.seq < mpi_data.seq
                  AND requestor.mpidid = earlier.mpidid
                  AND requestor.role = 'bb_actor'
                  AND requestor.sname = received.sname)
        -- but help was not received.
        AND NOT EXISTS (
          SELECT 1
            FROM mpi_data AS earlier
                 JOIN mpi_parts AS requestor
                   ON (requestor.mpidid = earlier.mpidid)
            WHERE earlier.mpiid = mpi_data.mpiid
                  AND earlier.seq < mpi_data.seq
                  AND requestor.mpidid = earlier.mpidid
                  AND requestor.role = 'bb_actor'
                  AND requestor.sname = received.sname
                  AND earlier.helped)
  ORDER BY mpis.date, received.sname, received.unksname, mpi_data.mpidid;

-- Generate a warning when there's a MPIS.Consort type of C and no
-- related CONSORTS row.
SELECT *, 'Warning: no related CONSORTS row'
  FROM mpis
  WHERE mpis.context_type = 'bb_consort_context'
        AND NOT EXISTS (SELECT 1
                          FROM consorts
                          WHERE consorts.mpiid = mpis.mpiid)
  ORDER BY mpis.date, mpis.mpiid;

-- Generate an error when there's not exactly 2 MPI_PARTS rows
-- for each MPI_DATA row.
SELECT mpis.mpiid
     , mpi_data.mpidid
     , mpis.date
     , mpi_data.seq
     , mpi_data.mpiact
     , 'Error: MPI_DATA rows must have exactly 2 corresponding rows on MPI_PARTS'
  FROM mpi_data
       JOIN mpis ON (mpis.mpiid = mpi_data.mpiid)
  WHERE (SELECT count(*)
           FROM mpi_parts
           WHERE mpi_data.mpidid = mpi_parts.mpidid) <> 2
  ORDER BY mpis.date, mpis.mpiid, mpi_data.seq, mpi_data.mpiact;

-- Generate a warning when more than two MPI_DATA rows, sharing a Mpiid,
-- have the same Seq value -- when there are more than two dyadic
-- interactions occurring simultaneously.
SELECT mpis.mpiid
     , mpi_data.mpidid
     , mpis.date
     , mpi_data.seq
     , mpi_data.mpiact
     , 'Warning: more than 2 dyadic interactions occurring simultaneously'
  FROM mpi_data
       JOIN mpis ON (mpis.mpiid = mpi_data.mpiid)
  WHERE (SELECT count(*)
           FROM mpi_data AS counted
           WHERE counted.mpiid = mpi_data.mpiid
                 AND counted.seq = mpi_data.seq) > 2
  ORDER BY mpis.date, mpis.mpiid, mpi_data.seq, mpi_data.mpiact;

-- Generate a warning when Active is TRUE and there are no subsequent
-- AH interactions where the help-requestee is the recipient of help in
-- the same multi-party interaction collection.
SELECT mpis.mpiid
     , mpi_data.mpidid
     , mpis.date
     , mpi_data.seq
     , mpi_data.mpiact
     , mpi_data.active
     , mpi_parts.sname
     , mpi_parts.unksname
     , 'Warning: MPI_DATA.Active is TRUE but there are no subsequent bb_active_help interactions where the help-requestee is the recipient of help in the same multi-party interaction collection'
  FROM mpis
       JOIN mpi_data ON (mpi_data.mpiid = mpis.mpiid)
       JOIN mpi_parts ON (mpi_parts.mpidid = mpi_data.mpidid)
  WHERE mpi_data.active
        AND mpi_parts.role = 'bb_actor'
        AND NOT EXISTS (SELECT 1
                      FROM mpi_data AS help_given
                           JOIN mpi_parts AS helped
                             ON (helped.mpidid = help_given.mpidid)
                      WHERE help_given.mpiid = mpi_data.mpiid
                            AND help_given.seq > mpi_data.seq
                            AND help_given.mpiact = 'bb_active_help'
                            AND helped.role = 'bb_actee'
                            AND (helped.sname = mpi_parts.sname
                                 OR helped.unksname = mpi_parts.unksname)
                      LIMIT 1)
  ORDER BY mpis.date, mpis.mpiid, mpi_data.seq, mpi_data.mpiact;


-- Generate a warning when Helped is TRUE and Active is FALSE and 
-- there are no subsequent PH interactions where the help-requestee
-- is the recipient of help in the same multi-party interaction
-- collection.
SELECT mpis.mpiid
     , mpi_data.mpidid
     , mpis.date
     , mpi_data.seq
     , mpi_data.mpiact
     , mpi_data.active
     , mpi_parts.sname
     , mpi_parts.unksname
     , 'Warning: MPI_DATA.Helped is TRUE and MPI_DATA.Active is FALSE but there are no subsequent bb_passive_help interactions where the help-requestee is the recipient of help in the same multi-party interaction collection'
  FROM mpis
       JOIN mpi_data ON (mpi_data.mpiid = mpis.mpiid)
       JOIN mpi_parts ON (mpi_parts.mpidid = mpi_data.mpidid)
  WHERE mpi_data.helped
        AND NOT(mpi_data.active)
        AND mpi_parts.role = 'bb_actor'
        AND NOT EXISTS (SELECT 1
                      FROM mpi_data AS help_given
                           JOIN mpi_parts AS helped
                             ON (helped.mpidid = help_given.mpidid)
                      WHERE help_given.mpiid = mpi_data.mpiid
                            AND help_given.seq > mpi_data.seq
                            AND help_given.mpiact = 'bb_passive_help'
                            AND helped.role = 'bb_actee'
                            AND (helped.sname = mpi_parts.sname
                                 OR helped.unksname = mpi_parts.unksname))
  ORDER BY mpis.date, mpis.mpiid, mpi_data.seq, mpi_data.mpiact;

--
-- Dartings
--

-- Generate a warning when mass is null.
SELECT dartings.sname, dartings.dartdaytime, 'Warning: DARTINGS.Mass is NULL'
  FROM dartings
  WHERE dartings.mass IS NULL
  ORDER BY dartings.dartdaytime
         , dartings.sname;

-- Generate a warning when pickuptime is more than 30 minutes after dartdaytime.
SELECT dartings.sname, dartings.dartdaytime, dartings.pickuptime, 'Warning: DARTINGS.Pickuptime is more than bb_pickuptime_warning_delay after DARTINGS.Dartdaytime'
  FROM dartings
  WHERE dartings.dartdaytime::time > dartings.pickuptime + 'bb_pickuptime_warning_delay'::interval
  ORDER BY dartings.dartdaytime
         , dartings.sname;

 -- Generate a warning when a darted female does not have a vaginal swab.
 SELECT biograph.bioid, dartings.dartid, dartings.sname, dartings.date
     , 'Warning: A female darted after bb_vaginalswab_start has a no vaginal swab'
  FROM dartings
       JOIN biograph ON (biograph.sname = dartings.sname)
  WHERE biograph.sex = 'bb_female'
        AND dartings.date >= 'bb_vaginalswab_start'
        AND NOT EXISTS (SELECT 1
                          FROM dart_samples
                          WHERE dart_samples.dartid = dartings.dartid
                                AND dart_samples.ds_type= bb_dst_vagswab
                          LIMIT 1)
  ORDER BY dartings.date, dartings.sname, dartings.dartid;

-- Generate a warning when young individuals have small testes.

-- (Don't bother to make a macro for these because they're going to go
-- into the db anyway as warning sql.)

/* We don't care about warnings for testes_arc
--
-- testes_arc
--

-- length
SELECT biograph.bioid
     , dartings.dartid
     , testes_arc.testesid
     , dartings.sname
     , biograph.birth
     , dartings.date
     , AGE(dartings.date, biograph.birth) AS age
     , testes_arc.testseq
     , testes_arc.testlength
     , 'Warning: Individual darted at less then bb_adult_testes_age has a testes length less than bb_min_adult_arc_testlength'
  FROM testes_arc
       JOIN dartings ON (dartings.dartid = testes_arc.dartid)
       JOIN biograph ON (biograph.sname = dartings.sname)
  WHERE testes_arc.testlength < bb_min_adult_arc_testlength
        AND AGE(dartings.date, biograph.birth)
            < 'bb_adult_testes_age'::INTERVAL
  ORDER BY dartings.sname, biograph.bioid, dartings.date, testes_arc.testseq;

-- width
SELECT biograph.bioid
     , dartings.dartid
     , testes_arc.testesid
     , dartings.sname
     , biograph.birth
     , dartings.date
     , AGE(dartings.date, biograph.birth) AS age
     , testes_arc.testseq
     , testes_arc.testwidth
     , 'Warning: Individual darted at less then bb_adult_testes_age has a testes length less than bb_min_adult_arc_testwidth'
  FROM testes_arc
       JOIN dartings ON (dartings.dartid = testes_arc.dartid)
       JOIN biograph ON (biograph.sname = dartings.sname)
  WHERE testes_arc.testwidth < bb_min_adult_arc_testwidth
        AND AGE(dartings.date, biograph.birth)
            < 'bb_adult_testes_age'::INTERVAL
  ORDER BY dartings.sname, biograph.bioid, dartings.date, testes_arc.testseq;
*/
--
-- testes_diam
--

-- length
SELECT biograph.bioid
     , dartings.dartid
     , testes_diam.testesid
     , dartings.sname
     , biograph.birth
     , dartings.date
     , AGE(dartings.date, biograph.birth) AS age
     , testes_diam.testseq
     , testes_diam.testlength
     , 'Warning: Individual darted at less then bb_adult_testes_age has a testes length less than bb_min_adult_diam_testlength'
  FROM testes_diam
       JOIN dartings ON (dartings.dartid = testes_diam.dartid)
       JOIN biograph ON (biograph.sname = dartings.sname)
  WHERE testes_diam.testlength < bb_min_adult_diam_testlength
        AND AGE(dartings.date, biograph.birth)
            < 'bb_adult_testes_age'::INTERVAL
  ORDER BY dartings.sname, biograph.bioid, dartings.date, testes_diam.testseq;

-- width
SELECT biograph.bioid
     , dartings.dartid
     , testes_diam.testesid
     , dartings.sname
     , biograph.birth
     , dartings.date
     , AGE(dartings.date, biograph.birth) AS age
     , testes_diam.testseq
     , testes_diam.testwidth
     , 'Warning: Individual darted at less then bb_adult_testes_age has a testes length less than bb_min_adult_diam_testwidth'
  FROM testes_diam
       JOIN dartings ON (dartings.dartid = testes_diam.dartid)
       JOIN biograph ON (biograph.sname = dartings.sname)
  WHERE testes_diam.testwidth < bb_min_adult_diam_testwidth
        AND AGE(dartings.date, biograph.birth)
            < 'bb_adult_testes_age'::INTERVAL
  ORDER BY dartings.sname, biograph.bioid, dartings.date, testes_diam.testseq;


--
-- Dad_data
--

-- Generate a warning if the male is not in the mother's supergroup at
-- any time during the fertile period.
-- (This is all the same query but I'm reluctant to macro-ize it
-- for fear of how that will play with the warning system.)

-- dad_excl
SELECT dad_data.dadid, dad_data.kid, dad_data.dad_excl, 'WARNING: DAD_DATA.Dad_excl is not in the mother''s supergroup during the fertile period'
  FROM dad_data, maternities
  WHERE maternities.child = dad_data.kid
        AND dad_data.dad_excl NOT IN
    (SELECT dadloc.sname
       FROM members AS dadloc
            JOIN members AS momloc 
              ON (momloc.date = dadloc.date
                  AND supergroup(momloc.grp, momloc.date)
                      = supergroup(dadloc.grp, dadloc.date))
       WHERE dadloc.date < maternities.zdate
             AND dadloc.date >= maternities.zdate - bb_ovulation_len)
  ORDER BY dad_data.kid;

-- dad_1perr
SELECT dad_data.dadid, dad_data.kid, dad_data.dad_1perr, 'WARNING: DAD_DATA.Dad_1perr is not in the mother''s supergroup during the fertile period'
  FROM dad_data, maternities
  WHERE maternities.child = dad_data.kid
        AND dad_data.dad_1perr NOT IN
    (SELECT dadloc.sname
       FROM members AS dadloc
            JOIN members AS momloc 
              ON (momloc.date = dadloc.date
                  AND supergroup(momloc.grp, momloc.date)
                      = supergroup(dadloc.grp, dadloc.date))
       WHERE dadloc.date < maternities.zdate
             AND dadloc.date >= maternities.zdate - bb_ovulation_len)
  ORDER BY dad_data.kid;

-- dad_5perr
SELECT dad_data.dadid, dad_data.kid, dad_data.dad_5perr, 'WARNING: DAD_DATA.Dad_5perr is not in the mother''s supergroup during the fertile period'
  FROM dad_data, maternities
  WHERE maternities.child = dad_data.kid
        AND dad_data.dad_5perr NOT IN
    (SELECT dadloc.sname
       FROM members AS dadloc
            JOIN members AS momloc 
              ON (momloc.date = dadloc.date
                  AND supergroup(momloc.grp, momloc.date)
                      = supergroup(dadloc.grp, dadloc.date))
       WHERE dadloc.date < maternities.zdate
             AND dadloc.date >= maternities.zdate - bb_ovulation_len)
  ORDER BY dad_data.kid;

-- dad_allmales
SELECT dad_data.dadid, dad_data.kid, dad_data.dad_allmales, 'WARNING: DAD_DATA.Dad_allmales is not in the mother''s supergroup during the fertile period'
  FROM dad_data, maternities
  WHERE maternities.child = dad_data.kid
        AND dad_data.dad_allmales NOT IN
    (SELECT dadloc.sname
       FROM members AS dadloc
            JOIN members AS momloc 
              ON (momloc.date = dadloc.date
                  AND supergroup(momloc.grp, momloc.date)
                      = supergroup(dadloc.grp, dadloc.date))
       WHERE dadloc.date < maternities.zdate
             AND dadloc.date >= maternities.zdate - bb_ovulation_len)
  ORDER BY dad_data.kid;

-- dad_consensus
SELECT dad_data.dadid, dad_data.kid, dad_data.dad_consensus, 'WARNING: DAD_DATA.Dad_consensus is not in the mother''s supergroup during the fertile period'
  FROM dad_data, maternities
  WHERE maternities.child = dad_data.kid
        AND dad_data.dad_consensus NOT IN
    (SELECT dadloc.sname
       FROM members AS dadloc
            JOIN members AS momloc 
              ON (momloc.date = dadloc.date
                  AND supergroup(momloc.grp, momloc.date)
                      = supergroup(dadloc.grp, dadloc.date))
       WHERE dadloc.date < maternities.zdate
             AND dadloc.date >= maternities.zdate - bb_ovulation_len)
  ORDER BY dad_data.kid;

--
-- There can be information as to whether the mom has been genetically
-- sampled if and only if the mom is known.
--
SELECT biograph.bioid
     , biograph.sname
     , biograph.pid
     , dad_data.dadid
     , dad_data.mom_sampled
     , 'Error: There can be information as to whether the mom has been genetically sampled (DAD_DATA.Mom_sampled is not NULL) if and only if the mom is known (BIOGRAPH.Pid is not NULL)'
  FROM biograph
       JOIN dad_data ON (dad_data.kid = biograph.sname)
  WHERE (biograph.pid IS NULL AND dad_data.mom_sampled IS NOT NULL)
        OR (biograph.pid IS NOT NULL AND dad_data.mom_sampled IS NULL)
  ORDER BY biograph.sname, biograph.pid;

--
-- SWERB
--

-- There is usually at least one observer in every observation team.
SELECT *
     , 'WARNING: No observer in an observation team -- a SWERB_DEPARTS_DATA row does not have at least one releated SWERB_OBSERVERS row'
  FROM swerb_departs_data
  WHERE NOT EXISTS (SELECT 1
                      FROM swerb_observers
                      WHERE swerb_observers.did = swerb_departs_data.did
                      LIMIT 1)
  ORDER BY swerb_departs_data.date, swerb_departs_data.did;


-- Observation teams should not leave camp too early or too late.
SELECT *
     , 'WARNING: Observation team leaving camp before bb_day_start or after bb_day_departures_end'
     FROM swerb_departs
     WHERE swerb_departs.time < 'bb_day_start'
           OR swerb_departs.time > 'bb_day_departures_end'
     ORDER BY swerb_departs.date, swerb_departs.time, swerb_departs.did;


-- Observation teams should collect data after leaving camp.
SELECT *
     , 'WARNING: A SWERB observation team departed from camp but did not collect any data'
  FROM swerb_departs
  WHERE NOT EXISTS (SELECT 1
                      FROM swerb_bes
                      WHERE swerb_bes.did = swerb_departs.did
                      LIMIT 1)
  ORDER BY swerb_departs.date, swerb_departs.time, swerb_departs.did;


-- Data collected after quad data collection was stopped should have
-- gps data.
SELECT *
     , 'WARNING: SWERB data collected without a gps unit after bb_quad_end'
  FROM swerb_departs_data
  WHERE NOT EXISTS (SELECT 1
                      FROM swerb_departs_gps
                      WHERE swerb_departs_gps.did = swerb_departs_data.did
                      LIMIT 1)
  ORDER BY swerb_departs_data.date
         , swerb_departs_data.time
         , swerb_departs_data.did;

-- Coordinates should be inside the regular study area.

-- swerb_data coordinates.
SELECT swerb_data.swid
     , swerb_bes.beid
     , swerb_departs_data.did
     , swerb_departs_data.date
     , swerb_bes.focal_grp
     , swerb_data.seen_grp
     , CASE WHEN swerb_data.event = 'bb_swerb_ogrp'
              THEN NULL
            ELSE swerb_bes.seq
       END AS seq
     , swerb_data.event
     , swerb_data.quad
     , ST_X(swerb_data.xyloc) AS x
     , ST_Y(swerb_data.xyloc) AS y
     , swerb_data.altitude
     , swerb_data.pdop
     , swerb_data.accuracy
     , swerb_data.ogdistance
     , swerb_data.subgroup
     , swerb_data.garmincode
     , 'WARNING: SWERB_DATA row outside the usual geographic boundaries (bb_ok_study_area)'
  FROM swerb_data
       JOIN swerb_bes ON (swerb_bes.beid = swerb_data.beid)
       JOIN swerb_departs_data ON (swerb_departs_data.did = swerb_bes.did)
  WHERE NOT(ST_Intersects(swerb_data.xyloc
                        , ST_PolygonFromText('bb_ok_study_area'
                                           , bb_amboseli_srid)))
  ORDER BY swerb_departs_data.date
         , swerb_data.seen_grp
         , swerb_data.time
         , swerb_bes.seq;

-- quad coordinates.
SELECT quad_data.quad
     , ST_X(quad_data.xyloc) AS x
     , ST_Y(quad_data.xyloc) AS y
     , quad_data.aerial
     , 'WARNING: QUAD_DATA row outside the usual geographic boundaries (bb_ok_study_area)'
  FROM quad_data
  WHERE NOT(ST_Intersects(quad_data.xyloc
                        , ST_PolygonFromText('bb_ok_study_area'
                                           , bb_amboseli_srid)))
  ORDER BY quad_data.quad;

-- swerb_departs_gps coordinates.
SELECT swerb_departs_data.did
     , swerb_departs_data.date
     , swerb_departs_data.time
     , ST_X(swerb_departs_gps.xyloc) AS x
     , ST_Y(swerb_departs_gps.xyloc) AS y
     , swerb_departs_gps.altitude
     , swerb_departs_gps.pdop
     , swerb_departs_gps.accuracy
     , swerb_departs_gps.gps
     , swerb_departs_gps.garmincode
     , 'WARNING: SWERB_DEPARTS_GPS row outside the usual geographic boundaries (bb_ok_study_area)'
  FROM swerb_departs_data
       JOIN swerb_departs_gps
            ON (swerb_departs_gps.did = swerb_departs_data.did)
  WHERE NOT(ST_Intersects(swerb_departs_gps.xyloc
                        , ST_PolygonFromText('bb_ok_study_area'
                                           , bb_amboseli_srid)))
  ORDER BY swerb_departs_data.date
         , swerb_departs_data.time
         , swerb_departs_data.did;


-- Warn when there's more than one departure per gps unit per day.
SELECT swerb_departs_data.did
     , swerb_departs_data.date
     , swerb_departs_data.time
     , ST_X(swerb_departs_gps.xyloc) AS x
     , ST_Y(swerb_departs_gps.xyloc) AS y
     , swerb_departs_gps.altitude
     , swerb_departs_gps.pdop
     , swerb_departs_gps.accuracy
     , swerb_departs_gps.gps
     , swerb_departs_gps.garmincode
     , 'WARNING: More than one departure per gps per day'
  FROM swerb_departs_data
       JOIN swerb_departs_gps 
            ON (swerb_departs_gps.did = swerb_departs_data.did)
  WHERE EXISTS (SELECT 1
                  FROM swerb_departs_data AS odata
                       JOIN swerb_departs_gps AS ogps
                            ON (ogps.did = odata.did)
                  WHERE odata.date = swerb_departs_data.date
                        AND ogps.gps = swerb_departs_gps.gps
                        AND odata.did <> swerb_departs_data.did
                  LIMIT 1)
  ORDER BY swerb_departs_data.date
         , swerb_departs_gps.gps
         , swerb_departs_data.time
         , swerb_departs_data.did;

-- Warn when there's a non-focal group observation and the seen group
-- is the same as the focal group.
SELECT swerb_data.swid
     , swerb_bes.beid
     , swerb_departs_data.did
     , swerb_bes.focal_grp
     , swerb_data.seen_grp
     , swerb_data.event
     , swerb_data.time
     , swerb_departs_data.date
     , swerb_data.subgroup
     , 'WARNING: SWERB observation of a non-focal group that''s the same group as the focal group'
  FROM swerb_data
       JOIN swerb_bes ON (swerb_bes.beid = swerb_data.beid)
       JOIN swerb_departs_data ON (swerb_departs_data.did = swerb_bes.beid)
  WHERE swerb_data.event = 'bb_swerb_ogrp'
        AND swerb_data.seen_grp = swerb_bes.focal_grp
  ORDER BY swerb_data.seen_grp
         , swerb_departs_data.date
         , swerb_data.time
         , swerb_data.swid;

--
-- No PDOP. accuracy. and altitude warnings.
--

-- Warning when there's SWERB_DATA for a GPS unit that supports
-- PDOP but no PDOP

SELECT swerb_data.swid
     , swerb_bes.beid
     , swerb_departs_data.did
     , swerb_departs_data.date
     , swerb_bes.focal_grp
     , swerb_data.seen_grp
     , swerb_bes.seq
     , swerb_data.event
     , ST_X(swerb_data.xyloc) AS x
     , ST_Y(swerb_data.xyloc) AS y
     , swerb_data.altitude
     , swerb_data.pdop
     , swerb_data.accuracy
     , swerb_data.ogdistance
     , swerb_data.subgroup
     , swerb_data.garmincode
     , swerb_departs_gps.gps
     , gps_units.descr
     , gps_units.label
     , 'WARNING: No PDOP value for SWERB_DATA row but the GPS unit supports PDOP reporting'
  FROM swerb_data
       JOIN swerb_bes ON (swerb_bes.beid = swerb_data.beid)
       JOIN swerb_departs_data ON (swerb_departs_data.did = swerb_bes.did)
       JOIN swerb_departs_gps
            ON (swerb_departs_gps.did = swerb_departs_data.did)
       JOIN gps_units ON (gps_units.gps = swerb_departs_gps.gps)
  WHERE swerb_data.pdop IS NULL
        AND gps_units.errortype = 'bb_gpsu_pdop'
  ORDER BY swerb_departs_data.date
         , swerb_data.seen_grp
         , swerb_data.time
         , swerb_bes.seq;

-- Warning when there's SWERB_DATA for a GPS unit that supports
-- accuracy but no accuracy

SELECT swerb_data.swid
     , swerb_bes.beid
     , swerb_departs_data.did
     , swerb_departs_data.date
     , swerb_bes.focal_grp
     , swerb_data.seen_grp
     , swerb_bes.seq
     , swerb_data.event
     , ST_X(swerb_data.xyloc) AS x
     , ST_Y(swerb_data.xyloc) AS y
     , swerb_data.altitude
     , swerb_data.pdop
     , swerb_data.accuracy
     , swerb_data.ogdistance
     , swerb_data.subgroup
     , swerb_data.garmincode
     , swerb_departs_gps.gps
     , gps_units.descr
     , gps_units.label
     , 'WARNING: No Accuracy value for SWERB_DATA row but the GPS unit supports accuracy reporting'
  FROM swerb_data
       JOIN swerb_bes ON (swerb_bes.beid = swerb_data.beid)
       JOIN swerb_departs_data ON (swerb_departs_data.did = swerb_bes.did)
       JOIN swerb_departs_gps
            ON (swerb_departs_gps.did = swerb_departs_data.did)
       JOIN gps_units ON (gps_units.gps = swerb_departs_gps.gps)
  WHERE swerb_data.accuracy IS NULL
        AND gps_units.errortype = 'bb_gpsu_accuracy'
  ORDER BY swerb_departs_data.date
         , swerb_data.seen_grp
         , swerb_data.time
         , swerb_bes.seq;

-- Warning when there's SWERB_DATA for a GPS unit but no altitude.

SELECT swerb_data.swid
     , swerb_bes.beid
     , swerb_departs_data.did
     , swerb_departs_data.date
     , swerb_bes.focal_grp
     , swerb_data.seen_grp
     , swerb_bes.seq
     , swerb_data.event
     , ST_X(swerb_data.xyloc) AS x
     , ST_Y(swerb_data.xyloc) AS y
     , swerb_data.altitude
     , swerb_data.pdop
     , swerb_data.accuracy
     , swerb_data.ogdistance
     , swerb_data.subgroup
     , swerb_data.garmincode
     , 'WARNING: No altitude value for SWERB_DATA row but there is a XYLoc'
  FROM swerb_data
       JOIN swerb_bes ON (swerb_bes.beid = swerb_data.beid)
       JOIN swerb_departs_data ON (swerb_departs_data.did = swerb_bes.did)
  WHERE swerb_data.altitude IS NULL
        AND swerb_data.xyloc IS NOT NULL
  ORDER BY swerb_data.seen_grp
         , swerb_data.time
         , swerb_bes.seq;

-- Warning when there's SWERB_LOC_GPS data for a GPS unit that supports
-- PDOP but no PDOP

SELECT swerb_data.swid
     , swerb_loc_data.loc
     , swerb_loc_data.adtime
     , swerb_bes.beid
     , swerb_departs_data.did
     , swerb_departs_data.date
     , swerb_bes.focal_grp
     , swerb_data.seen_grp
     , swerb_bes.seq
     , swerb_data.event
     , ST_X(swerb_loc_gps.xyloc) AS x
     , ST_Y(swerb_loc_gps.xyloc) AS y
     , swerb_loc_gps.altitude
     , swerb_loc_gps.pdop
     , swerb_loc_gps.accuracy
     , swerb_loc_gps.gps_datetime
     , swerb_loc_gps.garmincode
     , swerb_departs_gps.gps
     , gps_units.descr
     , gps_units.label
     , 'WARNING: No PDOP value for SWERB_LOC_GPS row but the GPS unit supports PDOP reporting'
  FROM swerb_loc_gps
       JOIN swerb_loc_data ON (swerb_loc_data.swid = swerb_loc_gps.swid)
       JOIN swerb_data ON (swerb_data.swid = swerb_loc_gps.swid)
       JOIN swerb_bes ON (swerb_bes.beid = swerb_data.beid)
       JOIN swerb_departs_data ON (swerb_departs_data.did = swerb_bes.did)
       JOIN swerb_departs_gps
            ON (swerb_departs_gps.did = swerb_departs_data.did)
       JOIN gps_units ON (gps_units.gps = swerb_departs_gps.gps)
  WHERE swerb_loc_gps.pdop IS NULL
        AND gps_units.errortype = 'bb_gpsu_pdop'
  ORDER BY swerb_departs_data.date
         , swerb_data.seen_grp
         , swerb_loc_gps.gps_datetime
         , swerb_bes.seq;

-- Warning when there's SWERB_LOC_GPS data for a GPS unit that supports
-- accuracy but no accuracy

SELECT swerb_data.swid
     , swerb_loc_data.loc
     , swerb_loc_data.adtime
     , swerb_bes.beid
     , swerb_departs_data.did
     , swerb_departs_data.date
     , swerb_bes.focal_grp
     , swerb_data.seen_grp
     , swerb_bes.seq
     , swerb_data.event
     , ST_X(swerb_loc_gps.xyloc) AS x
     , ST_Y(swerb_loc_gps.xyloc) AS y
     , swerb_loc_gps.altitude
     , swerb_loc_gps.pdop
     , swerb_loc_gps.accuracy
     , swerb_loc_gps.gps_datetime
     , swerb_loc_gps.garmincode
     , swerb_departs_gps.gps
     , gps_units.descr
     , gps_units.label
     , 'WARNING: No accuracy value for SWERB_LOC_GPS row but the GPS unit supports accuracy reporting'
  FROM swerb_loc_gps
       JOIN swerb_loc_data ON (swerb_loc_data.swid = swerb_loc_gps.swid)
       JOIN swerb_data ON (swerb_data.swid = swerb_loc_gps.swid)
       JOIN swerb_bes ON (swerb_bes.beid = swerb_data.beid)
       JOIN swerb_departs_data ON (swerb_departs_data.did = swerb_bes.did)
       JOIN swerb_departs_gps
            ON (swerb_departs_gps.did = swerb_departs_data.did)
       JOIN gps_units ON (gps_units.gps = swerb_departs_gps.gps)
  WHERE swerb_loc_gps.accuracy IS NULL
        AND gps_units.errortype = 'bb_gpsu_accuracy'
  ORDER BY swerb_departs_data.date
         , swerb_data.seen_grp
         , swerb_loc_gps.gps_datetime
         , swerb_bes.seq;

-- Warning when there's SWERB_LOC_GPS data but no altitude.

SELECT swerb_data.swid
     , swerb_loc_data.loc
     , swerb_loc_data.adtime
     , swerb_bes.beid
     , swerb_departs_data.did
     , swerb_departs_data.date
     , swerb_bes.focal_grp
     , swerb_data.seen_grp
     , swerb_bes.seq
     , swerb_data.event
     , ST_X(swerb_loc_gps.xyloc) AS x
     , ST_Y(swerb_loc_gps.xyloc) AS y
     , swerb_loc_gps.altitude
     , swerb_loc_gps.pdop
     , swerb_loc_gps.accuracy
     , swerb_loc_gps.gps_datetime
     , swerb_loc_gps.garmincode
     , 'WARNING: No altitude value for SWERB_LOC_GPS row'
  FROM swerb_loc_gps
       JOIN swerb_loc_data ON (swerb_loc_data.swid = swerb_loc_gps.swid)
       JOIN swerb_data ON (swerb_data.swid = swerb_loc_gps.swid)
       JOIN swerb_bes ON (swerb_bes.beid = swerb_data.beid)
       JOIN swerb_departs_data ON (swerb_departs_data.did = swerb_bes.did)
  WHERE swerb_loc_gps.altitude IS NULL
  ORDER BY swerb_data.seen_grp
         , swerb_data.time
         , swerb_bes.seq;

-- Warning when there's departure data from a GPS unit that supports
-- PDOP but there's no PDOP.

SELECT swerb_departs_data.did
     , swerb_departs_data.date
     , ST_X(swerb_departs_gps.xyloc) AS x
     , ST_Y(swerb_departs_gps.xyloc) AS y
     , swerb_departs_gps.altitude
     , swerb_departs_gps.pdop
     , swerb_departs_gps.accuracy
     , swerb_departs_gps.garmincode
     , swerb_departs_gps.gps
     , gps_units.descr
     , gps_units.label
     , 'WARNING: No PDOP value for SWERB_DEPARTS_DATA row but the GPS unit supports PDOP reporting'
  FROM swerb_departs_data
       JOIN swerb_departs_gps
            ON (swerb_departs_gps.did = swerb_departs_data.did)
       JOIN gps_units ON (gps_units.gps = swerb_departs_gps.gps)
  WHERE swerb_departs_gps.pdop IS NULL
        AND gps_units.errortype = 'bb_gpsu_pdop'
  ORDER BY swerb_departs_data.date
         , swerb_departs_data.time
         , swerb_departs_data.did;

-- Warning when there's departure data from a GPS unit that supports
-- accuracy but there's no accuracy.

SELECT swerb_departs_data.did
     , swerb_departs_data.date
     , ST_X(swerb_departs_gps.xyloc) AS x
     , ST_Y(swerb_departs_gps.xyloc) AS y
     , swerb_departs_gps.altitude
     , swerb_departs_gps.pdop
     , swerb_departs_gps.accuracy
     , swerb_departs_gps.garmincode
     , swerb_departs_gps.gps
     , gps_units.descr
     , gps_units.label
     , 'WARNING: No Accuracy value for SWERB_DEPARTS_DATA row but the GPS unit supports accuracy reporting'
  FROM swerb_departs_data
       JOIN swerb_departs_gps
            ON (swerb_departs_gps.did = swerb_departs_data.did)
       JOIN gps_units ON (gps_units.gps = swerb_departs_gps.gps)
  WHERE swerb_departs_gps.accuracy IS NULL
        AND gps_units.errortype = 'bb_gpsu_accuracy'
  ORDER BY swerb_departs_data.date
         , swerb_departs_data.time
         , swerb_departs_data.did;

-- Warning when there's departure data from a GPS unit but no altitude.

SELECT swerb_departs_data.did
     , swerb_departs_data.date
     , ST_X(swerb_departs_gps.xyloc) AS x
     , ST_Y(swerb_departs_gps.xyloc) AS y
     , swerb_departs_gps.altitude
     , swerb_departs_gps.pdop
     , swerb_departs_gps.accuracy
     , swerb_departs_gps.garmincode
     , swerb_departs_gps.gps
     , 'WARNING: No altitude value for SWERB_DEPARTS_DATA row but there is GPS data'
  FROM swerb_departs_data
       JOIN swerb_departs_gps
            ON (swerb_departs_gps.did = swerb_departs_data.did)
  WHERE swerb_departs_gps.altitude IS NULL
  ORDER BY swerb_departs_data.date
         , swerb_departs_data.time
         , swerb_departs_data.did;

-- Warning when there's grove/waterhole location data from a GPS unit
-- that supports PDOP but there's no PDOP value.

SELECT swerb_gws.loc
     , swerb_gws.type
     , swerb_gws.altname
     , swerb_gws.notes
     , swerb_gw_loc_data.sgwlid
     , swerb_gw_loc_data.date
     , swerb_gw_loc_data.time
     , ST_X(swerb_gw_loc_data.xyloc) AS x
     , ST_Y(swerb_gw_loc_data.xyloc) AS y
     , swerb_gw_loc_data.altitude
     , swerb_gw_loc_data.pdop
     , swerb_gw_loc_data.accuracy
     , swerb_gw_loc_data.gps
     , gps_units.descr
     , gps_units.label
     , 'WARNING: No PDOP value for SWERB_GW_LOC_DATA row but the GPS unit supports PDOP reporting'
  FROM swerb_gws
       JOIN swerb_gw_loc_data ON (swerb_gw_loc_data.loc = swerb_gws.loc)
       JOIN gps_units ON (gps_units.gps = swerb_gw_loc_data.gps)
  WHERE swerb_gw_loc_data.pdop IS NULL
        AND gps_units.errortype = 'bb_gpsu_pdop'
  ORDER BY swerb_gws.loc
         , swerb_gw_loc_data.date
         , swerb_gw_loc_data.time
         , swerb_gw_loc_data.sgwlid;

-- Warning when there's grove/waterhole location data from a GPS unit
-- that supports accuracy but there's no accuracy value.

SELECT swerb_gws.loc
     , swerb_gws.type
     , swerb_gws.altname
     , swerb_gws.notes
     , swerb_gw_loc_data.sgwlid
     , swerb_gw_loc_data.date
     , swerb_gw_loc_data.time
     , ST_X(swerb_gw_loc_data.xyloc) AS x
     , ST_Y(swerb_gw_loc_data.xyloc) AS y
     , swerb_gw_loc_data.altitude
     , swerb_gw_loc_data.pdop
     , swerb_gw_loc_data.accuracy
     , swerb_gw_loc_data.gps
     , gps_units.descr
     , gps_units.label
     , 'WARNING: No Accuracy value for SWERB_GW_LOC_DATA row but the GPS unit supports accuracy reporting'
  FROM swerb_gws
       JOIN swerb_gw_loc_data ON (swerb_gw_loc_data.loc = swerb_gws.loc)
       JOIN gps_units ON (gps_units.gps = swerb_gw_loc_data.gps)
  WHERE swerb_gw_loc_data.accuracy IS NULL
        AND gps_units.errortype = 'bb_gpsu_accuracy'
  ORDER BY swerb_gws.loc
         , swerb_gw_loc_data.date
         , swerb_gw_loc_data.time
         , swerb_gw_loc_data.sgwlid;

-- Warning when there's grove/waterhole location data with an XY location
-- but no altitude value.

SELECT swerb_gws.loc
     , swerb_gws.type
     , swerb_gws.altname
     , swerb_gws.notes
     , swerb_gw_loc_data.sgwlid
     , swerb_gw_loc_data.date
     , swerb_gw_loc_data.time
     , ST_X(swerb_gw_loc_data.xyloc) AS x
     , ST_Y(swerb_gw_loc_data.xyloc) AS y
     , swerb_gw_loc_data.altitude
     , swerb_gw_loc_data.pdop
     , swerb_gw_loc_data.accuracy
     , swerb_gw_loc_data.gps
     , 'WARNING: No accuracy value for SWERB_GW_LOC_DATA row but there is an XYLoc'
  FROM swerb_gws
       JOIN swerb_gw_loc_data ON (swerb_gw_loc_data.loc = swerb_gws.loc)
  WHERE swerb_gw_loc_data.altitude IS NULL
        AND swerb_gw_loc_data.xyloc IS NOT NULL
  ORDER BY swerb_gws.loc
         , swerb_gw_loc_data.date
         , swerb_gw_loc_data.time
         , swerb_gw_loc_data.sgwlid;

-- Warning when there's grove/waterhole that has never been located
-- anywhere.

SELECT swerb_gws.loc
     , swerb_gws.type
     , swerb_gws.altname
     , swerb_gws.notes
     , 'WARNING: There is no recorded location for this SWERB_GWS row -- no related SWERB_GW_LOCS_DATA row'
  FROM swerb_gws
  WHERE NOT EXISTS (SELECT 1
                      FROM swerb_gw_loc_data
                      WHERE swerb_gw_loc_data.loc = swerb_gws.loc
                      LIMIT 1)
  ORDER BY swerb_gws.loc;

-- Warning when a group is located at a waterhole but the related event
-- is not a drinking event.

SELECT swerb_departs_data.date
     , swerb_data.swid
     , swerb_data.event
     , swerb_data.time
     , swerb_data.seen_grp
     , swerb_data.subgroup
     , swerb_loc_data.slid
     , swerb_gws.loc
     , swerb_gws.type
     , swerb_gws.altname
     , swerb_gws.notes
     , 'WARNING: A group is located at a waterhole but the related swerb event is not a drinking event'
  FROM swerb_data
       JOIN swerb_bes ON (swerb_bes.beid = swerb_data.beid)
       JOIN swerb_departs_data ON (swerb_departs_data.did = swerb_bes.beid)
     , swerb_loc_data
     , swerb_gws
  WHERE swerb_loc_data.loc = swerb_gws.loc
        AND swerb_data.swid = swerb_loc_data.swid
        AND swerb_gws.type = 'bb_swerb_gws_water'
        AND swerb_data.event <> 'bb_swerb_water'
  ORDER BY swerb_departs_data.date
         , swerb_data.time
         , swerb_gws.loc
         , swerb_loc_data.slid;

-- Warning when a male is sighted alone more than 60 days before
-- his dispersedate.

SELECT swerb_departs_data.date
     , swerb_data.swid
     , swerb_data.event
     , swerb_data.time
     , swerb_data.lone_animal
     , swerb_data.seen_grp
     , dispersedates.dispersed
  FROM swerb_data
       JOIN dispersedates ON (dispersedates.sname = swerb_data.lone_animal)
       JOIN swerb_bes ON (swerb_bes.beid = swerb_data.beid)
       JOIN swerb_departs_data ON (swerb_departs_data.did = swerb_bes.beid)
  WHERE swerb_departs_data.date + 'bb_lonemale_lax_time days'::INTERVAL
          < dispersedates.dispersed
  ORDER BY swerb_departs_data.date
         , swerb_data.time
         , swerb_data.lone_animal;
