dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2006, 2008, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Add an automatic Mdate to cycpoints.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl
dnl
dnl m4 includes
include(`copyright.m4')
include(`constants.m4')
include(`macros.m4')
dnl

CREATE OR REPLACE FUNCTION _add_autom(this_sname cycles.sname%TYPE
                                    , ddate_row cycpoints)
  RETURNS VOID
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  -- Given a Ddate automatically insert a Mdate when necessary
  --
  -- GPL_notice(`  --', `2006, 2008', `Karl O. Pinc <kop@meme.com>')
  --
  -- Syntax: _add_autom(this_sname cycles.sname
  --                  , ddate_row cycpoints)
  --
  -- Input:
  --  this_sname  The individual we are concerned with.
  --  ddate_row   The cycpoints row of the Ddate after which we
  --              may need to automatically generate an Mdate.
  --
  -- Side Effects:
  --   Automatically generates an Mdate in cycpoints, if necessary.
  --
  -- Remarks:
  --   Used inside triggers.
  --
  -- Bugs:
  DECLARE
    cp_row RECORD;
    found_cp_row boolean;
    earliest_gap cycgapdays.date%TYPE;
    autom_date cycpoints.date%TYPE;

  BEGIN

    --RAISE INFO '_add_autom(%, %)', this_sname, ddate_row.date;

    -- Some obvious reasons not to make an automatic Mdate.

    IF ddate_row.source = 'bb_estimated' THEN
      -- Don't make automatic M dates for estimated Ddates.
      --RAISE INFO 'Return by estimated';
      RETURN;
    END IF;

    PERFORM 1
      FROM pregs
      WHERE conceive = ddate_row.cpid
      LIMIT 1;
    IF FOUND THEN
      -- Conceptions don't get an automatic Mdate.
      --RAISE INFO 'Return by conception';
      RETURN;
    END IF;

    --
    -- Cycpoints/gaps reasons not to make automatic Mdate.
    --

    -- Find the next cycpoints row.  (Should be T or Ddate.)
    SELECT INTO cp_row
           cycpoints.cpid
         , cycpoints.cid
         , cycpoints.date
         , cycpoints.code
         , cycpoints.source
      FROM cycpoints, cycles
      WHERE cycles.sname = this_sname
            AND cycpoints.cid = cycles.cid
            -- AND cycpoints.code <> 'bb_bleeding'
            AND date > ddate_row.date
      ORDER BY date, code
      LIMIT 1;
    found_cp_row := FOUND;

    -- Find the next gap.
    SELECT INTO earliest_gap date
      FROM cycgapdays
      WHERE cycgapdays.sname = this_sname
            AND cycgapdays.date > ddate_row.date
            AND (cycgapdays.date < cp_row.date
                 OR NOT found_cp_row)
      ORDER BY date
      LIMIT 1;

    -- When might we make an automatic Mdate?
    autom_date := ddate_row.date + CAST('bb_autom_interval days' AS INTERVAL);

    IF cp_row.code = 'bb_menses' THEN
      -- We only make an Mdate if the next Mdate is separated by a gap.
      IF earliest_gap IS NULL THEN
        -- No gap between Ddate and Mdate, no automatic generation.
--RAISE INFO 'Return by mdate and no gap';
        RETURN;
      END IF;
    ELSEIF cp_row.code = 'bb_turgesence' THEN
      -- The automatic Mdate never comes after the next Tdate.
      IF autom_date > cp_row.date THEN
        autom_date := cp_row.date;
      END IF;
    ELSEIF cp_row.code = 'bb_deturgesence' THEN
      -- Paranoia.  This should not happen.
      -- The automatic Mdate never comes after the next Ddate.
      IF autom_date > cp_row.date THEN
        autom_date := cp_row.date - 1;
        IF autom_date = ddate_row.date THEN
          -- No room for an automatic Mdate between the two Ddates.
          RETURN;
        END IF;
      END IF;
    END IF;

    -- Only make the Mdate if there's no gap between it and the Ddate.
    IF earliest_gap IS NOT NULL
         AND earliest_gap <= autom_date THEN
--RAISE INFO 'Return by gap';
      RETURN;
    END IF;

    -- Only make the Mdate if the individual is alive on that date.
    PERFORM 1
      FROM biograph
      WHERE biograph.sname = this_sname
            AND biograph.status <> 'bb_alive'
            AND biograph.statdate < autom_date
      LIMIT 1;
    IF FOUND THEN
--RAISE INFO 'Return by alive';
      RETURN;
    END IF;

    --
    -- Make the automatic Mdate.
    -- 

    IF NOT(found_cp_row)
       OR (earliest_gap IS NOT NULL
           AND earliest_gap < cp_row.date) THEN
      -- There's no next cycpoints row, or....
      -- There's a gap between the Ddate and the next cycpoints
      -- and that's where we're putting the automatic Mdate.
      -- So we need to make a new cycle for the automatic Mdate.
--RAISE INFO 'Inserting new autoM CYCLE at %', autom_date;
      INSERT INTO cycles (sname) VALUES (this_sname);
      INSERT INTO cycpoints (cid, date, code, source)
             VALUES (currval('cycles_cid_seq')
                   , autom_date
                   , 'bb_menses'
                   , 'bb_auto');
    ELSE
      -- Put the automatic mdate into the cycle that follows the Ddate.
      -- (Can't already have a Mdate or we would not have gotten this far.)
--RAISE INFO 'Inserting new autoM EVENT at %', autom_date;
      INSERT INTO cycpoints (cid, date, code, source)
             VALUES (cp_row.cid
                   , autom_date
                   , 'bb_menses'
                   , 'bb_auto');
    END IF;

--RAISE INFO 'Return with new autom!';
    RETURN;
  END;
$$;
grant_func_priv(`_add_autom(cycles.sname%TYPE, cycpoints)')
