dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2004, 2008, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Insert all the cycstats from a cycle row and an ending date.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl
dnl
dnl m4 includes
include(`copyright.m4')
include(`constants.m4')
include(`macros.m4')
dnl

CREATE OR REPLACE FUNCTION _compute_cycstats(
                             this_sname          cycles.sname%TYPE
                           , last_cid            cycles.cid%TYPE
                           , last_tdate          cycles.tdate%TYPE
                           , last_ddate          cycles.ddate%TYPE
                           , this_cycle_cid      cycles.cid%TYPE
                           , this_cycle_tdate    cycles.tdate%TYPE)
  RETURNS INT
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$

  -- Insert all the cycstats from a cycle row and an ending date.
  --
  -- GPL_notice(`  --', `2004, 2008', `Karl O. Pinc <kop@meme.com>')
  --
  -- Syntax: _compute_cycstats(sname
  --                          , last_cid
  --                          , last_tdate
  --                          , last_ddate
  --                          , this_cycle_cid
  --                          , this_cycle_tdate)
  --
  -- Arguments:
  --  sname  Sname of the individual to rebuild.
  --
  -- Remarks:
  -- This function is called from the CYCLES triggers to keep the
  -- by-day tables up to date.

  DECLARE

    this_pid   pregs.pid%TYPE;
    this_birth biograph.birth%TYPE;

  BEGIN


  SELECT INTO this_pid pid FROM pregs WHERE conceive = last_cid LIMIT 1;
  IF NOT FOUND THEN
    -- No pregnancy.
    -- This is a plain C pair of CYCLES.

    -- A regular C repstat has F, O, and L cycstats.

    IF last_ddate IS NOT NULL
       AND this_cycle_tdate IS NOT NULL THEN
      -- Make the L cycstat.
      PERFORM _make_cycstats(last_ddate
                             , this_cycle_tdate
                             , this_sname
                             , CAST ( 'bb_luetal' AS CHAR(1) )
                             , last_cid
                             , this_cycle_cid);
    END IF;

  END IF;

  -- All cycles have a C repstat with F and O cycstats.
  -- The length of the C varies, but the F and O are always
  -- based on the same variables.

  IF last_tdate IS NOT NULL
     AND last_ddate IS NOT NULL THEN
     -- Make F cycstat
     PERFORM _make_cycstats(last_tdate
                            , CAST ( last_ddate - CAST ('6 days' AS INTERVAL)
                                     AS DATE )
                            , this_sname
                            , CAST ( 'bb_follicular' AS CHAR(1) )
                            , last_cid
                            , last_cid);
  END IF;

  IF last_ddate IS NOT NULL THEN
     -- Make O cycstat
     PERFORM _make_cycstats(CAST ( last_ddate - CAST ('5 days' AS INTERVAL)
                                   AS DATE )
                            , CAST ( last_ddate - CAST ('1 days' AS INTERVAL)
                                     AS DATE )
                            , this_sname
                            , CAST ( 'bb_ovulating' AS CHAR(1) )
                            , last_cid
                            , last_cid);
  END IF;

  RETURN 0;
  END;
$$;

SELECT '_compute_cycstats(cycles.sname%TYPE
                           , cycles.cid%TYPE
                           , cycles.tdate%TYPE
                           , cycles.ddate%TYPE
                           , cycles.cid%TYPE
                           , cycles.tdate%TYPE)' AS done_with;
