dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2005, 2008, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Update CYCGAPDAYS from CYCGAPS after an insert operation.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl
dnl
dnl m4 includes
include(`copyright.m4')
include(`constants.m4')
include(`macros.m4')
dnl

CREATE OR REPLACE FUNCTION _cycgapdays_insert(cg cycgaps)
  RETURNS INT
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- Update CYCGAPDAYS from CYCGAPS after an insert operation.
  --
  -- GPL_notice(`  --', `2005, 2008', `Karl O. Pinc <kop@meme.com>')
  --
  -- Syntax: _cycgapdays_insert(cg)
  --
  -- Arguments:
  --  cg   An inserted CYCGAPS row.
  --
  -- Remarks:
  --  Called from the cycgaps triggers after the insert.
  --
  --  One non-obvious purpose is to allow people with no
  -- write permissions on CYCGAPDAYS to change the data.
  --
  --  The stragegy here is that if we find the CYCGAPS table in an
  -- illegal state we just don't do anything.  This gives the
  -- triggers a chance to fix things and call us again.
  --
  -- Bugs:

  DECLARE
    start_date cycgaps.date%TYPE;
    stop_date cycgaps.date%TYPE;
    this_date cycgaps.date%TYPE;

  BEGIN

    IF cg.code = 'bb_cg_point' THEN
      -- Points update one day.  Ta-Da!
      start_date := cg.date;
      stop_date := cg.date;

    ELSEIF cg.code = 'bb_cg_start' THEN
      -- Insert from here to the next end.
      start_date := cg.date;
      SELECT INTO stop_date date
        FROM cycgaps
        WHERE sname = cg.sname
              AND date > cg.date
              AND cg.code = 'bb_cg_end'
        ORDER BY date
        LIMIT 1;
      IF NOT(FOUND) THEN
        -- Something's wrong with the table.
        RETURN 0;
      END IF;

    ELSE
      -- cg.code = 'bb_cg_end'
      -- Insert from the previous start to here.
      stop_date := cg.date;
      SELECT INTO start_date date
        FROM cycgaps
        WHERE sname = cg.sname
              AND date < cg.date
              AND cg.code = 'bb_cg_start'
        ORDER BY date DESC
        LIMIT 1;
      IF NOT(FOUND) THEN
        -- Something's wrong with the table.
        RETURN 0;
      END IF;
    END IF;

    --
    -- Actually update the table.
    --
    this_date := start_date;
    WHILE this_date <= stop_date LOOP
      INSERT INTO cycgapdays (sname, date) VALUES (cg.sname, this_date);
      this_date := this_date + CAST('1 day' AS INTERVAL);
    END LOOP;

  RETURN 0;
  END;
$$;
grant_func_priv(`_cycgapdays_insert(cycgaps)')
