dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2005, 2008 Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl A census row is deleted, update members.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl
dnl
dnl m4 includes
include(`copyright.m4')
include(`constants.m4')

CREATE OR REPLACE FUNCTION _interp_delete(  this_sname  members.sname%TYPE
                                          , this_date   date
                                          , this_status census.status%TYPE)
  RETURNS VOID
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- A census row is deleted, update members.
  --
  -- GPL_notice(`  --', `2005, 2008', `Karl O. Pinc <kop@meme.com>')
  --
  -- Syntax: _interp_delete(this_sname, this_date, this_status);
  --
  -- Arguments:
  --   this_sname  Sname of individual
  --   this_date   Date of census row
  --   this_status Status code of the census row
  --
  -- Remarks:
  --  The comments use mathematical interval notation.  [ or ]
  -- mean a closed interval bound (lower and upper, respectively)
  -- where the endpoint is included in the interval.  ( or )
  -- mean an open interval bound (lower and upper, respectively)
  -- where the endpoint is not included in the interval.
  --
  DECLARE

  BEGIN
    -- RAISE INFO '_interp_delete(%, %, %)'
    --          , this_sname
    --          , this_date
    --          , this_status;
    -- Find the bounds of the interval which this delete effects.

    IF this_status = 'bb_absent' THEN
      -- See if there's another locating census on the same date.
      DECLARE
        -- Group on the census date.
        center_group census.grp%TYPE;
        -- Status on the census date.
        center_status census.status%TYPE;

      BEGIN
        SELECT INTO center_group, center_status
                    grp,          status
          FROM census
          WHERE sname = this_sname
                AND date = this_date
                AND status <> 'bb_absent'
          LIMIT 1;
        IF FOUND THEN
          -- Use the locating census to interpolate.
          IF center_status = 'bb_census'
             OR center_status = 'bb_demog'
             OR center_status = 'bb_manual' THEN
            -- Center status interpolates.
            PERFORM _interp_about_date(  this_sname
                                       , this_date
                                       , center_group
                                       , center_status);
          ELSE
            -- No locating and interpolating census.
            PERFORM _interp_about_date(  this_sname
                                       , this_date
                                       , NULL
                                       , this_status);
          END IF;
        ELSE
          -- No locating census.
          PERFORM _interp_about_date(this_sname, this_date, NULL, this_status);
        END IF;
      END;

    ELSEIF this_status = 'bb_census'
       OR this_status = 'bb_demog'
       OR this_status = 'bb_manual' THEN
      -- The census interpolates, it affects the nearest
      -- rows that also interpolate (and are not absent).
      PERFORM _interp_about_date(this_sname, this_date, NULL, this_status);

    ELSE
      -- Deleted row did not interpolate.
      PERFORM _interpish_about_date(  this_sname
                                    , this_date
                                    , NULL
                                    , this_status);
    END IF;

    RETURN;
  END;
$$;

SELECT '_interp_delete(  this_sname  members.sname%TYPE
                       , this_date   date
                       , this_status census.status%TYPE)' AS done_with;


