dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2005, 2008 Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl A census row is inserted, update members.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl
dnl
dnl m4 includes
include(`copyright.m4')
include(`constants.m4')

CREATE OR REPLACE FUNCTION _interp_insert(  this_sname  members.sname%TYPE
                                          , this_date   date
                                          , this_group  census.grp%TYPE
                                          , this_status census.status%TYPE)
  RETURNS VOID
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- A census row is inserted, update members.
  --
  -- GPL_notice(`  --', `2005, 2008', `Karl O. Pinc <kop@meme.com>')
  --
  -- Syntax: _interp_insert(this_sname, this_date, this_status);
  --
  -- Arguments:
  --   this_sname  Sname of individual
  --   this_date   Date of census row
  --   this_group  Group of the census row
  --   this_status Status code of the census row
  --
  -- Remarks:
  --  The comments use mathematical interval notation.  [ or ]
  -- mean a closed interval bound (lower and upper, respectively)
  -- where the endpoint is included in the interval.  ( or )
  -- mean an open interval bound (lower and upper, respectively)
  -- where the endpoint is not included in the interval.
  --

  DECLARE

  BEGIN
    IF this_status = 'bb_absent' THEN
      -- Inserted census is an absence.
      DECLARE
        -- Group on the date of the insert.
        -- (Needed for inserted absences.)
        center_group census.grp%TYPE := this_group;
        -- Status on date of the insert. (Again, needed for absences.)
        center_status census.status%TYPE := this_status;

      BEGIN
        SELECT INTO center_group, center_status
                    grp,          status
          FROM census
          WHERE sname = this_sname
                AND date = this_date
                AND status <> 'bb_absent'
          LIMIT 1;
        IF NOT FOUND THEN
          -- There's only an absence on this date.
          -- Interpolate over a single interval
          -- (A locating census on the same day would mean that we'd
          -- to interpolate over two intervals.)

          -- Find the bounds of the interval which this inserted
          -- absence effects.

          -- The census interpolates, it affects the nearest
          -- rows that also interpolate (and are not absent).
          -- (No center group.)

          PERFORM _interp_about_date(  this_sname
                                     , this_date
                                     , NULL
                                     , this_status);

        ELSE
          -- There's a census on the date as well as the newly
          -- inserted absence.
          -- Proceed to interpolate as if that census was inserted
          -- and the absence was already there.

          PERFORM _interp_about_date(  this_sname
                                     , this_date
                                     , center_group
                                     , center_status);
        END IF;
      END;
    ELSEIF this_status = 'bb_census'
           OR this_status = 'bb_demog'
           OR this_status = 'bb_manual' THEN
      -- Census at insertion interpolates.
      PERFORM _interp_about_date(  this_sname
                                 , this_date
                                 , this_group
                                 , this_status);
    ELSE
      -- Inserted row does not interpolate.
      PERFORM _interpish_about_date(  this_sname
                                    , this_date
                                    , this_group
                                    , this_status);
    END IF;

    RETURN;
  END;
$$;

SELECT '_interp_insert(  this_sname  members.sname%TYPE
                       , this_date   date
                       , this_group  census.grp%TYPE
                       , this_status census.status%TYPE)' AS done_with;

