dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2005, 2008 Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Find the midpoint between two dates, with rounding to even julian date
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl
dnl
dnl m4 includes
include(`copyright.m4')
include(`constants.m4')
dnl

CREATE OR REPLACE FUNCTION _interp_midpoint( early_bound DATE
                                           , late_bound DATE)
  RETURNS DATE
  LANGUAGE plpgsql
  IMMUTABLE
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- Find the midpoint between dates, rounding to an even julian date
  --
  -- GPL_notice(`  --', `2005, 2008', `Karl O. Pinc <kop@meme.com>')
  --
  -- Syntax: _interp_midpoint(this_sname, this_date, this_status);
  --
  -- Arguments:
  --   early_bound The earlier date
  --   late_bound  The later date, must be >= early_bound
  --
  -- Returns:
  --   A date half way between the early_bound date and the late_bound
  -- date.  This date is the (included) endpoint of the upper half
  -- of the interval.  The (included) endpoint of the lower half of
  -- the interval is the date - 1.
  --
  -- Side Effects: none
  --
  -- Remarks:
  --

  DECLARE
    days INT;
    midpoint DATE;

  BEGIN
    -- Determine the midpoint date, the last date on which to place
    -- the individual in the group.
    -- (Days in interval excluding endpoints.)
    days := late_bound - early_bound - 1;
    IF days % 2 <> 0 THEN
      -- Odd number of days in the interval.
      midpoint := early_bound + ((days + 1) / 2);
      IF julian(midpoint) % 2 <> 0 THEN
        -- "midpoint day" is an odd julian date
        -- "midpoint" is given to latter "half" of interval
        RETURN midpoint;
      ELSE
        -- "midpoint day" is an even julian date
        -- "midpoint" is given to earlier "half" of interval
        RETURN midpoint + 1;
      END IF;
    ELSE
      -- Even number of days in the interval.
      RETURN late_bound - days / 2;
    END IF;

  END;
$$;

SELECT '_interp_midpoint(DATE, DATE)' as done_with;
