dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2004, 2008, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Make a series of rows on REPSTATS for a particular state of a cycle.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl
dnl
dnl m4 includes
include(`copyright.m4')
include(`constants.m4')
include(`macros.m4')
dnl

CREATE OR REPLACE FUNCTION _make_repstats(start_date    date
                                        , end_date      date
                                        , this_sname    cycles.sname%TYPE
                                        , this_state    repstats.state%TYPE
                                        , this_cids     repstats.cids%TYPE
                                        , this_cide     repstats.cide%TYPE
                                        , this_pid      repstats.pid%TYPE)
  RETURNS INT
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- Make a series of rows on REPSTATS for a particular state of a cycle.
  --
  -- GPL_notice(`  --', `2004, 2008', `Karl O. Pinc <kop@meme.com>')
  --
  -- Syntax: _make_repstats(start_date
  --                              , end_date
  --                              , this_sname
  --                              , this_state
  --                              , this_cids
  --                              , this_cide
  --                              , this_pid)
  --
  -- Arguments:
  --  start_date       First date (inclusive) of series
  --  end_date         Last date (inclusive) of series
  --  this_sname       Sname for repstat
  --  this_state       State for repstat
  --  this_cids        Cids for repstat
  --  this_cide        Cide for repstat
  --  this_pid         Pid for repstat
  --
  -- Remarks:
  --  Called only by other babase functions.
  --
  -- Bugs:
  --  Could fail at epoch 32 bit int rollover (coming in 2036?).

  DECLARE
    days INT;
    seconds_in_day CONSTANT INT DEFAULT 86400;

  BEGIN

  days := ((CAST ( EXTRACT(EPOCH FROM end_date) AS INT )
            - CAST ( EXTRACT(EPOCH FROM start_date) AS INT ))
           / seconds_in_day)
          + 1;
  FOR daycnt IN 1..days LOOP
    INSERT INTO repstats (date, sname, state, dins, dr, cids, cide, pid)
                         values (start_date
                                  + CAST ( daycnt - 1 || ' days' AS INTERVAL)
                                 , this_sname
                                 , this_state
                                 , daycnt
                                 , days - daycnt
                                 , this_cids
                                 , this_cide
                                 , this_pid);
  END LOOP;

  RETURN 0;
  END;
$$;

SELECT '_make_repstats(date
                       , date
                       , cycles.sname%TYPE
                       , repstats.state%TYPE
                       , repstats.cids%TYPE
                       , repstats.cide%TYPE
                       , repstats.pid%TYPE)' AS done_with;
