dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2006, 2008, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Recalculate Seq and Series for a female's CYCLES from a point forward.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl
dnl
dnl m4 includes
include(`copyright.m4')
include(`constants.m4')
include(`macros.m4')
dnl

SELECT '_recalc_seq_series' AS function;
CREATE OR REPLACE FUNCTION _recalc_seq_series(this_sname cycgaps.sname%TYPE
                                 , start_date cycpoints.date%TYPE
                                 , new_cid cycpoints.cid%TYPE)
  RETURNS void
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  --
  -- Recalculate Seq and Series.
  --
  -- GPL_notice(`  --', `2006, 2008', `Karl O. Pinc <kop@meme.com>')
  --
  -- Syntax: _recalc_seq_series(this_sname cycgaps.sname%TYPE
  --                          , start_date cycpoints.date%TYPE
  --                          , new_cid cycpoints.cid%TYPE)
  --
  -- Input:
  --  this_sname  The individual we are concerned with.
  --  start_date  The earliest date affected.
  --  new_cid     The cid of the row that's caused the re-computation.
  --
  -- Side Effects:
  --   Updates CYCLES.Seq and Series, when necessary.
  --
  -- Remarks:
  -- We use brute force, find the earliest possible affected
  -- date and recalculate everything from there forward.
  --
  -- Bugs:

  DECLARE
    next_seq cycles.seq%TYPE;
    next_series cycles.series%TYPE;
    cy RECORD;

    last_date cycpoints.date%TYPE; -- Date of last cycle.

  BEGIN
--RAISE INFO '_recalc_seq_series(%, %, %)', this_sname, start_date, new_cid;
    -- Find the initial seq and series values.

    -- Start with the seq and series before the one that's triggered
    -- the recomputation.
    SELECT INTO next_seq,   next_series,   last_date
                cycles.seq, cycles.series, cycpoints.date
      FROM cycles, cycpoints
      WHERE cycles.sname = this_sname
            AND cycpoints.cid = cycles.cid
            AND cycpoints.date < start_date
            AND cycles.cid <> new_cid
      ORDER BY cycles.seq DESC
      LIMIT 1;
    IF NOT FOUND THEN
      next_seq := 0;
      next_series := 1;
    END IF;
--RAISE INFO '_recalc: next_seq %: next_series %', next_seq, next_series;
    -- Iterate over all cycles from (including) the starting date forward.
    FOR cy IN SELECT cycles.cid
                   , (SELECT cycpoints.date
                        FROM cycpoints
                        WHERE cycpoints.cid = cycles.cid
                              -- AND cycpoints.code <> 'bb_bleeding'
                        ORDER BY cycpoints.date
                        LIMIT 1) AS date
                   , cycles.seq
                   , cycles.series
                FROM cycles
                WHERE sname = this_sname
                      AND cid IN (SELECT DISTINCT cycles.cid
                                    FROM cycles, cycpoints
                                    WHERE cycles.sname = this_sname
                                          AND cycpoints.cid = cycles.cid
                                          AND cycpoints.date >= start_date)
                ORDER BY date
            LOOP

      -- Determine the seq
      next_seq := next_seq + 1;

      -- Determine the series
      PERFORM 1
        FROM cycgapdays
        WHERE sname = this_sname
              AND date > last_date  -- cleverly is not true if no last date
              AND date < cy.date
        LIMIT 1;
      IF FOUND THEN
        -- There's a gap betwen this series and the last.
        next_series := next_series + 1;
      END IF;

      IF cy.seq <> next_seq
         OR cy.seq IS NULL
         OR cy.series <> next_series
         OR cy.series IS NULL THEN
        -- Set the proper values.
        UPDATE cycles
          SET seq = next_seq
              , series = next_series
          WHERE cid = cy.cid;
      END IF;

      -- Prepare for next iteration.
      last_date := cy.date;

    END LOOP;

  RETURN;
  END;
$$;
grant_func_priv(`_recalc_seq_series(cycgaps.sname%TYPE, cycpoints.date%TYPE, cycles.cid%TYPE)')
