dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2006, 2008, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Remove an automatic Mdate.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl
dnl
dnl m4 includes
include(`copyright.m4')
include(`constants.m4')
include(`macros.m4')
dnl

CREATE OR REPLACE FUNCTION _remove_autom(this_sname cycles.sname%TYPE
                                       , ddate cycpoints.date%TYPE
                                       , this_cpid cycpoints.cpid%TYPE)
  RETURNS VOID
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  -- Given a Ddate remove any Mdate it automatically generated.
  --
  -- GPL_notice(`  --', `2006, 2008', `Karl O. Pinc <kop@meme.com>')
  --
  -- Syntax: _remove_autom(this_sname cycles.sname
  --                     , ddate cycpoints.date
  --                     , this_cpid cycpoints.cpid)
  --
  -- Input:
  --  this_sname  The individual we are concerned with.
  --  ddate       The Ddate that may have an associated automatic Mdate
  --  this_cpid   The cpid of the row that's triggering the change.
  --
  -- Side Effects:
  --   Deletes automatically generated Mdate, if any.
  --
  -- Remarks:
  --   Used in triggers.
  --
  --   Does not look for gaps because it assumes that any automatic
  -- Mdates that exist 'follow the rules' and are not created
  -- unless there's a preceeding Ddate and no gap.
  --
  --   The assumption here is that the AFTER triggers are fired after
  -- each row is inserted, but before the next row is inserted.
  --
  -- Bugs:
  DECLARE
    cp_row RECORD;

  BEGIN

--RAISE INFO '_remove_autom(%, %, %)', this_sname, ddate, this_cpid;
    -- Find the old automatic Mdate.

    -- Find the cycpoints row following the old date.
    SELECT INTO cp_row
           cycpoints.cpid
         , cycpoints.cid
         , cycpoints.date
         , cycpoints.code
         , cycpoints.source
      FROM cycpoints, cycles
      WHERE cycles.sname = this_sname
            AND cycpoints.cid = cycles.cid
            AND cycpoints.date > ddate
            AND cycpoints.cpid <> this_cpid
      ORDER BY cycpoints.date, cycpoints.code
      LIMIT 1;
    IF FOUND 
       AND cp_row.code = 'bb_menses'
       AND cp_row.source = 'bb_auto' THEN
      -- Found an automatic M date.
      -- Remove it.
      DELETE FROM cycpoints WHERE cpid = cp_row.cpid;
    END IF;

    RETURN;
  END;
$$;
grant_func_priv(`_remove_autom(cycles.sname%TYPE
                               , cycpoints.date%TYPE
                               , cycpoints.cpid%TYPE)')
