dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2011, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Create a PostGIS (geometry) point in the WGS 84 / UTM zone 37 South
dnl coordinate system.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl
dnl
dnl m4 includes
include(`copyright.m4')
include(`constants.m4')
include(`macros.m4')
dnl

CREATE OR REPLACE FUNCTION bb_makepoint(x DOUBLE PRECISION, y DOUBLE PRECISION)
  RETURNS geometry
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- One line description
  --
  -- Copyright (C) 2012 The Meme Factory, Inc.  http://www.meme.com/
  -- GPL_notice(`  --', `2011', `Karl O. Pinc <kop@meme.com>')
  --
  -- Syntax: bb_makepoint(x, y)
  --
  -- Arguments:
  --  x  The (double precision) X coordinate
  --  y  The (double precision) Y coordinate
  --
  -- Remarks:
  -- Create a PostGIS (geometry) point in the WGS 1984 UTM zone 37 South
  -- coordinate system.
  --
  -- Bugs:

  DECLARE

  BEGIN

  IF x IS NOT NULL AND y IS NOT NULL THEN
    RETURN ST_SetSRID(ST_MakePoint(x, y), bb_amboseli_srid);
  ELSIF x IS NULL AND y IS NULL THEN
    RETURN NULL;
  ELSE
    RAISE EXCEPTION data_exception USING
          MESSAGE = 'Error in bb_makepoint(): Either both X and Y must be NULL or neither can be NULL'
        , DETAIL = 'Value (X) = (' || textualize(`x')
                   || '), Value (Y) = (' || textualize(`y') || ')';
  END IF;
  END;
$$;
grant_func_priv(`bb_makepoint(DOUBLE PRECISION, DOUBLE PRECISION)')
