dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2006, 2008, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Create the cycgaps file by hand according to lab records.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl
dnl
dnl m4 includes
include(`copyright.m4')
include(`constants.m4')
include(`macros.m4')
dnl

dnl plpgsql fragment for a gap in a group.
dnl
dnl Syntax: do_gap(grpint, gstart, gstop)
dnl
dnl grpint  The interger portion of the group
dnl gstart  The starting date of the gap
dnl gstop   The ending date of the gap
dnl
dnl  It's a little brutal to make a macro for this rather than
dnl a function, but seeing as how we only want to use this
dnl during the conversion I want as few functions as possible.
dnl
changequote({,})
define({do_gap},{



    -- Go over each female in Altos & decended groups
    FOR thisrow IN SELECT biograph.sname
                        , biograph.matgrp
                        , maturedates.matured
                        , biograph.status
                        , biograph.statdate
                     FROM biograph, maturedates
                     WHERE biograph.sex = 'F'
                           AND CAST(biograph.matgrp AS int) = $1
                           AND maturedates.sname = biograph.sname
                     ORDER BY sname
                LOOP
      -- Our time period gaps are:
      -- Jan 1, 1990 - Dec 31, 1995 inclusive

      lastobsend := NULL;
      laststartstate := NULL;      

      -- Observation ends on or after the matdate
      IF thisrow.matured <= '$3' 
         -- There are cycles during the gap.
         AND (thisrow.statdate >= '$2' OR thisrow.status = 'bb_alive')
         -- Observation should have continued into the gap.
         THEN

--        IF thisrow.matured < '$2' THEN
          -- There was observation before the gap.

          --
          -- Mark the start of the gap.
          -- Might be nice to base this on maturity date, but I don't
          -- always have the maturity date.
          --
          INSERT INTO cycgaps (sname, date, code)
                 VALUES (thisrow.sname, '$2'::date - '1 day'::interval, 'bb_cg_end');
--        END IF;

        --
        -- Make each cycle in the gap it's own series of observations.
        --
        FOR crow IN SELECT DISTINCT cycles.cid
                      FROM cycles, cycpoints
                      WHERE cycles.sname = thisrow.sname
                            AND cycpoints.cid = cycles.cid
                            AND cycpoints.date >= '$2'
                            AND cycpoints.date <= '$3'
          LOOP

          -- Find the earliest and latest dates in the cycle.
          SELECT INTO obsstart, startcode
                      date,     code
                 FROM cycpoints
                 WHERE cycpoints.cid = crow.cid
                 ORDER BY date
                 LIMIT 1;
          SELECT INTO obsend
                      date
                 FROM cycpoints
                 WHERE cycpoints.cid = crow.cid
                 ORDER BY date DESC
                 LIMIT 1;

          IF obsstart < '$2' THEN
            -- Cycle starts before the gap, it must end in the
            -- gap so delete the start of the gap and use the end
            -- of the cycle to start.
            DELETE FROM cycgaps
                   WHERE sname = thisrow.sname
                         AND date = '$2'::date - '1 day'::interval;

            -- End the observation in the gap.
            INSERT INTO cycgaps (sname, date, code)
                   VALUES (thisrow.sname, obsend, 'bb_cg_end');
          ELSE
            -- Entire observation is after start of the gap.

            -- Start an observation in the gap.

            -- What state should it have.
            IF startcode = 'bb_turgesence' THEN
              -- Need to look at when the cycle's ddate is to figure
              -- out the state.
              SELECT INTO ddate date
                FROM cycpoints
                WHERE cid = crow.cid
                      AND code = 'bb_deturgesence';
              IF FOUND THEN
                IF ddate > obsstart + 'bb_ovulation_len days'::interval THEN
                  -- Ovulation period is longer than the interval between
                  -- ddate and tdate.
                  startstate := 'bb_cg_swelling';
                ELSE
                  -- Ovulating.
                  startstate := 'bb_cg_ovulating';
                END IF;
              ELSE
                -- When there's no ddate the state is swelling.
                startstate := 'bb_cg_swelling';
              END IF;
            ELSE
              -- Mdate or Ddate, just use the cycle's code as the state.
              -- (Cheating here because this is for the conversion,
              -- should really use the constants.)
              startstate := startcode;
            END IF;

            -- End the observation in the gap.
            IF obsstart = obsend THEN
              -- It's a point observation.
              cgstartcode := 'bb_cg_point';
            ELSE
              cgstartcode := 'bb_cg_start';
              -- We need to end the observation.
              INSERT INTO cycgaps (sname, date, code)
                     VALUES (thisrow.sname, obsend, 'bb_cg_end');
            END IF;

            -- Finally, the start of the observation.
            IF NOT EXISTS (SELECT 1
                             FROM cycgaps
                             WHERE cycgaps.sname = thisrow.sname
                                   AND cycgaps.date = obsstart) THEN
              -- Avoid duplicate point observations. (LIS, 1979-06-15)

              -- Points right after ends are no good.
              -- (PIC on 1990-01-01)
              IF cgstartcode = 'bb_cg_point'
                 AND  EXISTS (SELECT 1
                                FROM cycgaps
                                WHERE cycgaps.sname = thisrow.sname
                                      AND cycgaps.date = obsstart - 1
                                      AND cycgaps.code = 'bb_cg_end') THEN
                -- Move the end of observation one day forward.
                UPDATE cycgaps
                  SET date = date + 1
                  WHERE cycgaps.sname = thisrow.sname
                        AND cycgaps.date = obsstart - 1;
              ELSE
                -- Go ahead and insert the point.
                INSERT INTO cycgaps (sname, date, code, state)
                      VALUES (thisrow.sname, obsstart, cgstartcode, startstate);
              END IF;

            END IF;

          END IF;

          -- Keep track of where we ended observation.
          IF lastobsend IS NULL OR obsend > lastobsend THEN
            lastobsend := obsend;
            laststartstate := startstate;
          END IF;

        END LOOP;

        --
        -- Mark the end of the gap.
        --
        IF thisrow.status = 'bb_alive'
           OR thisrow.statdate > '$3' THEN
          -- Didn't die during gap.  Resume observation after gap.

          IF lastobsend >= '$3' THEN
            -- No need to start observations, we can simply
            -- not stop them.
            IF laststartstate = 'bb_cg_point' THEN
              UPDATE cycgaps
                SET state = 'bb_cg_start'
                WHERE sname = thisrow.sname
                      AND date = lastobsend;
            ELSE
              DELETE FROM cycgaps
                WHERE sname = thisrow.sname
                      AND date = lastobsend;
            END IF;
          ELSE

            -- Find the appropriate state for resumption of observation.
            SELECT INTO startcode,      crow.cid,   this_cpid
                        cycpoints.code, cycles.cid, cycpoints.cpid
              FROM cycles, cycpoints
              WHERE cycles.sname = thisrow.sname
                    AND cycpoints.cid = cycles.cid
                    AND date > '$3'
              ORDER BY cycpoints.date, cycpoints.code
              LIMIT 1;

            IF FOUND THEN
              -- Don't bother resuming observation if there's no more
              -- sexual cycles.

            -- Horrible duplicate code starts here.
            IF startcode = 'bb_turgesence' THEN
              -- See if there's a pregnancy going on.
              SELECT INTO this_pid pid
                FROM pregs
                WHERE resume = this_cpid
                LIMIT 1;
              IF FOUND THEN
                -- Figure out whether the start of observation is before
                -- or after birth.
                PERFORM 1
                  FROM biograph
                  WHERE pid = this_pid
                        AND birth <= '$3'
                  LIMIT 1;
                IF FOUND THEN
                  -- End of the gap is on or after the birth.
                  startstate := 'bb_cg_lactating';
                ELSE
                  -- No birth found before the end of the gap.
                  startstate := 'bb_cg_pregnant';
                END IF;
              ELSE
                -- Tdate is not a resumption after pregnancy.
                -- Need to look at when the cycle's ddate is to figure
                -- out the state.
                SELECT INTO ddate date
                  FROM cycpoints
                  WHERE cid = crow.cid
                        AND code = 'bb_deturgesence';
                IF FOUND THEN
                  IF ddate > obsstart + 'bb_ovulation_len days'::interval THEN
                    -- Ovulation period is longer than the interval between
                    -- ddate and tdate.
                    startstate := 'bb_cg_swelling';
                  ELSE
                    -- Ovulating.
                    startstate := 'bb_cg_ovulating';
                  END IF;
                ELSE
                  -- When there's no ddate the state is swelling.
                  startstate := 'bb_cg_swelling';
                END IF;
              END IF;
            ELSE
              -- Mdate or Ddate, just use the cycle's code as the state.
              -- (Cheating here because this is for the conversion,
              -- should really use the constants.)
              startstate := startcode;
            END IF;

            INSERT INTO cycgaps (sname, date, code, state)
                   VALUES (thisrow.sname, '$3'::date + '1 day'::interval, 'bb_cg_start', startstate);
            END IF;
          END IF;
        END IF;

      END IF;
    END LOOP;
})dnl
changequote(`,')dnl


CREATE OR REPLACE FUNCTION _install_cycgaps()
  RETURNS INT
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- Create the cycgaps table by hand according to lab records.
  --
  -- GPL_notice(`  --', `2006, 2008', `Karl O. Pinc <kop@meme.com>')
  --
  -- Syntax: _install_cycgaps()
  --
  -- Arguments:
  --
  -- Remarks:
  --
  -- Bugs:

  DECLARE

    thisrow RECORD;
    crow RECORD;
    obsstart DATE;
    startcode cycpoints.code%TYPE;
    startstate cycgaps.state%TYPE;
    obsend DATE;
    ddate DATE;
    cgstartcode cycgaps.code%TYPE;

    this_cpid cycpoints.cpid%TYPE;
    this_pid pregs.pid%TYPE;

    lastobsend DATE;
    laststartstate cycgaps.state%TYPE;

  BEGIN

    -- Start with all the females.
    -- 
    -- babase=> select matgrp, groups.name, min(date), max(date) from biograph, cycles, cycpoints, groups where biograph.sname = cycles.sname and cycles.cid = cycpoints.cid and groups.gid = biograph.matgrp group by biograph.matgrp, groups.name;
    --  matgrp |   name   |    min     |    max
    -- --------+----------+------------+------------
    --    1.00 | Alto's   | 1971-07-09 | 2005-05-29
    --    1.10 | Nyayo's  | 1993-01-22 | 2005-06-29
    --    1.20 | Dotty's  | 1993-08-21 | 2005-06-25
    --    1.21 | Omo's    | 2005-04-06 | 2005-04-28
    --    1.22 | Viola's  | 2004-04-28 | 2005-06-26
    --    2.00 | Hook's   | 1978-06-01 | 2005-06-16
    --    2.10 | Linda's  | 2000-11-06 | 2005-06-18
    --    2.20 | Weaver's | 2000-01-07 | 2005-06-26
    --    3.00 | Lodge    | 1984-02-05 | 1997-09-12
    -- 
    -- Altos start July 1, 1971.
    -- 
    -- Anyone in Altos, Dotty's, or Nyayo's from Jan 1, 1990 through
    -- Dec 31, 1995, inclusive, only gets observations that cover the
    -- cycles for these periods.
    -- 
    -- 
    -- 
    -- Proton's after Dec 31, 1990 only gets observed during cycles.
    -- 
    -- 
    -- Hooks has observations before this.
    -- 
    -- Begin date HOOKS (2.00)       01 Feb 1981
    -- End date HOOKS (2.00)         28 Feb 1995
    -- 
    -- Begin date LINDAS (2.10)      01 Mar 1995
    -- 
    -- Begin date WEAVERS (2.20) 01 Mar 1995
    -- 
    -- Begin Lodge (3.0)1 Jan 1985
    -- End Lodge 31 Dec 1995
    -- 
    -- Joy (3.1) and Nzige (3.2):
    -- Begin 1 Jan 1996
    -- End    31 Dec 1996
    -- 
    -- So, Hooks before start date and Altos in the gap.
    -- Otherwise everything is observed.
    -- So, our job is to make gaps for these periods.

    -- First, clear the cycgaps table.
    DELETE FROM cycgaps;

    -- Altos et-al
    do_gap(1, `Jan 1, 1990', `Dec 31, 1995')
    -- Hooks et-al
    do_gap(2, `Jan 1, 1978', `Jan 31, 1981')
    -- Lodge et-al
    do_gap(3, `Jan 1, 1984', `Dec 31, 1985')

  -- Well, for Hooks and Lodge, there's not enough data to want to
  -- write code.  But seeing as how we pretty much have the
  -- code already...  Anyhow, here's the data:
  -- babase=> select biograph.sname, cycpoints.code, cycpoints.date, cycles.cid from biograph, cycles, cycpoints where cast(biograph.matgrp as int) = 2 and sex = 'F' and cycles.sname = biograph.sname and cycpoints.cid = cycles.cid and cycpoints.date < '1 FEB, 1981' order by biograph.sname, cycpoints.date, cycpoints.code;

  --babase=> select biograph.sname, cycpoints.code, cycpoints.date, cycles.cid from biograph, cycles, cycpoints where cast(biograph.matgrp as int) = 3 and sex = 'F' and cycles.sname = biograph.sname and cycpoints.cid = cycles.cid and cycpoints.date < '1 JAN, 1985' order by biograph.sname, cycpoints.date, cycpoints.code;



-- These are the conception dates for transitions from parity in the
-- 1s into the 100s.  They should all be preceeded by a gap.
-- (But are not.)
-- Data from the first conversion without triggers.
--
-- 
-- babase=> select biograph.sname, biograph.matgrp, cycpoints.date from pregs, cycpoints, cycles, biograph where pregs.pid = (select min(mp.pid) from pregs as mp, cycles as mc, cycpoints as mcp where mc.sname = biograph.sname and mcp.cid = mc.cid and mp.conceive = mcp.cpid and mp.parity > 100 limit 1) and cycpoints.cpid = pregs.conceive and cycles.cid = cycpoints.cid and biograph.sname = cycles.sname order by biograph.matgrp, biograph.sname;
--  sname | matgrp |    date
-- -------+--------+------------
--  ALT   |   1.00 | 1972-12-23
--  EST   |   1.00 | 1972-01-05
--  FLU   |   1.00 | 1971-12-23
--  JAN   |   1.00 | 1972-07-15
--  JUD   |   1.00 | 1972-10-10
--  MOM   |   1.00 | 1973-01-09
--  OVA   |   1.00 | 1971-11-01
--  PRE   |   1.00 | 1973-04-21
--  RIN   |   1.00 | 1971-11-07
--  TTX   |   1.00 | 1971-08-01
--  ETA   |   2.00 | 1982-07-07
--  KUP   |   2.00 | 1980-09-15
--  LIS   |   2.00 | 1979-06-15
--  LOI   |   2.00 | 1981-08-08
--  NUB   |   2.00 | 1980-11-23
--  PIN   |   2.00 | 1980-10-10
--  RUK   |   2.00 | 1980-10-25
--  JAD   |   3.00 | 1985-03-27
--  MAL   |   3.00 | 1985-05-03
--  NDO   |   3.00 | 1984-05-25
--  REM   |   3.00 | 1985-06-01
--  UMB   |   3.00 | 1984-06-17
--  ZUM   |   3.00 | 1985-08-07
-- (23 rows)



  RETURN 0;
  END;
$$;
grant_func_priv(`_install_cycgaps()')
