dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2006, 2008, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl For the conversion.  Install the cycgaps we use to load the pregs.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl
dnl
dnl m4 includes
include(`copyright.m4')
include(`constants.m4')
include(`macros.m4')
dnl

CREATE OR REPLACE FUNCTION _install_cycgaps_pregs(action TEXT)
  RETURNS INT
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- One line description
  --
  -- GPL_notice(`  --', `2006, 2008', `Karl O. Pinc <kop@meme.com>')
  --
  -- Syntax: _install_cycgaps_pregs(action)
  --
  -- Arguments:
  --    action  Either 'install' or 'uninstall', depending on whether
  --            the rows should be added or removed.
  --
  -- Remarks:
  --
  -- When the conversion runs it loads the pregs before setting
  -- the resume column so that it can load all the pid columns
  -- in biograph.  To allow this we need gaps between each pregnancy.
  -- This program creates those gaps from the conversion.cycgaps_pregs
  -- table.
  --
  -- Program exists so that we don't have one giant update statement
  -- so we can control what order things go in.
  --
  -- Bugs:

  DECLARE
    cg cycgaps;

  BEGIN

  FOR cg IN SELECT * FROM conversion.cycgaps_pregs ORDER BY sname, date LOOP
    IF action = 'install' THEN
      IF EXISTS (SELECT 1
                   FROM cycgapdays
                   WHERE cycgapdays.sname = cg.sname
                         AND cycgapdays.date = cg.date) THEN
        RAISE INFO '%: %: Not inserting a gap to get a pregnancy in, already in a gap'
                 , cg.sname
                 , cg.date;
      ELSE
--        BEGIN TRANSACTION;
        INSERT INTO cycgaps (sname, date, code, state)
          values (cg.sname, cg.date, cg.code, cg.state);
--        COMMIT TRANSACTION;
      END IF;

    ELSE  -- action = 'uninstall'

      IF EXISTS (SELECT 1
                   FROM cycgaps
                   WHERE cycgaps.sname = cg.sname
                         AND cycgaps.date = cg.date
                         AND cycgaps.code = cg.code
                         AND cycgaps.state = cg.state) THEN
        -- There something where we did an insert, probably.
        -- Get rid of it.
--        BEGIN TRANSACTION;
        UPDATE cycgaps
          SET date = cg.date - 2
          WHERE sname = cg.sname
                AND date = cg.date;
--        COMMIT TRANSACTION;
      END IF;

    END IF;

  END LOOP;

  RETURN 0;
  END;
$$;
grant_func_priv(`_install_cycgaps_pregs(TEXT)')
