dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2009, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Template function, never executed.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl
dnl
dnl m4 includes
include(`copyright.m4')
include(`constants.m4')
include(`macros.m4')
dnl

CREATE OR REPLACE FUNCTION date_mod(period TEXT, ts TIMESTAMP)
  RETURNS INTERVAL
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- Return the remainder of a timestamp divided by a time period.
  --
  -- GPL_notice(`  --', `2009', `Karl O. Pinc <kop@meme.com>')
  --
  -- Syntax: date_mod(text, timestamp)
  --
  -- Arguments:
  --   text    String representing a time period. I.E. 'day', 'hour',
  --           etc.  Takes the same text arguments as the
  --           date_trunc(text, timestamp) function.
  --   ts      Value to take the remainder of.
  --
  -- Remarks:
  --   Like the numeric modulo function, only for timestamp values.
  --   Used to get, e.g., date_mod('day', timestamp) returns
  --   an interval representing the hours, minutes, seconds,
  --   and sub-seconds portion of the timestamp.
  --
  -- Bugs:

  DECLARE

  BEGIN

  RETURN ts - date_trunc(period, ts);
  END;
$$;
grant_func_priv(`date_mod(TEXT, TIMESTAMP)')


CREATE OR REPLACE FUNCTION date_mod(period TEXT, ts INTERVAL)
  RETURNS INTERVAL
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- Return the remainder of an interval divided by a time period.
  --
  -- GPL_notice(`  --', `2009', `Karl O. Pinc <kop@meme.com>')
  --
  -- Syntax: date_mod(text, interval)
  --
  -- Arguments:
  --   text    String representing a time period. I.E. 'day', 'hour',
  --           etc.  Takes the same text arguments as the
  --           date_trunc(text, timestamp) function.
  --   ts      Value to take the remainder of.
  --
  -- Remarks:
  --   Like the numeric modulo function, only for timestamp values.
  --   Used to get, e.g., date_mod('day', timestamp) returns
  --   an interval representing the hours, minutes, seconds,
  --   and sub-seconds portion of the timestamp.
  --
  -- Bugs:

  DECLARE

  BEGIN

  RETURN ts - date_trunc(period, ts);
  END;
$$;
grant_func_priv(`date_mod(TEXT, INTERVAL)')
