dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2009, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Template function, never executed.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl
dnl
dnl m4 includes
include(`copyright.m4')
include(`constants.m4')
include(`macros.m4')
dnl

CREATE OR REPLACE FUNCTION delete_census(this_sname CHAR(3)
                                         , from_date DATE
                                         , through_date DATE)
  RETURNS INT
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- Delete rows from census one at a time
  --
  -- Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
  -- GPL_notice(`  --', `2009', `Karl O. Pinc <kop@meme.com>')
  --
  -- Syntax: delete_census(sname, from_date, through_date)
  --
  -- Arguments:
  --  sname   Individual having the census rows to delete
  --  from_date    Earliest date to delete (inclusive)
  --  through_date Latest date to delete (inclusive)
  --
  -- Remarks:
  --  Handy for deleting the old style census code because deleting them
  -- in multiple rows at a time is something like a O(n^2) operation.
  --
  --  Deletes more recent rows first.
  --
  -- Bugs:
  --   Only needed because the intepolation is very slow with old style
  -- non-interpolating census rows.

  DECLARE
    this_record RECORD;

  BEGIN

    FOR this_record IN SELECT census.date 
                         FROM census
                         WHERE census.sname = this_sname
                               AND census.date >= from_date
                               AND census.date <= through_date
                         ORDER BY census.date desc
                    LOOP
      DELETE FROM census
             WHERE census.sname = this_sname
                   AND census.date = this_record.date;
    END LOOP;

  RETURN 0;
  END;
$$;
grant_func_priv(`delete_census(CHAR(3), DATE, DATE)')
