dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2010, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Return the hydrological year in which a given date falls.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl
dnl
dnl m4 includes
include(`copyright.m4')
include(`constants.m4')
include(`macros.m4')
dnl
dnl Make plpg/sql function that returns the hydrological year as an integer
dnl given different input data types.
dnl
dnl Syntax: _make_hydroyear_func(datatype, typecast)
dnl
dnl Input:
dnl   datatype  The postgres data type the function takes as input.
dnl   typecast  The "::foo" typecast to cast datatype to a date,
dnl             or nothing if no typecase is needed.
dnl
dnl Variables required:
dnl Restrictions:
changequote({,})
define({_make_hydroyear_func},{

CREATE OR REPLACE FUNCTION hydroyear(this_date $1)
  RETURNS INT
  LANGUAGE plpgsql
  IMMUTABLE
  bb_function_set_search_path
  AS $$

  -- Return the hydrological year in which a given date falls.
  --
  -- GPL_notice(`  --', `2010', `Karl O. Pinc <kop@meme.com>')
  --
  -- Syntax:  hydroyear(date)
  --
  -- Input:
  --   date  The date in question.  A $1.
  --
  -- Returns:
  --  The hydrological year.
  --
  -- Remarks:

  DECLARE
    hy INT := EXTRACT(YEAR FROM this_date$2);
    m DOUBLE PRECISION := EXTRACT(MONTH FROM this_date$2);

  BEGIN
    IF m > bb_hy_first_month
       OR (m = bb_hy_first_month
           AND EXTRACT(DAY FROM this_date$2) >=bb_hy_first_day) THEN
      -- The number of the year is the calendar year of the last day
      -- of the hydrological year.
      hy := hy + 1;
    END IF;

    RETURN hy;
  END;
$$;

grant_everybody_func_priv(`hydroyear($1)')
})
changequote(`,')dnl

-- Make the functions.
_make_hydroyear_func(`DATE', `')
_make_hydroyear_func(`TIMESTAMP', `')
_make_hydroyear_func(`TEXT', `::TIMESTAMP')
