dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2004, 2008, 2010 Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Julian Day functions to support babase on the server side.
dnl Karl O. Pinc <kop@meme.com>
dnl
dnl
dnl m4 includes
include(`copyright.m4')
include(`constants.m4')
include(`macros.m4')
dnl

--
-- Babase Julian Day Postgresql functions.
-- $Id: julian.m4,v 1.3 2004/09/13 20:42:47 kop Exp $
--

--
-- General purpose date functions.
--

dnl Make plpg/sql function that returns the julian date as an integer
dnl given different input data types.
dnl
dnl Syntax: _make_julian_func(datatype, caststr)
dnl
dnl Input:
dnl   datatype  The postgres data type the function takes as input.
dnl   caststr   The postgres cast, beginning with the 2 characters
dnl             '::', that cast the input data type to a pg DATE.
dnl
dnl Variables required:
dnl Restrictions:
changequote({,})
define({_make_julian_func},{

CREATE OR REPLACE FUNCTION julian(this_date $1)
  RETURNS INT
  LANGUAGE plpgsql
  IMMUTABLE
  bb_function_set_search_path
  AS $$

  -- Convert a date into its Julian Day.
  --
  -- GPL_notice(`  --', `2004, 2008', `Karl O. Pinc <kop@meme.com>')
  --
  -- Syntax:  julian(date)
  --
  -- Input:
  --   date  The date to convert.
  --
  -- Returns:
  --  The Julian Day of the date.
  --
  -- Remarks:
  --   Really ought to be built-into postgresql.

  DECLARE
-- *wrong value!*    day_zero CONSTANT DATE := TO_DATE('0', 'J');
    day_zero CONSTANT DATE
             := CURRENT_DATE - CAST (to_char(CURRENT_DATE, 'J') AS INT);

  BEGIN
    RETURN this_date$2 - day_zero;
  END;
$$;

grant_everybody_func_priv(`julian($1)')
})
changequote(`,')dnl

-- Make the functions.
_make_julian_func(`DATE', `')
_make_julian_func(`TIMESTAMP', `::DATE')


CREATE OR REPLACE FUNCTION julian_to(julian_day INT)
  RETURNS DATE
  LANGUAGE plpgsql
  IMMUTABLE
  bb_function_set_search_path
  AS $$

  -- Convert a Julian Day to its corresponding date.
  --
  -- GPL_notice(`  --', `2004, 2008', `Karl O. Pinc <kop@meme.com>')
  --
  -- Syntax:  julian_to(julian_day)
  --
  -- Input:
  --   julian_date  The date to convert.
  --
  -- Returns:
  --  The date value for the given Julian Day.
  --
  -- Remarks:
  --   Really ought to be built-into postgresql.

  DECLARE
    day_zero CONSTANT DATE
             := CURRENT_DATE - CAST (to_char(CURRENT_DATE, 'J') AS INT);

  BEGIN
    RETURN day_zero + julian_day;
  END;
$$;

grant_everybody_func_priv(`julian_to(INT)')
