dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2004, 2008, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Reinterpolate the entire MEMBERS table.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl
dnl
dnl m4 includes
include(`copyright.m4')
include(`constants.m4')
include(`macros.m4')
dnl

CREATE OR REPLACE FUNCTION rebuild_all_members()
  RETURNS INT
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- Reinterpolate the entire MEMBERS table.
  --
  -- GPL_notice(`  --', `2004, 2008', `Karl O. Pinc <kop@meme.com>')
  --
  -- Syntax: rebuild_all_members()
  --
  -- Arguments:
  --
  -- Remarks:
  -- Not as efficent as it could be, as it does not clear the
  -- entire table iteslf but works member-at-a-time, but
  -- the permissions work out neatly this way.
  -- Assumes the census triggers have correctly copied the
  -- census rows into members.  See remarks in rebuild_members().
  --
  -- Bugs:

  DECLARE
    thisrow RECORD;

  BEGIN

  -- Do rebuild.
  -- Select from biograph and members even though _update_members() wants
  -- rows on census so that we will delete all rows from members when
  -- necessary.
  FOR thisrow
      IN SELECT DISTINCT sname
           FROM (SELECT sname
                   FROM biograph
                   WHERE sname IS NOT NULL
                 UNION
                 SELECT DISTINCT sname
                   FROM members) AS everybody
           ORDER BY sname
      LOOP
    -- RAISE INFO 'rebuild_all_members: %', thisrow.sname;
    PERFORM rebuild_members(thisrow.sname);
  END LOOP;

  -- Log rebuild.
  RAISE LOG 'rebuild_all_members: MEMBERS rebuilt';

  RETURN 0;
  END;
$$;
grant_func_priv(`rebuild_all_members()')
