dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2006, 2008, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Rebuild all the repstats.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl
dnl
dnl m4 includes
include(`copyright.m4')
include(`constants.m4')
include(`macros.m4')
dnl

CREATE OR REPLACE FUNCTION rebuild_all_repstats()
  RETURNS INT
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- Re compute all the REPSTATS rows.
  --
  -- GPL_notice(`  --', `2006, 2008', `Karl O. Pinc <kop@meme.com>')
  --
  -- Syntax: rebuild_all_repstats()
  --
  -- Arguments:
  --
  -- Remarks:
  --  Very similar to rebuild_all_members.  Could be more efficient than
  -- rebuilding one sname at a time.
  --
  -- Bugs:

  DECLARE
    thisrow RECORD;

  BEGIN

  -- Do rebuild.
  -- Select from biograph and cycgapdays so that we will delete all
  -- automatic Mdates for sure.
  FOR thisrow
      IN SELECT DISTINCT sname
           FROM (SELECT sname
                   FROM biograph
                   WHERE sname IS NOT NULL
                 UNION
                 SELECT DISTINCT sname
                   FROM cycgapdays) AS everybody
           ORDER BY sname
      LOOP
    PERFORM rebuild_repstats(thisrow.sname);
  END LOOP;

  -- Log rebuild.
  RAISE LOG 'rebuild_all_repstats: REPSTATS rebuilt';

  RETURN 0;
  END;
$$;
grant_func_priv(`rebuild_all_repstats()')
