dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2006, 2008, 2009 Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Rebuild one individual's automatic Mdates on CYCPOINTS.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl
dnl Bugs:
dnl  This will only work if the triggers are turned off on CYCPOINTS!
dnl
dnl m4 includes
include(`copyright.m4')
include(`constants.m4')
include(`macros.m4')
dnl

CREATE OR REPLACE FUNCTION rebuild_automdates(this_sname cycles.sname%TYPE)
  RETURNS INT
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- Rebuild one individual's automatic Mdates on CYCPOINTS
  --
  -- GPL_notice(`  --', `2006, 2008, 2009', `Karl O. Pinc <kop@meme.com>')
  --
  -- Syntax: rebuild_automdates(this_sname)
  --
  -- Arguments:
  --  this_sname   Individual to rebuild
  --
  -- Remarks:
  --  Alternate method from that of triggers so the two can be run
  -- independently and compared.
  --
  -- Bugs:
  --  Only works if triggers are turned off on CYCPOINTS!
  -- If we wanted to we could test for this and not run otherwise,
  -- but we don't.

  DECLARE
    cp RECORD;
    last_cp RECORD;

    gap cycgapdays.date%TYPE;
    new_autom cycpoints.date%TYPE;
    this_statdate biograph.statdate%TYPE;
    this_status biograph.status%TYPE;

    first_trip BOOLEAN;

  BEGIN

  -- Log warning.
  RAISE WARNING 'rebuild_automdates: %: Triggers on CYCPOINTS had better be turned off.'
              , this_sname;

  -- Get the statdate and status.
  SELECT INTO this_statdate, this_status
              biograph.statdate, biograph.status
    FROM biograph
    WHERE biograph.sname = this_sname;

  -- Delete all the individual's automatic Mdates.
  DELETE
    FROM cycpoints
    WHERE cid IN (SELECT cycles.cid FROM cycles WHERE sname = this_sname)
          AND code = 'bb_menses'
          AND source = 'bb_auto';

  -- Find the first gap in observation that postdates a cycpoints row.
  SELECT INTO gap cycgapdays.date
    FROM cycgapdays
    WHERE cycgapdays.sname = this_sname
          AND cycgapdays.date >= (SELECT cycpoints.date
                                    FROM cycpoints, cycles
                                    WHERE cycles.sname = this_sname
                                          AND cycpoints.cid = cycles.cid
                                          AND cycpoints.code <> 'bb_bleeding'
                                    ORDER BY cycpoints.date
                                    LIMIT 1)
    ORDER BY cycgapdays.date
    LIMIT 1;

  -- Initalize for loop.
  -- This is a bit stupid, but we don't run this all that often
  -- and it works.  (Gotta find a better idiom.)
  first_trip = TRUE;

--  SELECT INTO last_cp
--              cycpoints.cpid, cycpoints.cid, cycpoints.date, cycpoints.code, cycpoints.source
--         FROM cycpoints, cycles
--         WHERE cycles.sname = this_sname 
--               AND cycpoints.cid = cycles.cid
--               AND cycpoints.code <> 'bb_bleeding'
--         ORDER BY cycpoints.date, cycpoints.code
--         LIMIT 1;

  -- Iterate over the cycpoints rows and reconstruct the automatic Mdates.
  FOR cp IN SELECT cycpoints.cpid
                 , cycpoints.cid
                 , cycpoints.date
                 , cycpoints.code
                 , cycpoints.source
              FROM cycpoints, cycles
              WHERE cycles.sname = this_sname 
                    AND cycpoints.cid = cycles.cid
                    AND cycpoints.code <> 'bb_bleeding'
              ORDER BY cycpoints.date, cycpoints.code
         LOOP

    IF first_trip THEN
      first_trip := FALSE;
    ELSE

    IF last_cp.code = 'bb_deturgesence' 
       AND last_cp.source <> 'bb_estimated' THEN
      -- We might want a automatic Mdate.

      PERFORM 1
        FROM pregs
        WHERE conceive = last_cp.cpid
        LIMIT 1;
      IF NOT FOUND THEN
        -- The Ddate does not start a preganancy.

        IF last_cp.date > gap THEN
          -- Time to find the next gap.
          -- (Account for estimated Ddates which can be in gaps.)
          SELECT INTO gap cycgapdays.date
            FROM cycgapdays
            WHERE sname = this_sname
                  AND date >= last_cp.date
            ORDER BY date
            LIMIT 1;
        END IF;

        -- When would we want the automatic Mdate?
        new_autom := last_cp.date + CAST('bb_autom_interval days' AS INTERVAL);
        IF cp.date <= new_autom THEN
          -- Can't have the automatic Mdate after the next sexual cycle event.
          IF cp.code = 'bb_turgesence' THEN
            -- Tdates can be on the same day as Mdates.
            new_autom := cp.date;
          ELSE
            new_autom := cp.date - CAST('1 day' AS INTERVAL);
          END IF;
        END IF;

        IF (cp.code <> 'bb_menses'
            OR (cp.code = 'bb_menses' AND gap < cp.date))
           AND (gap > new_autom OR gap IS NULL)
           AND (this_status = bb_alive OR
                new_autom <= this_statdate) THEN
          --
          -- Make the automatic Mdate.
          --
          IF cp.code = 'bb_menses' THEN
            -- Uh, oh. Gotta insert a new cycle.
            -- This is a problem because then we need to
            -- resequence the cycles.
            INSERT INTO cycles (sname) VALUES (this_sname);
            INSERT INTO cycpoints(cid, date, code, source)
                   VALUES(currval('cycles_cid_seq')
                        , new_autom
                        , 'bb_menses'
                        , 'bb_auto');
          ELSE
            -- Attach automatic M to the next cycle.
            INSERT INTO cycpoints(cid, date, code, source)
                   VALUES(cp.cid, new_autom, 'bb_menses', 'bb_auto');
          END IF;
        END IF;
      END IF;
    END IF;
    END IF;

    -- Prepare for next iteration.
    last_cp := cp;

  END LOOP;

  --
  -- Deal with the final row.
  --

  IF NOT(first_trip) THEN
    -- There was at least one cycpoints row.
  IF last_cp.code = 'bb_deturgesence' 
     AND last_cp.source <> 'bb_estimated' THEN
    -- We might want a automatic Mdate.

    PERFORM 1
      FROM pregs
      WHERE conceive = last_cp.cpid
      LIMIT 1;
    IF NOT FOUND THEN
      -- The Ddate does not start a preganancy.

      IF last_cp.date > gap THEN
        -- Time to find the next gap.
        SELECT INTO gap cycgapdays.date
          FROM cycgapdays
          WHERE sname = this_sname
                AND date >= last_cp.date
          ORDER BY date
          LIMIT 1;
      END IF;


      -- When would we want the automatic Mdate?
      new_autom := last_cp.date + CAST('bb_autom_interval days' AS INTERVAL);

      IF (gap > new_autom OR gap IS NULL)
         AND (this_status = bb_alive OR
              new_autom <= this_statdate) THEN
        -- Uh, oh. Gotta insert a new cycle.
        -- This is a problem because then we need to
        -- resequence the cycles.
        INSERT INTO cycles (sname) VALUES (this_sname);
        INSERT INTO cycpoints(cid, date, code, source)
               VALUES(currval('cycles_cid_seq')
                    , new_autom
                    , 'bb_menses'
                    , 'bb_auto');
      END IF;
    END IF;
  END IF;
  END IF;

  RETURN 0;
  END;
$$;
grant_func_priv(`rebuild_automdates(cycles.sname%TYPE)')
