dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2006, 2008, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Rebuild one individual's CYCGAPDAYS rows.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl
dnl Bugs:
dnl  I'm very ambigious about the use of a macro for do_insert.
dnl
dnl m4 includes
include(`copyright.m4')
include(`constants.m4')
include(`macros.m4')
dnl

dnl plpgsql fragment for inserting rows into cycgapdays over an interaval
dnl
dnl Syntax do_insert(start, end)
dnl
dnl start   loop variable, initialized by the caller to the first date
dnl         in the interval (inclusive)
dnl end     last date in the interval (inclusive)
dnl
changequote({,})
define({do_insert},{
      WHILE $1 <= $2 LOOP
        PERFORM 1
          FROM cycgaps
          WHERE sname = this_sname
                AND date = $1
          LIMIT 1;
        IF NOT FOUND THEN
          -- There's no point row for the day.
          INSERT INTO cycgapdays (sname, date) VALUES (this_sname, $1);
        END IF;
        $1 := $1 + CAST('1 day' AS INTERVAL);
      END LOOP;
})dnl
changequote(`,')dnl



CREATE OR REPLACE FUNCTION rebuild_cycgapdays(this_sname cycgapdays.sname%TYPE)
  RETURNS INT
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- Rebuild one individual's CYCGAPDAYS rows.
  --
  -- GPL_notice(`  --', `2006, 2008', `Karl O. Pinc <kop@meme.com>')
  --
  -- Syntax: rebuild_cycgapdays(this_sname)
  --
  -- Arguments:
  --  this_sname   Individual to rebuild
  --
  -- Remarks:
  --  Alternate method from that of triggers so the two can be run
  -- independently and compared.
  --
  --  Assumes the cycgaps table is valid, else who knows what will
  -- happen.
  --
  --  Has code in it to keep all cycgapdays after the maturity date
  -- bt this is commented out because the triggers keep it simple
  -- and don't bother with maturity date.
  --
  -- Bugs:

  DECLARE
    cg RECORD;
    end_date cycgaps.date%TYPE;
    this_date cycgaps.date%TYPE;
    loop_end cycgaps.date%TYPE;
    this_statdate biograph.statdate%TYPE;
    -- this_matured maturedates.matured%TYPE;

  BEGIN

  -- Delete all the individual's rows.
  DELETE FROM cycgapdays WHERE sname = this_sname;

  -- Get the statdate.
  SELECT INTO this_statdate
              statdate
    FROM biograph
    WHERE sname = this_sname;

  --  -- Get the maturity date.
  --  SELECT INTO this_matured
  --              matured
  --    FROM maturedates
  --    WHERE sname = this_sname;

  -- Iterate over the cycgaps rows and reconstruct the table.
  FOR cg IN SELECT date, code
              FROM cycgaps 
              WHERE sname = this_sname 
                    AND code <> 'bb_cg_point'
              ORDER BY date
         LOOP

    IF cg.code = 'bb_cg_end' THEN
      -- Remember where we're supposed to start inserting.
      end_date := cg.date;

    ELSE
      -- cg.code = 'bb_cg_start'
      -- Insert rows for those days between start and end.
      -- IF this_date < this_matured THEN
      --   -- Don't start before the maturity date.
      --   this_date := this_matured;
      -- ELSE

      -- Gap starts day after observation ends.
      this_date := end_date + CAST('1 day' AS INTERVAL);

      -- END IF;

      IF cg.date > this_statdate + bb_autom_interval THEN
        -- Don't end after the statdate.
        loop_end := this_statdate + bb_autom_interval;
      ELSE
        -- Gap ends day before the observation resumes.
        loop_end := cg.date - CAST('1 day' AS INTERVAL);
      END IF;

      do_insert(`this_date', `loop_end')
    END IF;
  END LOOP;

  IF cg.code = 'bb_cg_end' THEN
    -- Observation ends (maybe forever)
    -- Carry the gap up to and including the statdate.
    -- (And far enough beyond to influence all automatic Mdates.)

    this_date := end_date;

    -- IF this_date < this_matured THEN
    --   -- Don't start before the maturity date.
    --   this_date := this_matured;
    -- ELSE

    -- Gap starts day after observation ends.
    this_date := end_date + CAST('1 day' AS INTERVAL);
    -- END IF;

    do_insert(`this_date', `this_statdate + ' bb_autom_interval)
  END IF;

  RETURN 0;
  END;
$$;
grant_func_priv(`rebuild_cycgapdays(cycgapdays.sname%TYPE)')
