dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2006, 2008, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Rebuild one individual's CYCGAPDAYS rows.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl
dnl
dnl m4 includes
include(`copyright.m4')
include(`constants.m4')
include(`macros.m4')
dnl

dnl plpgsql fragment for ensuring the cycle changes.
dnl
dnl Syntax: force_new_cycle()
dnl
dnl (Should really be a function.)
dnl
changequote({,})
define({force_new_cycle},{
        IF last_cp.cid = cp.cid THEN
          -- Give the cp a new cycle.
          INSERT INTO cycles (sname) VALUES (this_sname);
          UPDATE cycpoints
            SET cid = currval('cycles_cid_seq')
            WHERE cpid = cp.cpid;
          cp.cid = currval('cycles_cid_seq');
--raise warning '%: %: new cycle', this_sname, cp.cpid;
        END IF;
})dnl
changequote(`,')dnl


dnl plpgsql fragment for ensuring the cycle does not change.
dnl
dnl Syntax: keep_old_cycle()
dnl
dnl (Should really be a function.)
dnl
changequote({,})
define({keep_old_cycle},{
            IF last_cp.cid <> cp.cid THEN
              UPDATE cycpoints
                SET cid = last_cp.cid
                WHERE cpid = cp.cpid;
              cp.cid = last_cp.cid;
--raise warning '%: %: moved cycle', this_sname, cp.cpid;
            END IF;
})dnl
changequote(`,')dnl




dnl plpgsql fragment for updating the seq and series.
dnl
dnl Syntax: update_cycle()
dnl
dnl (Should really be a function.)
dnl
changequote({,})
define({update_cycle},{
      -- Keep track of whether we changed the cycle.
      changed := FALSE;

      -- Set the seq.
      IF last_cy.seq <> next_seq OR last_cy.seq IS NULL THEN
        last_cy.seq := next_seq;
        changed := TRUE;
      END IF;

      -- Set the series.
      IF last_cy.series <> next_series OR last_cy.series IS NULL THEN
        last_cy.series := next_series;
        changed := TRUE;
      END IF;

      IF changed THEN
        -- Write out changes.
        UPDATE cycles
          SET seq = last_cy.seq
            , series = last_cy.series
          WHERE cid = last_cy.cid;
--raise warning '%: %: set seqeuence', this_sname, last_cy.cid;
      END IF;
})dnl
changequote(`,')dnl



CREATE OR REPLACE FUNCTION rebuild_cycles(this_sname cycles.sname%TYPE)
  RETURNS INT
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- Rebuild one individual's CYCLES including CYCLES.Seq and CYCLES.Series.
  --
  -- GPL_notice(`  --', `2006, 2008', `Karl O. Pinc <kop@meme.com>')
  --
  -- Syntax: rebuild_cycles(this_sname)
  --
  -- Arguments:
  --  this_sname   Individual to rebuild
  --
  -- Remarks:
  --  Alternate method from that of triggers so the two can be run
  -- independently and compared.
  --
  --  Assumes the cycgaps table is valid, else who knows what will
  -- happen.
  --
  -- Bugs:
  --  Must be run with triggers turned off!

  DECLARE

  BEGIN

  --
  -- Reconstruct the cycles from the cycpoints.
  --
  DECLARE
    cp RECORD;
    last_cp RECORD;
    gap cycgaps.date%TYPE;

    first_trip BOOLEAN NOT NULL := TRUE;
  BEGIN

  -- Find the last gap in observation that predates a cycpoints row.
  SELECT INTO gap cycgapdays.date
    FROM cycgapdays
    WHERE cycgapdays.sname = this_sname
          AND cycgapdays.date <= (SELECT cycpoints.date
                                    FROM cycpoints, cycles
                                    WHERE cycles.sname = this_sname
                                          AND cycpoints.cid = cycles.cid
                                          AND cycpoints.code <> 'bb_bleeding'
                                    ORDER BY cycpoints.date DESC
                                    LIMIT 1)
    ORDER BY cycgapdays.date DESC
    LIMIT 1;

  -- Iterate over the cycpoints rows and reconstruct the
  -- cycles, including seq and series.
  -- Go from later to earler so that we can have later
  -- Cids remain unchanged.

  FOR cp IN SELECT cycpoints.cpid, cycpoints.cid, cycpoints.code, cycpoints.date
              FROM cycles, cycpoints
              WHERE cycles.sname = this_sname
                    AND cycpoints.cid = cycles.cid
                    AND cycpoints.code <> 'bb_bleeding'
              ORDER BY cycpoints.date DESC, cycpoints.code DESC
         LOOP

    IF first_trip THEN
      first_trip := FALSE;
    ELSE

      IF last_cp.date < gap THEN
        -- Time to find the next gap.
        -- (Account for estimated Ddates which can be in gaps.)
        SELECT INTO gap cycgapdays.date
          FROM cycgapdays
          WHERE sname = this_sname
                AND date <= last_cp.date
          ORDER BY date DESC
          LIMIT 1;
      END IF;

      IF gap >= cp.date THEN
        -- Gap between this and last cycpoints,
        -- gaps mean new cycles.
        force_new_cycle()
      ELSE
        -- No gap, cycles change only between D and M dates.
        -- Cover all the cases in case there's missing data.
        IF last_cp.code = 'bb_menses' THEN
          -- (Prior cycle, cp, should be a Ddate.)
          -- Cycle should change.
          force_new_cycle()

        ELSEIF last_cp.code = 'bb_turgesence' THEN
          IF cp.code = 'bb_menses' THEN
            -- Cycle should not change.
            keep_old_cycle()
          ELSE
            -- A D or T preceeds the T.
            force_new_cycle()
          END IF;

        ELSE
          -- last_cp.code = D
          IF cp.code = 'bb_turgesence'
             OR cp.code = 'bb_menses' THEN
            -- Cycle should not change.
            keep_old_cycle()
          ELSE
            -- Another Ddate preceeds the D.
            force_new_cycle()
          END IF;
        END IF;
      END IF;
    END IF;

    -- Prepare for next iteration.
    last_cp := cp;

  END LOOP;
  END;

  --
  -- Reconsruct the series and seq on cycles.
  --
  DECLARE
    cy RECORD;
    last_cy RECORD;
    gap cycgaps.date%TYPE;

    first_trip BOOLEAN NOT NULL := TRUE;
    changed BOOLEAN NOT NULL := FALSE;

    next_seq cycles.seq%TYPE := 1;
    next_series cycles.series%TYPE := 1;
  BEGIN

  FOR cy IN SELECT cycles.cid
                 , (SELECT cycpoints.date
                      FROM cycpoints
                      WHERE cycpoints.cid = cycles.cid
                            AND cycpoints.code <> 'bb_bleeding'
                      ORDER BY cycpoints.date
                      LIMIT 1) AS date
                 , cycles.seq
                 , cycles.series
              FROM cycles
              WHERE sname = this_sname
                    AND cid IN (SELECT DISTINCT cycles.cid
                                  FROM cycles, cycpoints
                                  WHERE cycles.sname = this_sname
                                        AND cycpoints.cid = cycles.cid)
              ORDER BY date
         LOOP

    IF first_trip THEN
      first_trip := FALSE;
    ELSE

      update_cycle()

      -- Prep for next iteration.
      next_seq := next_seq + 1;
      -- See if the series should change.
      PERFORM 1
        FROM cycgapdays
        WHERE sname = this_sname
              AND date >= last_cy.date
              AND date <= cy.date
        LIMIT 1;
      IF FOUND THEN
        -- Series must change.
        next_series := next_series + 1;
      END IF;

    END IF;

    -- Prep for the next iteration.
    last_cy := cy;

  END LOOP;

  IF NOT(first_trip) THEN
    -- There were some rows.
    -- Finish with the last cycle.
    update_cycle()
  END IF;

  END;

  --
  -- Assign bleeding dates to cycles.
  --
  DECLARE
    cp RECORD;
    mtd RECORD;

    e BOOLEAN;

  BEGIN

  -- Go through the B dates.
  FOR cp IN SELECT cycpoints.cpid, cycpoints.date, cycpoints.cid
              FROM cycles, cycpoints
              WHERE cycles.sname = this_sname
                    AND cycpoints.cid = cycles.cid
                    AND cycpoints.code = 'bb_bleeding'
              ORDER BY date
         LOOP

      -- No error at this point.
      e := FALSE;

      -- Find the prior cycpoint.
      SELECT INTO mtd cycpoints.date, cycpoints.cid, cycpoints.cpid
        FROM cycles, cycpoints
        WHERE cycles.sname = this_sname
              AND cycpoints.cid = cycles.cid
              AND cycpoints.code <> 'bb_bleeding'
              AND cycpoints.date <= cp.date
        ORDER BY date DESC
        LIMIT 1;

      -- Is there a gap between now and then?
      PERFORM 1
        FROM cycgapdays
        WHERE sname = this_sname
              AND date < cp.date
              AND date >= mtd.date
        LIMIT 1;
      IF FOUND OR mtd.cpid IS NULL THEN
        -- Yes, there is a gap, or no prior MTDdate.
        -- Find a Tdate or Ddate after this date.
        SELECT INTO mtd cycpoints.date, cycpoints.cid, cycpoints.cpid
          FROM cycles, cycpoints
          WHERE cycles.sname = this_sname
                AND cycpoints.cid = cycles.cid
                AND cycpoints.date >= cp.date
                AND cycpoints.cpid <> cp.cpid
                AND (cycpoints.code = 'bb_deturgesence'
                     OR cycpoints.code = 'bb_turgesence')
          ORDER BY DATE
          LIMIT 1;

        -- Again, is there a gap between now and then?
        PERFORM 1
          FROM cycgapdays
          WHERE sname = this_sname
                AND date > cp.date
                AND date <= mtd.date
          LIMIT 1;
        IF FOUND OR mtd.cpid IS NULL THEN
          -- This row cannot be assigned a date and should not
          -- be in cycpoints.
          RAISE WARNING 'Cpid: %: This Bdate cannot be assigned to a cycle and should not be in CYCPOINTS.'
                      , cp.cpid;
          e := TRUE;
        END IF;
      END IF;

      IF NOT e THEN
        -- Assign the cycle.
        IF cp.cid <> mtd.cid THEN
          UPDATE cycpoints
            SET cid = mtd.cid
            WHERE cpid = cp.cpid;
--raise warning '%: %: moved bdate', this_sname, cp.cpid;
        END IF;
      END IF;      

    END LOOP;
  END;

  --
  -- Re-assign sexskins to cycles.
  --
  -- (Same logic as Bdates, *sigh*.)
  DECLARE
    ss RECORD;
    mtd RECORD;

    e BOOLEAN;

  BEGIN

  -- Go through the sexsksins.
  FOR ss IN SELECT sexskins.sxid, sexskins.date, sexskins.cid
              FROM cycles, sexskins
              WHERE cycles.sname = this_sname
                    AND sexskins.cid = cycles.cid
              ORDER BY date
         LOOP

      -- No error at this point.
      e := FALSE;

      -- Find the prior cycpoint.
      SELECT INTO mtd cycpoints.date, cycpoints.cid, cycpoints.cpid
        FROM cycles, cycpoints
        WHERE cycles.sname = this_sname
              AND cycpoints.cid = cycles.cid
              AND cycpoints.code <> 'bb_bleeding'
              AND cycpoints.date <= ss.date
        ORDER BY date DESC
        LIMIT 1;

      -- Is there a gap between now and then?
      PERFORM 1
        FROM cycgapdays
        WHERE sname = this_sname
              AND date < ss.date
              AND date >= mtd.date
        LIMIT 1;
      IF FOUND OR mtd.cpid IS NULL THEN
        -- Yes, there is a gap, or no prior MTDdate.
        -- Find a Tdate or Ddate after this date.
        SELECT INTO mtd cycpoints.date, cycpoints.cid, cycpoints.cpid
          FROM cycles, cycpoints
          WHERE cycles.sname = this_sname
                AND cycpoints.cid = cycles.cid
                AND cycpoints.date >= ss.date
                AND (cycpoints.code = 'bb_deturgesence'
                     OR cycpoints.code = 'bb_turgesence')
          ORDER BY DATE
          LIMIT 1;

        -- Again, is there a gap between now and then?
        PERFORM 1
          FROM cycgapdays
          WHERE sname = this_sname
                AND date > ss.date
                AND date <= mtd.date
          LIMIT 1;
        IF FOUND OR mtd.cpid IS NULL THEN
          -- This row cannot be assigned a date and should not
          -- be in cycpoints.
          RAISE WARNING 'Sxid: %: This SEXSKINS row cannot be assigned to a cycle and should not be in CYCPOINTS.'
                      , ss.sxid;
          -- DELETE FROM sexskins WHERE sxid = ss.sxid;
          e := TRUE;
        END IF;
      END IF;

      IF NOT e THEN
        -- Assign the cycle.
        IF ss.cid <> mtd.cid THEN
          UPDATE sexskins
            SET cid = mtd.cid
            WHERE sxid = ss.sxid;
--raise warning '%: %: moved sexskin', this_sname, ss.sxid;
        END IF;
      END IF;      

    END LOOP;
  END;

  --
  -- Delete all orphaned cycles.
  --
  DELETE
    FROM cycles
    WHERE sname = this_sname
          AND cid NOT IN (SELECT DISTINCT cycles.cid
                            FROM cycles, cycpoints
                            WHERE cycles.sname = this_sname
                                  AND cycpoints.cid = cycles.cid);

  RETURN 0;
  END;
$$;
grant_func_priv(`rebuild_cycles(cycles.sname%TYPE)')
