dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2006, 2008, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Rebuild one individual's CYCSTATS rows in batch.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl
dnl Bugs:
dnl
dnl m4 includes
include(`copyright.m4')
include(`constants.m4')
include(`macros.m4')
dnl


dnl plpgsql fragment that tests if a cycpoints row begins a pregnancy.
dnl
dnl Syntax: _pregnancy_test(cp)
dnl
dnl Args:
dnl  cp         A cycpoints row.
dnl
dnl Side Effects:
dnl  Sets the pregnancy boolean.
dnl
changequote({,})
define({_pregnancy_test},{
    -- Pregnancies are handled differently.
    pregnancy := FALSE;
    IF $1.code = 'bb_deturgesence' THEN
      PERFORM 1
        FROM pregs
        WHERE conceive = $1.cpid
        LIMIT 1;
      IF FOUND THEN
        -- The ddate is a pregnancy.
        pregnancy := TRUE;
      END IF;
    END IF;
})dnl
changequote(`,')dnl


dnl plpgsql fragment for rebuilding cycstats over a date range.
dnl
dnl Syntax: _doit()
dnl
dnl Globals Input:
dnl  obs_start  The start of the date range.
dnl  obs_end    The end of the date range.
dnl  obs_state  The state at the start of the date range.
dnl             May be NULL when cycgaps has no information.
dnl
changequote({,})
define({_doit},{

--debug-- raise warning '% % %', obs_start, obs_end, obs_state;
--
-- Construct cycstats from the start of the period of
-- observation through the first sexual cycle event, inclusive.
--

-- Get the first sexual cycle event in the interval.
SELECT INTO cp cycpoints.date, cycpoints.code, cycpoints.cpid
  FROM cycpoints, cycles
  WHERE cycles.sname = this_sname
        AND cycpoints.cid = cycles.cid
        AND cycpoints.date >= obs_start
        AND cycpoints.date <= obs_end
        AND cycpoints.code <> 'bb_bleeding'
  ORDER BY cycpoints.date
  LIMIT 1;
IF FOUND THEN
  -- There are some sexual cycle events in the observation
  -- interval.

  IF cp.date <> obs_start THEN
    --
    -- Have to make the cycstats between start of observation
    -- (maturity at minimum) and the first cycpoints row.
    --

    --
    -- Determine the state of the interval.
    --
    pregnancy := FALSE;
    IF obs_state IS NULL THEN
      -- We have no state as observation starts.
      -- (We are starting at menarche.)
      -- Look to the first cycle event to figure it out
      IF cp.code = 'bb_deturgesence' THEN
        this_state := 'bb_swelling';
      ELSEIF cp.code = 'bb_turgesence' THEN
        -- Be sure that we're not in a pregnancy.
        PERFORM 1
          FROM pregs
          WHERE pregs.resume = cp.cpid
          LIMIT 1;
        IF FOUND THEN
          -- We are in a pregnancy/lactation.
          -- Remember this so we don't make cycstats.
          pregnancy := TRUE;
        ELSE
          this_state := 'bb_menses';
        END IF;
      ELSE -- Must be menses.
        this_state := 'bb_deturgesence';
      END IF;
    ELSE
      -- The cycgaps state determines the cycstats state.
      IF obs_state = 'bb_cg_menses' THEN
        this_state := 'bb_menses';
      ELSEIF obs_state = 'bb_cg_swelling' THEN
        this_state := 'bb_swelling';
      ELSEIF obs_state = 'bb_cg_ovulating' THEN
        this_state := 'bb_ovulating';
      ELSEIF obs_state = 'bb_cg_deturgesence' THEN
        this_state := 'bb_deturgesence';
      ELSE
        -- We're pregnant or lactating.
        pregnancy := TRUE;
      END IF;
    END IF;

    IF NOT pregnancy THEN
      -- Create the cycstats from beginning of observation up to
      -- the first cycpoints.

      this_date := obs_start;
      WHILE this_date < cp.date LOOP

        -- What is the state?
        IF this_state = 'bb_swelling'
           AND cp.code = 'bb_deturgesence'
           AND cp.date - this_date <= bb_ovulation_len THEN
          -- We were swelling, but are now close enough to
          -- the Ddate (should we be testing for ddate?)
          -- that we're ovulating.
          this_state := 'bb_ovulating';
        END IF;

        --debug-- raise warning 'first %', this_date;

        INSERT INTO cycstats (date, sname, state, dins, dr, cpids, cpide)
                    VALUES (this_date
                          , this_sname
                          , this_state
                          , NULL
                          , (cp.date - this_date) - 1
                          , NULL
                          , cp.cpid);
        this_date := this_date + CAST('1 day' AS INTERVAL);
      END LOOP;
    END IF;
  END IF;

--
-- Construct cycstats from the day of each sexual cycle event
-- through the day before next sexual cycle event, inclusive.
--
  last_cp := cp;
  last_cp_cpid := last_cp.cpid;
  last_cp_date := last_cp.date;
  FOR cp IN SELECT cycpoints.date, cycpoints.code, cycpoints.cpid
              FROM cycpoints, cycles
              WHERE cycles.sname = this_sname
                    AND cycpoints.cid = cycles.cid
                    AND cycpoints.date >= last_cp_date
                    AND cycpoints.date <= obs_end
                    AND cycpoints.cpid <> last_cp_cpid
                    AND cycpoints.code <> 'bb_bleeding'
              ORDER BY cycpoints.date, cycpoints.code
      LOOP
    --debug--raise warning '% % %', cp.date, cp.code, cp.cpid;
    -- What is the state?
    IF last_cp.code = 'bb_turgesence' THEN
      -- Must look to the next Ddate to see what the state is.
      IF cp.code = 'bb_deturgesence' THEN
        IF cp.date - last_cp.date > bb_ovulation_len THEN
          this_state := 'bb_swelling';
        ELSE
          this_state := 'bb_ovulating';
        END IF;
      ELSE
        -- No next Ddate, just call it swelling.
        this_state := 'bb_swelling';
      END IF;
    ELSE
      -- Not a Tdate, the state is the same as the cp code.
      this_state := last_cp.code;
    END IF;


    _pregnancy_test(last_cp)

    IF NOT pregnancy THEN
      -- We get rows in cycstats.

      this_date := last_cp.date;
      WHILE this_date < cp.date LOOP

        -- What is the state?
        IF this_state = 'bb_swelling'
           AND cp.code = 'bb_deturgesence'
           AND cp.date - this_date <= bb_ovulation_len THEN
          -- We were swelling, but are now close enough to
          -- the Ddate (should we be testing for ddate?)
          -- that we're ovulating.
          this_state := 'bb_ovulating';
        END IF;

        --debug-- raise warning 'second %', this_date;

        INSERT INTO cycstats (date, sname, state, dins, dr, cpids, cpide)
                    VALUES (this_date
                          , this_sname
                          , this_state
                          , (this_date - last_cp.date) + 1
                          , (cp.date - this_date) - 1
                          , last_cp.cpid
                          , cp.cpid);
        this_date := this_date + CAST('1 day' AS INTERVAL);
      END LOOP;
    END IF;

    last_cp := cp;
  END LOOP;

  --
  -- Construct cycstats from the day of the last sexual cycle
  -- event through the last day in the interval, inclusive.
  --

  -- What should the state be?
  IF last_cp.code = 'bb_turgesence' THEN
    -- No next Ddate, just call it swelling.
    this_state := 'bb_swelling';
  ELSE
    -- Not a Tdate, the state is the same as the cp code.
    this_state := last_cp.code;
  END IF;

  -- Set the pregnancy boolean.
  _pregnancy_test(last_cp)

  IF NOT pregnancy THEN
    -- We get rows in cycstats.

    this_date := last_cp.date;
    WHILE this_date <= obs_end LOOP

      --debug-- raise warning 'third %', this_date;

      INSERT INTO cycstats (date, sname, state, dins, dr, cpids, cpide)
                  VALUES (this_date
                        , this_sname
                        , this_state
                        , (this_date - last_cp.date) + 1
                        , NULL
                        , last_cp.cpid
                        , NULL);
      this_date := this_date + CAST('1 day' AS INTERVAL);
    END LOOP;
  END IF;

ELSE
  -- There are no sexual cycle events in the observation interval.
  IF obs_state IS NOT NULL
     AND obs_state <> 'bb_cg_pregnant'
     AND obs_state <> 'bb_cg_lactating' THEN
    -- We have a state we can assign to the interval.

    IF obs_state = 'bb_cg_menses' THEN
      this_state := 'bb_menses';
    ELSEIF obs_state = 'bb_cg_swelling' THEN
      this_state := 'bb_swelling';
    ELSEIF obs_state = 'bb_cg_ovulating' THEN
      this_state := 'bb_ovulating';
    ELSEIF obs_state = 'bb_cg_deturgesence' THEN
      this_state := 'bb_deturgesence';
    END IF;

    -- Assign the state to every day in the interval.
    this_date := obs_start;
    WHILE this_date <= obs_end LOOP

      --debug-- raise warning 'none %', this_date;

      INSERT INTO cycstats (date, sname, state, dins, dr, cpids, cpide)
             VALUES (this_date
                   , this_sname
                   , this_state
                   , NULL
                   , NULL
                   , NULL
                   , NULL);
      this_date := this_date + CAST('1 day' AS INTERVAL);
    END LOOP;
  END IF;
END IF;

})dnl
changequote(`,')dnl



CREATE OR REPLACE FUNCTION rebuild_cycstats(this_sname cycles.sname%TYPE)
  RETURNS INT
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- Rebuild one individual's CYCSTATS rows.
  --
  -- GPL_notice(`  --', `2006, 2008', `Karl O. Pinc <kop@meme.com>')
  --
  -- Syntax: rebuild_cycstats(this_sname)
  --
  -- Arguments:
  --  this_sname   Individual to rebuild
  --
  -- Remarks:
  --  Alternate method from that of triggers so the two can be run
  -- independently and compared.
  --
  -- Bugs:

  DECLARE
    matured maturedates.matured%TYPE;
    statdate biograph.statdate%TYPE;
    this_sex biograph.sex%TYPE;

    this_date DATE;
    this_state cycstats.state%TYPE;

--    cp cycpoints;
--    last_cp cycpoints;
    cg cycgaps;
    cp RECORD;
    last_cp RECORD;

    obs_state cycgaps.state%TYPE;
    obs_start DATE;
    obs_end DATE;

    last_cp_cpid cycpoints.cpid%TYPE;
    last_cp_date cycpoints.date%TYPE;
    pregnancy BOOLEAN;
  BEGIN

  -- Delete all the individual's cycstats.
  DELETE
    FROM cycstats
    WHERE sname = this_sname;

  --
  -- Iterate over the periods between gaps.
  --

  -- Start with matured date.
  SELECT INTO matured maturedates.matured
    FROM maturedates
    WHERE sname = this_sname;
  IF NOT FOUND THEN
    -- Individual is not mature, all done.
    RETURN 0;
  END IF;

  -- Go through the statdate.
  SELECT INTO statdate,          this_sex
              biograph.statdate, biograph.sex
    FROM biograph
    WHERE sname = this_sname;
  IF this_sex <> 'bb_female' THEN
    -- Individual is not female, all done.
    RETURN 0;
  END IF;

  obs_start := matured;

  cg.code = NULL; -- For testing if we had any gaps at all.
--  FOR cg IN SELECT code, date, state
  FOR cg IN SELECT *
              FROM cycgaps
              WHERE sname = this_sname
              ORDER BY date
      LOOP

    IF cg.code <> 'bb_cg_end' THEN
      -- Remember the state at observation start.      
      obs_state := cg.state;
      IF cg.date > obs_start THEN
        -- The start of observation is always (on or) after maturation.
        obs_start := cg.date;
      END IF;
    END IF;

    IF cg.code <> 'bb_cg_start' THEN
      -- We've reached the end of a period of observation
      -- Figure out when we want cycstats to end.
      IF cg.date < statdate THEN
        obs_end := cg.date;
      ELSE
        -- The end of observation is always on or before the statdate.
        obs_end := statdate;
      END IF;

      IF obs_end >= obs_start THEN
        -- The observation does not end before the maturation date
        -- or start after the statdate.

        -- Make the cycstats for the observation period.
        _doit()
      END IF;
    END IF;

  END LOOP;

  IF cg.code IS NULL
     OR (cg.code = 'bb_cg_start' AND obs_start <= statdate) THEN
    -- There are no gaps, or the last gap ends before the statdate.
    -- Finish up the last period of observation.
    obs_end := statdate;
    _doit()
  END IF;

  RETURN 0;
  END;
$$;
grant_func_priv(`rebuild_cycstats(cycles.sname%TYPE)')
