dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2008, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Rebuild one individual's MDINTERVALS rows in batch.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl
dnl Bugs:
dnl
dnl m4 includes
include(`copyright.m4')
include(`constants.m4')
include(`macros.m4')
dnl


dnl plpgsql fragment for rebuilding mdintervals over a date range.
dnl
dnl Syntax: _doit()
dnl
dnl Globals Input:
dnl  obs_start  The start of the date range.
dnl  obs_end    The end of the date range.
dnl  obs_state  The state at the start of the date range.
dnl             May be NULL when cycgaps has no information.
dnl
changequote({,})
define({_doit},{
-- _do`'it() -- begin expansion

--debug-- raise warning '% % %', obs_start, obs_end, obs_state;
--
-- Construct mdintervals from the start of the period of
-- observation through the first sexual cycle event, inclusive.
--
  -- Keep track of whether there are any sexual cycle events.
  cp_seen := FALSE;

  --
  -- Is there a pregnancy at the start of observation?
  --
  pregnancy := FALSE;
  IF obs_state IS NOT NULL THEN
    -- There is a prior gap in observation.
    -- The cycgaps state determines the cycstats state.
    IF obs_state = 'bb_cg_pregnant' 
       OR obs_state = 'bb_cg_lactating' THEN
      -- We're pregnant or lactating.
      pregnancy := TRUE;
    END IF;
  END IF;

  IF pregnancy THEN
    -- The mdintervals start with the end of pregnancy.
    -- Can't cycle while pregnant.  The first
    -- cycle event we have after the pregnancy must
    -- end the pregnancy.  (Regardless of the cycpoints.code.)
    SELECT INTO this_start MIN(cycpoints.date)
      FROM cycpoints, cycles
      WHERE cycles.sname = this_sname
            AND cycpoints.cid = cycles.cid
            AND cycpoints.date >= obs_start
            AND cycpoints.date <= obs_end
            AND cycpoints.code <> 'bb_bleeding';
    -- No prior Mdate so no data for dini and cpids
    this_cpids := NULL;

  ELSE
    -- No pregnancy, but no prior Mdate either, meaning
    -- no data for dini and cpids.
    this_cpids := NULL;
    IF obs_state = 'bb_cg_deturgesence' THEN
      -- We're not in a M-to-D interval.  Don't even make
      -- the mdintervals until we see another Mdate.
      this_start := NULL;
    ELSE
      -- Create the mdintervals from beginning of observation up to
      -- the first cycpoints.
      this_start := obs_start;
    END IF;
  END IF;

  --
  -- Iterate over the sexual cycle events and when we
  -- hit an endpoint of a mdinterval (pregnancy or Ddate) 
  -- construct the rows.
  --
  -- Construct mdintervals from the day of each Mdate
  -- through the day before the end of the interval, inclusive.
  -- (Look at Tdates too, just so we can tell if CYCSTATS has rows.)
  --
  FOR cp IN SELECT cycpoints.date, cycpoints.code, cycpoints.cpid
              FROM cycpoints, cycles
              WHERE cycles.sname = this_sname
                    AND cycpoints.cid = cycles.cid
                    AND cycpoints.date >= obs_start
                    AND cycpoints.date <= obs_end
                    AND cycpoints.code <> 'bb_bleeding'
              ORDER BY cycpoints.date, cycpoints.code
                                       -- lucky, MT is alpha sort
                                       -- to get right sequence when M and
                                       -- T are on the same day
      LOOP
    --debug--raise warning '% % %', cp.date, cp.code, cp.cpid;

    cp_seen := TRUE;

    IF cp.code = 'bb_deturgesence' THEN
      -- Write out the mdintervals.

      -- Will we get NULL dini values or not?
      IF this_cpids IS NULL THEN
        last_mdate_date := NULL;  -- (anything +/- NULL = NULL)
      ELSE
        last_mdate_date := last_mdate.date;
      END IF;

      this_date := this_start; -- May be NULL, because the pregnancy never
                               -- ended.  In which case we don't loop.
      WHILE this_date < cp.date LOOP

        --debug-- raise warning 'second %', this_date;

        INSERT INTO mdintervals (date, sname, dini, dr, cpids, cpide)
                    VALUES (this_date
                          , this_sname
                          , (this_date - last_mdate_date) + 1
                          , (cp.date - this_date) - 1
                          , this_cpids
                          , cp.cpid);
        this_date := this_date + CAST('1 day' AS INTERVAL);
      END LOOP;

      -- Remember we're done with mdintervals until we find
      -- another Mdate.
      this_start := NULL;
      this_cpids := NULL;

    ELSEIF cp.code = 'bb_menses' THEN

      -- Prep for next mdinterval set
      last_mdate := cp;
      this_cpids := last_mdate.cpid;
      this_start := cp.date;

    END IF;

  END LOOP;

  --
  -- Construct mdintervals from the day of the last Mdate
  -- event through the last day in the interval, inclusive.
  --

  IF this_start IS NOT NULL
     -- (If it is NULL then the pregnancy did not end, so we
     -- don't have mmintervals because the pregnancy is ongoing.)
     AND cp_seen
     -- If there are cycpoints then CYCSTATS exist.
    THEN

    -- Will we get NULL dini values or not?
    IF this_cpids IS NULL THEN
      last_mdate_date := NULL;  -- (anything +/- NULL = NULL)
    ELSE
      last_mdate_date := last_mdate.date;
    END IF;

    this_date := this_start; -- May be NULL, because the pregnancy never
                             -- ended.  In which case we don't loop.
    WHILE this_date <= obs_end LOOP

      --debug-- raise warning 'third %', this_date;

      INSERT INTO mdintervals (date, sname, dini, dr, cpids, cpide)
                  VALUES (this_date
                        , this_sname
                        , (this_date - last_mdate_date) + 1
                        , NULL
                        , this_cpids
                        , NULL);
      this_date := this_date + CAST('1 day' AS INTERVAL);
    END LOOP;
  END IF;

-- _do`'it() -- end expansion
})dnl
changequote(`,')dnl



CREATE OR REPLACE FUNCTION rebuild_mdintervals(this_sname mdintervals.sname%TYPE)
  RETURNS INT
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- Rebuild one individual's MDINTERVALS rows.
  --
  -- GPL_notice(`  --', `2008', `Karl O. Pinc <kop@meme.com>')
  --
  -- Syntax: rebuild_mdintervals(this_sname)
  --
  -- Arguments:
  --  this_sname   Individual to rebuild
  --
  -- Remarks:
  --  Alternate method from that of triggers so the two can be run
  -- independently and compared -- and so data can be regenerated
  -- without re-entry.  (The triggers only fire on data
  --  entry/modification.)
  --
  -- Bugs:

  DECLARE
    matured maturedates.matured%TYPE;
    statdate biograph.statdate%TYPE;
    this_sex biograph.sex%TYPE;

    this_date DATE;

--    cp cycpoints;
--    last_cp cycpoints;
    cg cycgaps;
    cp RECORD;
    last_mdate RECORD;
    last_mdate_date DATE;

    obs_state cycgaps.state%TYPE;
    obs_start DATE;
    obs_end DATE;

    this_cpids cycpoints.cpid%TYPE;
    this_start DATE;

    pregnancy BOOLEAN;
    cp_seen BOOLEAN;
  BEGIN

  -- Delete all the individual's mdintervals.
  DELETE
    FROM mdintervals
    WHERE sname = this_sname;

  --
  -- Iterate over the periods between gaps.
  --

  -- Start with matured date.
  SELECT INTO matured maturedates.matured
    FROM maturedates
    WHERE sname = this_sname;
  IF NOT FOUND THEN
    -- Individual is not mature, all done.
    RETURN 0;
  END IF;

  -- Go through the statdate.
  SELECT INTO statdate,          this_sex
              biograph.statdate, biograph.sex
    FROM biograph
    WHERE sname = this_sname;
  IF this_sex <> 'bb_female' THEN
    -- Individual is not female, all done.
    RETURN 0;
  END IF;

  obs_start := matured;

  cg.code = NULL; -- For testing if we had any gaps at all.
--  FOR cg IN SELECT code, date, state
  FOR cg IN SELECT *
              FROM cycgaps
              WHERE sname = this_sname
              ORDER BY date
      LOOP

    IF cg.code <> 'bb_cg_end' THEN
      -- Remember the state at observation start.      
      obs_state := cg.state;
      IF cg.date > obs_start THEN
        -- The start of observation is always (on or) after maturation.
        obs_start := cg.date;
      END IF;
    END IF;

    IF cg.code <> 'bb_cg_start' THEN
      -- We've reached the end of a period of observation
      -- Figure out when we want cycstats to end.
      IF cg.date < statdate THEN
        obs_end := cg.date;
      ELSE
        -- The end of observation is always on or before the statdate.
        obs_end := statdate;
      END IF;

      IF obs_end >= obs_start THEN
        -- The observation does not end before the maturation date
        -- or start after the statdate.

        -- Make the mdintervals for the observation period.
        _doit()
      END IF;
    END IF;

  END LOOP;

  IF cg.code IS NULL
     OR (cg.code = 'bb_cg_start' AND obs_start <= statdate) THEN
    -- There are no gaps, or the last gap ends before the statdate.
    -- Finish up the last period of observation.
    obs_end := statdate;
    _doit()
  END IF;

  RETURN 0;
  END;
$$;
grant_func_priv(`rebuild_cycstats(cycles.sname%TYPE)')
