dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2006, 2008, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Reassign a female's sexskins to their cycle.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl
dnl
dnl m4 includes
include(`copyright.m4')
include(`constants.m4')
include(`macros.m4')
include(`sexskinmacros.m4')
dnl

CREATE OR REPLACE FUNCTION rebuild_sexskins(this_sname cycles.sname%TYPE)
  RETURNS INT
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- Reassign a female's sexskins to a cycle based on date.
  --
  -- GPL_notice(`  --', `2006, 2008', `Karl O. Pinc <kop@meme.com>')
  --
  -- Syntax: rebuild_sexskins(this_sname)
  --
  -- Arguments:
  --   this_sname  The sname of the female who's sexskins should be re-assigned.
  --
  -- Remarks:
  --  Aborts with an exception if comes across a sexskin it can't assign.
  --
  --  Really only useful when triggers are turned off.
  --
  --  When triggers are on may as well do a:
  --    UPDATE sexskins
  --      SET sxid = sexskins.sxid
  --      FROM cycles
  --      WHERE cycles.sname = 'THIS_SNAME'
  --            AND sexskins.cid = cycles.cid;
  --
  -- Bugs:
  --   Not a independent method of assigning sexskins
  -- to cycles.  When triggers are on the triggers
  -- (the sexskins before insert trigger) will go ahead and
  -- re-assign the sexskin to whereever it wants to anyway.
  -- When triggers are off uses the same macro that the
  -- trigger uses anyhow.
  --

  DECLARE
    this_sexskin sexskins;
    new_cid cycpoints.cid%TYPE;

  BEGIN

    -- Over all the female's sexskins do the usual assignment to
    -- sexual cycle.
    FOR this_sexskin IN
      SELECT sexskins.* 
        FROM cycles, sexskins
        WHERE cycles.sname = this_sname
              AND sexskins.cid = cycles.cid
        ORDER BY sexskins.date
      LOOP

      -- Get a value for new_cid
      _find_sexskin_cid(`this_sexskin', `new_cid')

      IF new_cid <> this_sexskin.cid THEN
        -- Cid should change, update the data.
        UPDATE sexskins
          SET cid = new_cid
          WHERE sxid = sexskins.sxid;
      END IF;

    END LOOP;

  RETURN 0;
  END;
$$;
grant_func_priv(`rebuild_sexskins(cycles.sname%TYPE)')
