dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2004, 2006-2008, 2010, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Rnkdate function to convert date to a RANKS style date.
dnl Karl O. Pinc <kop@meme.com>
dnl
dnl
dnl m4 includes
include(`copyright.m4')
include(`constants.m4')
include(`macros.m4')
dnl

CREATE OR REPLACE FUNCTION rnkdate(this_date DATE)
  RETURNS DATE
  LANGUAGE plpgsql
  IMMUTABLE
  bb_function_set_search_path
  AS $$

  -- Convert date to RANKS style rnkdate.
  --
  -- GPL_notice(`  --', `2004, 2006-2008', `Karl O. Pinc <kop@meme.com>')
  --
  -- Syntax:  rnkdate(date)
  --
  -- Input:
  --   date  The date to convert.
  --
  -- Returns:
  --  The first of the month.
  --
  -- Remarks:
  --  Same function as date_trunc('month', this_date) but provided for
  -- foxpro compatibility and to avoid problems with timezones.

  DECLARE

  BEGIN
    RETURN CAST(date_trunc('month', CAST(this_date AS timestamp)) AS DATE);
  END;
$$;

grant_everybody_func_priv(`rnkdate(DATE)')

CREATE OR REPLACE FUNCTION rnkdate(this_date timestamp)
  RETURNS DATE
  LANGUAGE plpgsql
  IMMUTABLE
  bb_function_set_search_path
  AS $$

  -- Convert date to RANKS style rnkdate.
  --
  -- GPL_notice(`  --', `2004, 2006-2008, 2010', `Karl O. Pinc <kop@meme.com>')
  --
  -- Syntax:  rnkdate(date)
  --
  -- Input:
  --   date  The timetstamp to convert.
  --
  -- Returns:
  --  The first of the month.
  --
  -- Remarks:
  --  Same function as date_trunc('month', this_date::date) but provided for
  -- foxpro compatibility and to avoid problems with timezones.

  DECLARE

  BEGIN
    RETURN CAST(date_trunc('month', this_date) AS DATE);
  END;
$$;

grant_everybody_func_priv(`rnkdate(TIMESTAMP)')
