dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2005, 2008, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Seconds past midnight functions to support babase on the server side.
dnl Karl O. Pinc <kop@meme.com>
dnl
dnl
dnl m4 includes
include(`copyright.m4')
include(`constants.m4')
include(`macros.m4')
dnl

--
-- Babase Seconds Past Midnight Postgresql functions.
--

--
-- General purpose time functions.
--

--
-- To Seconds Past Midnight
--

CREATE OR REPLACE FUNCTION spm(this_time TIME)
  RETURNS DOUBLE PRECISION
  LANGUAGE plpgsql
  IMMUTABLE
  bb_function_set_search_path
  AS $$

  -- Convert a time into a number of seconds past midnight.
  --
  -- GPL_notice(`  --', `2005, 2008', `Karl O. Pinc <kop@meme.com>')
  --
  -- Syntax:  spm(time)
  --
  -- Input:
  --   time  The time to convert.
  --
  -- Returns:
  --  The number of seconds past midnight of the time.
  --
  -- Remarks:
  --   Really ought to be built-into postgresql.

  DECLARE

  BEGIN
    RETURN EXTRACT(EPOCH FROM this_time);
  END;
$$;

grant_everybody_func_priv(`spm(TIME)')



CREATE OR REPLACE FUNCTION spm(this_interval INTERVAL)
  RETURNS DOUBLE PRECISION
  LANGUAGE plpgsql
  IMMUTABLE
  bb_function_set_search_path
  AS $$

  -- Convert an interval into a number of seconds past midnight.
  --
  -- GPL_notice(`  --', `2005, 2008', `Karl O. Pinc <kop@meme.com>')
  --
  -- Syntax:  spm(interval)
  --
  -- Input:
  --   interval  The interval to convert.
  --
  -- Returns:
  --  The number of seconds in the interval modulo the number
  --  of seconds in a day.
  --
  -- Remarks:
  --   Slightly wierd.

  DECLARE

  BEGIN
    RETURN spm(CAST(this_interval AS TIME));
  END;
$$;

grant_everybody_func_priv(`spm(INTERVAL)')


CREATE OR REPLACE FUNCTION spm(this_timestamp TIMESTAMP)
  RETURNS DOUBLE PRECISION
  LANGUAGE plpgsql
  IMMUTABLE
  bb_function_set_search_path
  AS $$

  -- Convert a timestamp into a number of seconds past midnight.
  --
  -- GPL_notice(`  --', `2005, 2008', `Karl O. Pinc <kop@meme.com>')
  --
  -- Syntax:  spm(timestamp)
  --
  -- Input:
  --   timestamp  The timestamp to convert.
  --
  -- Returns:
  --  The number of seconds past midnight of the time portion of the
  --  timestamp.
  --
  -- Remarks:
  --   Really ought to be built-into postgresql.

  DECLARE

  BEGIN
    RETURN spm(CAST(this_timestamp AS TIME));
  END;
$$;

grant_everybody_func_priv(`spm(TIMESTAMP)')


-- Apparently, TIME(0) is the same as TIME when it comes to polymorphisim.
-- CREATE OR REPLACE FUNCTION spm(this_time TIME(0))
--   RETURNS INT
--   LANGUAGE plpgsql
--   IMMUTABLE
--   bb_function_set_search_path
--   AS $$
-- 
--   -- Convert a time(0) into a number of seconds past midnight.
--   --
--   -- GPL_notice(`  --', `2005, 2008', `Karl O. Pinc <kop@meme.com>')
--   --
--   -- Syntax:  spm(time)
--   --
--   -- Input:
--   --   time  The time to convert.
--   --
--   -- Returns:
--   --  The number of seconds past midnight of the time portion of the
--   --  time.
--   --
--   -- Remarks:
--   --   Really ought to be built-into postgresql.
-- 
--   DECLARE
-- 
--   BEGIN
--     RETURN CAST(spm(CAST(this_time AS TIME)) AS INT);
--   END;
-- $$;
-- 
-- grant_everybody_func_priv(`spm(TIME(0))')

--
-- From the number of seconds past midnight to a time.
--

CREATE OR REPLACE FUNCTION spm_to(secs DOUBLE PRECISION)
  RETURNS TIME
  LANGUAGE plpgsql
  IMMUTABLE
  bb_function_set_search_path
  AS $$

  -- Convert a number of seconds past midnight to its corresponding time.
  --
  -- GPL_notice(`  --', `2005, 2008', `Karl O. Pinc <kop@meme.com>')
  --
  -- Syntax:  spm_to(secs)
  --
  -- Input:
  --   secs  The number of seconds to convert.
  --
  -- Returns:
  --  The time value for the given number of seconds past midnight.
  --
  -- Remarks:
  --   Really ought to be built-into postgresql.

  DECLARE

  BEGIN
    RETURN CAST ( secs * CAST ('1 second' AS interval) AS time);
  END;
$$;

grant_everybody_func_priv(`spm_to(DOUBLE PRECISION)')


CREATE OR REPLACE FUNCTION spm_to(secs INT)
  RETURNS TIME
  LANGUAGE plpgsql
  IMMUTABLE
  bb_function_set_search_path
  AS $$

  -- Convert a number of seconds past midnight to its corresponding time.
  --
  -- GPL_notice(`  --', `2005, 2008', `Karl O. Pinc <kop@meme.com>')
  --
  -- Syntax:  spm_to(secs)
  --
  -- Input:
  --   secs  The number of seconds to convert.
  --
  -- Returns:
  --  The time value for the given number of seconds past midnight.
  --
  -- Remarks:
  --   Really ought to be built-into postgresql.

  DECLARE

  BEGIN
    RETURN CAST ( secs * CAST ('1 second' AS interval) AS time);
  END;
$$;

grant_everybody_func_priv(`spm_to(INT)')
