dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2007, 2008, 2010, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Compute a group's supergroup on a given date.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl
dnl
dnl m4 includes
include(`copyright.m4')
include(`constants.m4')
include(`types.m4')
include(`macros.m4')
dnl

CREATE OR REPLACE FUNCTION supergroup(grp b_grp_type, date DATE)
  RETURNS b_grp_type
  LANGUAGE plpgsql
  STABLE
  SECURITY INVOKER
  bb_function_set_search_path
  AS $$
  -- Compute a group's supergroup on a given date.
  --
  -- GPL_notice(`  --', `2007, 2008, 2010', `Karl O. Pinc <kop@meme.com>')
  --
  -- Syntax: supergroup(grp, date)
  --
  -- Arguments:
  --   grp    The group to find the supergroup for.
  --   date   Find supergroup as of the given date
  --
  -- Returns:
  --   The supergroup of the given group on the given date.
  --
  -- Remarks:
  --   A group's supergroup changes, depending on when we're
  --  talking about.
  --
  -- Bugs:

  DECLARE
    this_gid groups.gid%TYPE := grp;
    this_group groups%ROWTYPE;

  BEGIN

    LOOP
      -- Find the current group row.
      SELECT INTO this_group *
        FROM groups
        WHERE groups.gid = this_gid;

      IF this_group.permanent <= date THEN
        -- We found where the group became permanent.
        EXIT;
      END IF;

      this_gid = this_group.from_group;

      IF this_gid IS NULL THEN
        -- Return NULL instead of raising an exception to support
        -- easy use of the supergroup in queries.
        --RAISE EXCEPTION 'Cannot find supergroup of group % on %: group % has no supergroup before %'
        --                , grp
        --                , date
        --                , this_group.gid
        --                , this_group.permanent;
        RETURN NULL;
      END IF;
    END LOOP;

    RETURN this_group.supergroup;
  END;
$$;
grant_func_priv(`supergroup('b_grp_type`, DATE)')
