dnl Copyright (C) 2016 Jake Gordon, <jacob.gordon@duke.edu>
dnl Copyright (C) 2015 The Meme Factory, Inc.  http://www.meme.com/
dnl Copyright (C) 2007, 2008, 2010, 2011, Karl O. Pinc
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl
dnl Macros used to create tables.
dnl
dnl
dnl Define m4 macros so we don't have to type so much.
dnl (Or at least type stuff that's more fun.)
dnl
dnl Watch out for using the single quote char as it has speical
dnl meaning to m4.
dnl
dnl The dates and timestamps should probably have some sort of
dnl reasonable constraints on them to prevent operator error.
dnl (Like entering dates from the last century.)
dnl
dnl Bugs:
dnl
changequote([{[,]}])dnl m4 foolery so includes include only once.
dnl                     Once the macro is in the text, change the quotes back
ifdef([{[_tablemacros.m4]}], [{[changequote(`,')]}], [{[dnl
changequote(`,')dnl
dnl
dnl Standard test for having already included the file.
define(`_tablemacros.m4')dnl
dnl
dnl Don't output anything while defining macros.
divert(-1)

dnl Make a table with/without a INTEGER SERIAL id.
dnl Syntax: serial_table(tablename, id_datatype, tablecols)
changequote([,])
define([serial_table],[
ifelse($2, [INTEGER],changequote([,])dnl  Remove braces surrounding expanded text
[dnl
dnl Integer key, use SERIAL datatype
CREATE TABLE $1 (
  id SERIAL PRIMARY KEY,
]changequote(`,')dnl  See above.
, changequote([,])dnl  See above.
[dnl
dnl Non-INTEGER key, simulate SERIAL.
CREATE SEQUENCE $1_id_seq;
CREATE TABLE $1 (
  id $2 PRIMARY KEY NOT NULL DEFAULT nextval('$1_id_seq'),
]changequote(`,')dnl  See above.
)dnl
$3dnl
);
grant_priv($1)
])
changequote(`,')


dnl A hierarchical table.
dnl Syntax: hier_table(tablename, id_datatype, parent, parent_datatype)
define(`hier_table',`
serial_table($1, $2, `dnl
  descr TEXT NOT NULL,
  $3 $4 NOT NULL CONSTRAINT "$3 must be on $3" REFERENCES $3
')
')

dnl Create a support table.
dnl Syntax: support_table(tablename, id_name, id_datatype, id_constraint)
define(`support_table',`
CREATE TABLE $1 (
  $2 $3 PRIMARY KEY NOT NULL $4,
  descr TEXT NOT NULL UNIQUE
        empty_string_check(descr)
);
grant_priv($1)
')

--
-- Data types/column definitions
--

-- Sname
define(`sname_column', `sname_type_column(sname, Sname, NULL)')dnl

dnl A Sname column with a different name.
dnl Syntax: sname_type_column(colname, Colname, null)
dnl
dnl colname  The name of the column in lower case.
dnl Colname  The name of the column in camelcaps.
dnl null     Either `NULL' or `NOT NULL'.
dnl
define(`sname_type_column', `
  $1 CHAR(3) $3
        CONSTRAINT "$2 on BIOGRAPH" REFERENCES biograph(sname)')dnl

-- Sname -- on MATUREDATES
-- There's a few tables that require a maturity date.
define(`maturedates_sname_column', `
  sname CHAR(3) NOT NULL
        CONSTRAINT "Sname on MATUREDATES" REFERENCES maturedates(sname)')dnl

--
-- Constraints
--

-- A column cannot contain the empty string or only whitespace.
--
-- Syntax: empty_string_check(col)
--
-- Input:
--   col  Name of the column to check.
--
changequote([,])
define([empty_string_check], [           CONSTRAINT "$1: Cannot be empty or only whitespace characters"
                CHECK(btrim($1, E' \r\n\t\f\x0B') <> '')])dnl
changequote(`,')dnl  See above.

-- A column cannot contain only whitespace.
--
-- Syntax: no_spaces_check(col)
--
-- Input:
--   col  Name of the column to check.
--
changequote([,])
define([no_spaces_check], [           CONSTRAINT "$1: Cannot be only whitespace characters"
                CHECK($1 IS NULL OR $1 = '' OR btrim($1, E' \r\n\t\f\x0B') <> '')])dnl
changequote(`,')dnl  See above.

-- A column cannot contain any whitespace.
--
-- Syntax: space_not_allowed_check(col)
--
-- Input:
--   col  Name of the column to check.
--
changequote([,])
define([space_not_allowed_check], [           CONSTRAINT "$1: Cannot contain any whitespace characters"
                CHECK($1 IS NULL OR
                      $1 !~ '( |\r|\n|\t|\f|\x0B)')])dnl
changequote(`,')dnl  See above.

-- A time-ish column cannot contain seconds or fractions of a second.
--
-- Syntax: no_seconds_check(col)
--
-- Input:
--   col  Name of the column to check.
--
changequote([,])
define([no_seconds_check], [              CONSTRAINT "$1: Cannot contain seconds or fractions thereof"
                CHECK (EXTRACT(SECONDS FROM $1) = 0)])dnl
changequote(`,')dnl  See above

-- A pair of columns must both be either NULL or non-NULL.
--
-- Syntax: null_iff_null(col1, col2)
--
-- Input:
--   col1  Name of one of the columns.
--   col2  Name of the other column.
--  
changequote([,])
define([null_iff_null], [            CONSTRAINT "$1 must be NULL if and only if $2 is NULL"
                       CHECK(($1 IS NULL AND $2 IS NULL)
                             OR ($1 IS NOT NULL AND $2 IS NOT NULL))])dnl
changequote(`,')dnl  See above

-- One column must be NULL if another is.
--
-- Syntax: null_if_null(col1, col2)
--
-- Input:
--   col1  This column must be null if col2 is null.
--   col2  Name of the referenced column.
--  
changequote([,])
define([null_if_null], [            CONSTRAINT "$1 must be NULL if $2 is NULL"
                       CHECK(($2 IS NULL AND $1 IS NULL)
                             OR $2 IS NOT NULL)])dnl
changequote(`,')dnl  See above

-- Number must be between min and max, inclusive
--
-- Syntax: between_incl_num(col, min, max)
--
-- Input:
--   col   Column we're constraining
--   min   Minimum (inclusive) allowed number
--   max   Maximum (inclusive) allowed number
--
changequote([,])
define([between_incl_num], [            CONSTRAINT "$1 must be between $2 and $3, inclusive"
                       CHECK($1 BETWEEN $2 AND $3)])dnl
changequote(`,')dnl  See above

-- Value must be between min and max, inclusive
-- (Same as above, but uses '' around values)
--
-- Syntax: between_incl_value(col, min, max)
--
-- Input:
--   col   Column we're constraining
--   min   Minimum (inclusive) allowed value (not a number)
--   max   Maximum (inclusive) allowed value (not a number)
--
changequote([,])
define([between_incl_value], [            CONSTRAINT "$1 must be between $2 and $3, inclusive"
                       CHECK($1 BETWEEN '$2' AND '$3')])dnl
changequote(`,')dnl  See above

--
--  Macros to make whole tables.
--

-- TESTES_foo
--
-- Syntax: _make_testes_table(suffix)
--
-- Input:
--   suffix  The suffix at the end of the table name.
--

changequote([,])
define([_make_testes_table], [dnl
-- TESTES_$1
dnl Ugly ifelse hack here to increase testlength for testes_arc.
CREATE TABLE testes_$1 (
  testesid SERIAL PRIMARY KEY,
  dartid INT NOT NULL
         CONSTRAINT "Dartid on DARTINGS" REFERENCES dartings,
  testside CHAR(1) NOT NULL
           CONSTRAINT "Testside must be bb_left or bb_right"
             CHECK (testside = 'bb_left' or testside = 'bb_right'),
  testlength NUMERIC(ifelse($1, `ARC', `4', `3'),1) NULL
         CONSTRAINT "Testlength cannot be less than translit(bb_min_$1_testlength, A-Z, a-z)`'mm"
           CHECK (testlength >= translit(bb_min_$1_testlength, A-Z, a-z))
         CONSTRAINT "Testlength cannot be more than translit(bb_max_$1_testlength, A-Z, a-z)`'mm"
           CHECK (testlength <= translit(bb_max_$1_testlength, A-Z, a-z)),
  testwidth NUMERIC(3,1) NULL
   CONSTRAINT "Testwidth cannot be less than translit(bb_min_$1_testwidth, A-Z, a-z)`'mm"
           CHECK (testwidth >= translit(bb_min_$1_testwidth, A-Z, a-z))
         CONSTRAINT "Testwidth cannot be more than translit(bb_max_$1_testwidth, A-Z, a-z)`'mm"
           CHECK (testwidth <= translit(bb_max_$1_testwidth, A-Z, a-z)),
  testseq SMALLINT NULL
         CONSTRAINT "Testseq cannot be less than 1"
           CHECK (testseq >= 1)
  CONSTRAINT "Testlength and Testwidth cannot both be NULL"
    CHECK (Testlength IS NOT NULL OR Testwidth IS NOT NULL)
) WITHOUT OIDS;

grant_priv(`testes_$1')
grant_seq_priv(`testes_$1', `testesid')])dnl
changequote(`,')dnl  See above

-- Done defining macros.
divert`'dnl

]}])dnl End of ifdef over the whole file.
