dnl Copyright (C) 2016-17 Jake Gordon, <jacob.gordon@duke.edu>
dnl Copyright (C) 2015 The Meme Factory, Inc.  http://www.meme.com/
dnl Copyright (C) 2004, 2005, 2008, 2009, 2011 Karl O. Pinc
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl
dnl Remarks:
dnl
dnl m4 includes
include(`copyright.m4')
include(`indexmacros.m4')

-- Create indexes on all the babase tables.

-- Remarks:
-- A separate file from maketables.m4 so that we can
-- index on the result of applying our functions to our data.
-- Smaller tables, like support tables, don't benefit
-- from indexes.
-- All tables with primary keys already have indexes on these.
-- Unless specificly noted, the UNIQUE keyword is not there
-- to enforce uniqueness, which may be done in the triiger,
-- but is there to help the query optimizer.

-- All support tables already have unique indexes on Descr as well.
-- statuses
-- bstats
-- mstats
-- sexes
-- pcscolors
-- ncodes
-- foodtypes
-- psions

dnl Macros


-- dcauses
-- Created to enforce uniqueness.  It would be useful for
-- searching but the table has too little data to make it worthwhile.
CREATE UNIQUE INDEX dcauses_nature_agent ON dcauses (nature, agent);

-- observers
-- Created to enforce uniqueness.  It would be useful for
-- searching but the table has too little data to make it worthwhile.
CREATE UNIQUE INDEX observers_name ON observers (name);

-- acts
CREATE UNIQUE INDEX acts_descr ON acts (descr);

-- rnktypes
CREATE UNIQUE INDEX rnktypes_descr ON rnktypes (descr);

-- foodcodes
CREATE UNIQUE INDEX foodcodes_descr ON foodcodes (descr);

-- activities
CREATE UNIQUE INDEX activities_descr ON activities(descr);

-- ncodes
CREATE UNIQUE INDEX ncodes_descr ON ncodes(descr);

-- postures
CREATE UNIQUE INDEX postures_descr ON postures (descr);

-- groups
CREATE UNIQUE INDEX groups_name ON groups (name);
CREATE UNIQUE INDEX groups_three_letter_code ON groups (three_letter_code);
-- We'd want this for speed, but groups is too small.
--CREATE INDEX groups_supergroup ON groups (supergroup);

-- behave_gaps
CREATE INDEX behave_gaps_gid ON behave_gaps (grp);
CREATE INDEX behave_gaps_gap_start ON behave_gaps (gap_start);
CREATE INDEX behave_gaps_gap_end ON behave_gaps (gap_end);
CREATE INDEX behave_gaps_gap_end_status ON behave_gaps (gap_end_status);

-- biograph

-- CREATE INDEX biograph_pid ON biograph (pid) WHERE pid IS NOT NULL;
CREATE UNIQUE INDEX biograph_pid_sname ON biograph (pid, sname);
CREATE UNIQUE INDEX biograph_matgrp_sname ON biograph (matgrp, sname);
CREATE UNIQUE INDEX biograph_name ON biograph(LOWER(name))
                    WHERE name IS NOT NULL;

-- alternate_snames

CREATE INDEX alternate_snames_sname
       ON alternate_snames (sname);
CREATE INDEX altername_snames_sname_alternate
       ON alternate_snames (sname_alternate);
CREATE INDEX altername_snames_name_alternate
       ON alternate_snames (name_alternate);

-- maturedates
-- rankdates
-- consortdates
-- dispersedates

-- pregs

-- Enforce uniqueness.
CREATE UNIQUE INDEX pregs_conceive ON pregs (conceive);
CREATE UNIQUE INDEX pregs_conceive_pid ON pregs (conceive, pid);
-- Enforce uniqueness.
CREATE UNIQUE INDEX pregs_resume ON pregs (resume);
CREATE UNIQUE INDEX pregs_resume_pid ON pregs (resume, pid);

-- cycgaps

-- Enforce uniqueness in the triggers so we can drop
-- a start and end on the same date to eliminate a gap.
CREATE INDEX cycgaps_sname_date ON cycgaps (sname, date);

-- cycgapdays
-- (This first makes the triggers ever so much faster.)
CREATE UNIQUE INDEX cycgapdays_sname_date ON cycgapdays (sname, date);
CREATE UNIQUE INDEX cycgapdays_date_sname ON cycgapdays (date, sname);
CREATE INDEX cycgapdays_sname ON cycgapdays (sname);

-- cycpoints
CREATE INDEX cycpoints_date_code ON cycpoints (date, code);
-- Is it really worth the date here?
CREATE INDEX cycpoints_cid_date ON cycpoints (cid, date);
-- Enforce uniqueness
CREATE INDEX cycpoints_cid_code ON cycpoints (cid, code);

-- cycles
CREATE INDEX cycles_sname_cid ON cycles (sname, cid);
CREATE INDEX cycles_sname_seq ON cycles (sname, seq);

-- sexskins

CREATE INDEX sexskins_cid ON sexskins (cid);

-- repstats

CREATE UNIQUE INDEX repstats_date_sname_pid ON repstats (date, sname, pid);
CREATE INDEX repstats_sname_pid ON repstats (sname, pid);
CREATE INDEX repstats_pid ON repstats (pid);

-- cycstats

CREATE UNIQUE INDEX cycstats_date_sname ON cycstats (date, sname);
CREATE INDEX cycstats_sname_csid ON cycstats (sname, csid);    
CREATE INDEX cycstats_cpids_csid ON cycstats (cpids, csid);
CREATE INDEX cycstats_cpide_csid ON cycstats (cpide, csid);

-- mmintervals
CREATE UNIQUE INDEX mmintervals_date_sname ON mmintervals (date, sname);
CREATE INDEX mmintervals_sname_mmid on mmintervals (sname, mmiid);
CREATE INDEX mmintervals_cpids ON mmintervals (cpids);
CREATE INDEX mmintervals_cpide on mmintervals (cpide);

-- mdintervals
CREATE UNIQUE INDEX mdintervals_date_sname ON mdintervals (date, sname);
CREATE INDEX mdintervals_sname_mmid on mdintervals (sname, mdiid);
CREATE INDEX mdintervals_cpids ON mdintervals (cpids);
CREATE INDEX mdintervals_cpide on mdintervals (cpide);

-- members

CREATE INDEX members_sname_grp ON members (sname, grp);
CREATE UNIQUE INDEX members_date_sname ON members (date, sname);
CREATE INDEX members_grp_date ON members (grp, date);
-- This is not really needed execpt for re-interpolating
-- all of members, because the old census types mean
-- that so often _all_ the interpolated rows are deleted.
-- But otherwise re-interpolating everybody (or loading the db
-- for the first time) takes days.
CREATE INDEX members_sname_origin ON members (sname, origin);
-- Try this index for the members delete trigger.
CREATE INDEX members_sname_rnkdate_grp
  ON members (sname, rnkdate(date), grp);

-- census

CREATE INDEX census_sname ON census (sname);
CREATE INDEX census_date_sname ON census (date, sname);
CREATE INDEX census_grp_date ON census (grp, date);
-- Not ordinarly useful but speeds initial data load.
CREATE INDEX census_status_sname ON census (status, sname);
-- Maybe this will speed things in the future...
-- As of pg 8.0.3 the optimizer does not seem to choose this index.
CREATE INDEX census_sname_date ON census (sname, date);

-- demog

CREATE UNIQUE INDEX demog_reference_cenid ON demog (reference, cenid);

-- ranks

-- Enforce uniqueness.
CREATE UNIQUE INDEX ranks_uniqueness
       ON ranks (rnktype, sname, rnkdate, grp);
--
-- Try out this whole business of merging index bitmaps.
--
-- Go wild indexing ranks.
--CREATE INDEX ranks_sname_rnkdate_grp
--       ON ranks (sname, rnkdate, grp);
--CREATE INDEX ranks_grp_sname
--       ON ranks (grp, sname);
--CREATE INDEX ranks_sname_rnktype_grp
--       ON ranks (sname, rnktype, grp);
--CREATE INDEX ranks_rnkdate_grp
--       ON ranks (rnkdate, grp);
CREATE INDEX ranks_sname ON ranks (sname);
CREATE INDEX ranks_grp_ndx ON ranks (grp);
CREATE INDEX ranks_rnktype ON ranks (rnktype);
CREATE INDEX ranks_rnkdate ON ranks (rnkdate);

-- interact_data

CREATE UNIQUE INDEX interact_data_sid_iid ON interact_data (sid, iid);
CREATE UNIQUE INDEX interact_data_act_iid ON interact_data (act, iid);
CREATE UNIQUE INDEX interact_data_date_iid ON interact_data (date, iid);

-- parts

CREATE INDEX parts_sname_iid ON parts (sname, iid);
CREATE INDEX parts_iid ON parts (iid);
--CREATE UNIQUE INDEX parts_iid_role ON parts (iid, role);

-- samples

CREATE INDEX samples_sname_grp_date ON samples (sname, grp, date);
CREATE INDEX samples_grp_date ON samples (grp, date);
CREATE INDEX samples_sname_date ON samples (sname, date);
CREATE INDEX samples_date ON samples (date);

-- point_data

CREATE INDEX point_data_sid_pntid ON point_data (sid, pntid);
-- Enforce some uniqueness.
CREATE UNIQUE INDEX point_data_ptime_sid_pntid
       ON point_data (ptime, sid, pntid);

-- fpoints

-- neighbors

CREATE INDEX neighbors_sname_pntid ON neighbors (sname, pntid);
CREATE INDEX neighbors_unkid_pntid ON neighbors (unksname, pntid);
-- Enforce uniqueness
CREATE UNIQUE INDEX neighbors_pntid_ncode ON neighbors (pntid, ncode);

-- wreadings
CREATE UNIQUE INDEX wreadings_daytimes_station
  ON wreadings (wrdaytime, wstation);

-- rgsetups
CREATE UNIQUE INDEX rgsetups_daytimes_station
  ON rgsetups (rgsdaytime, wstation);

-- weatherhawk
CREATE UNIQUE INDEX weatherhawk_timestamp_wstation
  ON weatherhawk (timestamp, wstation);

-- Should help performance for calculating TimeStampRain
CREATE INDEX weatherhawk_wstation_timestamp_yearlyrain
  ON weatherhawk (wstation, timestamp, yearlyrain);

-- mpis

CREATE INDEX mpis_date ON mpis (date);

-- consorts

CREATE INDEX consorts_female ON consorts (female);
CREATE INDEX consorts_had ON consorts (had);
CREATE INDEX consorts_got ON consorts (got);

-- mpiacts do not have enough rows to index

-- mpi_data

CREATE INDEX mpi_data_mpiid ON mpi_data (mpiid);
CREATE INDEX mpi_data_mpiact ON mpi_data (mpiact);
CREATE INDEX mpi_data_mpidid_seq_col ON mpi_data (mpidid, seq);

-- mpi_parts

CREATE INDEX mpi_parts_mpidid ON mpi_parts (mpidid);
CREATE INDEX mpi_parts_sname ON mpi_parts (sname);

-- dad_data

CREATE UNIQUE INDEX dad_data_kid ON dad_data(kid);
CREATE INDEX dad_data_dad_excl ON dad_data(dad_excl);
CREATE INDEX dad_data_dad_1perr ON dad_data(dad_1perr);
CREATE INDEX dad_data_dad_5perr ON dad_data(dad_5perr);
CREATE INDEX dad_data_dad_allmales ON dad_data(dad_allmales);
CREATE INDEX dad_data_dad_consensus ON dad_data(dad_consensus);

 -- toothcodes

CREATE INDEX toothcodes_toothsite ON toothcodes(toothsite);

-- dartings

CREATE INDEX dartings_sname ON dartings(sname);
CREATE INDEX dartings_date ON dartings(date);
CREATE UNIQUE INDEX dartings_sname_date ON dartings(sname, date);

-- anesths

CREATE INDEX anesths_dartid ON anesths(dartid);
CREATE INDEX anesths_antime ON anesths(antime);

-- crownrumps

CREATE INDEX crownrumps_dartid ON crownrumps(dartid);

-- chests

CREATE INDEX chests_dartid ON chests(dartid);

-- dart_samples

CREATE INDEX dart_samples_dartid on dart_samples(dartid);
CREATE INDEX dart_samples_ds_type on dart_samples(ds_type);
CREATE UNIQUE INDEX dart_samples_dartid_ds_type on dart_samples(dartid, ds_type);

-- wbc_counts

CREATE INDEX wbc_counts_dartid on wbc_counts(dartid);
CREATE UNIQUE INDEX wbc_counts_dartid_slide_number on wbc_counts(dartid, slide_number);

-- ulnas

CREATE INDEX ulnas_dartid ON ulnas(dartid);

-- humeruses

CREATE INDEX humeruses_dartid ON humeruses(dartid);

-- dphys

CREATE UNIQUE INDEX dphys_dartid ON dphys(dartid);

-- pvcs

CREATE INDEX pvcs_dartid ON pcvs(dartid);

-- bodytemps

CREATE INDEX bodytemps_dartid ON bodytemps(dartid);
CREATE INDEX bodytemps_bttime ON bodytemps(bttime);

-- teeth

CREATE INDEX teeth_dartid_tooth ON teeth(dartid, tooth);

-- testes_arc

_make_testes_indexes(`ARC')

-- testes_diam

_make_testes_indexes(`DIAM')

-- ticks

CREATE INDEX ticks_dartid ON ticks(dartid);
CREATE UNIQUE INDEX ticks_dartid_bodypart_tickkind
  ON ticks(dartid, bodypart, tickkind);

-- SWERB Indexes

-- gps_units

CREATE UNIQUE INDEX gps_units_gps ON gps_units (gps);
CREATE INDEX gps_units_start ON gps_units (start);
CREATE INDEX gps_units_finish ON gps_units (finish);

-- swerb_data

CREATE INDEX swerb_data_beid ON swerb_data (beid);
CREATE INDEX swerb_data_seen_grp ON swerb_data (seen_grp);
CREATE INDEX swerb_data_time ON swerb_data (time);
CREATE INDEX swerb_data_quad ON swerb_data (quad);
CREATE INDEX swerb_data_xyloc ON swerb_data USING GIST (xyloc);
CREATE INDEX swerb_data_altitude ON swerb_data (altitude);

-- swerb_departs_data

CREATE INDEX swerb_departs_data_date ON swerb_departs_data (date);
CREATE INDEX swerb_departs_data_time ON swerb_departs_data (time);
CREATE INDEX swerb_departs_data_date_time ON swerb_departs_data (date, time);

-- swerb_departs_gps

CREATE UNIQUE INDEX swerb_departs_gps_did ON swerb_departs_gps (did);
CREATE INDEX swerb_departs_gps_altitude ON swerb_departs_gps (altitude);
CREATE INDEX swerb_departs_gps_xyloc ON swerb_departs_gps USING GIST (xyloc);
CREATE INDEX swerb_departs_gps_gps ON swerb_departs_gps (gps);

-- swerb_bes

CREATE INDEX swerb_bes_did ON swerb_bes (did);
CREATE INDEX swerb_bes_focal_grp_seq ON swerb_bes (focal_grp, seq);
CREATE INDEX swerb_bes_start ON swerb_bes (start);
CREATE INDEX swerb_bes_stop ON swerb_bes (stop);

-- quad_data

CREATE INDEX quad_data_xyloc ON quad_data USING GIST (xyloc);

-- swerb_loc_data

CREATE INDEX swerb_loc_data_loc ON swerb_loc_data (loc);

-- swerb_gw_loc_data

CREATE INDEX swerb_gw_loc_data_xyloc 
             ON swerb_gw_loc_data USING GIST(xyloc);
CREATE INDEX swerb_gw_loc_data_date
             ON swerb_gw_loc_data (date);
CREATE INDEX swerb_gw_loc_data_altitude
             ON swerb_gw_loc_data (altitude);
CREATE INDEX swerb_gw_loc_data_gps
             ON swerb_gw_loc_data (gps);

-- trees

CREATE UNIQUE INDEX trees_loc_tree ON trees (loc, tree);

-- hybridgene_analyses

CREATE INDEX hybridgene_analyses_date ON hybridgene_analyses (date);
CREATE INDEX hybridgene_analyses_analyzed_by ON hybridgene_analyses (analyzed_by);
CREATE INDEX hybridgene_analyses_software ON hybridgene_analyses (software);
CREATE INDEX hybridgene_analyses_marker ON hybridgene_analyses (marker);

-- hybridgene_scores

CREATE INDEX hybridgene_scores_hgaid ON hybridgene_scores (hgaid);
CREATE INDEX hybridgene_scores_sname ON hybridgene_scores (sname);

CREATE UNIQUE INDEX hybridgene_scores_hgaid_sname
                    ON hybridgene_scores (hgaid, sname);


--
-- The warning system.
--
include(`makewarningindexes.m4')
