dnl Copyright (C) 2013-2017 Jake Gordon, <jacob.gordon@duke.edu>
dnl Copyright (C) 2011-2015 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2004-2011 Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl
dnl Remarks:
dnl   All the uniqueness constraints (primary keys and oids excepted)
dnl are done in the indexes, UNIQUE() is not used.  This is because
dnl uniqueness constraints create indexes and it's easier to see
dnl all the indexes on a table together.
dnl
dnl m4 includes
include(`copyright.m4')
include(`constants.m4')
include(`grants.m4')
include(`types.m4')
include(`tablemacros.m4')

-- Make all the babase tables and give out privilges.
-- $Id: maketables.m4,v 1.75 2004/11/09 19:09:51 kop Exp $


--
-- Make all the tables and give out privilges.
--

--
-- Support Tables
--

--  Meaning of status codes hardcoded into integrity rules.
support_table(`demog_references', `reference', `NUMERIC(5,3)')
support_table(`statuses', `status', `SMALLINT', `')
support_table(`bstatuses', `bstatus', `NUMERIC(2,1)', `')
support_table(`confidences', `confidence', `SMALLINT', `')
support_table(`entrytypes', `entrytype', `CHAR(1)'
              , `empty_string_check(`entrytype')')
support_table(`mstatuses', `mstatus', `CHAR(1)'
              , `empty_string_check(`mstatus')')
support_table(`pcscolors', `color', `CHAR(1)'
              , `empty_string_check(`color')')
support_table(`foodtypes', `ftype', `CHAR(1)'
              , `empty_string_check(`ftype')')
support_table(`kidcontacts', `kidcontact', `CHAR(1)'
              , `empty_string_check(`kidcontact')')
support_table(`suckles', `suckle', `CHAR(1)'
              , `empty_string_check(`suckle')')
support_table(`palmtops', `palmtop', `SMALLINT', `')
support_table(`data_structures', `data_structure', `SMALLINT', `')
support_table(`gap_end_statuses',`gap_end_status', `SMALLINT')
dnl support_table(`sexes', `sex', `CHAR(1)'
dnl               , `empty_string_check(`sex')')
support_table(`wstations', `wstation', `VARCHAR(3)'
              , `empty_string_check(`wstation')')
support_table(`partunks', `unksname', `CHAR(1)'
              , `empty_string_check(`unksname')')
support_table(`context_types', `context_type', `CHAR(1)'
              , `empty_string_check(`context_type')')
support_table(`observer_roles', `role', `VARCHAR(bb_observer_roles_len)', `')
dnl support_table(`cr_statuses', `cr_status', `CHAR(1)',
dnl              `empty_string_check(`cr_status')')
support_table(`drugs', `drug', `CHAR(1)',
              `empty_string_check(`drug')')
support_table(`lymphstates', `lymphstate', `CHAR(1)',
              `empty_string_check(`lymphstate')')
support_table(`parasites', `parasite', `CHAR(1)',
              `empty_string_check(`parasite')')
support_table(`dart_sample_cats', `ds_cat', `SMALLINT')
support_table(`tconditions', `tcondition', `CHAR(1)',
              `empty_string_check(`tcondition')')
support_table(`tickstatuses', `tickstatus', `SMALLINT')
support_table(`toothsites', `toothsite', `SMALLINT')
support_table(`tstates', `tstate', `CHAR(1)',
              `empty_string_check(`tstate')')
-- swerb
support_table(`swerb_loc_confidences', `confidence', `NUMERIC(2,1)')
support_table(`swerb_loc_statuses', `loc_status', `CHAR(1)',
              `empty_string_check(`loc_status')')
support_table(`swerb_time_sources', `source', `VARCHAR(5)',
              `empty_string_check(`source')')
support_table(`place_types', `place', `CHAR(1)',
              `empty_string_check(`place')')
support_table(`swerb_xysources', `xysource', `CHAR(1)',
              `empty_string_check(`XYSource')
               CONSTRAINT "bb_xysource_quad is not an allowed XYSource value"
                 CHECK(xysource <> ''`bb_xysource_quad''`)')
support_table(`paternity_completeness', `completeness', `SMALLINT')
support_table(`paternity_mismatches', `mismatch', `VARCHAR(12)',
               `empty_string_check(`mismatch')')
support_table(`hybridgene_software', `software', `SMALLINT')
support_table(`markers', `marker', `SMALLINT')
support_table(`deathnatures', `nature', `VARCHAR(30)')


CREATE TABLE weatherhawk_softwares (
  wsoftware VARCHAR(3) PRIMARY KEY NOT NULL
            empty_string_check(`wsoftware'),
  name      TEXT NOT NULL
            empty_string_check(`name'),
  descr     TEXT NOT NULL
            empty_string_check(`descr')
  ) WITHOUT OIDS;
grant_priv(`weatherhawk_softwares')

--
-- The primary tables
-- 

-- Note that we don't have to grant execute privileges on the functions
-- we use in triggers as the triggers execute with our (the database
-- owner's) permissions.

-- dcauses
CREATE TABLE dcauses (
  dcause   SMALLINT PRIMARY KEY NOT NULL,
  nature   VARCHAR(30) NOT NULL
           -- Using VARCHAR and not SMALLINT because leadership didn't
           -- like the look of DCAUSES when "nature" was just an
           -- integer from another support table. For similar reasons,
           -- 30 character length was chosen because one of the
           -- intended nature values really is that long, and
           -- apparently no one wants to use an abbreviation and
           -- require people to actually look at the DEATHNATURES
           -- table for more details.
           CONSTRAINT "Nature on DEATHNATURES" REFERENCES deathnatures,
  agent    TEXT NOT NULL
           empty_string_check(`agent')

) WITHOUT OIDS;
grant_priv(`dcauses')

-- observers
CREATE TABLE observers (
  initials       VARCHAR(4) PRIMARY KEY NOT NULL
                 empty_string_check(`Initials'),
  oldgpsinitials TEXT NULL
                 empty_string_check(`Oldgpsinitials'),
  name           TEXT NOT NULL
                 empty_string_check(`name'),
  timeframe      TEXT NULL
                 empty_string_check(`timeframe'),
  notes          TEXT NULL
                 empty_string_check(`notes'),
  role           VARCHAR(bb_observer_roles_len) NOT NULL
                 CONSTRAINT "Role on OBSERVER_ROLES" REFERENCES observer_roles,
  swerb_observer_role VARCHAR(bb_observer_roles_len) NOT NULL
                 CONSTRAINT "SWERB_Observer_Role on OBSERVER_ROLES" REFERENCES observer_roles,
  swerb_driver_role VARCHAR(bb_observer_roles_len) NOT NULL
                 CONSTRAINT "SWERB_Driver_Role on OBSERVER_ROLES" REFERENCES observer_roles

) WITHOUT OIDS;
grant_priv(`observers')

-- acts

CREATE TABLE acts (
  act     CHAR(2) PRIMARY KEY NOT NULL
          empty_string_check(`act'),
  descr   TEXT NOT NULL
          empty_string_check(`descr'),
  class   CHAR(1) NOT NULL
          CONSTRAINT "Class on ACTS" REFERENCES acts,
  retired BOOLEAN NOT NULL,
  CONSTRAINT "Class one of bb_agonism, bb_grooming, bb_mounts, bb_consortship, bb_ejaculation, bb_approach, or bb_request_to_groom"
    CHECK(class = 'bb_agonism'
          OR class = 'bb_grooming'
          OR class = 'bb_mounts'
          OR class = 'bb_consortship'
          OR class = 'bb_ejaculation'
          OR class = 'bb_approach'
          OR class = 'bb_request_to_groom')

) WITHOUT OIDS;
grant_priv(`acts')


-- rnktypes

CREATE TABLE rnktypes (
  rnktype VARCHAR(6) PRIMARY KEY NOT NULL
          empty_string_check(`rnktype'),
  descr TEXT NOT NULL,
  query TEXT NOT NULL
) WITHOUT OIDS;
grant_priv(`rnktypes')


-- foodcodes
CREATE TABLE foodcodes (
  fcode VARCHAR(3) PRIMARY KEY NOT NULL
        empty_string_check(`fcode'),
  ftype CHAR(1) NOT NULL
        CONSTRAINT "Ftype on FOODTYPES" REFERENCES foodtypes,
  descr TEXT NOT NULL
        empty_string_check(`descr')
) WITHOUT OIDS;
grant_priv(`foodcodes')


-- activities
CREATE TABLE activities(
  activity CHAR(1) PRIMARY KEY NOT NULL
           empty_string_check(`activity'),
  juvenile BOOLEAN NOT NULL,
  female BOOLEAN NOT NULL,
  descr TEXT NOT NULL
        empty_string_check(`descr')
) WITHOUT OIDS;
grant_priv(`activities')


-- ncodes
CREATE TABLE ncodes(
  ncode CHAR(1) PRIMARY KEY NOT NULL
        empty_string_check(`ncode'),
  juvenile BOOLEAN NOT NULL,
  female BOOLEAN NOT NULL,
  requires CHAR(1) NULL
           CONSTRAINT "Requires on NCODES" REFERENCES ncodes,
  nsex CHAR(1) NOT NULL
       CONSTRAINT "Nsex one of bb_nc_any, bb_nc_male, bb_nc_opposite"
         CHECK (nsex = 'bb_nc_any'
                OR nsex = 'bb_nc_male'
                OR nsex = 'bb_nc_opposite'),
  nunique BOOLEAN NOT NULL,
  descr TEXT NOT NULL
        empty_string_check(`descr'),

  CONSTRAINT "Requires can't be NULL when Nsex is bb_nc_opposite"
             CHECK (NOT(nsex = 'bb_nc_opposite' AND requires IS NULL))
) WITHOUT OIDS;
grant_priv(`ncodes')


-- unksnames
CREATE TABLE unksnames (
  unksname CHAR(3) PRIMARY KEY NOT NULL
           empty_string_check(`unksname'),
  lonemale BOOLEAN NOT NULL,
  descr TEXT NOT NULL UNIQUE
        empty_string_check(`descr')
) WITHOUT OIDS;
grant_priv(`unksnames')


-- postures
CREATE TABLE postures (
  posture CHAR(1) PRIMARY KEY NOT NULL
          empty_string_check(`posture'),
  juvonly BOOLEAN NOT NULL,
  descr TEXT NOT NULL
        empty_string_check(`descr')
) WITHOUT OIDS;
grant_priv(`postures')


-- setupids
CREATE TABLE setupids (
  setupid SMALLINT PRIMARY KEY NOT NULL,
  sid_string VARCHAR(32) NOT NULL
             empty_string_check(`sid_string'),
  data_structure SMALLINT NOT NULL
                 CONSTRAINT "Data_Structure on DATA_STRUCTURES"
                            REFERENCES data_structures,
  descr TEXT NOT NULL
        empty_string_check(`descr')
) WITHOUT OIDS;
grant_priv(`setupids')


-- programids
CREATE TABLE programids (
  programid SMALLINT PRIMARY KEY NOT NULL,
  pid_string VARCHAR(32) NOT NULL
             empty_string_check(`pid_string'),
  descr TEXT NOT NULL
        empty_string_check(`descr')
) WITHOUT OIDS;
grant_priv(`programids')


-- dad_software
CREATE TABLE dad_software (
  software VARCHAR(bb_dad_software_id_len) PRIMARY KEY,
  version TEXT NULL
        empty_string_check(version),
  descr TEXT NOT NULL UNIQUE
        empty_string_check(descr)
) WITHOUT OIDS;
grant_priv(`dad_software')


-- Bodyparts
CREATE TABLE bodyparts (
  bpid VARCHAR(bb_bodyparts_bpid_len) PRIMARY KEY
       empty_string_check(`bpid'),
  descr TEXT NOT NULL
        empty_string_check(`Descr'),
  bodyside CHAR(1) NULL
           CONSTRAINT "BODYPARTS.Bodyside must be bb_leftside, bb_rightside, or NULL"
             CHECK (bodyside IS NULL
                    OR bodyside = 'bb_leftside'
                    OR bodyside = 'bb_rightside'),
  innerouter CHAR(1) NULL
             CONSTRAINT "BODYPARTS.Innerouter must be bb_inner, bb_outer, or NULL"
               CHECK (innerouter IS NULL
                      OR innerouter = 'bb_inner'
                      OR innerouter = 'bb_outer'),
  bodyregion VARCHAR(bb_bodyparts_bpid_len) NOT NULL
             CONSTRAINT "Bodyregion on BODYPARTS" REFERENCES bodyparts
) WITHOUT OIDS;
grant_priv(`bodyparts')


-- Toothcodes
CREATE TABLE toothcodes (
  tooth VARCHAR(5) PRIMARY KEY
        empty_string_check(`tooth'),
  descr TEXT NOT NULL UNIQUE
        empty_string_check(descr),
  canine BOOLEAN NOT NULL,
  deciduous BOOLEAN NOT NULL,
  toothsite SMALLINT NOT NULL
            CONSTRAINT "TOOTHCODES.Toothsite on TOOTHSITES" 
                       REFERENCES toothsites
) WITHOUT OIDS;
grant_priv(`toothcodes')


-- adcodes
CREATE TABLE adcodes (
  adcode VARCHAR(2) PRIMARY KEY
    empty_string_check(`adcode'),
  adn CHAR(1) NOT NULL
    CONSTRAINT "ADN must be one of: bb_swerb_descent, bb_swerb_ascent, bb_swerb_notad"
                CHECK (adn = 'bb_swerb_descent'
                       OR adn = 'bb_swerb_ascent'
                       OR adn = 'bb_swerb_notad'),
  time BOOLEAN NOT NULL,
  descr TEXT NOT NULL UNIQUE
        empty_string_check(`descr')
) WITHOUT OIDS;

grant_priv(`adcodes')


-- aerials
CREATE TABLE aerials (
  aerial SMALLINT PRIMARY KEY
      CONSTRAINT "Aerial must be >= 1"
                 CHECK (aerial >= 1),
  date DATE NOT NULL
) WITHOUT OIDS;

grant_priv(`aerials')


-- quad_data
CREATE TABLE quad_data (
  quad VARCHAR(4) PRIMARY KEY
       empty_string_check(`quad'),
  xyloc GEOMETRY(POINT, bb_amboseli_srid) NULL
        CONSTRAINT "X coord of XYLoc must have only 1 digit right of the decimal"
          CHECK(ST_X(xyloc) * 10 = TRUNC(ST_X(xyloc) * 10))
        CONSTRAINT "Y coord of XYLoc must have only 1 digit right of the decimal"
          CHECK(ST_Y(xyloc) * 10 = TRUNC(ST_Y(xyloc) * 10))
        CONSTRAINT "XYLoc within study area (see Babase Technical Specifications)"
          CHECK(ST_Intersects(xyloc
                            , ST_PolygonFromText('bb_hard_study_area'
                                               , bb_amboseli_srid))),
  aerial SMALLINT NULL
         CONSTRAINT "Aerial on AERIALS" REFERENCES aerials
) WITHOUT OIDS;

grant_priv(`quad_data')


-- gps_units
CREATE TABLE gps_units (
  gps NUMERIC(2) PRIMARY KEY
         CONSTRAINT "GPS cannot be negative"
           CHECK(gps = ABS(gps)),
  descr TEXT NOT NULL
        empty_string_check(`descr'),
  make VARCHAR(100) NOT NULL
        empty_string_check(`make'),
  model VARCHAR(100) NOT NULL
        empty_string_check(`model'),
  errortype VARCHAR(8) NOT NULL
            CONSTRAINT "Errortype must be either 'bb_gpsu_pdop' or 'bb_gpsu_accuracy'"
            CHECK(errortype = 'bb_gpsu_pdop'
                  OR errortype = 'bb_gpsu_accuracy'),
  label CHAR(1) NOT NULL
        empty_string_check(`label'),
  start DATE NOT NULL
        CONSTRAINT "Start cannot be before bb_gps_start"
          CHECK(start >= 'bb_gps_start'),
  finish DATE NULL
         CONSTRAINT "Finish cannot be before Start"
           CHECK(finish >= start)
) WITHOUT OIDS;

grant_priv(`gps_units')



-- groups

CREATE TABLE groups (
  gid        b_grp_type PRIMARY KEY,
  name       VARCHAR(32)
             empty_string_check(`name'),
  from_group b_grp_type
             CONSTRAINT "From_group on GROUPS" REFERENCES groups,
  permanent  DATE NULL
             CONSTRAINT "Permanent cannot be before Start"
               CHECK(permanent IS NULL
                     OR start IS NULL
                     OR start <= permanent),
  supergroup b_grp_type 
             CONSTRAINT "Supergroup on GROUPS" REFERENCES groups,
  start      DATE,
  cease_to_exist     DATE
             CONSTRAINT "Cease_To_Exist must be after Start"
               CHECK(cease_to_exist IS NULL
                     OR start IS NULL
                     OR start < cease_to_exist)
             CONSTRAINT "Cease_To_Exist must be after or same day as Last_Reg_Census"
               CHECK(cease_to_exist IS NULL
                     OR last_reg_census IS NULL
                     OR last_reg_census <= cease_to_exist),
  last_reg_census DATE 
             CONSTRAINT "Last_Reg_Census must be after Start"
               CHECK(last_reg_census IS NULL
                     OR start IS NULL
                     OR start < last_reg_census),
  three_letter_code CHAR(3) NULL
             CONSTRAINT "Three_letter_code must be uppercase"
               CHECK(three_letter_code = UPPER(three_letter_code)),
  one_letter_code CHAR(1) NULL
             CONSTRAINT "One_letter_code must be uppercase"
               CHECK(one_letter_code = UPPER(one_letter_code)),
  study_grp BOOLEAN NOT NULL
  CONSTRAINT "The From_group may not be the same as the Gid"
    CHECK(from_group <> gid),
  CONSTRAINT "Non-Permanent groups must have a non-NULL From_group"
    CHECK(permanent IS NOT NULL OR from_group IS NOT NULL)
  ) WITHOUT OIDS;
grant_priv(`groups')

-- biograph

CREATE TABLE biograph (
  bioid      SERIAL PRIMARY KEY,
  sname      CHAR(3) UNIQUE,
             CONSTRAINT "Sname must be uppercase" 
               CHECK(sname = UPPER(sname)),
             CONSTRAINT "Sname must be 3 characters" 
               CHECK(sname IS NULL OR char_length(sname) = 3),
             CONSTRAINT "Sname must not be bb_unidentified_sname"
               CHECK(sname IS NULL OR sname <> 'bb_unidentified_sname'),

  name       VARCHAR(32),
             CONSTRAINT "Name required when Sname not NULL (not abortion)"
             CHECK(sname IS NULL OR name IS NOT NULL),
             empty_string_check(`name'),

  pid        VARCHAR(6)
             empty_string_check(`pid'),
             -- References PREGS is added below after pregs is created.

  birth      DATE NOT NULL,
             CONSTRAINT "Birth <= Statdate" CHECK(birth <= statdate),

  bstatus    NUMERIC(2,1) NOT NULL
             CONSTRAINT "Bstatus on BSTATUSES" REFERENCES bstatuses,

  sex        CHAR(1) NOT NULL
             CONSTRAINT "Sex one of bb_male, bb_female, or bb_unk_sex"
                        CHECK(sex = 'bb_male' OR sex = 'bb_female' OR sex = 'bb_unk_sex'),

  matgrp     b_grp_type NOT NULL
              CONSTRAINT "Matgrp on GROUPS" REFERENCES groups,
  matgrpconfidence
             SMALLINT NOT NULL
             CONSTRAINT "Matgrpconfidence on CONFIDENCES"
                         REFERENCES confidences,

  statdate   DATE NOT NULL,
             CONSTRAINT "Statdate = Birth when Sname is NULL (abortion)"
             CHECK((sname IS NULL AND statdate = birth)
                   OR sname IS NOT NULL),

  status     SMALLINT NOT NULL
             CONSTRAINT "Status on STATUSES" REFERENCES statuses,
  CONSTRAINT "Status <> bb_alive (alive) when Sname is NULL (abortion)"
    CHECK((sname IS NULL AND status <> bb_alive) OR sname IS NOT NULL),
  CONSTRAINT "Dcause for live animal, or no dcause for dead animal"
    CHECK((status = bb_alive AND dcause = bb_no_dcause)
          OR (status <> bb_alive AND dcause <> bb_no_dcause)) ,

  dcause     SMALLINT NOT NULL
             CONSTRAINT "Dcause on DCAUSES" REFERENCES dcauses,
  alt_snames BOOLEAN NOT NULL,
  entrydate  DATE
             CONSTRAINT "Entrydate and Sname must both be NULL or both non-NULL"
               CHECK((entrydate IS NULL AND sname IS NULL)
                     OR (entrydate IS NOT NULL AND sname IS NOT NULL))
             CONSTRAINT "Entrydate <= Statdate" CHECK(entrydate <= statdate),
  entrytype  CHAR(1)
             CONSTRAINT "Entrytype and Sname must both be NULL or both non-NULL"
               CHECK((entrytype IS NULL AND sname IS NULL)
                     OR (entrytype IS NOT NULL AND sname IS NOT NULL))
             CONSTRAINT "Entrytype on ENTRYTYPES" REFERENCES entrytypes,
  dcausenatureconfidence    SMALLINT NOT NULL
             CONSTRAINT "DcauseNatureConfidence on CONFIDENCES" REFERENCES confidences,
             CONSTRAINT "BIOGRAPH.DcauseNatureConfidence must be bb_no_dcauseconfidence when Dcause is bb_no_dcause"
               CHECK(NOT(dcause = bb_no_dcause AND dcausenatureconfidence <> bb_no_dcauseconfidence)),
  dcauseagentconfidence    SMALLINT NOT NULL
             CONSTRAINT "DcauseAgentConfidence on CONFIDENCES" REFERENCES confidences,
             CONSTRAINT "BIOGRAPH.DcauseAgentConfidence must be bb_no_dcauseconfidence when Dcause is bb_no_dcause"
               CHECK(NOT(dcause = bb_no_dcause AND dcauseagentconfidence <> bb_no_dcauseconfidence)),
  CONSTRAINT "Birth = Entrydate when Entrytype = 'bb_entrytype_birth' (Birth)"
    CHECK((birth = entrydate AND entrytype = 'bb_entrytype_birth')
           OR entrytype <> 'bb_entrytype_birth'),
  CONSTRAINT "Birth <> Entrydate when Entrytype <> 'bb_entrytype_birth' (Birth)"
    CHECK((birth <> entrydate AND entrytype <> 'bb_entrytype_birth')
           OR entrytype = 'bb_entrytype_birth')
  ) WITHOUT OIDS;
grant_priv(`biograph')
grant_seq_priv(`biograph', `bioid')

-- alternate_snames

create table alternate_snames (
  sname_column,
  sname_alternate CHAR(3) NOT NULL
             CONSTRAINT "Sname_Alternate must be uppercase" 
               CHECK(sname_alternate = UPPER(sname_alternate))
             CONSTRAINT "Sname_Alternate must be 3 characters" 
               CHECK(char_length(sname_alternate) = 3)
             CONSTRAINT "Sname and Sname_Alternate must differ"
               CHECK (sname <> sname_alternate)
             CONSTRAINT "Sname_Alternate must not be bb_unidentified_sname"
               CHECK(sname_alternate <> 'bb_unidentified_sname'),
  name_alternate       VARCHAR(32),
             empty_string_check(`name_alternate'),
  notes TEXT NOT NULL
        empty_string_check(`Notes')
);

grant_priv(`alternate_snames')

-- maturedates

CREATE TABLE maturedates (
  sname_column PRIMARY KEY,

  matured    DATE NOT NULL,

  mstatus    CHAR(1) NOT NULL
             CONSTRAINT "Mstatus on MSTATUSES" REFERENCES mstatuses
  ) WITHOUT OIDS;
grant_priv(`maturedates')

-- rankdates

CREATE TABLE rankdates (
  maturedates_sname_column PRIMARY KEY,

  ranked    DATE NOT NULL
            CONSTRAINT "RANKDATES.Ranked must fall on the first of the month"
                       CHECK (ranked = date_trunc('month', ranked)),

  rstatus    CHAR(1) NOT NULL
             CONSTRAINT "Rstatus on MSTATUSES" REFERENCES mstatuses
  ) WITHOUT OIDS;
grant_priv(`rankdates')


-- consortdates

CREATE TABLE consortdates (
  maturedates_sname_column PRIMARY KEY,

  consorted    DATE NOT NULL
  ) WITHOUT OIDS;
grant_priv(`consortdates')

-- dispersedates

CREATE TABLE dispersedates (
  sname_column PRIMARY KEY,

  dispersed    DATE NOT NULL,
  dispconfidence    SMALLINT NOT NULL
               CONSTRAINT "Dispconfidence on CONFIDENCES" REFERENCES confidences
  ) WITHOUT OIDS;
grant_priv(`dispersedates')


-- pregs

CREATE TABLE pregs (
  pid         VARCHAR(6) PRIMARY KEY,
  parity      SMALLINT NULL
              CONSTRAINT "Parity must be between 1 and 199, inclusive"
                CHECK (parity >= 1 AND parity <= 199),
              -- References CYCLES added below after cycles created.
  conceive    INT NOT NULL,
              -- References CYCLES added below after cycles created.
  resume      INT
  ) WITHOUT OIDS;
grant_priv(`pregs')
-- Change biograph to know about pregs.
ALTER TABLE biograph
  ADD CONSTRAINT "Pid on PREGS" FOREIGN KEY (pid) REFERENCES pregs;

-- cycgaps

CREATE TABLE cycgaps (
  gapid         SERIAL PRIMARY KEY,
  sname_column,
  code          CHAR(1) NOT NULL
                CONSTRAINT "Code one of bb_cg_start, bb_cg_end, or bb_cg_point"
                  CHECK (code = 'bb_cg_start'
                         OR code = 'bb_cg_end'
                         OR code = 'bb_cg_point'),
  date          DATE NOT NULL,
  state         CHAR(1) NULL
                CONSTRAINT "State one of bb_cg_menses, bb_cg_swelling, bb_cg_ovulating, bb_cg_deturgesence, bb_cg_pregnant, or bb_cg_lactating"
                CHECK (state = 'bb_cg_menses'
                       OR state = 'bb_cg_swelling'
                       OR state = 'bb_cg_ovulating'
                       OR state = 'bb_cg_deturgesence'
                       OR state = 'bb_cg_pregnant'
                       OR state = 'bb_cg_lactating'),
  CONSTRAINT "State must be NULL when Code is bb_cg_end, and not NULL otherwise"
    CHECK ((code = 'bb_cg_end' and state IS NULL)
           OR state IS NOT NULL)
  ) WITHOUT OIDS;
grant_priv(`cycgaps')
grant_seq_priv(`cycgaps', `gapid')


-- cycgapdays
CREATE TABLE cycgapdays (
  cgdid SERIAL PRIMARY KEY,
  sname_column,
  date  DATE NOT NULL
) WITHOUT OIDS;
grant_ro_priv(`cycgapdays')


-- cycles

CREATE TABLE cycles (
  cid           SERIAL PRIMARY KEY,
  sname_column,
  seq           INT NULL,     -- triggers never allow NULL
  series        INT NULL      -- triggers never allow NULL
  ) WITHOUT OIDS;
grant_priv(`cycles')
grant_seq_priv(`cycles', `cid')

-- cycpoints

CREATE TABLE cycpoints (
  cpid          SERIAL PRIMARY KEY,
  cid           INT NOT NULL
                CONSTRAINT "Cid on CYCLES" REFERENCES cycles,
  date          DATE NOT NULL,
  edate         DATE,
  ldate         DATE,
  source        CHAR(1) NOT NULL DEFAULT 'bb_data'
                CONSTRAINT "Source one of bb_data, bb_auto, or bb_estimated"
                  CHECK (source = 'bb_data'
                         OR source = 'bb_auto'
                         OR source = 'bb_estimated'),
  code          CHAR(1) NOT NULL
                CONSTRAINT "Code one of bb_menses, bb_turgesence, or bb_deturgesence"
                  CHECK (code = 'bb_menses'
                         OR code = 'bb_turgesence'
                         OR code = 'bb_deturgesence'),

  CONSTRAINT "Only Mdates may have Source = bb_auto"
    CHECK (code = 'bb_menses' OR source <> 'bb_auto'),
  CONSTRAINT "Edate <= date" CHECK (edate <= date),
  CONSTRAINT "date <= Ldate" CHECK (date <= ldate)
  ) WITHOUT OIDS;
grant_priv(`cycpoints')
grant_seq_priv(`cycpoints', `cpid')

-- Change pregs to know about cycpoints.
ALTER TABLE pregs
  ADD CONSTRAINT "Conceive on CYCPOINTS"
                 FOREIGN KEY (conceive) REFERENCES cycpoints;
ALTER TABLE pregs
  ADD CONSTRAINT "Resume on CYCPOINTS"
                 FOREIGN KEY (resume) REFERENCES cycpoints;


-- sexskins

CREATE TABLE sexskins (
  sxid   SERIAL PRIMARY KEY,
  date   DATE NOT NULL,
  size   NUMERIC(3,1) NOT NULL
         CONSTRAINT "Size must be between bb_min_skin and bb_max_skin inclusive" 
           CHECK(size >= bb_min_skin AND size <= bb_max_skin)
         CONSTRAINT "Size must be integral execpt for bb_frac_skin"
           CHECK(size = TRUNC(size) OR size = bb_frac_skin),
  cid    INT NOT NULL CONSTRAINT "Cid on CYCLES" REFERENCES cycles
  ) WITHOUT OIDS;
grant_priv(`sexskins')
grant_seq_priv(`sexskins', `sxid')


-- pcskins

CREATE TABLE pcskins (
  pcsid  SERIAL PRIMARY KEY,
  sname_column,
  date   DATE NOT NULL,
  color  CHAR(1) NOT NULL
         CONSTRAINT "Color on PCSCOLORS" REFERENCES pcscolors
  ) WITHOUT OIDS;
grant_priv(`pcskins')
grant_seq_priv(`pcskins', `pcsid')


-- repstats

CREATE TABLE repstats (
  rid           SERIAL PRIMARY KEY,
  date          DATE NOT NULL,
  sname_column,
  state         CHAR(1) NOT NULL
                CONSTRAINT "State one of bb_pregnant, bb_cycling, or bb_lactating"
                CHECK(state = 'bb_pregnant' OR state = 'bb_cycling' or state = 'bb_lactating'),
  dins          INT NULL
                CONSTRAINT "Dins >= 1"
                CHECK(dins > 0 OR dins IS NULL),
  dr            INT NULL
                CONSTRAINT "Dr >= 0"
                CHECK(dr >= 0 OR dr IS NULL),
  pid           VARCHAR(6)
                CONSTRAINT "Pid on PREGS" REFERENCES pregs
  ) WITHOUT OIDS;
dnl Updated by trigger so has read-only rights.
grant_ro_priv(`repstats')
grant_ro_seq_priv(`repstats', `rid')

-- cycstats

CREATE TABLE cycstats (
  csid          SERIAL PRIMARY KEY,
  date          DATE NOT NULL,
  sname_column,
  state         CHAR(1) NOT NULL
                CONSTRAINT "State one of bb_menses, bb_swelling, bb_ovulating, or bb_deturgesence"
                CHECK(state = 'bb_menses' OR state = 'bb_swelling'  OR state = 'bb_ovulating' OR state = 'bb_deturgesence'),
  dins          INT NULL
                CONSTRAINT "Dins >= 1"
                CHECK(dins > 0 OR dins IS NULL),
  dr            INT NULL
                CONSTRAINT "Dr >= 0"
                CHECK(dr >= 0 OR dr IS NULL),
  cpids         INT NULL
                CONSTRAINT "Cpids on CYCPOINTS" REFERENCES cycpoints,
  cpide         INT NULL
                CONSTRAINT "Cpide on CYCPOINTS" REFERENCES cycpoints
  ) WITHOUT OIDS;
dnl Updated by trigger so has read-only rights.
grant_ro_priv(`cycstats')
grant_ro_seq_priv(`cycstats', `csid')

-- mmintervals

CREATE TABLE mmintervals (
  mmiid         SERIAL PRIMARY KEY,
  date          DATE NOT NULL,
  sname_column,
  dini          INT NULL
                CONSTRAINT "Dini >= 1"
                CHECK(dini > 0 OR dini IS NULL),
  dr            INT NULL
                CONSTRAINT "Dr >= 0"
                CHECK(dr >= 0 OR dr IS NULL),
  cpids         INT NULL
                CONSTRAINT "Cpids on CYCPOINTS" REFERENCES cycpoints,
  cpide         INT NULL
                CONSTRAINT "Cpide on CYCPOINTS" REFERENCES cycpoints
  ) WITHOUT OIDS;
dnl Updated by trigger so has read-only rights.
grant_ro_priv(`mmintervals')
grant_ro_seq_priv(`mmintervals', `mmiid')


-- mdintervals

CREATE TABLE mdintervals (
  mdiid         SERIAL PRIMARY KEY,
  date          DATE NOT NULL,
  sname_column,
  dini          INT NULL
                CONSTRAINT "Dini >= 1"
                CHECK(dini > 0 OR dini IS NULL),
  dr            INT NULL
                CONSTRAINT "Dr >= 0"
                CHECK(dr >= 0 OR dr IS NULL),
  cpids         INT NULL
                CONSTRAINT "Cpids on CYCPOINTS" REFERENCES cycpoints,
  cpide         INT NULL
                CONSTRAINT "Cpide on CYCPOINTS" REFERENCES cycpoints
  ) WITHOUT OIDS;
dnl Updated by trigger so has read-only rights.
grant_ro_priv(`mdintervals')
grant_ro_seq_priv(`mdintervals', `mdiid')

-- members

CREATE TABLE members (
  membid SERIAL PRIMARY KEY,
  sname_column,
  date   DATE NOT NULL,
  grp    b_grp_type NOT NULL CONSTRAINT "Grp on GROUPS" REFERENCES groups,
  origin CHAR(1) NOT NULL
         CONSTRAINT "Origin one of bb_census, bb_interpolated, bb_demog, bb_manual, bb_manual_no_interp, bb_S, bb_E, bb_F, bb_B, bb_G, bb_T, bb_L, or bb_R"
         CHECK(origin = 'bb_census'
               OR origin = 'bb_interpolated'
               OR origin = 'bb_demog'
               OR origin = 'bb_manual'
               OR origin = 'bb_manual_no_interp'
               OR origin = 'bb_S'
               OR origin = 'bb_E'
               OR origin = 'bb_F'
               OR origin = 'bb_B'
               OR origin = 'bb_G'
               OR origin = 'bb_T'
               OR origin = 'bb_L'
               OR origin = 'bb_R'),
  interp INT NULL
         CONSTRAINT "Interp must be non-negative or NULL"
         CHECK(interp >= 0 OR interp IS NULL)
  ) WITHOUT OIDS;
dnl Updated by trigger so has read-only rights.
grant_ro_priv(`members')

CREATE TABLE census (
  cenid  SERIAL PRIMARY KEY,
  date   b_cendate_type NOT NULL
         CONSTRAINT "Date may not be more than one year in the future"
         CHECK(date <= LOCALTIMESTAMP + CAST('1 year' AS INTERVAL)),
  sname_column,
  grp    b_grp_type NOT NULL CONSTRAINT "Grp on GROUPS" REFERENCES groups,
  status b_censtatus_type NOT NULL
         CONSTRAINT "Status one of bb_census, bb_demog, bb_absent bb_manual, bb_manual_no_interp, bb_S, bb_E, bb_F, bb_B, bb_G, bb_T, bb_L, or bb_R"
         CHECK(status = 'bb_census'
               OR status = 'bb_demog'
               OR status = 'bb_absent'
               OR status = 'bb_manual'
               OR status = 'bb_manual_no_interp'
               OR status = 'bb_S'
               OR status = 'bb_E'
               OR status = 'bb_F'
               OR status = 'bb_B'
               OR status = 'bb_G'
               OR status = 'bb_T'
               OR status = 'bb_L'
               OR status = 'bb_R'),
  cen    BOOLEAN NOT NULL
         CONSTRAINT "Cen must be TRUE when Status bb_census or bb_absent, FALSE otherwise unless bb_demog"
         CHECK (cen = TRUE AND (status = 'bb_census'
                                OR status = 'bb_demog'
                                OR status = 'bb_absent')
                OR (cen = FALSE AND NOT (status = 'bb_census'
                                         OR status = 'bb_absent')))
  ) WITHOUT OIDS;
grant_priv(`census')
grant_seq_priv(`census', `cenid')

CREATE TABLE demog (
  cenid     INT NOT NULL
            CONSTRAINT "Cenid on CENSUS" PRIMARY KEY REFERENCES census,
  reference NUMERIC(5,3) NOT NULL 
            CONSTRAINT "Reference on DEMOG_REFERENCES" REFERENCES demog_references,
  comment   TEXT NULL
            empty_string_check(`comment')
  ) WITHOUT OIDS;
grant_priv(`demog')

CREATE TABLE ranks (
  rnkid   SERIAL PRIMARY KEY,
  sname_column,
  rnkdate DATE NOT NULL
          CONSTRAINT "Rnkdate between bb_min_rnkdate_year and bb_max_rnkdate_year (inclusive)"
            CHECK (CAST('bb_min_rnkdate_year-01-01' AS DATE) <= rnkdate
                   AND rnkdate <= CAST('bb_max_rnkdate_year-12-31' AS DATE)),
          CONSTRAINT "Rnkdate must be the first of the month"
            CHECK (rnkdate = date_trunc('month', rnkdate)),
  grp     b_grp_type NOT NULL
          CONSTRAINT "Grp on GROUPS" REFERENCES groups,
  rnktype VARCHAR(6) NOT NULL
          CONSTRAINT "Rnktype on RNKTYPES" REFERENCES rnktypes,
  rank    INT NOT NULL
          CONSTRAINT "Rank must be >= 1" CHECK (rank >= 1)
  ) WITHOUT OIDS;
grant_priv(`ranks')
grant_seq_priv(`ranks', `rnkid')


CREATE TABLE samples (
  sid      SERIAL PRIMARY KEY,
  date     DATE NOT NULL,
  -- If stime is ever required then make date and stime a single column.
  stime    TIME(0) NOT NULL
           CONSTRAINT "Stime required after Jan 1, 07"
           CHECK (NOT(stime IS NULL AND date >= '2007-01-01')),
  observer VARCHAR(4) NOT NULL
           CONSTRAINT "Observer on OBSERVERS" REFERENCES observers,
  stype    CHAR(1) NOT NULL
           CONSTRAINT "Stype one of bb_juvenile_point, bb_female_point"
           CHECK(stype = 'bb_juvenile_point'
                 OR stype = 'bb_female_point'),
  grp      b_grp_type NOT NULL 
           CONSTRAINT "Grp on GROUPS" REFERENCES groups,
  sname_column,
  -- MINutes in Sample
  mins     INT NOT NULL
           CONSTRAINT "Mins must be >= 0 and <= bb_max_mins"
           CHECK (mins >= 0 AND mins <= bb_max_mins),
  -- MINutes In Sight
  minsis   INT DEFAULT 0
           NOT NULL
           CONSTRAINT "Minsis must be >= 0 and <= Mins"
             CHECK (minsis >= 0 AND minsis <= mins),
  programid SMALLINT NULL
            CONSTRAINT "Programid on PROGRAMIDS" REFERENCES programids,
  setupid   SMALLINT NULL
            CONSTRAINT "Setupid on SETUPIDS" REFERENCES setupids,
  palmtop  SMALLINT NOT NULL
            CONSTRAINT "Palmtop on PALMTOPS" REFERENCES palmtops
  ) WITHOUT OIDS;
grant_priv(`samples')
grant_seq_priv(`samples', `sid')

CREATE TABLE interact_data(
  iid        SERIAL PRIMARY KEY,
  sid        INT 
             CONSTRAINT "Sid on SAMPLES" REFERENCES samples,
  act        CHAR(2) NOT NULL 
             CONSTRAINT "Act on ACTS" REFERENCES acts,
             CONSTRAINT "Approach only when sid exists, during all-occurences sampling"
               CHECK ((act = 'bb_approach' AND sid IS NOT NULL)
                      OR act <> 'bb_approach'),
             CONSTRAINT "Request to groom only when sid exists, during all-occurences sampling"
               CHECK ((act = 'bb_request_to_groom' AND sid IS NOT NULL)
                      OR act <> 'bb_request_to_groom'),
             CONSTRAINT "Mount not allowed when sid exists, during all-occurences sampling"
               CHECK ((act = 'bb_mounts' AND sid IS NULL)
                      OR act <> 'bb_mounts'),
             CONSTRAINT "Consortship not allowed when sid exists, during all-occurences sampling"
               CHECK ((act = 'bb_consortship' AND sid IS NULL)
                      OR act <> 'bb_consortship'),
             CONSTRAINT "Ejaculation not allowed when sid exists, during all-occurences sampling"
               CHECK ((act = 'bb_ejaculation' AND sid IS NULL)
                      OR act <> 'bb_ejaculation'),
  date       DATE NOT NULL,
  start      TIME(0)
             CONSTRAINT "Start >= bb_day_start"
               CHECK (start >= 'bb_day_start'::time)
             CONSTRAINT "Start <= bb_day_end"
               CHECK (start <= 'bb_day_end'::time)
             CONSTRAINT "The seconds in the Start value must be 0 when Sid is NULL"
               CHECK (sid IS NOT NULL OR EXTRACT('seconds' FROM start) = 0),
  stop       TIME(0)
             CONSTRAINT "Stop >= bb_day_start"
               CHECK (stop >= 'bb_day_start'::time)
             CONSTRAINT "Stop <= bb_day_end"
               CHECK (stop <= 'bb_day_end'::time)
             CONSTRAINT "The seconds in the Stop value must be 0 when Sid is NULL"
               CHECK (sid IS NOT NULL OR EXTRACT('seconds' FROM stop) = 0),
  observer   VARCHAR(4) NULL
                CONSTRAINT "Observer on OBSERVERS" REFERENCES observers,
  handwritten BOOLEAN NOT NULL
                CONSTRAINT "Interactions during a focal cannot be handwritten"
                  CHECK ((sid IS NULL)
                         OR (NOT handwritten))
  ) WITHOUT OIDS;
grant_priv(`interact_data')
grant_seq_priv(`interact_data', `iid')

CREATE TABLE parts (
  partid SERIAL PRIMARY KEY,
  sname_column,
  role  CHAR(1) NOT NULL
        CONSTRAINT "Role on PARTS is one of bb_actor or bb_actee"
        CHECK (role = 'bb_actor' OR role = 'bb_actee'),
  iid   INT NOT NULL
        CONSTRAINT "Iid on INTERACT_DATA" REFERENCES interact_data
  ) WITHOUT OIDS;
grant_priv(`parts')
grant_seq_priv(`parts', `partid')

CREATE TABLE point_data (
  pntid    SERIAL PRIMARY KEY,
  sid      INT NOT NULL
           CONSTRAINT "Sid on SAMPLES" REFERENCES samples,
  min      SMALLINT NOT NULL  -- Triggers don't allow NULL
           CONSTRAINT "Min > 0"
           CHECK (min IS NULL OR min > 0),
  activity CHAR(1) NOT NULL,
  posture  CHAR(1) NOT NULL
           CONSTRAINT "Posture on POSTURES" REFERENCES postures,
  foodcode VARCHAR(3) 
           CONSTRAINT "Foodcode on FOODCODES" REFERENCES foodcodes,
           CONSTRAINT "Foodcode present if and only if Activity = bb_feed_activity"
           CHECK (foodcode IS NULL
                  OR (foodcode IS NOT NULL
                      AND activity = 'bb_feed_activity')),
  ptime    TIME(0) NOT NULL
           CONSTRAINT "Ptime >= bb_day_start"
             CHECK (ptime >= 'bb_day_start'::time)
           CONSTRAINT "Ptime <= bb_day_early_end"
             CHECK (ptime <= 'bb_day_early_end'::time)
  ) WITHOUT OIDS;
grant_priv(`point_data')
grant_seq_priv(`point_data', `pntid')

-- FPOINTS could inherit from POINT_DATA, but the pg syntax
-- is not standard and we can always make a view so I'm
-- going to hold off on this.
CREATE TABLE fpoints (
  pntid      INT PRIMARY KEY CONSTRAINT "Pntid on POINT_DATA" REFERENCES point_data,
  kidcontact CHAR(1) NOT NULL
             CONSTRAINT "Kidcontact one of bb_kidcon_away, bb_kidcon_ventral, bb_kidcon_dorsal, bb_kidcon_other, bb_kidcon_none"
             CHECK(kidcontact = 'bb_kidcon_away'
                   OR kidcontact = 'bb_kidcon_ventral'
                   OR kidcontact = 'bb_kidcon_dorsal'
                   OR kidcontact = 'bb_kidcon_other'
                   OR kidcontact = 'bb_kidcon_none'),
  kidsuckle  CHAR(1) NOT NULL
             CONSTRAINT "Kidsuckle one of bb_kidsuck_suckling, bb_kidsuck_notsuckling, bb_kidsuck_unknown"
             CHECK(kidsuckle = 'bb_kidsuck_suckling'
                   OR kidsuckle = 'bb_kidsuck_notsuckling'
                   OR kidsuckle = 'bb_kidsuck_unknown')
  ) WITHOUT OIDS;
grant_priv(`fpoints')

CREATE TABLE neighbors (
  nghid        SERIAL PRIMARY KEY,
  pntid        INT NOT NULL
               CONSTRAINT "Pntid on POINT_DATA" REFERENCES point_data,
  sname        CHAR(3)
               CONSTRAINT "Sname on BIOGRAPH" REFERENCES biograph(sname),
  ncode        CHAR(1) NOT NULL
               CONSTRAINT "Ncode on NCODES" REFERENCES ncodes,
  unksname     CHAR(3)
               CONSTRAINT "Unksname on UNKSNAMES" REFERENCES unksnames,
  CONSTRAINT "Either an Sname or a Unksname but not both is required"
               CHECK ((sname IS NULL AND unksname IS NOT NULL)
                      OR (sname IS NOT NULL AND unksname IS NULL))
  ) WITHOUT OIDS;
grant_priv(`neighbors')
grant_seq_priv(`neighbors', `nghid')

CREATE TABLE allmiscs (
  almid         SERIAL PRIMARY KEY,
  sid           INT NOT NULL CONSTRAINT "Sid on SAMPLES" REFERENCES samples,
  atime         TIME NOT NULL
                CONSTRAINT "Atime >= bb_day_start"
                  CHECK (atime >= 'bb_day_start'::time)
                CONSTRAINT "Atime <= bb_day_early_end;"
                  CHECK (atime <= 'bb_day_early_end'::time),
  txt           TEXT NOT NULL
                empty_string_check(`txt')
  ) WITHOUT OIDS;
grant_priv(`allmiscs')
grant_seq_priv(`allmiscs', `almid')

CREATE TABLE weatherhawk (
  wid          SERIAL PRIMARY KEY,
  wstation      VARCHAR(3) NOT NULL
                CONSTRAINT "WStation on WSTATIONS" REFERENCES wstations,
  timestamp     TIMESTAMP NOT NULL
                CONSTRAINT "TimeStamp must be on the bb_timestamp_precision"
                           CHECK (timestamp
                                   = date_trunc('bb_timestamp_precision'
                                              , timestamp)),
  wsoftware     VARCHAR(3) NOT NULL
                CONSTRAINT "WSoftware on WEATHERHAWK_SOFTWARES" REFERENCES weatherhawk_softwares,
  recordnum     INT,
  batvolt       NUMERIC(4,2) NOT NULL
                between_incl_num(`BatVolt', `bb_min_batvolt', `bb_max_batvolt'),
  batvolt_min   NUMERIC(4,2)
                between_incl_num(`BatVolt_Min', `bb_min_batvolt', `bb_max_batvolt'),
  airtemp_avg   NUMERIC(3,1) NOT NULL
                between_incl_num(`AirTemp_Avg', `bb_min_temp', `bb_max_temp'),
  relativehumidity_avg  NUMERIC(4,1) NOT NULL
                between_incl_num(`RelativeHumidity_Avg', `bb_min_relhumidity', `bb_max_relhumidity'),
  windspeed_avg_km_hr  NUMERIC(2,0)
                between_incl_num(`WindSpeed_Avg_Km_Hr', `bb_min_windspeed_km_hr', `bb_max_windspeed_km_hr'),
  windspeed_avg_m_s  NUMERIC(3,1)
                between_incl_num(`WindSpeed_Avg_M_S', `bb_min_windspeed_m_s', `bb_max_windspeed_m_s'),
  solar         NUMERIC(5,1)
                between_incl_num(`Solar', `bb_min_solar', `bb_max_solar'),
  airtemp_min   NUMERIC(3,1)
                between_incl_num(`AirTemp_Min', `bb_min_temp', `bb_max_temp'),
  airtemp_min_time  TIMESTAMP
                no_seconds_check(`AirTemp_Min_Time')
                CONSTRAINT "AirTemp_Min_Time must be before or equal to TimeStamp"
                           CHECK (airtemp_min_time <= timestamp),
  airtemp_max   NUMERIC(3,1)
                between_incl_num(`AirTemp_Max', `bb_min_temp', `bb_max_temp'),
  airtemp_max_time  TIMESTAMP
                no_seconds_check(`AirTemp_Max_Time')
                CONSTRAINT "AirTemp_Max_Time must be before or equal to TimeStamp"
                           CHECK (airtemp_max_time <= timestamp),
  wind_dir      numeric(4,1) NOT NULL
                between_incl_num(`Wind_Dir', `bb_min_winddir', `bb_max_winddir'),
  windspeed_max_km_hr  NUMERIC(2,0)
                between_incl_num(`WindSpeed_Max_Km_Hr', `bb_min_windspeed_km_hr', `bb_max_windspeed_km_hr'),
  windspeed_max_m_s  NUMERIC(3,1)
                between_incl_num(`WindSpeed_Max_M_S', `bb_min_windspeed_m_s', `bb_max_windspeed_m_s'),
  windspeed_max_time  TIMESTAMP
                no_seconds_check(`WindSpeed_Max_Time')
                CONSTRAINT "WindSpeed_Max_Time must be before or equal to TimeStamp"
                           CHECK (windspeed_max_time <= timestamp),
  barometer     NUMERIC(3,1) NOT NULL
                between_incl_num(`Barometer', `bb_min_barom', `bb_max_barom'),
  yearlyrain    INT NOT NULL
                CONSTRAINT "YearlyRain must be greater than or equal to bb_min_yearlyrain"
                           CHECK (yearlyrain
                                   >= bb_min_yearlyrain),
  timestamprain  INT NOT NULL,
                CONSTRAINT "TimeStampRain cannot be negative"
                           CHECK (timestamprain
                                   >= 0),
  null_if_null(`windspeed_max_km_hr', `windspeed_avg_km_hr'),
  null_if_null(`windspeed_max_m_s', `windspeed_avg_m_s'),
  CONSTRAINT "Average Wind Speed must be recorded"
             CHECK (COALESCE(windspeed_avg_km_hr, windspeed_avg_m_s)
                      IS NOT NULL),
  CONSTRAINT "Average Wind Speed must be in only one of WindSpeed_Avg_Km_Hr and WindSpeed_Avg_M_S"
             CHECK (windspeed_avg_km_hr IS NULL AND windspeed_avg_m_s IS NOT NULL
                      OR
                      windspeed_avg_km_hr IS NOT NULL AND windspeed_avg_m_s IS NULL),
  null_iff_null(`AirTemp_Min', `AirTemp_Min_Time'),
  null_iff_null(`AirTemp_Max', `AirTemp_Max_Time'),
  CONSTRAINT "WindSpeed_Max_Time and (either WindSpeed_Max_Km_Hr or _M_S) must both be NULL or both non-NULL"
             CHECK ((COALESCE(windspeed_max_km_hr, windspeed_max_m_s) IS NULL
                      AND windspeed_max_time IS NULL)
                     OR
                     (COALESCE(windspeed_max_km_hr, windspeed_max_m_s) IS NOT NULL
                      AND windspeed_max_time IS NOT NULL))
  ) WITHOUT OIDS;
grant_priv(`weatherhawk')
grant_seq_priv(`weatherhawk', `wid')

CREATE TABLE wreadings (
  wrid          SERIAL PRIMARY KEY,
  wstation      VARCHAR(3) NOT NULL
                CONSTRAINT "Wstation on WSTATIONS" REFERENCES wstations,
  wrdaytime     TIMESTAMP NOT NULL,
  estdaytime    BOOLEAN NOT NULL,
  wrperson      VARCHAR(4) NOT NULL
                CONSTRAINT "WRperson on OBSERVERS" REFERENCES observers,
  wrnotes       TEXT NULL
                empty_string_check(`wrnotes')
  ) WITHOUT OIDS;
grant_priv(`wreadings')
grant_seq_priv(`wreadings', `wrid')

CREATE TABLE tempmins (
  wrid          INT PRIMARY KEY
                CONSTRAINT "WRid on WREADINGS" REFERENCES wreadings,
  tempmin       NUMERIC(3,1) NOT NULL,
                CONSTRAINT "Tempmin must be between bb_min_tempmin and bb_max_tempmin (inclusive)"
                           CHECK(tempmin >= bb_min_tempmin AND tempmin
<= bb_max_tempmin),
                CONSTRAINT "Tempmin must be a multiple of 0.5"
                            CHECK((tempmin * 2) % 1 = 0)
) WITHOUT OIDS;
grant_priv(`tempmins')

CREATE TABLE tempmaxs (
  wrid          INT PRIMARY KEY
                CONSTRAINT "WRid on WREADINGS" REFERENCES wreadings,
  tempmax       NUMERIC(3,1) NOT NULL
                CONSTRAINT "Tempmax must be between bb_min_tempmax and bb_max_tempmax (inclusive)"
                           CHECK(tempmax >= bb_min_tempmax
                                 AND tempmax <= bb_max_tempmax)
                CONSTRAINT "Tempmax must be a multiple of 0.5"
                            CHECK((tempmax * 2) % 1 = 0)
) WITHOUT OIDS;
grant_priv(`tempmaxs')

CREATE TABLE raingauges (
  wrid          INT PRIMARY KEY
                CONSTRAINT "WRid on WREADINGS" REFERENCES wreadings,
  rgspan        INTERVAL(0) NOT NULL,
  estrgspan     BOOLEAN NULL,
  rain          NUMERIC(4,1) NOT NULL
                CONSTRAINT "Rain must be non-negative"
                           CHECK(rain >= bb_min_rain)
                CONSTRAINT "Rain cannot be more than bb_max_rain`'mm"
                           CHECK(rain <= bb_max_rain)
) WITHOUT OIDS;
grant_priv(`raingauges')

CREATE TABLE rgsetups (
  rgsid         SERIAL PRIMARY KEY,
  wstation      VARCHAR(3) NOT NULL
                CONSTRAINT "Wstation on WSTATIONS" REFERENCES wstations,
  rgsdaytime    TIMESTAMP NOT NULL,
  rgsestdaytime BOOLEAN NOT NULL,
  rgsperson     VARCHAR(4) NOT NULL
                CONSTRAINT "RGSperson on OBSERVERS" REFERENCES observers
  ) WITHOUT OIDS;
grant_priv(`rgsetups')
grant_seq_priv(`rgsetups', `rgsid')

-- mpis

CREATE TABLE mpis (
  mpiid SERIAL PRIMARY KEY,
  date DATE NOT NULL,
  context_type CHAR(1) NOT NULL
          CONSTRAINT "Context_type on CONTEXT_TYPES"
                     REFERENCES context_types(context_type)
          CONSTRAINT "Context_type of bb_no_context requires Context be NULL"
             CHECK(context_type <> 'bb_no_context'
                   OR (context_type = 'bb_no_context' AND context IS NULL))
          CONSTRAINT "Context_type of bb_consort_context requires Context be NULL"
             CHECK(context_type <> 'bb_consort_context'
                   OR (context_type = 'bb_consort_context' AND context IS NULL)),
  context TEXT NULL
          empty_string_check(`context')
) WITHOUT OIDS;

grant_priv(`mpis')
grant_seq_priv(`mpis', `mpiid')

-- consorts

CREATE TABLE consorts (
  mpiid    INT PRIMARY KEY
           CONSTRAINT "Mpiid on MPIS" REFERENCES mpis,
  female   CHAR(3) NULL
           CONSTRAINT "Female on BIOGRAPH" REFERENCES biograph(sname),
  had      CHAR(3) NOT NULL
           CONSTRAINT "Had on BIOGRAPH" REFERENCES biograph(sname),
  got      CHAR(3) NOT NULL
           CONSTRAINT "Got on BIOGRAPH" REFERENCES biograph(sname)
) WITHOUT OIDS;

grant_priv(`consorts')


-- mpiacts

CREATE TABLE mpiacts (
  mpiact   VARCHAR(5) PRIMARY KEY,
  descr    TEXT NOT NULL UNIQUE,
  kind     CHAR(1) NOT NULL
  CONSTRAINT "Kind must be one of: bb_agonism_mact, bb_help_request_mact, bb_help_given_mact, or bb_other_mact"
             CHECK(kind = 'bb_agonism_mact'
                   OR kind = 'bb_help_request_mact'
                   OR kind = 'bb_help_given_mact'
                   OR kind = 'bb_other_mact'),
  decided  BOOLEAN NOT NULL
  CONSTRAINT "Decided can only be TRUE when Kind is bb_agonism_mact"
             CHECK(NOT(decided AND kind <> 'bb_agonism_mact')),
  multi_first BOOLEAN NOT NULL
  CONSTRAINT "Multi_first can only be TRUE when Kind is bb_agonism_mact"
             CHECK(NOT(multi_first AND kind <> 'bb_agonism_mact'))
) WITHOUT OIDS;

grant_priv(`mpiacts')

-- mpi_data

CREATE TABLE mpi_data (
  mpidid  SERIAL PRIMARY KEY,
  mpiid   INT NOT NULL
          CONSTRAINT "Mpiid on MPIS" REFERENCES mpis,
  mpiact  VARCHAR(5) NOT NULL
          CONSTRAINT "Mpiact on MPIACTS" REFERENCES mpiacts,
  seq     INT NOT NULL,
  helped  BOOLEAN NULL,
  active  BOOLEAN NULL
) WITHOUT OIDS;

grant_priv(`mpi_data')
grant_seq_priv(`mpi_data', `mpidid')

-- mpi_parts
CREATE TABLE mpi_parts (
  mpipid   SERIAL PRIMARY KEY,
  mpidid   INT NOT NULL
           CONSTRAINT "MPIDid on MPI_DATA" REFERENCES mpi_data,
  sname    CHAR(3) NULL
           CONSTRAINT "Sname on BIOGRAPH" REFERENCES biograph(sname),
  unksname CHAR(1) NULL
           CONSTRAINT "Unksname on PARTUNKS" REFERENCES partunks,
  role  CHAR(1) NOT NULL
        CONSTRAINT "Role on MPI_PARTS is one of bb_actor or bb_actee"
        CHECK (role = 'bb_actor' OR role = 'bb_actee'),
  CONSTRAINT "Either Sname or Unksname but not both must be NULL"
    CHECK (((sname IS NULL) OR (unksname IS NULL))
           AND ((sname IS NOT NULL) OR (unksname IS NOT NULL)))
) WITHOUT OIDS;

grant_priv(`mpi_parts')
grant_seq_priv(`mpi_parts', `mpipid')

-- dad_data
CREATE TABLE dad_data (
  dadid SERIAL PRIMARY KEY,
  sname_type_column(`kid', `Kid', `NOT NULL'),
  mom_sampled BOOLEAN NULL,
  dad_sampled BOOLEAN NULL
              null_iff_null(`Dad_sampled', `Dad_consensus'),
  sname_type_column(`dad_excl', `Dad_excl', `NULL'),
  loci_excl SMALLINT NULL
            null_iff_null(`Loci_excl', `Dad_excl')
            CONSTRAINT "Loci_excl must be NULL or between bb_min_loci_excl and bb_max_loci_excl, inclusive"
                       CHECK (loci_excl IS NULL 
                              OR (bb_min_loci_excl <= loci_excl
                                  AND loci_excl <= bb_max_loci_excl)),
  pdads_considered SMALLINT NOT NULL
                   CONSTRAINT "Pdads_considered must be between 0 and bb_max_pdads_considered, inclusive"
                       CHECK (0 <= pdads_considered
                              AND pdads_considered <= bb_max_pdads_considered),
  pdads_typed SMALLINT NOT NULL
              CONSTRAINT "Pdads_typed must be <= Pdads_considered"
              CHECK (pdads_typed <= pdads_considered),
              CONSTRAINT "Pdads_typed must be >= 0"
              CHECK (pdads_typed >= 0),
  sname_type_column(`dad_1perr', `Dad_1perr', `NULL'),
  conf_1perr NUMERIC(3,2) NULL
             null_iff_null(`Conf_1perr', `Dad_1perr')
             CONSTRAINT "Conf_1perr must be between bb_almost_never and bb_almost_surely, inclusive"
                        CHECK (conf_1perr >= bb_almost_never
                               AND conf_1perr <= bb_almost_surely),
  sname_type_column(`dad_5perr', `Dad_5perr', `NULL'),
  conf_5perr NUMERIC(3,2) NULL
             null_iff_null(`Conf_5perr', `Dad_5perr')
             CONSTRAINT "Conf_5perr must be between bb_almost_never and bb_almost_surely, inclusive"
                        CHECK (conf_5perr >= bb_almost_never
                               AND conf_5perr <= bb_almost_surely),
  sname_type_column(`dad_allmales', `Dad_allmales', `NULL'),
  conf_allmales NUMERIC(3,2) NULL
             null_iff_null(`Conf_allmales', `Dad_allmales')
             CONSTRAINT "Conf_allmales must be between bb_almost_never and bb_almost_surely, inclusive"
                        CHECK (conf_allmales >= bb_almost_never
                               AND conf_allmales <= bb_almost_surely),
  sname_type_column(`dad_consensus', `Dad_consensus', `NULL'),
  date DATE NOT NULL,
  software VARCHAR(bb_dad_software_id_len) NOT NULL
           CONSTRAINT "Software code must be on DAD_SOFTWARE"
                      REFERENCES dad_software,
  comments TEXT NULL
           empty_string_check(`comments'),
  consensus_mismatch VARCHAR(12)
                     CONSTRAINT "Consensus_Mismatch on PATERNITY_MISMATCHES"
                                REFERENCES paternity_mismatches
                     null_iff_null(`Dad_consensus', `Consensus_Mismatch'),
  completeness SMALLINT NOT NULL
               CONSTRAINT "Completeness on PATERNITY_COMPLETENESS"
                          REFERENCES paternity_completeness
);

grant_priv(`dad_data')
grant_seq_priv(`dad_data', `dadid')


-- dartings

CREATE TABLE dartings (
  dartid SERIAL PRIMARY KEY,
  sname_column,  
  date DATE NOT NULL,
  darttime TIME(0) NULL
              no_seconds_check(`Darttime')
              CONSTRAINT "Darttime >= bb_day_start"
                CHECK (darttime::time >= 'bb_day_start'::time)
              CONSTRAINT "Darttime <= bb_day_end"
                CHECK (darttime::time <= 'bb_day_end'::time),
  downtime TIME(0) NULL
           no_seconds_check(`Downtime')
               CONSTRAINT "Downtime cannot be before Darttime"
                 CHECK (downtime >= darttime)
               CONSTRAINT "Downtime cannot be later than bb_downtime_max_delay after Darttime"
                 CHECK (downtime <= darttime
                                    + 'bb_downtime_max_delay'::interval),
  pickuptime TIME(0) NULL
             no_seconds_check(`Pickuptime')
                 CONSTRAINT "Pickuptime cannot be before Downtime"
                   CHECK (pickuptime >= downtime)
                 CONSTRAINT "Pickuptime cannot be later than bb_pickuptime_max_delay after Downtime"
                   CHECK (Pickuptime < downtime + 'bb_pickuptime_max_delay'::interval)
                 CONSTRAINT "Pickuptime cannot be before Darttime"
                   CHECK (pickuptime >= darttime)
                 CONSTRAINT "Pickuptime cannot be later than bb_pickuptime_max_delay after Darttime"
                   CHECK (pickuptime < darttime
                                       + 'bb_pickuptime_max_delay'::interval),
  drug CHAR(1) NOT NULL
         CONSTRAINT "Drug on DRUGS" REFERENCES drugs,
  mass NUMERIC(3, 1) NULL
       CONSTRAINT "Mass cannot be less than 1Kg"
         CHECK (mass >= 1)
       CONSTRAINT "Mass cannot be more than 40Kg"
         CHECK (mass <= 40),
  logisticnotes TEXT NULL empty_string_check(`Logisticnotes'),
  dartcomments TEXT NULL empty_string_check(`Dartcomments'),
  crnotes TEXT NULL empty_string_check(`CRnotes'),
  chnotes TEXT NULL empty_string_check(`Chnotes'),
  ulnotes TEXT NULL empty_string_check(`Ulnotes'),
  hunotes TEXT NULL empty_string_check(`Hunotes'),
  dphysnotes TEXT NULL empty_string_check(`Dphysnotes'),
  pcvnotes TEXT NULL empty_string_check(`PCVnotes'),
  bodytempnotes TEXT NULL empty_string_check(`Bodytempnotes'),
  dsamplenotes TEXT NULL empty_string_check(`Dsamplenotes'),
  teethnotes TEXT NULL empty_string_check(`Teethnotes'),
  caninenotes TEXT NULL empty_string_check(`Caninenotes'),
  testesnotes TEXT NULL empty_string_check(`Testesnotes'),
  ticknotes TEXT NULL empty_string_check(`Ticknotes')  
) WITHOUT OIDS;

grant_priv(`dartings')
grant_seq_priv(`dartings', `dartid')

-- anesths

CREATE TABLE anesths (
  anesthid SERIAL PRIMARY KEY,
  dartid INT NOT NULL
         CONSTRAINT "Dartid on DARTINGS" REFERENCES dartings,
  drug CHAR(1) NOT NULL
         CONSTRAINT "Drug on DRUGS" REFERENCES drugs,
  antime TIME(0) NULL
         no_seconds_check(`antime'),
  anamount NUMERIC(3, 2) NOT NULL
           CONSTRAINT "Anamount must not be more than bb_max_anamount"
              CHECK (anamount <= bb_max_anamount)
           CONSTRAINT "Anamount cannot be negative"
              CHECK (anamount >= 0)
) WITHOUT OIDS;

grant_priv(`anesths')
grant_seq_priv(`anesths', `anesthid')

--dart_sample_types

CREATE TABLE dart_sample_types (
  ds_type SMALLINT PRIMARY KEY NOT NULL,
  ds_cat SMALLINT NOT NULL
         CONSTRAINT "DS_Cat on DART_SAMPLE_CATS" REFERENCES dart_sample_cats,
  descr TEXT NOT NULL
        empty_string_check(`descr'),
  sex CHAR(1)
      empty_string_check(`sex')
      CONSTRAINT "Sex either bb_male or bb_female if not NULL"
                 CHECK(sex IS NULL OR sex = 'bb_male' OR sex = 'bb_female'),
  minimum SMALLINT NOT NULL,
  maximum SMALLINT NOT NULL
) WITHOUT OIDS;

grant_priv(`dart_sample_types')

--dart_samples

CREATE TABLE dart_samples (
  ds_id SERIAL PRIMARY KEY,
  dartid INT NOT NULL
         CONSTRAINT "Dartid on DARTINGS" REFERENCES dartings,
  ds_type INT NOT NULL
         CONSTRAINT "DS_Type on DART_SAMPLE_TYPES" REFERENCES dart_sample_types,
  num SMALLINT NOT NULL
      CONSTRAINT "Num must be > 0"
        CHECK (num>0)
) WITHOUT OIDS;

grant_priv(`dart_samples')
grant_seq_priv(`dart_samples', `ds_id')


-- crownrumps

CREATE TABLE crownrumps (
  crid SERIAL PRIMARY KEY,
  dartid INT NOT NULL
         CONSTRAINT "Dartid on DARTINGS" REFERENCES dartings,
  crlength NUMERIC(3, 1) NOT NULL
           CONSTRAINT "CRlength may not be less than bb_min_crlength`'cm"
             CHECK (crlength >= bb_min_crlength),
dnl  cr_status CHAR(1) NOT NULL
dnl            CONSTRAINT "CR_Status on CR_STATUSES" REFERENCES cr_statuses,
  crseq smallint NULL
        CONSTRAINT "CRseq cannot be less than 1"
          CHECK (crseq >= 1),
  crobserver VARCHAR(4) NULL
             CONSTRAINT "CRobserver on Observers" REFERENCES observers
) WITHOUT OIDS;

grant_priv(`crownrumps')
grant_seq_priv(`crownrumps', `crid')

-- chests

CREATE TABLE chests (
  chid SERIAL PRIMARY KEY,
  dartid INT NOT NULL
         CONSTRAINT "Dartid on DARTINGS" REFERENCES dartings,
  chcircum NUMERIC(3, 1) NOT NULL
           CONSTRAINT "Chcircum may not be less than bb_min_chcircum`'cm"
             CHECK (chcircum >= bb_min_chcircum),
  chunadjusted NUMERIC(3, 1) NULL
               CONSTRAINT "Chunadjusted may not be less than bb_min_chcircum`'cm"
                 CHECK (chunadjusted >= bb_min_chcircum),
  chseq smallint NULL
        CONSTRAINT "Chseq cannot be less than 1"
          CHECK (chseq >= 1),
  chobserver VARCHAR(4) NULL
             CONSTRAINT "Chobserver on Observers" REFERENCES observers
) WITHOUT OIDS;

grant_priv(`chests')
grant_seq_priv(`chests', `chid')


-- ulnas

CREATE TABLE ulnas (
  ulid SERIAL PRIMARY KEY,
  dartid INT NOT NULL
         CONSTRAINT "Dartid on DARTINGS" REFERENCES dartings,
  ullength NUMERIC(3, 1) NOT NULL
           CONSTRAINT "Ullength may not be less than bb_min_ullength`'cm"
             CHECK (ullength >= bb_min_ullength)
           CONSTRAINT "Ullength may not be greater than bb_max_ullength`'cm"
             CHECK (ullength <= bb_max_ullength),
  ulunadjusted NUMERIC(3, 1) NULL
               CONSTRAINT "Ulunadjusted may not be less than bb_min_ullength`'cm"
                 CHECK (ulunadjusted >= bb_min_ullength)
           CONSTRAINT "Ulunadjusted may not be greater than bb_max_ullength`'cm"
             CHECK (ulunadjusted <= bb_max_ullength),
  ulseq SMALLINT NULL
        CONSTRAINT "Ulseq cannot be less than 1"
          CHECK (ulseq >= 1),
  ulobserver VARCHAR(4) NULL
             CONSTRAINT "Ulobserver on Observers" REFERENCES observers
) WITHOUT OIDS;

grant_priv(`ulnas')
grant_seq_priv(`ulnas', `ulid')


-- humeruses

CREATE TABLE humeruses (
  huid SERIAL PRIMARY KEY,
  dartid INT NOT NULL
         CONSTRAINT "Dartid on DARTINGS" REFERENCES dartings,
  hulength NUMERIC(3, 1) NOT NULL
           CONSTRAINT "Hulength may not be less than bb_min_hulength`'cm"
             CHECK (hulength >= bb_min_hulength)
           CONSTRAINT "Hulength may not be greater than bb_max_hulength`'cm"
             CHECK (hulength <= bb_max_hulength),
  huunadjusted NUMERIC(3, 1) NULL
               CONSTRAINT "Huunadjusted may not be less than bb_min_hulength`'cm"
                 CHECK (huunadjusted >= bb_min_hulength)
           CONSTRAINT "Huunadjusted may not be greater than bb_max_hulength`'cm"
             CHECK (huunadjusted <= bb_max_hulength),
  huseq SMALLINT NULL
        CONSTRAINT "Huseq cannot be less than 1"
          CHECK (huseq >= 1),
  huobserver VARCHAR(4) NULL
             CONSTRAINT "Huobserver on Observers" REFERENCES observers
) WITHOUT OIDS;

grant_priv(`humeruses')
grant_seq_priv(`humeruses', `huid')


-- dphys

CREATE TABLE dphys (
  dphysid SERIAL PRIMARY KEY,
  dartid INT NOT NULL
         CONSTRAINT "Dartid on DARTINGS" REFERENCES dartings,
  pulse SMALLINT NULL
        CONSTRAINT "Pulse must be greater than bb_min_pulse"
          CHECK (pulse > bb_min_pulse),
  respiration SMALLINT NULL
              CONSTRAINT "Respiration must be greater than bb_min_respiration"
                CHECK (respiration > bb_min_respiration),
  ringnode CHAR(1) NULL
           CONSTRAINT "Ringnode on LYMPHSTATES" REFERENCES lymphstates,
  lingnode CHAR(1) NULL
           CONSTRAINT "Lingnode on LYMPHSTATES" REFERENCES lymphstates,
  raxnode CHAR(1) NULL
           CONSTRAINT "Raxnode on LYMPHSTATES" REFERENCES lymphstates,
  laxnode CHAR(1) NULL
           CONSTRAINT "Laxnode on LYMPHSTATES" REFERENCES lymphstates,
  rsubmandnode CHAR(1) NULL
           CONSTRAINT "Rsubmandnode on LYMPHSTATES" REFERENCES lymphstates,
  lsubmandnode CHAR(1) NULL
           CONSTRAINT "Lsubmandnode on LYMPHSTATES" REFERENCES lymphstates
) WITHOUT OIDS;

grant_priv(`dphys')
grant_seq_priv(`dphys', `dphysid')


-- pcvs

CREATE TABLE pcvs (
  pcvid SERIAL PRIMARY KEY,
  dartid INT NOT NULL
         CONSTRAINT "Dartid on DARTINGS" REFERENCES dartings,
  pcv SMALLINT NOT NULL
      CONSTRAINT "PCV >= bb_min_pcv"
        CHECK (pcv >= bb_min_pcv)
      CONSTRAINT "PCV <= bb_max_pcv"
        CHECK (pcv <= bb_max_pcv),
  pcvseq SMALLINT NULL
         CONSTRAINT "PCVseq cannot be less than 1"
           CHECK (pcvseq >= 1)
) WITHOUT OIDS;

grant_priv(`pcvs')
grant_seq_priv(`pcvs', `pcvid')


-- bodytemps

CREATE TABLE bodytemps (
  btid SERIAL PRIMARY KEY,
  dartid INT NOT NULL
         CONSTRAINT "Dartid on DARTINGS" REFERENCES dartings,
  btemp NUMERIC(3,1) NOT NULL
        CONSTRAINT "Btemp cannot be less than than bb_min_btemp"
          CHECK (btemp >= bb_min_btemp)
      CONSTRAINT "Btemp cannot be more than bb_max_btemp"
        CHECK (btemp <= bb_max_btemp),
  bttime TIME(0) NULL
         no_seconds_check(`bttime')
) WITHOUT OIDS;

grant_priv(`bodytemps')
grant_seq_priv(`bodytemps', `btid')


-- wbc_counts

CREATE TABLE wbc_counts(
  wcid         SERIAL PRIMARY KEY,
  dartid       INT NOT NULL
               CONSTRAINT "Dartid on DARTINGS" REFERENCES dartings,
  count_date   DATE NOT NULL,
  basophils    SMALLINT NOT NULL
               CONSTRAINT "Basophils cannot be negative"
                 CHECK (basophils >= 0),
  eosinophils  SMALLINT NOT NULL
               CONSTRAINT "Eosinophils cannot be negative"
                 CHECK (eosinophils >= 0),
  monocytes    SMALLINT NOT NULL
               CONSTRAINT "Monocytes cannot be negative"
                 CHECK (monocytes >= 0),
  lymphocytes  SMALLINT NOT NULL
               CONSTRAINT "Lymphocytes cannot be negative"
                 CHECK (lymphocytes >= 0),
  neutrophils  SMALLINT NOT NULL
               CONSTRAINT "Neutrophils cannot be negative"
                 CHECK (neutrophils >= 0),
  counted_by   VARCHAR(4) NOT NULL
               CONSTRAINT "Counted_By on OBSERVERS"
                            REFERENCES observers,
  slide_number SMALLINT NOT NULL
               CONSTRAINT "Slide_number must be positive"
                 CHECK (slide_number > 0)
) WITHOUT OIDS;

grant_priv(`wbc_counts')
grant_seq_priv(`wbc_counts', `wcid')

-- teeth

CREATE TABLE teeth (
  teethid SERIAL PRIMARY KEY,
  dartid INT NOT NULL
         CONSTRAINT "Dartid on DARTINGS" REFERENCES dartings,
  tooth VARCHAR(5) NOT NULL
        CONSTRAINT "Tooth on TOOTHCODES" REFERENCES toothcodes,
  tstate CHAR(1) NULL -- dis-allow NULL in triggers
        CONSTRAINT "Tstate on TSTATES" REFERENCES tstates,
  tcondition CHAR(1) NULL
        CONSTRAINT "Tcondition on TCONDITIONS" REFERENCES tconditions
  CONSTRAINT "Tcondition must be NULL when Tstate is bb_tooth_missing"
             CHECK (NOT(tstate = 'bb_tooth_missing'
                        AND tcondition IS NOT NULL))
  CONSTRAINT "Tcondition must be non-NULL unless Tstate is bb_tooth_missing"
             CHECK (NOT(tstate <> 'bb_tooth_missing'
                        AND tcondition IS NULL))
) WITHOUT OIDS;

grant_priv(`teeth')
grant_seq_priv(`teeth', `teethid')


-- testes_foo tables

_make_testes_table(`ARC')
_make_testes_table(`DIAM')



-- ticks

CREATE TABLE ticks (
  tickid SERIAL PRIMARY KEY,
  dartid INT NOT NULL
         CONSTRAINT "Dartid on DARTINGS" REFERENCES dartings,
  bodypart VARCHAR(8) NOT NULL
           CONSTRAINT "Bodypart on BODYPARTS" REFERENCES bodyparts,
  tickkind VARCHAR(1) NOT NULL
           CONSTRAINT "Tickkind on PARASITES" REFERENCES parasites,
  tickcount SMALLINT NULL
           CONSTRAINT "Tickcount cannot be less than bb_min_tickcount"
             CHECK (tickcount >= bb_min_tickcount)
           CONSTRAINT "Tickcount cannot be greater than bb_max_tickcount"
             CHECK (tickcount <= bb_max_tickcount),
  tickstatus SMALLINT NOT NULL
           CONSTRAINT "Tickstatus on TICKSTATUSES" REFERENCES tickstatuses
           CONSTRAINT "Tickstatus must be bb_tickstatus_zero if and only if Tickcount is 0"
             CHECK(COALESCE((tickcount <> 0
                             AND tickstatus <> bb_tickstatus_zero)
                            OR (tickcount = 0 
                                AND tickstatus = bb_tickstatus_zero)
                          , tickcount IS NULL
                            AND tickstatus <> bb_tickstatus_zero))
           CONSTRAINT "Tickstatus must be bb_tickstatus_ticks when Tickcount is non-NULL and non-zero"
             CHECK (tickcount IS NULL
                    OR tickcount = 0
                    OR tickstatus = bb_tickstatus_ticks),
  tickbpnotes TEXT NULL
              empty_string_check(`tickbpnotes')
) WITHOUT OIDS;

grant_priv(`ticks')
grant_seq_priv(`ticks', `tickid')


-- swerb_departs_data

CREATE TABLE swerb_departs_data (
  did SERIAL PRIMARY KEY,
  date DATE NOT NULL
       CONSTRAINT "Date must be on or after bb_swerb_start"
         CHECK (date >= 'bb_swerb_start'),
  time TIME(0)  NULL
       CONSTRAINT "Time >= bb_day_early_start and Time <= bb_day_end"
         CHECK(time IS NULL
               OR (time >= 'bb_day_early_start' AND time <= 'bb_day_end'))
       no_seconds_check(`time')
) WITHOUT OIDS;

grant_priv(`swerb_departs_data')
grant_seq_priv(`swerb_departs_data', `did')


-- swerb_departs_gps

CREATE TABLE swerb_departs_gps (
  did INT PRIMARY KEY
      CONSTRAINT "DID on SWERB_DEPARTS_DATA" REFERENCES swerb_departs_data,
  xyloc GEOMETRY(POINT, bb_amboseli_srid) NOT NULL
        CONSTRAINT "X coord of XYLoc must have only 1 digit right of the decimal"
          CHECK(ST_X(xyloc) * 10 = TRUNC(ST_X(xyloc) * 10))
        CONSTRAINT "Y coord of XYLoc must have only 1 digit right of the decimal"
          CHECK(ST_Y(xyloc) * 10 = TRUNC(ST_Y(xyloc) * 10))
        CONSTRAINT "XYLoc within study area (see Babase Techinical Specifications)"
          CHECK(ST_Intersects(xyloc
                            , ST_PolygonFromText('bb_hard_study_area'
                                               , bb_amboseli_srid))),
  altitude NUMERIC(5,1) NULL
           CONSTRAINT "Altitude must be between bb_altitude_min and bb_altitude max, inclusive"
             CHECK(altitude IS NULL
                   OR (bb_altitude_min <= altitude
                       AND altitude <= bb_altitude_max)),
  pdop NUMERIC(3,1) NULL
        CONSTRAINT "PDOP must be between bb_pdop_min and bb_pdop_max, inclusive"
          CHECK(pdop IS NULL
                OR (bb_pdop_min <= pdop
                    AND pdop <= bb_pdop_max)),
  accuracy NUMERIC(3,1) NULL
           CONSTRAINT "Accuracy must be between bb_accuracy_min and bb_accuracy_max, inclusive"
           CHECK(accuracy IS NULL
                 OR (bb_accuracy_min <= accuracy
                     AND accuracy <= bb_accuracy_max)),
  gps NUMERIC(2) NULL
      CONSTRAINT "GPS on GPS_UNITS" REFERENCES gps_units,
  garmincode TEXT NULL
      no_spaces_check(`Garmincode')
) WITHOUT OIDS;

grant_priv(`swerb_departs_gps')


-- swerb_observers

CREATE TABLE swerb_observers (
  swerboid SERIAL PRIMARY KEY,
  did INT CONSTRAINT "Did on SWERB_DEPARTS_DATA"
            REFERENCES swerb_departs_data,
  observer VARCHAR(4) NOT NULL
           CONSTRAINT "Observer on OBSERVERS" REFERENCES observers,
  role VARCHAR(bb_observer_roles_len) NOT NULL
       CONSTRAINT "Role on OBSERVER_ROLES" REFERENCES observer_roles
) WITHOUT OIDS;

grant_priv(`swerb_observers')
grant_seq_priv(`swerb_observers', `swerboid')


-- swerb_bes

CREATE TABLE swerb_bes (
  beid SERIAL PRIMARY KEY,
  did INT NOT NULL
      CONSTRAINT "Did on SWERB_DEPARTS_DATA" REFERENCES swerb_departs_data,
  focal_grp b_grp_type NOT NULL
            CONSTRAINT "Focal_grp on GROUPS" REFERENCES groups,
  start TIME(0) NULL
        CONSTRAINT "Start >= bb_day_start and Start <= bb_day_end"
          CHECK(start IS NULL
                OR (start >= 'bb_day_start' AND start <= 'bb_day_end'))
        no_seconds_check(`Start'),
  btimeest BOOLEAN NULL
           null_if_null(`Start', `Btimeest'),
  bsource VARCHAR(5) NULL
          CONSTRAINT "SWERB_BES.Bsource on SWERB_TIME_SOURCES"
                     REFERENCES swerb_time_sources
          null_iff_null(`Btimeest', `Bsource')
          CONSTRAINT "Btimeest = FALSE when Bsource = bb_norecord"
                     CHECK((bsource = 'bb_norecord' AND btimeest = FALSE)
                           OR bsource IS NULL
                           OR bsource <> 'bb_norecord'),
  stop TIME(0) NULL
       CONSTRAINT "Stop >= bb_day_start and Stop <= bb_day_end"
         CHECK(stop IS NULL
               OR (stop >= 'bb_day_start' AND stop <= 'bb_day_end'))
       CONSTRAINT "Stop >= Start"
                   CHECK(start IS NULL
                         OR stop IS NULL
                         OR start <= stop)
       no_seconds_check(`Stop'),
  etimeest BOOLEAN NULL
           null_if_null(`Stop', `Etimeest'),
  esource VARCHAR(5) NULL
          CONSTRAINT "Esource on SWERB_TIME_SOURCES"
                     REFERENCES swerb_time_sources
          null_iff_null(`Etimeest', `Esource')
          CONSTRAINT "Etimeest = FALSE when Esource = bb_norecord"
                     CHECK((esource = 'bb_norecord' AND etimeest = FALSE)
                           OR esource IS NULL
                           OR esource <> 'bb_norecord'),
  seq SMALLINT NULL
      CONSTRAINT "SWERB_BES.Seq cannot be less than 1"
        CHECK(seq >= 1),
  is_effort BOOLEAN NOT NULL,
  notes TEXT NULL
        empty_string_check(`Notes')
) WITHOUT OIDS;

grant_priv(`swerb_bes')
grant_seq_priv(`swerb_bes', `beid')


-- swerb_data (gps_points)

CREATE TABLE swerb_data (
  swid SERIAL PRIMARY KEY,
  beid INT NOT NULL
       CONSTRAINT "BEId on SWERB_BEIDS" REFERENCES swerb_bes,
  seen_grp b_grp_type NOT NULL
      CONSTRAINT "Seen_grp on GROUPS" REFERENCES groups,
  lone_animal CHAR(3)
      CONSTRAINT "Lone_Animal on BIOGRAPH" REFERENCES biograph (sname)
      CONSTRAINT "Lone_Animal must be NULL unless Event = bb_swerb_ogrp"
                 CHECK(event = 'bb_swerb_ogrp' OR lone_animal IS NULL)
      CONSTRAINT "Lone_Animals must be in the bb_alone_grp group"
                 CHECK(lone_animal IS NULL OR seen_grp = bb_alone_grp),
  event CHAR(1) NOT NULL
      CONSTRAINT "SWERB_DATA.Event must be one of: bb_swerb_begin, bb_swerb_end, bb_swerb_obs, bb_swerb_water, bb_swerb_ogrp"
        CHECK(event = 'bb_swerb_begin'
              OR event = 'bb_swerb_end'
              OR event = 'bb_swerb_obs'
              OR event = 'bb_swerb_water'
              OR event = 'bb_swerb_ogrp'),
  time TIME(0) NULL
       no_seconds_check(`time')
       CONSTRAINT "Time >= bb_day_start and Time <= bb_day_end"
         CHECK(time IS NULL
               OR (time >= 'bb_day_start' AND time <= 'bb_day_end')),
  quad VARCHAR(4) NULL
       CONSTRAINT "Quad on QUAD_DATA" REFERENCES quad_data,
  xyloc GEOMETRY(POINT, bb_amboseli_srid) NULL
        CONSTRAINT "X coord of XYLoc must have only 1 digit right of the decimal"
          CHECK(ST_X(xyloc) * 10 = TRUNC(ST_X(xyloc) * 10))
        CONSTRAINT "Y coord of XYLoc must have only 1 digit right of the decimal"
          CHECK(ST_Y(xyloc) * 10 = TRUNC(ST_Y(xyloc) * 10))
        CONSTRAINT "XYLoc within study area (see Babase Techinical Specifications)"
          CHECK(ST_Intersects(xyloc
                            , ST_PolygonFromText('bb_hard_study_area'
                                               , bb_amboseli_srid))),
  altitude NUMERIC(5,1) NULL
           null_if_null(`Altitude', `XYLoc')
           CONSTRAINT "Altitude must be between (inclusive) bb_altitude_min and bb_altitude_max"
             CHECK(altitude IS NULL
                   OR (bb_altitude_min <= altitude
                       AND altitude <= bb_altitude_max)),
  pdop NUMERIC(3,1) NULL
        null_if_null(`Pdop', `XYLoc')
        CONSTRAINT "PDOP must be between bb_pdop_min and bb_pdop_max, inclusive"
          CHECK(pdop IS NULL
                OR (bb_pdop_min <= pdop
                    AND pdop <= bb_pdop_max)),
  accuracy NUMERIC(3,1) NULL
           null_if_null(`Accuracy', `XYLoc')
           CONSTRAINT "Accuracy must be between bb_accuracy_min and bb_accuracy_max, inclusive"
           CHECK(accuracy IS NULL
                 OR (bb_accuracy_min <= accuracy
                     AND accuracy <= bb_accuracy_max)),
  subgroup BOOLEAN NOT NULL,
  ogdistance NUMERIC(bb_swerb_ogdistance_len,0) NULL
             CONSTRAINT "Ogdistance must be >= 0"
                        CHECK(ogdistance IS NULL OR ogdistance >= 0)
             CONSTRAINT "Ogdistance must have a 0 in the ones digit"
                        CHECK(ogdistance IS NULL OR MOD(ogdistance, 10) = 0)
             CONSTRAINT "Ogdistance must be NULL unless Event = bb_swerb_ogrp"
                        CHECK(event = 'bb_swerb_ogrp'
                              OR ogdistance IS NULL),
  gps_datetime TIMESTAMP NULL
    null_if_null(`GPS_Datetime', `XYLoc')
    CONSTRAINT "GPS_Datetime cannot be more than bb_swerb_entry_delay after Time"
      CHECK (NOT(event<>'bb_swerb_water' AND
               gps_datetime::TIME
               > (time + 'bb_swerb_entry_delay'::interval)))
    CONSTRAINT "GPS_Datetime cannot be more than bb_swerb_wh_entry_delay after Time, when Event is bb_swerb_water"
      CHECK (NOT(event='bb_swerb_water' AND
               gps_datetime::TIME
               > (time + 'bb_swerb_wh_entry_delay'::interval)))
    CONSTRAINT "Time cannot be more than bb_swerb_entry_anticipation after GPS_Datetime"
      CHECK (NOT(time > (gps_datetime
                         + 'bb_swerb_entry_anticipation'::interval)::TIME)),
  garmincode VARCHAR(bb_garmincode_len) NULL
    null_if_null(`Garmincode', `XYLoc')
    no_spaces_check(`Garmincode')
) WITHOUT OIDS;

grant_priv(`swerb_data')
grant_seq_priv(`swerb_data', `swid')


-- swerb_gws

CREATE TABLE swerb_gws (
  loc VARCHAR(10) PRIMARY KEY
      CONSTRAINT "Loc cannot contain lower case"
        CHECK(loc = upper(loc))
      empty_string_check(`loc'),
  type CHAR(1) NOT NULL
    CONSTRAINT "Type on PLACE_TYPES" REFERENCES place_types
    CONSTRAINT "Type must be bb_swerb_gws_grove when Loc is bb_swerb_unk_grove"
      CHECK ((type = 'bb_swerb_gws_grove' AND loc = 'bb_swerb_unk_grove')
             OR loc <> 'bb_swerb_unk_grove'),
  altname VARCHAR(45) NULL
       empty_string_check(`altname'),
  start      DATE NOT NULL
    CONSTRAINT "Start cannot be before bb_swerb_start"
      CHECK (start >= CAST('bb_swerb_start' AS DATE)),
  finish     DATE NULL,
  notes TEXT NULL
    empty_string_check(`notes')
) WITHOUT OIDS;

grant_priv(`swerb_gws')


-- swerb_gw_loc_data

CREATE TABLE swerb_gw_loc_data (
  sgwlid SERIAL PRIMARY KEY,
  loc VARCHAR(10) NOT NULL
      CONSTRAINT "Loc on SWERB_GWS" REFERENCES swerb_gws
      CONSTRAINT "The unknown grove (bb_swerb_unknown_grove) may not be located anywhere"
                 CHECK(loc <> 'bb_swerb_unk_grove'),
  date DATE NOT NULL,
  time TIME(0) NULL
       no_seconds_check(`time')
       CONSTRAINT "Time >= bb_day_start and Time <= bb_day_end"
         CHECK(time IS NULL
               OR (time >= 'bb_day_start' AND time <= 'bb_day_end')),
  quad VARCHAR(4) NULL
       CONSTRAINT "Quad on QUAD_DATA" REFERENCES quad_data
       CONSTRAINT "Quad must be NULL after bb_quad_end"
         CHECK (quad IS NULL OR date <= 'bb_quad_end'),
  xysource CHAR(1) NULL
           CONSTRAINT "XYSource on SWERB_XYSOURCES" REFERENCES swerb_xysources
           null_iff_null(`XYSource', `XYLoc'),
  xyloc GEOMETRY(POINT, bb_amboseli_srid)
        CONSTRAINT "X coord of XYLoc must have only 1 digit right of the decimal"
          CHECK(ST_X(xyloc) * 10 = TRUNC(ST_X(xyloc) * 10))
        CONSTRAINT "Y coord of XYLoc must have only 1 digit right of the decimal"
          CHECK(ST_Y(xyloc) * 10 = TRUNC(ST_Y(xyloc) * 10))
        CONSTRAINT "XYLoc within study area (see Babase Techinical Specifications)"
          CHECK(ST_Intersects(xyloc
                            , ST_PolygonFromText('bb_hard_study_area'
                                               , bb_amboseli_srid)))
        CONSTRAINT "XYLoc must be NULL before bb_gps_start when XYSource is NULL"
          CHECK (xyloc IS NULL
                 OR date >= 'bb_gps_start'
                 OR xysource IS NOT NULL),
  altitude NUMERIC(5,1) NULL
           null_if_null(`Altitude', `XYLoc')
           CONSTRAINT "Altitude must be between (inclusive) bb_altitude_min and bb_altitude_max"
             CHECK(altitude IS NULL
                   OR (bb_altitude_min <= altitude
                       AND altitude <= bb_altitude_max)),
           CONSTRAINT "Cannot have sub-meter altitude accuracy before bb_subm_alt_date"
             CHECK(altitude IS NULL
                   OR date >= 'bb_subm_alt_date'
                   OR altitude = TRUNC(altitude)),
  pdop NUMERIC(3,1) NULL
       null_if_null(`Pdop', `XYLoc')
       CONSTRAINT "PDOP must be between bb_pdop_min and bb_pdop_max, inclusive"
         CHECK(pdop IS NULL
               OR (bb_pdop_min <= pdop
                   AND pdop <= bb_pdop_max))
       CONSTRAINT "PDOP values must be NULL before bb_earliest_pdop and after bb_latest_pdop"
         CHECK(pdop IS NULL
               OR ('bb_earliest_pdop' <= date
                   AND date <= 'bb_latest_pdop')),
  accuracy NUMERIC(3,1) NULL
           null_if_null(`Accuracy', `XYLoc')
           CONSTRAINT "Accuracy must be between bb_accuracy_min and bb_accuracy_max, inclusive"
             CHECK(accuracy IS NULL
                   OR (bb_accuracy_min <= accuracy
                       AND accuracy <= bb_accuracy_max))
           CONSTRAINT "Accuracy must be NULL before bb_earliest_accuracy"
             CHECK(accuracy IS NULL
                   OR date >= 'bb_earliest_accuracy'),
  gps NUMERIC(2) NULL
      null_if_null(`GPS', `XYLoc')
      CONSTRAINT "GPS on GPS_UNITS" REFERENCES gps_units
      CONSTRAINT "GPS units were not used before bb_gps_start"
        CHECK(gps IS NULL OR date >= 'bb_gps_start'),
  notes TEXT NULL
      empty_string_check(`Notes')
) WITHOUT OIDS;

grant_priv(`swerb_gw_loc_data')
grant_seq_priv(`swerb_gw_loc_data', `sgwlid')


-- swerb_loc_data

CREATE TABLE swerb_loc_data (
  swid INT PRIMARY KEY
       CONSTRAINT "SWid on SWERB_DATA" REFERENCES swerb_data,
  loc VARCHAR(10) NOT NULL
      CONSTRAINT "Loc on SWERB_GWS" REFERENCES swerb_gws,
  adcode VARCHAR(2) NOT NULL
      CONSTRAINT "ADcode on ADCODES" REFERENCES adcodes,
  loc_status CHAR(1) NOT NULL
      CONSTRAINT "Loc_Status on SWERB_LOC_STATUSES" REFERENCES swerb_loc_statuses
      CONSTRAINT "Loc_Status must be bb_swerb_certain when Loc is bb_swerb_unk_grove"
        CHECK(loc <> 'bb_swerb_unk_grove'
              OR (loc = 'bb_swerb_unk_grove' AND loc_status = 'bb_swerb_certain')),
  adtime TIME(0) NULL
       no_seconds_check(`adtime')
       CONSTRAINT "ADTime >= bb_day_start and ADTime <= bb_day_end"
         CHECK(adtime IS NULL
               OR (adtime >= 'bb_day_start' AND adtime <= 'bb_day_end'))
) WITHOUT OIDS;

grant_priv(`swerb_loc_data')


-- swerb_loc_data_confidences

CREATE TABLE swerb_loc_data_confidences (
  swid INT PRIMARY KEY
       CONSTRAINT "SWId on SWERB_LOC_DATA" REFERENCES swerb_loc_data
       -- Yes, this references SWERB_LOC_DATA, not SWERB_DATA
       ,
  confidence NUMERIC(2,1) NOT NULL
       CONSTRAINT "Confidence on SWERB_LOC_CONFIDENCES"
                   REFERENCES swerb_loc_confidences,
  nearest_loc VARCHAR(10) NOT NULL
       CONSTRAINT "Nearest_Loc on SWERB_GWS" REFERENCES swerb_gws
) WITHOUT OIDS;

grant_priv(`swerb_loc_data_confidences')

-- swerb_loc_gps

CREATE TABLE swerb_loc_gps (
  swid INT PRIMARY KEY
       CONSTRAINT "SWid on SWERB_DATA" REFERENCES swerb_data,
  xyloc GEOMETRY(POINT, bb_amboseli_srid) NOT NULL
        CONSTRAINT "X coord of XYLoc must have only 1 digit right of the decimal"
          CHECK(ST_X(xyloc) * 10 = TRUNC(ST_X(xyloc) * 10))
        CONSTRAINT "Y coord of XYLoc must have only 1 digit right of the decimal"
          CHECK(ST_Y(xyloc) * 10 = TRUNC(ST_Y(xyloc) * 10))
        CONSTRAINT "XYLoc within study area (see Babase Techinical Specifications)"
          CHECK(ST_Intersects(xyloc
                            , ST_PolygonFromText('bb_hard_study_area'
                                               , bb_amboseli_srid))),
  altitude NUMERIC(5,1) NULL
           CONSTRAINT "Altitude must be between (inclusive) bb_altitude_min and bb_altitude_max"
             CHECK(altitude IS NULL
                   OR (bb_altitude_min <= altitude
                       AND altitude <= bb_altitude_max)),
           CONSTRAINT "Cannot have sub-meter altitude accuracy before bb_subm_alt_date"
             CHECK(altitude IS NULL
                   OR gps_datetime::DATE >= 'bb_subm_alt_date'
                   OR altitude = TRUNC(altitude)),
  pdop NUMERIC(3,1) NULL
       CONSTRAINT "PDOP must be between bb_pdop_min and bb_pdop_max, inclusive"
         CHECK(pdop IS NULL
               OR (bb_pdop_min <= pdop
                   AND pdop <= bb_pdop_max))
       CONSTRAINT "PDOP values must be NULL before bb_earliest_pdop and after bb_latest_pdop"
         CHECK(pdop IS NULL
               OR ('bb_earliest_pdop' <= gps_datetime::DATE
                   AND gps_datetime::date <= 'bb_latest_pdop')),
  accuracy NUMERIC(3,1) NULL
           CONSTRAINT "Accuracy must be between bb_accuracy_min and bb_accuracy_max, inclusive"
             CHECK(accuracy IS NULL
                   OR (bb_accuracy_min <= accuracy
                       AND accuracy <= bb_accuracy_max))
           CONSTRAINT "Accuracy must be NULL before bb_earliest_accuracy"
             CHECK(accuracy IS NULL
                   OR gps_datetime::date >= 'bb_earliest_accuracy'),
  gps_datetime TIMESTAMP NOT NULL
    CONSTRAINT "SWERB_LOC_GPS rows may not exist before bb_subm_alt_date"
      CHECK(gps_datetime::date >= 'bb_subm_alt_date'),
  garmincode VARCHAR(bb_garmincode_len) NOT NULL
    no_spaces_check(`Garmincode')
) WITHOUT OIDS;

grant_priv(`swerb_loc_gps')


-- swerb_trees

CREATE TABLE trees (
  tid SERIAL PRIMARY KEY,
  loc VARCHAR(10) NOT NULL
      CONSTRAINT "Loc on SWERB_GWS" REFERENCES swerb_gws
      CONSTRAINT "Trees may not be placed in the unknown grove (bb_swerb_unknown_grove)"
                 CHECK(loc <> 'bb_swerb_unk_grove'),
  tree INT NOT NULL
) WITHOUT OIDS;

grant_priv(`trees')
grant_seq_priv(`trees', `tid')


-- behave_gaps

CREATE TABLE behave_gaps (
  bgid SERIAL PRIMARY KEY,
  grp b_grp_type NOT NULL
      CONSTRAINT "Grp on GROUPS" REFERENCES groups,
  gap_start DATE NOT NULL,
  gap_end DATE,
  gap_end_status SMALLINT
      CONSTRAINT "Gap_End_Status on GAP_END_STATUSES"
                 REFERENCES gap_end_statuses
      CONSTRAINT
          "Gap_End and Gap_End_Status must both be NULL or both non-NULL"
          CHECK((gap_end IS NULL AND gap_end_status IS NULL)
                OR (gap_end IS NOT NULL AND gap_end_status IS NOT NULL)),
  notes TEXT NOT NULL
        empty_string_check(`Notes'),
  CONSTRAINT "Gap_End must be after Gap_Start, or NULL"
             CHECK(gap_end IS NULL OR gap_end > gap_start)
) WITHOUT OIDS;

grant_priv(`behave_gaps')
grant_seq_priv(`behave_gaps', `bgid')


-- hybridgene_analyses

CREATE TABLE hybridgene_analyses (
  hgaid       SERIAL PRIMARY KEY NOT NULL,
  date        DATE NOT NULL,
  analyzed_by VARCHAR(4) NOT NULL
              CONSTRAINT "Analyzed_By on OBSERVERS"
                REFERENCES observers,
  software    SMALLINT NOT NULL
              CONSTRAINT "Software on HYBRIDGENE_SOFTWARE"
                REFERENCES hybridgene_software,
  marker      SMALLINT NOT NULL
              CONSTRAINT "Marker on MARKERS" REFERENCES markers,
  comments    TEXT
              empty_string_check(`comments')
) WITHOUT OIDS;

grant_priv(`hybridgene_analyses')
grant_seq_priv(`hybridgene_analyses', `hgaid')


-- hybridgene_scores

CREATE TABLE hybridgene_scores (
  hgsid        SERIAL PRIMARY KEY NOT NULL,
  hgaid        INT NOT NULL
               CONSTRAINT "HGAId on HYBRIDGENE_ANALYSES"
                 REFERENCES hybridgene_analyses,
  sname_type_column(sname, Sname, NOT NULL),
  score        NUMERIC(8,7) NOT NULL
               CONSTRAINT "Score between Lower_Conf and Upper_Conf"
                 CHECK(score BETWEEN lower_conf AND upper_conf),
  lower_conf   NUMERIC(8,7) NOT NULL
               CONSTRAINT "Lower_Conf must be between 0 and 1"
                 CHECK(lower_conf BETWEEN 0 AND 1),
  upper_conf   NUMERIC(8,7) NOT NULL
               CONSTRAINT "Upper_Conf must be between 0 and 1"
                 CHECK(upper_conf BETWEEN 0 AND 1)

) WITHOUT OIDS;

grant_priv(`hybridgene_scores')
grant_seq_priv(`hybridgene_scores', `hgsid')


-- We keep the warning system in a separate file.
include(`makewarningtables.m4')
