dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2006, 2008, 2009, Karl O. Pinc
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Triggers for the ACTIVITIES table.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl


dnl plpgsql fragment that prevents a used code from dis-allowing the
dnl manner in which the code is already used. (Activities used in
dnl female point samples must allow such use, etc.)
dnl
dnl Syntax: check_usage(type, code)
dnl
dnl type   `juvenile' or `female', the point sample allowed to use the
dnl        activity.
dnl code   `bb_juvenile_point' or `bb_female_point', the code used in
dnl        SAMPLES to denote the type of sample.
dnl
changequote({,})
define({check_usage},{
    IF OLD.$1 AND OLD.$1 <> NEW.$1 THEN
      PERFORM 1
        FROM samples, point_data
        WHERE samples.stype = '$2'
              AND point_data.sid = samples.sid
              AND point_data.activity = NEW.activity
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION '%: Used in a $1 point sample activity -- cannot change'
                        , NEW.activity;
        RETURN NULL;
      END IF;
    END IF;
})dnl
changequote(`,')dnl

--  
-- activities
--
SELECT 'activities' AS table;

SELECT 'activities_func' AS function;
CREATE FUNCTION activities_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  BEGIN
  -- Function for activities insert and update triggers
  --
  -- GPL_notice(`  --', `2006, 2008, 2009', `Karl O. Pinc <kop@meme.com>')

  IF TG_OP = 'UPDATE' THEN
    -- If a code is used for female or juvenile point sampling,
    -- then we must continue to allow it's use.
    check_usage(`juvenile', `bb_juvenile_point')
    check_usage(`female', `bb_female_point')

    -- Cannot change the feeding code.
    restrict_change(`ACTIVITIES', `Activity', bb_feed_activity)

  END IF;

  RETURN NULL;
  END;
$$;


SELECT 'activities_trigger' AS trigger;
CREATE TRIGGER activities_trigger
  AFTER INSERT OR UPDATE
  ON activities FOR EACH ROW
  EXECUTE PROCEDURE activities_func();

