dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2007-2009, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl

--  
-- acts
--
SELECT 'acts' AS table;

SELECT 'acts_func' AS function;
CREATE FUNCTION acts_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  BEGIN
  -- Function for acts insert and update triggers
  --
  -- GPL_notice(`  --', `2007-2009', `Karl O. Pinc <kop@meme.com>')

  IF TG_OP = 'UPDATE' THEN

    -- Rows that represent an entire class (act = class) cannot have
    -- their act, class, or descr values changed.
    IF OLD.act = OLD.class
       AND (NEW.act <> OLD.act
            OR NEW.class <> OLD.class
            OR NEW.descr <> OLD.descr)
       AND NOT(has_table_privilege('acts', 'trigger')) THEN
      RAISE EXCEPTION 'ACTS: When Act = Class then the row has special meaning to Babase and Act, Class, and Descr may not be changed by ordinary users';
      RETURN NULL;
    END IF;

  END IF;

  -- Only one level of hierarchy is allowed.  The Class referred to
  -- must have it's Act = it's Class.

  IF NEW.act <> NEW.class THEN

    -- Make sure the child has a parent with Act = Class.
    PERFORM 1
      FROM acts
      WHERE act = NEW.class
            AND ACTS.act = ACTS.class
      LIMIT 1;
    IF NOT FOUND THEN
      RAISE EXCEPTION 'Bad Class % in Act %: The ACTS row referenced by ACTS.Class must have it''s Act value equal to it''s Class value'
                    , NEW.class
                    , NEW.act;
      RETURN NULL;
    END IF;

    -- Make sure that there are no children unless Act = Class.
    PERFORM 1
      FROM acts
      WHERE class = NEW.act
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION 'Act %: Cannot be referred to in any row''s ACTS.Class unless it has a Act value equal to it''s Class value'
                    , NEW.act;
      RETURN NULL;
    END IF;
  END IF;

  RETURN NULL;
  END;
$$;


SELECT 'acts_trigger' AS trigger;
CREATE TRIGGER acts_trigger
  AFTER INSERT OR UPDATE
  ON acts FOR EACH ROW
  EXECUTE PROCEDURE acts_func();
