dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl


dnl plpgsql fragment for BLAH
dnl
dnl Syntax: BLAH(ARG)
dnl
dnl ARG   description
dnl
changequote({,})
define({BLAH},{
})dnl
changequote(`,')dnl

--  
-- adcodes
--
SELECT 'adcodes' AS table;

SELECT 'adcodes_func' AS function;
CREATE FUNCTION adcodes_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  BEGIN
  -- Function for adcodes insert and update triggers
  --
  -- GPL_notice(`  --', `2012', `The Meme Factory, Inc., http://www.meme.com/')

  IF TG_OP = 'UPDATE' THEN
    cannot_change(`ADCODES', `ADCode')

    -- There are too many rules surrounding ascents and descents to
    -- make it worthwhile to check them all here.  Instead a new
    -- code will have to be created, used, and the old deleted and
    -- re-created and re-used, and the new code deleted.  It's not like
    -- this is going to happen much, if ever.
    cannot_change(`ADCODES', `ADN')

    IF OLD.time <> NEW.time THEN
      IF NEW.time THEN
        PERFORM 1
          FROM swerb_loc_data
          WHERE swerb_loc_data.time IS NULL
        LIMIT 1;

        IF FOUND THEN
          RAISE EXCEPTION 'ADcode %: Time %: Time cannot be set to TRUE because there are releated SWERB_LOC_DATA.time values that are NULL'
                        , NEW.adcode
                        , NEW.time;
          RETURN NULL;
        END IF;
      ELSE
        -- NEW.time is FALSE
        PERFORM 1
          FROM swerb_loc_data
          WHERE swerb_loc_data.time IS NOT NULL
        LIMIT 1;

        IF FOUND THEN
          RAISE EXCEPTION 'ADcode %: Time %: Time cannot be set to FALSE because there are releated SWERB_LOC_DATA.time values that are not NULL'
                        , NEW.adcode
                        , NEW.time;
          RETURN NULL;
        END IF;
      END IF;
    END IF;
  END IF;

  IF (NEW.adcode = 'bb_swerb_ascent_code'
      OR NEW.adcode = 'bb_swerb_descent_code')
     AND NOT(NEW.time) THEN
    -- A SWERB_LOC_DATA.ADTime value is required or else when the protocol
    -- requires 2 rows be entered for a sleeping rove and the second
    -- row is missing then the SWERB_UPLOAD program will not detect
    -- the problem.  This is because the SWERB_UPLOAD program uses
    -- these codes every time a begin or end row has an associated sleeping
    -- grove, temporarly, until it encounters the second waypoint
    -- line which has the time, or if not the time then the actual
    -- SWERB_LOC_DATA.ADCode to use.
    RAISE EXCEPTION 'ADCode %: The ''bb_swerb_ascent_code'' and ''bb_swerb_descent_code'' ADCODES.ADCode values must have a Time value of TRUE'
                  , NEW.adcode;
    RETURN NULL;
  END IF;

  RETURN NULL;
  END;
$$;


SELECT 'adcodes_trigger' AS trigger;
CREATE TRIGGER adcodes_trigger
  AFTER INSERT OR UPDATE
  ON adcodes FOR EACH ROW
  EXECUTE PROCEDURE adcodes_func();
