dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2007, 2008, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl

--  
-- allmiscs
--
SELECT 'allmiscs' AS table;

SELECT 'allmiscs_func' AS function;
CREATE FUNCTION allmiscs_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  DECLARE
    this_code TEXT;
    comma_pos INT;

  BEGIN
  -- Function for allmiscs insert and update triggers
  --
  -- GPL_notice(`  --', `2007, 2008', `Karl O. Pinc <kop@meme.com>')
  --
  -- We could do this with a constraint, but not as cleanly.

  -- Txt column

  comma_pos := position('bb_almtxt_delim' in NEW.txt);

  -- Make sure that Txt has a comma in it.
  IF comma_pos = 0 THEN
    RAISE EXCEPTION 'Almid %: ALLMISCS.Txt must contain a ''bb_almtxt_delim'' character to delimit the code: %'
      , NEW.almid
      , NEW.txt;
    RETURN NULL;
  END IF;

  -- Make sure that what's before the comma is one of the allowed codes.
  this_code := substring(NEW.txt from 1 for comma_pos - 1);

  IF this_code <> 'bb_almtxt_consortship'
     AND this_code <> 'bb_almtxt_unknown'
     AND this_code <> 'bb_almtxt_other' THEN
    RAISE EXCEPTION 'Almid %: ALLMISCS.Txt must begin with one of the following codes: bb_almtxt_consortship, bb_almtxt_unknown, bb_almtxt_other: %'
      , NEW.almid
      , NEw.txt;
    RETURN NULL;
  END IF;

  RETURN NULL;
  END;
$$;



SELECT 'allmiscs_trigger' AS trigger;
CREATE TRIGGER allmiscs_trigger
  AFTER INSERT OR UPDATE
  ON allmiscs FOR EACH ROW
  EXECUTE PROCEDURE allmiscs_func();
