dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl


dnl plpgsql fragment for BLAH
dnl
dnl Syntax: BLAH(ARG)
dnl
dnl ARG   description
dnl
changequote({,})
define({BLAH},{
})dnl
changequote(`,')dnl

--  
-- alternate_snames
--
SELECT 'alternate_snames' AS table;

SELECT 'alternate_snames_insert_func' AS function;
CREATE FUNCTION alternate_snames_insert_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  BEGIN
  -- Function for alternate_snames insert triggers
  --
  -- GPL_notice(`  --', `2012', `The Meme Factory, Inc., http://www.meme.com/')

    -- Update BIOGRAPH.Alt_Sname
    UPDATE biograph
      SET alt_snames = TRUE
      WHERE (sname = NEW.sname
             OR sname = NEW.sname_alternate)
            AND alt_snames = FALSE;

    RETURN NULL;
  END;
$$;

SELECT 'alternate_snames_update_func' AS function;
CREATE FUNCTION alternate_snames_update_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  BEGIN
  -- Function for alternate_snames update triggers
  --
  -- GPL_notice(`  --', `2012', `The Meme Factory, Inc., http://www.meme.com/')

    IF OLD.sname <> NEW.sname THEN
      -- See if the old sname still has alternates.
      PERFORM 1
        FROM alternate_snames
        WHERE alternate_snames.sname_alternate = OLD.sname
        LIMIT 1;
      IF NOT FOUND THEN
        -- The old sname has no alternates.
        UPDATE biograph
          SET alt_snames = FALSE
          WHERE sname = OLD.sname
                AND alt_snames = TRUE;
      END IF;

      -- The new sname is an alternate.
      UPDATE biograph
        SET alt_snames = TRUE
        WHERE sname = NEW.sname
              AND alt_snames = FALSE;
    END IF;

    IF OLD.sname_alternate <> NEW.sname_alternate THEN
      -- See if the old sname_alternate still has alternates.
      PERFORM 1
        FROM alternate_snames
        WHERE alternate_snames.sname = OLD.sname_alternate
              OR alternate_snames.sname_alternate = OLD.sname_alternate
        LIMIT 1;
      IF NOT FOUND THEN
        -- The old sname_alternate has no alternates.
        UPDATE biograph
          SET alt_snames = FALSE
          WHERE sname = OLD.sname_alternate
                AND alt_snames = TRUE;
      END IF;

      -- The new sname_alternate is an alternate.
      UPDATE biograph
        SET alt_snames = TRUE
        WHERE sname = NEW.sname_alternate
              AND alt_snames = FALSE;
    END IF;

    RETURN NULL;
  END;
$$;


SELECT 'alternate_snames_delete_func' AS function;
CREATE FUNCTION alternate_snames_delete_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- Function for alternate_snames delete trigger
  --
  -- GPL_notice(`  --', `2012', `The Meme Factory, Inc., http://www.meme.com/')

  DECLARE

  BEGIN

    -- Update biograph.alt_snames.

    -- Check sname.
    PERFORM 1
      FROM alternate_snames
      WHERE alternate_snames.sname_alternate = OLD.sname
      LIMIT 1;
    IF NOT FOUND THEN
      UPDATE biograph
        SET alt_snames = FALSE
        WHERE sname = OLD.sname
              AND alt_snames = TRUE;
    END IF;

    -- Check sname_alternate.
    PERFORM 1
      FROM alternate_snames
      WHERE alternate_snames.sname = OLD.sname_alternate
            OR alternate_snames.sname_alternate = sname_alternate
      LIMIT 1;
    IF NOT FOUND THEN
      UPDATE biograph
        SET alt_snames = FALSE
        WHERE sname = OLD.sname_alternate
              AND alt_snames = TRUE;
    END IF;

    RETURN NULL;
  END;
$$;


SELECT 'alternate_snames_insert_trigger' AS trigger;
CREATE TRIGGER alternate_snames_insert_trigger
  AFTER INSERT
  ON alternate_snames FOR EACH ROW
  EXECUTE PROCEDURE alternate_snames_insert_func();

SELECT 'alternate_snames_update_trigger' AS trigger;
CREATE TRIGGER alternate_snames_update_trigger
  AFTER UPDATE
  ON alternate_snames FOR EACH ROW
  EXECUTE PROCEDURE alternate_snames_update_func();

SELECT 'alternate_snames_delete_trigger' AS trigger;
CREATE TRIGGER alternate_snames_delete_trigger
  AFTER DELETE
  ON alternate_snames FOR EACH ROW
  EXECUTE PROCEDURE alternate_snames_delete_func();
