dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2010, 2011, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl


dnl plpgsql fragment for checking on update or delete that not having
dnl sedation at the OLD time does not mean that there's too long
dnl a period without sedation.  This checks rows other than the
dnl one updated/deleted.
dnl
dnl Syntax: _anesths_delete_check()
dnl
changequote({,})
define({_anesths_delete_check},{

  -- Antime cannot be more than bb_max_knockout_interval after the later of
  -- DARTINGS.Darttime and the prior ANESTHS.Antime for
  -- the darting.  Make sure that deleting an ANESTHS row did not leave
  -- later rows too far in the future.

  -- There must be later rows, and either an earlier row or a non-NULL
  -- DARTINGS.Darttime.
  PERFORM 1
    FROM anesths, dartings
    WHERE anesths.dartid = OLD.dartid
          AND anesths.antime > OLD.antime -- discards NULL antimes too
          -- No point in re-checking the same row on UPDATE.
          -- This test is not needed when deleting but does not hurt.
          AND anesths.anesthid <> OLD.anesthid
          -- And there must be an earlier row or a non-NULL Darttime
          -- against which we can validate.
          AND dartings.dartid = OLD.dartid
          AND (dartings.darttime IS NOT NULL
               OR EXISTS (SELECT 1
                            FROM anesths AS earlier
                            WHERE earlier.dartid = OLD.dartid
                                  AND earlier.antime < OLD.antime
                            LIMIT 1))
    LIMIT 1;
  IF FOUND THEN
    -- There are other ANESTHS rows for this darting after the deleted
    -- anesthetic administration.
    PERFORM 1
      FROM dartings, anesths
      WHERE dartings.dartid = OLD.dartid
            AND anesths.dartid = OLD.dartid
            AND anesths.antime > OLD.antime
            -- No point in re-checking the same row on UPDATE.
            -- This test is not needed when deleting but does not hurt.
            AND anesths.anesthid <> OLD.anesthid
            AND dartings.darttime
                    >= anesths.antime - 'bb_max_knockout_interval'::interval
      LIMIT 1;
    IF NOT FOUND THEN
      -- None of the later rows are within bb_max_knockout_interval
      -- of the darting itself.
      PERFORM 1
        FROM anesths
        WHERE anesths.anesthid <> OLD.anesthid
              AND anesths.dartid = OLD.dartid
              AND anesths.antime > OLD.antime
              AND anesths.antime
                  <= (SELECT last.antime
                             + 'bb_max_knockout_interval'::interval
                        FROM anesths AS last
                        WHERE last.dartid = OLD.dartid
                              AND last.antime < OLD.antime
                        ORDER BY last.antime DESC
                        LIMIT 1)
        LIMIT 1;
      IF NOT FOUND THEN
        -- There are no other anesthetic administrations within
        -- bb_max_knockout_interval of the administration of anesthetic
        -- before the one being deleted/updated.
        RAISE EXCEPTION 'ANESTHS.Anesthid %: ANESTHS.Dartid %: ANESTHS.Antime %: Anestheic administration must occur within bb_max_knockout_interval of the previous dose',
                        OLD.anesthid,
                        OLD.dartid,
                       OLD.antime;
        RETURN NULL;
      END IF;
    END IF;
  END IF;

  -- BTEMPS.Bttime cannot be more than bb_max_knockout_interval
  -- after the later of DARTINGS.Darttime and the prior
  -- ANESTHS.Antime for the darting.
  PERFORM 1
    FROM bodytemps, dartings
      WHERE dartings.dartid = OLD.dartid
            AND bodytemps.dartid = OLD.dartid
            AND bodytemps.bttime IS NOT NULL
            AND bodytemps.bttime
                  >= dartings.darttime
                     + 'bb_max_knockout_interval'::interval
            AND NOT EXISTS (SELECT 1
                              FROM anesths
                              WHERE anesths.dartid = OLD.dartid
                                    AND anesths.antime < bodytemps.bttime
                                    AND anesths.antime
                                         >= bodytemps.bttime
                                            - 'bb_max_knockout_interval'::interval                        LIMIT 1)
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION 'ANESTHS.Anesthid %: ANESTHS.Dartid %: ANESTHS.Antime %: Anestheic administration must occur within bb_max_knockout_interval of existant body temperature measurements (BODYTEMPS.Bttime)',
                      OLD.anesthid,
                      OLD.dartid,
                      OLD.antime;
      RETURN NULL;
    END IF;
})dnl
changequote(`,')dnl

--  
-- anesths
--
SELECT 'anesths' AS table;

SELECT 'anesths_func' AS function;
CREATE FUNCTION anesths_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  BEGIN
  -- Function for anesths insert and update triggers
  --
  -- GPL_notice(`  --', `2010, 2011', `Karl O. Pinc <kop@meme.com>')

  IF TG_OP = 'UPDATE' THEN
    -- Cannot change the Anesthid.
    cannot_change(`ANESTHS', `Anesthid')
    -- Cannot change the Dartid.
    cannot_change(`ANESTHS', `Dartid')
  END IF;

  -- Cannot administer anesthetic twice at one time.
  PERFORM 1
    FROM anesths
    WHERE anesths.dartid = NEW.dartid
          AND anesths.antime = NEW.antime
          AND anesths.anesthid <> NEW.anesthid
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'ANESTHS.Anesthid %: ANESTHS.Dartid %: ANESTHS.Antime %: The combination of Dartid and Antime must be unique',
                    NEW.anesthid,
                    NEW.dartid,
                    NEW.antime;
    RETURN NULL;
  END IF;

  -- Antime cannot be before DARTINGS.Darttime.
  check_keyed_ge(`ANESTHS'
                  , `Anesthid'
                  , `Antime'
                  , `DARTINGS'
                  , `Dartid'
                  , `Darttime')

  -- Antime cannot be more than bb_max_knockout_interval after the later of
  -- DARTINGS.Darttime and the prior ANESTHS.Antime for
  -- the darting.
  PERFORM 1
    FROM dartings
    WHERE dartings.dartid = NEW.dartid
          AND (dartings.darttime IS NULL
               OR dartings.darttime
                  >= NEW.antime - 'bb_max_knockout_interval'::interval)
    LIMIT 1;
  IF NOT FOUND THEN
    PERFORM 1
      FROM anesths
      WHERE anesths.anesthid <> NEW.anesthid
            AND anesths.dartid = NEW.dartid
            AND anesths.antime
                  >= NEW.antime - 'bb_max_knockout_interval'::interval
      LIMIT 1;
    IF NOT FOUND THEN
      RAISE EXCEPTION 'ANESTHS.Anesthid %: ANESTHS.Dartid %: ANESTHS.Antime %: Anestheic administration must occur within bb_max_knockout_interval of the previous dose',
                      NEW.anesthid,
                      NEW.dartid,
                      NEW.antime;
      RETURN NULL;
    END IF;
  END IF;

  IF TG_OP = 'UPDATE' THEN
    IF NEW.antime <> OLD.antime
       AND NOT(NEW.antime IS NULL AND OLD.antime IS NULL) THEN

      -- Check that changing the time has not left a gap in sedation
      -- that's too long.
      _anesths_delete_check()
    END IF;
  END IF;

  RETURN NULL;
  END;
$$;


SELECT 'anesths_delete_func' AS function;
CREATE FUNCTION anesths_delete_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- Function for anesths delete trigger
  --
  -- GPL_notice(`  --', `2010, 2011', `Karl O. Pinc <kop@meme.com>')

  DECLARE

  BEGIN

  _anesths_delete_check()

  RETURN NULL;
  END;
$$;


SELECT 'anesths_trigger' AS trigger;
CREATE TRIGGER anesths_trigger
  AFTER INSERT OR UPDATE
  ON anesths FOR EACH ROW
  EXECUTE PROCEDURE anesths_func();

SELECT 'anesths_delete_trigger' AS trigger;
CREATE TRIGGER anesths_delete_trigger
  AFTER DELETE
  ON anesths FOR EACH ROW
  EXECUTE PROCEDURE anesths_delete_func();
