dnl Copyright (C) 2016 Jake Gordon, <jacob.gordon@duke.edu>
dnl Copyright (C) 2016 The Meme Factory, Inc., http://www.meme.com/
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl

--
-- behave_gaps
--
SELECT 'behave_gaps' AS table;

SELECT 'behave_gaps_func' AS function;
CREATE FUNCTION behave_gaps_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  BEGIN
  -- Function for behave_gaps insert and update triggers
  --
  -- GPL_notice(`  --', `2016', `Jacob Gordon, <jacob.gordon@duke.edu>')

  -- Gap_start must be during the group's lifetime.

    -- Check if NEW.gap_start is before the group started
    PERFORM 1
      FROM groups
      WHERE groups.gid = NEW.grp
            AND groups.start > NEW.gap_start
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION integrity_constraint_violation USING
            MESSAGE = 'Error on ' || TG_OP || ' of BEHAVE_GAPS'
          , DETAIL = 'Key (BGId) = (' || NEW.bgid
                       || '): Value (Grp) = (' || NEW.grp
                       || '): Value (Gap_Start) = (' || NEW.gap_start
                       || '): Cannot start a gap for a group before its '
                       || 'GROUPS.Start';
      RETURN NULL;
    END IF;

    -- Check if NEW.gap_start is after the group ceases to exist.
    PERFORM 1
      FROM groups
      WHERE groups.gid = NEW.grp
            AND groups.cease_to_exist < NEW.gap_start
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION integrity_constraint_violation USING
            MESSAGE = 'Error on ' || TG_OP || ' of BEHAVE_GAPS'
          , DETAIL = 'Key (BGId) = (' || NEW.bgid
                       || '): Value (Grp) = (' || NEW.grp
                       || '): Value (Gap_Start) = (' || NEW.gap_start
                       || '): Cannot start a gap for a group after its '
                       || 'GROUPS.Cease_To_Exist';
      RETURN NULL;
    END IF;

  -- Gap_end must be during the group's lifetime, or NULL.

    -- NULL gap_end only allowed if group has not ceased to exist.
    IF NEW.gap_end IS NULL THEN
      PERFORM 1
        FROM groups
        WHERE groups.gid = NEW.grp
              AND groups.cease_to_exist IS NOT NULL
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION integrity_constraint_violation USING
              MESSAGE = 'Error on ' || TG_OP || ' of BEHAVE_GAPS'
            , DETAIL = 'Key (BGId) = (' || NEW.bgid
                         || '): Value (Grp) = (' || NEW.grp
                         || '): Cannot have a NULL Gap_End for a group when '
                         || 'its GROUPS.Cease_To_Exist is not NULL';
        RETURN NULL;
      END IF;

    ELSE
    -- Gap_end is not NULL, so validate it

      -- Check if gap_end is before the group started
      PERFORM 1
        FROM groups
        WHERE groups.gid = NEW.grp
              AND groups.start > NEW.gap_end
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION integrity_constraint_violation USING
              MESSAGE = 'Error on ' || TG_OP || ' of BEHAVE_GAPS'
            , DETAIL = 'Key (BGId) = (' || NEW.bgid
                         || '): Value (Grp) = (' || NEW.grp
                         || '): Value (Gap_End) = (' || NEW.gap_end
                         || '): Value (Gap_End_Status) = (' || NEW.gap_end_status
                         || '): Cannot end a gap for a group before its '
                         || 'GROUPS.Start';
        RETURN NULL;
      END IF;

      -- Check if gap_end is after the group ceases to exist
      PERFORM 1
        FROM groups
        WHERE groups.gid = NEW.grp
              AND groups.cease_to_exist < NEW.gap_end
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION integrity_constraint_violation USING
              MESSAGE = 'Error on ' || TG_OP || ' of BEHAVE_GAPS'
            , DETAIL = 'Key (BGId) = (' || NEW.bgid
                         || '): Value (Grp) = (' || NEW.grp
                         || '): Value (Gap_End) = (' || NEW.gap_end
                         || '): Cannot end a gap for a group after its '
                         || 'GROUPS.Cease_To_Exist';
        RETURN NULL;
      END IF;
    END IF;

  RETURN NULL;
  END;
$$;

SELECT 'behave_gaps_trigger' AS trigger;
CREATE TRIGGER behave_gaps_trigger
  AFTER INSERT OR UPDATE
  ON behave_gaps FOR EACH ROW
  EXECUTE PROCEDURE behave_gaps_func();
