dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2010, 2011, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl


dnl plpgsql fragment for BLAH
dnl
dnl Syntax: BLAH(ARG)
dnl
dnl ARG   description
dnl
changequote({,})
define({BLAH},{
})dnl
changequote(`,')dnl

--  
-- bodytemps
--
SELECT 'bodytemps' AS table;

SELECT 'bodytemps_func' AS function;
CREATE FUNCTION bodytemps_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  BEGIN
  -- Function for bodytemps insert and update triggers
  --
  -- GPL_notice(`  --', `2010, 2011', `Karl O. Pinc <kop@meme.com>')

  IF TG_OP = 'UPDATE' THEN
    -- Cannot change the Btid.
    cannot_change(`BODYTEMPS', `Btid')
    -- Cannot change the Dartid.
    cannot_change(`BODYTEMPS', `Dartid')
  END IF;

  -- Bttime cannot be before DARTINGS.Pickuptime.
  check_keyed_ge(`BODYTEMPS'
                 , `Btid'
                 , `Bttime'
                 , `DARTINGS'
                 , `Dartid'
                 , `Pickuptime')

  -- Bttime cannot be before DARTINGS.Darttime.
  check_keyed_ge(`BODYTEMPS'
                 , `Btid'
                 , `Bttime'
                 , `DARTINGS'
                 , `Dartid'
                 , `Darttime')

  IF NEW.bttime IS NOT NULL THEN
    -- Bttime cannot be more than bb_max_knockout_interval after the
    -- later of DARTINGS.Darttime and the last administration of extra
    -- sedation.  We could use another constant to control
    -- this but the anesthetic time seems to be the controlling
    -- variable.
    PERFORM 1
      FROM dartings
      WHERE dartings.dartid = NEW.dartid
            AND dartings.darttime
                    >= NEW.bttime - 'bb_max_knockout_interval'::interval
      LIMIT 1;
    IF NOT FOUND THEN
      PERFORM 1
        FROM anesths
        WHERE anesths.dartid = NEW.dartid
              AND anesths.antime
                    >= NEW.bttime - 'bb_max_knockout_interval'::interval
        LIMIT 1;
      IF NOT FOUND THEN
        RAISE EXCEPTION 'BODYTEMPS.Btid %: BODYTEMPS.Dartid %: BODYTEMPS.Bttime %: Body temperature measurement must occur within bb_max_knockout_interval of the last dose of anesthetic',
                        NEW.btid,
                        NEW.dartid,
                        NEW.bttime;
        RETURN NULL;
      END IF;
    END IF;
  END IF;

  RETURN NULL;
  END;
$$;


SELECT 'bodytemps_trigger' AS trigger;
CREATE TRIGGER bodytemps_trigger
  AFTER INSERT OR UPDATE
  ON bodytemps FOR EACH ROW
  EXECUTE PROCEDURE bodytemps_func();
