dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2009, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl

dnl plpgsql fragment for checking that an sname column relates to a male
dnl
dnl Syntax: _check_male(col)
dnl
dnl  col    The name of the column to check
dnl
changequote({,})
define({_check_male},{
  -- $1 must be a male
  PERFORM 1
    FROM biograph
    WHERE biograph.sname = NEW.$1
    LIMIT 1;
  IF NOT FOUND THEN
    RAISE EXCEPTION 'Mpiid %: CONSORTS.$1 %: is not a male'
                    , NEW.mpiid
                    , NEW.$1;
    RETURN NULL;
  END IF;          
})dnl
changequote(`,')dnl

--  
-- consorts
--
SELECT 'consorts' AS table;

SELECT 'consorts_func' AS function;
CREATE FUNCTION consorts_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  BEGIN
  -- Function for consorts insert and update triggers
  --
  -- GPL_notice(`  --', `2009', `Karl O. Pinc <kop@meme.com>')

  IF TG_OP = 'UPDATE' THEN
    cannot_change(`CONSORTS', `Mpiid')
  END IF;

  -- MPIS.Context_type must be a consort context type
  PERFORM 1
    FROM mpis
    WHERE mpis.mpiid = NEW.mpiid
          AND mpis.context_type <> 'bb_consort_context'
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'Mpiid %: A MPIS row related to a CONSORTS row must have a Context_type of bb_consort_context',
                    NEW.mpiid;
    RETURN NULL;
  END IF;

  -- Both Had and Got must be male.

  _check_male(`Had')

  _check_male(`Got')

  -- -- Had must be a participant in the initial agonism.
  -- PERFORM 1
  --   FROM mpi_data, mpi_parts
  --   WHERE mpi_data.mpiid = NEW.mpiid
  --         AND mpi_data_seq = 1
  --         AND mpi_parts.mpidid = mpi_data.mpidid
  --         AND mpi_parts.sname = NEW.Had
  --   LIMIT 1;
  -- IF NOT FOUND THEN
  --   RAISE EXCEPTION 'Mpiid %: CONSORTS.Had (%) must be a participant in the interaction starting (MPI_DATA.Seq = 1) the MPI'
  --                   , NEW.mpiid
  --                   , NEW.had;
  --   RETURN NULL;
  -- END IF;

  RETURN NULL;
  END;
$$;


SELECT 'consorts_trigger' AS trigger;
CREATE TRIGGER consorts_trigger
  AFTER INSERT OR UPDATE
  ON consorts FOR EACH ROW
  EXECUTE PROCEDURE consorts_func();
