dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2009, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl

--  
-- context_types
--
SELECT 'context_types' AS table;

SELECT 'context_types_func' AS function;
CREATE FUNCTION context_types_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  BEGIN
  -- Function for context_types insert and update triggers
  --
  -- GPL_notice(`  --', `2009', `Karl O. Pinc <kop@meme.com>')

  IF TG_OP = 'UPDATE' THEN
    -- Cannot change the 'no context' code.
    restrict_change(`CONTEXT_TYPES', `Context_type', bb_no_context)

    -- Cannot change the 'Consortship' code.
    restrict_change(`CONTEXT_TYPES', `Context_type', bb_consort_context)
  END IF;

  RETURN NULL;
  END;
$$;


SELECT 'context_types_delete_func' AS function;
CREATE FUNCTION context_types_delete_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  -- Function for context_types delete trigger
  --
  -- GPL_notice(`  --', `2009', `Karl O. Pinc <kop@meme.com>')

  DECLARE

  BEGIN

  -- Cannot delete the 'no context' value.
  restrict_delete(`CONTEXT_TYPES', `Context_type', bb_no_context)

  -- Cannot delete the consortship value.
  restrict_delete(`CONTEXT_TYPES', `Context_type', bb_consort_context)

  RETURN NULL;
  END;
$$;


SELECT 'context_types_trigger' AS trigger;
CREATE TRIGGER context_types_trigger
  AFTER INSERT OR UPDATE
  ON context_types FOR EACH ROW
  EXECUTE PROCEDURE context_types_func();

SELECT 'context_types_delete_trigger' AS trigger;
CREATE TRIGGER context_types_delete_trigger
  AFTER DELETE
  ON context_types FOR EACH ROW
  EXECUTE PROCEDURE context_types_delete_func();
