dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2010, 2011, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl
include(`seqmacros.m4')dnl

--  
-- crownrumps
--
SELECT 'crownrumps' AS table;

SELECT 'crownrumps_before_insert_func' AS function;
CREATE FUNCTION crownrumps_before_insert_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  BEGIN
  -- Function for crownrumps before insert trigger
  --
  -- GPL_notice(`  --', `2010', `Karl O. Pinc <kop@meme.com>')
  --
  -- Remarks:
  -- WARNING: See remarks in crownrumps_func().

    _init_seq_value(`CROWNRUMPS', `Dartid', `CRseq')

    RETURN NEW;
  END;
$$;

SELECT 'crownrumps_func' AS function;
CREATE FUNCTION crownrumps_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  BEGIN
  -- Function for crownrumps insert and update triggers
  --
  -- GPL_notice(`  --', `2010', `Karl O. Pinc <kop@meme.com>')
  --
  -- Remarks:

    _seq_insert_update_func(`CROWNRUMPS', `CRid', `Dartid', `CRseq')

  RETURN NULL;
  END;
$$;


SELECT 'crownrumps_delete_func' AS function;
CREATE FUNCTION crownrumps_delete_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- Function for crownrumps delete trigger
  --
  -- GPL_notice(`  --', `2010, 2011', `Karl O. Pinc <kop@meme.com>')
  --
  -- Remarks:
  -- WARNING: See remarks in crownrumps_func().

  BEGIN

    _fill_seq_gap(`CROWNRUMPS', `CRid', `Dartid', `CRseq')

  RETURN NULL;
  END;
$$;

SELECT 'crownrumps_before_insert_trigger' AS trigger;
CREATE TRIGGER crownrumps_before_insert_trigger
  BEFORE INSERT
  ON crownrumps FOR EACH ROW
  EXECUTE PROCEDURE crownrumps_before_insert_func();

SELECT 'crownrumps_trigger' AS trigger;
CREATE TRIGGER crownrumps_trigger
  AFTER INSERT OR UPDATE
  ON crownrumps FOR EACH ROW
  EXECUTE PROCEDURE crownrumps_func();

SELECT 'crownrumps_delete_trigger' AS trigger;
CREATE TRIGGER crownrumps_delete_trigger
  AFTER DELETE
  ON crownrumps FOR EACH ROW
  EXECUTE PROCEDURE crownrumps_delete_func();
