dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2005, 2006, 2008, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl
include(`func_macros.m4')dnl

dnl plpgsql fragment for BLAH
dnl
dnl Syntax: BLAH(ARG)
dnl
dnl ARG   description
dnl
changequote({,})
define({BLAH},{
})dnl
changequote(`,')dnl



dnl plpgsql fragment for rebuilding automatic Mdates surrounding a row.
dnl
dnl Syntax: _cg_rebuild_autom(rowtype, exitlabel)
dnl
dnl rowtype   A row (NEW or OLD).
dnl exitlabel A statement label to go to when we're done.
dnl
dnl Requires:
dnl  The variables start_range and stop_range be delcared.
dnl
dnl Bugs:
dnl  Something of an over-optimization that the block with the EXIT
dnl label target and the start_range and stop_range variables are
dnl not part of the macro, but it worked out that way as I wrote it
dnl so I'm leaving it.
dnl
changequote({,})
define({_cg_rebuild_autom},{
      --
      -- Rebuild the automatic Mdates surrounding the $1 row.
      --

      -- Find the preceeding start/end of observation.
      SELECT INTO start_range cycgaps.date - bb_autom_interval
        FROM cycgaps
        WHERE cycgaps.sname = $1.sname
              AND cycgaps.date < $1.date
              AND cycgaps.code <> 'bb_cg_point'
        ORDER BY cycgaps.date DESC
        LIMIT 1;
      IF NOT FOUND THEN
        -- Go all the way back to the maturity date.
        SELECT INTO start_range maturedates.matured
          FROM maturedates
          WHERE sname = $1.sname;

        IF NOT FOUND THEN
          -- No maturity date, so no sexual cycle data to worry about
          -- the automatic Mdates over.
          EXIT $2;
        END IF;
      END IF;

      -- Find the following start/end of observation.
      SELECT INTO stop_range cycgaps.date
        FROM cycgaps
        WHERE cycgaps.sname = $1.sname
              AND cycgaps.date > $1.date
              AND cycgaps.code <> 'bb_cg_point'
        ORDER BY cycgaps.date
        LIMIT 1;

      IF NOT FOUND THEN
        -- Use the statdate.
        SELECT INTO stop_range biograph.statdate
          FROM biograph
          WHERE biograph.sname = $1.sname;
      END IF;

      -- Update the Ddates so that all the automatic Mdates are re-generated.
      -- (It's a Miracle!)
      UPDATE cycpoints
        SET cpid = cycpoints.cpid
        FROM cycles
        WHERE cycles.sname = $1.sname
              AND cycpoints.cid = cycles.cid
              AND cycpoints.date >= start_range
              AND cycpoints.date <= stop_range
              AND cycpoints.code = 'bb_deturgesence';
})dnl
changequote(`,')dnl




dnl plpgsql fragment for raising a CYCGAPS.State/CYCPOINTS error message
dnl
dnl Syntax: state_error(correct_state)
dnl
dnl correct_state   The state the cycgaps row should have.
dnl
changequote({,})
define({state_error},{
              RAISE EXCEPTION '%: %: Code %: State %: CYCGAPS.State should be $1, does not correspond with CYCPOINTS'
                            , NEW.sname
                            , NEW.date
                            , NEW.code
                            , NEW.state;
              RETURN NULL;
})dnl
changequote(`,')dnl



dnl plpgsql fragment to see if there's CYCPOINTS in the gaps.
dnl
dnl Syntax: check_gaps(newold)
dnl
dnl newold   NEW or OLD, where to get the sname from.
dnl
dnl Should only be invoked after cycgapdays is updated.
dnl
changequote({,})
define({check_gaps},{
  --
  -- Now that we've got CYCGAPDAYS we can check to see that
  -- there's no CYCPOINTS occuring during the gaps.
  -- (Except estimated Ddates (that are there to support pregnancy.))
  --
  PERFORM 1
    FROM cycpoints, cycles, cycgapdays
    WHERE cycgapdays.sname = $1.sname
          AND cycles.sname = $1.sname
          AND cycpoints.date = cycgapdays.date
          AND cycpoints.cid = cycles.cid
          AND NOT(cycpoints.code = 'bb_deturgesence'
                  AND cycpoints.source = 'bb_estimated')
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION '%: %: %: Cannot have CYCPOINTS (except estimated Ddates) when there''s no observation'
                  , $1.sname
                  , $1.date
                  , $1.code;
    RETURN NULL;
  END IF;
})dnl
changequote(`,')dnl

--  
-- cycgaps
--
SELECT 'cycgaps' AS table;

-----------------------------------------------------------------
SELECT 'cycgaps_func' AS function;
CREATE FUNCTION cycgaps_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  BEGIN
  -- Function for cycgaps insert and update triggers
  --
  -- GPL_notice(`  --', `2005, 2006, 2008', `Karl O. Pinc <kop@meme.com>')
  --
  -- Remarks:
  --  This routine is a bit too tricksy for my taste, with all the
  -- self-updating going on.
  --
  --  I'm a bit worried about what happens if a cycgaps.code changes,
  -- but I don't want to dis-allow changes entirely as we rely on them
  -- to do some of the tricksey things.
  --
  --  An alternative to all of this sneaky stuff with automatic
  -- alteration is to force the user to insert multiple rows at a
  -- time.  Maybe a function or something would make this easier.
  --
  -- No doubt we're doing the same query more than once in this
  -- code but it does not get run all that often and I've not
  -- optimzed it.

--RAISE INFO 'cycgaps_func NEW: %', NEW;
--IF TG_OP = 'UPDATE' THEN
--  RAISE INFO '    cycgaps_func OLD: %', OLD;
--END IF;

  --
  -- Consistency checks.
  --

  -- To keep things painless, don't let the sname change.
  IF TG_OP = 'UPDATE' THEN
    cannot_change(`CYCGAPS', `Sname')
  END IF;

  -- No need to check a female's first cycgaps row is a end of
  -- observation, the other rules ensure this.

  -- End may follow start, but otherwise two cycgaps can't be
  -- 'daywise neighbors'.
  -- (Don't check for same day here so we can handle speical case
  -- below.)
  DECLARE
    day_before cycgaps.code%TYPE;
    day_after cycgaps.code%TYPE;

  BEGIN
    SELECT INTO day_before code
      FROM cycgaps
      WHERE sname = NEW.sname
            AND date = NEW.date - CAST('1 day' AS interval);
    SELECT INTO day_after code
      FROM cycgaps
      WHERE sname = NEW.sname
            AND date = NEW.date + CAST('1 day' AS interval);

    IF NEW.code = 'bb_cg_end'
       AND day_before = 'bb_cg_point' THEN
      -- Transform the point into a start.
      UPDATE cycgaps
        SET code = 'bb_cg_start'
        WHERE sname = NEW.sname
              AND date = NEW.date - CAST('1 day' AS interval)
              AND code <> 'bb_cg_start';  -- avoid infinite loop
      day_before := 'bb_cg_start';  -- just in case
    ELSE
      IF day_before IS NOT NULL
         AND NOT(day_before = 'bb_cg_start' AND NEW.code = 'bb_cg_end') THEN
        RAISE EXCEPTION '%: %: %: Except for ends following starts, no two of a female''s cycgaps rows may be on successive days'
                        , NEW.sname
                        , NEW.date
                        , NEW.code;
        RETURN NULL;
      END IF;
    END IF;

    IF day_after IS NOT NULL
       AND NOT(NEW.code = 'bb_cg_start' AND day_after = 'bb_cg_end') THEN
      RAISE EXCEPTION '%: %: %: Except for starts following ends, no two of a female''s cycgaps rows may be one day apart'
                      , NEW.sname
                      , NEW.date
                      , NEW.code;
      RETURN NULL;
    END IF;
  END;


  DECLARE
    prior_start cycgaps.date%TYPE;
    prior_end cycgaps.date%TYPE;
    prior_point cycgaps.date%TYPE;
    next_start cycgaps.date%TYPE;
    next_end cycgaps.date%TYPE;
    next_point cycgaps.date%TYPE;

    made_new_gap BOOLEAN;

  BEGIN
    -- The prior rows.
    SELECT INTO prior_start date
      FROM cycgaps
      WHERE sname = NEW.sname
            AND code = 'bb_cg_start'
            AND date < NEW.date
      ORDER BY date DESC
      LIMIT 1;
    SELECT INTO prior_end date
      FROM cycgaps
      WHERE sname = NEW.sname
            AND code = 'bb_cg_end'
            AND date < NEW.date
      ORDER BY date DESC
      LIMIT 1;
    SELECT INTO prior_point date
      FROM cycgaps
      WHERE sname = NEW.sname
            AND code = 'bb_cg_point'
            AND date < NEW.date
      ORDER BY date DESC
      LIMIT 1;
    -- The subsequent rows.
    SELECT INTO next_start date
      FROM cycgaps
      WHERE sname = NEW.sname
            AND code = 'bb_cg_start'
            AND date > NEW.date
      ORDER BY date
      LIMIT 1;
    SELECT INTO next_end date
      FROM cycgaps
      WHERE sname = NEW.sname
            AND code = 'bb_cg_end'
            AND date > NEW.date
      ORDER BY date
      LIMIT 1;
    SELECT INTO next_point date
      FROM cycgaps
      WHERE sname = NEW.sname
            AND code = 'bb_cg_point'
            AND date > NEW.date
      ORDER BY date
      LIMIT 1;

    --
    -- Can't have a start of observation be first, or follow another
    -- start of observation without an intervening end of observation.
    -- If we see such a thing, insert a end of observation 2 days before.
    --
    made_new_gap := FALSE;
    IF NEW.code = 'bb_cg_start'
       AND (prior_start > prior_end       -- Two starts in a row
            OR prior_end IS NULL)         -- No prior end
       AND NOT EXISTS (SELECT 1           -- Putting start on end to get rid
                         FROM cycgaps     -- of them both.
                         WHERE cycgaps.sname = NEW.sname
                               AND cycgaps.date = NEW.date
                               AND cycgaps.code = 'bb_cg_end') THEN
      -- Inserting starts into an observation period
      -- means create a new gap.
      IF prior_end + CAST('1 days' AS interval) >= NEW.date THEN
        -- In theory, this gets us a nicer error message than we'd
        -- get if the new, automatic, end-of-obs row bumped up too close to
        -- an earlier prior start.
        RAISE EXCEPTION 'Sname %: Start %: Cannot create a new gap so close to the prior gap'
                        , NEW.sname
                        , NEW.date;
        RETURN NULL;
      END IF;

      -- Go ahead and create a gap.
      INSERT INTO cycgaps (sname, code, date)
                  values (NEW.sname
                          , 'bb_cg_end'
                          , NEW.date - CAST('2 days' AS interval));
      made_new_gap := TRUE;
      -- The new insert will trigger data updates elsewhere.
      -- Keep going so as to keep testing for problems & update other
      -- tables.  (We do redundant updates on CYCGAPDAYS, too bad.)
    END IF;

    -- Put a start of observation on top of a stop of observation
    -- and they both go away.  (But in different ways depending on
    -- whether they represent a gap or a period of observation.)
    IF TG_OP = 'UPDATE' 
       AND NEW.code = 'bb_cg_start' THEN
      PERFORM 1 FROM cycgaps
        WHERE sname = NEW.sname
              AND date = NEW.date
              AND code = 'bb_cg_end';
      IF FOUND THEN
        -- Now check where the start used to be to make sure it's
        -- still ok.
        IF NOT(_cycgaps_delete_ok(OLD, TRUE)) THEN
          -- Something's wrong where the OLD row was.
          RETURN NULL;
        END IF;

        -- Presumeably, this will update and test as necessary.
        IF OLD.date < NEW.date THEN
          -- Just shrunk a period of observation down to 1 day.
          -- Turn it into a point observation (without ever
          -- leaving the day in a 'not observed' condition.)
          -- (This will delete the old start/end pair and _then_
          -- check for conflicts.)
          INSERT
            INTO cycgaps (sname, code, date, state)
            VALUES (NEW.sname, 'bb_cg_point', NEW.date, NEW.state);
        ELSE
          -- Shrunk a period of no observation down to 0 days.
          DELETE FROM cycgaps
                 WHERE sname = NEW.sname
                       AND date = NEW.date;
          -- The delete trigger does all the checking needed, on the
          -- NEW side.
        END IF;

        -- Make sure cycgapdays is right.
        IF OLD.date <> NEW.date
           OR OLD.code <> NEW.code THEN
          PERFORM _cycgapdays_interval_rebuild(OLD);
        END IF;

        -- Make sure automatic Mdates are right.
        <<assign_old_autom>>
        DECLARE
          start_range cycgaps.date%TYPE;
          stop_range cycgaps.date%TYPE;

        BEGIN
          -- Rebuild the automatic Mdates surrounding the old row.
          _cg_rebuild_autom(`OLD', `assign_old_autom')
        END;

        -- Check for cycpoints during gaps.
--raise info 'first';
        check_gaps(`OLD')

        -- Everything else should be checked for and adjusted by
        -- the triggers that ran when we altered the database.
        RETURN NULL;
      END IF;
    END IF;

    -- One female cannot have two cycgaps on the same day.
    PERFORM 1 FROM cycgaps
      WHERE sname = NEW.sname
            AND date = NEW.date
            AND gapid <> NEW.gapid
      LIMIT 1;
    IF FOUND THEN
      PERFORM 1
        FROM cycgaps AS p
           , cycgaps AS s
           , cycgaps AS e
        WHERE TG_OP = 'INSERT'
              AND NEW.code = 'bb_cg_point'
              AND p.gapid = NEW.gapid
              AND s.sname = NEW.sname
              AND s.date = NEW.date
              AND s.code = 'bb_cg_start'
              AND e.sname = NEW.sname
              AND e.date = NEW.date
              AND e.code = 'bb_cg_end'
              AND NOT EXISTS (SELECT 1  -- and nothing else
                                FROM cycgaps
                                WHERE sname = NEW.sname
                                      AND date = NEW.date
                                      AND gapid <> NEW.gapid
                                      AND gapid <> s.gapid
                                      AND gapid <> e.gapid)
        LIMIT 1;
      IF FOUND THEN
        -- Just inserted a point to replace a start observation 
        -- moved (forward) onto an end of observation.

        -- Make sure that there's not other cycpoints on the same
        -- day, in which case we don't know what's going on.
        IF (SELECT count(*)
              FROM cycgaps
              WHERE cycgaps.sname = NEW.sname
                    AND cycgaps.date = NEW.date) > 3 THEN
          RAISE EXCEPTION '%: %: %: Individual has too many CYCGAPS rows on the sname date'
                          , NEW.sname
                          , NEW.date
                          , NEW.code;
          RETURN NULL;
        END IF;

        -- Remove the old start/end pair.
        DELETE
          FROM cycgaps
          WHERE sname = NEW.sname
                AND date = NEW.date 
                AND gapid <> NEW.gapid;
      ELSE
        RAISE EXCEPTION '%: %: %: A female cannot have two CYCGAPS rows for the same day'
                        , NEW.sname
                        , NEW.date
                        , NEW.code;
        RETURN NULL;
      END IF;
    END IF;

    --
    -- Look for points in the wrong place.
    --
    IF NEW.code = 'bb_cg_point' THEN
      IF prior_start > prior_end -- (Note this comparison accounts for NULL)
         OR (prior_start IS NOT NULL AND prior_end IS NULL) THEN
        RAISE EXCEPTION '%: %: %: Insert of point CYCGAPS into period of observation'
                        , NEW.sname
                        , NEW.date
                        , NEW.code;
        RETURN NULL;
      END IF;
    ELSEIF (NEW.code = 'bb_cg_start'
            AND (next_point < next_end
                 OR (next_end IS NULL AND next_point IS NOT NULL)))
           OR (NEW.code = 'bb_cg_end'
               AND (prior_start < prior_point
                    OR (prior_start IS NULL AND prior_point IS NOT NULL))) THEN
      RAISE EXCEPTION '%: %: %: Point CYCGAPS fall into period of observation'
                      , NEW.sname
                      , NEW.date
                      , NEW.code;
      RETURN NULL;
    END IF;

    --
    -- Look for two starts in a row.
    --
    IF NEW.code = 'bb_cg_start'
       AND ((prior_end < prior_start       -- New start is coming after a start
             AND NOT made_new_gap)         -- Special case for inserting new gap
            OR (next_start < next_end      -- New start is coming before a start
                OR (next_start IS NOT NULL AND next_end IS NULL))) THEN
      RAISE EXCEPTION '%: %: %: Can''t have successive start of observations without an intervening end of observation'
                        , NEW.sname
                        , NEW.date
                        , NEW.code;
      RETURN NULL;
    END IF;

    --
    -- Look for two ends in a row.
    --
    IF NEW.code = 'bb_cg_end'
       AND ((prior_start < prior_end       -- New end is coming after an end
             OR (prior_start IS NULL AND prior_end IS NOT NULL))
            OR (next_end < next_start      -- New end is coming before a end
                OR (next_start IS NULL AND next_end IS NOT NULL))) THEN
      RAISE EXCEPTION '%: %: %: Can''t have successive end of observations without an intervening start of observation'
                        , NEW.sname
                        , NEW.date
                        , NEW.code;
      RETURN NULL;
    END IF;

  END;


  IF TG_OP = 'UPDATE' THEN
    -- Test where the row used to be to be sure things are still good.
    IF NOT(_cycgaps_delete_ok(OLD, FALSE)) THEN
       -- It raises exceptions for us.
       RETURN NULL;
    END IF;
  END IF;

  --
  -- Done with most checks, update CYCGAPDAYS.
  --
  IF TG_OP = 'UPDATE' THEN
    IF OLD.date <> NEW.date
       OR OLD.code <> NEW.code THEN
      -- (Possible redundant work here.  Too bad.)
      PERFORM _cycgapdays_interval_rebuild(OLD);
      PERFORM _cycgapdays_interval_rebuild(NEW);
    END IF;
  ELSE -- TG_OP = 'INSERT'
    PERFORM _cycgapdays_interval_rebuild(NEW);
  END IF;

  --
  -- Now that we've updated cycgapdays we can (easily) update the
  -- automatic Mdates just by doing an update operation on the Ddates
  -- and not changning anything.
  --
  -- Follow the usual approach.  Find the range of those dates that
  -- might be affected surrounding both the OLD and NEW rows and
  -- rebuild all the automatic Mdates in both those intervals.
  -- What might be affected is any date bb_autom_interval days
  -- before a preceeding start or end of observation through
  -- a following start or end of observation.

  <<assign_autom>>
  DECLARE
    start_range cycgaps.date%TYPE;
    stop_range cycgaps.date%TYPE;

  BEGIN
    IF TG_OP = 'UPDATE' THEN
      -- Rebuild the automatic Mdates surrounding the old row.
      _cg_rebuild_autom(`OLD', `assign_autom')

    END IF; -- TG_OP = 'UPDATE'

    -- Rebuild the automatic Mdates surrounding the new row.
    _cg_rebuild_autom(`NEW', `assign_autom')
  END;

  --
  -- Check for cycpoints during gaps.
  -- It's important to do this after updating the automatic Mdates
  -- because the gap checking code also checks automatic Mdates.
  -- (Although it wouldn't be hard to _not_ check automatic Mdates,
  -- why would you not want to check them?
  --
--raise info 'second';
  check_gaps(`NEW')
--raise info 'third';
  IF TG_OP = 'UPDATE' THEN
    check_gaps(`OLD')
  END IF;


  --
  -- Check state against CYCPOINTS.
  -- (Do this here so we can use cycgapdays.)
  --
  IF NEW.code <> 'bb_cg_end' THEN
    DECLARE
      next_code cycpoints.code%TYPE;
      next_date cycpoints.date%TYPE;
      next_cid cycpoints.cid%TYPE;

    BEGIN
      -- (Special order by here for sharing of M and Tdates.)
      IF NEW.code = 'bb_cg_start' THEN
        SELECT INTO next_code,      next_date,      next_cid
                    cycpoints.code, cycpoints.date, cycpoints.cid
          FROM cycles
            JOIN cycpoints ON (cycpoints.cid = cycles.cid)
            LEFT OUTER JOIN cycgaps ON (cycpoints.date <= cycgaps.date
                                        AND cycgaps.sname = NEW.sname
                                        AND cycgaps.date > NEW.date
                                        AND cycgaps.code = 'bb_cg_end')
          WHERE cycles.sname = NEW.sname
                AND cycpoints.date >= NEW.date
                -- AND cycpoints.code <> 'bb_bleeding'
          ORDER BY cycpoints.date, cycpoints.code 
          LIMIT 1;

      ELSEIF NEW.code = 'bb_cg_point' THEN
        SELECT INTO next_code,      next_cid
                    cycpoints.code, cycpoints.cid
          FROM cycles, cycpoints
          WHERE cycles.sname = NEW.sname
                AND cycpoints.cid = cycles.cid
                AND cycpoints.date = NEW.date
                AND cycpoints.code <> 'bb_bleeding'
          ORDER BY cycpoints.code DESC
          LIMIT 1;
        next_date := NEW.date;
      END IF;

      IF FOUND THEN
        -- There's a cycpoints to worry about.
        IF next_date = NEW.date THEN
          --
          -- The cycpoints falls on the day observation starts.
          --
          IF next_code = 'bb_menses' THEN
            IF NEW.state <> 'bb_cg_menses' THEN
              state_error(`bb_cg_menses')
            END IF;

          ELSEIF next_code = 'bb_deturgesence' THEN
            IF NEW.state <> 'bb_cg_deturgesence' THEN
              state_error(`bb_cg_deturgesence')
            END IF;

          ELSE -- next_code = 'bb_turgesence'
            -- Find the subsequent Ddate.            
            DECLARE
              ddate cycpoints.date%TYPE;

            BEGIN
              SELECT INTO ddate
                          cycpoints.date
                FROM cycpoints
                WHERE cycpoints.cid = next_cid
                      AND cycpoints.code = 'bb_deturgesence';
              IF ddate <= next_date
                          + CAST('bb_ovulation_len days' AS INTERVAL)
                 OR ddate IS NULL THEN
                IF NEW.state <> 'bb_cg_ovulating' THEN
                  state_error(`bb_cg_ovulating')
                END IF;
              ELSE
                IF NEW.state <> 'bb_cg_swelling' THEN
                  state_error(`bb_cg_swelling')
                END IF;
              END IF;
            END;
          END IF;

        ELSE
          --
          -- The next cycgaps falls after the start of observation date.
          --
          IF next_code = 'bb_menses' THEN
            IF NEW.state <> 'bb_cg_deturgesence' THEN
              state_error(`bb_cg_deturgesence')
            END IF;

          ELSEIF next_code = 'bb_deturgesence' THEN
            IF next_date > NEW.date
                           + CAST('bb_ovulation_len days' AS INTERVAL) THEN
              IF NEW.state <> 'bb_cg_swelling' THEN
                state_error(`bb_cg_swelling')
              END IF;
            ELSE
              IF NEW.state <> 'bb_cg_ovulating' THEN
                state_error(`bb_cg_ovulating')
              END IF;
            END IF;

          ELSE
            -- next_code = 'bb_turgesence'
            -- There are 3 possiblities, we don't know which is right
            -- but at this point we can check for one of the three.
            IF NEW.state <> 'bb_cg_menses'
               AND NEW.state <> 'bb_cg_pregnant'
               AND NEW.state <> 'bb_cg_lactating' THEN
              state_error(`bb_cg_menses or bb_cg_pregnant or bb_cg_lactating')
            END IF;
          END IF;
        END IF;
      END IF;
    END;
  END IF;


  -- Inserting an end of observation will affect the cycles' series
  -- and can 'split' cycles.
  -- (Start of observation and point observations won't have any cycles
  -- in them when they are first created.)
  IF NEW.code = 'bb_cg_end' THEN
    DECLARE
      next_date cycpoints.date%TYPE;
      next_series cycles.series%TYPE;
      next_cid cycles.cid%TYPE;

      prior_date cycpoints.date%TYPE;
      prior_cid cycpoints.cid%TYPE;

      new_cid cycles.cid%TYPE := NULL;

    BEGIN
      -- Find the cycpoint and cycle following the gap.
      SELECT INTO next_date,      next_series,   next_cid
                  cycpoints.date, cycles.series, cycles.cid
        FROM cycles, cycpoints
        WHERE cycles.sname = NEW.sname
              AND cycpoints.cid = cycles.cid
              AND cycpoints.date > NEW.date
              -- AND cycpoints.code <> 'bb_bleeding'
        ORDER BY cycpoints.date
        LIMIT 1;
      IF FOUND THEN
        -- There's a subsequent cycpoint to worry about.

        -- Find the cycpoint and cycle preceeding the gap.
        SELECT INTO prior_date,     prior_cid
                    cycpoints.date, cycles.cid
          FROM cycles, cycpoints
          WHERE cycles.sname = NEW.sname
                AND cycpoints.cid = cycles.cid
                AND cycpoints.date <= NEW.date
                -- AND cycpoints.code <> 'bb_bleeding'
          ORDER BY cycpoints.date DESC
          LIMIT 1;
        IF FOUND THEN
          -- There's cycpoints before the gap too, so we
          -- might need to fiddle with the series.
          --IF next_series <> prior_series + 1 THEN
            IF prior_cid = next_cid THEN
              -- We need to split a cycle.
              INSERT INTO cycles (sname) VALUES (NEW.sname);
              new_cid := currval('cycles_cid_seq');
              UPDATE cycpoints
                SET cid = new_cid
                WHERE cid = prior_cid
                      AND date > NEW.date;
            ELSE
              -- Not splitting a cycle, which takes care of seq and
              -- series by itself, but there's still a new gap
              -- so we need to take care of the seq and the series.
              PERFORM _recalc_seq_series(NEW.sname
                                       , prior_date
                                       , prior_cid);
            END IF;
          --END IF;
        END IF;
      END IF;
    END;
  END IF;

  RETURN NULL;
  END;
$$;

-----------------------------------------------------------------
SELECT 'cycgaps_delete_func' AS function;
CREATE FUNCTION cycgaps_delete_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  -- Function for cycgaps delete trigger
  --
  -- GPL_notice(`  --', `2005, 2006, 2008', `Karl O. Pinc <kop@meme.com>')

  DECLARE

  BEGIN

  IF NOT(_cycgaps_delete_ok(OLD, TRUE)) THEN
    RETURN NULL;
  END IF;

  --
  -- Done with easy checks, update CYCGAPDAYS.
  --
  PERFORM _cycgapdays_interval_rebuild(OLD);

  --
  -- Rebuild the automatic Mdates.
  <<assign_autom>>
  DECLARE
    start_range cycgaps.date%TYPE;
    stop_range cycgaps.date%TYPE;

  BEGIN
    -- Rebuild the automatic Mdates surrounding the old row.
    _cg_rebuild_autom(`OLD', `assign_autom')
  END;

  --
  -- Deal with ongoing pregnancies.
  --

  --
  -- Update PREGS.Resume with the Tdate, if it's right
  -- after the conception cycle and there's no gap.
  -- If that is not necessary, then check the rules
  -- with respect to a NULL Resume:
  --
  -- Make the earliest Tdate before a gap be the pregancy resume
  -- date when there's no resume.
  --
  -- And make sure there's otherwise no cycles after an ongoing
  -- pregnancy.

  DECLARE
    this_gap      cycgapdays.date%TYPE;
    preg_pid      pregs.pid%TYPE;
    preg_date     cycpoints.date%TYPE;
    preg_resume   pregs.resume%TYPE;
    cycpoint_code cycpoints.code%TYPE;
    cycpoint_cpid cycpoints.cpid%TYPE;

  BEGIN

    -- Find prior pregnancy, it's date and resume.
    SELECT INTO preg_pid,  preg_date,      preg_resume
                pregs.pid, cycpoints.date, pregs.resume
      FROM cycles, cycpoints, pregs
      WHERE cycles.sname = OLD.sname
            AND cycpoints.cid = cycles.cid
            AND cycpoints.date < OLD.date
            AND pregs.conceive = cycpoints.cpid
      ORDER BY cycpoints.date DESC
      LIMIT 1;

    IF FOUND AND preg_resume IS NULL THEN
      -- We do need to either update or validate against the
      -- last pregnancy.

      -- Find the gap following the pregnancy.
      SELECT INTO this_gap cycgapdays.date
        FROM cycgapdays
        WHERE cycgapdays.sname = OLD.sname
              AND cycgapdays.date > preg_date
        ORDER BY cycgapdays.date
        LIMIT 1;

      -- Find the cycpoints between the pregnancy and the gap.
      SELECT INTO cycpoint_cpid,  cycpoint_code
                  cycpoints.cpid, cycpoints.code
        FROM cycles, cycpoints
        WHERE cycles.sname = OLD.sname
              AND cycpoints.cid = cycles.cid
              AND cycpoints.date > preg_date
              AND (cycpoints.date < this_gap
                   OR this_gap IS NULL)
        ORDER BY cycpoints.date
        LIMIT 1;

      IF FOUND THEN
        -- The date of the cycpoints row is not separated
        -- from the pregnancy by a gap.  Our work is not yet done.

        IF cycpoint_code = 'bb_turgesence' THEN
          -- Tdate immediately follows the pregnancy.
          -- Make it the resume date.
          UPDATE pregs
            SET resume = cycpoint_cpid
            WHERE pid = preg_pid;
        ELSE
          -- Some sexual cycle follows a ongoing pregnancy.
          RAISE EXCEPTION '%: %: Cannot delete CYCGAPS -- Ongoing pregnancy (NULL Resume) cannot have subsequent CYCPOINTS (and no intervening gap in observation)'
              , OLD.sname
              , OLD.date;
          RETURN NULL;
        END IF;
      END IF;
    END IF;
  END;

--  --
--  -- The first Tdate after an ongoing pregnancy ends the pregnancy.
--  -- Otherwise,
--  -- Female cannot have any cycpoints rows postdating a pregnancy with
--  -- a null resume, unless they postdate a gap that follows the pregnancy.
--  --
--
--  -- Deleting a period of no observation means we have to re-test
--  -- pregnancies before the gap against cycles after the gap.
--  IF OLD.code = 'bb_cg_end' THEN
--    DECLARE
--      prior_preg_date cycpoints.date%TYPE;
--      next_gap cycgapdays.date%TYPE;
--
--    BEGIN
--      -- Find a pregnancy with no resume before the gap
--      SELECT INTO prior_preg_date cycpoints.date
--        FROM cycpoints
--             JOIN cycles
--               ON (cycles.sname = OLD.sname
--                   AND cycpoints.cid = cycles.cid
--                   AND cycpoints.date < OLD.date)
--             JOIN pregs
--               ON (pregs.conceive = cycpoints.cpid
--                   AND pregs.resume IS NULL)
--        ORDER BY cycpoints.date DESC
--        LIMIT 1;
--
--      IF FOUND THEN
--        -- There is a pregnancy with no resume to worry about.
--        -- Find the gap following the pregnancy.
--        SELECT INTO next_gap cycgapdays.date
--          FROM cycgapdays
--          WHERE cycgapdays.sname = OLD.sname
--                AND cycgapdays.date > prior_preg_date
--          ORDER BY cycgapdays.date
--          LIMIT 1;
--
--        -- See if there's cycpoints between the pregnancy and the gap.
--        PERFORM 1
--          FROM cycles, cycpoints
--          WHERE cycles.sname = OLD.sname
--                AND cycpoints.cid = cycles.cid
--                AND cycpoints.date > prior_preg_date
--                AND (cycpoints.date < next_gap
--                     OR next_gap IS NULL)
--          LIMIT 1;
---- This does not work.
----    PERFORM 1
----      FROM cycpoints AS pregscycpoints
----           JOIN cycles AS pregscycles
----             ON (pregscycpoints.date < OLD.date
----                 AND pregscycles.sname = OLD.sname
----                 AND pregscycles.cid = pregscycpoints.cid)
----           JOIN pregs
----             ON (pregs.conceive = pregscycpoints.cpid
----                 AND pregs.resume IS NULL)
----           JOIN cycles AS aftercycles
----             ON (aftercycles.sname = pregscycles.sname)
----           LEFT OUTER JOIN cycgaps
----             ON (cycgaps.sname = pregscycles.sname
----                 AND cycgaps.code = 'bb_cg_end'
----                 AND cycgaps.date > pregscycpoints.date)
----           JOIN cycpoints AS aftercycpoints
----             ON (aftercycpoints.cid = aftercycles.cid
----                 AND aftercycpoints.date > pregscycpoints.date
----                 AND (aftercycpoints.date <= cycgaps.date
----                      OR cycgaps.date IS NULL))
----                         -- which we need because of the outer join.
----      LIMIT 1;
--        IF FOUND THEN
--          RAISE EXCEPTION '%: %: %: Cannot delete; ongoing pregnancy (NULL Resume) has subsequent CYCPOINTS (and no intervening gap in observation)'
--                        , OLD.sname
--                        , OLD.date
--                        , OLD.code;
--          RETURN NULL;
--        END IF;
--      END IF;
--    END;
--  END IF;
--
  -- Now that cycgapdays is updated do the final checks
  -- for cycpoints in the gaps.
--raise info 'fourth';
  check_gaps(`OLD')

  --
  -- Re-series the cycles now that a gap is gone.
  --

  DECLARE
    next_series cycles.series%TYPE;
    next_date cycpoints.date%TYPE;
    next_code cycpoints.code%TYPE;
    next_cid cycles.cid%TYPE;

    prior_series cycles.series%TYPE;
    prior_date cycpoints.date%TYPE;
    prior_code cycpoints.code%TYPE;
    prior_cid cycles.cid%TYPE;

    right_series cycles.series%TYPE;

  BEGIN
    -- Find the cycle above the now-gone gap.
    SELECT INTO next_series,   next_date,      next_code,      next_cid
                cycles.series, cycpoints.date, cycpoints.code, cycles.cid
      FROM cycles, cycpoints
      WHERE cycles.sname = OLD.sname
            AND cycpoints.cid = cycles.cid
            AND cycpoints.date > OLD.date
            -- AND cycpoints.code <> 'bb_bleeding'
      ORDER BY cycpoints.date, cycpoints.code
      LIMIT 1;

    IF FOUND THEN
      -- There are subsequent cycles to worry about.

      -- Find the cycle below the now-gone gap.
      SELECT INTO prior_series,  prior_date,     prior_code,     prior_cid
                  cycles.series, cycpoints.date, cycpoints.code, cycles.cid
        FROM cycles, cycpoints
        WHERE cycles.sname = OLD.sname
              AND cycpoints.cid = cycles.cid
              AND cycpoints.date < OLD.date
              -- AND cycpoints.code <> 'bb_bleeding'
        ORDER BY cycpoints.date DESC, cycpoints.code DESC
        LIMIT 1;
      IF FOUND THEN
        -- The prior series(es) could be greater than 1 so must be
        -- worried about.

        -- Is there a gap between the two cycles?
        PERFORM 1
          FROM cycgapdays
          WHERE sname = OLD.sname
                AND date > prior_date
                AND date < next_date
          LIMIT 1;
        IF FOUND THEN
          right_series = prior_series + 1;
        ELSE
          -- No gap between the two cycles.
          right_series = prior_series;
        END IF;

        IF next_series <> right_series THEN
          -- Adjust the subsequent cycles' series by the difference.
          UPDATE cycles
           SET series = series + (right_series - next_series)
           WHERE sname = OLD.sname
                 AND cid IN (SELECT DISTINCT cycpoints.cid
                               FROM cycpoints
                               WHERE cycpoints.cid = cycles.cid
                                     AND cycpoints.date >= next_date);
        END IF;

        IF right_series = prior_series THEN
          -- There is no longer a gap between successive cycles.

          -- Should the two cycles be merged now that they are no
          -- longer separated by a gap?
          -- (Wait to do this until after updating series numbers
          -- for various reasons, one of them being that this
          -- deletes another CYCLES row.)

          IF (prior_code = 'bb_menses'
              AND (next_code = 'bb_turgesence'
                   OR next_code = 'bb_deturgesence'))
             OR (prior_code = 'bb_turgesence'
                 AND next_code = 'bb_deturgesence') THEN
            -- Merge the earlier cycpoints into the later cycle.
            -- Cycpoints triggers take care of deleting the cycle.
            UPDATE cycpoints
              SET cid = next_cid
              WHERE cid = prior_cid;
          END IF;
        END IF;
      END IF;
    END IF;
  END;

  RETURN NULL;
  END;
$$;

-----------------------------------------------------------------
SELECT '_cycgaps_delete_ok(cycgaps, deleting)';
CREATE FUNCTION _cycgaps_delete_ok(cg cycgaps, deleting BOOLEAN)
  RETURNS BOOLEAN
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  -- Does deleting a cycgaps row leave a legal cycgaps table?
  --
  -- GPL_notice(`  --', `2005, 2008', `Karl O. Pinc <kop@meme.com>')

  -- Syntax: _cycgaps_delete_ok(cg cycgaps, deleting BOOLEAN)
  --
  -- Input:
  --  cg   A ROWTYPE that contains a deleted cycgaps row.
  --  deleting  Whether or not we are deleting the row.
  --            True means the row is really being deleted.
  --            False means the row is being updated.
  --
  -- Returns:
  --  Boolean  True when it's ok to delete the row, based only on
  -- what's on CYCGAPS not on other tables.
  -- ??
  --
  -- Side Effects:
  --   Raises exceptions.
  --
  -- Called by the cycgaps triggers.

  BEGIN

--RAISE INFO '_cycgaps_delete_ok(%, %)', cg, deleting;

  IF cg.code = 'bb_cg_point' THEN
--     -- Only problem would be there's an observation on that day.
--     PERFORM 1
--       FROM cycles, cycpoints
--       WHERE cycles.name = cg.name
--             AND cycpoints.cid = cycles.cid
--             AND cycpoints.date = cg.date
--       LIMIT 1;
--     IF FOUND THEN
--       - There's a cycpoints on that date.
--       RAISE EXCEPTION '%: %: Cannot delete from CYCGAPS where Code is bb_cg_point because the female has a CYCPOINTS row on that day'
--                     , cg.sname
--                     , cg.date;
--       RETURN FALSE;
--     END IF;

    RETURN TRUE;
  END IF;


    -- A female's earliest cycgaps must be an 'end'.
    -- (When there's another cycgap on the same day for the same
    -- inidividual that means that we'll be automatically deleting
    -- the pair of them.)
  DECLARE
    earliest_code cycgaps.code%TYPE;

  BEGIN
    SELECT INTO earliest_code early_cg.code
      FROM cycgaps AS early_cg
      WHERE early_cg.sname = cg.sname
            AND NOT EXISTS (SELECT 1 -- another cycgap on the same day
                              FROM cycgaps AS other_cg
                              WHERE other_cg.sname = cg.sname
                                    AND other_cg.date = early_cg.date
                                    AND other_cg.gapid <> cg.gapid
                                    AND other_cg.gapid <> early_cg.gapid)
            AND (early_cg.gapid <> cg.gapid
                 AND deleting)
      ORDER BY early_cg.date
      LIMIT 1;

    IF earliest_code <> 'bb_cg_end' THEN
--    Former code inside the next declare that gets earliest and latest.
--    IF prior_range_code IS NULL
--       AND ((now_code IS NULL AND next_range_code = 'bb_cg_start')
--            OR now_code = 'bb_cg_start') THEN
      RAISE EXCEPTION '%: %: %: Deleting this row means the first CYCGAPS row does not end observation'
                      , cg.sname
                      , cg.date
                      , cg.code;
      RETURN FALSE;
    END IF;
  END;

  -- Find the code for the female's cycgaps row on the
  -- given date, if any, and the codes for the female's
  -- cycgaps rows "to either side", immediately prior and
  -- immediately subsequent.

  -- Ignore those days that have more than one cygapdays
  -- row.  Those are the ones we're either getting rid of
  -- or replacing with a point observation.
  DECLARE
    now_code cycgaps.code%TYPE;
    prior_range_code cycgaps.code%TYPE;
    prior_range_date cycgaps.date%TYPE;
    next_range_code cycgaps.code%TYPE;
    next_range_date cycgaps.date%TYPE;

  BEGIN

    SELECT INTO now_code code
      FROM cycgaps
      WHERE sname = cg.sname
            AND date = cg.date
            AND NOT EXISTS (SELECT 1
                              FROM cycgaps AS other
                              WHERE other.sname = cg.sname
                                    AND other.date = cg.date
                                    AND other.gapid <> cycgaps.gapid)
    LIMIT 1;
    SELECT INTO prior_range_code, prior_range_date
                code            , date
      FROM cycgaps
      WHERE sname = cg.sname
            AND date < cg.date
            AND code <> 'bb_cg_point'
            AND NOT EXISTS (SELECT 1
                              FROM cycgaps AS other
                              WHERE other.sname = cg.sname
                                    AND other.date = cycgaps.date
                                    AND other.gapid <> cycgaps.gapid)
    ORDER BY date DESC
    LIMIT 1;
    SELECT INTO next_range_code, next_range_date
                code           , date
      FROM cycgaps
      WHERE sname = cg.sname
            AND date > cg.date
            AND code <> 'bb_cg_point'
            AND NOT EXISTS (SELECT 1
                              FROM cycgaps AS other
                              WHERE other.sname = cg.sname
                                    AND other.date = cycgaps.date
                                    AND other.gapid <> cycgaps.gapid)
      ORDER BY date
      LIMIT 1;

    --
    -- No two starts or two ends in a row.
    --
    DECLARE
      e BOOLEAN;

    BEGIN
      IF now_code IS NULL THEN
        e := prior_range_code = next_range_code;
--RAISE INFO 'this one';
      ELSE
        -- Moved start on top of end to eliminate them both.
        e := prior_range_code = now_code
             OR now_code = next_range_code;
--RAISE INFO 'that one';
      END IF;

      IF e THEN
        RAISE EXCEPTION '%: %: %: Deletion results in CYCGAPS starting observations without ending them or ending them without starting them'
                        , cg.sname
                        , cg.date
                        , cg.code;
        RETURN FALSE;
      END IF;
    END;

    --
    -- Can't have points in periods of observation.
    --
    DECLARE
      e BOOLEAN;

    BEGIN
      IF now_code IS NULL OR now_code = 'bb_cg_point' THEN
        -- Check the range from start to end.
        IF prior_range_code = 'bb_cg_start' 
           OR prior_range_code IS NULL THEN
          PERFORM 1
            FROM cycgaps
            WHERE sname = cg.sname
                  AND (date > prior_range_date
                       OR prior_range_date IS NULL)
                  AND (date < next_range_date
                       OR next_range_date IS NULL)
                  AND code = 'bb_cg_point'
            LIMIT 1;
            e := FOUND;
          END IF;
      ELSE
        -- now code is a start or an end.
        IF now_code = 'bb_cg_start' THEN
          -- Check the time before the deleted row.
          PERFORM 1
            FROM cycgaps
            WHERE sname = cg.sname
                  AND date > cg.date
                  AND (date < next_range_date
                       OR next_range_date IS NULL)
                  AND code = 'bb_cg_point'
            LIMIT 1;
          e := FOUND;
        ELSE
          -- now code is an end code.
          -- Check the time before the deleted row.
          PERFORM 1
            FROM cycgaps
            WHERE sname = cg.sname
                  AND date < cg.date
                  AND (date > prior_range_date
                       OR prior_range_date IS NULL)
                  AND code = 'bb_cg_point'
            LIMIT 1;
          e := FOUND;
        END IF;
      END IF;

      IF e THEN
        RAISE EXCEPTION '%: %: %: Change results in CYCGAPS with points of observation during periods of observation'
                        , cg.sname
                        , cg.date
                        , cg.code;
        RETURN FALSE;
      END IF;
    END;
  END;

  -- No other violiations can be caused by deletion.
  RETURN TRUE;

  END;
$$;
grant_func_priv(`_cycgaps_delete_ok(cycgaps, BOOLEAN)')


-----------------------------------------------------------------
SELECT '_cycgapdays_interval_rebuild(cycgaps)';
CREATE FUNCTION _cycgapdays_interval_rebuild(cg cycgaps)
  RETURNS VOID
  SECURITY DEFINER
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  -- Rebuild an interval on cycgapdays surrounding a inserted or 
  -- deleted cycgaps row.
  --
  -- GPL_notice(`  --', `2006, 2008', `Karl O. Pinc <kop@meme.com>')

  -- Syntax: _cycgapdays_interval_rebuild(cg cycgaps)
  --
  -- Input:
  --  cg   A ROWTYPE that contains a inserted or deleted cycgaps row.
  --
  -- Returns:
  --
  -- Side Effects:
  --  Modifies cycgapdays.
  --
  -- Remarks:
  --  The basic idea is that the farthest afield any particular cycgaps
  -- row can affect (the most distant in time) is the start or end of
  -- observation dates to either side of it's location in time.
  -- So, find those intervals and recompute everything in them.
  --
  -- It's not like we update cycgaps that much anyhow.
  --
  -- Called by the cycgaps triggers.

  DECLARE
    start_range cycgaps.date%TYPE;
    start_code cycgaps.code%TYPE;
    stop_range cycgaps.date%TYPE;
    some_cycgaps_start BOOLEAN := TRUE;
    this_date cycgapdays.date%TYPE;
    this_code cycgaps.code%TYPE;

  BEGIN

--RAISE INFO '_cycgapdays_interval_rebuild(%)', cg;

  --
  -- Find the start and end intervals to either side of
  -- the given date.  That is, the nearest start or end
  -- of observation on each side.
  --

  -- Find the lower end of the range.
  SELECT INTO start_range,  start_code
              cycgaps.date, cycgaps.code
    FROM cycgaps
    WHERE cycgaps.sname = cg.sname
          AND cycgaps.date < cg.date
          AND cycgaps.code <> 'bb_cg_point'
    ORDER BY cycgaps.date DESC
    LIMIT 1;
  IF NOT FOUND THEN
    -- Use the current date as the lower limit.
    start_range := cg.date;

    SELECT INTO start_code cycgaps.code
      FROM cycgaps
      WHERE cycgaps.sname = cg.sname
            AND cycgaps.date = cg.date;
    some_cycgaps_start := FOUND;
  END IF;

  -- Find the upper end of the range.
  SELECT INTO stop_range cycgaps.date
    FROM cycgaps
    WHERE cycgaps.sname = cg.sname
          AND cycgaps.date > cg.date
          AND cycgaps.code <> 'bb_cg_point'
    ORDER BY cycgaps.date
    LIMIT 1;
  IF NOT FOUND THEN
    -- Use the statdate as the end of the range,
    -- plus some allowance for automatic Mdates that go past
    -- the end of the range.
    SELECT INTO stop_range biograph.statdate + bb_autom_interval + 1
      FROM biograph
      WHERE sname = cg.sname;
  END IF;


--RAISE INFO 'start_range %: stop_range %: some_cycgaps_start %', start_range, stop_range, some_cycgaps_start;

  --
  -- Erase all the days between the interval.
  --
  DELETE
    FROM cycgapdays
    WHERE cycgapdays.sname = cg.sname
          AND cycgapdays.date > start_range
          AND cycgapdays.date < stop_range;

  --
  -- Insert cycgapdays rows over the interval based on what's
  -- in the cycgaps table now.
  --
  IF NOT some_cycgaps_start THEN
    -- There is no cycgaps at the start of the range.
    -- We must have deleted the first gap.
    -- Nothing to do.
    RETURN;
  END IF;

  -- Find when observation ends and starts and we need to be putting
  -- cycgap rows in place.

  -- Where does the observation end?
  IF start_code = 'bb_cg_end' THEN
    this_date := start_range + 1;
  ELSE
    SELECT INTO this_date cycgaps.date + 1
      FROM cycgaps
      WHERE cycgaps.sname = cg.sname
            AND cycgaps.date > start_range
            AND cycgaps.code = 'bb_cg_end'
            AND cycgaps.date < stop_range
      ORDER BY cycgaps.date
      LIMIT 1;
    IF NOT FOUND THEN
      -- There are no gaps in observation in the interval
      RETURN;
    END IF;
  END IF;

  WHILE this_date < stop_range LOOP
--RAISE INFO 'Looping this_date: %', this_date;
    -- Create all the cycgapdays rows in the gap.
    SELECT INTO this_code cycgaps.code
      FROM cycgaps
      WHERE cycgaps.sname = cg.sname
            AND cycgaps.date = this_date;
    IF FOUND THEN
      IF this_code = 'bb_cg_point' THEN
        -- Point observations mean no gap for the day.
      ELSEIF this_code = 'bb_cg_start' THEN

        -- Observation has started, no gap for this day.
        -- Find the next end of observation.
        SELECT INTO this_date cycgaps.date
          FROM cycgaps
          WHERE cycgaps.sname = cg.sname
                AND cycgaps.date > this_date
                AND cycgaps.code = 'bb_cg_end'
                AND cycgaps.date < stop_range
          ORDER BY cycgaps.date
          LIMIT 1;
        IF NOT FOUND THEN
          -- There are no gaps in observation in the interval
          RETURN;
        END IF;
      END IF;

    ELSE -- No cycgaps row found.  We're in a gap in observation.
--RAISE INFO 'insert %: %', cg.sname, this_date;
      INSERT INTO cycgapdays (sname, date)
             values (cg.sname, this_date);
    END IF;

    -- Prepare for next iteration.
    this_date := this_date + 1;
  END LOOP;

  RETURN;

  END;
$$;
grant_func_priv(`_cycgapdays_interval_rebuild(cycgaps)')

SELECT 'cycgaps_trigger' AS trigger;
CREATE TRIGGER cycgaps_trigger
  AFTER INSERT OR UPDATE
  ON cycgaps FOR EACH ROW
  EXECUTE PROCEDURE cycgaps_func();

SELECT 'cycgaps_delete_trigger' AS trigger;
CREATE TRIGGER cycgaps_delete_trigger
  AFTER DELETE
  ON cycgaps FOR EACH ROW
  EXECUTE PROCEDURE cycgaps_delete_func();
