dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2004-2006, 2008, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl

--
-- cycles
-- $Id: cycles.m4,v 1.27 2004/09/13 20:42:53 kop Exp $
--
SELECT 'cycles' AS table;

--SELECT 'cycles_before_func' AS function;
--CREATE FUNCTION cycles_before_func ()
--  RETURNS trigger
--  LANGUAGE plpgsql
--  bb_function_set_search_path
--  AS $$
--  -- Function for cycles insert and update run before changes applied
--  --
--  -- GPL_notice(`  --', `2004-2006, 2008', `Karl O. Pinc <kop@meme.com>')
--
--  BEGIN
--
--  -- *sigh* recursive checking problem here.  Can't enter
--  -- Pregs until after cycle exists, but empty cycle can't
--  -- exist unless there's a Preg.
--  -- Leaving in on the off chance there's ever a "run trigger on
--  -- transaction commit."  Meanwhile, moving to the database check.
--  -- 
--  --   -- Cycles must have some date, unless the cycle is for a conception
--  --   -- all the data can't be null.
--  --   PERFORM NEW.cid FROM pregs
--  --     WHERE pregs.conceive = NEW.cid;
--  --   IF COALESCE(NEW.mdate
--  --               , NEW.emdate
--  --               , NEW.lmdate
--  --               , NEW.tdate
--  --               , NEW.etdate
--  --               , NEW.ltdate
--  --               , NEW.ddate
--  --               , NEW.eddate
--  --               , NEW.lddate) IS NULL
--  --       AND NOT FOUND THEN
--  --     RAISE EXCEPTION 'CYCLES %: cycle is not a conception and has no dates'
--  --           , NEW.cid;
--  --     RETURN NULL;
--  --   END IF;
--
--  -- There should be no overlap between cycles
--  -- (also ensuring that there's no overlap between pregnancies.)
--
--  RETURN NEW;
--  END;
--$$;

SELECT 'cycles_func' AS function;
CREATE FUNCTION cycles_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  -- Function for cycles insert and update triggers
  --
  -- GPL_notice(`  --', `2004-2006, 2008', `Karl O. Pinc <kop@meme.com>')

  DECLARE

  BEGIN
--RAISE INFO '%: Cycles %: Cid %: Seq %: Series %', TG_OP, NEW.sname, NEW.cid, NEW.Seq, NEW.Series;

  -- Get arguments.

  -- cid cannot change
  IF TG_OP = 'UPDATE' THEN
    cannot_change(`CYCLES', `Cid')
    -- Can't change the SNAME or the SEXSKINS constraint
    -- checking can be thrown off, as well as the PREGS conceive female
    -- matches resume female check.
    cannot_change(`CYCLES', `Sname')
  END IF;

  -- Cycles only belong to females.
  PERFORM NEW.sname
    FROM biograph
    WHERE biograph.sname = NEW.sname
          AND biograph.sex = 'bb_female'
    LIMIT 1;
  IF NOT FOUND THEN
    RAISE EXCEPTION 'CYCLES %: %: Only females can have cycles'
                    , NEW.cid
                    , NEW.sname;
  END IF;

/* Well, it seems that if we're going to allow arbitrary
 * date changes (or as tested allow inserts into cycpoints
 * that have arbitrary dates and link to some random one of
 * the female's cycles) that we can't check the seq column
 * for validity.  The problem comes when the triggers insert
 * automatic Mdates.  So, don't bother checking either Seq
 * or Series dynamically, do it in batch instead.
 *   IF NEW.seq IS NOT NULL THEN
 * 
 * / * This test cannot be done on a per-row basis because the
 *  * cycles seq numbers are updated per-row, always from least
 *  * to greatest, and when we alter the
 *  * the conception cycle seq (for some reason, like increasing all
 *  * the seqs by 1) the resumption cycle test
 *  * will fail.  So instead we do this test in batch.
 *  *
 *  *  --
 *  *  -- Conception cycle seq numbers must be one less than resumption
 *  *  -- cycle seq numbers on PREGS.
 *  *  --
 *  *
 *  *    -- If it is NULL then we'll check later when we make it un-NULL.
 *  *
 *  *    -- Test for when the current cycle is the conception cycle.
 *  *    PERFORM 1
 *  *      FROM cycpoints AS conceive, pregs, cycpoints AS resume, cycles
 *  *      WHERE conceive.cid = NEW.cid
 *  *            AND pregs.conceive = conceive.cpid
 *  *            AND resume.cpid = pregs.resume
 *  *            AND cycles.cid = resume.cid
 *  *            AND cycles.seq <> NEW.seq + 1
 *  *            AND cycles.seq IS NOT NULL
 *  *      LIMIT 1;
 *  *    IF FOUND THEN
 *  *      RAISE EXCEPTION '%: Cid %: Seq %: Conception cycle is not one less than resumption cycle'
 *  *                    , NEW.sname
 *  *                    , NEW.cid
 *  *                    , NEW.date;
 *  *      RETURN NULL;
 *  *    END IF;
 *  *
 *  *
 *  *    -- Test for when the current cycle is the resumption cycle.
 *  *    PERFORM 1
 *  *      FROM cycpoints AS resume, pregs, cycpoints AS conceive, cycles
 *  *      WHERE resume.cid = NEW.cid
 *  *            AND pregs.resume = resume.cpid
 *  *            AND conceive.cpid = pregs.conceive
 *  *            AND cycles.cid = conceive.cid
 *  *            AND cycles.seq <> NEW.seq - 1
 *  *            AND cycles.seq IS NOT NULL
 *  *      LIMIT 1;
 *  *    IF FOUND THEN
 *  *      RAISE EXCEPTION '%: Cid %: Seq %: Invalid CYCLES row: Conception cycle is not one less than resumption cycle'
 *  *                    , NEW.sname
 *  *                    , NEW.cid
 *  *                    , NEW.date;
 *  *      RETURN NULL;
 *  *    END IF;
 *  * /
 *     -- Make sure the seq is correct.
 *     -- Look only at the prior cycle and only when the seq
 *     -- has changed and hope for the best that other triggers
 *     -- are going to honor this convention and we won't get
 *     -- the trigger equalivent of a deadlock.  (*ick*)
 *     -- The alternative to this logic is to examine current_user
 *     -- and use SECURITY DEFINER in the other code that messes with Seq.
 *     IF TG_OP = 'UPDATE' THEN
 *       IF OLD.seq <> NEW.seq THEN
 *         PERFORM 1
 *           FROM cycpoints
 *           WHERE cid = NEW.cid
 *                 AND code <> 'bb_bleeding'
 *           LIMIT 1;
 *         IF FOUND THEN
 *           -- This cycle has cycpoints.
 *           -- (No point in checking seq if we have no cycpoints.)
 *           DECLARE
 *             last_seq cycles.seq%TYPE;
 *             e BOOLEAN;
 *   
 *           BEGIN
 *             SELECT INTO last_seq cycles.seq
 *               FROM cycles, cycpoints AS ours, cycpoints AS theirs
 *               WHERE ours.cid = NEW.cid
 *                     AND cycles.sname = NEW.sname
 *                     AND theirs.cid = cycles.cid
 *                     AND theirs.date < ours.date
 *                     AND ours.code <> 'bb_bleeding'
 *                     AND theirs.code <> 'bb_bleeding'
 *               ORDER BY theirs.date DESC, theirs.code DESC
 *               LIMIT 1;
 *             IF FOUND THEN
 *               e := last_seq IS NOT NULL AND NEW.seq <> last_seq + 1;
 *             ELSE
 *               e := NEW.seq <> 1;
 *             END IF;
 *             IF e THEN
 *               RAISE EXCEPTION '%: Cid %: Seq %: CYCLES.Seq out of sequence'
 *                             , NEW.sname
 *                             , NEW.cid
 *                             , NEW.seq;
 *               RETURN NULL;
 *             END IF;
 *           END;
 *         END IF;
 *       END IF;
 *     END IF;
 * 
 *   ELSE
 *     -- NEW.seq IS NULL
 * 
 *     -- Can't change from non-null to NULL.
 *     IF TG_OP = 'UPDATE' THEN
 *       IF OLD.seq IS NOT NULL THEN
 *         RAISE EXCEPTION '%: Cid %: Cannot change Seq from non-NULL to NULL'
 *                       , NEW.sname
 *                       , NEW.cid;
 *       END IF;
 *     END IF;
 * 
 *     -- You can't have related cycpoints when you've a NULL seq.
 *     PERFORM 1
 *       FROM cycpoints
 *       WHERE cid = NEW.cid
 *       LIMIT 1;
 *     IF FOUND THEN
 *       RAISE EXCEPTION '%: Cid %: Invalid CYCLES row: Seq is NULL but there aer related CYCPOINTS'
 *                     , NEW.sname
 *                     , NEW.cid;
 *       RETURN NULL;
 *     END IF;
 *   END IF;
 * 
 * 
 *   -- Make sure the series is correct.
 *   -- Do this like we do seq
 *   -- Look only at the prior cycle and only when the series
 *   -- has changed and hope for the best that other triggers
 *   -- are going to honor this convention and we won't get
 *   -- the trigger equalivent of a deadlock.  (*ick*)
 *   IF TG_OP = 'UPDATE' THEN
 * 
 *       IF OLD.series IS NOT NULL AND NEW.series IS NULL THEN
 *         RAISE EXCEPTION '%: Cid %: Cannot change CYCLES.Series from non-NULL to NULL'
 *                       , NEW.sname
 *                       , NEW.cid;
 *         RETURN NULL;
 *       END IF;
 * 
 *     IF OLD.series <> NEW.series AND NEW.series IS NOT NULL THEN
 * 
 *       DECLARE
 *         last_series cycles.series%TYPE;
 *         last_date cycpoints.date%TYPE;
 * 
 *         this_date cycpoints.date%TYPE;
 * 
 *         e BOOLEAN;
 * 
 *       BEGIN
 * 
 *         -- Don't bother if the cycle has no related cycpoints.
 *         SELECT INTO this_date cycpoints.date
 *           FROM cycpoints
 *           WHERE cid = NEW.cid
 *                 AND code = 'bb_bleeding'
 *           ORDER BY date
 *           LIMIT 1;
 *         IF FOUND THEN
 *           -- There are related cycpoints.
 * 
 *           SELECT INTO last_series,   last_date
 *                       cycles.series, cycpoints.date
 *             FROM cycles, cycpoints AS ours, cycpoints AS theirs
 *             WHERE ours.cid = NEW.cid
 *                   AND cycles.sname = NEW.sname
 *                   AND theirs.cid = cycles.cid
 *                   AND theirs.date < ours.date
 *                   AND ours.code <> 'bb_bleeding'
 *                   AND theirs.code <> 'bb_bleeding'
 *             ORDER BY theirs.date DESC, theirs.code DESC
 *             LIMIT 1;
 *           IF FOUND THEN
 *             IF last_series IS NULL THEN
 *               e := FALSE;
 *             ELSE
 *               -- There is a prior cycle.  Look for a gap between now and then.
 * 
 *               PERFORM 1
 *                 FROM cycgapdays
 *                 WHERE sname = NEW.sname
 *                       AND date < this_date
 *                       AND date > last_date
 *                 LIMIT 1;
 * 
 *               IF FOUND THEN
 *                 e := NEW.series <> last_series + 1;                
 *               ELSE
 *                 e := NEW.series <> last_series;
 *               END IF;
 * 
 *             END IF;
 *           ELSE
 *             -- No prior cycle.
 *             e := NEW.series <> 1;
 *           END IF;
 * 
 *           IF e THEN
 *             RAISE EXCEPTION '%: Cid %: %:: CYCLES.Series incorrect'
 *                           , NEW.sname
 *                           , NEW.cid
 *                           , NEW.series;
 *             RETURN NULL;
 *           END IF;
 *         END IF;
 *       END;
 *     END IF;
 *   END IF;
 */
  -- We would like to be able to check sexskins, but we can't
  -- because the cycpoints trigger re-sorts cycpoints into
  -- cycles before it re-sorts sexskins into cycles.  So,
  -- there's a window during which sexskins can be in the wrong
  -- cycle and that could make the checks fail.  Instead, do the
  -- checks in a 'batch' fashion.  (There's no point in checking
  -- just on INSERT because there's no related cycpoints at that
  -- time.)
  --
  -- Check that turgecesence dosen't commence before or after
  -- the cycle's T date, D date occurs duing peak turgesence, etc. NOT


  --
  -- Done checking things.  Now update the rest of the database
  -- where appropriate.
  --

------------------------

  -- Do something to update cycstats/repstats?

  RETURN NULL;
  END;
$$;

SELECT 'cycles_delete_func' AS function;
CREATE FUNCTION cycles_delete_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  -- Function for cycles delete trigger
  --
  -- GPL_notice(`  --', `2004-2006, 2008', `Karl O. Pinc <kop@meme.com>')

  DECLARE

  BEGIN

  -- Get the seq right.
  -- (This is actually being done in the cycpoints trigger,  and I don't
  -- _think_ it's possible to delete here without having deleted there,
  -- so there's no point in doing it twice.  It may well be that this code
  -- is cleaner and should be used instead of the code in the cycpoints
  -- delete trigger...)
  -- UPDATE cycles
  --   SET seq = seq - 1
  --   WHERE sname = OLD.sname
  --         AND seq > OLD.seq;  -- cleverly accounts for OLD.seq = NULL
----------------------------

  --
  -- Done checking things.  Now update the rest of the database
  -- where appropriate.
  --

  -- Do something to update cycstats/repstats?

  RETURN NULL;
  END;
$$;

--SELECT 'cycles_before_trigger' AS trigger;
--CREATE TRIGGER cycles_before_trigger
--  BEFORE INSERT OR UPDATE
--  ON cycles FOR EACH ROW
--  EXECUTE PROCEDURE cycles_before_func();

SELECT 'cycles_after_trigger' AS trigger;
CREATE TRIGGER cycles_after_trigger
  AFTER INSERT OR UPDATE
  ON cycles FOR EACH ROW
  EXECUTE PROCEDURE cycles_func();


SELECT 'cycles_delete_trigger' AS trigger;
CREATE TRIGGER cycles_delete_trigger
  AFTER DELETE
  ON cycles FOR EACH ROW
  EXECUTE PROCEDURE cycles_delete_func();


