dnl Copyright (C) 2011, 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2005, 2006, 2008, 2010, 2011, Karl O. Pinc
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl
dnl Remarks:
dnl  See remarks in cycpoints_func()

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl
include(`func_macros.m4')dnl

dnl plpgsql fragment for BLAH
dnl
dnl Syntax: BLAH(ARG)
dnl
dnl ARG   description
dnl
changequote({,})
define({BLAH},{
})dnl
changequote(`,')dnl


dnl plpgsql fragment for checking that a conception has a
dnl dad that's alive at the time of conception.
dnl
dnl Syntax: _living_dad(column)
dnl
dnl column  A dad column on DAD_DATA.
dnl
dnl Restrictions: Must be used in a FOR EACH ROW trigger function
dnl on cycpoints.
changequote({,})
define({_living_dad},{
      -- Make sure the dad ($1) of the conception is living at the time.
      PERFORM 1
        FROM pregs
             JOIN biograph AS kid ON (kid.pid = pregs.pid)
             JOIN dad_data ON (dad_data.kid = kid.sname)
             JOIN biograph AS dad ON (dad.sname = dad_data.$1)
        WHERE pregs.conceive = NEW.cpid
              AND dad.statdate
                  < NEW.date
                    - bb_ovulation_len
                    - CASE WHEN dad.status = bb_alive
                           THEN bb_interpolation_max
                           ELSE 0
                      END
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION 'Cpid %: Date %: Kid has a related DAD_DATA.$1 dad who is not alive during the mom''s fertile period -- on or after the Zdate (%) - bb_ovulation_len (- another bb_interpolation_max if the dad is alive)'
                      , NEW.cpid
                      , NEW.date
                      , NEW.date;
        RETURN NULL;
      END IF;

      -- Dad ($1) must be mature during conception.
      PERFORM 1
        FROM pregs
             JOIN biograph AS kid ON (kid.pid = pregs.pid)
             JOIN dad_data ON (dad_data.kid = kid.sname)
             JOIN biograph AS dad ON (dad.sname = dad_data.$1)
             LEFT OUTER JOIN maturedates ON (maturedates.sname = dad.sname)
        WHERE pregs.conceive = NEW.cpid
              AND (maturedates.matured IS NULL
                   OR NEW.date < maturedates.matured)
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION 'Cpid %: (Z)Date %: Has a related DAD_DATA.$1 dad who is not yet matured on the Zdate'
                      , NEW.cpid
                      , NEW.date;
        RETURN NULL;
      END IF;
})dnl
changequote(`,')dnl

dnl plpgsql fragment for adjusting automatic Mdates when a row is removed
dnl
dnl Syntax: _handle_cycpoint_deletion()
dnl
changequote({,})
define({_handle_cycpoint_deletion},{
        --
        -- Find the prior Ddate.
        -- (Turns out that we can always affect an earlier automatic Mdate.)
        --
        SELECT INTO ddate_row
                    cycpoints.*
          FROM cycles, cycpoints
          WHERE cycles.sname = this_sname
                AND cycpoints.cid = cycles.cid
                AND cycpoints.date < OLD.date
                AND cycpoints.code = 'bb_deturgesence'
          ORDER BY cycpoints.date DESC
          LIMIT 1;

        --
        -- Delete the old automatic Mdate.
        --
        IF OLD.code = 'bb_deturgesence' THEN
          IF ddate_row.cpid IS NOT NULL
             AND ddate_row.date >= OLD.date
                                  - CAST('bb_autom_interval days'
                                         AS INTERVAL) THEN
            -- The removal of the Ddate could affect the prior automatic
            -- Mdate.
            PERFORM _remove_autom(this_sname, ddate_row.date, OLD.cpid);
            PERFORM _add_autom(this_sname, ddate_row);
          END IF;
          PERFORM _remove_autom(this_sname, OLD.date, OLD.cpid);

        ELSEIF OLD.code = 'bb_turgesence' THEN
          -- Find the old automatic Mdate.

          IF ddate_row.cpid IS NOT NULL
             AND ddate_row.date >= OLD.date
                                  - CAST('bb_autom_interval days'
                                         AS INTERVAL) THEN
            -- The Tdate can affect the automatic Mdate placement.

            -- Find the next cycpoints row.
            -- If it's a automatic Mdate then delete it.
            PERFORM _remove_autom(this_sname, ddate_row.date, OLD.cpid);
            -- Recreate automatic Mdate for old Ddate.
            PERFORM _add_autom(this_sname, ddate_row);
          END IF;

        ELSE
          -- OLD.Code = 'bb_menses'
          -- You'd think that all that can be happening is
          -- that the Mdate moves and
          -- makes room for an automatic Mdate.
          -- But really, there could be a number of partal cycles
          -- that mess with this so we're better off re-creating
          -- the automatic Mdate.

          -- Find the prior Ddate
          IF ddate_row.cpid IS NOT NULL THEN
            -- Remove any automatic Mdate that might be there.
            PERFORM _remove_autom(this_sname, ddate_row.date, OLD.cpid);
            -- Recreate automatic Mdate for old Ddate.
            -- (In case we removed something we should not have.)
            PERFORM _add_autom(this_sname, ddate_row);
          END IF;
        END IF;
})dnl
changequote(`,')dnl

dnl plpgsql fragment for checking dates against gaps in observation.
dnl
dnl Syntax: check_against_gap(col)
dnl
dnl col   Either Date, Edate or Ldate.
dnl
dnl Exists so we get a nice error message instead of 'E or L or date'
dnl 
dnl Should not be called for estimated Ddates.
dnl
changequote({,})
define({check_against_gap},{
    -- Can't have a (non-Ddate, non-estimated) sexual cycle event
    -- unless there's observation going on.
    PERFORM 1
      FROM cycgapdays
      WHERE sname = this_sname
            AND date = NEW.$1
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION '%: %: CYCPOINTS row has a $1 during a period of no observation'
                    , this_sname
                    , NEW.$1;
      RETURN NULL;
    END IF;
})dnl
changequote(`,')dnl
dnl
dnl plpgsql fragment to make sure CID and Code are unique.
dnl
dnl Syntax: check_cid_code_unique(newold)
dnl
dnl newold   Which version of the data to check.  Either NEW or OLD.
dnl
dnl This is a kludge. The validation in this trigger implicitly enforces
dnl this, so ideally this is unnecessary. Occasionally some unusual
dnl cases slip past the validation, so we add an explicit check here
dnl just in case.
dnl
dnl Should only be called at the end of these functions, when we "think"
dnl we're done with validation and after automdates are assigned.
dnl
changequote({,})
define({check_cid_code_unique},{
    DECLARE
      dup_sname cycles.sname%TYPE;
      dup_cid cycpoints.cid%TYPE;
      dup_code cycpoints.code%TYPE;
      firstdate cycpoints.date%TYPE;
      lastdate cycpoints.date%TYPE;
    BEGIN
      SELECT INTO dup_sname,    dup_cid, dup_code, firstdate,     lastdate
                  cycles.sname, $1.cid, $1.code, MIN($1.date), MAX($1.date)
        FROM cycles
        WHERE cycles.cid = $1.cid
        GROUP BY cycles.sname, $1.cid, $1.code
          HAVING COUNT(*) > 1
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION integrity_constraint_violation USING
              MESSAGE = 'Error on ' || TG_OP || ' of CYCPOINTS'
            , DETAIL = 'Key (CId) = (' || dup_cid
                         || '): Value (CYCLES.Sname) = (' || dup_sname
                         || '): Value (Code) = (' || dup_code
                         || '): Every CId-Code pair in CYCPOINTS must'
                         || ' be unique (Found duplicates on ' || firstdate
                         || ' and ' || lastdate || ')';
        RETURN NULL;
      END IF;
    END;
})dnl
changequote(`,')dnl
--  
-- cycpoints
--
SELECT 'cycpoints' AS table;

-----------------------------------------------------------------
--SELECT 'cycpoints_before_func' AS function;
--CREATE FUNCTION cycpoints_before_func()
--  RETURNS trigger
--  LANGUAGE plpgsql
--  bb_function_set_search_path
--  AS $$
--  DECLARE
--
--  BEGIN
--  -- Before function for cycpoints insert and update triggers
--  --
--  -- GPL_notice(`  --', `2006, 2008', `Karl O. Pinc <kop@meme.com>')
--  --
--  -- Remarks:
--  --
--  -- Bugs:
--
--  RETURN NEW;
--  END;
--$$;

-----------------------------------------------------------------
SELECT 'cycpoints_func' AS function;
CREATE FUNCTION cycpoints_func()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  DECLARE
    this_sname cycles.sname%TYPE;

  BEGIN
  -- Function for cycpoints insert and update triggers
  --
  -- GPL_notice(`  --', `2005, 2006, 2008, 2010, 2011', `Karl O. Pinc <kop@meme.com>')
  --
  -- Remarks:
  --   The cycpoints table is sometimes updated by the other triggers
  --  with "UPDATE cycpoints SET cpid = cpid WHERE cpid = X" in order
  --  to make sure the automatic Mdates are re-adjusted in response
  --  to the change made to the other table.
  --

--declare
--t text:=timeofday();
--begin
--RAISE INFO '%: Cycpoints %: %: Cid %: %', TG_OP, NEW.code, NEW.date, NEW.cid, t;
--end;

  -- cpid cannot change
  IF TG_OP = 'UPDATE' THEN
    cannot_change(`CYCPOINTS', `Cpid')
    cannot_change(`CYCPOINTS', `Code')
    cannot_change(`CYCPOINTS', `Source')

    -- Can't move a cycle between females by switching cids.
    IF NEW.cid <> OLD.cid THEN
      PERFORM 1
        FROM cycles AS newcycle, cycles AS oldcycle
        WHERE newcycle.cid = NEW.cid
              AND oldcycle.cid = OLD.cid
              AND newcycle.sname <> oldcycle.sname
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION 'CYCPOINTS %: %: %: Cannot change female''s Sname by changing Cid'
                        , OLD.cpid
                        , OLD.code
                        , OLD.date;
        RETURN NULL;
      END IF;
    END IF;
  END IF;

--  IF NEW.code = 'bb_bleeding' THEN
--    -- Don't do anything.
--    RETURN NEW;
--  END IF;

  --
  -- Get the sname of the new cycle, we'll need it a lot.
  --
  SELECT INTO this_sname cycles.sname
    FROM cycles
    WHERE cycles.cid = NEW.cid;


  --
  -- Make/destroy automatic Mdates.
  --

  IF NOT(NEW.code = 'bb_menses' AND NEW.source = 'bb_auto') THEN
    -- Automatic Mdates do not get validated.
    DECLARE
      ddate_row cycpoints;

    BEGIN
      --
      --   Get rid of any relevant automatically generated Mdates
      -- and then re-create them as required.
      --
      -- This is not particularly efficient as we could destroy and
      -- re-create automatic mdates up to 2 times more than necessary.
      --
      --


      IF TG_OP = 'UPDATE' THEN
        _handle_cycpoint_deletion()
      END IF; -- TG_OP = UPDATE

      --
      -- Find the prior Ddate.
      -- (Turns out that we can always affect an earlier automatic Mdate.)
      --
      SELECT INTO ddate_row
                  cycpoints.*
        FROM cycles, cycpoints
        WHERE cycles.sname = this_sname
              AND cycpoints.cid = cycles.cid
              AND cycpoints.date < NEW.date
              AND cycpoints.code = 'bb_deturgesence'
        ORDER BY cycpoints.date DESC
        LIMIT 1;

      --
      -- Create the new automatic Mdate.
      --

      IF NEW.code = 'bb_deturgesence' THEN

        IF ddate_row.cpid IS NOT NULL
           AND ddate_row.date >= NEW.date
                                - CAST('bb_autom_interval days' AS INTERVAL) THEN
          -- The ddate can affect automatic Mdate placement of
          -- the prior Ddate's automatic Mdate.
          PERFORM _remove_autom(this_sname, ddate_row.date, NEW.cpid);
          PERFORM _add_autom(this_sname, ddate_row);
        END IF;

        -- Recreate automatic Ddate from this Ddate.
        PERFORM _add_autom(this_sname, NEW);

      ELSEIF NEW.code = 'bb_turgesence' THEN
        IF ddate_row.cpid IS NOT NULL
           AND ddate_row.date >= NEW.date
                                - CAST('bb_autom_interval days' AS INTERVAL) THEN
          -- The Tdate can affect the automatic Mdate placement.

          -- Find the next cycpoints row.
          -- If it's a automatic Mdate then delete it.
          PERFORM _remove_autom(this_sname, ddate_row.date, NEW.cpid);
          -- Recreate automatic Mdate for old Ddate.
          PERFORM _add_autom(this_sname, ddate_row);
        END IF;

      ELSE
        -- NEW.code = 'bb_menses'

        IF ddate_row.cpid IS NOT NULL THEN
          -- Find the next cycpoints row.
          -- If it's an automatic Mdate and there's no gap between
          -- This row and the prior Ddate then delete it.
          PERFORM _remove_autom(this_sname, ddate_row.date, NEW.cpid);
          -- Recreate automatic Mdate for old Ddate.
          -- (Just in case there's gaps or other partial cycles etc.,
          -- and we just removed something we should not have.)
          PERFORM _add_autom(this_sname, ddate_row);
        END IF;
      END IF;
    END;

    --
    -- Validate the data
    --

    IF TG_OP = 'INSERT' THEN
      -- Quick and dirty, to allow duplicate date tests.
      -- We do more extensive testing for replacement of
      -- an automatic M date later.

      -- Delete an automatic Mdate if we're replacing it with
      -- a regular Mdate.
      IF NEW.code = 'bb_menses' AND NEW.source <> 'bb_auto' THEN
        DELETE
          FROM cycpoints
          USING cycles
          WHERE cycpoints.code = 'bb_menses'
                AND cycpoints.source = 'bb_auto'
                AND cycpoints.date = NEW.date
                AND cycpoints.cid = cycles.cid
                AND cycles.sname = this_sname;
      END IF;
    END IF;

    -- Mdates and Tdates can be the same date, as can
    -- Bdates and Tdates and Bdates and Ddates, otherwise
    -- you can't share a date.
    PERFORM 1
      FROM cycpoints
         , cycles AS that_cycle
      WHERE that_cycle.sname = this_sname
            AND cycpoints.cid = that_cycle.cid
            AND cycpoints.date = NEW.date
            AND cycpoints.cpid <> NEW.cpid
            AND NOT((NEW.code = 'bb_menses'
                     AND cycpoints.code = 'bb_turgesence')
                    OR (NEW.code = 'bb_turgesence'
                        AND cycpoints.code = 'bb_menses')
-- Comment out the Bdate stuff
--                    OR (NEW.code = 'bb_turgesence'
--                        AND (cycpoints.code = 'bb_menses'
--                             OR cycpoints.code = 'bb_bleeding'))
--                    OR (NEW.code = 'bb_bleeding'
--                        AND (cycpoints.code = 'bb_turgesence'
--                             OR cycpoints.code = 'bb_deturgesence'))
--                    OR (NEW.code = 'bb_deturgesence'
--                        AND cycpoints.code = 'bb_bleeding')
)
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION '%: CYCPOINTS.Cpid %: %: %: Bad date; Another CYCPOINTS row exists'
                    , this_sname
                    , NEW.cpid
                    , NEW.date
                    , NEW.code;
      RETURN NULL;
    END IF;

    --
    -- Pregs related (reverse rules)
    --

    -- Can't change a ddate to something else if it's used for a
    -- conception.
    -- (If this is a new insert, then it can't be used for a conception.)
    IF TG_OP = 'UPDATE' THEN
      IF OLD.code = 'bb_deturgesence' AND NEW.code <> 'bb_deturgesence' THEN
        PERFORM 0
          FROM pregs
          WHERE pregs.conceive = NEW.cpid
          LIMIT 1;
        IF FOUND THEN
          RAISE EXCEPTION '%: CYCPOINTS %: Referenced in PREGS.Conceive, cannot change Code from bb_deturgesence'
                          , this_sname
                          , NEW.cpid;
          RETURN NULL;
        END IF;
      END IF;
    END IF;

    -- Can't change a tdate to something else if it's used for a
    -- resume.
    -- (If this is a new insert, then it can't be used for a resumption.)
    IF TG_OP = 'UPDATE' THEN
      IF OLD.code = 'bb_turgesence' AND NEW.code <> 'bb_turgesence' THEN
        PERFORM 0 FROM pregs
          WHERE pregs.resume = NEW.cpid
          LIMIT 1;
        IF FOUND THEN
          RAISE EXCEPTION '%: CYCPOINTS %: Referenced in PREGS.Resume, cannot change Code from bb_turgesence'
                          , this_sname
                          , NEW.cpid;
          RETURN NULL;
        END IF;
      END IF;
    END IF;

    --
    -- Can't have an Mdate in a sexual cycle associated with resumption
    -- of pregnancy.
    --

    -- New/changed menses case.
    IF NEW.code = 'bb_menses' THEN
      PERFORM 1 FROM pregs, cycpoints
        WHERE cycpoints.cid = NEW.cid
              AND pregs.resume = cycpoints.cpid
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION '%: CYCPOINTS %: Cannot have a Mdate in a cycle associated with a PREGS.Resume'
                        , this_sname
                        , NEW.cpid;
        RETURN NULL;
      END IF;
    END IF;

    -- Can't have any other cycpoints between a conception cycle and
    -- a resumption cycle.  In the pregs trigger this is taken care of
    -- by checking that the resume cycle seq is 1 more than the conception
    -- cycle sequence.  This should take care of the same check.
    PERFORM 1
      FROM pregs
           , cycpoints AS ddate
           , cycpoints AS resumedate
           , cycpoints AS badpoints
           , cycles AS this_cycle
           , cycles AS dcycle
           , cycles AS badcycles
      WHERE this_cycle.cid = NEW.cid
            AND dcycle.sname = this_cycle.sname
            AND ddate.cid = dcycle.cid
            AND ddate.cpid = pregs.conceive
            AND resumedate.cpid = pregs.resume
            AND badcycles.sname = this_cycle.sname
            AND badpoints.cid = badcycles.cid
            AND badpoints.date >= ddate.date
            AND badpoints.cpid <> pregs.conceive
            AND badpoints.date <= resumedate.date
            AND badpoints.cpid <> pregs.resume
       LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION 'CYCPOINTS %: %: %: Cannot have any CYCPOINTS between a PREGS conception and it''s resumption'
                      , NEW.cpid
                      , NEW.date
                      , NEW.code;
      RETURN NULL;
    END IF;

    -- Change in cycle case.
    IF TG_OP = 'UPDATE' THEN
      IF NEW.cid <> OLD.cid THEN
        PERFORM 1 FROM pregs, cycpoints
          WHERE pregs.resume = NEW.cpid
                AND cycpoints.cid = NEW.cid
                AND cycpoints.code = 'bb_menses'
          LIMIT 1;
        IF FOUND THEN
          RAISE EXCEPTION '%: CYCPOINTS %: Cannot have a Mdate in a cycle associated with a PREGS.Resume'
                          , this_sname
                          , Cpid;
          RETURN NULL;
        END IF;
      END IF;
    END IF;

    --
    -- SAMPLES related
    -- 

    -- Can't have a juvenile sample once there's a conception.
    PERFORM 1
      FROM samples, pregs, cycles
      WHERE cycles.cid = NEW.cid
            AND samples.sname = cycles.sname
            AND samples.stype = 'bb_juvenile_point'
            AND NEW.date <= samples.date
            AND pregs.conceive = NEW.cpid
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION '%: %: %: Can''t have a juvenile sample on SAMPLES once there''s a conception'
                      , this_sname
                      , NEW.date
                      , NEW.code;
      RETURN NULL;
    END IF;

    --
    -- Maturation date tests
    --

    -- Get maturation date
    DECLARE

      this_matured maturedates.matured%TYPE;
      this_mstatus maturedates.mstatus%TYPE;

    BEGIN

      SELECT INTO this_matured, this_mstatus
                  matured,      mstatus
        FROM maturedates
        WHERE sname = this_sname
        LIMIT 1;
      IF NOT FOUND THEN
        RAISE EXCEPTION 'CYCPOINTS %: %: %: No MATUREDATES row'
                      , NEW.cpid
                      , NEW.date
                      , this_sname;
        RETURN NULL;
      END IF;


      -- When Mstatsus is "ON" set Matured to the earliest Tdate.
      -- (Unfortuately, can't use CASE to force short circuit
      -- evaluation so we don't accidently evaluate OLD when
      -- it does not exist.  So we make a variable.)
      DECLARE
        some_turgesence BOOLEAN;

      BEGIN
        IF TG_OP = 'UPDATE' THEN
          some_turgesence := OLD.code = 'bb_turgesence'
                             OR NEW.code = 'bb_turgesence';
        ELSE 
          some_turgesence := NEW.code = 'bb_turgesence';
        END IF;
        IF this_mstatus = 'bb_on' AND some_turgesence THEN
          -- Earliest Tdate may have changed.
          PERFORM 1
            FROM (SELECT that_cycpoint.date
                    FROM cycles AS that_cycle
                       , cycpoints AS that_cycpoint
                    WHERE that_cycle.sname = this_sname
                          AND that_cycpoint.cid = that_cycle.cid
                          AND that_cycpoint.code = 'bb_turgesence'
                    ORDER BY that_cycpoint.date
                    LIMIT 1) as min_tdate
            WHERE min_tdate.date = NEW.date;
          IF FOUND THEN
            UPDATE maturedates
              SET matured = NEW.date
              WHERE sname = this_sname;
            this_matured := NEW.date;
          END IF;
        END IF;
      END;

      IF NOT(NEW.code = 'bb_bleeding') THEN
        -- Deal with M, T, and Ddates.

-- Comment out for Bdates.  Already tested when selected.
--        -- Must have a maturation date.
--        IF this_matured IS NULL THEN
--          RAISE EXCEPTION 'CYCPOINTS %: %; No maturity date on MATUREDATES (and Code is not bb_bleeding)',
--                         , NEW.cpid
--                         , this_sname;
--          RETURN NULL;
--        END IF;

        -- No Mdates, Tdates or Ddates dates before the maturation date.
        IF this_matured > NEW.date THEN
          RAISE EXCEPTION 'CYCPOINTS %: %: %: No Mdates, Tdates or Ddates before MATUREDATES.Matured'
                         , NEW.cpid
                         , NEW.date
                         , this_sname;
          RETURN NULL;
        END IF;

        -- Not a rule any more
--        -- Tdate can be on the maturity date, when it's an "ON" date,
--        -- but never M or Ddates.
--        IF this_matured = NEW.date AND NEW.Code <> 'bb_turgesence' THEN
--          RAISE EXCEPTION 'CYCPOINTS %: %: %: Mdates and Ddates must be after MATUREDATES.Matured'
--                        , NEW.cpid
--                        , NEW.date
--                        , this_sname;
--          RETURN NULL;
--        END IF;
        IF NEW.code = 'bb_turgesence' THEN
--          IF this_matured = NEW.date 
--             AND this_mstatus <> 'bb_on' THEN
--            RAISE EXCEPTION 'CYCPOINTS %: %: %: Tdates must be after MATUREDATES.Matured when Mstatus is not bb_on'
--                          , NEW.cpid
--                          , NEW.date
--                          , this_sname;
--            RETURN NULL;
--          END IF;

          -- Tdates must be after the birthdate.
          PERFORM 1
            FROM BIOGRAPH
            WHERE sname = this_sname
                  AND birth >= NEW.date;
          IF FOUND THEN
            RAISE EXCEPTION 'CYCPOINTS %: %: %: Tdates must be after the birthdate'
                          , NEW.cpid
                          , NEW.date
                          , this_sname;
            RETURN NULL;
          END IF;
        END IF;

--      ELSE
-- Comment out for Bdates.
--        -- A Bdates
--        IF this_mstatus = 'bb_on' THEN
--          -- Can't have Bdates before the ON date.
--          IF NEW.date < this_matured THEN
--            RAISE EXCEPTION 'CYCPOINTS %: %: %: Cannot have Bdates before MATUREDATES.Matured when Mstatus is bb_on'
--                          , NEW.cpid
--                          , NEW.date
--                          , this_sname;
--            RETURN NULL;
--          END IF;
--        ELSE
--          -- Bdates must fall after the birthdate.
--dnl       check_gt(`CYCPOINTS', `Date', `BIOGRAPH', `Birth')
--        END IF;
      END IF;
    END;

    -- All early dates must fall after the birthdate.
    PERFORM 1
      FROM biograph
      WHERE biograph.sname = this_sname
            AND biograph.Birth >= NEW.Edate
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION '%: Cpid %: Edate %: Code %: BIOGRAPH.Birth cannot be bigger than (after) CYCPOINTS.
Edate'
                      , this_sname
                      , NEW.cpid
                      , NEW.edate
                      , NEW.code;
      RETURN NULL;
    END IF;

    -- All Dates must be on or before the Statdate.
    PERFORM 1
      FROM biograph
      WHERE biograph.sname = this_sname
            AND biograph.Statdate < NEW.Date
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION '%: Cpid %: Date %: Code %: BIOGRAPH.Statdate cannot be less than CYCPOINTS.Date'
                      , this_sname
                      , NEW.cpid
                      , NEW.date
                      , NEW.code;
      RETURN NULL;
    END IF;

    -- All Edates must be on or before the Statdate.
    PERFORM 1
      FROM biograph
      WHERE biograph.sname = this_sname
            AND biograph.Statdate < NEW.edate
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION '%: Cpid %: Edate %: Code %: BIOGRAPH.Statdate cannot be less than CYCPOINTS.Edate'
                      , this_sname
                      , NEW.cpid
                      , NEW.edate
                      , NEW.code;
      RETURN NULL;
    END IF;

    --
    -- Check against gaps in observation.
    --

    IF NOT(NEW.code = 'bb_deturgesence' AND NEW.source = 'bb_estimated') THEN
      -- Check all the kinds of dates to be sure there's none when
      -- there's a gap in observation.
      check_against_gap(`Date')
      check_against_gap(`Edate')
      check_against_gap(`Ldate')
    END IF;

  END IF; -- Test for automatic Mdate.

  --
  -- DAD_DATA
  --

  IF TG_OP = 'UPDATE' THEN
    -- Isn't a conception unless is an update.  
    IF NEW.code = 'bb_deturgesence' THEN
      -- Make sure the dad of the conception is living at the time.
      _living_dad(`dad_excl')
      _living_dad(`dad_1perr')
      _living_dad(`dad_5perr')
      _living_dad(`dad_allmales')
      _living_dad(`dad_consensus')
    END IF;
  END IF;

  --
  -- Done checking.
  -- Automatically update other tables.
  --

  --
  -- Place into cycles
  -- We have a cid, but it may not be right now that things have
  -- changed.  This is where the maximum one M/T/Ddate per cycle is ensured.
  --
  DECLARE
    prior_code cycpoints.code%TYPE;
    prior_cid cycpoints.cid%TYPE;
    next_code cycpoints.code%TYPE;
    next_cid cycpoints.cid%TYPE;
    now_code cycpoints.code%TYPE;
    now_cid cycpoints.cid%TYPE;

    prior_cycle cycles.cid%TYPE;  -- cid of cycle that comes before this, if any
  BEGIN
    --
    -- Before we make any changes, find the cid of the prior cycle
    -- (not separated from us by a gap), if any.
    -- We need it for updating the sexskins.
    -- (It is only the temporal location of the NEW row that
    -- we're interested in, because that'd be the cycle that might
    -- have some of it's sexskins transferred into our cycle.
    -- The cycle next to the OLD location can only pick up some of
    -- this cycle's sexskins.)
    --
--declare
--t text:=timeofday();
--begin
--RAISE INFO '%: Cycpoints % before find prior cid: %: Cid %: %', TG_OP, NEW.code, NEW.date, NEW.cid, t;
--end;
    SELECT INTO prior_cycle cycpoints.cid
      FROM cycles, cycpoints
      WHERE cycles.sname = this_sname
            AND cycpoints.cid = cycles.cid
            -- AND cycpoints.code <> 'bb_bleeding'
            AND cycpoints.date < NEW.date
            AND cycpoints.cid <> NEW.cid
            AND NOT EXISTS (SELECT 1
                              FROM cycgapdays
                              WHERE cycgapdays.sname = this_sname
                                    AND cycgapdays.date < NEW.date
                                    AND cycgapdays.date > cycpoints.date)
      ORDER BY cycpoints.date DESC, cycpoints.code DESC
      LIMIT 1;
--declare
--t text:=timeofday();
--begin
--RAISE INFO '%: Cycpoints % before prior cycpoints: %: Cid %: %', TG_OP, NEW.code, NEW.date, NEW.cid, t;
--end;

    --
    -- Find the previous and the following MTD cycpoints.
    -- They are not interesting if separated by a gap.
    -- 
    SELECT INTO prior_code,     prior_cid
                cycpoints.code, cycpoints.cid
      FROM cycles, cycpoints
      WHERE cycles.sname = this_sname
            AND cycpoints.cid = cycles.cid
            AND cycpoints.date < NEW.date
-- No Bdates
--            AND cycpoints.code <> 'bb_bleeding'
            AND NOT EXISTS (SELECT 1
                              FROM cycgapdays
                              WHERE cycgapdays.sname = this_sname
                                    AND cycgapdays.date < NEW.date
                                    AND cycgapdays.date > cycpoints.date)
      ORDER BY cycpoints.date DESC
      LIMIT 1;
--declare
--t text:=timeofday();
--begin
--RAISE INFO '%: Cycpoints % before next cycpoints: %: Cid %: %', TG_OP, NEW.code, NEW.date, NEW.cid, t;
--end;

    SELECT INTO next_code,      next_cid
                cycpoints.code, cycpoints.cid
      FROM cycles, cycpoints
      WHERE cycles.sname = this_sname
            AND cycpoints.cid = cycles.cid
            AND cycpoints.date > NEW.date
-- No Bdates
--            AND cycpoints.code <> 'bb_bleeding'
            AND NOT EXISTS (SELECT 1
                              FROM cycgapdays
                              WHERE cycgapdays.sname = this_sname
                                    AND cycgapdays.date > NEW.date
                                    AND cycgapdays.date < cycpoints.date)
      ORDER BY cycpoints.date
      LIMIT 1;
--declare
--t text:=timeofday();
--begin
--RAISE INFO '%: Cycpoints % before now cycpoints: %: Cid %: %', TG_OP, NEW.code, NEW.date, NEW.cid, t;
--end;

    SELECT INTO now_code,       now_cid
                cycpoints.code, cycpoints.cid
      FROM cycles, cycpoints
      WHERE cycles.sname = this_sname
            AND cycpoints.cid = cycles.cid
            AND cycpoints.date = NEW.date
-- No Bdates
--            AND cycpoints.code <> 'bb_bleeding'
            AND cycpoints.cpid <> NEW.cpid;

--declare
--t text:=timeofday();
--begin
--RAISE INFO '%: Cycpoints % after now cycpoints: %: Cid %: %', TG_OP, NEW.code, NEW.date, NEW.cid, t;
--end;
    --
    -- Look at where the new row fits in to CYCLES
    -- on a code-by-code basis.
    --
    IF NEW.code = 'bb_deturgesence' THEN
      --
      -- Put Ddate into cycles
      --

      IF prior_code = 'bb_deturgesence'
         OR prior_code IS NULL THEN
        -- NEW's Ddate follows a ddate, possibly separating it from
        -- other cycpoints in the same cycle.
        -- Or, there is no prior cycle and NEW needs to be
        -- in it's own cycle.

        -- Is the Ddate already in a cycle with no other cycpoints?
        PERFORM 1
          FROM cycpoints
          WHERE cycpoints.cid = NEW.cid
                AND cycpoints.cpid <> NEW.cpid
          LIMIT 1;
        IF FOUND THEN  -- No there are other cycpoints in the cycle
          -- Need to make a new cycle for the cycpoints rows that
          -- shared the cycle with the ddate.
          INSERT INTO cycles (sname) VALUES(this_sname);
          -- (The update will re-trigger this trigger and set the
          -- new cycle's seq and series properly.)
          UPDATE cycpoints
            SET cid = currval('cycles_cid_seq')
            WHERE cid = NEW.cid
                  AND cpid <> NEW.cpid;
        END IF;

      ELSE
        -- NEW's Ddate belongs with the prior cycle.

        IF NEW.cid <> prior_cid THEN
          -- How do we know which cid to use?
          -- Give NEW the prior_cid as that way there's no
          -- getting the prior M/T row(s) in with
          -- the M/Tdates the Ddate used to have.
          UPDATE cycpoints
            SET cid = prior_cid
            WHERE cpid = NEW.cpid;
          NEW.cid := prior_cid;
        END IF;

        IF next_cid = prior_cid THEN
          -- Split subsequent cycpoints into a new cycle.
          INSERT INTO cycles (sname) VALUES(this_sname);
          -- Again, the new invocation of this trigger should update
          -- the series and seq of the new cycle.
          UPDATE cycpoints
            SET cid = currval('cycles_cid_seq')
            WHERE cycpoints.cid = prior_cid
                  AND cycpoints.date > NEW.date;
        END IF;
      END IF;

    ELSEIF NEW.code = 'bb_turgesence' THEN
      --
      -- Put Tdate into cycles.
      --

      IF now_code = 'bb_menses' THEN
        -- Ooch, Mdates and Tdates can share a date.
        -- In which case they fall in the same cycle.
        IF NEW.cid <> now_cid THEN
          UPDATE cycpoints SET cid = now_cid WHERE cpid = NEW.cpid;

          IF next_code = 'bb_turgesence' THEN
            -- (The assumption here is that the data is right and
            -- the M and Tdate are already in the same cycle.)
            -- Split subsequent cycpoints into a new cycle.
            INSERT INTO cycles (sname) VALUES (this_sname);
            -- Again, the new invocation of this trigger should update
            -- the series and seq of the new cycle.
            UPDATE cycpoints
              SET cid = currval('cycles_cid_seq')
              WHERE cyclepoints.cid = NEW.cid
                    AND date > NEW.date;
          END IF;

          NEW.cid := now_cid;
        END IF;

      ELSEIF prior_code = 'bb_menses' THEN
        -- NEW's Tdate belongs with the prior cycle.
        IF NEW.cid <> prior_cid THEN
          UPDATE cycpoints
            SET cid = prior_cid
            WHERE cpid = NEW.cpid;

          IF next_code = 'bb_turgesence' THEN
            -- Split subsequent cycpoints into a new cycle.
            -- (The assumption here is that the data is right and
            -- the M and Tdate are already in the same cycle.)
            INSERT INTO cycles (sname) values(this_sname);
            -- Again, the new invocation of this trigger should update
            -- the series and seq of the new cycle.
            UPDATE cycpoints
              SET cid = currval('cycles_cid_seq')
              WHERE cycpoints.cid = NEW.cid
                    AND cycpoints.date > NEW.date;
          END IF;

          NEW.cid := prior_cid;
        END IF;

      ELSEIF next_code = 'bb_deturgesence' THEN
        -- The NEW Tdate belongs in the same cycle as the
        -- Ddate that follows it.
        IF NEW.cid <> next_cid THEN
          UPDATE cycpoints SET cid = next_cid WHERE cpid = NEW.cpid;
          NEW.cid := next_cid;
        END IF;

      ELSE
        -- The Tdate does not go with either the previous or the subsequent
        -- cycle.

        -- Is the Tdate already in a cycle with no other cycpoints?
        PERFORM 1
          FROM cycpoints
          WHERE cid = NEW.cid
                AND cpid <> NEW.cpid
          LIMIT 1;
        IF FOUND THEN  -- No there are other cycpoints in the cycle
          -- We need our own (new) cycle.
          INSERT INTO cycles (sname) VALUES (this_sname);
          UPDATE cycpoints
            SET cid = currval('cycles_cid_seq')
            WHERE cpid = NEW.cpid;
          NEW.cid := currval('cycles_cid_seq');
        END IF;
      END IF;

    ELSEIF NEW.code = 'bb_menses' THEN
      --
      -- Put Mdate into cycles.
      --

      IF now_code = 'bb_turgesence' THEN
        -- Attach the cycle to the cycle of the Tdate.
        IF NEW.cid <> now_cid THEN
          UPDATE cycpoints SET cid = now_cid WHERE cpid = NEW.cpid;
        END IF;
      ELSEIF next_code <> 'bb_menses'
         AND next_code IS NOT NULL THEN
        IF NEW.cid <> next_cid THEN
          -- Attach to the cycle of the next cycpoint.
          UPDATE cycpoints SET cid = next_cid WHERE cpid = NEW.cpid;
          NEW.cid := next_cid;
        END IF;
      ELSE
        -- We need our own (new) cycle.

        -- Is the Mdate already in a cycle with no other cycpoints?
        PERFORM 1
          FROM cycpoints
          WHERE cid = NEW.cid
                AND cpid <> NEW.cpid
          LIMIT 1;
        IF FOUND THEN  -- No there are other cycpoints in the cycle
          -- We don't have our own cycle, give us one.
          INSERT INTO cycles (sname) VALUES (this_sname);
          UPDATE cycpoints
            SET cid = currval('cycles_cid_seq')
            WHERE cpid = NEW.cpid;
          NEW.cid := currval('cycles_cid_seq');
        END IF;
      END IF;
    ELSEIF NEW.code = 'bb_bleeding' THEN
      -- Ugh.  Cant' deal with Bdates right now.
      RAISE EXCEPTION 'Sorry, Bdates are not allowed yet.  (We hateses them. Gollum. Gollum.)';
      RETURN NULL;
      -- Use prior MTD, if any and no gap.
      -- Otherwise, use subsequent TD if any and no gap.
      -- Otherwise, reject change.
      -- (Meanwhile, other changes in MTD dates can move Bdates
      -- between cycles.)
    END IF;

    --
    -- Re-assign the sexskins now that we've messed with the cycles.
    -- There are up to two cycles at issue, the one that the cycpoint
    -- originally belonged in, and may still belong in, and a possible
    -- second cycle that the cycpoints row was moved into.  Fortunately
    -- we only have to deal with the first, because the the update that
    -- creates the second will fire this trigger again and that will
    -- deal with it.
    --
    -- What's going on here is that the cycle after the changed cycle
    -- (any changed cycle in question, removed from or added to)
    -- will never "lose" sexskins to the changed, cycle.  That is, the
    -- sexskins will never move from the cycle after a change to the
    -- cycle changed.  However, the cycle before the change can see
    -- some of it's sexskins transferred to the changed cycle.  So,
    -- we need to re-analyze both the changed cycle and the cycle before
    -- the cycle that changed to get the sexskins re-assigned properly.
    --
    PERFORM _reassign_sexskins(NEW.cid);
    IF prior_cycle IS NOT NULL THEN
      PERFORM _reassign_sexskins(prior_cycle);
    END IF;
    IF TG_OP = 'UPDATE' THEN
      -- We may have moved the cycpoints out from under the sexskins.
      PERFORM _reassign_sexskins(OLD.cid);
    END IF;

  END;

  --
  -- Remove unused CYCLES rows.
  --
  IF TG_OP = 'UPDATE' THEN
    PERFORM 1 FROM cycpoints WHERE cid = OLD.cid LIMIT 1;
    IF NOT FOUND THEN
      -- The cycle row we started with now has no related
      -- cycpoints, get rid of it.
      DELETE FROM cycles WHERE cid = OLD.cid;
    END IF;
  END IF;


  --
  -- Recalculate Seq and Series.
  -- Begin with this, so when we make automatic mdates the
  -- seq and series are right.
  --
  DECLARE
    oldestdate cycpoints.date%TYPE;

  BEGIN
    IF TG_OP = 'INSERT' THEN
      oldestdate := NEW.date;
    ELSEIF NEW.date < OLD.date THEN
      oldestdate := NEW.date;
    ELSE
      oldestdate := OLD.date;
    END IF;

    PERFORM _recalc_seq_series(this_sname, oldestdate, NEW.cid);
  END;
/* Old Seq and Series code that attempts subltey and is, instead, broken.
 *
 *  --
 *  -- Make sure CYCLES is correct.
 *  --
 *  IF NEW.code <> 'bb_bleeding' THEN
 *    PERFORM 1 FROM cycpoints WHERE cid = OLD.cid LIMIT 1;
 *    IF NOT FOUND THEN
 *      -- The cycle row we started with now has no related
 *      -- cycpoints, get rid of it.
 *      -- (This could have been done from OLD, only on update but
 *      -- we need to know whether to update other stuff too.)
 *      DELETE FROM cycles WHERE cid = OLD.cid;
 *    ELSE
 *      -- Update seq
 *      -- (The OLD on update is taken care of by either being left alone
 *      -- with other unchanged cycpoints or by being deleted.  If they
 *      -- change, we get them on the "new" pass through the trigger.)
 *      DECLARE
 *        last_seq cycles.seq%TYPE;
 *        last_date cycpoints.date%TYPE;
 *        last_series cycles.series%TYPE;
 *        next_cycle cycles.cid%TYPE;
 *        next_seq cycles.seq%TYPE;
 *  
 *      BEGIN
 *        -- Find the prior cycle based on cycpoints date and update
 *        -- ourselves and subsequent cycles if necessary.
 *        SELECT INTO last_seq,   last_date,      last_series
 *                    cycles.seq, cycpoints.date, cycles.series
 *          FROM cycles, cycpoints
 *          WHERE cycles.sname = this_sname
 *                AND cycpoints.cid = cycles.cid
 *                AND cycpoints.date < NEW.date
 *                AND cycpoints.cid <> NEW.cid
 *          ORDER BY cycpoints.date DESC
 *          LIMIT 1;
 *        IF NOT FOUND THEN
 *          -- This is the earliest cycle.
 *          last_seq := 0;
 *        END IF;
 *  
 *        -- Set the seq of the current cycle.
 *        UPDATE cycles
 *          SET seq = last_seq + 1
 *          WHERE cid = NEW.cid
 *                AND seq <> last_seq + 1;
 *  
 *        -- Find the next cycle based on cycpoints date
 *        -- and see if it needs to be updated.
 *        -- If so, then all the subsequent cycles need to
 *        -- be updated too.
 *        -- When the next cycle has a NULL seq it will
 *        -- take care of itself and subsequent cycles.
 *        --
 *        SELECT INTO next_seq seq
 *          FROM cycles, cycpoints
 *          WHERE cycles.sname = this_sname
 *                AND cycpoints.cid = cycles.cid
 *                AND cycpoints.date > NEW.date
 *                AND cycpoints.cid <> NEW.cid
 *          ORDER BY cycpoints.date
 *          LIMIT 1;
 *        IF next_seq IS NOT NULL
 *           AND NOT(next_seq = last_seq + 2) THEN
 *          UPDATE cycles
 *            SET seq = seq + 1
 *            WHERE sname = this_sname
 *                  AND seq >= last_seq + 1
 *                  AND cid <> NEW.cid;
 *        END IF;
 *      END;
 *  
 *      --
 *      -- Make sure the series is good.
 *      --
 *      DECLARE
 *        -- Did we (maybe) start a series where there was none?
 *        new_series BOOLEAN;
 *        right_series cycles.series%TYPE;
 *
 *      BEGIN
 *        -- Be handy to have the following defintion of series, but
 *        -- that's not it.
 *        -- -- The series is the number of ends of observation
 *        -- -- that preceede the date, plus the number of points that
 *        -- -- preceede the date, plus 1.
 *        -- SELECT INTO right_series count(*) + 1
 *        --   FROM cycgaps
 *        --   WHERE sname = this_sname
 *        --         AND date < NEW.date
 *        --         AND (code = 'bb_cg_end'
 *        --              OR code = 'bb_cg_start');
 *        -- IF NOT FOUND THEN
 *        --   right_series := 1;
 *        -- END IF;
 *        --
 *
 *        -- Use a definition based on the previous series.
 *        IF last_series IS NULL THEN
 *          -- No prior cycles.  Start at 1.
 *          right_series := 1;
 *          new_series := TRUE;
 *        ELSE
 *          PERFORM 1
 *            FROM cycgapdays
 *            WHERE sname = this_sname
 *                  AND date < NEW.date
 *                  AND date > last_date
 *            LIMIT 1;
 *          IF FOUND THEN
 *            -- There is a gap between us and the last series.
 *            right_series := last_series + 1;
 *            new_series := TRUE;
 *          ELSE
 *            right_series := last_series;
 *            new_series := FALSE;
 *          END IF;
 *        END IF;
 *
 *        UPDATE cycles
 *          SET series = right_series
 *          WHERE cid = NEW.cid
 *                AND series <> right_series;
 *
 *        IF new_series THEN
 *          -- We need to test to see if we really did start a new series
 *          -- in a period of observation where there was none before.
 *          -- If so, then we need to bump the series number of any subsequent
 *          -- series/cycles.
 *          PERFORM 1
 *            FROM cycpoints, cycles AS other, cycgapdays, cycles AS our
 *            WHERE other.sname = this_sname
 *                  AND cycpoints.cid = other.cid
 *                  AND our.cid = NEW.cid
 *                  AND other.seq = our.seq + 1
 *                  AND cycgapdays.date > NEW.date
 *                  AND cycgapdays.date < cycpoints.date
 *            LIMIT 1;
 *          IF FOUND THEN
 *            -- Yep, the next cycle is after a gap.
 *            -- Bump it's series.
 *            UPDATE cycles
 *              SET series = series + 1
 *              FROM cycles AS other
 *              WHERE cycles.sname = this_sname
 *                    AND other.cid = NEW.cid
 *                    AND cycles.seq > other.cid;
 *          END IF;
 *        END IF;
 *      END;
 *    END IF;
 *  END IF;
 */  


  IF NOT(NEW.code = 'bb_menses' AND NEW.source = 'bb_auto') THEN
    -- Automatic Mdates do not get validated.

    --
    -- Deal with ongoing pregnancies.
    --

    --
    -- Update PREGS.Resume with the Tdate, if it's right
    -- after the conception cycle and there's no gap.
    -- If that is not necessary, then check the rules
    -- with respect to a NULL Resume:
    --
    -- Make the earliest Tdate before a gap be the pregancy resume
    -- date when there's no resume.
    --
    -- And make sure there's otherwise no cycles after an ongoing
    -- pregnancy.

    DECLARE
      last_gap    cycgapdays.date%TYPE;
      preg_pid    pregs.pid%TYPE;
      preg_date   cycpoints.date%TYPE;
      preg_resume pregs.resume%TYPE;

    BEGIN

      -- Find prior pregnancy, it's date and resume.
      SELECT INTO preg_pid,  preg_date,      preg_resume
                  pregs.pid, cycpoints.date, pregs.resume
        FROM cycles, cycpoints, pregs
        WHERE cycles.sname = this_sname
              AND cycpoints.cid = cycles.cid
              AND cycpoints.date < NEW.date
              AND pregs.conceive = cycpoints.cpid
        ORDER BY cycpoints.date DESC
        LIMIT 1;

      IF FOUND AND preg_resume IS NULL THEN
        -- We do need to either update or validate against the
        -- last pregnancy.

        -- Find the immediately preceeding gap.
        SELECT INTO last_gap cycgapdays.date
          FROM cycgapdays
          WHERE cycgapdays.sname = this_sname
                AND cycgapdays.date < NEW.date
          ORDER BY cycgapdays.date DESC
          LIMIT 1;

        IF (FOUND AND preg_date > last_gap) OR NOT FOUND THEN
          -- The date of this cycpoints row is not separated
          -- from the pregnancy by a gap.  Our work is not yet done.

          -- Are there any cycpoints at all between us and the pregnancy?
          -- (If there are, one of them might be a Tdate and so get rid of
          -- the NULL resume.  If that does not happen, then the earliest
          -- one will generate the error.  (Unless it's an automatic Mdate,
          -- which is not validated and we hope not inserted by the users.)
          PERFORM 1
            FROM cycles, cycpoints
            WHERE cycles.sname = this_sname
                  AND cycpoints.cid = cycles.cid
                  AND cycpoints.date <= NEW.date  -- Tdate could share with Mdate
                  AND cycpoints.date > preg_date
                  AND cycpoints.cpid <> NEW.cpid  -- Tdate could share with Mdate
            LIMIT 1;

          IF NOT FOUND THEN
            IF NEW.code = 'bb_turgesence' THEN
              -- Tdate immediately follows the pregnancy.
              -- Make it the resume date.
              UPDATE pregs
                SET resume = NEW.cpid
                WHERE pid = preg_pid;
            ELSE
              -- Some sexual cycle follows a ongoing pregnancy.
              RAISE EXCEPTION '%: %: %: Ongoing pregnancy (NULL Resume) cannot have subsequent CYCPOINTS (and no intervening gap in observation)'
                  , this_sname
                  , NEW.date
                  , NEW.code;
              RETURN NULL;
            END IF;
          END IF;

--    --
--    -- Female cannot have any cycpoints rows postdating a pregnancy with
--    -- a null resume, unless they postdate a gap that follows the pregnancy.
--    --
--
--      PERFORM 1
--        FROM cycles AS pregcycles
--          JOIN cycpoints AS pregcycpoints
--            ON (pregcycpoints.cid = pregcycles.cid
--                AND pregcycpoints.date < NEW.date
--                AND (pregcycpoints.date > last_gap
--                     OR last_gap IS NULL))
--          JOIN pregs
--            ON (pregs.conceive = pregcycpoints.cpid
--                AND pregs.resume IS NULL)
--        WHERE pregcycles.sname = this_sname
--        LIMIT 1;
--      IF FOUND THEN
-- then raise the execption

        END IF;
      END IF;
    END;
  END IF;  -- 2nd test for automatic Mdate.
  --
  -- Ensure that cycpoints state is correct.
  --

  IF TG_OP = 'UPDATE' THEN
    IF OLD.date <> NEW.date
       OR OLD.code <> NEW.code THEN
      -- Gotta update the state of possibly two observation intervals.
      -- (Could be doing extra work.  Too bad.)
      PERFORM _cycgaps_state_sync(this_sname, TRUE,  OLD);
      PERFORM _cycgaps_state_sync(this_sname, FALSE, NEW);
    END IF;
  ELSE   -- TG_OP = 'INSERT'
    PERFORM _cycgaps_state_sync(this_sname, FALSE, NEW);
  END IF;

  RETURN NULL;

  --
  -- After all of the above, ensure that CYCPOINTS.CId and
  -- Code are still unique.
  --
  check_cid_code_unique(`NEW')
  END;
$$;

-----------------------------------------------------------------
SELECT 'cycpoints_delete_func' AS function;
CREATE FUNCTION cycpoints_delete_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  -- Function for cycpoints delete trigger
  --
  -- GPL_notice(`  --', `2005, 2006, 2008', `Karl O. Pinc <kop@meme.com>')

  DECLARE
    this_sname cycles.sname%TYPE;
    ddate_row cycpoints;

  BEGIN

    -- Find the individual we're dealing with.
    SELECT INTO this_sname cycles.sname
      FROM cycles
      WHERE cycles.cid = OLD.cid;

    -- Re-classify any sexskins associated with the cycle.
    -- Now that things have changed associate them with whatever
    -- cycle is appropriate.
    PERFORM _reassign_sexskins(OLD.cid);

    --
    -- If the cycle row we started with now has no related
    -- cycpoints then get rid of it.
    --
    PERFORM 1 FROM cycpoints WHERE cid = OLD.cid LIMIT 1;
    IF NOT FOUND THEN
      DELETE FROM cycles WHERE cid = OLD.cid;
    END IF;

    -- Update state on the cycgaps table.
    PERFORM _cycgaps_state_sync(this_sname, TRUE, OLD);

    -- Do we need to remove an automatic Mdate?
    IF NOT(OLD.code = 'bb_menses' AND OLD.source = 'bb_auto') THEN
      -- (And it's not a Bdate because we don't have Bdates right now.)
      -- Yup, we might be affecting a automatic Mdate.

      _handle_cycpoint_deletion()
    END IF;

    -- Recalculate the seq and series.
    -- (The cid arg is not used.)
    PERFORM _recalc_seq_series(this_sname, OLD.DATE, OLD.cid);

    RETURN NULL;

  --
  -- Ensure that CYCPOINTS.CId and Code are still unique.
  --
  check_cid_code_unique(`OLD')
  check_cid_code_unique(`NEW')
  END;
$$;


-----------------------------------------------------------------
SELECT '_cycgaps_state_sync' AS function;
CREATE FUNCTION _cycgaps_state_sync(this_sname cycgaps.sname%TYPE
                                  , is_old BOOLEAN
                                  , cp cycpoints)
  RETURNS integer
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  -- Brings cycgaps.state into sync with cycpoints row
  --
  -- GPL_notice(`  --', `2006, 2008', `Karl O. Pinc <kop@meme.com>')
  --
  -- Syntax: _cycgaps_state_sync(this_sname cycgaps.sname%TYPE
  --                           , is_old BOOLEAN
  --                           , cp RECORD)
  --
  -- Input:
  --  this_sname  The individual we are concerned with.
  --  is_old      Whether or not the cycgaps row supplied has
  --              been deleted.  True if the row is the OLD row,
  --              FALSE if the row is the NEW row.
  --  cp          A cycpoints row that's been chagned or
  --              inserted or deleted.
  --
  -- Side Effects:
  --  May alter the cycgaps.state value.
  --
  -- Remarks:
  --
  -- Bugs:

  DECLARE
    obs_start cycgaps.date%TYPE;
    start_code cycgaps.code%TYPE;
    start_state cycgaps.state%TYPE;
    start_gapid cycgaps.gapid%TYPE;

    relevant_date cycpoints.date%TYPE;
    relevant_state cycpoints.code%TYPE;
    obs_end cycgaps.date%TYPE;

  BEGIN
--RAISE INFO '_cycgaps_state_sync(%, %, %)', this_sname, is_old, cp;

  IF cp.code = 'bb_bleeding' THEN
    -- Bleeding does not change cycgaps state.
    RETURN 0;
  END IF;

  IF cp.code = 'bb_deturgesence' AND cp.source = 'bb_estimated' THEN
    -- Oh boy, an estimated Ddate.  These can be in periods of
    -- no observation, in which case nothing needs updating.
    PERFORM 1
      FROM cycgapdays
      WHERE sname = this_sname
            AND date = cp.date
      LIMIT 1;
    IF FOUND THEN
      -- We're in a period of no observation.
      RETURN 0;
    END IF;
  END IF;

  -- Find the period of observation we are in.
  -- (Assume things are valid and we're in period of observation.)
  SELECT INTO obs_start, start_state, start_code, start_gapid
              date,      state,       code,       gapid
    FROM cycgaps
    WHERE sname = this_sname
          AND date <= cp.date
          AND (code = 'bb_cg_start'
               OR code = 'bb_cg_point')
    ORDER BY date DESC
    LIMIT 1;
  IF NOT(FOUND) THEN
    -- There's no gap in observation to update.
    RETURN 0;
  END IF;

  -- Are there any cycpoints rows between us and the start of
  -- observation.  (Ignore bleeding dates.)
  -- This is made tricky by the cycpoints that can occur on the
  -- same date, (Mdates and Tdates)
  PERFORM 1
    FROM cycles, cycpoints
    WHERE cycles.sname = this_sname
          AND cycpoints.cid = cycles.cid
--          AND cycpoints.code <> 'bb_bleeding'
          AND cycpoints.date >= obs_start
          AND ((cp.code = 'bb_turgesence'
                AND cycpoints.date <= cp.date
                AND cycpoints.cid <> cp.cid)
               OR cycpoints.date < cp.date)
     LIMIT 1;
  IF FOUND THEN
    -- Found another cycpoints that affects the start state.
    -- Don't need to update the start of observation.
    RETURN 0;
  END IF;              

  --
  -- We're going to need to update the cycgaps.state.
  --

  IF is_old THEN
    --
    -- The cp row has been deleted/changed.
    -- Find the row that affects the state.
    --

    -- Find the end of observation.
    IF start_code = 'bb_cg_point' THEN
      -- The start of observation is also the end of observation.
      relevant_date := obs_start;
      relevant_state := start_state;
    ELSE

      -- Find the first row after the start of observation.
      -- (Take care for where an M and T date share the same day.)
      SELECT INTO relevant_date,  relevant_state
                  cycpoints.date, cycpoints.code
        FROM cycles, cycpoints
        WHERE cycles.sname = this_sname
              AND cycpoints.cid = cycles.cid
              AND cycpoints.date >= cp.date
              AND cycpoints.code <> 'bb_bleeding'
        ORDER BY cycpoints.date, cycpoints.code
        LIMIT 1;

      -- Look to the future young man, there lies the end of observation.
      -- (We could use the cycles.series here, but, well, we're not.)
      SELECT INTO obs_end
                  cycgaps.date
        FROM cycgaps
        WHERE cycgaps.sname = this_sname
              AND cycgaps.date > cp.date
              AND cycgaps.code = 'bb_cg_end'
        ORDER BY date
        LIMIT 1;
      IF FOUND AND obs_end < relevant_date THEN
        -- End of observation comes before next cycpoints.
        -- No cycpoints in the period of observation.
        -- Don't alter the starting state.
        RETURN 0;
      END IF;
    END IF;
  ELSE
    -- The row being inserted is the earliest after start of obs.
    relevant_date := cp.date;
    relevant_state := cp.code;
  END IF;

  -- Finally, update the cycgaps table.
  DECLARE
    new_state cycgaps.state%TYPE;
    next_ddate cycpoints.date%TYPE;

  BEGIN
    IF relevant_date = obs_start THEN
      -- Base the new state on the cycpoints state.
      IF relevant_state = 'bb_menses' THEN
        new_state := 'bb_cg_menses';
      ELSEIF relevant_state = 'bb_deturgesence' THEN
        new_state := 'bb_cg_deturgesence';
      ELSE
        -- relevant_state is bb_turgesence
        -- State depends on how soon the next Ddate comes.
        -- The next Ddate will be in the same cycle (or it's
        -- after a gap and we don't want it.)
        SELECT INTO next_ddate
                    ddate.date
          FROM cycles, cycpoints AS ddate, cycpoints AS tdate
          WHERE cycles.sname = this_sname
                AND tdate.date = relevant_date
                AND tdate.code = 'bb_turgesence'
                AND cycles.cid = tdate.cid
                AND ddate.cid = cycles.cid
                AND ddate.code = 'bb_deturgesence';
        IF NOT FOUND
           OR next_ddate > relevant_date
                           + CAST('bb_ovulation_len days' AS interval) THEN
          new_state := 'bb_cg_swelling';
        ELSE
          new_state := 'bb_cg_ovulating';
        END IF;
      END IF;
    ELSE
      -- First cycpoints row is after observation starts.
      IF relevant_state = 'bb_menses' THEN
        new_state = 'bb_cg_deturgesence';
      ELSEIF relevant_state = 'bb_deturgesence' THEN
        IF relevant_date > obs_start
                           + CAST('bb_ovulation_len days' AS interval) THEN
          new_state := 'bb_cg_swelling';
        ELSE
          new_state := 'bb_cg_ovulating';
        END IF;
      ELSE
         -- Tdate.  There are 3 possiblities for Tdates so do nothing.
         -- We can't check the cycgap's state here because changing it
         -- to what would now be a legal value, (bb_cg_menses, bb_cg_pregnant,
         -- or bb_cg_lactating) would be an illegal value before now.
         -- So, this is checked 'in batch', and must be fixed manually.
         new_state := start_state;
      END IF; 
    END IF;

    IF new_state <> start_state THEN
      UPDATE cycgaps
        SET state = new_state
        WHERE gapid = start_gapid;
    END IF;
  END;

  RETURN NULL;
  END;
$$;
grant_func_priv(`_cycgaps_state_sync(cycgaps.sname%TYPE, BOOLEAN, cycpoints)')

--SELECT 'cycpoints_before_trigger' AS trigger;
--CREATE TRIGGER cycpoints_before_trigger
--  BEFORE INSERT OR UPDATE
--  ON cycpoints FOR EACH ROW
--  EXECUTE PROCEDURE cycpoints_before_func();

SELECT 'cycpoints_trigger' AS trigger;
CREATE TRIGGER cycpoints_trigger
  AFTER INSERT OR UPDATE
  ON cycpoints FOR EACH ROW
  EXECUTE PROCEDURE cycpoints_func();

SELECT 'cycpoints_delete_trigger' AS trigger;
CREATE TRIGGER cycpoints_delete_trigger
  AFTER DELETE
  ON cycpoints FOR EACH ROW
  EXECUTE PROCEDURE cycpoints_delete_func();
