dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2010, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl


dnl plpgsql fragment for BLAH
dnl
dnl Syntax: BLAH(ARG)
dnl
dnl ARG   description
dnl
changequote({,})
define({BLAH},{
})dnl
changequote(`,')dnl

dnl plpgsql fragment for validating a dad column
dnl
dnl Syntax: _dad_check(col)
dnl
dnl col   The name of the column holding a potential dad.
dnl
changequote({,})
define({_dad_check},{
  IF NEW.$1 IS NOT NULL THEN
    -- The dad must be male.
    PERFORM 1
      FROM biograph
      WHERE biograph.sname = NEW.$1
            AND biograph.sex <> 'bb_male'
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION 'Dadid %: $1 %: DAD_DATA.$1 must be male'
                    , NEW.dadid
                    , NEW.$1;
      RETURN NULL;
    END IF;

    -- Test against the conception date.
    DECLARE
      zdate cycpoints.date%TYPE;
    BEGIN
      SELECT cycpoints.date INTO zdate
        FROM biograph, pregs, cycpoints
        WHERE biograph.sname = NEW.kid
              AND pregs.pid = biograph.pid
              AND cycpoints.cpid = pregs.conceive
        LIMIT 1;
      IF zdate IS NOT NULL THEN
        -- We know the conception date.

        -- $1 must be alive during the mother's fertile period.
        PERFORM 1
          FROM biograph
          WHERE biograph.sname = NEW.$1
                AND biograph.statdate < zdate
                                        - bb_ovulation_len
                                        - CASE
                                            WHEN biograph.status = 'bb_alive'
                                              THEN bb_interpolation_max
                                            ELSE 0
                                          END
          LIMIT 1;
        IF FOUND THEN
          RAISE EXCEPTION 'Dadid %: $1 %: Kid %: Zdate %: $1 is not alive during the mom''s fertile period -- on or after the Zdate - bb_ovulation_len (- another bb_interpolation_max if the dad is alive)'
                        , NEW.dadid
                        , NEW.$1
                        , NEW.kid
                        , zdate;
          RETURN NULL;
        END IF;

        -- $1 must be mature as of the zdate.
        PERFORM 1
          FROM maturedates
          WHERE maturedates.sname = NEW.$1
                AND maturedates.matured < zdate
          LIMIT 1;
        IF NOT FOUND THEN
          RAISE EXCEPTION 'Dadid %: $1 %: Kid %: Zdate %: $1 must be mature before the Zdate'
                        , NEW.dadid
                        , NEW.$1
                        , NEW.kid
                        , zdate;
          RETURN NULL;
        END IF;
      END IF;
    END;
  END IF;
})dnl
changequote(`,')dnl

--  
-- dad_data
--
SELECT 'dad_data' AS table;

SELECT 'dad_data_func' AS function;
CREATE FUNCTION dad_data_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  BEGIN
  -- Function for dad_data insert and update triggers
  --
  -- GPL_notice(`  --', `2010', `Karl O. Pinc <kop@meme.com>')

  -- The birth date must not be NULL.
  PERFORM 0
    FROM biograph
    WHERE biograph.sname = NEW.kid
          AND biograph.birth IS NULL
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'Dadid %: Kid %: The BIOGRAPH.birth of the offspring cannot be NULL'
                   , NEW.dadid
                   , NEW.kid;
    RETURN NULL;
  END IF;

  IF NEW.mom_sampled IS NULL THEN
    PERFORM 1
      FROM biograph
      WHERE biograph.sname = NEW.kid
            AND biograph.pid IS NOT NULL
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION 'Dadid %: Kid %: DAD_DATA.Mom_sampled must not be NULL when the offspring BIOGRAPH.Pid is not NULL'
                    , NEW.dadid
                    , NEW.kid;
      RETURN NULL;
    END IF;
  ELSE -- NEW.mom_sampled IS NOT NULL
    PERFORM 1
      FROM biograph
      WHERE biograph.sname = NEW.kid
            AND biograph.pid IS NULL
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION 'Dadid %: Kid %: DAD_DATA.Mom_sampled must be NULL when the offspring BIOGRAPH.Pid is NULL'
                    , NEW.dadid
                    , NEW.kid;
      RETURN NULL;
    END IF;
  END IF;

  -- The dad columns must pass the validity checks.
  _dad_check(`Dad_excl')
  _dad_check(`Dad_1perr')
  _dad_check(`Dad_5perr')
  _dad_check(`Dad_allmales')
  _dad_check(`Dad_consensus')

  -- The analysis date is after the birth date.
  PERFORM 0
    FROM biograph
    WHERE biograph.sname = NEW.kid
          AND biograph.birth > NEW.date
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'Dadid %: Kid %: Date %: DAD_DATA.Date must not be before the offspring''s birth date'
                  , NEW.Dadid
                  , NEW.Kid
                  , NEW.Date;
    RETURN NULL;
  END IF;

  RETURN NULL;
  END;
$$;


SELECT 'dad_data_trigger' AS trigger;
CREATE TRIGGER dad_data_trigger
  AFTER INSERT OR UPDATE
  ON dad_data FOR EACH ROW
  EXECUTE PROCEDURE dad_data_func();
