dnl Copyright (C) 2014, 2017 Jake Gordon <jacob.gordon@duke.edu>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl

--  
-- dart_sample_types
--
SELECT 'dart_sample_types' AS table;

SELECT 'dart_sample_types_func' AS function;
CREATE FUNCTION dart_sample_types_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  BEGIN
  -- Function for dart_sample_types insert and update triggers
  --
  -- GPL_notice(`  --', `2014, 2017', `Jake Gordon, <jacob.gordon@duke.edu>')

  IF TG_OP = 'UPDATE' THEN
    cannot_change(`DART_SAMPLE_TYPES', `DS_Type')

  -- If a DS_Type becomes sex-specific, or if its sex-specificity
  -- changes, verify that any samples in DART_SAMPLES with this
  -- DS_Type are from individuals with the right sex.

    IF NEW.sex IS NOT NULL
       AND NEW.sex <> OLD.sex THEN
      PERFORM 1
        FROM biograph
             JOIN dartings ON (dartings.sname = biograph.sname)
             JOIN dart_samples ON (dart_samples.dartid = dartings.dartid
                                   AND
                                   dart_samples.ds_type = NEW.ds_type)
        WHERE biograph.sex <> NEW.sex
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION integrity_constraint_violation USING
              MESSAGE = 'Error on ' || TG_OP || ' of DART_SAMPLE_TYPES'
            , DETAIL = 'Key (DS_Type) = (' || NEW.ds_type
                         || '): Value (Sex) = (' || NEW.sex
                         || '): Cannot change to a sex-specific type when'
                         || ' samples in DART_SAMPLES belong to an individual'
                         || ' with the "wrong" sex';
        RETURN NULL;
      END IF;

    END IF;

  -- Enforce mins and maxes
    IF NEW.minimum <> OLD.minimum
       OR NEW.maximum <> OLD.maximum THEN
      PERFORM 1
        FROM dartings
        WHERE dartings.ds_type = NEW.ds_type
              AND (dartings.num < NEW.minimum
                  OR dartings.num > NEW.maximum)
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION integrity_constraint_violation USING
              MESSAGE = 'Error on ' || TG_OP || ' of DART_SAMPLE_TYPES'
            , DETAIL = 'Key (DS_Type) = (' || NEW.ds_type
                         || '): Value (Minimum) = (' || NEW.minimum
                         || '): Value (Maximum) = (' || NEW.maximum
                         || '): Cannot change Minimum or Maximum if related'
                         || ' rows in DART_SAMPLES have a Num outside the'
                         || ' Minimum-Maximum range (inclusive)';
        RETURN NULL;
      END IF;
    END IF;
  END IF;

  RETURN NULL;
  END;
$$;

SELECT 'dart_sample_types_trigger' AS trigger;
CREATE TRIGGER dart_sample_types_trigger
  AFTER INSERT OR UPDATE
  ON dart_sample_types FOR EACH ROW
  EXECUTE PROCEDURE dart_sample_types_func();
