dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2005, 2006, 2008, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl

--  
-- dcauses
--
SELECT 'dcauses' AS table;

SELECT 'dcauses_func' AS function;
CREATE FUNCTION dcauses_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  BEGIN
  -- Function for dcauses insert and update triggers
  --
  -- GPL_notice(`  --', `2005, 2006, 2008, 2009', `Karl O. Pinc <kop@meme.com>')

  IF TG_OP = 'UPDATE' THEN
    -- Cannot change the alive code.
    IF (NEW.Dcause = 'bb_no_dcause' OR OLD.Dcause = 'bb_no_dcause')
      AND (NEW.Dcause <> OLD.Dcause
           OR NEW.nature <> OLD.nature
           OR NEW.agent <> OLD.agent)
      AND NOT(has_table_privilege('DCAUSES', 'trigger')) THEN
      RAISE EXCEPTION insufficient_privilege USING
            MESSAGE = 'Error on ' || TG_OP || ' of DCAUSES'
          , DETAIL = 'Key(Dcause) = (0): This row has special meaning to '
                   || 'Babase and may not be changed by ordinary users'
          , HINT = 'The change can only be made by a user who is allowed '
                   || 'to create triggers on the table';
    END IF;
  END IF;

  RETURN NULL;
  END;
$$;


SELECT 'dcauses_delete_func' AS function;
CREATE FUNCTION dcauses_delete_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  -- Function for dcauses delete trigger
  --
  -- GPL_notice(`  --', `2005, 2008, 2009', `Karl O. Pinc <kop@meme.com>')

  DECLARE

  BEGIN

  restrict_delete(`DCAUSES', `Dcause', bb_no_dcause)

  RETURN NULL;
  END;
$$;


SELECT 'dcauses_trigger' AS trigger;
CREATE TRIGGER dcauses_trigger
  AFTER INSERT OR UPDATE
  ON dcauses FOR EACH ROW
  EXECUTE PROCEDURE dcauses_func();

SELECT 'dcauses_delete_trigger' AS trigger;
CREATE TRIGGER dcauses_delete_trigger
  AFTER DELETE
  ON dcauses FOR EACH ROW
  EXECUTE PROCEDURE dcauses_delete_func();
