dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2004, 2008, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>

dnl m4 includes
include(`copyright.m4')dnl
include(`macros.m4')dnl

--  
-- demog
--
SELECT 'demog' AS table;

SELECT 'demog_func' AS function;
CREATE FUNCTION demog_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  -- Function for demog insert and update triggers
  --
  -- GPL_notice(`  --', `2004, 2008', `Karl O. Pinc <kop@meme.com>')

  BEGIN
  --
  -- First the integrity checking.
  --

  --
  -- Things to check only on update.
  --
  IF TG_OP = 'UPDATE' THEN
    -- Keep things simple for now when updating other tables.
    cannot_change(`DEMOG', `Cenid')
  END IF;

  --
  -- Checks for insert or update.
  --

  --
  -- Done checking things.  Now update the rest of the database
  -- where appropriate.
  --

  -- Census row shows absent, but demography note places individual there.
  -- Change census status.
  UPDATE census
    SET status = 'bb_demog'
    WHERE cenid = NEW.cenid
          AND status = 'bb_absent';

  RETURN NULL;
  END;
$$;

SELECT 'demog_delete_func' AS function;
CREATE FUNCTION demog_delete_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  -- Function for demog delete trigger
  --
  -- GPL_notice(`  --', `2004, 2008', `Karl O. Pinc <kop@meme.com>')

  DECLARE

  BEGIN

  --
  -- Done checking things.  Now update the rest of the database
  -- where appropriate.
  --

  --
  -- Update the census table.
  --

  -- Census row reflects census record, must have been absent.
  UPDATE census
    SET status = 'bb_absent'
    WHERE cenid = OLD.cenid
          AND status = 'bb_demog'
          AND cen;

  -- Census row exists only because of demography note, get rid of it.
  DELETE FROM census
    WHERE cenid = OLD.cenid
          AND status = 'bb_demog'
          AND NOT cen;

  RETURN NULL;
  END;
$$;



SELECT 'demog_trigger' AS trigger;
CREATE TRIGGER demog_trigger
  AFTER INSERT OR UPDATE
  ON demog FOR EACH ROW
  EXECUTE PROCEDURE demog_func();


SELECT 'demog_delete_trigger' AS trigger;
CREATE TRIGGER demog_delete_trigger
  AFTER DELETE
  ON demog FOR EACH ROW
  EXECUTE PROCEDURE demog_delete_func();
