dnl Copyright (C) 2016 Jake Gordon, <jacob.gordon@duke.edu>
dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2004-2006, 2008, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl

--
-- dispersedates
--
SELECT 'dispersedates' AS table;

SELECT 'dispersedates_func' AS function;
CREATE FUNCTION dispersedates_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  -- Function for dispersedates insert and update triggers
  --
  -- Copyright (C) 2012 The Meme Factory, Inc.  http://www.meme.com/
  -- GPL_notice(`  --', `2004-2006, 2008', `Karl O. Pinc <kop@meme.com>')
  DECLARE
    this_sex      biograph.sex%TYPE;
    this_status   biograph.status%TYPE;
    this_statdate biograph.statdate%TYPE;

    this_group    census.grp%TYPE;
    some_absence  census.date%TYPE;  -- First absence in group after statdate

  BEGIN

  -- Get a bunch of info we'll need from biograph.
  SELECT INTO this_sex,     this_status,     this_statdate
              biograph.sex, biograph.status, biograph.statdate
    FROM biograph
    WHERE biograph.sname = NEW.sname;

  -- Dispersals only happen to males.
  IF this_sex <> 'bb_male' THEN
    RAISE EXCEPTION 'DISPERSEDATES %: %: Only males can have dispersal dates'
                    , NEW.sname
                    , NEW.dispersed;
    RETURN NULL;
  END IF;

  IF this_statdate < NEW.dispersed THEN
    IF this_status = 'bb_alive' THEN
      -- The individual is alive.  He can have a disperse date
      -- after his statdate only if it's also not after the
      -- next time he's censused absent from the group where
      -- he was last found to be present.

      -- Find the group in which the individual was last censused.
      SELECT INTO this_group census.grp
        FROM census
        WHERE census.sname = NEW.sname
              AND census.status <> 'bb_absent'
        ORDER BY census.date DESC
        LIMIT 1;
      IF NOT FOUND THEN
        -- Error, can't be after statdate when no census.
        RAISE EXCEPTION '%: %: DISPERSEDATES.Dispersed cannot be after BIOGRAPH.Statdate (%) when the individual has never been censused present in a group'
                      , NEW.sname
                      , NEW.dispersed
                      , this_statdate;
        RETURN NULL;
      ELSE
        -- Find the last absence in the censused group.
        SELECT INTO some_absence census.date
          FROM census
          WHERE census.sname = NEW.sname
                AND census.grp = this_group
                AND census.date > this_statdate
                AND census.status = 'bb_absent'
          ORDER BY census.date
          LIMIT 1;
        IF NOT FOUND OR some_absence < NEW.dispersed THEN
          RAISE EXCEPTION '%: %: DISPERSEDATES.Dispersed that are after the Statdate must be on or before the next recorded absence from the group in which the individual was last censused'
                        , NEW.sname
                        , NEW.dispersed;
          RETURN NULL;
        END IF;
      END IF;
    ELSE
      -- Individual is not alive.
      -- dispersed must be on or before statdate
      RAISE EXCEPTION '%: %: DISPERSEDATES.Dispersed cannot be after BIOGRAPH.Statdate (%) unless the individual is alive'
                    , NEW.sname
                    , NEW.dispersed
                    , this_statdate;
      RETURN NULL;
    END IF;
  END IF;

  -- Dispersed must be on or after Entrydate
  check_entrydate(`DISPERSEDATES', `Dispersed')

  RETURN NULL;
  END;
$$;

SELECT 'dispersedates_trigger' AS trigger;
CREATE TRIGGER dispersedates_trigger
  AFTER INSERT OR UPDATE
  ON dispersedates FOR EACH ROW
  EXECUTE PROCEDURE dispersedates_func();
