dnl Copyright (C) 2011, 2012 The Meme Factory, Inc.
dnl Copyright (C) 2011, Karl O. Pinc
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl

dnl plpgsql fragment for checking that the error reporting supported
dnl by the gps unit is the only type of error reporting recorded
dnl when that GPS unit is used.
dnl
dnl Syntax: _check_error_reporting(errortype, nullcol)
dnl
dnl errortype  The value of this GPS unit's GPS_UNITS.Errortype.
dnl nullcol    The name of the column that should be NULL given the Errortype.
dnl            Should be written as a camel-caps column name for presentation
dnl            to the user.
dnl
dnl BUGS:
dnl Relies on swerb_departs_gps and swerb_data using the same column
dnl names for their pdop and accuracy values.
dnl
changequote({,})
define({_check_error_reporting},{
    IF NEW.errortype = '$1' THEN

      -- Check departure records.
      PERFORM 1
        FROM swerb_departs_gps
        WHERE swerb_departs_gps.gps = NEW.gps
              AND swerb_departs_gps.$2 IS NOT NULL
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION 'GPS %: Errortype %: Cannot have related SWERB_DEPARTS_GPS rows with non-NULL $2 values for a GPS_UNITS.Errortype of $1'
                      , NEW.gps
                      , NEW.errortype;
        RETURN NULL;
      END IF;

      -- Check regular SWERB records.
      PERFORM 1
        FROM swerb_departs_gps
             JOIN swerb_bes ON (swerb_bes.did = swerb_departs_gps.did)
             JOIN swerb_data ON (swerb_data.beid = swerb_bes.beid)
        WHERE swerb_departs_gps.gps = NEW.gps
              AND swerb_data.$2 IS NOT NULL
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION 'GPS %: Errortype %: Cannot have related SWERB_DATA rows with non-NULL $2 values for a GPS_UNITS.Errortype of $1'
                      , NEW.gps
                      , NEW.errortype;
        RETURN NULL;
      END IF;

      -- Check SWERB_LOC_GPS records.
      PERFORM 1
        FROM swerb_departs_gps 
             JOIN swerb_departs_data
                  ON (swerb_departs_data.did = swerb_departs_gps.did)
             JOIN swerb_bes ON (swerb_bes.did = swerb_departs_data.did)
             JOIN swerb_data ON (swerb_data.beid = swerb_bes.beid)
             JOIN swerb_loc_gps ON (swerb_loc_gps.swid = swerb_data.swid)
        WHERE swerb_departs_gps.gps = NEW.gps
              AND swerb_loc_gps.$2 IS NOT NULL
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION 'GPS %: Errortype %: Cannot have related SWERB_LOC_GPS rows with non-NULL $2 values for a GPS_UNITS.Errortype of $1'
                      , NEW.gps
                      , NEW.errortype;
        RETURN NULL;
      END IF;

      -- Check SWERB_GW_LOC_DATA records.
      PERFORM 1
        FROM swerb_gw_loc_data
        WHERE swerb_gw_loc_data.gps = NEW.gps
              AND swerb_gw_loc_data.$2 IS NOT NULL
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION 'GPS %: Errortype %: Cannot have related SWERB_GW_LOC_DATA rows with non-NULL $2 values for a GPS_UNITS.Errortype of $1'
                      , NEW.gps
                      , NEW.errortype;
        RETURN NULL;
      END IF;
    END IF;
})dnl
changequote(`,')dnl

--  
-- gps_units
--
SELECT 'gps_units' AS table;

SELECT 'gps_units_func' AS function;
CREATE FUNCTION gps_units_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  BEGIN
  -- Function for gps_units insert and update triggers
  --
  -- Copyright (C) 2012 The Meme Factory, Inc.  http://www.meme.com/
  -- GPL_notice(`  --', `2011', `Karl O. Pinc <kop@meme.com>')

  IF TG_OP = 'UPDATE' THEN
    -- Can't use a unit before it's start date.
    IF NEW.Start > OLD.Start THEN
      --  Can't depart from camp with a unit before it's start date.
      PERFORM 1
        FROM swerb_departs_gps
             JOIN swerb_departs_data 
                  ON (swerb_departs_data.did = swerb_departs_gps.did)
        WHERE swerb_departs_gps.gps = NEW.gps
              AND swerb_departs_data.date < NEW.start
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION 'GPS %:NEW Start %: Cannot change Start because the unit is used in SWERB_DEPARTS_DATA before the new Start'
                      , NEW.gps
                      , NEW.start;
        RETURN NULL;
      END IF;

      --  Can't geolocate a place with a unit before it's start date.
      PERFORM 1
        FROM swerb_gw_loc_data
        WHERE swerb_gw_loc_data.gps = NEW.gps
              AND swerb_departs_data.date < NEW.start
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION 'GPS %:NEW Start %: Cannot change Start because the unit is used in SWERB_GW_LOC_DATA before the new Start'
                      , NEW.gps
                      , NEW.start;
        RETURN NULL;
      END IF;
    END IF;

    -- Can't use a unit after it's finish date.
    IF NEW.Finish < OLD.Finish
       OR (NEW.Finish IS NOT NULL AND OLD.Finish IS NULL) THEN
      -- Can't depart from camp with a unit after it's finish date.
      PERFORM 1
        FROM swerb_departs_gps
             JOIN swerb_departs_data 
                  ON (swerb_departs_data.did = swerb_departs_gps.did)
        WHERE swerb_departs_gps.gps = NEW.gps
              AND swerb_departs_data.date > NEW.finish
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION 'GPS %:NEW Finish %: Cannot change Finish because the unit is used in SWERB_DEPARTS_DATA after the new Finish'
                      , NEW.gps
                      , NEW.finish;
        RETURN NULL;
      END IF;

      -- Can't geolocate a place with a unit after it's finish date.
      PERFORM 1
        FROM swerb_gw_loc_data
        WHERE swerb_gw_loc_data.gps = NEW.gps
              AND swerb_gw_loc_data.date > NEW.finish
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION 'GPS %:NEW Finish %: Cannot change Finish because the unit is used in SWERB_GW_LOC_DATA after the new Finish'
                      , NEW.gps
                      , NEW.finish;
        RETURN NULL;
      END IF;
    END IF;

    -- Can't have accuracy readings for a unit that reports error as a PDOP.
    _check_error_reporting(`bb_gpsu_pdop', `Accuracy')


    -- Can't have PDOP readings for a unit that reports errors as an accuracy.
    _check_error_reporting(`bb_gpsu_accuracy', `PDOP')
  END IF;

  -- The label on the gps unit must be unique during the time period
  -- in which the gps unit was in use.
  PERFORM 1
    FROM gps_units
    WHERE gps_units.gps <> NEW.gps
          AND gps_units.label = NEW.label
          AND gps_units.start <= NEW.finish
          AND gps_units.finish >= NEW.start
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'GPS %: Label %: Start %: Finish %: The label on the GPS unit must be unique during the time period when the GPS unit was in use'
                  , NEW.gps
                  , NEW.label
                  , NEW.start
                  , NEW.finish;
    RETURN NULL;
  END IF;

  RETURN NULL;
  END;
$$;


SELECT 'gps_units_trigger' AS trigger;
CREATE TRIGGER gps_units_trigger
  AFTER INSERT OR UPDATE
  ON gps_units FOR EACH ROW
  EXECUTE PROCEDURE gps_units_func();
