dnl Copyright (C) 2013, 2014, 2016 Jacob Gordon, <jacob.gordon@duke.edu>
dnl Copyright (C) 2012, 2014, 2016 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2005, 2007-2009, 2011, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl 

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl

--  
-- groups
--
SELECT 'groups' AS table;

SELECT 'groups_func' AS function;
CREATE FUNCTION groups_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  BEGIN
  -- Function for groups insert and update triggers
  --
  -- Copyright (C) 2013-2014, 2016 Jacob Gordon, <jacob.gordon@duke.edu>
  -- Copyright (C) 2012, 2016 The Meme Factory, Inc.  http://www.meme.com/
  -- GPL_notice(`  --', `2005, 2007-2009, 2011', `Karl O. Pinc <kop@meme.com>')

  IF NEW.one_letter_code IS NOT NULL THEN
    -- One_letter_code must be unique within the timeframe of Start through
    -- Cease_To_Exist, inclusive.
    PERFORM 1
      FROM groups
      WHERE (NEW.cease_to_exist IS NULL
             OR groups.start IS NULL
             OR groups.start <= NEW.cease_to_exist)
            AND (NEW.start IS NULL
                 OR NEW.start <= groups.cease_to_exist
                 OR groups.cease_to_exist IS NULL)
            AND groups.gid <> NEW.gid
            AND groups.one_letter_code = NEW.one_letter_code
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION 'Gid %: One_letter_code %: Start %: Cease_To_Exist %: GROUPS.One_letter_code must be unique within the Start to Cease_To_Exist timeframe, inclusive of endpoints'
                    , NEW.gid
                    , NEW.one_letter_code
                    , NEW.start
                    , NEW.cease_to_exist;
      RETURN NULL;
    END IF;
  END IF;

  IF NEW.cease_to_exist IS NOT NULL THEN
    -- Cease_To_Exist must be greater than or equal to the Start of 
    -- any of the group's subgroups.
    PERFORM 1
      FROM groups AS subgrp
      WHERE NEW.gid = subgrp.from_group
        AND NEW.cease_to_exist < subgrp.start
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION integrity_constraint_violation USING
            MESSAGE = 'Error on ' || TG_OP || ' of GROUPS'
          , DETAIL = 'Key (GId) = (' || NEW.gid
                       || '): Value (Cease_To_Exist) = (' || NEW.cease_to_exist
                       || '): When GROUPS.Cease_To_Exist is NOT NULL then '
                       || 'GROUPS.Cease_To_Exist must be greater than or equal'
                       || ' to the GROUPS.Start of any of that group''s'
                       || ' subgroups';
      RETURN NULL;
    END IF;
  END IF;

  IF NEW.start IS NOT NULL 
    AND NEW.from_group IS NOT NULL THEN
    -- Start must be less than or equal to the Cease_To_Exist of 
    -- the group's From_Group.
    PERFORM 1
      FROM groups AS frgrp
      WHERE NEW.from_group = frgrp.gid
        AND NEW.start > frgrp.cease_to_exist
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION integrity_constraint_violation USING
            MESSAGE = 'Error on ' || TG_OP || ' of GROUPS'
          , DETAIL = 'Key (GId) = (' || NEW.gid
                       || '): Value (Start) = (' || NEW.start
                       || '): When GROUPS.Start is NOT NULL then'
                       || ' GROUPS.Start must be less than or equal'
                       || ' to the GROUPS.Cease_To_Exist of that '
                       || 'group''s from_group';
      RETURN NULL;
    END IF;
  END IF;


  -- Not-Permanent groups may not be their own Supergroup.
  -- We can't do this in a constraint as the before trigger must
  -- run and assign a supergroup.
  IF NEW.permanent IS NULL AND NEW.gid = NEW.supergroup THEN
    RAISE EXCEPTION '%: GROUPS.Gid of a non-Permanent group cannot be the Supergroup'
                    , NEW.gid;
    RETURN NULL;
  END IF;

  IF TG_OP = 'UPDATE' THEN
    -- Cannot change the unknown group code.
    IF (NEW.gid = bb_unk_grp OR NEW.gid = bb_alone_grp)
       AND (NEW.gid <> OLD.gid
            OR NEW.name <> OLD.name
            OR (NEW.from_group <> OLD.from_group
                AND NOT (NEW.from_group IS NULL AND OLD.from_group IS NULL))
            OR (NEW.permanent <> OLD.permanent
                AND NOT (NEW.permanent IS NULL AND OLD.permanent IS NULL))
            OR NEW.supergroup <> OLD.supergroup
            OR (NEW.start <> OLD.start
                AND NOT (NEW.start IS NULL AND OLD.start IS NULL))
            OR (NEW.three_letter_code <> OLD.three_letter_code
                AND NOT (NEW.three_letter_code IS NULL
                         AND OLD.three_letter_code IS NULL))
            OR (NEW.one_letter_code <> OLD.one_letter_code
                AND NOT (NEW.one_letter_code IS NULL
                         AND OLD.one_letter_code IS NULL))
           )
       AND NOT(has_table_privilege('groups', 'trigger')) THEN
      RAISE EXCEPTION 'GROUPS Gid: bb_unk_grp and bb_alone_grp: cannot_change_msg';
    END IF;

    -- Individual's can't have rows in CENSUS if the Start column is NULL.
    IF NEW.start IS NULL THEN
      PERFORM 1
        FROM census
        WHERE census.grp = NEW.gid
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION '%: Start column may not be NULL when individuals have rows in CENSUS'
                        , NEW.gid;
        RETURN NULL;
      END IF;
    END IF;

    -- Individual's can't have rows in CENSUS before the Start date,
    -- inclusive.
    PERFORM 1
      FROM census
      WHERE census.grp = NEW.gid
            AND census.date < NEW.start
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION '%: %: Individual''s can''t have rows in CENSUS before the Start date'
                      , NEW.gid
                      , NEW.Start;
      RETURN NULL;
    END IF;

    -- Individual's can't have rows in CENSUS after the Cease_To_Exist date,
    -- inclusive.
    PERFORM 1
      FROM census
      WHERE census.grp = NEW.gid
            AND census.date > NEW.Cease_To_Exist
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION '%: %: Individual''s can''t have rows in CENSUS after the Cease_To_Exist date'
                      , NEW.gid
                      , NEW.cease_to_exist;
      RETURN NULL;
    END IF;

    -- Can't be a swerb focal group before the group exists.
    PERFORM 1
      FROM swerb_bes, swerb_departs_data
      WHERE swerb_bes.did = swerb_departs_data.did
            AND swerb_bes.focal_grp = NEW.gid
            AND swerb_departs_data.date < NEW.start
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION '%: %: Cannot be a SWERB_BES.Focal_grp before the group''s Start date'
                    , NEW.gid
                    , NEW.start;
      RETURN NULL;
    END IF;

    -- Can't be a swerb focal group after the group ceases to exist.
    PERFORM 1
      FROM swerb_bes, swerb_departs_data
      WHERE swerb_bes.did = swerb_departs_data.did
            AND swerb_bes.focal_grp = NEW.gid
            AND swerb_departs_data.date > NEW.cease_to_exist
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION '%: %: Cannot be a SWERB_BES.Focal_grp after the group''s Cease_To_Exist date'
                    , NEW.gid
                    , NEW.cease_to_exist;
      RETURN NULL;
    END IF;
    -- Can't be a swerb seen group before the group exists.
    PERFORM 1
      FROM swerb_bes, swerb_departs_data, swerb_data
      WHERE swerb_bes.did = swerb_departs_data.did
            AND swerb_data.beid = swerb_bes.beid
            AND swerb_data.seen_grp = NEW.gid
            AND swerb_departs_data.date < NEW.start
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION '%: %: Cannot be a SWERB_DATA.Seen_grp before the group''s Start date'
                    , NEW.gid
                    , NEW.start;
      RETURN NULL;
    END IF;

    -- Can't be a swerb seen group after the group ceases to exist.
    PERFORM 1
      FROM swerb_bes, swerb_departs_data, swerb_data
      WHERE swerb_bes.did = swerb_departs_data.did
            AND swerb_data.beid = swerb_bes.beid
            AND swerb_data.seen_grp = NEW.gid
            AND swerb_departs_data.date > NEW.cease_to_exist
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION '%: %: Cannot be a SWERB_DATA.Seen_grp after the group''s Cease_To_Exist date'
                    , NEW.gid
                    , NEW.cease_to_exist;
      RETURN NULL;
    END IF;

    -- Can't begin a behave gap before the group starts
    PERFORM 1
      FROM behave_gaps
      WHERE behave_gaps.grp = NEW.gid
            AND behave_gaps.gap_start < NEW.start
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION integrity_constraint_violation USING
            MESSAGE = 'Error on ' || TG_OP || ' of GROUPS'
          , DETAIL = 'Key (Gid) = (' || NEW.gid
                       || '): Value (Start) = (' || NEW.start
                       || '): Cannot start a behave gap for a group before '
                       || 'its GROUPS.Start';
      RETURN NULL;
    END IF;

    -- Can't begin a behave gap after the group ceases to exist
    PERFORM 1
      FROM behave_gaps
      WHERE behave_gaps.grp = NEW.gid
            AND behave_gaps.gap_start > NEW.cease_to_exist
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION integrity_constraint_violation USING
            MESSAGE = 'Error on ' || TG_OP || ' of GROUPS'
          , DETAIL = 'Key (Gid) = (' || NEW.gid
                       || '): Value (Cease_To_Exist) = (' || NEW.cease_to_exist
                       || '): Cannot start a behave gap for a group after '
                       || 'its GROUPS.Cease_To_Exist';
      RETURN NULL;
    END IF;

    -- Can't have NULL behave gap end if group has ceased to exist
    IF NEW.cease_to_exist IS NOT NULL THEN
      PERFORM 1
        FROM behave_gaps
        WHERE behave_gaps.grp = NEW.gid
              AND behave_gaps.gap_end IS NULL
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION integrity_constraint_violation USING
              MESSAGE = 'Error on ' || TG_OP || ' of GROUPS'
            , DETAIL = 'Key (Gid) = (' || NEW.gid
                         || '): Value(Cease_To_Exist) = (' || NEW.cease_to_exist
                         || '): Behave Gap for a group cannot have a NULL end '
                         || 'when the group''s Cease_To_Exist is not NULL';
        RETURN NULL;
      END IF;
    END IF;

    -- Can't end a behave gap before the group starts
    PERFORM 1
      FROM behave_gaps
      WHERE behave_gaps.grp = NEW.gid
            AND behave_gaps.gap_end IS NOT NULL
            AND behave_gaps.gap_end < NEW.start
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION integrity_constraint_violation USING
            MESSAGE = 'Error on ' || TG_OP || ' of GROUPS'
          , DETAIL = 'Key (Gid) = (' || NEW.gid
                       || '): Value (Start) = (' || NEW.start
                       || '): Cannot end a behave gap for a group before '
                       || 'its GROUPS.Start';
      RETURN NULL;
    END IF;

    -- Can't end a behave gap after the group ceases to exist
    PERFORM 1
      FROM behave_gaps
      WHERE behave_gaps.grp = NEW.gid
            AND behave_gaps.gap_end IS NOT NULL
            AND behave_gaps.gap_end > NEW.cease_to_exist
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION integrity_constraint_violation USING
            MESSAGE = 'Error on ' || TG_OP || ' of GROUPS'
          , DETAIL = 'Key (Gid) = (' || NEW.gid
                       || '): Value (Cease_To_Exist) = (' || NEW.cease_to_exist
                       || '): Cannot end a behave gap for a group after '
                       || 'its GROUPS.Cease_To_Exist';
      RETURN NULL;
    END IF;

    --
    -- All done checking things.
    -- Let's update related data.
    --

    -- Update the supergroup of all the child groups of this group.
    -- Do this by invoking the BEFORE trigger of the child groups.
    -- We don't want to check if the from_group (or permenant)
    -- of this particular group actually changed because the
    -- from_group won't actually have changed for grandchildren
    -- and further decendents (permanent won't have changed for
    -- children and further decendents), and this is the code that
    -- propagates the change from the grandchild on down.
    UPDATE groups
      SET gid = groups.gid
      WHERE groups.from_group = OLD.gid;

  END IF;
  RETURN NULL;
  END;
$$;


SELECT 'groups_before_func' AS function;
CREATE FUNCTION groups_before_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- Function for groups before trigger
  --
  -- GPL_notice(`  --', `2007, 2008', `Karl O. Pinc <kop@meme.com>')
  --
  -- Make sure the Supergroup column is right.

  DECLARE

  BEGIN

  IF NEW.permanent IS NOT NULL
     -- Constraints have not been checked so avoid infinite loop
     -- in case where Permanent and From_group are both NULL.
     OR NEW.from_group IS NULL THEN
    -- Permanent groups must be their own Supergroup.
    NEW.Supergroup := NEW.gid;
  ELSE
    -- Find the group's supergroup.
    DECLARE
      ancestor groups.gid%TYPE := NEW.from_group;

    BEGIN
      LOOP
        PERFORM 1
          FROM groups
          WHERE groups.gid = ancestor
                AND groups.permanent IS NOT NULL;
        IF FOUND THEN
          -- The first Permanent ancestor is the supergroup.
          EXIT;
        END IF;

        SELECT INTO ancestor groups.from_group
          FROM groups
          WHERE groups.gid = ancestor;
      END LOOP;

      NEW.supergroup := ancestor;
    END;

  END IF;

  RETURN NEW;
  END;
$$;



SELECT 'groups_delete_func' AS function;
CREATE FUNCTION groups_delete_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- Function for groups delete trigger
  --
  -- GPL_notice(`  --', `2005, 2008, 2009', `Karl O. Pinc <kop@meme.com>')

  DECLARE

  BEGIN

  IF OLD.gid = bb_unk_grp
     AND NOT(has_table_privilege('groups', 'trigger')) THEN
    RAISE EXCEPTION 'GROUPS Gid: bb_unk_grp: cannot_delete_msg';
  END IF;

  RETURN NULL;
  END;
$$;

SELECT 'groups_commit_func' AS function;
CREATE OR REPLACE FUNCTION groups_commit_func()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  -- Function for groups insert or update fired upon transaction commit.
  --
  -- Copyright (C) 2016 Jacob Gordon, <jacob.gordon@duke.edu>
  -- GPL_notice(`  --', `2012', `The Meme Factory, Inc.  http://www.meme.com/')
  --
  BEGIN

  -- Get the latest values of the row
  SELECT * INTO NEW
    FROM groups
    WHERE groups.gid = NEW.gid;
  IF NOT FOUND THEN
    -- Whatever row was inserted was subsequently deleted.
    -- Nothing to do.
    RETURN NULL;
  END IF;

  -- If a group has a non-NULL From_Group and a non-NULL Permanent
  -- date, the Cease_To_Exist of its parent group cannot be NULL and
  -- must equal the subgroup's Permanent.

  -- First, check parent group for NULL Cease_To_Exist.

  IF NEW.cease_to_exist IS NULL THEN
    PERFORM 1
      FROM groups
      WHERE groups.from_group = NEW.gid
            AND groups.permanent IS NOT NULL
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION integrity_constraint_violation USING
            MESSAGE = 'Error on GROUPS ' || TG_OP || ' commit'
          , DETAIL = 'Key(Gid) = (' || NEW.gid
                       || '): Group cannot have a NULL Cease_To_Exist'
                       || ' if any subgroups have a non-NULL Permanent'
                       || ' date.';
      RETURN NULL;
    END IF;
  END IF;

  -- Next, make sure parent Cease_To_Exist and subgroup Permanent are
  -- equal.

  IF NEW.from_group IS NOT NULL
     AND NEW.permanent IS NOT NULL THEN
    PERFORM 1
      FROM groups
      WHERE groups.gid = NEW.from_group
            AND groups.cease_to_exist <> NEW.permanent
      -- Should never be more than 1, but just in case
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION integrity_constraint_violation USING
            MESSAGE = 'Error on GROUPS ' || TG_OP || ' commit'
          , DETAIL = 'Key (Gid) = (' || NEW.gid
                       || '): Value (From_group) = (' || NEW.from_group
                       || '): Value (Permanent) = (' || NEW.permanent
                       || '): When From_group and Permanent are both not'
                       || ' NULL, Permanent must be equal to the parent'
                       || ' group''s Cease_To_Exist';
      RETURN NULL;
    END IF;
  END IF;

  RETURN NULL;
  END;
$$;


SELECT 'groups_trigger' AS trigger;
CREATE TRIGGER groups_trigger
  AFTER INSERT OR UPDATE
  ON groups FOR EACH ROW
  EXECUTE PROCEDURE groups_func();

SELECT 'groups_before_trigger' AS trigger;
CREATE TRIGGER groups_before_trigger
  BEFORE INSERT OR UPDATE
  ON groups FOR EACH ROW
  EXECUTE PROCEDURE groups_before_func();

SELECT 'groups_delete_trigger' AS trigger;
CREATE TRIGGER groups_delete_trigger
  AFTER DELETE
  ON groups FOR EACH ROW
  EXECUTE PROCEDURE groups_delete_func();

SELECT 'groups_commit_trigger' AS trigger;
CREATE CONSTRAINT TRIGGER groups_commit_trigger
  AFTER INSERT OR UPDATE
  ON groups
  DEFERRABLE INITIALLY DEFERRED
  FOR EACH ROW
  EXECUTE PROCEDURE groups_commit_func();
