dnl Copyright (C) 2016 Jake Gordon, <jacob.gordon@duke.edu>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl


--
-- hybridgene_analyses
--
SELECT 'hybridgene_analyses' AS table;

SELECT 'hybridgene_analyses_func' AS function;
CREATE FUNCTION hybridgene_analyses_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  BEGIN
  -- Function for hybridgene_analyses insert and update triggers
  --
  -- GPL_notice(`  --', `2016', `Jake Gordon, <jacob.gordon@duke.edu>')

  -- Check that analysis date is after entrydate of all individuals in
  -- the analysis.
  --
  -- No need to check against statdate, because an analysis is allowed
  -- to happen after an individual's death.

  PERFORM 1
    FROM biograph, hybridgene_scores
    WHERE hybridgene_scores.hgaid = NEW.hgaid
          AND biograph.sname = hybridgene_scores.sname
          AND biograph.entrydate >= NEW.date
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION integrity_constraint_violation USING
          MESSAGE = 'Error on ' || TG_OP || ' of HYBRIDGENE_ANALYSES'
        , DETAIL = 'Key (HGAId) = (' || NEW.hgaid
                     || '): Value (Date) = (' || NEW.date
                     || '): HYBRIDGENE_ANALYSES.Date must be after '
                     || 'the BIOGRAPH.Entrydate of all individuals '
                     || 'in the analysis in HYBRIDGENE_SCORES'
        , HINT = 'Can''t get a hybrid score before entering population';
    RETURN NULL;
  END IF;

  RETURN NULL;
  END;
$$;


SELECT 'hybridgene_analyses_trigger' AS trigger;
CREATE TRIGGER hybridgene_analyses_trigger
  AFTER INSERT OR UPDATE
  ON hybridgene_analyses FOR EACH ROW
  EXECUTE PROCEDURE hybridgene_analyses_func();
