dnl Copyright (C) 2016 Jake Gordon, <jacob.gordon@duke.edu>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl


--
-- hybridgene_scores
--
SELECT 'hybridgene_scores' AS table;

SELECT 'hybridgene_scores_func' AS function;
CREATE FUNCTION hybridgene_scores_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  BEGIN
  -- Function for hybridgene_scores insert and update triggers
  --
  -- GPL_notice(`  --', `2016', `Jake Gordon, <jacob.gordon@duke.edu>')

  -- Check that the score's date is after the individual's entrydate.

  -- No need to check against statdate, because an analysis is allowed
  -- to happen after an individual's death.

  PERFORM 1
    FROM biograph, hybridgene_analyses
    WHERE hybridgene_analyses.hgaid = NEW.hgaid
          AND biograph.sname = NEW.sname
          AND biograph.entrydate >= hybridgene_analyses.date
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION integrity_constraint_violation USING
          MESSAGE = 'Error on ' || TG_OP || ' of HYBRIDGENE_ANALYSES'
        , DETAIL = 'Key (HGSId) = (' || NEW.hgsid
                     || '): Value (HGAId) = (' || NEW.hgaid
                     || '): Value (Sname) = (' || NEW.sname
                     || '): HYBRIDGENE_SCORES.Sname must have a '
                     || 'BIOGRAPH.Entrydate before the score''s '
                     || 'HYBRIDGENE_ANALYSES.Date'
        , HINT = 'Can''t get a hybrid score before entering population';
    RETURN NULL;
  END IF;

  RETURN NULL;
  END;
$$;


SELECT 'hybridgene_scores_trigger' AS trigger;
CREATE TRIGGER hybridgene_scores_trigger
  AFTER INSERT OR UPDATE
  ON hybridgene_scores FOR EACH ROW
  EXECUTE PROCEDURE hybridgene_scores_func();
