dnl Copyright (C) 2016 The Meme Factory, Inc.  http://www.meme.com/
dnl Copyright (C) 2004-2006, 2008-2010, Karl O. Pinc <kop@meme.com>
dnl Copyright (C) 2016, Jake Gordon, <jacob.gordon@duke.edu>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl
dnl
changequote([{[,]}])dnl m4 foolery so includes include only once.
dnl                     Once the macro is in the text, change the quotes back
ifdef([{[_macros.m4]}], [{[changequote(`,')]}], [{[dnl
changequote(`,')dnl
dnl
dnl Standard test for having already included the file.
define(`_macros.m4')dnl
dnl
dnl
dnl Discard any output.  Copyright and edit warning is all that's output.
divert(-1)

dnl m4 includes
include(`constants.m4')
include(`globalmacros.m4')

dnl
dnl String constants.
dnl

define(`bb_transaction_hint',
       `Put database changes in consistent bundles using transactions')

dnl
dnl Inline expressions
dnl

dnl sql fragment to cast values which might be NULL to text.
dnl
dnl Syntax: textualize(expr)
dnl
dnl expr  An expression which might be NULL.
dnl
dnl Remarks:
dnl  Useful when raising errors to get a uniform external NULL
dnl representation.
dnl
changequote({,})
define({textualize},{CASE WHEN $1 IS NULL THEN 'NULL' ELSE $1::TEXT END
})dnl
changequote(`,')


dnl plpgsql fragment for ensuring that a column
dnl have a value at least as big as another on another table.
dnl
dnl Syntax: check_ge(up_table, up_column, ref_table, ref_column)
dnl
dnl 'up' values must be bigger or equal to 'ref' values.
dnl
dnl Variables required:
dnl Restrictions: Must be used in a FOR EACH ROW trigger function
dnl on the up_table.  Tables are linked on the sname column.
changequote({,})
define({check_ge},{
  PERFORM $3.sname FROM $3
    WHERE $3.sname = NEW.sname
          AND $3.$4 > NEW.$2
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION '%: $3.$4 cannot be bigger than $1.$2'
                    , NEW.sname;
    RETURN NULL;
  END IF;
})
changequote(`,')


dnl plpgsql fragment for ensuring that a column
dnl have a value strictly bigger than another on another table.
dnl
dnl Syntax: check_gt(up_table, up_column, ref_table, ref_column)
dnl
dnl 'up' values must be strictly bigger than 'ref' values.
dnl
dnl Variables required:
dnl Restrictions: Must be used in a FOR EACH ROW trigger function
dnl on the up_table.  Tables are linked on the sname column.
changequote({,})
define({check_gt},{
  PERFORM $3.sname FROM $3
    WHERE $3.sname = NEW.sname
          AND $3.$4 >= NEW.$2
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION '%: $3.$4 cannot be bigger than (after) $1.$2'
                    , NEW.sname;
    RETURN NULL;
  END IF;
})
changequote(`,')


dnl plpgsql fragment for ensuring that a column
dnl have a value smaller than or equal to another on another table.
dnl
dnl Syntax: check_le(up_table, up_column, ref_table, ref_column)
dnl
dnl 'up' values must be less than or equal to 'ref' values.
dnl
dnl Variables required:
dnl Restrictions: Must be used in a FOR EACH ROW trigger function
dnl on the up_table.  Tables are linked on the sname column.
changequote({,})
define({check_le},{
  PERFORM $3.sname FROM $3
    WHERE $3.sname = NEW.sname
          AND $3.$4 < NEW.$2
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION '%: $3.$4 cannot be less than (before) $1.$2'
                    , NEW.sname;
    RETURN NULL;
  END IF;
})
changequote(`,')


dnl plpgsql fragment for ensuring that a column
dnl have a value strictly smaller than another on another table.
dnl
dnl Syntax: check_lt(up_table, up_column, ref_table, ref_column)
dnl
dnl 'up' values must be strictly less than 'ref' values.
dnl
dnl Variables required:
dnl Restrictions: Must be used in a FOR EACH ROW trigger function
dnl on the up_table.  Tables are linked on the sname column.
changequote({,})
define({check_lt},{
  PERFORM $3.sname FROM $3
    WHERE $3.sname = NEW.sname
          AND $3.$4 <= NEW.$2
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION '%: $3.$4 must be larger than (after) $1.$2'
                    , NEW.sname;
    RETURN NULL;
  END IF;
})
changequote(`,')



dnl
dnl These rev_* comparison macros are like the regular ones
dnl but are used when working with the tables on the right
dnl side of the inequality operator rather than the left.
dnl

dnl plpgsql fragment for checking changes to a sname keyed table
dnl and ensuring that a column on another table does not
dnl have a value bigger than the a column on the sname keyed table.
dnl Syntax: rev_check_ge(rev_table, rev_column, up_table, up_column)
dnl
dnl 'rev' values must be bigger than or equal to 'up' values.
dnl
dnl Variables required:
dnl Restrictions: Must be used in a FOR EACH ROW trigger function
dnl on the up_table.
changequote({,})
define({rev_check_ge},{
  PERFORM $1.$2 FROM $1
    WHERE $1.sname = NEW.sname
          AND NEW.$4 > $1.$2
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION '%: $3.$4 cannot be bigger than (after) $1.$2'
                    , NEW.sname;
    RETURN NULL;
  END IF;
})
changequote(`,')



dnl plpgsql fragment for checking changes to a sname keyed table
dnl and ensuring that a column on another table has
dnl a value smaller than the a column on the sname keyed table.
dnl Syntax: rev_check_gt(rev_table, rev_column, up_table, up_column)
dnl
dnl 'rev' values must be strictly bigger than 'up' values.
dnl
dnl Variables required:
dnl Restrictions: Must be used in a FOR EACH ROW trigger function
dnl on the up_table.
changequote({,})
define({rev_check_gt},{
  PERFORM $1.$2 FROM $1
    WHERE $1.sname = NEW.sname
          AND NEW.$4 >= $1.$2
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION '%: $3.$4 must be smaller than (before) $1.$2'
                    , NEW.sname;
    RETURN NULL;
  END IF;
})
changequote(`,')



dnl plpgsql fragment for checking changes to a sname keyed table
dnl and ensuring that a column on another table does not
dnl have a value smaller than the a column on the sname keyed table.
dnl Syntax: rev_check_le(rev_table, rev_column, up_table, up_column)
dnl
dnl 'rev' values must be less than or equal to 'up' values.
dnl
dnl Variables required:
dnl Restrictions: Must be used in a FOR EACH ROW trigger function
dnl on the up_table.
changequote({,})
define({rev_check_le},{
  PERFORM $1.$2 FROM $1
    WHERE $1.sname = NEW.sname
          AND NEW.$4 < $1.$2
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION '%: $3.$4 cannot be smaller than (before) $1.$2'
                    , NEW.sname;
    RETURN NULL;
  END IF;
})
changequote(`,')



dnl plpgsql fragment for checking changes to a sname keyed table
dnl and ensuring that a column on another table has
dnl a value less than the a column on the sname keyed table.
dnl Syntax: rev_check_lt(rev_table, rev_column, up_table, up_column)
dnl
dnl 'rev' values must be strictly less than 'up' values.
dnl
dnl Variables required:
dnl Restrictions: Must be used in a FOR EACH ROW trigger function
dnl on the up_table.
changequote({,})
define({rev_check_lt},{
  PERFORM $1.$2 FROM $1
    WHERE $1.sname = NEW.sname
          AND NEW.$4 <= $1.$2
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION '%: $3.$4 must be larger than (after) $1.$2'
                    , NEW.sname;
    RETURN NULL;
  END IF;
})
changequote(`,')


dnl
dnl More general comparison functions where the foreign key does
dnl note have to be the Sname column.
dnl

dnl plpgsql fragment for ensuring that a column
dnl have a value at least as big as another on another table.
dnl
dnl Syntax: check_keyed_ge(up_table, up_key, up_column, ref_table, ref_key, ref_column)
dnl
dnl 'up' values must be bigger or equal to 'ref' values.
dnl
dnl Variables required:
dnl Restrictions: Must be used in a FOR EACH ROW trigger function
dnl on the up_table.  Tables are linked on the ref_key column which
dnl must have the same name on both tables.
changequote({,})
define({check_keyed_ge},{
  PERFORM $4.$5 FROM $4
    WHERE $4.$5 = NEW.$5
          AND $4.$6 > NEW.$3
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION '$1.$2 %: $1.$5 %: $1.$3 %: $1.$3 cannot be less than (before) the related $4.$6',
                    NEW.$2,
                    NEW.$5,
                    NEW.$3;
    RETURN NULL;
  END IF;
})
changequote(`,')


dnl plpgsql fragment for checking changes to a sname keyed table
dnl and ensuring that a column on another table does not
dnl have a value bigger than the a column on the sname keyed table.
dnl Syntax: rev_check_keyed_ge(rev_table, rev_key, rev_column, up_table, up_ey, up_column)
dnl
dnl 'rev' values must be bigger than or equal to 'up' values.
dnl
dnl Remarks:
dnl   rev_key is not used but is included for consistency.
dnl
dnl Variables required:
dnl Restrictions: Must be used in a FOR EACH ROW trigger function
dnl on the up_table.
changequote({,})
define({rev_check_keyed_ge},{
  PERFORM $1.$3 FROM $1
    WHERE $1.$5 = NEW.$5
          AND NEW.$6 > $1.$3
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION '$4.$5 %: $4.$6 %: $4.$6 cannot be bigger than (after) $1.$3'
                    , NEW.$5
                    , NEW.$6;
    RETURN NULL;
  END IF;
})
changequote(`,')



dnl plpgsql fragment for ensuring that a column does not
dnl have a value bigger than the biograph statdate.
dnl (Regardless of whether the status of the individual shows they are dead.)
dnl Syntax: check_statdate(table, column)
dnl Restrictions: Must be used in a FOR EACH ROW trigger function.
changequote({,})
define({check_statdate},{
  PERFORM BIOGRAPH.sname FROM BIOGRAPH
    WHERE BIOGRAPH.sname = NEW.sname
          AND BIOGRAPH.Statdate < CAST(NEW.$2 AS DATE)
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION '%: BIOGRAPH.Statdate cannot be less than $1.$2'
                    , NEW.sname;
    RETURN NULL;
  END IF;
})
changequote(`,')


dnl plpgsql fragment for ensuring that a column does not
dnl have a value before the biograph birth date.
dnl
dnl Syntax: check_birth(table, column)
dnl
dnl Restrictions: Must be used in a FOR EACH ROW trigger function.
changequote({,})
define({check_birth},{
  PERFORM BIOGRAPH.sname FROM BIOGRAPH
    WHERE BIOGRAPH.sname = NEW.sname
          AND BIOGRAPH.birth > NEW.$2
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION '%: BIOGRAPH.Birth cannot be greater than $1.$2'
                    , NEW.sname;
    RETURN NULL;
  END IF;
})
changequote(`,')


dnl Plpgsql fragment for ensuring that a column does not
dnl have a value before the biograph entrydate.
dnl
dnl Syntax: check_entrydate(table, column)
dnl
dnl Restrictions: Must be used in a FOR EACH ROW trigger function.
changequote({,})
define({check_entrydate},{
  PERFORM BIOGRAPH.sname FROM BIOGRAPH
    WHERE BIOGRAPH.sname = NEW.sname
          AND BIOGRAPH.entrydate > NEW.$2
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION integrity_constraint_violation USING
          MESSAGE = 'Error on ' || TG_OP || ' of $1'
        , DETAIL = 'Value (Sname) = (' || NEW.sname
                     || '): Value ($2) = (' || NEW.$2
                     || '): BIOGRAPH.Entrydate cannot be '
                     || 'greater than $1.$2';
    RETURN NULL;
  END IF;
})
changequote(`,')


dnl Plpgsql fragment for checking that a date falls within
dnl a date range (inclusive), in years of age.
dnl Syntax: check_age(table, column, minage, maxage)
dnl Variables required:
dnl Restrictions: Must be used in a FOR EACH ROW trigger function.
changequote({,})
define({check_age}, {
check_age_units(`$1', `$2', `$3', `$4', `years')dnl
})
changequote(`,')




dnl Plpgsql fragment for checking that a birthdate falls within
dnl a date range (inclusive), in years of age, of another column
dnl in a table other than biograph.  (Used in the biograph trigger.)
dnl
dnl Syntax: rev_check_age(table, column, minage, maxage)
dnl Variables required:
dnl Restrictions: Must be used in a FOR EACH ROW trigger function
dnl on BIOGRAPH.
changequote({,})
define({rev_check_age}, {
rev_check_age_units(`$1', `$2', `$3', `$4', `years')dnl
})
changequote(`,')



dnl Plpgsql fragment for checking that a date falls within
dnl a date range (inclusive), in days of age.
dnl Syntax: check_age_days(table, column, minage, maxage)
dnl Variables required:
dnl Restrictions: Must be used in a FOR EACH ROW trigger function.
changequote({,})
define({check_age_days}, {
check_age_units(`$1', `$2', `$3', `$4', `days')dnl
})
changequote(`,')




dnl Plpgsql fragment for checking that a birthdate falls within
dnl a date range (inclusive), in days of age, of another column
dnl in a table other than biograph.  (Used in the biograph trigger.)
dnl
dnl Syntax: rev_check_age_days(table, column, minage, maxage)
dnl Variables required:
dnl Restrictions: Must be used in a FOR EACH ROW trigger function
dnl on BIOGRAPH.
changequote({,})
define({rev_check_age_days}, {
rev_check_age_units(`$1', `$2', `$3', `$4', `days')dnl
})
changequote(`,')




dnl Plpgsql fragment for checking that a date falls within
dnl a date range (inclusive), in some units of age.
dnl Syntax: check_age_units(table, column, minage, maxage, units)
dnl
dnl Units is "days", or "years" or some other time unit understood
dnl by postgresql.
dnl
dnl Includes entire interval during which the indiviudal is that
dnl age in the endpoint.  E.g. minage = 2, maxage = 2 means
dnl an entire year from the 2nd birthdate up to the third.
dnl
dnl Variables required:
dnl Restrictions: Must be used in a FOR EACH ROW trigger function.
changequote({,})
define({check_age_units}, {
  PERFORM biograph.sname FROM biograph
    WHERE NEW.sname = biograph.sname
          AND NEW.$2 - CAST ('incr($4) $5' AS INTERVAL) < biograph.birth 
          AND biograph.birth <= NEW.$2 - CAST ('$3 $5' AS interval)
    LIMIT 1;
  IF NOT FOUND THEN
    RAISE EXCEPTION '%: $1.$2 dates must be between $3 and $4 $5 of BIOGRAPH.Birth'
                    , NEW.sname;
    RETURN NULL;
  END IF;
})
changequote(`,')




dnl Plpgsql fragment for checking that a birthdate falls within
dnl a date range (inclusive), in some units of age, of another column
dnl in a table other than biograph.  (Used in the biograph trigger.)
dnl
dnl Units is "days", or "years" or some other time unit understood
dnl by postgresql.
dnl
dnl Syntax: rev_check_age_units(table, column, minage, maxage, units)
dnl
dnl Includes entire interval during which the indiviudal is that
dnl age in the endpoint.  E.g. minage = 2, maxage = 2 means
dnl an entire year from the 2nd birthdate up to the third.
dnl Variables required:
dnl
dnl Restrictions: Must be used in a FOR EACH ROW trigger function
dnl on BIOGRAPH.
changequote({,})
define({rev_check_age_units}, {
  PERFORM NEW.sname FROM $1
    WHERE NEW.sname = $1.sname
          AND NOT ($1.$2 - CAST ('incr($4) $5' AS interval) < NEW.birth 
                   AND NEW.birth <= $1.$2 - CAST ('$3 $5' AS interval))
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION '%: $1.$2 dates must be between $3 and $4 $5 of BIOGRAPH.Birth'
                    , NEW.sname;
    RETURN NULL;
  END IF;
})
changequote(`,')




dnl Plpgsql fragment for ensuring that an individual is at least a
dnl certain age, in some units, as of a date in a table's column.
dnl
dnl Syntax: check_age_ge_units(table, column, minage, units)
dnl
dnl Units is "days", or "years" or some other time unit understood
dnl by postgresql.
dnl
dnl Variables required:
dnl Restrictions: Must be used in a FOR EACH ROW trigger function.
changequote({,})
define({check_age_ge_units}, {
  PERFORM biograph.sname FROM biograph
    WHERE NEW.sname = biograph.sname
          AND NEW.$2 - CAST ('$3 $4' AS INTERVAL) < biograph.birth 
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION '%: $1.$2 dates must be at least $3 $4 after BIOGRAPH.Birth'
                    , NEW.sname;
    RETURN NULL;
  END IF;
})
changequote(`,')



dnl Plpgsql fragment to ensure an individual is a particular age or more,
dnl in some units, as of a date given in a column of a a table other than
dnl biograph.  (Used in the biograph trigger.)
dnl
dnl Units is "days", or "years" or some other time unit understood
dnl by postgresql.
dnl
dnl Syntax: rev_check_age_ge_units(table, column, minage, units)
dnl
dnl Restrictions: Must be used in a FOR EACH ROW trigger function
dnl on BIOGRAPH.
changequote({,})
define({rev_check_age_ge_units}, {
  PERFORM NEW.sname FROM $1
    WHERE NEW.sname = $1.sname
          AND $1.$2 - CAST ('$3 $4' AS interval) < NEW.birth
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION '%: $1.$2 dates must be at least $3 $4 after BIOGRAPH.Birth'
                    , NEW.sname;
    RETURN NULL;
  END IF;
})
changequote(`,')




dnl Plpgsql fragment for ensuring that an individual is less than a
dnl certain age, in some units, as of a date in a table's column.
dnl
dnl Syntax: check_age_lt_units(table, column, maxage, units)
dnl
dnl Units is "days", or "years" or some other time unit understood
dnl by postgresql.
dnl
dnl Variables required:
dnl Restrictions: Must be used in a FOR EACH ROW trigger function.
changequote({,})
define({check_age_lt_units}, {
  PERFORM biograph.sname FROM biograph
    WHERE NEW.sname = biograph.sname
          AND biograph.birth <= NEW.$2 - CAST ('$3 $4' AS INTERVAL)
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION '%: $1.$2 dates must not be $3 $4 or more after BIOGRAPH.Birth'
                    , NEW.sname;
    RETURN NULL;
  END IF;
})
changequote(`,')




dnl Plpgsql fragment to ensure an individual is less than a particular age,
dnl in some units, as of a date given in a column of a a table other than
dnl biograph.  (Used in the biograph trigger.)
dnl
dnl Units is "days", or "years" or some other time unit understood
dnl by postgresql.
dnl
dnl Syntax: rev_check_age_lt_units(table, column, minage, units)
dnl
dnl Restrictions: Must be used in a FOR EACH ROW trigger function
dnl on BIOGRAPH.
changequote({,})
define({rev_check_age_lt_units}, {
  PERFORM NEW.sname FROM $1
    WHERE NEW.sname = $1.sname
          AND NEW.Birth <= $1.$2 - CAST ('$3 $4' AS interval)
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION '%: $1.$2 dates must not be $3 $4 or more after BIOGRAPH.Birth'
                    , NEW.sname;
    RETURN NULL;
  END IF;
})
changequote(`,')





dnl Plpgsql fragment for ensuring that an individual is at least a
dnl certain age, in years, as of a date in a table's column.
dnl
dnl Syntax: check_age_ge_years(table, column, minage)
dnl
dnl Variables required:
dnl Restrictions: Must be used in a FOR EACH ROW trigger function.
changequote({,})
define({check_age_ge_years}, {
check_age_ge_units(`$1', `$2', `$3', `years')
})
changequote(`,')



dnl Plpgsql fragment to ensure an individual is a particular age or more,
dnl in years, as of a date given in a column of a a table other than
dnl biograph.  (Used in the biograph trigger.)
dnl
dnl Syntax: rev_check_age_ge_years(table, column, minage)
dnl
dnl Restrictions: Must be used in a FOR EACH ROW trigger function
dnl on BIOGRAPH.
changequote({,})
define({rev_check_age_ge_years}, {
rev_check_age_ge_units(`$1', `$2', `$3', `years')
})
changequote(`,')




dnl Plpgsql fragment for ensuring that an individual is less than a
dnl certain age, in years, as of a date in a table's column.
dnl
dnl Syntax: check_age_lt_years(table, column, maxage)
dnl
dnl Variables required:
dnl Restrictions: Must be used in a FOR EACH ROW trigger function.
changequote({,})
define({check_age_lt_years}, {
check_age_lt_units(`$1', `$2', `$3', `years')
})
changequote(`,')




dnl Plpgsql fragment to ensure an individual is less than a particular age,
dnl in years, as of a date given in a column of a a table other than
dnl biograph.  (Used in the biograph trigger.)
dnl
dnl Syntax: rev_check_age_lt_years(table, column, minage)
dnl
dnl Restrictions: Must be used in a FOR EACH ROW trigger function
dnl on BIOGRAPH.
changequote({,})
define({rev_check_age_lt_years}, {
rev_check_age_lt_units(`$1', `$2', `$3', `years')
})
changequote(`,')



dnl Plpgsql fragment for ensuring that an individual is at least a
dnl certain age, in days, as of a date in a table's column.
dnl
dnl Syntax: check_age_ge_days(table, column, minage)
dnl
dnl Variables required:
dnl Restrictions: Must be used in a FOR EACH ROW trigger function.
changequote({,})
define({check_age_ge_days}, {
check_age_ge_units(`$1', `$2', `$3', `days')
})
changequote(`,')



dnl Plpgsql fragment to ensure an individual is a particular age or more,
dnl in days, as of a date given in a column of a a table other than
dnl biograph.  (Used in the biograph trigger.)
dnl
dnl Syntax: rev_check_age_ge_days(table, column, minage)
dnl
dnl Restrictions: Must be used in a FOR EACH ROW trigger function
dnl on BIOGRAPH.
changequote({,})
define({rev_check_age_ge_days}, {
rev_check_age_ge_units(`$1', `$2', `$3', `days')
})
changequote(`,')




dnl Plpgsql fragment for ensuring that an individual is less than a
dnl certain age, in days, as of a date in a table's column.
dnl
dnl Syntax: check_age_lt_days(table, column, maxage)
dnl
dnl Variables required:
dnl Restrictions: Must be used in a FOR EACH ROW trigger function.
changequote({,})
define({check_age_lt_days}, {
check_age_lt_units(`$1', `$2', `$3', `days')
})
changequote(`,')




dnl Plpgsql fragment to ensure an individual is less than a particular age,
dnl in days, as of a date given in a column of a a table other than
dnl biograph.  (Used in the biograph trigger.)
dnl
dnl Syntax: rev_check_age_lt_days(table, column, minage)
dnl
dnl Restrictions: Must be used in a FOR EACH ROW trigger function
dnl on BIOGRAPH.
changequote({,})
define({rev_check_age_lt_days}, {
rev_check_age_lt_units(`$1', `$2', `$3', `days')
})
changequote(`,')







dnl Plpgsql fragment for checking MATUREDATES.Matured that there is only one
dnl CYCLES.Mdate (or emdate, or lmdate) is before the sexual maturity
dnl date.
dnl
dnl Syntax: rev_check_one_matdate(column)
dnl Variables required:
dnl Restrictions: Must be used in a FOR EACH ROW trigger function
dnl on MATUREDATES.
changequote({,})
define({rev_check_one_matdate}, {
  -- Only one of an individual's $1s can be before the matdate.
  DECLARE
    cnt INT;

  BEGIN
    SELECT INTO cnt count(*)
      FROM cycles
      WHERE cycles.sname = NEW.sname
            AND NEW.matured > cycles.$1;
    IF cnt > 1 THEN
      RAISE EXCEPTION '%: More than one CYCLES.$1 before the MATUREDATES.Matured'
                      , NEW.sname;
      RETURN NULL;
    END IF;
  END;
})
changequote(`,')




dnl Plpgsql fragment for preventing a column from changing.
dnl
dnl Syntax: cannot_change(table, column)
dnl Variables required:
dnl Restrictions: Must be used in a FOR EACH ROW update trigger function.
changequote({,})
define({cannot_change}, {
  IF NEW.$2 <> OLD.$2 THEN
    -- $2 has changed
    RAISE EXCEPTION integrity_constraint_violation USING
          MESSAGE = 'Error on UPDATE of $1'
        , DETAIL =  'Value ($2) = (' || OLD.$2
                    || '): $1.$2 cannot be changed';
    RETURN NULL;
  END IF;
})
changequote(`,')



dnl Plpgsql fragment for preventing a column from being NULL.
dnl
dnl Syntax: not_null(table, column)
dnl Variables required:
dnl Restrictions: Must be used in a FOR EACH ROW update trigger function.
changequote({,})
define({not_null}, {
  IF NEW.$2 IS NULL THEN
    -- $2 cannot be NULL
    RAISE EXCEPTION '$1.$2 cannot be NULL';
    RETURN NULL;
  END IF;
})
changequote(`,')

dnl Strings used in error messages
dnl
define(`cannot_change_msg', `This row has special meaning to Babase and may not be changed by ordinary users')dnl
define(`cannot_delete_msg', `This row has special meaning to Babase and may not be deleted by ordinary users')dnl
define(`restricted_hint_msg', `The change can only be made by a user who is allowed to create triggers on the table')dnl

dnl plpgsql fragment for checking that a column cannot be changed
dnl by regular users
dnl
dnl Syntax: restrict_all_change(table, column)
dnl
dnl table   The table
dnl column  The column
dnl
changequote({,})
define({restrict_all_change},{
    IF (NEW.$2 <> OLD.$2
        OR NEW.$2 IS NULL AND OLD.$2 IS NOT NULL
        OR NEW.$2 IS NOT NULL AND OLD.$2 IS NULL)
       AND NOT(has_table_privilege('$1', 'trigger')) THEN
      RAISE EXCEPTION insufficient_privilege USING
            MESSAGE = 'Error on ' || TG_OP || ' of $1'
          , DETAIL = '$1.$2: This column has special meaning to Babase '
                     'and may not be changed by ordinary users'
          , HINT = 'restricted_hint_msg';
    END IF;
})dnl
changequote(`,')dnl

dnl plpgsql fragment for checking that a value cannot be changed by regular users
dnl
dnl Syntax: restrict_change(table, column, value)
dnl
dnl table   The table
dnl column  The column
dnl value   The value that can't be changed by ordinary users.
dnl
changequote({,})
define({restrict_change},{
    IF (NEW.$2 = '$3' OR OLD.$2 = '$3')
       AND (NEW.$2 <> OLD.$2
            OR NEW.descr <> OLD.descr)
       AND NOT(has_table_privilege('$1', 'trigger')) THEN
      RAISE EXCEPTION insufficient_privilege USING
            MESSAGE = 'Error on ' || TG_OP || ' of $1'
          , DETAIL = 'Key($2) = ($3): cannot_change_msg'
          , HINT = 'restricted_hint_msg';
    END IF;
})dnl
changequote(`,')dnl

dnl plpgsql fragment for checking that a value cannot be deleted by regular users
dnl
dnl Syntax: restrict_delete(table, column, value)
dnl
dnl table   The table
dnl column  The column
dnl value   The value that can't be deleted by ordinary users.
dnl
changequote({,})
define({restrict_delete},{
  IF OLD.$2 = '$3'
     AND NOT(has_table_privilege('$1', 'trigger')) THEN
    RAISE EXCEPTION insufficient_privilege USING
          MESSAGE = 'Error on ' || TG_OP || ' of $1'
        , DETAIL = 'Key($2) = ($3): cannot_delete_msg'
        , HINT = 'restricted_hint_msg';
  END IF;
})dnl
changequote(`,')dnl

dnl plpgsql fragment for checking ensuring a value is NULL unless a related
dnl row exists on another table
dnl
dnl Syntax: null_unless_exists(table, column, fkey, rtable, rkey)
dnl
dnl table   The table to check
dnl column  The column that must be NULL
dnl fkey    The foreign key on table used in the relation
dnl rtable  The related table to check
dnl rkey    The key on the related table
dnl
dnl Restrictions: Must be used in a FOR EACH ROW trigger function
dnl on the table.
changequote({,})
define({null_unless_exists},{
  IF NEW.$2 IS NOT NULL THEN
    PERFORM 1
      FROM $4
      WHERE $4.$5 = NEW.$3
      LIMIT 1;
    IF NOT FOUND THEN
      RAISE EXCEPTION '$1.$2 %: $1.$3 %: $1.$2 must be NULL unless a related row exists on $4 where $4.$5 = $1.$3',
        NEW.$2,
        NEW.$3;
      RETURN NULL;
    END IF;
  END IF;
})dnl
changequote(`,')dnl

dnl Turn output back on
divert`'dnl
]}])dnl End of ifdef over the whole file.
