dnl Copyright (C) 2012, 2015 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C)  2007, 2008, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl


SELECT 'members_delete_commit_func' AS function;
CREATE FUNCTION members_delete_commit_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  DECLARE
    this_grp ranks.grp%TYPE;

  BEGIN
  -- Function for members delete fired upon transaction commit
  --
  -- GPL_notice(`  --', `2015', `The Meme Factory, Inc.  http://www.meme.com/')
  -- Copyright (C) 2007, 2008 Karl O. Pinc <kop@meme.com>

  -- Be sure that a ranked individual is always present in
  -- the ranked group's supergroup.

  PERFORM 1
    FROM members
    WHERE members.sname = OLD.sname
          AND members.date = OLD.date
          AND members.grp = OLD.grp;
  IF FOUND THEN
    -- The row was deleted but later re-inserted.  Nothing to do.
    RETURN NULL;
  END IF;

  SELECT ranks.grp
    INTO this_grp
    FROM ranks
    WHERE ranks.sname = OLD.sname
          AND ranks.rnkdate = rnkdate(OLD.date)
          AND NOT EXISTS
            (SELECT 1
               FROM members
               WHERE members.sname = OLD.sname
                     -- No rnkdate(date) index on members
                     -- so cheat and use a range so the
                     -- date index is used.
                     AND members.date >= ranks.rnkdate
                     AND members.date < ranks.rnkdate
                                        + CAST('1 month' AS INTERVAL)
                     AND supergroup(members.grp, members.date)
                         = supergroup(ranks.grp, members.date));

  IF FOUND THEN
    RAISE EXCEPTION integrity_constraint_violation USING
          MESSAGE = 'Error on MEMBERS delete commit'
        , DETAIL = 'Value (RANKS.Sname) = (' || OLD.sname
                   || '): Value (RANKS.Rnkdate) = (' || rnkdate(OLD.DATE)
                   || '): Value (RANKS.Grp) = (' || this_grp
                   || '): Value (MEMBERS.Sname) = (' || OLD.sname
                   || '): Value (MEMBERS.Date) = (' || OLD.date
                   || '): Value (MEMBERS.Grp) = (' || OLD.grp
                   || '): Individual ranked in RANKS would no longer be'
                   || ' a member of the ranked group''s supergroup';
    RETURN NULL;
  END IF;

  RETURN NULL;
  END;
$$;


SELECT 'members_delete_commit_trigger' AS trigger;
CREATE CONSTRAINT TRIGGER members_delete_commit_trigger
  AFTER DELETE
  ON members
  DEFERRABLE INITIALLY DEFERRED
  FOR EACH ROW
  EXECUTE PROCEDURE members_delete_commit_func();
