dnl Copyright (C) 2016 Jake Gordon, <jacob.gordon@duke.edu>
dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2008, 2009, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl


dnl plpgsql fragment for BLAH
dnl
dnl Syntax: BLAH(ARG)
dnl
dnl ARG   description
dnl
changequote({,})
define({BLAH},{
})dnl
changequote(`,')dnl

--  
-- mpi_parts
--
SELECT 'mpi_parts' AS table;

SELECT 'mpi_parts_func' AS function;
CREATE FUNCTION mpi_parts_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  BEGIN
  -- Function for mpi_parts insert and update triggers
  --
  -- GPL_notice(`  --', `2008', `Karl O. Pinc <kop@meme.com>')

  -- mpipid cannot change

  IF TG_OP = 'UPDATE' THEN
    cannot_change(`MPI_PARTS', `Mpipid')
    cannot_change(`MPI_PARTS', `Mpidid') -- Let's not have to think too hard
    --
    -- -- CONSORTS.Had must be a participant in the intitial interaction.
    -- -- (Don't worry about OLD.sname being NULL because that means
    -- -- it's not a known male.)
    -- IF OLD.sname IS NOT NULL 
    --    AND (NEW.sname IS NOT NULL
    --         AND OLD.sname <> NEW.sname)
    --    OR NEW.sname IS NULL THEN
    --   PERFORM 1
    --     FROM mpi_data this_mpi_data
    --          , consorts
    --          , mpi_parts
    --          , mpi_data allthese_mpi_data
    --     WHERE this_mpi_data.mpidid = NEW.mpidid
    --           AND consorts.mpiid = this_mpi_data.mpiid
    --           AND allthese_mpi_data.mpiid = this_mpi_data.mpiid
    --           AND mpi_parts.mpidid = allthese_mpi_data.mpidid
    --           AND mpi_parts.sname = consorts.had
    --     LIMIT 1;
    --   IF NOT FOUND THEN
    --     RAISE EXCEPTION 'Mpipid %: Cannot change the Sname (from % to %) because CONSORTS.Had must participate in the initial MPI interaction'
    --                     , NEW.mpipid
    --                     , OLD.sname
    --                     , NEW.sname
    --                     , OLD.Seq
    --                     , NEW.Seq;
    --     RETURN NULL;
    --   END IF;
    -- END IF;
  END IF;

  -- We probably really should abstract all these parts
  -- checks and use them in in INTERACT_DATA and MPI_DATA.

  -- No more than two MPI_PARTS rows per interaction.
  -- (We'd like to check for exactly two, but then it'd
  -- be difficult to insert new rows.)
  DECLARE
    part_count INT;

  BEGIN
    SELECT INTO part_count count(*)
           FROM mpi_parts
           WHERE mpi_parts.mpidid = NEW.mpidid;
    IF part_count > 2 THEN
      RAISE EXCEPTION '%: Mpidid %: More than 2 MPI_PARTS rows per MPI_DATA row'
                      , NEW.sname
                      , NEW.mpidid;
      RETURN NULL;
    END IF;
  END;

  -- The other mpi part that goes with the mpi interaction cannot have
  -- the same role.
  DECLARE
    part_count INT;

  BEGIN
    SELECT INTO part_count count(*)
           FROM mpi_parts
           WHERE mpi_parts.mpidid = NEW.mpidid
                 AND mpi_parts.role = NEW.role;
    IF part_count > 1 THEN
      RAISE EXCEPTION '%: Mpidid %: %: The two MPI_PARTS of a MPI_DATA cannot have the same Role'
                      , NEW.sname
                      , NEW.mpidid
                      , NEW.role;
      RETURN NULL;
    END IF;
  END;

  -- The actor and the actee cannot be the same individual.
  PERFORM 1
    FROM mpi_parts
    WHERE mpi_parts.mpidid = NEW.mpidid
          AND mpi_parts.sname = NEW.sname
          AND mpi_parts.mpipid <> NEW.mpipid
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION '%: Mpidid %: The actor and the actee on MPI_PARTS cannot be the same individual'
                    , NEW.sname
                    , NEW.mpidid;
    RETURN NULL;
  END IF;

  -- Repeat the date checks done in the mpi_data triggers as this
  -- is where we really find out who the individuals are.

  --
  -- Check the interaction date against the entrydate and statdate.
  -- 

  -- Entrydate
  PERFORM biograph.sname FROM biograph, mpi_data, mpis
    WHERE mpi_data.mpidid = NEW.mpidid
          AND mpis.mpiid = mpi_data.mpiid
          AND biograph.sname = NEW.sname
          AND mpis.date < biograph.entrydate
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION integrity_constraint_violation USING
          MESSAGE = 'Error on ' || TG_OP || ' of MPI_PARTS'
        , DETAIL = 'Key (Mpidid) = (' || NEW.mpidid
                     || '): Value (Sname) = (' || NEW.Sname
                     || '): BIOGRAPH.Entrydate cannot be bigger '
                     || 'than MPIS.Date';
    RETURN NULL;
  END IF;

  -- Statdate
  PERFORM biograph.sname FROM biograph, mpi_data, mpis
    WHERE mpi_data.mpidid = NEW.mpidid
          AND mpis.mpiid = mpi_data.mpiid
          AND biograph.sname = NEW.sname
          AND biograph.statdate < mpis.date
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'Mpidid %: %: BIOGRAPH.Statdate cannot be less than MPIS.Date'
                    , NEW.mpidid
                    , NEW.sname;
    RETURN NULL;
  END IF;

  -- MPI_DATA.Helped must be false when help is requested from an unknown
  -- individual.
  IF NEW.role = 'bb_actee' AND NEW.unksname IS NOT NULL THEN
    PERFORM 1
      FROM mpi_data, mpiacts
      WHERE mpi_data.mpidid = NEW.mpidid
            AND NOT(mpi_data.helped = FALSE) -- goofy, to allow for NULL
            AND mpiacts.mpiact = mpi_data.mpiact
            AND mpiacts.kind = 'bb_help_request_mact'
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION 'Mpipid %: Mpidid %: MPI_DATA.Helped must be FALSE when help is requested from an unknown individual'
                    , NEW.mpipid
                    , NEW.mpidid;
      RETURN NULL;
    END IF;
  END IF;

  RETURN NULL;
  END;
$$;


SELECT 'mpi_parts_trigger' AS trigger;
CREATE TRIGGER mpi_parts_trigger
  AFTER INSERT OR UPDATE
  ON mpi_parts FOR EACH ROW
  EXECUTE PROCEDURE mpi_parts_func();
