dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2009, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl


dnl plpgsql fragment for finding out if an mpiacts row is used
dnl
dnl Syntax: _used()
dnl
dnl Sets FOUND
dnl
changequote({,})
define({_used},{
  PERFORM 1
    FROM mpi_data
    WHERE mpi_data.mpiact = NEW.mpiact
    LIMIT 1;
})dnl
changequote(`,')dnl

--  
-- mpiacts
--
SELECT 'mpiacts' AS table;

SELECT 'mpiacts_update_func' AS function;
CREATE FUNCTION mpiacts_update_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  BEGIN
  -- Function for mpiacts insert and update triggers
  --
  -- GPL_notice(`  --', `2009', `Karl O. Pinc <kop@meme.com>')

  -- When the act is a reqeust MPI_DATA.Helped and MPI_DATA.Active
  -- must be non-NULL, otherwise they must be NULL.
  IF NEW.kind <> OLD.kind THEN
    IF NEW.kind = 'bb_help_request_mact' THEN
      _used()
      IF FOUND THEN
        -- Existing mpi_data.helped and mpi_data.active values are
        -- NULL and suddenly they should have a value.
        RAISE EXCEPTION 'MPIACTS.MPIAct %: This MPIAct is in use: Cannot change the Kind into a help request (from % to %) because existing MPI_DATA.Helped and MPI_DATA.Active values are NULL'
                        , NEW.mpiact
                        , OLD.kind
                        , NEW.kind;
        RETURN NULL;
      END IF;
    ELSIF OLD.kind = 'bb_help_request_mact' THEN
      _used()
      IF FOUND THEN
        -- Existing mpi_datal.helped and mpi-data.active values have
        -- a value and now should be null.
        RAISE EXCEPTION 'MPIACTS.MPIAct %: This MPIAct is in use: Cannot change the Kind from a help request to something else (from % to %) because existing MPI_DATA.Helped and MPI_DATA.Active values are non-NULL'
                        , NEW.mpiact
                        , OLD.kind
                        , NEW.kind;
        RETURN NULL;
      END IF;
    ELSIF OLD.kind = 'bb_agonism_mact' THEN
      PERFORM 1
        FROM mpi_data
        WHERE mpi_data.seq = 1
              AND mpi_data.mpiact = NEW.mpiact
        LIMIT 1;
      IF FOUND THEN
        -- The initial acts must be agonisms.
        RAISE EXCEPTION 'MPIACTS.MPIAct %: Cannot change the MPIACTS.Kind from % to % -- This MPIAct is in use as the initial interaction of a multi-party interaction; such interactions must be agonisms'
                        , NEW.mpiact
                        , OLD.kind
                        , NEW.kind;
        RETURN NULL;
      END IF;
    END IF;
  END IF;

  IF OLD.multi_first AND NOT(NEW.multi_first) THEN
    PERFORM 1
      FROM mpi_data AS first, mpi_data AS second
      WHERE first.mpiact = NEW.mpiact
            AND first.mpiid = second.mpiid
            AND first.mpidid <> second.mpidid
            AND first.seq = 1
            AND second.seq = 1
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION 'MPIAct %: Cannot change the MPIACTS.Multi_first from TRUE to FALSE -- This MPIAct is used as the intitial interaction of a multi-party interaction where there are multiple dyadic interactions with a MPI_DATA.Seq of 1'
                      , NEW.mpiact;
      RETURN NULL;
    END IF;
  END IF;

  RETURN NULL;
  END;
$$;


SELECT 'mpiacts_update_trigger' AS trigger;
CREATE TRIGGER mpiacts_update_trigger
  AFTER UPDATE
  ON mpiacts FOR EACH ROW
  EXECUTE PROCEDURE mpiacts_update_func();
