dnl Copyright (C) 2016 Jake Gordon, <jacob.gordon@duke.edu>
dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2008, 2009, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl

--  
-- mpis
--
SELECT 'mpis' AS table;

SELECT 'mpis_func' AS function;
CREATE FUNCTION mpis_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  DECLARE

  BEGIN
  -- Function for mpis insert and update triggers
  --
  -- Copyright (C) 2011 The Meme Factory, Inc.
  -- GPL_notice(`  --', `2008, 2009', `Karl O. Pinc <kop@meme.com>')

  IF TG_OP = 'UPDATE' THEN
    -- mpiid cannot change
    cannot_change(`MPIS', `Mpiid')

    -- Must have a consort context_type if there's a consort.
    IF OLD.context_type = 'bb_consort_context'
       AND NEW.context_type <> 'bb_consort_context' THEN
      PERFORM 1
        FROM consorts
        WHERE consorts.mpiid = NEW.mpiid
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION 'Mpiid %: Cannot change MPIS.Context_type from bb_consort_context (to %) when a related CONSORTS row exists'
                        , NEW.mpiid
                        , NEW.context_type;
        RETURN NULL;
      END IF;
    END IF;
  END IF;

  --
  -- Check the interaction date against the entrydate and statdate.
  -- Really, this is mostly checked in the MPI_PARTS triggers but
  -- there's no special rule saying date can't change, etc.
  -- 

  -- Entrydate
  PERFORM 1
    FROM mpi_data, mpi_parts, biograph
    WHERE mpi_data.mpiid = NEW.mpiid
          AND mpi_parts.mpidid = mpi_data.mpidid
          AND biograph.sname = mpi_parts.sname
          AND biograph.entrydate > NEW.date
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION integrity_constraint_violation USING
          MESSAGE = 'Error on ' || TG_OP || ' of MPIS'
        , DETAIL = 'Key (Mpiid) = (' || NEW.mpiid
                     || '): Value (Date) = (' || NEW.date
                     || '): MPIS.Date cannot be before '
                     || 'BIOGRAPH.Entrydate';
    RETURN NULL;
  END IF;

  -- Statdatedate
  PERFORM 1
    FROM mpi_data, mpi_parts, biograph
    WHERE mpi_data.mpiid = NEW.mpiid
          AND mpi_parts.mpidid = mpi_data.mpidid
          AND biograph.sname = mpi_parts.sname
          AND biograph.statdate < NEW.date
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'Mpiid %: %: BIOGRAPH.Statdate cannot be before MPIS.Date'
                    , NEW.mpiid
                    , NEW.date;
    RETURN NULL;
  END IF;

  RETURN NULL;
  END;
$$;


SELECT 'mpis_trigger' AS trigger;
CREATE TRIGGER mpis_trigger
  AFTER INSERT OR UPDATE
  ON mpis FOR EACH ROW
  EXECUTE PROCEDURE mpis_func();
