dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2006, 2008, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl


dnl plpgsql fragment for BLAH
dnl
dnl Syntax: BLAH(ARG)
dnl
dnl ARG   description
dnl
changequote({,})
define({BLAH},{
})dnl
changequote(`,')dnl

--  
-- ncodes
--
SELECT 'ncodes' AS table;

SELECT 'ncodes_func' AS function;
CREATE FUNCTION ncodes_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  BEGIN
  -- Function for ncodes insert and update triggers
  --
  -- GPL_notice(`  --', `2006, 2008', `Karl O. Pinc <kop@meme.com>')

  IF TG_OP = 'UPDATE' THEN
    -- Can't change the Ncode code.  It makes life too complicated
    -- to check.
    cannot_change(`NCODES', `Ncode')

    -- When their are already juvenile point samples using the neighbor,
    -- then they have to be able to continue to be used.
    IF OLD.juvenile AND NEW.juvenile <> OLD.juvenile THEN
      PERFORM 1
        FROM neighbors, point_data, samples
        WHERE neighbors.ncode = NEW.ncode
              AND point_data.pntid = neighbors.pntid
              AND samples.sid = points.sid
              AND samples.stype = 'bb_juvenile_point'
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION '%: There are already juvenile point samples using the ncode, NCODES.Juvenile cannot be made false'
                        , NEW.ncode;
        RETURN NULL;
      END IF;
    END IF;

    -- When their are already female point samples using the neighbor,
    -- then they have to be able to continue to be used.
    IF OLD.female AND NEW.female <> OLD.female THEN
      PERFORM 1
        FROM neighbors, point_data, samples
        WHERE neighbors.ncode = NEW.ncode
              AND point_data.pntid = neighbors.pntid
              AND samples.sid = points.sid
              AND samples.stype = 'bb_female_point'
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION '%: There are already female point samples using the ncode, NCODES.Female cannot be made false'
                        , NEW.ncode;
        RETURN NULL;
      END IF;
    END IF;

    -- Requires column

    -- When there are already neighbors using the code then a new neighbor
    -- requirement may not be added unless the data already satisfies the
    -- requirement.
    IF (OLD.requires IS NULL
        AND NEW.requires IS NOT NULL)
       OR (OLD.required IS NOT NULL
           AND NEW.required IS NOT NULL
           AND OLD.required <> NEW.required) THEN
      PERFORM 1
        FROM neighbors AS n
        WHERE n.ncode = NEW.ncode
              AND NOT EXISTS (SELECT 1
                                FROM neighbors AS reqd
                                WHERE reqd.pntid = n.pntid
                                      AND reqd.ncode = NEW.requires
                                LIMIT 1)
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION '%: Cannot make % a required ncode because there are some neighbors that don''t meet the requirement'
                       , NEW.ncode
                       , NEW.requires;
        RETURN NULL;
      END IF;
    END IF;

    -- Nsex column

    -- Change to make Nsex bb_nc_male
    -- Requirement for being male may not be added unless the data already
    -- satisfies this requirement.
    IF OLD.nsex <> 'bb_nc_male' AND NEW.nsex = 'bb_nc_male' THEN
      PERFORM 1
        FROM neighbors, biograph
        WHERE neighbors.ncode = NEW.ncode
              AND biograph.sname = neighbors.sname
              AND biograph.sex <> 'bb_male'
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION '%: Cannot make Nsex bb_nc_male because some NEIGHBORS with the ncode are not male'
                        , NEW.ncode;
        RETURN NULL;
      END IF;
    END IF;

    -- Nsex and Require columns

    -- Change to make Nsex bb_nc_opposite
    -- Requirement for being of the opposite sex may not be added unless
    -- the data already satisfies this requirement.
    -- (Because you can't have a nsex of bb_nc_opposite unless there's
    -- a non-NULL Require column we don't need to test for change in
    -- Require.)
    IF OLD.nsex <> 'bb_nc_opposite' AND NEW.nsex = 'bb_nc_opposite' THEN
      -- Note that neighbors with Unksname are always of the opposite
      -- sex because when you've a Unksname you've a NULL Sname
      -- and a NULL Sname will never join with biograph in the query below.
      PERFORM 1
        FROM neighbors AS this_neighbor
             , neighbors AS other_neighbor
             , biograph AS this_biograph
             , biograph AS other_biograph
        WHERE this_neighbor.ncode = NEW.ncode
              AND this_biograph.sname = this_neighbor_sname
              AND other_neighbor.pntid = this_neighbor.pntid
              AND other_neighbor.ncode = this_neighbor.require
              AND other_biograph.sname = other_neighbor.sname
              AND this_biograph.sex = other_biograph.sex
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION '%: Cannot make Nsex bb_nc_opposite because some NEIGHBORS with this ncode are not of a different sex'
                        , NEW.ncode;
        RETURN NULL;
      END IF;
    END IF;

    -- Requirement for being unique may not be added unless the data already
    -- satisfies this requirement.
    IF NOT(OLD.Nunique) AND NEW.Nunique THEN
      PERFORM 1
        FROM neighbors AS this_neighbor, neighbors AS other_neighbor
        WHERE this_neighbor.ncode = NEw.ncode
              AND other_neighbor.pntid = this_neighbor.pntid
              AND other_neighbor.nghid <> this_neighbor.nghid
              AND other_neighbor.sname = this_neighbor.sname
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION '%: Cannot make Nunique true because some POINTS having this ncode are not unique'
                        , NEW.ncode;
        RETURN NULL;
      END IF;
    END IF;

  END IF; -- Test for update

  RETURN NULL;
  END;
$$;



SELECT 'ncodes_trigger' AS trigger;
CREATE TRIGGER ncodes_trigger
  AFTER INSERT OR UPDATE
  ON ncodes FOR EACH ROW
  EXECUTE PROCEDURE ncodes_func();
