dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl


dnl plpgsql fragment for BLAH
dnl
dnl Syntax: BLAH(ARG)
dnl
dnl ARG   description
dnl
changequote({,})
define({BLAH},{
})dnl
changequote(`,')dnl

--  
-- observers
--
SELECT 'observers' AS table;

SELECT 'observers_func' AS function;
CREATE FUNCTION observers_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  BEGIN
  -- Function for observers insert and update triggers
  --
  -- GPL_notice(`  --', `2012', `The Meme Factory, Inc., http://www.meme.com/')

  IF NOT(has_table_privilege('observers', 'trigger')) THEN
    -- Observer codes cannot look like the beginning of other observer
    -- codes.  This is because the SWERB_UPLOAD view cannot always tell
    -- such observers apart because the data format coming off the
    -- GPS units has no delimiter between observers.
    --
    -- But, we do allow the Babase administrator to make such codes.

    -- A short code cannot be the same as an existing code
    -- that is truncated to the short code's length.
    PERFORM 1
      FROM observers
      WHERE observers.initials LIKE (NEW.initials || '%')
            AND observers.initials <> NEW.initials
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION 'Initials %: The Initials chosen match the starting portion of the Initials of an existing OBSERVERS row; the SWERB_UPLOAD view cannot reliably distinguish between such Initials, creating an OBSERVERS row with the chosen Initials is not recommended; Only the Babase administrator can create such Initials'
                    , NEW.initials;
      RETURN NULL;
    END IF;

    -- A long code cannot begin with the characters that make up the entirety
    -- of an existing code.
    FOR i IN REVERSE CHAR_LENGTH(NEW.initials) - 1 .. 1 LOOP
      -- Go through each possible sequence of initial characters in the
      -- chosen code.
      PERFORM 1
        FROM observers
        WHERE observers.initials = SUBSTRING(NEW.initials FROM 1 FOR i);
      IF FOUND THEN
        RAISE EXCEPTION 'Initials %: Some starting portion (%) of the Initials chosen match the entirety of the Initials of an existing OBSERVERS row; the SWERB_UPLOAD view cannot reliably distinguish between such Initials, creating an OBSERVERS row with the chosen Initials is not recommended; Only the Babase administrator can create such Initials'
                      , NEW.initials
                      , SUBSTRING(NEW.initials FROM 1 FOR i);
        RETURN NULL;
      END IF;
    END LOOP;
  END IF;
  RETURN NULL;
  END;
$$;


SELECT 'observers_trigger' AS trigger;
CREATE TRIGGER observers_trigger
  AFTER INSERT OR UPDATE
  ON observers FOR EACH ROW
  EXECUTE PROCEDURE observers_func();
