dnl Copyright (C) 2016 Jake Gordon, <jacob.gordon@duke.edu>
dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2004, 2006, 2008, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl


dnl plpgsql fragment for BLAH
dnl
dnl Syntax: BLAH(ARG)
dnl
dnl ARG   description
dnl
changequote({,})
define({BLAH},{
})dnl
changequote(`,')dnl

--  
-- parts
-- $Id: parts.m4,v 1.1 2004/10/06 17:59:35 kop Exp $
--
SELECT 'parts' AS table;

SELECT 'parts_func' AS function;
CREATE FUNCTION parts_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  BEGIN
  -- Function for parts insert and update triggers
  --
  -- GPL_notice(`  --', `2004, 2006, 2008', `Karl O. Pinc <kop@meme.com>')

  -- No more than two PARTS rows per interaction.
  -- (We'd like to check for exactly two, but then it'd
  -- be difficult to insert new rows.)
  DECLARE
    part_count INT;

  BEGIN
    SELECT INTO part_count count(*)
           FROM parts
           WHERE parts.iid = NEW.iid;
    IF part_count > 2 THEN
      RAISE EXCEPTION '%: %: More than 2 PARTS rows per INTERACT'
                      , NEW.sname
                      , NEW.iid;
      RETURN NULL;
    END IF;
  END;

  -- The other part that goes with the interaction cannot have
  -- the same role.
  DECLARE
    part_count INT;

  BEGIN
    SELECT INTO part_count count(*)
           FROM parts
           WHERE parts.iid = NEW.iid
                 AND parts.role = NEW.role;
    IF part_count > 1 THEN
      RAISE EXCEPTION '%: %: %: The two parts of an INTERACT cannot have the same Role'
                      , NEW.sname
                      , NEW.iid
                      , NEW.role;
      RETURN NULL;
    END IF;
  END;

  -- The actor and the actee cannot be the same individual.
  PERFORM 1
    FROM parts
    WHERE parts.iid = NEW.iid
          AND parts.sname = NEW.sname
          AND parts.partid <> NEW.partid
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION '%: %: The actor and the actee on PARTS cannot be the same individual'
                    , NEW.sname
                    , NEW.iid;
    RETURN NULL;
  END IF;

  -- Check sex of actor and actee as dependent upon the class of interaction.

  PERFORM 1
    FROM acts, interact_data
    WHERE interact_data.iid = NEW.iid
          AND acts.act = interact_data.act
          AND (acts.class = 'bb_mounts'
               OR acts.class = 'bb_consortship'
               OR acts.class = 'bb_ejaculation')
    LIMIT 1;
  IF FOUND THEN
    -- Mounts Consortships and Ejactultions.
    DECLARE
      this_sex biograph.sex%TYPE;
    BEGIN

    -- Get the sex
    SELECT INTO this_sex sex
      FROM biograph
      WHERE biograph.sname = NEW.sname
      LIMIT 1;

    IF NEW.role = 'bb_actor' THEN
      -- For mounts consortships and ejactultions the male is the actor.
      IF this_sex <> 'bb_male' THEN
        RAISE EXCEPTION '%: %: %: For mounts, consortships, and ejaculations the actor must be male'
                        , NEW.sname
                        , NEW.iid
                        , NEW.role;
        RETURN NULL;
      END IF;

    ELSE
      -- NEW.role is bb_actee.
      -- For mounts consortships and ejactultions the female is the actee.
      IF this_sex <> 'bb_female' THEN
        RAISE EXCEPTION '%: %: %: For mounts, consortships, and ejaculations the actee must be female'
                        , NEW.sname
                        , NEW.iid
                        , NEW.role;
        RETURN NULL;
      END IF;
    END IF;

    IF this_sex = 'bb_male' THEN
      -- A male may not participate in a mount, consort, or ejaculation
      -- before 4 years of age.
      PERFORM 1
        FROM biograph, interact_data
        WHERE interact_data.iid = NEW.iid
              AND biograph.sname = NEW.sname
              AND interact_data.date - CAST('bb_min_male_mce_age' AS interval)
                  < biograph.birth
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION '%: %: For mounts, consortships, and ejaculations males must be at least bb_min_male_mce_age of age'
                        , NEW.sname
                        , NEW.iid;
        RETURN NULL;
       END IF;
    ELSE
      -- A female may not participate in a mount, consort, or ejaculation
      -- before menarche.
      PERFORM 1
        FROM maturedates, interact_data
        WHERE maturedates.sname = NEW.sname
              AND interact_data.date >= maturedates.matured
        LIMIT 1;
      IF NOT FOUND THEN
        RAISE EXCEPTION '%: %: A female may not participate in a mount, consort, or ejaculation before menarche'
                        , NEW.sname
                        , NEW.iid;
        RETURN NULL;
      END IF;
    END IF;

    END;
  END IF;

  -- Repeat the date checks done in the interact triggers as this
  -- is where we really find out who the individuals are.
  --
  -- Check the interaction date against the entrydate and statdate.

  -- Entrydate
  -- Groomings before bb_groming_transition compare only against the
  -- first of the month where the other interaction classes compare
  -- directly against the entrydate.
  PERFORM biograph.sname FROM biograph, interact_data, acts
    WHERE interact_data.iid = NEW.iid
          AND acts.act = interact_data.act
          AND biograph.sname = NEW.sname
          AND (((acts.class = 'bb_grooming'
                  AND interact_data.date < 'bb_grooming_transition')
                AND rnkdate(biograph.entrydate) > interact_data.date)
               OR (NOT (acts.class = 'bb_grooming'
                         AND interact_data.date < 'bb_grooming_transition')
                   AND biograph.entrydate > interact_data.date))
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION integrity_constraint_violation USING
          MESSAGE = 'Error on ' || TG_OP || ' of PARTS'
        , DETAIL = 'Key (IId) = (' || NEW.iid
                     || '): Value (Sname) = (' || NEW.sname
                     || '): BIOGRAPH.Entrydate cannot be bigger than '
                     || 'INTERACT_DATA.Date, or in the case of bb_grooming '
                     || 'before bb_grooming_transition the month and year '
                     || 'of BIOGRAPH.Entrydate cannot be bigger than '
                     || 'INTERACT_DATA.Date';
    RETURN NULL;
  END IF;

  -- Statdate

  -- Grooming before bb_grooming_transition compare only against the
  -- first of the month where the other interaction classes compare
  -- directly against the statdate.
  -- (But, we don't care because the first of the month moves the
  -- recorded date to an _earlier_ point.  Doh!)
  PERFORM biograph.sname FROM biograph, interact_data --, acts
    WHERE interact_data.iid = NEW.iid
          -- AND acts.act = interact_data.act
          AND biograph.sname = NEW.sname
          -- AND ((acts.class = 'bb_grooming'
          --       AND interact_data.date < 'bb_grooming_transition'
          --       AND date_trunc('month', biograph.statdate) < interact_data.date)
          --      OR (NOT(acts.class = 'bb_grooming'
          --              AND interact_data.date < 'bb_grooming_transition')
                   AND biograph.statdate < interact_data.date
          --    ))
    LIMIT 1;
  IF FOUND THEN
    -- RAISE EXCEPTION 'Iid %: %: BIOGRAPH.Statdate cannot be less than INTERACT_DATA.Date, or in the case of bb_grooming before bb_grooming_transition the month and year of BIOGRAPH.Statdate cannot be less than INTERACT_DATA.Date'

    RAISE EXCEPTION 'Iid %: %: BIOGRAPH.Statdate cannot be less than INTERACT_DATA.Date'
                    , NEW.iid
                    , NEW.sname;
    RETURN NULL;
  END IF;


  RETURN NULL;
  END;
$$;

SELECT 'parts_delete_commit_func' AS function;
CREATE FUNCTION parts_delete_commit_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  DECLARE
    c INT;

  BEGIN
  -- Function fired on commit for parts delete trigger
  --
  -- GPL_notice(`  --', `2012', `The Meme Factory, Inc., http://www.meme.com/')

  -- Each INTERACT_DATA row should have exactly two corresponding Parts rows
  -- unless the focal individual of an ad-lib
  -- focal point sample is the lone participant.

  -- Start by finding out how many related parts rows there are.
  SELECT INTO c count(*)
    FROM parts
    WHERE parts.iid = OLD.iid;

  PERFORM 1
    FROM interact_data
    WHERE interact_data.iid = OLD.iid
          AND (c <> 2
               OR (interact_data.sid IS NOT NULL
                   AND c <> 1
                   AND NOT EXISTS
                           (SELECT *
                              FROM samples
                              WHERE samples.sid = interact_data.sid
                                    AND samples.sname = OLD.sname)));
    IF FOUND THEN
      RAISE EXCEPTION 'Error on PARTS delete commit: Iid %: Partid %: Sname %: INTERACT rows must have exactly 2 corresponding rows on PARTS unless they are collected via ad-lib focal point sampling: There are % related PARTS rows'
          , OLD.iid
          , OLD.partid
          , OLD.sname
          , c;
      RETURN NULL;
    END IF;

  RETURN NULL;
  END;
$$;

SELECT 'parts_trigger' AS trigger;
CREATE TRIGGER parts_trigger
  AFTER INSERT OR UPDATE
  ON parts FOR EACH ROW
  EXECUTE PROCEDURE parts_func();

SELECT 'parts_delete_commmit_trigger' AS trigger;
CREATE CONSTRAINT TRIGGER parts_delete_commit_trigger
  AFTER DELETE
  ON parts
  DEFERRABLE INITIALLY DEFERRED
  FOR EACH ROW
  EXECUTE PROCEDURE parts_delete_commit_func();
