dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2004-2006, 2008, 2010, 2011, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl
dnl These routines, perhaps foolishly, do not use the pid to get
dnl the sname.  They always go through conception cycpoints to
dnl cycles.sname.

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl

dnl plpgsql fragment for decrementing parity to fill a gap
dnl
dnl Syntax: dec_parity()
dnl
dnl Decrement the parity of a female's pregnancies
dnl because we deleted or updated a row and made a gap.
dnl
changequote({,})
define({dec_parity},{
  -- Fill the gap by decrementing
  -- and all the ones with a higher parity.  Do this by cascading
  -- the trigger.
  UPDATE pregs
    SET parity = parity - 1
    FROM cycpoints AS this_cycpoints
       , cycles AS this_cycles
       , cycles AS that_cycles
       , cycpoints AS that_cycpoints
    WHERE this_cycpoints.cpid = OLD.conceive
          AND this_cycles.cid = this_cycpoints.cid
          AND that_cycles.sname = this_cycles.sname
          AND that_cycpoints.cid = that_cycles.cid
          AND that_cycpoints.cpid <> this_cycpoints.cpid
          AND pregs.conceive = that_cycpoints.cpid
          AND pregs.parity % bb_unkmod_parity = OLD.parity % bb_unkmod_parity + 1;
})dnl
changequote(`,')dnl


dnl plpgsql fragment for incrementing parity to remove duplicates
dnl
dnl Syntax: inc_parity(amount)
dnl
dnl Input:
dnl  amount  How much to increment (1 or 0 is typcial)
dnl          (Supplly 0 to make the bb_unkmod_parity's place match
dnl          between the pregnancy and it's successive pregnancy.)
dnl
dnl Increment the parity of a female's pregnancies
dnl because we inserted a row or changed to make a duplicate.
dnl Make sure that if the current parity is in the bb_unkmod_parity's then
dnl subsequent paritys will be too.
dnl
changequote({,})
define({inc_parity},{

  -- Check for a duplicate, which means bump up the duplicate
  -- and all the ones with a higher parity.  Do this by cascading
  -- the trigger.
  UPDATE pregs
    SET parity = parity + $1
                 + (CASE WHEN parity - (parity % bb_unkmod_parity)
                              < NEW.parity - (NEW.parity % bb_unkmod_parity)
                         THEN bb_unkmod_parity
                         ELSE 0
                    END)
    FROM cycpoints AS this_cycpoints
       , cycles AS this_cycles
       , cycles AS that_cycles
       , cycpoints AS that_cycpoints
    WHERE this_cycpoints.cpid = NEW.conceive
          AND this_cycles.cid = this_cycpoints.cid
          AND that_cycles.sname = this_cycles.sname
          AND that_cycpoints.cid = that_cycles.cid
          AND that_cycpoints.cpid <> this_cycpoints.cpid
          AND pregs.conceive = that_cycpoints.cpid
          AND pregs.parity % bb_unkmod_parity = NEW.parity % bb_unkmod_parity;
})dnl
changequote(`,')dnl


dnl plpgsql fragment to ensure that the dad of the fetus
dnl is alive during conception.
dnl
dnl Syntax: _living_dad(column)
dnl
dnl column  A dad column on DAD_DATA.
dnl
dnl Restrictions: Must be used in a FOR EACH ROW trigger function
dnl on pregs.
changequote({,})
define({_living_dad},{

  -- Dad ($1) must be alive during conception.
  PERFORM 1
    FROM cycpoints
       , biograph AS kid
         JOIN dad_data ON (dad_data.kid = kid.sname)
         JOIN biograph AS dad ON (dad.sname = dad_data.$1)
    WHERE cycpoints.cpid = NEW.conceive
          AND kid.pid = NEW.pid
          AND dad.statdate
                < cycpoints.date
                  - bb_ovulation_len
                  - CASE WHEN dad.status = bb_alive
                         THEN bb_interpolation_max
                         ELSE 0
                    END
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'Pid %: Has a related DAD_DATA.$1 dad who is not alive during the mom''s fertile period -- on or after the Zdate - bb_ovulation_len (- another bb_interpolation_max if the dad is alive)'
                    , NEW.pid;
    RETURN NULL;
  END IF;

  -- Dad ($1) must be mature during conception.
  PERFORM 1
    FROM cycpoints
       , biograph AS kid
         JOIN dad_data ON (dad_data.kid = kid.sname)
         JOIN biograph AS dad ON (dad.sname = dad_data.$1)
         LEFT OUTER JOIN maturedates ON (maturedates.sname = dad.sname)
    WHERE cycpoints.cpid = NEW.conceive
          AND kid.pid = NEW.pid
          AND (maturedates.matured IS NULL
               OR cycpoints.date < maturedates.matured)
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'Pid %: Has a related DAD_DATA.$1 dad who is not yet matured on the Zdate'
                    , NEW.pid;
    RETURN NULL;
  END IF;
})dnl
changequote(`,')dnl


--
-- pregs
-- $Id: pregs.m4,v 1.29 2004/11/07 20:08:04 kop Exp $
--
SELECT 'pregs' AS table;

SELECT 'pregs_func' AS function;
CREATE FUNCTION pregs_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  -- Function for pregs insert and update triggers
  --
  -- GPL_notice(`  --', `2004-2006, 2008, 2010, 2011', `Karl O. Pinc <kop@meme.com>')

  DECLARE

  BEGIN

  -- Pid cannot change.
  IF TG_OP = 'UPDATE' THEN
    cannot_change(`PREGS', `Pid')
  END IF;

  -- Silly users are relying on the first 3 chars of the Pid
  -- to be the mom's Sname (they don't go through cycpoints to
  -- get the mom's Sname.)
  -- Be sure that the first 3 chars of the Pid are the mom's sname.
  -- (AFIK, this is really what locks us into 3 character Snames.)
  PERFORM 1
    FROM cycpoints, cycles
    WHERE cycpoints.cpid = NEW.conceive
          AND cycles.cid = cycpoints.cid
          AND substr`'(NEW.pid, 1, 3) <> cycles.sname
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'PREGS %; Mismatch between first 3 chars of Pid and CYCLES.Sname, via Conceive''s CYCPOINTS.Cid.  Pregnancy does not go with mother.'
                    , NEW.pid;
    RETURN NULL;
  END IF;

  -- Resumption cycle event female must be conception cycle event female.
  PERFORM NEW.pid
    FROM cycpoints AS conception, cycpoints AS resumption
         , cycles AS ccycle, cycles AS rcycle
    WHERE NEW.conceive = conception.cpid
          AND ccycle.cid = conception.cid
          AND NEW.resume = resumption.cpid
          AND rcycle.cid = resumption.cid
          AND ccycle.sname <> rcycle.sname
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'PREGS %: CYCPOINTS.Sname of pregnancy''s resumption cycle is not the Sname (CYCPOINTS.Sname) of the pregnancy''s conception cycle'
                    , NEW.pid;
    RETURN NULL;
  END IF;

  -- Conception cycle event date no later than birthdate.
  PERFORM biograph.sname FROM biograph, cycpoints
    WHERE biograph.pid = NEW.pid
          AND NEW.conceive = cycpoints.cpid
          AND ( ( (biograph.birth < cycpoints.date) IS TRUE )
                OR ( (biograph.birth < cycpoints.ldate) IS TRUE ) )
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'PREGS %: BIOGRAPH.Birth is before one of the PREGS.Conceive (CYCPOINTS) dates'
                    , NEW.pid;
    RETURN NULL;
  END IF;

  -- Birthdate cannot follow resumption cycle date.
  PERFORM biograph.sname FROM biograph, cycpoints
    WHERE biograph.pid = NEW.pid
          AND NEW.resume = cycpoints.cpid
          AND ( ( (biograph.birth > cycpoints.date) IS TRUE )
                OR ( (biograph.birth > cycpoints.edate) IS TRUE ) )
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'PREGS %: BIOGRAPH.Birth is after one of the PREGS.Resume (CYCPOINTS) dates'
                    , NEW.pid;
    RETURN NULL;
  END IF;

  -- Resumption cycle sequence number must be conception cycle
  -- sequence number + 1.
  PERFORM NEW.pid 
    FROM cycpoints AS conception
        , cycpoints AS resumption
        , cycles AS ccycle
        , cycles AS rcycle
    WHERE NEW.conceive = conception.cpid
          AND ccycle.cid = conception.cid
          AND NEW.resume = resumption.cpid
          AND rcycle.cid = resumption.cid
          AND ccycle.seq + 1 <> rcycle.seq
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'PREGS %: CYCPOINTS.Seq of pregnancy''s resumption cycle is not one bigger than the CYCPOINTS.Seq of the pregnancy''s conception cycle'
                    , NEW.pid;
    RETURN NULL;
  END IF;

  -- Parity

  -- All parity checks become unnecessary as we're automatically generating
  -- the parity.
  -- 
  --   DECLARE
  --     min_parity pregs.parity%TYPE;
  --     max_parity pregs.parity%TYPE;
  --     rcount BIGINT;
  --   BEGIN
  --     -- Parity should not have 'gaps'.
  --     -- Lowest numbered parity should either be 1 or bb_unkfirst_parity.
  --     SELECT INTO min_parity,  max_parity,  rcount
  --                 min(parity), max(parity), count(*)
  --       FROM pregs, cycpoints AS thisevent, cycpoints AS otherevents
  --       WHERE pregs.conceive = otherevents.cpid
  --             AND otherevents.sname = thisevent.sname
  --             AND thisevent.cpid = NEW.conceive;
  --     IF min_parity <> 1 AND min_parity <> bb_unkfirst_parity THEN
  --       RAISE EXCEPTION 'PREGS %: The first parity of each female must be either 1 or bb_unkfirst_parity'
  --                       , NEW.pid;
  --       RETURN NULL;
  --     END IF;
  --     IF TG_OP = 'INSERT'
  --        AND rcount <> (max_parity - min_parity + 1) THEN
  --       -- We don't test on update as then their'd be no way to
  --       -- correct mistakes.
  --       RAISE EXCEPTION 'PREGS %: PREGS.parity must be sequential for each female'
  --                       , NEW.pid;
  --       RETURN NULL;
  --     END IF;
  --   END;

  -- conceive

  -- conceive row must be a Ddate.
  PERFORM 1 FROM cycpoints
    WHERE NEW.conceive = cycpoints.cpid
          AND cycpoints.code <> 'bb_deturgesence'
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'PREGS %: PREGS.Conceive must reference a CYCPOINTS row which represents a Ddate'
                    , NEW.pid; 
    RETURN NULL;
  END IF;

  -- Can't have a juvenile point samples once there's a conception.
  PERFORM 1
    FROM samples, cycpoints, cycles
    WHERE NEW.conceive = cycpoints.cpid
          AND cycles.cid = cycpoints.cid 
          AND samples.sname = cycles.sname
          AND samples.stype = 'bb_juvenile_point'
          AND cycpoints.date <= samples.date
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'PREGS %: Cannot have a conception on or before a juvenile SAMPLES date'
                    , NEW.pid;
    RETURN NULL;
  END IF;

  -- DAD_DATA
  _living_dad(`dad_excl')
  _living_dad(`dad_1perr')
  _living_dad(`dad_5perr')
  _living_dad(`dad_allmales')
  _living_dad(`dad_consensus')

  --
  -- Having done some checking, update the conception automatic Mdate, if any.
  -- (Rely on the cycpoints trigger.)
  --
  UPDATE cycpoints SET cpid = cpid WHERE cpid = NEW.conceive;
  IF TG_OP = 'UPDATE' THEN
    UPDATE cycpoints SET cpid = cpid WHERE cpid = OLD.conceive;
  END IF;

  -- Checked for in CYCPOINTS.
  --   -- conceive's CYCPOINTS date must be on or after the maturity
  --   -- date of the mother.
  --   PERFORM 1 FROM cycpoints, maturedates
  --     WHERE NEW.conceive = cycpoints.cpid
  --           AND cycpoints.sname = maturedates.sname
  --           AND cycpoints.date < maturedates.matured
  --     LIMIT 1;
  --   IF FOUND THEN
  --     RAISE EXCEPTION 'PREGS %: Ddate of PREGS.Conceive CYCPOINTS row cannot be before the MATUREDATES.Matured of the mother'
  --                     , NEW.pid;
  --     RETURN NULL;
  --   END IF;

  -- resume

  IF NEW.resume IS NOT NULL THEN
    -- resume row must be a Tdate.
    PERFORM 1 FROM cycpoints
      WHERE NEW.resume = cycpoints.cpid
            AND cycpoints.code <> 'bb_turgesence'
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION 'PREGS %: PREGS.Resume must reference a CYCPOINTS row which represents a Tdate'
                      , NEW.pid; 
      RETURN NULL;
    END IF;

    -- Resume cycle cannot have a M date.
    DECLARE
      m_source cycpoints.source%TYPE;
      m_cpid cycpoints.cpid%TYPE;

    BEGIN
      SELECT INTO m_source, m_cpid
                  m.source, m.cpid
        FROM cycpoints AS r, cycpoints AS m
        WHERE r.cpid = NEW.resume
              AND m.cid = r.cid
              AND m.code = 'bb_menses'
        LIMIT 1;
      IF FOUND THEN
        IF m_source = 'bb_auto' THEN
          -- Silly automatic Mdate does not belong.
          -- (Really this should be redundant, as conception cycle + 1 is
          -- resumption cycle and conception cycle can't make a Mdate either,
          -- but I can't bring myself to take it out.)
          DELETE FROM cycpoints WHERE cpid = m_cpid;
        ELSE
          RAISE EXCEPTION 'PREGS %: There must not be a Mdate in the sexual cycle related to Resume'
              , NEW.pid;
          RETURN NULL;
        END IF;
      END IF;
    END;

    -- Can't have a resume without progeny.
    PERFORM 1 FROM biograph
      WHERE biograph.pid = NEW.pid
      LIMIT 1;
    IF NOT FOUND THEN
      RAISE EXCEPTION 'PREGS %: Cannot have a resume without progeny on BIOGRAPH'
                      , NEW.pid;
      RETURN NULL;
    END IF;
  END IF;  -- Resume not NULL.

  --
  -- Check parity
  --

  -- Parity must not be NULL.
  not_null(`PREGS', `Parity')


  --
  -- Done checking things.  Now update the rest of the database
  -- where appropriate.
  -- (We've already deleted any automatic Mdates that don't belong
  -- in the resume cycle.)
  --
--
--  DECLARE
--    this_sname cycles.sname%TYPE;
--
--  BEGIN
--    SELECT INTO this_sname cycles.sname FROM cycles WHERE cid = NEW.conceive;
--
--    IF TG_OP = 'INSERT' THEN
--      -- Reflect pregnancy in CYCSTATS and REPSTATS.
--      PERFORM rebuild_cyclestats(this_sname);
--    END IF;   
---
--    IF TG_OP = 'UPDATE' THEN
--      IF (OLD.conceive <> NEW.conceive)
--         OR (OLD.resume <> NEW.resume) THEN
--        -- REPSTATS and CYCSTATS need updating.
--        PERFORM rebuild_cyclestats(this_sname);
--
--        PERFORM * FROM cycles AS oldcc,
--                       cycles AS newcc
--                WHERE oldcc.cid = OLD.conceive
--                      AND newcc.cid = NEW.conceive
--                      AND oldcc.sname <> newcc.sname;
--        IF FOUND THEN
--          -- The pregnancy has changed mothers!
--          -- Update the old mother's cycle info.
--          SELECT INTO this_sname cycles.sname
--                 FROM cycles
--                 WHERE cid = OLD.conceive;
--          PERFORM rebuild_cyclestats(this_sname);
--        END IF;
--      END IF;
--    END IF;
--  END;

  RETURN NULL;
  END;
$$;

CREATE OR REPLACE FUNCTION pregs_before_func()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  -- Before function for pregs insert and update triggers
  --
  -- GPL_notice(`  --', `2005, 2006, 2008', `Karl O. Pinc <kop@meme.com>')

  -- Manages the auto-caculation of parity.
  --
  -- This routine gets away with leaving temporary "holes" in
  -- the parity because the AFTER triggers only check the
  -- immediately prior pregnancy when checking for parity problems.
  --
  -- Bugs:
  --  There is extra work going on here if NEW.parity > OLD.parity.
  -- The paritys > NEW.parity get recomputed twice, but it works
  -- so tough.
  DECLARE

  BEGIN

  -- Do we assign the parity?
  IF NEW.parity IS NULL THEN
    -- Parity should be 1 larger than previous parity.
    -- (This is where the code should go that automatically
    -- looks at the gaps and sets the parity into the bb_unkmod_paritys.)
    SELECT INTO NEW.parity pregs.parity + 1
      FROM cycpoints AS this_cycpoints
         , cycles AS this_cycles
         , cycles AS that_cycles
         , cycpoints AS that_cycpoints
         , pregs
      WHERE this_cycpoints.cpid = NEW.conceive
            AND this_cycles.cid = this_cycpoints.cid
            AND that_cycles.sname = this_cycles.sname
            AND that_cycpoints.cid = that_cycles.cid
            AND that_cycpoints.date < this_cycpoints.date
            AND pregs.conceive = that_cycpoints.cpid
      ORDER BY that_cycpoints.date DESC
      LIMIT 1;

    IF NEW.parity IS NULL THEN
      RAISE EXCEPTION '%: NULL Parity and cannot compute the Parity'
                    , NEW.pid;
      RETURN NULL;
    END IF;
  END IF;

  IF TG_OP = 'UPDATE' THEN
    -- What happens if we just change a conceive date and "move"
    -- a pregnancy "past" other pregnancies the female has?
    -- We need to decrement the pregnancies after the OLD row
    -- and increment the pregnancies after the NEW row.
    IF NEW.parity % bb_unkmod_parity <> OLD.parity % bb_unkmod_parity THEN
      inc_parity(`1')
      IF OLD.parity % bb_unkmod_parity < NEW.parity % bb_unkmod_parity THEN
        -- Only decrement when we need to.
        dec_parity()
      END IF; 
    ELSEIF NEW.parity = OLD.parity + bb_unkmod_parity THEN
      inc_parity(`0')
    END IF;
  ELSE -- TG_OP = 'INSERT'
    inc_parity(`1')
  END IF;

  RETURN NEW;
  END;
$$;


CREATE OR REPLACE FUNCTION pregs_delete_func()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  -- Function for pregs delete triggers
  --
  -- GPL_notice(`  --', `2005, 2006, 2008', `Karl O. Pinc <kop@meme.com>')

  -- Manages the auto-caculation of parity.
  DECLARE

  BEGIN
  dec_parity()

  --
  -- Update the conception automatic Mdate, if any.
  -- (Rely on the cycpoints trigger.)
  --
  UPDATE cycpoints SET cpid = cpid WHERE cpid = OLD.conceive;

  RETURN NULL;
  END;
$$;


CREATE OR REPLACE FUNCTION pregs_commit_func()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  -- Function for pregs insert and update fired upon transaction commit.
  --
  -- GPL_notice(`  --', `2012', `Karl O. Pinc <kop@meme.com>')
  --
  BEGIN

  -- Get the latest values of the row
  SELECT * INTO NEW FROM pregs WHERE pregs.pid = NEW.pid;
  IF NOT FOUND THEN
    -- Whatever row was inserted was subsequently deleted.
    -- Nothing to do.
    RETURN NULL;
  END IF;

  -- Parity must be unique per female.
  PERFORM 1
    FROM pregs
       , cycpoints AS thisevent
       , cycles AS thiscycle
       , cycpoints AS otherevents
       , cycles AS othercycle
    WHERE pregs.pid <> NEW.pid
          AND pregs.conceive = otherevents.cpid
          AND othercycle.cid = otherevents.cid
          AND othercycle.sname = thiscycle.sname
          AND thiscycle.cid = thisevent.cid
          AND thisevent.cpid = NEW.conceive
          AND NEW.parity % bb_unkmod_parity = pregs.parity % bb_unkmod_parity
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'Error on PREGS commit: PREGS %: PREGS.Parity must be unique per female'
                    , NEW.pid;
    RETURN NULL;
  END IF;

  IF NEW.parity % bb_unkmod_parity <> 1 THEN
    -- Do sequence checking against the prior parity.
    -- (Incidently, ensures that the first parity is 1 or bb_unkfirst_parity.)
    DECLARE
      last_parity pregs.parity%TYPE;
      last_conception cycpoints.date%TYPE;

    BEGIN
      -- Get the prior parity and conception date.
      SELECT INTO last_parity,  last_conception
                  pregs.parity, last_cycpoint.date
        FROM cycpoints AS this_cycpoint
           , cycles AS this_cycle
           , cycles AS last_cycle
           , cycpoints AS last_cycpoint
           , pregs
        WHERE this_cycpoint.cpid = NEW.conceive
              AND this_cycle.cid = this_cycpoint.cid
              AND last_cycle.sname = this_cycle.sname
              AND last_cycpoint.cid = last_cycle.cid
              AND pregs.conceive = last_cycpoint.cpid
              AND pregs.parity % bb_unkmod_parity < NEW.parity % bb_unkmod_parity
        ORDER BY pregs.parity DESC
        LIMIT 1;

      -- Prior parity must exist
      IF NOT FOUND THEN
        RAISE EXCEPTION 'Error on PREGS commit: %: %: Parity out of sequence: No preceeding parity'
                      , NEW.pid
                      , NEW.parity;
        RETURN NULL;
      END IF;

      -- Prior parity must be one less than current parity.
      IF last_parity % bb_unkmod_parity <> NEW.parity % bb_unkmod_parity - 1 THEN
        RAISE EXCEPTION 'Error on PREGS commit: %: %: Parity out of sequence: Mismatch with preceeding pregnancy'
                      , NEW.pid
                      , NEW.parity;
        RETURN NULL;
      END IF;

      -- Prior parity must have a lesser conception date.
      IF last_conception >= (SELECT cycpoints.date
                             FROM cycpoints
                             WHERE cpid = NEW.conceive) THEN
        RAISE EXCEPTION 'Error on PREGS commit: %: %: Parity out of sequence: Conception date predates that of the prior pregnancy'
                      , NEW.pid
                      , NEW.parity;
        RETURN NULL;
      END IF;
    END;
  END IF;

  --
  -- Female cannot have any cycpoints rows postdating a pregnancy with
  -- a null resume, unless they postdate a gap that follows the pregnancy.
  --
  IF NEW.resume IS NULL THEN
    DECLARE
      next_gap cycgapdays.date%TYPE;
      mom cycles.sname%TYPE;

    BEGIN
      -- Find the mom.
      SELECT INTO mom cycles.sname
        FROM cycpoints, cycles
        WHERE cycpoints.cpid = NEW.conceive
              AND cycles.cid = cycpoints.cid;

      -- Find the start of the next gap, if any.
      SELECT INTO next_gap cycgapdays.date
        FROM cycgapdays, cycpoints
        WHERE cycgapdays.sname = mom
              AND cycpoints.cpid = NEW.conceive
              AND cycgapdays.date > cycpoints.date
        ORDER BY cycgapdays.date
        LIMIT 1;

      PERFORM 1
        FROM cycpoints as thiscycpoints 
          JOIN cycles AS aftercycles
            ON (aftercycles.sname = mom
                AND thiscycpoints.cpid = NEW.conceive)
          JOIN cycpoints AS aftercycpoints
            ON (aftercycpoints.cid = aftercycles.cid
                AND aftercycpoints.date > thiscycpoints.date
                AND (aftercycpoints.date < next_gap
                     OR next_gap IS NULL))
        LIMIT 1;


--  PERFORM 1
--      FROM cycpoints AS thiscycpoints
--        JOIN cycles AS thiscycles
--          ON (thiscycpoints.cpid = NEW.conceive
--              AND thiscycles.cid = thiscycpoints.cid)
--        JOIN cycles AS aftercycles
--          ON (aftercycles.sname = thiscycles.sname)
--        LEFT OUTER JOIN cycgaps
--          ON (cycgaps.sname = thiscycles.sname
--              AND cycgaps.code = 'bb_cg_end'
--              AND cycgaps.date > thiscycpoints.date)
--        JOIN cycpoints AS aftercycpoints
--          ON (aftercycpoints.cid = aftercycles.cid
--              AND aftercycpoints.date > thiscycpoints.date
--              AND NOT(cycgaps.date < aftercycpoints.date))
--              -- This last equalivent to:
--              -- AND (aftercycpoints.date <= cycgaps.date
--              --      OR cycgaps.date IS NULL)
--              -- which we need because of the outer join.
--      LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION 'Error on PREGS commit: %: Ongoing pregnancy (NULL Resume) has subsequent CYCPOINTS (and no intervening gap in observation)'
                      , NEW.pid;
        RETURN NULL;
      END IF;
    END;
  END IF;

  RETURN NULL;
END;
$$;

SELECT 'pregs_trigger' AS trigger;
CREATE TRIGGER pregs_trigger
  AFTER INSERT OR UPDATE
  ON pregs FOR EACH ROW
  EXECUTE PROCEDURE pregs_func();

SELECT 'pregs_before_trigger' AS trigger;
CREATE TRIGGER pregs_before_trigger
  BEFORE INSERT OR UPDATE
  ON pregs FOR EACH ROW
  EXECUTE PROCEDURE pregs_before_func();

SELECT 'pregs_delete_trigger' AS trigger;
CREATE TRIGGER pregs_delete_trigger
  AFTER DELETE
  ON pregs FOR EACH ROW
  EXECUTE PROCEDURE pregs_delete_func();

SELECT 'pregs_commit_trigger' AS trigger;
CREATE CONSTRAINT TRIGGER pregs_commit_trigger
  AFTER INSERT OR UPDATE
  ON pregs
  DEFERRABLE INITIALLY DEFERRED
  FOR EACH ROW
  EXECUTE PROCEDURE pregs_commit_func();
