dnl Copyright (C) 2011, 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2009, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl

--  
-- Raingauges
--
SELECT 'raingauges' AS table;

SELECT 'raingauges_update_func' AS function;
CREATE FUNCTION raingauges_update_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$

  BEGIN
  -- Function for raingauges insert and update triggers
  --
  -- GPL_notice(`  --', `2009', `Karl O. Pinc <kop@meme.com>')

  -- Cannot change the WRID.
  cannot_change(`RAINGAUGES', `WRid')

  -- Cannot change the RGspan.
  IF NEW.rgspan <> OLD.rgspan
     AND NOT(has_table_privilege('raingauges', 'trigger')) THEN
    RAISE EXCEPTION 'RAINGAUGES RGspan: This column is automatically maintained by Babase and may not be changed';
    RETURN NULL;
  END IF;

  -- Cannot change the Estrgspan.
  IF NEW.estrgspan <> OLD.estrgspan
     AND NOT(has_table_privilege('raingauges', 'trigger')) THEN
    RAISE EXCEPTION 'RAINGAUGES Estrgspan: This column is automatically maintained by Babase and may not be changed';
    RETURN NULL;
  END IF;

  RETURN NULL;
  END;
$$;


SELECT 'raingauges_before_insert_func' AS function;
CREATE FUNCTION raingauges_before_insert_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  DECLARE
    prior_wrdaytime wreadings.wrdaytime%TYPE;
    prior_estdaytime wreadings.estdaytime%TYPE;
    this_wstation wreadings.wstation%TYPE;
    this_wrdaytime wreadings.wrdaytime%TYPE;
    this_estdaytime wreadings.estdaytime%TYPE;
    init_daytime rgsetups.rgsdaytime%TYPE;
    init_estdaytime rgsetups.rgsestdaytime%TYPE;
    next_wrid wreadings.wrid%TYPE;
    next_wrdaytime wreadings.wrdaytime%TYPE;
    next_estdaytime wreadings.estdaytime%TYPE;

  BEGIN
  -- Function for raingauges before INSERT trigger
  --
  -- GPL_notice(`  --', `2009', `Karl O. Pinc <kop@meme.com>')

  -- There's no particular reason to do this next check before
  -- insert rather than after, but here we are.
  -- Check that there's no raingauge measurement at the same time
  -- as a raingauge setup event.
  PERFORM 1
    FROM wreadings, rgsetups
    WHERE wreadings.wrid = NEW.wrid
          AND rgsetups.rgsdaytime = wreadings.wrdaytime
          AND rgsetups.wstation = wreadings.wstation
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'WRid %: Rain %: Cannot insert a RAINGAUGES row for the same time and location as a RGSETUPS row'
                  , NEW.wrid
                  , NEW.rain;
    RETURN NULL;
  END IF;

  -- Find the wstation that goes with this reading.
  SELECT wreadings.wstation, wreadings.wrdaytime, wreadings.estdaytime
    INTO this_wstation,      this_wrdaytime,      this_estdaytime
    FROM wreadings
    WHERE wreadings.wrid = NEW.wrid;

  -- See if the raingauge was reset since the last reading.
  SELECT rgsetups.rgsdaytime, rgsetups.rgsestdaytime
    INTO init_daytime,        init_estdaytime
    FROM rgsetups
    WHERE rgsetups.wstation = this_wstation
          AND rgsetups.rgsdaytime < this_wrdaytime
    ORDER BY rgsetups.rgsdaytime DESC
    LIMIT 1;

  IF NOT FOUND THEN
    -- The rainguage was not installed before the reading was taken.
    RAISE EXCEPTION 'WRid %: RAINGAUGES row cannot be inserted for % because there is no prior RGSETUPS row for WSTATION %'
                  , NEW.wrid
                  , this_wrdaytime
                  , this_wstation;
    RETURN NULL;
  END IF;

  -- Get the timestamp of the prior reading at this wstation.
  SELECT wreadings.wrdaytime, wreadings.estdaytime
    INTO prior_wrdaytime,     prior_estdaytime
    FROM wreadings, raingauges
    WHERE wreadings.wstation = this_wstation
          AND raingauges.wrid = wreadings.wrid
          AND wreadings.wrdaytime < this_wrdaytime
    ORDER BY wreadings.wrdaytime DESC
    LIMIT 1;

  -- Figure out the interval in seconds to the prior raingauges measurement.
  IF FOUND AND (init_daytime < prior_wrdaytime) THEN
    NEW.rgspan = this_wrdaytime - prior_wrdaytime;
    NEW.estrgspan = this_estdaytime OR prior_estdaytime;
  ELSE
    NEW.rgspan = this_wrdaytime - init_daytime;
    NEW.estrgspan = this_estdaytime OR init_estdaytime;
  END IF;

  -- Update the next reading's rgspan.
  SELECT raingauges.wrid, wreadings.wrdaytime, wreadings.estdaytime
    INTO next_wrid,       next_wrdaytime,      next_estdaytime
    FROM wreadings, raingauges
    WHERE wreadings.wstation = this_wstation
          AND wreadings.wrdaytime > this_wrdaytime
          AND raingauges.wrid = wreadings.wrid
    ORDER BY wreadings.wrdaytime
    LIMIT 1;

  IF FOUND AND NOT EXISTS (SELECT 1
                             FROM rgsetups
                             WHERE rgsetups.wstation = this_wstation
                                   AND rgsetups.rgsdaytime >= this_wrdaytime
                                   AND rgsetups.rgsdaytime < next_wrdaytime)
  THEN
    UPDATE raingauges
      SET rgspan = next_wrdaytime - this_wrdaytime,
          estrgspan = this_estdaytime OR next_estdaytime
      WHERE raingauges.wrid = next_wrid;
  END IF;

  RETURN NEW;
  END;
$$;



SELECT 'raingauges_delete_func' AS function;
CREATE FUNCTION raingauges_delete_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  SECURITY DEFINER -- so raingauges can be updated
  bb_function_set_search_path
  AS $$
  -- Function for raingauges delete trigger
  --
  -- GPL_notice(`  --', `2009', `Karl O. Pinc <kop@meme.com>')

  DECLARE
    prior_wrdaytime wreadings.wrdaytime%TYPE;
    prior_estdaytime wreadings.estdaytime%TYPE;
    this_wstation wreadings.wstation%TYPE;
    this_wrdaytime wreadings.wrdaytime%TYPE;
    next_wrid wreadings.wrid%TYPE;
    next_wrdaytime wreadings.wrdaytime%TYPE;
    next_estdaytime wreadings.estdaytime%TYPE;
    init_daytime rgsetups.rgsdaytime%TYPE;
    init_estdaytime rgsetups.rgsestdaytime%TYPE;

  BEGIN

  --
  -- Update the next measurement's rgspan.
  --

  -- Find the wstation that goes with this reading.
  SELECT wreadings.wstation, wreadings.wrdaytime
    INTO this_wstation,      this_wrdaytime
    FROM wreadings
    WHERE wreadings.wrid = OLD.wrid;

  -- Get the timestamp of the next reading at this wstation.
  SELECT raingauges.wrid, wreadings.wrdaytime, wreadings.estdaytime
    INTO next_wrid,       next_wrdaytime,      next_estdaytime
    FROM wreadings, raingauges
    WHERE wreadings.wstation = this_wstation
          AND wreadings.wrdaytime > this_wrdaytime
          AND raingauges.wrid = wreadings.wrid
    ORDER BY wreadings.wrdaytime
    LIMIT 1;

  IF NOT FOUND THEN
    -- This was the last raingauge measurement at the location.
    -- Nothing to do.
    RETURN NULL;
  END IF;

  -- Get the last reset of the raingauge before the next reading.
  SELECT rgsetups.rgsdaytime, rgsetups.rgsestdaytime
    INTO init_daytime,        init_estdaytime
    FROM rgsetups
    WHERE rgsetups.wstation = this_wstation
          AND rgsetups.rgsdaytime < next_wrdaytime
    ORDER BY rgsetups.rgsdaytime DESC
    LIMIT 1;

  IF init_daytime >= this_wrdaytime THEN
    -- Next measurement got it's span from an initialization
    -- after the measurement being deleted.
    -- Nothing to do.
    RETURN NULL;
  END IF;

  -- Get the timestamp of the last reading at this wstation.
  SELECT wreadings.wrdaytime, wreadings.estdaytime
    INTO prior_wrdaytime,     prior_estdaytime
    FROM wreadings, raingauges
    WHERE wreadings.wstation = this_wstation
          AND raingauges.wrid = wreadings.wrid
          AND wreadings.wrdaytime < this_wrdaytime
    ORDER BY wreadings.wrdaytime DESC
    LIMIT 1;

  IF FOUND AND prior_wrdaytime > init_daytime THEN
    -- The span is based on the prior raingauge measurement.
    UPDATE raingauges
      SET rgspan = next_wrdaytime - this_wrdaytime,
          estrgspan = prior_estdaytime OR next_estdaytime
      WHERE raingauges.wrid = next_wrid;
  ELSE
    -- The span is based on the initialization.
    UPDATE raingauges
      SET rgspan = next_wrdaytime - init_daytime,
          estrgspan = init_estdaytime OR next_estdaytime
      WHERE raingauges.wrid = next_wrid;
  END IF;

  RETURN NULL;
  END;
$$;

SELECT 'raingauges_before_insert_trigger' AS trigger;
CREATE TRIGGER raingauges_before_insert_trigger
  BEFORE INSERT
  ON raingauges FOR EACH ROW
  EXECUTE PROCEDURE raingauges_before_insert_func();

SELECT 'raingauges_update_trigger' AS trigger;
CREATE TRIGGER raingauges_update_trigger
  AFTER UPDATE
  ON raingauges FOR EACH ROW
  EXECUTE PROCEDURE raingauges_update_func();

SELECT 'raingauges_delete_trigger' AS trigger;
CREATE TRIGGER raingauges_delete_trigger
  AFTER DELETE
  ON raingauges FOR EACH ROW
  EXECUTE PROCEDURE raingauges_delete_func();
