dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2004, 2005, 2008, 2010, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl

--  
-- rankdates
-- $Id: rankdates.m4,v 1.4 2004/11/08 02:23:13 kop Exp $
--
SELECT 'rankdates' AS table;

SELECT 'rankdates_func' AS function;
CREATE FUNCTION rankdates_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  -- Function for rankdates insert and update triggers
  --
  -- GPL_notice(`  --', `2004, 2005, 2008, 2010', `Karl O. Pinc <kop@meme.com>')

  BEGIN
  -- rank date must be on or after 5 years of age.
  check_age_ge_years(`RANKDATES', `Ranked', `bb_min_rankdate_age')

  -- When we watched them reach rank, ranking must come
  -- before 12 years of age.
  IF NEW.Rstatus = 'bb_on' THEN
    check_age_lt_years(`RANKDATES', `Ranked', `bb_max_rankdate_age')
  END IF;

  PERFORM 1
    FROM maturedates
    WHERE sname = NEW.sname
          AND mstatus = 'bb_on';
  -- A constraint already exists that requires there be a maturedate.
  IF FOUND THEN
    -- ranked must be after the sexual maturity date.
    check_gt(`RANKDATES', `Ranked', `MATUREDATES', `Matured')
  ELSE
    -- ranked cannot be before the sexual maturity date.
    check_ge(`RANKDATES', `Ranked', `MATUREDATES', `Matured')
  END IF;

  -- ranked must be on or before statdate (and not just when
  -- the individual is dead.
  check_le(`RANKDATES', `Ranked', `BIOGRAPH', `Statdate')

  -- Juvenile male or unknown samples can't be more than a year
  -- after rank date.
  PERFORM 1
    FROM samples, biograph
    WHERE samples.sname = NEW.sname
          AND samples.stype = 'bb_juvenile_point'
          AND samples.date >= NEW.ranked + CAST ('1 year' AS Interval)
          AND biograph.sname = NEW.sname
          AND biograph.sex <> 'bb_female'
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION '%: A male''s (or unknown sex) RANKDATES.ranked cannot be a year or more before a juvenile point sample date'
                    , NEW.sname;
    RETURN NULL;
  END IF;

  RETURN NULL;
  END;
$$;

SELECT 'rankdates_trigger' AS trigger;
CREATE TRIGGER rankdates_trigger
  AFTER INSERT OR UPDATE
  ON rankdates FOR EACH ROW
  EXECUTE PROCEDURE rankdates_func();
