dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2004, 2006, 2008, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl
dnl
dnl
dnl
dnl plpgsql fragment for checking sequential rankings
dnl
dnl Syntax: seq_rank(virtualtable)
dnl
dnl virtualtable  Either NEW or OLD, to reference changed row.
dnl
dnl Variables required:
dnl
changequote({,})
define({seq_rank},{

  -- Like the sequence number check on cycles, but again,
  -- there's a difference in that we can check integrity on
  -- insert and update (as the ranks program can delete
  -- everything and then re-insert it easily).
  DECLARE
    min_rank ranks.rank%TYPE;
    max_rank ranks.rank%TYPE;
    rcount BIGINT;
  BEGIN
    -- rank should not have 'gaps'.
    -- Lowest numbered rank should be 1.
    SELECT INTO min_rank, max_rank, rcount
      min(rank), max(rank), count(*)
      FROM ranks
      WHERE ranks.rnkdate = $1.rnkdate
            AND ranks.grp = $1.grp
            AND ranks.rnktype = $1.rnktype;
    IF min_rank <> 1 THEN
      RAISE EXCEPTION 'Rnkdate %: Grp %: Rnktype %: The first rank of each Rnkdate/Grp/Rnktype must be 1'
                      , $1.rnkdate
                      , $1.grp
                      , $1.rnktype;
      RETURN NULL;
    END IF;
      IF rcount <> (max_rank - min_rank + 1) THEN
        RAISE EXCEPTION 'Rnkdate %: Grp %: Rnktype %: %: RANKS.Rank must be sequential for each Rnkdate/Grp/Rnktype'
                        , $1.rnkdate
                        , $1.grp
                        , $1.rnktype
                        , $1.sname;
      RETURN NULL;
    END IF;
  END;
})dnl
changequote(`,')dnl

--  
-- ranks
--
SELECT 'ranks' AS table;

SELECT 'ranks_func' AS function;
CREATE FUNCTION ranks_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  -- Function for RANKS insert and update triggers
  --
  -- GPL_notice(`  --', `2004, 2006, 2008', `Karl O. Pinc <kop@meme.com>')

  BEGIN

  -- Be sure the ranks are sequential.
  seq_rank(`NEW')

  -- Individual must be in supergroup, or some sub-group of the
  -- same supergroup, during rank month.
  PERFORM members.sname
    FROM members, groups as super, groups as sub
    WHERE members.sname = NEW.sname
          AND rnkdate(members.date) = NEW.rnkdate
          AND super.gid = NEW.grp
          AND sub.gid = members.grp
          AND members.origin <> 'bb_absent'
          AND supergroup(sub.gid, members.date)
              = supergroup(super.gid, members.date)
    LIMIT 1;
  IF NOT FOUND THEN
    RAISE EXCEPTION 'RANKS %: %: %: Individual not found in supergroup on Rnkdate'
                    , NEW.sname
                    , NEW.grp
                    , NEW.rnkdate;
    RETURN NULL;
  END IF;

  RETURN NULL;
  END;
$$;

SELECT 'ranks_delete_func' AS function;
CREATE FUNCTION ranks_delete_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  -- Function for ranks delete trigger
  --
  -- GPL_notice(`  --', `2004, 2006, 2008', `Karl O. Pinc <kop@meme.com>')

  BEGIN

  -- Be sure ranks are sequential.
  seq_rank(`OLD')

  RETURN NULL;
  END;
$$;



SELECT 'ranks_trigger' AS trigger;
CREATE TRIGGER ranks_trigger
  AFTER INSERT OR UPDATE
  ON ranks FOR EACH ROW
  EXECUTE PROCEDURE ranks_func();


SELECT 'ranks_delete_trigger' AS trigger;
CREATE TRIGGER ranks_delete_trigger
  AFTER DELETE
  ON ranks FOR EACH ROW
  EXECUTE PROCEDURE ranks_delete_func();
