dnl Copyright (C) 2011, 2012, The Meme Factory, Inc.
dnl Copyright (C) 2010, 2011, Karl O. Pinc
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl
dnl Remarks:
dnl  Makes real use of m4.  It's slightly disappointing to get beyond
dnl simple text substution but it beats duplicating a lot of code
dnl to make the TESTES table work.
dnl
dnl Bugs:
dnl   The invoking code needs two different macros to generate
dnl the test that selects the set of rows to be sequenced.  This
dnl can be fixed in a straightforward manner by having sequence
dnl macro accept an additional argument which determines NEW or OLD,
dnl But that means yet another argument being passed through and
dnl the macro calls defined here have enough arguments as it is.
dnl
changequote([{[,]}])dnl m4 foolery so includes include only once.
dnl                     Once the macro is in the text, change the quotes back
ifdef([{[_seqmacros.m4]}], [{[changequote(`,')]}], [{[dnl
changequote(`,')dnl
dnl
dnl Standard test for having already included the file.
define(`_seqmacros.m4')dnl
dnl
dnl
dnl Discard any output.  Copyright and edit warning is all that's output.
divert(-1)

dnl m4 includes
include(`constants.m4')
include(`macros.m4')

dnl Helper m4 macro to "evaluate" a supplied extra criteria macro.
dnl
dnl Syntax: _seqmacros_eval(gen, table)
dnl
dnl gen_sql_test  A m4 macro with the following definition:
dnl
dnl            Syntax: gen(t)
dnl
dnl            t    Name of table being sequenced
dnl
dnl            Produces: SQL code.
dnl
dnl table   The table being sequenced.
changequote({,})
define({_seqmacros_eval},{dnl
ifdef(`$1', `$1($2)')})dnl
changequote(`,')dnl

dnl Helper m4 macro expands defined macro, prepending a comma if expanding
dnl
dnl syntax: _comma_expand(macro)
dnl
dnl macro  A macro definition (or not).
dnl
changequote({,})
define({_comma_expand},{dnl
ifdef(`$1', `,
$1')})dnl
changequote(`,')dnl

dnl Helper m4 macro to "evaluate" a supplied extra criteria macro.
dnl
dnl Syntax: _seqmacros_eval_sql_test(table,
dnl                                  sql_default_set_selector,
dnl                                  gen_sql_test,
dnl                                  join_macro)
dnl
dnl table   The table being sequenced.
dnl
dnl sql_default_set_selector SQL expression which defines the set of rows
dnl                          that are sequenced.
dnl                          (E.g. testes.dartid = OLD.dartid)
dnl
dnl gen_sql_test  A optional m4 macro with the following definition:
dnl
dnl            Syntax: gen_sql_test(t)
dnl
dnl            t    Name of table being sequenced
dnl
dnl            Produces: SQL code that selects for distinct sequence-ness.
dnl
dnl join_macro Used as a flag.  Defined means the sequenced set of rows
dnl            is determined by a join.  Not defined means the sequenced set
dnl            of rows is those with equalivent 'ukey's, possibly further
dnl            restructed by the gen_sql_test criteria.
dnl              When defined the gen_sql_test macro
dnl            must be defined and is used to generate
dnl            the entire set critera that is sequenced.  When not defined
dnl            the gen_sql_test macro is ANDed with the default set defined
dnl            by sql_default_set_selector.
dnl
changequote({,})
define({_seqmacros_eval_sql_test},{dnl
ifdef(`$4', `$3($1)', `$2' ifdef(`$3', `AND $3($1)'))})dnl
changequote(`,')dnl

dnl Helper macro to generate an entire FROM clause from a join generator.
dnl
dnl Synatx: _seqmacros_gen_from(t, a, join_macro)
dnl
dnl Input:
dnl   t          The table being sequenced.
dnl   a          Alias to use for table being sequenced.
dnl   join_macro Macro used to add JOIN clauses to an existing FROM clause.
dnl
dnlchangequote({,})
dnldefine({_seqmacros_gen_from},{dnl
dnl    ifdef(`$3', `FROM $1 AS $2 _seqmacros_eval($3, $2)')dnl
dnlchangequote(`,')dnl

dnl plpgsql fragment for filling in gaps in the sequence due
dnl to row deletion or updates that increase Seq values.
dnl
dnl Syntax: _fill_seq_gap(table, key, ukey, seqcol, gen_sql_test, gen_from)
dnl
dnl table  The table with the Seq column
dnl key    The primary key on table
dnl ukey   The name of the column holding the key that's being sequenced
dnl        (Usually a foreign key of some event that contains sequenced
dnl        sub-events).
dnl seqcol The name of the column containing sequential values
dnl gen_sql_test
dnl        Optional m4 macro with the following definition:
dnl        Syntax: gen_sql_test(t)
dnl
dnl            t    Name of table being sequenced
dnl
dnl            Produces: SQL code that selects for distinct sequence-ness.
dnl            The SQL code is used to restrict the sequencing
dnl            further so as to sequence by columns in addition to the
dnl            ukey column.  An example production would be:
dnl            `testes.testside = 'R''.
dnl
dnl gen_from
dnl        Optional m4 macro with the following definition:
dnl        Syntax: gen_from()
dnl
dnl            Produces: SQL JOIN sub-clause to join tables needed
dnl            to select for distinct sequence-ness -- to join
dnl            whatever tables are needed by
dnl            the gen_sql_test macro above.
dnl
dnl (Gaps created by updates that decrease Seq values are
dnl automatically filled in as the "old" rows sequences
dnl are incremented in the duplicate removal process.)
dnl
dnl Restrictions:
dnl   Must be used in a FOR EACH ROW trigger function
dnl on the table.
dnl
dnl Notes:
dnl   Not generally used in the insert/update function because it's
dnl invoked by the  _seq_insert_update_func macro.
dnl

changequote({,})
define({_fill_seq_gap},{

  DECLARE
    newseq $1.$4%TYPE;

  BEGIN

    PERFORM 1
      FROM $1 _comma_expand(`$6')
      WHERE _seqmacros_eval_sql_test(`$1', `$1.$3 = OLD.$3', `$5', `$6')
            AND $1.$4 = OLD.$4
      LIMIT 1;
    IF NOT FOUND THEN
      -- Deleting, or an update that has increased a Seq value,
      -- has created a gap in the sequencing that must
      -- be filled by "shifting" sequence numbers downward.

      -- In case we're changing multiple rows at once, find the "bottom"
      -- of the gap in sequence numbers.
      SELECT INTO newseq $1.$4 + 1
        FROM $1 _comma_expand(`$6')
        WHERE _seqmacros_eval_sql_test(`$1', `$1.$3 = OLD.$3', `$5', `$6')
              AND $1.$4 < OLD.$4
        ORDER BY $1.$4 DESC
        LIMIT 1;

      IF newseq IS NULL THEN
        newseq := 1;
      END IF;

      -- Ensure that Seq values remain contiguous
      UPDATE $1
       SET $4 = newseq
         ifdef(`$6', `FROM $6')
       WHERE _seqmacros_eval_sql_test(`$1',
                                      `$1.$3 = OLD.$3',
                                      `$5',
                                      `$6')
             AND $1.$4 = (SELECT nextup.$4
                            FROM $1 AS nextup _comma_expand(`$6')
                            WHERE  _seqmacros_eval_sql_test(`nextup',
                                                            `nextup.$3 = OLD.$3',
                                                            `$5',
                                                            `$6')
                                  AND nextup.$4 > OLD.$4
                            ORDER BY nextup.$4
                            LIMIT 1);
    END IF;
  END;
})dnl
changequote(`,')dnl

dnl plpgsql fragment for setting the Seq value when NULL is supplied
dnl on INSERT.
dnl
dnl Syntax: _init_seq_value(table, ukey, seqcol, gen_seq_test, gen_from)
dnl
dnl table  The table with the Seq column
dnl ukey   The name of the column holding the key that's being sequenced
dnl        (Usually a foreign key of some event that contains sequenced
dnl        sub-events.)
dnl seqcol The name of the column containing sequential values
dnl gen_sql_test
dnl        Optional m4 macro with the following definition:
dnl        Syntax: gen_sql_test(t)
dnl
dnl            t    Name of table being sequenced
dnl
dnl            Produces: SQL code that selects for distinct sequence-ness.
dnl            The SQL code is used to restrict the sequencing
dnl            further so as to sequence by columns in addition to the
dnl            ukey column.  An example production would be:
dnl            `testes.testside = 'R''.
dnl
dnl gen_from
dnl        Optional m4 macro with the following definition:
dnl        Syntax: gen_from()
dnl
dnl            Produces: SQL JOIN sub-clause to join tables needed
dnl            to select for distinct sequence-ness -- to join
dnl            whatever tables are needed by
dnl            the gen_sql_test macro above.
dnl
dnl Restrictions: Must be used in a BEFORE INSERT FOR EACH ROW
dnl trigger function on the table.
changequote({,})
define({_init_seq_value},{
    IF NEW.$3 IS NULL THEN
      SELECT INTO NEW.$3 $1.$3 + 1
        FROM $1 _comma_expand(`$5')
        WHERE _seqmacros_eval_sql_test(`$1', `$1.$2 = NEW.$2', `$4', `$5')
        ORDER BY $1.$3 DESC
        LIMIT 1;
      IF NEW.$3 IS NULL THEN
        NEW.$3 := 1;
      END IF;
    END IF;
})dnl
changequote(`,')dnl

dnl plpgsql fragment serving as a template for an insert/update
dnl trigger function supporting a Seq column.
dnl
dnl Syntax: _seq_insert_update_func(table
dnl                               , key
dnl                               , ukey
dnl                               , seqcol
dnl                               , gen_sql_test
dnl                               , gen_from)
dnl
dnl table  The table with the Seq column
dnl key    The primary key on the table
dnl ukey   The name of the column holding the key that's being sequenced
dnl        (Usually a foreign key of some event that contains sequenced
dnl        sub-events.)
dnl seqcol The name of the column containing sequential values
dnl gen_sql_test
dnl        Optional m4 macro with the following definition:
dnl        Syntax: gen_sql_test(t)
dnl
dnl            t    Name of table being sequenced
dnl
dnl            Produces: SQL code that selects for distinct sequence-ness.
dnl            The SQL code is used to restrict the sequencing
dnl            further so as to sequence by columns in addition to the
dnl            ukey column.  An example production would be:
dnl            `testes.testside = 'R''.
dnl gen_from
dnl        Optional m4 macro with the following definition:
dnl        Syntax: gen_from()
dnl
dnl            Produces: SQL JOIN sub-clause to join tables needed
dnl            to select for distinct sequence-ness -- to join
dnl            whatever tables are needed by
dnl            the gen_sql_test macro above.
dnl
dnl        The generated code assumes that 'ukey' is used to perform
dnl        the join.  This assumpition is reflected in the code that
dnl        prevents the ukey value from being updated.  If this assumpition
dnl        is not correct this code must be changed, and the invoking
dnl        code must also ensure that updates do not change the set
dnl        of rows sequenced thereby throwing off the sequencing (without
dnl        explicitly updating the sequence numbers themselves).
dnl
dnl Remarks:
dnl  Note that this template contains the basic "can't change"
dnl constraints on the primary key and the foreign 'ukey'.
dnl
dnl Restrictions: Must be used in a FOR EACH ROW trigger function
dnl on the table.  See also the note at the end of gen_from description
dnl above.
dnl
dnl Bugs:
dnl   See the note at the end of the gen_from description above.
dnl
changequote({,})
define({_seq_insert_update_func},{
  -- The primary purpose of this trigger is to resequence the
  -- sequence numbers.  This is done recursively.
  --
  -- Care must be taken to account for the update of multiple
  -- rows at once, particularly because it is indeterminate
  -- which rows will fire their triggers first.
  --
  -- The approach is to first shift sequence numbers upwards
  -- to remove any duplicates.  Then shift sequence numbers
  -- back down to remove any gaps in sequencing.  As a result
  -- changing, say, A B C D E to A C D B E by assigning a
  -- Seq of 5 to B means transitioning through
  -- A <gap> C D B E, a state reached by incrementing the
  -- Seq value of E, and then reducing the Seq values
  -- of C D B and E.  So B gets a new Seq value which then
  -- must be corrected and E is operated on and then the changes
  -- are undone.  This approach makes the code simpler.
  --
  -- WARNING: 
  --   An UPDATE statement that relies on automatic resequencing
  -- to eliminate more than one duplicate Seq produces indeterminate
  -- results.  (Technically an UPDATE statement that, in the
  -- absence of any triggers, would result in more than one Seq
  -- value for any given $3 within a contiguous series of
  -- Seq values as examined after the UPDATE is an UPDATE that
  -- results in an indeterminate ordering.)

  IF TG_OP = 'UPDATE' THEN
    -- --debug
    -- RAISE NOTICE '$2 %: OLD.$4 %: NEW.$4 %',
    --  NEW.$2,
    --  OLD.$4,
    --  NEW.$4;

    cannot_change(`$1', `$2')
    cannot_change(`$1', `$3')

    -- You're not allowed to update the row to make it NULL.
    not_null(`$1', `$4')

  END IF;

  -- First get rid of a duplicate in this "slot" by shifting the "old
  -- stuff" to higher sequence numbers.  Duplicates could happen
  -- either when "shifting up" or when inserting new rows.

  UPDATE $1
    SET $4 = NEW.$4 + 1
    ifdef(`$6', `FROM $6')
    WHERE _seqmacros_eval_sql_test(`$1',
                                   `$3 = NEW.$3',
                                   `$5',
                                   `$6')
          AND $1.$2 <> NEW.$2
          AND $1.$4 = NEW.$4;

  IF TG_OP = 'INSERT' AND NEW.$4 <> 1 THEN
    -- Make sure that the sequence remains contiguous.
    PERFORM 1
      FROM $1 _comma_expand(`$6')
      WHERE _seqmacros_eval_sql_test(`$1', `$1.$3 = NEW.$3', `$5', `$6')
            AND $1.$4 = NEW.$4 - 1
      LIMIT 1;
    IF NOT FOUND THEN
      RAISE EXCEPTION '$1.$2 %: $1.$3 %: $1.$4 %: $1.$4 values must be contiguous and there is a "gap" below $4 = %',
                      NEW.$2,
                      NEW.$3,
                      NEW.$4,
                      NEW.$4;
      RETURN NULL;
    END IF;
  END IF;

  IF TG_OP = 'UPDATE' THEN
    IF OLD.$4 <> NEW.$4 THEN

      IF NOT EXISTS (SELECT 1
                       FROM $1 _comma_expand(`$6')
                       WHERE _seqmacros_eval_sql_test(`$1',
                                                      `$1.$3 = NEW.$3',
                                                      `$5',
                                                      `$6')
                            AND $1.$4 = NEW.$4 + 1
                       LIMIT 1)
         THEN
        -- There's no row having the next available sequence number.
        -- Make sure that the Seq "slot" "above" this one is full
        -- as is necessary when "shifting down".

        UPDATE $1
          SET $4 = NEW.$4 + 1
          ifdef(`$6', `FROM $6')
          WHERE _seqmacros_eval_sql_test(`$1',
                                         `$3 = NEW.$3',
                                         `$5',
                                         `$6')
                AND $1.$4 = (SELECT nextup.$4
                            FROM $1 AS nextup _comma_expand(`$6')
                            WHERE _seqmacros_eval_sql_test(`nextup',
                                                           `nextup.$3 = NEW.$3',
                                                           `$5',
                                                           `$6')
                                  AND nextup.$4 >= NEW.$4 + 2
                            ORDER BY nextup.$4
                            LIMIT 1);
      END IF;

      IF OLD.$4 < NEW.$4 THEN
        _fill_seq_gap(`$1', `$2', `$3', `$4', `$5', `$6')
      END IF;

    END IF;
  END IF;
})dnl
changequote(`,')dnl

dnl Turn output back on
divert`'dnl
]}])dnl End of ifdef over the whole file.
