dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2004, 2006, 2008, 2010, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl
dnl
dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl
include(`sexskinmacros.m4')

--
-- sexskins
--
SELECT 'sexskins' AS table;

SELECT 'sexskins_before_func' AS function;
CREATE FUNCTION sexskins_before_func ()
  RETURNS TRIGGER
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  -- Before function for sexskins insert and update triggers
  --
  -- GPL_notice(`  --', `2004, 2006, 2008', `Karl O. Pinc <kop@meme.com>')
  --
  -- Assign the sexskins row to a cycle.
  --
  -- Every row inserted or updated is re-assigned to the correct cycle.
  --

  DECLARE
    this_sname cycles.sname%TYPE;
    new_cid cycles.cid%TYPE;

  BEGIN
    --
    -- Get the female's sname, we'll need it a lot.
    -- One will always be found because of the table constraints.
    --
    SELECT INTO this_sname cycles.sname
      FROM cycles
      WHERE cycles.cid = NEW.cid
      LIMIT 1;

    _find_sexskin_cid(`NEW', `new_cid')

    -- Make sure the sexskins row is in the right cycle.
    NEW.cid = new_cid;

    RETURN NEW;
  END;
$$;

SELECT 'sexskins_func' AS function;
CREATE FUNCTION sexskins_func ()
  RETURNS TRIGGER
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  -- Function for sexskins insert and update triggers
  --
  -- GPL_notice(`  --', `2004, 2006, 2008, 2010', `Karl O. Pinc <kop@meme.com>')

  BEGIN

  -- Check the simple stuff first.

  IF TG_OP = 'UPDATE' THEN
    cannot_change(`SEXSKINS', `Sxid')

    -- Can't move the sexskin between females by changing
    -- the cid.
    IF OLD.cid <> NEW.cid THEN
      PERFORM 1
        FROM cycles AS new_cycle, cycles AS old_cycle
        WHERE new_cycle.cid = NEW.cid
              AND old_cycle.cid = OLD.cid
              AND old_cycle.sname <> new_cycle.sname
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION 'Sxid %: Old Cid %: %: Cannot move sexskins between females'
                        , NEW.sxid
                        , OLD.cid
                        , NEW.date;
        RETURN NULL;
      END IF;
    END IF;

    IF OLD.date = NEW.date AND OLD.size = NEW.size THEN
      -- Nothing's changed but perhaps the cid.
      -- Don't bother to check any further because the
      -- validation checks have already validated and the
      -- checks that compare against other sexskins in the
      -- cycle may compare against sexskins that have not
      -- yet been updated to be in the correct cycle.

      RETURN NULL;
    END IF;

  END IF;

  -- Check date column.

  -- Must be after the birthdate.
  PERFORM 1
    FROM biograph, cycles
    WHERE NEW.cid = cycles.cid
          AND biograph.sname = cycles.sname
          AND biograph.birth >= NEW.date
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'SEXSKINS Cid %: Sxid %: %: BIOGRAPH.Birth must be before SEXSKINS.Date'
                    , NEW.cid
                    , NEW.sxid
                    , NEW.date;
    RETURN NULL;
  END IF;

  -- Must be before the statdate
  PERFORM 1
    FROM biograph, cycles
    WHERE NEW.cid = cycles.cid
          AND biograph.sname = cycles.sname
          AND biograph.statdate < NEW.date
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'SEXSKINS Cid %: Sxid %: %: BIOGRAPH.Statdate cannot be less than SEXSKINS.Date'
                    , NEW.cid
                    , NEW.sxid
                    , NEW.date;
    RETURN NULL;
  END IF;


  -- Sname from CYCLES and date should be unique.
  -- (It's not enough to have the cid, date be unique
  -- in case there's another cycle with the same date.)
  -- (This also takes care of the date/cid uniqueness.)
  PERFORM 1
    FROM cycles AS this_cycle, cycles AS other_cycles, sexskins
    WHERE NEW.cid = this_cycle.cid
          AND this_cycle.sname = other_cycles.sname
          AND other_cycles.cid = sexskins.cid
          AND sexskins.date = NEW.date
          AND NEW.sxid <> sexskins.sxid
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'SEXSKINS %: %: Duplicate Sname, Date combination'
                    , NEW.cid
                    , NEW.date;
  END IF;

  --
  -- Because the checks below are not done when cycpoints is changed
  -- these checks are duplicated in the batched error checking.
  --

  -- Check that turgecesence dosen't commence before or after
  -- the cycle's T date.

  -- Of all measurements in a cycle, the earliest masurement larger
  -- than 5 must have a date >= the cycle's tdate.

  IF NEW.size > bb_max_skin_before_t THEN
    PERFORM 1
      FROM cycpoints
      WHERE cycpoints.cid = NEW.cid
            AND cycpoints.code = 'bb_turgesence'
            AND cycpoints.date > NEW.date
      LIMIT 1;
    IF FOUND THEN
      -- It's before the Tdate
      RAISE EXCEPTION 'CID %: %: Turgesence (from SEXSKINS) commences for this cycle before the cycle''s T date (Size bigger than bb_max_skin_before_t before the Tdate)'
                      , NEW.cid
                      , NEW.date;
      RETURN NULL;
    END IF;
  END IF;       

  -- The Tdate of a cycle must be after the dates of all the
  -- cycle's sexskin measurements of zero that precede the earliest
  -- non-zero measurement occurring in the cycle.
  -- Which means:
  -- If the size is zero and before the first non-zero of the cycle
  -- then it must also be before the cycle's Tdate.
  IF NEW.size = 0 THEN
    PERFORM 1
      FROM cycpoints
      WHERE cycpoints.cid = NEW.cid
            AND cycpoints.date <= NEW.date -- not before the Tdate
            AND cycpoints.code = 'bb_turgesence'
            AND NEW.date < (SELECT sexskins.date
                              FROM sexskins
                              WHERE sexskins.cid = NEW.cid
                                    AND sexskins.size >= 1
                              ORDER BY sexskins.date
                              LIMIT 1)
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION 'CID %: %: All zero SEXSKINS measurements before the first that is 1 or larger must be before the cycle''s Tdate'
                    , NEW.cid
                    , NEW.date;
      RETURN NULL;
    END IF;
  END IF;          

  -- Earliest >0 measurement must be after the cycle's M date.
  -- (Except <= bb_max-skin_on_mdate measurements are allowed on the Mdate.)
  IF NEW.size > bb_max_skin_before_m THEN
    DECLARE
      this_mdate cycpoints.date%TYPE;

    BEGIN
      -- First check to see that we've got a Mdate in this cycle.
      SELECT INTO this_mdate cycpoints.date
        FROM cycpoints
        WHERE cycpoints.cid = NEW.cid
              AND cycpoints.code = 'bb_menses';
      IF NOT FOUND THEN
        -- Make sure there's not an Mdate because there was a pregnancy.
        -- (Note: No need to test cycpoints.code.  It'll be right.)
        PERFORM 1
          FROM cycpoints, pregs
          WHERE cycpoints.cid = NEW.cid
                AND pregs.resume = cycpoints.cpid
          LIMIT 1;
        IF NOT FOUND THEN
          -- Make sure there's no Mdate because this is the first cycle.
          PERFORM 1
            FROM cycles, cycpoints, maturedates
            WHERE cycles.cid = NEW.cid
                  AND cycpoints.cid = NEW.cid
                  AND cycpoints.code = 'bb_turgesence'
                  AND maturedates.sname = cycles.sname
                  AND maturedates.matured = cycpoints.date
                  AND maturedates.mstatus = 'bb_on'
            LIMIT 1;
          IF NOT FOUND THEN
            RAISE EXCEPTION 'Sxid %: Cid %: %: No Mdate found for this cycle'
                          , NEW.sxid
                          , NEW.cid
                          , NEW.date;
            RETURN NULL;
          END IF;
        END IF;
      END IF;

      -- Then check to see that we're not before the Mdate.
      IF NEW.date <= this_mdate THEN
        IF NEW.date = this_mdate THEN
          IF NEW.size > bb_max_skin_on_m THEN
            PERFORM 1
              FROM cycpoints
              WHERE cycpoints.cid = NEW.cid
                    AND cycpoints.date = this_mdate
              LIMIT 1;
            IF FOUND THEN
              IF NEW.size > bb_max_skin_on_mt THEN
                RAISE EXCEPTION 'Sxid %: Cid %: %: Turgesence %: Turgesence (from SEXSKINS) on the cycle''s shared M and Tdate > bb_max_skin_on_mt'
                               , NEW.sxid
                               , NEW.cid
                               , NEW.date
                               , NEW.size;
                RETURN NULL;
              END IF;
            ELSE
              -- The Mdate is not a Tdate.  No mitigating circumstances.
              -- Report an error.
              RAISE EXCEPTION 'Sxid %: Cid %: %: Turgesence %: Turgesence (from SEXSKINS) on the cycle''s Mdate > bb_max_skin_on_m'
                              , NEW.sxid
                              , NEW.cid
                              , NEW.date
                              , NEW.size;
              RETURN NULL;
            END IF;
          END IF;
        ELSE
          -- Swelling starts before Mdate.
          -- Report an error.
          RAISE EXCEPTION 'Sxid %: Cid %: %: Turgesence (from SEXSKINS) commences for this cycle before the cycle''s M date'
                          , NEW.sxid
                          , NEW.cid
                          , NEW.date;
          RETURN NULL;
        END IF;
      END IF;
    END;
  END IF;          

  -- We don't check here that the Ddate occurs after the last measurement
  -- before the largest measurment of the cycle so that there's flexibility
  -- in the order in which sexskins are entered.

  -- There should be no overlap of sexskins dates between cycles.
  -- (This is taken care of by the automatic assignment to cycles.)

  RETURN NULL;
  END;
$$;

/* Looks like we don't need a delete trigger for sexskins
 * SELECT 'sexskins_delete_func' AS function;
 * CREATE FUNCTION sexskins_delete_func ()
 *   RETURNS TRIGGER
 *   LANGUAGE plpgsql
  bb_function_set_search_path
 *   AS $$
 *   -- Function for sexskins delete trigger
 *   --
 *   -- GPL_notice(`  --', `2004, 2008', `Karl O. Pinc <kop@meme.com>')
 * 
 *   BEGIN
 * 
 * 
 *   -- We don't check here that the Ddate occurs after the last measurement
 *   -- before the largest measurment of the cycle so that there's flexibility
 *   -- in the order in which sexskins are deleted.
 * 
 *   RETURN NULL;
 *   END;
 * $$;
 */

SELECT 'sexskins_before_trigger' AS trigger;
CREATE TRIGGER sexskins_before_trigger
  BEFORE INSERT OR UPDATE
  ON sexskins FOR EACH ROW
  EXECUTE PROCEDURE sexskins_before_func ();

SELECT 'sexskins_trigger' AS trigger;
CREATE TRIGGER sexskins_trigger
  AFTER INSERT OR UPDATE
  ON sexskins FOR EACH ROW
  EXECUTE PROCEDURE sexskins_func ();

/*
 * SELECT 'sexskins_delete_trigger' AS trigger;
 * CREATE TRIGGER sexskins_delete_trigger
 *   AFTER DELETE
 *   ON sexskins FOR EACH ROW
 *   EXECUTE PROCEDURE sexskins_delete_func ();
 */
