dnl Copyright (C) 2011-2013 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2011, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl
include(`seqmacros.m4')dnl


dnl plpgsql fragment for producing code that tests for unique seq-ness
dnl by focal_grp and places the row in the proper sequence.
dnl
dnl Syntax: _initial_seq_per_new_focal_grp(table)
dnl
dnl table   The name of the swerb_bes table.
dnl
dnl Remarks:
dnl  Does not pay attention to SWERB_DATA because there won't be
dnl any when the SWERB_BES row is inserted and we don't try to
dnl automatically sequence existing rows when Start is NULL.
dnl
changequote({,})
define({_initial_seq_per_new_focal_grp},{dnl
this_sdd.did = NEW.did
AND $1.did = other_sdd.did
AND $1.beid <> NEW.beid
AND $1.focal_grp = NEW.focal_grp AND other_sdd.date = this_sdd.date
AND (NEW.start IS NULL
     OR (NEW.start IS NOT NULL
         AND (($1.start IS NULL
               AND EXISTS (SELECT 1
             FROM swerb_data AS other_sd_start
             WHERE other_sd_start.beid = $1.beid
                   AND other_sd_start.time <= NEW.start)
              ) OR ($1.start IS NOT NULL
                    AND $1.start <= NEW.start))))})dnl
changequote(`,')dnl

dnl plpgsql fragment for producing code that tests for unique seq-ness
dnl by focal_grp.
dnl
dnl Syntax: _seq_per_new_focal_grp(table)
dnl
dnl table   The name of the swerb_bes table.
dnl
changequote({,})
define({_seq_per_new_focal_grp},{dnl
this_sdd.did = NEW.did
AND $1.did = other_sdd.did
AND $1.beid <> NEW.beid
AND $1.focal_grp = NEW.focal_grp AND other_sdd.date = this_sdd.date
})dnl
changequote(`,')dnl


dnl plpgsql fragment for producing code that tests for unique seq-ness
dnl by focal_grp.
dnl
dnl Syntax: _seq_per_old_focal_grp(table)
dnl
dnl table   The name of the swerb_bes table.
dnl
changequote({,})
define({_seq_per_old_focal_grp},{dnl
this_sdd.did = OLD.did AND $1.did = other_sdd.did
 AND $1.beid <> OLD.beid AND $1.focal_grp = OLD.focal_grp AND other_sdd.date = this_sdd.date})dnl
changequote(`,')dnl


dnl Generate a JOIN sub-clause to join swerb_bes to swerb_departs_data.
dnl
dnl Syntax: _gen_from()
dnl
dnl _gen_from   A m4 macro that takes no arguments.
changequote({,})
define({_gen_from},{dnl
swerb_departs_data AS other_sdd
, swerb_departs_data AS this_sdd})dnl
changequote(`,')dnl

--  
-- swerb_bes
--
SELECT 'swerb_bes' AS table;
SELECT 'swerb_bes_before_insert_func' AS function;
CREATE FUNCTION swerb_bes_before_insert_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  BEGIN
  -- Function for swerb_bes before insert trigger
  --
  -- GPL_notice(`  --', `2011', `Karl O. Pinc <kop@meme.com>')
  --
  -- Remarks:
  -- WARNING: See remarks in swerb_bes_func().

    _init_seq_value(`SWERB_BES',
                    `DId',
                    `Seq',
                    `_initial_seq_per_new_focal_grp',
                    `_gen_from')

    RETURN NEW;
  END;
$$;

SELECT 'swerb_bes_func' AS function;
CREATE FUNCTION swerb_bes_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  BEGIN
  -- Function for swerb_bes insert and update triggers
  --
  -- Copyright (C) 2011-2012, The Meme Factory, Inc.
  -- GPL_notice(`  --', `2011', `Karl O. Pinc <kop@meme.com>')
  --
  -- Bugs:
  --  We could probably get some efficency by moving the reseqencing
  -- to the commit trigger.  Or not.

/* debug
  if TG_OP = 'INSERT' THEN
    raise warning 'TG_OP %: BEId %: DId %: Focal_grp %: Start %: Btimeest %: Stop %: Etimetest %: Seq %'
                , TG_OP
                , NEW.beid
                , NEW.did
                , NEW.focal_grp
                , NEW.start
                , NEW.btimeest
                , NEW.stop
                , NEW.etimeest
                , NEW.seq;
  else
    raise warning 'TG_OP %:BEId %: DId %: Focal_grp %:old Start %: Start %: Btimeest %:old Stop %: Stop %: Etimetest %:old Seq %: Seq %'
                , TG_OP
                , NEW.beid
                , NEW.did
                , NEW.focal_grp
                , OLD.start
                , NEW.start
                , NEW.btimeest
                , OLD.stop
                , NEW.stop
                , NEW.etimeest
                , OLD.seq
                , NEW.seq;
  end if;
*/

  IF TG_OP = 'UPDATE' THEN
    -- Cannot change things that would throw the sequencing off.
    -- (The Beid and Did is tested for in the sequencing macro.)
    cannot_change(`SWERB_BES', `Focal_grp')
  END IF;

  --
  -- Start and stop times are assigned by the SWERB_DATA trigger
  -- from the SWERB_DATA.Time values of begin and end rows.
  --

  IF NEW.start IS NOT NULL THEN

    -- Start time must be after the departature time, if any.
    PERFORM 1
      FROM swerb_departs_data
      WHERE swerb_departs_data.did = NEW.did
            AND swerb_departs_data.time >= NEW.start
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION 'BEId %: DId %: Start %: The SWERB_BES.Start time must be after the releated SWERB_DEPARTS_DATA.Time'
                    , NEW.beid
                    , NEW.did
                    , NEW.start;
      RETURN NULL;
    END IF;

    -- Start time cannot be after SWERB_DATA.Time.
    PERFORM 1
      FROM swerb_data
      WHERE swerb_data.beid = NEW.beid
            AND swerb_data.time < NEW.start
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION 'BEId %: Did %: Start %: The SWERB_BES.Start time cannot be after related SWERB_DATA.Time values'
                    , NEW.beid
                    , NEW.did
                    , NEW.start;
      RETURN NULL;
    END IF;
  END IF;

  -- Stop time cannot be before SWERB_DATA.time.
  PERFORM 1
    FROM swerb_data
    WHERE swerb_data.beid = NEW.beid
          AND swerb_data.time > NEW.stop
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'BEId %: Did %: Stop %: The SWERB_BES.Stop time cannot be before related SWERB_DATA.Time values'
                    , NEW.beid
                    , NEW.did
                    , NEW.stop;
    RETURN NULL;
  END IF;

  -- Observed group must be in existance.
  DECLARE
    this_date swerb_departs_data.date%TYPE;

  BEGIN
    -- Get the date of the observation.
    SELECT INTO STRICT this_date
                       swerb_departs_data.date
      FROM swerb_departs_data
      WHERE swerb_departs_data.did = NEW.did;

    -- Can't use before the group comes into existance.
    PERFORM 1
      FROM groups
      WHERE groups.gid = NEW.focal_grp
            AND groups.start > this_date
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION 'BEId %: DId %: Focal_grp %: SWERB_DEPARTS_DATA.Date %: The SWERB_BES.Focal_grp must be in existance when observed: Error SWERB_DEPARTS_DATA.Date < GROUPS.Start'
                    , NEW.beid
                    , NEW.did
                    , NEW.focal_grp
                    , this_date;
      RETURN NULL;
    END IF;

    -- Can't use after the group ceases to exist.
    PERFORM 1
      FROM groups
      WHERE groups.gid = NEW.focal_grp
            AND this_date > groups.cease_to_exist
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION 'BEId %: DId %: Focal_grp %: SWERB_DEPARTS_DATA.Date %: The SWERB_BES.Focal_grp must be in existance when observed: Error SWERB_DEPARTS_DATA.Date > GROUPS.Cease_To_Exist'
                    , NEW.beid
                    , NEW.did
                    , NEW.focal_grp
                    , this_date;
      RETURN NULL;
    END IF;
  END;

  -- Observed other group must not be the same as the focal group unless the
  -- observed group is a subgroup or is the unknown group.
  PERFORM 1
    FROM swerb_data
    WHERE swerb_data.beid = NEW.beid
          AND swerb_data.event = 'bb_swerb_ogrp'
          AND swerb_data.subgroup = FALSE
          AND swerb_data.seen_grp = NEW.focal_grp
          AND swerb_data.seen_grp <> bb_unk_grp
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION integrity_constraint_violation USING
          MESSAGE = 'Error on ' || TG_OP || ' of SWERB_BES'
        , DETAIL = 'Key (Beid) = (' || NEW.beid
                     || '): Value (Did) = (' || NEW.did
                     || '): Value (Focal_Grp) = (' || NEW.focal_grp
                     || '): The SWERB_BES.Focal_grp must not be the same as '
                     || 'the SWERB_DATA.Seen_grp when SWERB_DATA.Event = '
                     || 'bb_swerb_ogrp and SWERB_DATA.Subgroup = TRUE unless '
                     || 'the SWERB_DATA.Seen_grp <> bb_unk_grp';
    RETURN NULL;
  END IF;

  -- Observed group must be the same as the focal group when
  -- not doing an ad-hoc observation of another group.
  PERFORM 1
    FROM swerb_data
    WHERE swerb_data.beid = NEW.beid
          AND swerb_data.event <> 'bb_swerb_ogrp'
          AND swerb_data.seen_grp <> NEW.focal_grp
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'BEId %: DId %: Focal_grp %: The SWERB_BES.Focal_grp must be the same as the SWERB_DATA.Seen_grp when SWERB_DATA.Event <> bb_swerb_ogrp'
                  , NEW.beid
                  , NEW.did
                  , NEW.focal_grp;
    RETURN NULL;
  END IF;

  -- Automatically resequence by Start/Stop times.
  -- We do this here instead of with the macros so as to separeate
  -- out duplicate and gap removal from the sequencing itself.
  IF TG_OP = 'UPDATE' THEN
    IF NEW.seq = OLD.seq
       -- The seq was not deliberately changed, either manually or by
       -- a recursive trigger invocation so it's ok to automatically
       -- sequence by time.
       AND (NEW.start <> OLD.start
             -- Changing Start to NULL does not affect existing sequencing.
             -- OR (NEW.start IS NULL AND OLD.start IS NOT NULL)
             -- But a NULL Start needs to be re-sequenced by the SWERB_DATA
             -- trigger when rows are deleted.
             OR (NEW.start IS NULL AND OLD.start IS NULL)
             OR (NEW.start IS NOT NULL AND OLD.start IS NULL)) THEN

        -- Seq by stop when start is the same, but we don't want to do
        -- this because we're not doing it when initially sequencing.
        --   OR (NEW.start IS NULL
        --        AND OLD.start IS NULL
        --        AND (NEW.stop <> OLD.stop
        --             OR (NEW.stop IS NULL AND OLD.stop IS NOT NULL)
        --             OR (NEW.stop IS NOT NULL AND OLD.stop IS NULL)))) THEN

      DECLARE targetseq swerb_bes.seq%TYPE;

      BEGIN

        SELECT swerb_bes.seq INTO targetseq
          FROM swerb_bes
               JOIN swerb_departs_data
                    AS other_sdd
                    ON (other_sdd.did = swerb_bes.did)
             , swerb_departs_data AS this_sdd
          WHERE this_sdd.did = NEW.did
                AND other_sdd.date = this_sdd.date
                AND swerb_bes.focal_grp = NEW.focal_grp
                AND swerb_bes.beid <> NEW.beid
                AND swerb_bes.start > NEW.start
                -- Again, not sequencing by stop within start.
                --     OR (NEW.start IS NULL
                --         AND swerb_bes.start IS NOT NULL
                --         AND (NEW.stop < swerb_bes.stop
                --              OR swerb_bes.stop IS NULL
                --              OR NEW.stop IS NULL)))
          ORDER BY swerb_bes.start
               --, swerb_bes.stop
          LIMIT 1;

        IF targetseq <> NEW.seq THEN

          IF targetseq IS NULL THEN
            -- There are no bouts of observeration with a larger Seq.
            -- This should be sequenced after the last Seq.
            SELECT swerb_bes.seq + 1 INTO targetseq
              FROM swerb_bes
                  JOIN swerb_departs_data
                         AS other_sdd
                         ON (other_sdd.did = swerb_bes.did)
                     , swerb_departs_data AS this_sdd
              WHERE this_sdd.did = NEW.did
                    AND other_sdd.date = this_sdd.date
                    AND swerb_bes.focal_grp = NEW.focal_grp
                    AND swerb_bes.beid <> NEW.beid
              ORDER BY swerb_bes.start DESC
              LIMIT 1;

            IF targetseq IS NOT NULL
               AND targetseq <> NEW.seq THEN
              UPDATE swerb_bes
                SET seq = targetseq
                WHERE beid = NEW.beid;
              NEW.seq := targetseq;  -- Pickup the change
              -- If nothing was found (targetseq is still NULL and so
              -- we don't update) then this row is the only
              -- swerb_bes row for that date/group.
              -- the ordianary gap removal
              -- process will ensure that we have a Seq of 1.
            END IF;

          ELSE
            -- targetseq IS NOT NULL
            UPDATE swerb_bes
              SET seq = targetseq
              WHERE beid = NEW.beid;
            NEW.seq := targetseq;  -- Pickup the change
          END IF;
        END IF;

      END;
    END IF;
  END IF;

  -- Everything checks out, now resequence removing gaps and duplicates.
  _seq_insert_update_func(`SWERB_BES',
                          `Beid',
                          `Did',
                          `Seq',
                          `_seq_per_new_focal_grp',
                          `_gen_from')

  -- Note that at this point there is no guarentee that a
  -- recursive resequencing has finished, because the above
  -- macro does not issue a single atomic update but instead
  -- issues a number of updates.  But we do know that
  -- the sequencing is in the right order; it's just that
  -- there may still be gaps in the sequencing.

  -- Pick up the new Seq value in case it's changed.
  SELECT INTO NEW.seq swerb_bes.seq
    FROM swerb_bes
    WHERE swerb_bes.beid = NEW.beid;

  -- At this point the seq should be unique per group/day.
  -- (So there's no need to test for this error unless you're debugging.)
  PERFORM 1
    FROM swerb_bes
         JOIN swerb_departs_data
              AS other_sdd
              ON (other_sdd.did = swerb_bes.did)
       , swerb_departs_data AS this_sdd
    WHERE this_sdd.did = NEW.did
          AND other_sdd.date = this_sdd.date
          AND swerb_bes.focal_grp = NEW.focal_grp
          AND swerb_bes.seq = NEW.seq
          AND swerb_bes.beid <> NEW.beid
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'Beid %: Did %: Focal_grp %: Seq %: The combination of Seq, Focal_grp and Date must be unique on SWERB_BES'
                  , NEW.beid
                  , NEW.did
                  , NEW.focal_grp
                  , NEW.seq;
    RETURN NULL;
  END IF;

  -- Test that the sequencing is valid by Start.  When Start or
  -- is NULL then use the first SWERB_DATA.Time value.
  -- This test is necessary to make sure that manual resequencing does
  -- not break the rules.

  --
  -- Test for rows sequenced before the current row.
  --

  -- Neither the current row nor the row tested against has a NULL Start.
  PERFORM 1
    FROM swerb_bes
         JOIN swerb_departs_data AS other_sdd
              ON (other_sdd.did = swerb_bes.did)
       , swerb_departs_data AS this_sdd
    WHERE this_sdd.did = NEW.did
          AND other_sdd.date = this_sdd.date
          AND swerb_bes.focal_grp = NEW.focal_grp
          AND swerb_bes.seq < NEW.seq
          AND swerb_bes.start > NEW.start
    LIMIT 1;

  IF FOUND THEN
    RAISE EXCEPTION 'BEId %: Did %: Focal_grp %: Start %: Seq %: Cannot sequence this SWERB_BES row after another SWERB_BES row for the same group/day that has a later Start time'
                  , NEW.beid
                  , NEW.did
                  , NEW.focal_grp
                  , NEW.start
                  , NEW.seq;
    RETURN NULL;
  END IF;

  -- The row tested against has a NULL Start and sequences before
  -- this row.  This is always ok.

  -- The current row has a NULL Start.
  PERFORM 1
    FROM swerb_bes
         JOIN swerb_departs_data AS other_sdd
              ON (other_sdd.did = swerb_bes.did)
       , swerb_departs_data AS this_sdd
    WHERE this_sdd.did = NEW.did
          AND other_sdd.date = this_sdd.date
          AND swerb_bes.focal_grp = NEW.focal_grp
          AND swerb_bes.seq < NEW.seq
          AND swerb_bes.start IS NOT NULL
          AND NEW.start IS NULL
          AND EXISTS (SELECT 1
                        FROM swerb_data AS this_sd
                        WHERE this_sd.beid = NEW.beid
                              AND this_sd.time < swerb_bes.start)
    LIMIT 1;

  IF FOUND THEN
    RAISE EXCEPTION 'BEId %: Did %: Focal_grp %: Start %: Seq %: Cannot sequence this SWERB_BES row (having Start = NULL) after another SWERB_BES row for the same group/day when this SWERB_BES row has a related SWERB_DATA row with a Time value that is before the other SWERB_BES.Start value'
                  , NEW.beid
                  , NEW.did
                  , NEW.focal_grp
                  , NEW.start
                  , NEW.seq;
    RETURN NULL;
  END IF;

  -- If swerb_bes.start IS NULL AND NEW.start IS NULL
  -- then either one can be sequenced first regardless of
  -- SWERB_DATA.Time values.

  --
  -- Look for errors in those rows that sequence after this row.
  --

  -- Neither the current row nor the row tested against has a NULL Start.
  PERFORM 1
    FROM swerb_bes
         JOIN swerb_departs_data AS other_sdd
              ON (other_sdd.did = swerb_bes.did)
       , swerb_departs_data AS this_sdd
    WHERE this_sdd.did = NEW.did
          AND other_sdd.date = this_sdd.date
          AND swerb_bes.focal_grp = NEW.focal_grp
          AND NEW.seq < swerb_bes.seq
          AND NEW.start > swerb_bes.start
    LIMIT 1;

  IF FOUND THEN
    RAISE EXCEPTION 'BEId %: Did %: Focal_grp %: Start %: Seq %: Cannot sequence this SWERB_BES row before another SWERB_BES row for the same group/day that has a earlier Start time'
                  , NEW.beid
                  , NEW.did
                  , NEW.focal_grp
                  , NEW.start
                  , NEW.seq;
    RETURN NULL;
  END IF;

  -- The row we are comparing this row against has a NULL Start.
  PERFORM 1
    FROM swerb_bes
         JOIN swerb_departs_data AS other_sdd
              ON (other_sdd.did = swerb_bes.did)
       , swerb_departs_data AS this_sdd
    WHERE this_sdd.did = NEW.did
          AND other_sdd.date = this_sdd.date
          AND swerb_bes.focal_grp = NEW.focal_grp
          AND NEW.seq < swerb_bes.seq
          AND swerb_bes.start IS NULL
          AND NEW.start IS NOT NULL
          AND EXISTS (SELECT 1
                        FROM swerb_data AS other_sd
                        WHERE other_sd.beid = swerb_bes.beid
                              AND NEW.start < other_sd.time)
    LIMIT 1;

  IF FOUND THEN
    RAISE EXCEPTION 'BEId %: Did %: Focal_grp %: Start %: Seq %: Cannot sequence this SWERB_BES row before another SWERB_BES row for the same group/day with a NULL Start that has a related SWERB_DATA row with a Time value before %'
                  , NEW.beid
                  , NEW.did
                  , NEW.focal_grp
                  , NEW.start
                  , NEW.seq
                  , NEW.start;
    RETURN NULL;
  END IF;

  -- If this row has a NULL Start it can sequence before any other row.

  -- If swerb_bes.start IS NULL AND NEW.start IS NULL
  -- then either one can be sequenced last regardless of
  -- SWERB_DATA.Time values.

  RETURN NULL;
  END;
$$;


SELECT 'swerb_bes_delete_func' AS function;
CREATE FUNCTION swerb_bes_delete_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- Function for swerb_bes delete trigger
  --
  -- GPL_notice(`  --', `2011', `Karl O. Pinc <kop@meme.com>')
  --
  -- Remarks:
  -- WARNING: See remarks in swerb_bes_func().

  BEGIN

    _fill_seq_gap(`SWERB_BES',
                  `Beid',
                  `Did',
                  `Seq',
                  `_seq_per_old_focal_grp',
                  `_gen_from')

  RETURN NULL;
  END;
$$;

SELECT 'swerb_bes_commit_func' AS function;
CREATE OR REPLACE FUNCTION swerb_bes_commit_func()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  -- Function for swerb_bes insert and update fired upon transaction commit.
  --
  -- GPL_notice(`  --', `2012, 2013', `The Meme Factory, Inc.  http://www.meme.com/')
  --
  DECLARE
    this_date swerb_departs_data.date%TYPE;

  BEGIN

  -- Get the latest values of the row
  SELECT * INTO NEW FROM swerb_bes WHERE swerb_bes.beid = NEW.beid;
  IF NOT FOUND THEN
    -- Whatever row was inserted was subsequently deleted.
    -- Nothing to do.
    RETURN NULL;
  END IF;

  -- When Start is NULL either Btimeest must also be NULL
  -- or Bsource must be bb_norecord.
  IF NEW.start IS NULL THEN
    IF NEW.btimeest IS NOT NULL
       AND NEW.bsource <> 'bb_norecord' THEN
      RAISE EXCEPTION integrity_constraint_violation USING
            MESSAGE = 'Error on SWERB_BES ' || TG_OP || ' commit'
          , DETAIL = 'Key (BEId) = (' || NEW.beid
                       || '): Value (Focal_grp) = (' || NEW.focal_grp
                       || '): Value (DId) = (' || NEW.did
                       || '): Value (Start) = ('
                          || textualize(`NEW.start')
                       || '): Value (Btimeest) = ('
                          || textualize(`NEW.btimeest')
                       || '): Value (Bsource) = ('
                          || textualize(`NEW.bsource')
                       || '): Value (SWERB_DEPARTS_DATA.Date) = ('
                          || (SELECT swerb_departs_data.date
                                FROM swerb_departs_data
                                WHERE swerb_departs_data.did = NEW.did)
                       || '): When SWERB_BES.Start time is NULL then SWERB_BES.Btimeest must also be NULL unless SWERB_BES.Bsource = bb_norecord';
      RETURN NULL;
    END IF;
  ELSE -- Start is not NULL
    IF NEW.bsource = 'bb_norecord' THEN
      RAISE EXCEPTION integrity_constraint_violation USING
            MESSAGE = 'Error on SWERB_BES ' || TG_OP || ' commit'
          , DETAIL = 'Key (BEId) = (' || NEW.beid
                       || '): Value (Focal_grp) = (' || NEW.focal_grp
                       || '): Value (DId) = (' || NEW.did
                       || '): Value (Start) = ('
                          || textualize(`NEW.start')
                       || '): Value (Btimeest) = ('
                          || textualize(`NEW.btimeest')
                       || '): Value (Bsource) = (' ||
                          || textualize(`NEW.bsource')
                       || '): Value (SWERB_DEPARTS_DATA.Date) = ('
                          || (SELECT swerb_departs_data.date
                                FROM swerb_departs_data
                                WHERE swerb_departs_data.did = NEW.did)
                       || '): When SWERB_BES.Bsource = bb_norecord then SWERB_BES.Start time must be NULL';
      RETURN NULL;
    END IF;
  END IF;

  -- When Stop is NULL either Etimeest must also be NULL
  -- or Esource must be bb_norecord.
  IF NEW.stop IS NULL THEN
    IF NEW.etimeest IS NOT NULL
       AND NEW.esource <> 'bb_norecord' THEN
      RAISE EXCEPTION integrity_constraint_violation USING
            MESSAGE = 'Error on SWERB_BES ' || TG_OP || ' commit'
          , DETAIL = 'Key (BEId) = (' || NEW.beid
                       || '): Value (Focal_grp) = (' || NEW.focal_grp
                       || '): Value (DId) = (' || NEW.did
                       || '): Value (Stop) = ('
                          || textualize(`NEW.stop')
                       || '): Value (Etimeest) = ('
                          || textualize(`NEW.etimeest')
                       || '): Value (Esource) = (' 
                          || textualize(`NEW.esource')
                       || '): Value (SWERB_DEPARTS_DATA.Date) = ('
                          || (SELECT swerb_departs_data.date
                                FROM swerb_departs_data
                                WHERE swerb_departs_data.did = NEW.did)
                       || '): When SWERB_BES.Stop time is NULL then SWERB_BES.Etimeest must also be NULL unless SWERB_BES.Esource = bb_norecord'
          , HINT = 'Is the group missing a SWERB end line?';
      RETURN NULL;
    END IF;
  ELSE -- Stop is not NULL
    IF NEW.esource = 'bb_norecord' THEN
      RAISE EXCEPTION integrity_constraint_violation USING
            MESSAGE = 'Error on SWERB_BES ' || TG_OP || ' commit'
          , DETAIL = 'Key (BEId) = (' || NEW.beid
                       || '): Value (Focal_grp) = (' || NEW.focal_grp
                       || '): Value (DId) = (' || NEW.did
                       || '): Value (Stop) = ('
                          || textualize(`NEW.stop')
                       || '): Value (Etimeest) = ('
                          || textualize(`NEW.etimeest')
                       || '): Value (Esource) = (' ||
                          || textualize(`NEW.esource')
                       || '): Value (SWERB_DEPARTS_DATA.Date) = ('
                          || (SELECT swerb_departs_data.date
                                FROM swerb_departs_data
                                WHERE swerb_departs_data.did = NEW.did)
                       || '): When SWERB_BES.Esource = bb_norecord then SWERB_BES.Stop time must be NULL';
      RETURN NULL;
    END IF;
  END IF;


  -- There must be exactly 1 SWERB_DATA begin row.
  DECLARE
    c INTEGER;

  BEGIN
    SELECT INTO c COUNT(*)
      FROM swerb_data
      WHERE swerb_data.beid = NEW.beid
            AND swerb_data.event = 'bb_swerb_begin';

    IF c <> 1 THEN
      RAISE EXCEPTION 'Error on SWERB_BES commit: BEId %: DId %: Start %: There must be exactly one begin per bout of observation, exactly one SWERB_DATA row with a SWERB_DATA.BEId = ''%'' and a SWERB_DATA.Event = ''bb_swerb_begin'': % found'
                    , NEW.beid
                    , NEW.did
                    , NEW.start
                    , NEW.beid
                    , c;
      RETURN NULL;
    END IF;
  END;

  -- Start time must be that of the SWERB_DATA begin row.
  PERFORM 1
    FROM swerb_data
    WHERE swerb_data.beid = NEW.beid
          AND swerb_data.event = 'bb_swerb_begin'
          AND (swerb_data.time = NEW.start
               OR (swerb_data.time IS NULL AND NEW.start IS NULL))
    LIMIT 1;
  IF NOT FOUND THEN
    RAISE EXCEPTION 'Error on SWERB_BES commit: BEId %: DId %: Start %: The SWERB_BES.Start time must be equal to the SWERB_DATA.Time value of an existant related SWERB_DATA row having a Event value of bb_swerb_begin'
                    , NEW.beid
                    , NEW.did
                    , NEW.start;
    RETURN NULL;
  END IF;

  -- There must be exactly 1 SWERB_DATA end row.
  DECLARE
    c INTEGER;

  BEGIN
    SELECT INTO c COUNT(*)
      FROM swerb_data
      WHERE swerb_data.beid = NEW.beid
            AND swerb_data.event = 'bb_swerb_end';

    IF c <> 1 THEN
      SELECT swerb_departs_data.date
        INTO this_date
        FROM swerb_departs_data
        WHERE swerb_departs_data.did = NEW.did
        LIMIT 1;

      RAISE EXCEPTION integrity_constraint_violation USING
            MESSAGE = 'Error on commit after ' || TG_OP || ' of SWERB_BES'
          , DETAIL = 'Key (BEId) = (' || NEW.beid
                     || '): Value(DId) = (' || NEW.did
                     || '): Value(Seq) = (' || NEW.seq
                     || '): Value(Stop) = (' || textualize(`NEW.stop')
                     || '): Value(SWERB_DEPARTS_DATA.Date) = ('
                     || textualize(`this_date')
                     || '): There must be exactly one end per bout of observation, exactly one SWERB_DATA row with a SWERB_DATA.BEId = '''
                     || NEW.beid
                     || ''' and a SWERB_DATA.Event = ''bb_swerb_end'': '
                     || c || ' found'
          , HINT = 'bb_transaction_hint';
      RETURN NULL;
    END IF;
  END;

  -- Stop time must be that of the SWERB_DATA end row.
  PERFORM 1
    FROM swerb_data
    WHERE swerb_data.beid = NEW.beid
          AND swerb_data.event = 'bb_swerb_end'
          AND (swerb_data.time = NEW.stop
               OR (swerb_data.time IS NULL AND NEW.stop IS NULL))
    LIMIT 1;
  IF NOT FOUND THEN
    RAISE EXCEPTION 'Error on SWERB_BES commit: BEId %: DId %: Stop %: The SWERB_BES.Stop time must be equal to the SWERB_DATA.Time value of an existant related SWERB_DATA row having a Event value of bb_swerb_end'
                    , NEW.beid
                    , NEW.did
                    , NEW.stop;
    RETURN NULL;
  END IF;

  --
  -- An observation team's first observation of a group
  -- for the day must be related to
  -- a record of descent from a sleeping grove.
  --
  PERFORM 1
    FROM swerb_bes
    WHERE swerb_bes.did = NEW.did
          AND swerb_bes.focal_grp = NEW.focal_grp
          AND swerb_bes.seq < NEW.seq
    LIMIT 1;

  IF NOT FOUND THEN
    -- This is the first observation of the group for the day.

    IF NEW.focal_grp <> 'bb_unk_grp' THEN
      -- The unknown group is exempt from sleeping in groves,
      -- but this is not the unknown group.

      -- The observation bout must be related to
      -- a record of descent from a sleeping grove.
      PERFORM 1
        FROM swerb_data
             JOIN swerb_loc_data ON (swerb_loc_data.swid = swerb_data.swid)
             JOIN adcodes ON (adcodes.adcode = swerb_loc_data.adcode)
        WHERE swerb_data.beid = NEW.beid
              AND adcodes.adn = 'bb_swerb_descent'
        LIMIT 1;
      IF NOT FOUND THEN
        SELECT swerb_departs_data.date
          INTO this_date
          FROM swerb_departs_data
          WHERE swerb_departs_data.did = NEW.did
          LIMIT 1;

        RAISE EXCEPTION integrity_constraint_violation USING
              MESSAGE = 'Error on commit after ' || TG_OP || ' of SWERB_BES'
            , DETAIL = 'Key (BEId) = (' || NEW.beid
                       || '): Value(DId) = (' || NEW.did
                       || '): Value(Seq) = (' || NEW.seq
                       || '): Value(Start) = (' || textualize(`NEW.start')
                       || '): Value(SWERB_DEPARTS_DATA.Date) = ('
                       || textualize(`this_date')
                       || '): Unless the focal group is the unknown group (bb_unk_grp), the first bout of observation of a group by an observation team for each day must have a related SWERB_LOC_DATA row recording descent from a sleeping grove'
            , HINT = 'bb_transaction_hint';
        RETURN NULL;
      END IF;

      -- The observation bout cannot begin later than the time
      -- of descent from the sleeping grove.
      PERFORM 1
        FROM swerb_data
             JOIN swerb_loc_data ON (swerb_loc_data.swid = swerb_data.swid)
        WHERE swerb_data.beid = NEW.beid
              AND swerb_loc_data.adtime < NEW.start
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION integrity_constraint_violation USING
              MESSAGE = 'Error on commit after ' || TG_OP || ' of SWERB_BES'
            , DETAIL = 'Key (BEId) = (' || NEW.beid
                       || '): Value (DId) = (' || NEW.did
                       || '): Value (Start) = ('
                          || textualize(`NEW.start')
                       || '): Value (SWERB_DEPARTS_DATA.Date) = ('
                          || (SELECT swerb_departs_data.date
                                FROM swerb_departs_data
                                WHERE swerb_departs_data.did = NEW.did
                                LIMIT 1)
                       || '): The first bout of observation of a group by an observation team for each day must start (have a SWERB_BES.Start value) at or before the time (SWERB_LOC_DATA.ADtime) of descent from and before the time of ascent into sleeping groves';
        RETURN NULL;
      END IF;
    END IF;
  ELSE
    -- Not the first observation of a team of the group for the day.
    -- Observations that are not a team's first of the day for the group
    -- cannot be involved in descent from a sleeping grove unless,
    -- ignoring those that are subgroup descents from the unknown grove,
    -- all the descents from sleeping groves are of subgroups.
    PERFORM 1
      FROM swerb_data
           JOIN swerb_loc_data ON (swerb_loc_data.swid = swerb_data.swid)
           JOIN adcodes ON (adcodes.adcode = swerb_loc_data.adcode)
      WHERE swerb_data.beid = NEW.beid
            AND NOT(swerb_data.subgroup
                    AND swerb_loc_data.loc = 'bb_swerb_unk_grove')
            AND adcodes.adn = 'bb_swerb_descent'
      LIMIT 1;
    IF FOUND THEN
      PERFORM 1
        FROM swerb_bes
             JOIN swerb_data ON (swerb_data.beid = swerb_bes.beid)
             JOIN swerb_loc_data ON (swerb_loc_data.swid = swerb_data.swid)
             JOIN adcodes ON (adcodes.adcode = swerb_loc_data.adcode)
        WHERE swerb_bes.did = NEW.did
              AND swerb_bes.focal_grp = NEW.focal_grp
              AND NOT(swerb_data.subgroup)
              AND adcodes.adn = 'bb_swerb_descent'
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION 'Error on SWERB_BES commit: BEId %: DId %: Start %: Only the first bout of observation of a group by an observation team for each day can have a related SWERB_LOC_DATA row recording descent from a sleeping grove unless all descents are of subgroups'
                      , NEW.beid
                      , NEW.did
                      , NEW.start;
        RETURN NULL;
      END IF;
    END IF;
  END IF;


  --
  -- An observation team's last bout of observation of a group
  -- for the day must be related to
  -- a record of ascent into a sleeping grove.
  -- The last bout of observation is one of: the ones with the
  -- largest Stop time, or having a NULL Stop time.
  --

  --
  -- Is this the last observation of the group by the observation
  -- team for the day?
  --
  PERFORM 1
    FROM swerb_bes
    WHERE swerb_bes.did = NEW.did
          AND swerb_bes.focal_grp = NEW.focal_grp
          AND swerb_bes.stop > NEW.stop
    LIMIT 1;
  IF NOT FOUND THEN
    -- This is one of the last observations (by Stop or NULL-ness).
    DECLARE
      this_is_ascent BOOLEAN;
      other_ascent_exists BOOLEAN;

      this_subgroup swerb_data.subgroup%TYPE;
      this_loc swerb_loc_data.loc%TYPE;

    BEGIN
      -- Does this bout of observation have an ascent?
      SELECT swerb_data.subgroup, swerb_loc_data.loc
        INTO this_subgroup,       this_loc
        FROM swerb_data
             JOIN swerb_loc_data ON (swerb_loc_data.swid = swerb_data.swid)
             JOIN adcodes ON (adcodes.adcode = swerb_loc_data.adcode)
        WHERE swerb_data.beid = NEW.beid
              AND adcodes.adn = 'bb_swerb_ascent'
        LIMIT 1;
      this_is_ascent := FOUND;

      -- Are there any other observations that also have
      -- ascents?
      PERFORM 1
        FROM swerb_bes
             JOIN swerb_data ON (swerb_data.beid = swerb_bes.beid)
             JOIN swerb_loc_data ON (swerb_loc_data.swid = swerb_data.swid)
             JOIN adcodes ON (adcodes.adcode = swerb_loc_data.adcode)
        WHERE swerb_bes.did = NEW.did
              AND swerb_bes.focal_grp = NEW.focal_grp
              AND swerb_bes.beid <> NEW.beid
              AND adcodes.adn = 'bb_swerb_ascent'
        LIMIT 1;
      other_ascent_exists := FOUND;

      IF this_is_ascent THEN
        IF other_ascent_exists
           AND NOT(this_subgroup AND this_loc = 'bb_swerb_unk_grove') THEN
          -- Can't have more than one bout of observation by a single
          -- team for a single group on a single day record sleeping grove
          -- ascent, ignoring those bouts that are ascents into the
          -- unknown grove, unless all of the ascents are of subgroups.
          PERFORM 1
            FROM swerb_bes
                 JOIN swerb_data ON (swerb_data.beid = swerb_bes.beid)
                 JOIN swerb_loc_data ON (swerb_loc_data.swid = swerb_data.swid)
                 JOIN adcodes ON (adcodes.adcode = swerb_loc_data.adcode)
            WHERE swerb_bes.did = NEW.did
                  AND swerb_bes.focal_grp = NEW.focal_grp
                  AND NOT(swerb_data.subgroup)
                  AND adcodes.adn = 'bb_swerb_ascent'
            LIMIT 1;
          IF FOUND THEN
            RAISE EXCEPTION integrity_constraint_violation USING
                  MESSAGE = 'Error on commit after ' || TG_OP
                            || ' of SWERB_BES'
                , DETAIL = 'Key (BEId) = (' || NEW.beid
                           || '): Value (DId) = (' || NEW.did
                           || '): Value (Stop) = ('
                              || textualize(`NEW.stop')
                           || '): Value (SWERB_DEPARTS_DATA.Date) = ('
                              || (SELECT swerb_departs_data.date
                                    FROM swerb_departs_data
                                    WHERE swerb_departs_data.did = NEW.did
                                    LIMIT 1)
                           || '): Only one bout of observation of a group that meets the qualifications for being the last of the observation team''s for the day can have a related SWERB_LOC_DATA row recording ascent into a sleeping grove unless all of the ascents are of subgroups';
            RETURN NULL;
          END IF;
        END IF;

        -- The observation bout cannot begin later than the time
        -- of ascent into the sleeping grove.
        PERFORM 1
          FROM swerb_data
               JOIN swerb_loc_data ON (swerb_loc_data.swid = swerb_data.swid)
          WHERE swerb_data.beid = NEW.beid
                AND swerb_loc_data.adtime < NEW.start
          LIMIT 1;
        IF FOUND THEN
          RAISE EXCEPTION integrity_constraint_violation USING
                MESSAGE = 'Error on commit after ' || TG_OP
                          || ' of SWERB_BES'
              , DETAIL = 'Key (BEId) = (' || NEW.beid
                         || '): Value (DId) = (' || NEW.did
                         || '): Value (Start) = ('
                            || textualize(`NEW.start')
                         || '): Value (SWERB_DEPARTS_DATA.Date) = ('
                            || (SELECT swerb_departs_data.date
                                  FROM swerb_departs_data
                                  WHERE swerb_departs_data.did = NEW.did
                                  LIMIT 1)
                         || '): The last bout of observation of a group by an observation team for each day must start (have a SWERB_BES.Start value) before the time (SWERB_LOC_DATA.ADtime) before the time of ascent into sleeping groves, and before the time of descent from the sleeping grove if descent is related to the bout of observation';
          RETURN NULL;
        END IF;
      ELSE
        -- This row does not have a related ascent.
        IF NOT other_ascent_exists
           AND NEW.focal_grp <> 'bb_unk_grp' THEN
          -- When the focal is other than the unknown group
          -- there must be at least one ascent for the team/group/day.
          SELECT swerb_departs_data.date
            INTO this_date
            FROM swerb_departs_data
            WHERE swerb_departs_data.did = NEW.did
            LIMIT 1;

          RAISE EXCEPTION integrity_constraint_violation USING
                MESSAGE = 'Error on commit after ' || TG_OP || ' of SWERB_BES'
              , DETAIL = 'Key (BEId) = (' || NEW.beid
                         || '): Value(DId) = (' || NEW.did
                         || '): Value(Seq) = (' || NEW.seq
                         || '): Value(Stop) = (' || textualize(`NEW.stop')
                         || '): Value(SWERB_DEPARTS_DATA.Date) = ('
                         || textualize(`this_date')
                         || '): Unless the focal group is the unknown group (bb_unk_grp), one bout of observation of a group that meets the qualifications for being the last of the observation team''s for the day must have a related SWERB_LOC_DATA row recording ascent into a sleeping grove'
              , HINT = 'bb_transaction_hint';
          RETURN NULL;
        END IF;
      END IF;
    END;
  ELSE
    -- This is not the last observation of the group for the day.

    -- Observations that are not the last of the day for the
    -- observation team for the group
    -- cannot be involved in ascent into a sleeping grove,
    -- ignoring those that are both subgroups and ascents into the
    -- unknown grove, unless all ascents are of subgroups.
    PERFORM 1
      FROM swerb_data
           JOIN swerb_loc_data ON (swerb_loc_data.swid = swerb_data.swid)
           JOIN adcodes ON (adcodes.adcode = swerb_loc_data.adcode)
      WHERE swerb_data.beid = NEW.beid
            AND NOT(swerb_data.subgroup
                    AND swerb_loc_data.loc = 'bb_swerb_unk_grove')
            AND adcodes.adn = 'bb_swerb_ascent'
      LIMIT 1;
    IF FOUND THEN
      PERFORM 1
        FROM swerb_bes
             JOIN swerb_data ON (swerb_data.beid = swerb_bes.beid)
             JOIN swerb_loc_data ON (swerb_loc_data.swid = swerb_data.swid)
             JOIN adcodes ON (adcodes.adcode = swerb_loc_data.adcode)
        WHERE swerb_bes.did = NEW.did
              AND swerb_bes.focal_grp = NEW.focal_grp
              AND NOT(swerb_data.subgroup)
              AND adcodes.adn = 'bb_swerb_ascent'
        LIMIT 1;
      IF FOUND THEN
        SELECT swerb_departs_data.date
          INTO this_date
          FROM swerb_departs_data
          WHERE swerb_departs_data.did = NEW.did
          LIMIT 1;

        RAISE EXCEPTION integrity_constraint_violation USING
              MESSAGE = 'Error on commit after ' || TG_OP || ' of SWERB_BES'
            , DETAIL = 'Key (BEId) = (' || NEW.beid
                       || '): Value(DId) = (' || NEW.did
                       || '): Value(Seq) = (' || NEW.seq
                       || '): Value(Stop) = (' || textualize(`NEW.stop')
                       || '): Value(SWERB_DEPARTS_DATA.Date) = ('
                       || textualize(`this_date')
                       || '): Only the bout of observation of a group that is the last of the observation team''s for the day can have a related SWERB_LOC_DATA row recording ascent into a sleeping grove unless all ascents are of subgroups';
        RETURN NULL;
      END IF;
    END IF;
  END IF;

  RETURN NULL;
  END;
$$;


SELECT 'swerb_bes_delete_commit_func' AS function;
CREATE OR REPLACE FUNCTION swerb_bes_delete_commit_func()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  As $$
  -- Function for swerb_bes delete fired upon transaction commit.
  --
  -- GPL_notice(`  --', `2012', `The Meme Factory, Inc.  http://www.meme.com/')
  --
  DECLARE
    begin_beid swerb_bes.beid%TYPE;
    end_time swerb_bes.stop%TYPE;

  BEGIN
  -- Every day of observeration by a observation team
  -- of a group must begin and end
  -- with descent from/ascent into a sleeping grove.

  -- Find the first begin of the team of the group for the day.
  SELECT swerb_bes.beid
    INTO begin_beid
    FROM swerb_bes
    WHERE swerb_bes.did = OLD.did
          AND swerb_bes.focal_grp = OLD.focal_grp
          AND OLD.focal_grp <> 'bb_unk_grp'
    ORDER BY swerb_bes.seq
    LIMIT 1;

  IF FOUND THEN
    -- There are still observation by the team of the group on the day.
    -- (And the focal is not the unknown group, which is
    -- exempt from sleeping in groves.)
    -- We can check that begins and ends still decend
    -- from ascend into sleeping groves.

    -- Check that the group decends from a grove on the
    -- observation date.  The first observation of the
    -- group by the team for the day must be a descent.
    -- (If the transaction only deleted rows then we don't      
    -- need to check for just the first descent being
    -- allowed to be the entire group.  If more than delete
    -- is happending then the other updates will have done the
    -- subgroup related checks for us so we don't need to do
    -- that here either.)

    PERFORM 1
      FROM swerb_data
           JOIN swerb_loc_data ON (swerb_loc_data.swid = swerb_data.swid)
           JOIN adcodes ON (adcodes.adcode = swerb_loc_data.adcode)
      WHERE swerb_data.beid = first_beid
            AND adcoces.adn = 'bb_swerb_descent'
     LIMIT 1;
    IF NOT FOUND THEN
      RAISE EXCEPTION 'Error on SWERB_BES delete commit:: DId %: Focal_grp %: Too many SWERB_BES deleted: Every day of each observation team''s observation of a group must begin with descent from a sleeping grove'
                      , OLD.did
                      , OLD.focal_grp;
      RETURN NULL;
    END IF;

    -- Check that the group ascends into a grove on the
    -- observation date.  (One of) the last observation(s)
    -- of the day must be ascent into a sleeping grove.
    -- (See also remarks above regarding descent from sleeping
    -- groves and subgroups.  The same applies here.)

    -- Find when observation ends for the group/team/day.
    SELECT swerb_bes.stop
      INTO end_time
      FROM swerb_bes
      WHERE swerb_bes.did = OLD.did
            AND swerb_bes.focal_grp = OLD.focal_grp
            AND swerb_bes.stop IS NOT NULL
      ORDER BY swerb_bes.stop DESC
      LIMIT 1;

    -- Make sure there's an ascent into a sleeping grove
    -- with either the end time or a NULL end time.
    PERFORM 1
      FROM swerb_bes
           JOIN swerb_data ON (swerb_data.beid = swerb_bes.beid)
           JOIN swerb_loc_data ON (swerb_loc_data.swid = swerb_data.swid)
           JOIN adcodes ON (adcodes.adcode = swerb_loc_data.adcode)
      WHERE swerb_bes.did = OLD.did
            AND swerb_bes.focal_grp = OLD.focal_grp
            AND (swerb_bes.stop = end_time
                 OR swerb_bes.stop IS NULL)
            AND adcoces.adn = 'bb_swerb_ascent'
      LIMIT 1;
    IF NOT FOUND THEN
      RAISE EXCEPTION 'Error on SWERB_BES delete commit:: DId %: Focal_grp %: Too many SWERB_BES deleted: Every day of each observation team''s observation of a group must end with ascent into a sleeping grove'
                      , OLD.did
                      , OLD.focal_grp;
      RETURN NULL;
    END IF;
  END IF;

  RETURN NULL;
  END;
$$;

SELECT 'swerb_bes_before_insert_trigger' AS trigger;
CREATE TRIGGER swerb_bes_before_insert_trigger
  BEFORE INSERT
  ON swerb_bes FOR EACH ROW
  EXECUTE PROCEDURE swerb_bes_before_insert_func();


SELECT 'swerb_bes_trigger' AS trigger;
CREATE TRIGGER swerb_bes_trigger
  AFTER INSERT OR UPDATE
  ON swerb_bes FOR EACH ROW
  EXECUTE PROCEDURE swerb_bes_func();

SELECT 'swerb_bes_delete_trigger' AS trigger;
CREATE TRIGGER swerb_bes_delete_trigger
  AFTER DELETE
  ON swerb_bes FOR EACH ROW
  EXECUTE PROCEDURE swerb_bes_delete_func();

SELECT 'swerb_bes_commit_trigger' AS trigger;
CREATE CONSTRAINT TRIGGER swerb_bes_commit_trigger
  AFTER INSERT OR UPDATE
  ON swerb_bes
  DEFERRABLE INITIALLY DEFERRED
  FOR EACH ROW
  EXECUTE PROCEDURE swerb_bes_commit_func();

SELECT 'swerb_bes_delete_commit_trigger' AS trigger;
CREATE CONSTRAINT TRIGGER swerb_bes_delete_commit_trigger
  AFTER DELETE
  ON swerb_bes
  DEFERRABLE INITIALLY DEFERRED
  FOR EACH ROW
  EXECUTE PROCEDURE swerb_bes_delete_commit_func();
