dnl Copyright (C) 2011, 2012, 2014, 2015, 2016 The Meme Factory, Inc.,
dnl   http://www.meme.com/
dnl Copyright (C) 2015 Jacob Gordon, <jacob.gordon@duke.edu>
dnl Copyright (C) 2011, Karl O. Pinc
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl

--  
-- swerb_data
--
SELECT 'swerb_data' AS table;

SELECT 'swerb_data_func' AS function;
CREATE FUNCTION swerb_data_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  BEGIN
  -- Function for swerb_data insert and update triggers
  --
  -- Copyright (C) 2011, 2012, 2016 The Meme Factory, Inc.,
  -- http://www.meme.com/
  -- Copyright (C) 2015 Jacob Gordon, <jacob.gordon@duke.edu>
  -- GPL_notice(`  --', `2011', `Karl O. Pinc <kop@meme.com>')

  /* debug
  if TG_OP = 'INSERT' then
  raise warning 'TG_OP: %: SWId %: BEId %: Seen_grp %: Event %: Time %'
              , TG_OP
              , NEW.swid
              , NEW.beid
              , NEW.seen_grp
              , NEW.event
              , NEW.time;
 else
  raise warning 'TG_OP: %: SWId %: BEId %: Seen_grp %:old Event %: Event %: old Time %: Time %'
              , TG_OP
              , NEW.swid
              , NEW.beid
              , NEW.seen_grp
              , OLD.event
              , NEW.event
              , OLD.time
              , NEW.time;
 end if;
*/

  IF TG_OP = 'UPDATE' THEN
    -- Cannot change the id.
    cannot_change(`SWERB_DATA', `SWId')
    -- Cannot change the beid.
    cannot_change(`SWERB_DATA', `BEId')
  END IF;

  -- The group must be the focal group, unless the Event is bb_swerb_ogrp
  -- and it's an observation of a subgroup.
  IF NEW.Event = 'bb_swerb_ogrp' THEN
    -- Other group ad-hoc observation.
    IF NOT(NEW.subgroup) THEN
      -- When the group is together then an ad-hoc observation of
      -- another group must be of another group, unless the "other"
      -- group is the unknown group.  But when the group has split
      -- up we can observe another fragment of the focal group. (In
      -- the latter case we get a warning.)
      PERFORM 1
        FROM swerb_bes
        WHERE swerb_bes.beid = NEW.beid
              AND swerb_bes.focal_grp = NEW.seen_grp
              AND NEW.seen_grp <> bb_unk_grp
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION integrity_constraint_violation USING
              MESSAGE = 'Error on ' || TG_OP || ' of SWERB_DATA'
            , DETAIL = 'Key (SWId) = (' || NEW.swid
                         || '): Value (Event) = (' || NEW.event
                         || '): Value (Seen_grp) = (' || NEW.seen_grp
                         || '): Value (Subgroup) = (' || NEW.subgroup
                         || '): Value (Time) = (' || NEW.time
                         || ': The observed group must not be the'
                         || ' related SWERB_BES.Focal_grp when the'
                         || ' EVENT = (bb_swerb_ogrp) and'
                         || ' SWERB_DATA.Subgroup = FALSE unless the'
                         || ' SWERB_DATA.Seen_grp = (bb_unk_grp)';
        RETURN NULL;
      END IF;
    END IF;
  ELSE
    -- Observation of the focal group.

    -- Not an ad-hoc observation of a non-focal group.
    -- Note that in the process of group fission we don't consider "regular"
    -- subgroups to be "swerb subgroups".  A "swerb subgroup"
    -- (NEW.Subgroup = t) _must_ be a portion of the focal group no matter
    -- whether the focal group is a subgrouping which occurs during group
    -- fission or not.
    PERFORM 1
      FROM swerb_bes
      WHERE swerb_bes.beid = NEW.beid
            AND swerb_bes.focal_grp <> NEW.seen_grp
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION 'SWId %: Event %: Seen_grp %: Subgroup %: Time %: The observed group must be the related SWERB_BES.Focal_grp unless the Event = bb_swerb_ogrp'
                    , NEW.swid
                    , NEW.event
                    , NEW.seen_grp
                    , NEW.subgroup
                    , NEW.time
        USING HINT = 'Is the group missing a SWERB "begin" line?';
      RETURN NULL;
    END IF;
  END IF;

  IF NEW.lone_animal IS NOT NULL THEN

    -- Lone animal sightings cannot be before the individual's entrydate.
    PERFORM 1
      FROM swerb_bes
           JOIN swerb_departs_data ON (swerb_departs_data.did = swerb_bes.did)
         , biograph
      WHERE swerb_bes.beid = NEW.beid
            AND biograph.sname = NEW.lone_animal
            AND biograph.entrydate > swerb_departs_data.date
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION integrity_constraint_violation USING
            MESSAGE = 'Error on ' || TG_OP || ' of SWERB_DATA'
          , DETAIL = 'Key (SWId) = (' || NEW.swid
                       || '): Value(Lone_Animal) = (' || NEW.lone_animal
                       || '): Value(Time) = (' || NEW.time
                       || '): Lone animal cannot be sighted before their '
                       || 'BIOGRAPH.Entrydate';
      RETURN NULL;
    END IF;

    -- Lone animal sightings cannot be after the individual's statdate.
    PERFORM 1
      FROM swerb_bes
           JOIN swerb_departs_data ON (swerb_departs_data.did = swerb_bes.did)
         , biograph
      WHERE swerb_bes.beid = NEW.beid
            AND biograph.sname = NEW.lone_animal
            AND biograph.statdate < swerb_departs_data.date
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION 'SWId %: Lone_Animal %: BEId %: Event %: Seen_grp %: Subgrp %: Time %: Lone animal cannot be sighted after their BIOGRAPH.Statdate'
                    , NEW.swid
                    , NEW.lone_animal
                    , NEW.beid
                    , NEW.event
                    , NEW.seen_grp
                    , NEW.subgroup
                    , NEW.time;
      RETURN NULL;
    END IF;
  END IF;

  -- Can have at most one begin per bout of observation.
  IF NEW.event = 'bb_swerb_begin' THEN
    PERFORM 1
      FROM swerb_data
      WHERE swerb_data.beid = NEW.beid
            AND swerb_data.event = 'bb_swerb_begin'
            AND swerb_data.swid <> NEW.swid
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION integrity_constraint_violation USING
            MESSAGE = 'Error on ' || TG_OP || ' of SWERB_DATA'
          , DETAIL = 'Key (SWId) = (' || NEW.swid
                     || '): Value (BEId) = (' || NEW.beid
                     || '): Value (Event) = (' || NEW.event
                     || '): Value (Seen_grp) = (' || NEW.seen_grp
                     || '): Value (Subgroup) = (' || NEW.subgroup
                     || '): Value (Time) = (' || NEW.time
                     || '): Value (SWERB_DEPARTS_DATA.Date) = ('
                     || (SELECT swerb_departs_data.date
                           FROM swerb_bes
                                  JOIN swerb_departs_data
                                       ON (swerb_departs_data.did
                                           = swerb_bes.did)
                           WHERE swerb_bes.beid = NEW.beid
                           LIMIT 1)
                     || '): Cannot have more than one SWERB_DATA row with an Event of bb_swerb_begin per BEId value (per observation bout)';
      RETURN NULL;
    END IF;
  END IF;

  -- Can have at most one end per bout of observation.
  IF NEW.event = 'bb_swerb_end' THEN
    PERFORM 1
      FROM swerb_data
      WHERE swerb_data.beid = NEW.beid
            AND swerb_data.event = 'bb_swerb_end'
            AND swerb_data.swid <> NEW.swid
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION integrity_constraint_violation USING
            MESSAGE = 'Error on ' || TG_OP || ' of SWERB_DATA'
          , DETAIL = 'Key (SWId) = (' || NEW.swid
                     || '): Value (BEId) = (' || NEW.beid
                     || '): Value (Event) = (' || NEW.event
                     || '): Value (Seen_grp) = (' || NEW.seen_grp
                     || '): Value (Subgroup) = (' || NEW.subgroup
                     || '): Value (Time) = (' || NEW.time
                     || '): Value (SWERB_DEPARTS_DATA.Date) = ('
                     || (SELECT swerb_departs_data.date
                           FROM swerb_bes
                                  JOIN swerb_departs_data
                                       ON (swerb_departs_data.did
                                           = swerb_bes.did)
                           WHERE swerb_bes.beid = NEW.beid
                           LIMIT 1)
                     || '): Cannot have more than one SWERB_DATA row with an Event of bb_swerb_end per BEId value (per observation bout)';
      RETURN NULL;
    END IF;
  END IF;

  -- Can't have a time be before the start of the bout of observation or
  -- after the end of the bout of observation.

  -- Start

  IF NEW.Event <> 'bb_swerb_begin' THEN
    -- Check against SWERB_BES.Start.
    PERFORM 1
      FROM swerb_bes
      WHERE swerb_bes.beid = NEW.beid
            AND swerb_bes.start > NEW.time
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION 'SWId %: Event %: Seen_grp %: Time %: Cannot have a SWERB.Time before the related SWERB_BES.Start time'
                    , NEW.swid
                    , NEW.event
                    , NEW.seen_grp
                    , NEW.time;
      RETURN NULL;
    END IF;
  END IF;

  -- Because SERB_BES.Start can be NULL check against the departure time.
  PERFORM 1
    FROM swerb_bes
         JOIN swerb_departs_data ON (swerb_departs_data.did = swerb_bes.did)
    WHERE swerb_bes.did = NEW.beid
          AND NEW.time < swerb_departs_data.time
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'SWId %: Event %: Seen_grp %: Time %: Cannot have a SWERB.Time before the related SWERB_DEPARTS_DATA.Time time'
                  , NEW.swid
                  , NEW.event
                  , NEW.seen_grp
                  , NEW.time;
    RETURN NULL;
  END IF;

  -- Stop

  IF NEW.Event <> 'bb_swerb_end' THEN
    -- Check against SWERB_BES.Stop.
    PERFORM 1
      FROM swerb_bes
      WHERE swerb_bes.beid = NEW.beid
            AND swerb_bes.stop < NEW.time
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION 'SWId %: Event %: Seen_grp %: Time %: Cannot have a SWERB.Time after the related SWERB_BES.Stop time'
                    , NEW.swid
                    , NEW.event
                    , NEW.seen_grp
                    , NEW.time;
      RETURN NULL;
    END IF;
  END IF;

  -- Can't have GPS data before bb_gps_start.
  IF NEW.xyloc IS NOT NULL THEN
    PERFORM 1
      FROM swerb_bes
           JOIN swerb_departs_data ON (swerb_departs_data.did = swerb_bes.did)
      WHERE swerb_bes.beid = NEW.beid
            AND swerb_departs_data.date < 'bb_gps_start'
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION 'SWId %: BEId %: Seen_grp %: Event %: Time %: X %: Y %: Must a have a NULL XYLoc before bb_gps_start'
                    , NEW.swid
                    , NEW.beid
                    , NEW.seen_grp
                    , NEW.event
                    , NEW.time
                    , ST_X(NEW.xyloc)
                    , ST_Y(NEW.xyloc);
      RETURN NULL;
    END IF;
  END IF;

  -- Can't have quad data after bb_quad_end.
  IF NEW.quad IS NOT NULL THEN
    PERFORM 1
      FROM swerb_bes
           JOIN swerb_departs_data ON (swerb_departs_data.did = swerb_bes.did)
      WHERE swerb_bes.beid = NEW.beid
            AND swerb_departs_data.date > 'bb_quad_end'
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION 'SWId %: BEId %: Seen_grp %: Event %: Time %: Quad %: Must have a NULL Quad after bb_quad_end'
                    , NEW.swid
                    , NEW.beid
                    , NEW.seen_grp
                    , NEW.event
                    , NEW.time
                    , NEW.quad;
      RETURN NULL;
    END IF;
  END IF;

  -- Altitude cannot have digits to the right of the decimal
  -- before bb_subm_alt_date.
  IF TRUNC(NEW.altitude) <> NEW.altitude THEN
    PERFORM 1
      FROM swerb_bes
           JOIN swerb_departs_data ON (swerb_departs_data.did = swerb_bes.did)
      WHERE swerb_bes.beid = NEW.beid
            AND swerb_departs_data.date < 'bb_subm_alt_date'
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION 'SWId %: Altitude %: Altitude cannot have digits to the right of the decimal before bb_subm_alt_date'
                    , NEW.swid
                    , NEW.altitude;
      RETURN NULL;
    END IF;
  END IF;

  IF NEW.accuracy IS NOT NULL THEN
    -- Can't have Accuracy readings when the GPS unit reports it's error
    -- as a PDOP.
    PERFORM 1
      FROM swerb_bes
           JOIN swerb_departs_gps ON (swerb_departs_gps.did = swerb_bes.did)
           JOIN gps_units ON (gps_units.gps = swerb_departs_gps.gps)
      WHERE swerb_bes.beid = NEW.beid
            AND gps_units.errortype = 'bb_gpsu_pdop'
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION 'SWId %: Accuracy %: Cannot have a (non-NULL) Accuracy value when the GPS unit does not report error as an accuracy'
                    , NEW.swid
                    , NEW.accuracy;
      RETURN NULL;
    END IF;

    -- Can't have accuracy reading before bb_earliest_accuracy.
    PERFORM 1
      FROM swerb_bes
           JOIN swerb_departs_data ON (swerb_departs_data.did = swerb_bes.did)
      WHERE swerb_bes.beid = NEW.beid
            AND swerb_departs_data.date < 'bb_earliest_accuracy'
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION 'SWId %: Accuracy %: Cannot have a (non-NULL) Accuracy value before bb_earliest_accuracy'
                    , NEW.swid
                    , NEW.accuracy;
      RETURN NULL;
    END IF;
  END IF;

  IF NEW.pdop IS NOT NULL THEN
    -- Can't have PDOP readings when the GPS unit reports it's error
    -- as an accuracy.
    PERFORM 1
      FROM swerb_bes
           JOIN swerb_departs_gps ON (swerb_departs_gps.did = swerb_bes.did)
           JOIN gps_units ON (gps_units.gps = swerb_departs_gps.gps)
      WHERE swerb_bes.beid = NEW.beid
            AND gps_units.errortype = 'bb_gpsu_accuracy'
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION 'SWId %: PDOP %: Cannot have a (non-NULL) PDOP value when the GPS unit does not report error as a PDOP'
                   , NEW.swid
                   , NEW.pdop;
      RETURN NULL;
    END IF;

    -- Can't have PDOP readings before bb_earliest_pdop or after
    -- bb_latest_pdop.
    PERFORM 1
      FROM swerb_bes
           JOIN swerb_departs_data ON (swerb_departs_data.did = swerb_bes.did)
      WHERE swerb_bes.beid = NEW.beid
            AND ('bb_earliest_pdop' > swerb_departs_data.date
                 OR swerb_departs_data.date > 'bb_latest_pdop')
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION 'SWId %: PDOP %: Cannot have a (non-NULL) PDOP value before bb_earliest_pdop or after bb_latest_pdop'
                   , NEW.swid
                   , NEW.pdop;
      RETURN NULL;
    END IF;
  END IF;

  -- gps_datetime
  IF NEW.gps_datetime IS NOT NULL THEN
    -- Must have a NULL gps_datetime before &bb_subm_alt_date.
    IF NEW.gps_datetime < 'bb_subm_alt_date' THEN
      RAISE EXCEPTION 'SWId %: GPS_Datetime %: GPS_Datetime must be NULL before bb_subm_alt_date'
                     , NEW.swid
                     , NEW.gps_datetime;
      RETURN NULL;
    END IF;

    -- Can't have a GPS_Datetime date that's not the same as the departure date.
    PERFORM 1
      FROM swerb_bes
           JOIN swerb_departs_data ON (swerb_departs_data.did = swerb_bes.did)
      WHERE swerb_bes.beid = NEW.beid
            AND swerb_departs_data.date <> NEW.gps_datetime::DATE
      LIMIT 1;
    IF FOUND THEN
        RAISE EXCEPTION 'SWId %: GPS_Datetime %: The date portion of the SWERB_DATA.GPS_Datetime value must be equal to the related SWERB_DEPARTS_DATA.Date value'
                      , NEW.swid
                      , NEW.gps_datetime;
      RETURN NULL;
    END IF;
  END IF;

  IF NEW.garmincode IS NOT NULL THEN
    -- Can't have a non-NULL Garmincode before bb_subm_alt_date.
    PERFORM 1
      FROM swerb_bes
           JOIN swerb_departs ON (swerb_departs.did = swerb_bes.did)
      WHERE swerb_bes.beid = NEW.beid
            AND swerb_departs.date < 'bb_subm_alt_date'
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION 'SWId %: Garmincode %: Cannot have a non-NULL Garmincode before bb_subm_alt_date'
                    , NEW.swid
                    , NEW.garmincode;
      RETURN NULL;
    END IF;
  END IF;

  -- Can't be seen before the group exists.
  PERFORM 1
    FROM swerb_bes
         JOIN swerb_departs ON (swerb_departs.did = swerb_bes.did)
       , groups
    WHERE swerb_bes.beid = NEW.beid
          AND groups.gid = NEW.seen_grp
          AND swerb_departs.date < groups.start
      LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'SWId %: Seen_grp %: Cannot observe a group before it exists'
                  , NEW.swid
                  , NEW.seen_grp;
    RETURN NULL;
  END IF;

  -- Can't be seen after the group ceases to exist.
  PERFORM 1
    FROM swerb_bes
         JOIN swerb_departs ON (swerb_departs.did = swerb_bes.did)
       , groups
    WHERE swerb_bes.beid = NEW.beid
          AND groups.gid = NEW.seen_grp
          AND swerb_departs.date > groups.cease_to_exist
      LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'SWId %: Seen_grp %: Cannot observe a group after it ceases to exist'
                  , NEW.swid
                  , NEW.seen_grp;
    RETURN NULL;
  END IF;

  --
  -- Everything checks out, but the sequencing.
  --

  -- Update the SWERB_BES Start or Stop columns if this row is a start or stop
  /*
  DECLARE
    copytime BOOLEAN := FALSE;

  BEGIN
    IF NEW.event = 'bb_swerb_begin'
       OR NEW.event =  'bb_swerb_end' THEN
      IF TG_OP = 'INSERT'
        copytime := TRUE;
      ELSE -- operation is an UPDATE
        IF OLD.time <> NEW.time 
           OR OLD.event <> NEW.event THEN
          copytime := TRUE
        END IF;
      END IF;
    END IF;

    IF copytime THEN
      IF NEW.event = 'bb_swerb_begin' THEN
        UPDATE swerb_bes
          SET start = NEW.time
          WHERE swerb_bes.beid = NEW.beid;
      ELSEIF NEW.event =  'bb_swerb_end' THEN
        UPDATE swerb_bes
          SET stop = NEW.time
          WHERE swerb_bes.beid = NEW.beid;
      END IF;
      IF TG_OP = 'UPDATE' THEN
        IF NEW.event <> OLD.event THEN
          IF OLD.event = 'bb_swerb_begin' THEN
            UPDATE swerb_bes
    END IF;
  END;
  */

  IF TG_OP = 'INSERT' THEN
    IF NEW.event = 'bb_swerb_begin' THEN
      UPDATE swerb_bes
        SET start = NEW.time
        WHERE swerb_bes.beid = NEW.beid;
    ELSEIF NEW.event =  'bb_swerb_end' THEN
      UPDATE swerb_bes
        SET stop = NEW.time
        WHERE swerb_bes.beid = NEW.beid;
    END IF;
  ELSE -- TG_OP = 'UPDATE'
    IF NEW.event = 'bb_swerb_begin' THEN
      IF OLD.event = 'bb_swerb_end' THEN
        UPDATE swerb_bes
          SET start = NEW.time
            , stop = NULL
          WHERE swerb_bes.beid = NEW.beid;
      ELSEIF NEW.time <> OLD.time THEN
        UPDATE swerb_bes
          SET start = NEW.time
          WHERE swerb_bes.beid = NEW.beid;
      END IF;
    ELSEIF NEW.event = 'bb_swerb_end' THEN
      IF OLD.event = 'bb_swerb_begin' THEN
        UPDATE swerb_bes
          SET stop = NEW.time
            , start = NULL
          WHERE swerb_bes.beid = NEW.beid;
      ELSEIF NEW.time <> OLD.time THEN
        UPDATE swerb_bes
          SET stop = NEW.time
          WHERE swerb_bes.beid = NEW.beid;
      END IF;
    ELSEIF OLD.event = 'bb_swerb_begin' THEN
      -- The new event is not bb_swerb_begin.
      UPDATE swerb_bes
        SET start = NULL
        WHERE swerb_bes.beid = NEW.beid;
    ELSEIF OLD.event = 'bb_swerb_end' THEN
      -- The new event is not bb_swerb_end.
      UPDATE swerb_bes
        SET stop = NULL
        WHERE swerb_bes.beid = NEW.beid;
    END IF;
  END IF;

/*
  -- The SWERB_DATA row checks out, but if the related SWERB_BEs row
  -- has a NULL Start value then it depends on the SWERB_DATA row for
  -- automatic setting of the Seq value (only when newly inserted), in which case the trigger
  -- must be invoked to do the automatic sequencing.
  DECLARE
    resequence BOOLEAN := FALSE;
    this_seq swerb_bes.seq%TYPE;

  BEGIN;
    IF TG_OP = 'INSERT'
      resequence := TRUE;
    ELSE
      IF OLD.time <> NEW.time THEN
        resequence := TRUE
      END IF;
    END IF;

    IF resequence THEN
      SELECT swerb_bes.seq INTO this_seq
        FROM swerb_bes
        WHERE swerb_bes.beid = NEW.beid
              AND swerb_bes.start IS NULL
        LIMIT 1;

      IF this_seq IS NOT NULL THEN
        UPDATE swerb_bes
          SET seq = this_seq
          WHERE swerb_bes.beid = NEW.beid;
      END IF;
    END IF
  END;
*/

  -- Because SWERB_BES.Start can be NULL we must test for validity
  -- of the SWERB_BES.Seq as when Start is NULL ordering is determined
  -- by SWERB_DATA.Time compared against the non-NULL SWERB_BES.Start
  -- of the non-related SWERB_BES row.

  PERFORM 1
    FROM swerb_bes AS this_be
         JOIN swerb_departs_data AS this_sdd
              ON (this_sdd.did = this_be.did)
       , swerb_bes AS other_be
         JOIN swerb_departs_data AS other_sdd
              ON (other_sdd.did = other_be.did)
    WHERE this_be.beid = NEW.beid
          AND other_sdd.date = this_sdd.date
          AND other_be.focal_grp = this_be.focal_grp
          AND this_be.start IS NULL
          AND other_be.seq < this_be.seq
          AND other_be.start > NEW.time
    LIMIT 1;

  IF FOUND THEN
    RAISE EXCEPTION 'SWId %: BEId %: Time %: Incorrect sequencing of SWERB_BES: A SWERB_BES (having SWERB_BES.BEId = ''%'') row related to this SWERB_DATA row has a NULL Start value and is sequenced after a different SWERB_BES row, for the same group/day, even though %, the time by which the first bout of observervation must have started, is before the Start of the 2nd SWERB_BES row'
                  , NEW.swid
                  , NEW.beid
                  , NEW.time
                  , NEW.beid
                  , NEW.time;
    RETURN NULL;
  END IF;

  --
  -- Now that sequencing is updated we can check to see that
  -- begin or end rows which place the group at a sleeping grove
  -- is the first or last observation of the day.
  --
  -- Or we could if we wanted to check immediately.  Instead we
  -- perform this check on commit.

  RETURN NULL;
  END;
$$;

SELECT 'swerb_data_delete_func' AS function;
CREATE FUNCTION swerb_data_delete_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- Function for swerb_data delete trigger
  --
  -- Copyright (C) 2011 The Meme Factory, Inc., http://www.meme.com/
  -- GPL_notice(`  --', `2011', `Karl O. Pinc <kop@meme.com>')

  DECLARE
/*
    this_seq swerb_bes.seq%TYPE;

  -- If the related SWERB_BEs row
  -- has a NULL Start value then it depends on the SWERB_DATA row for
  -- automatic setting of the Seq value, in which case the trigger
  -- must be invoked to do the automatic sequencing.
*/

  BEGIN
/* debug
  raise warning 'TG_OP: %: SWId %: BEId %: Seen_grp %: Event %: Time %'
              , TG_OP
              , OLD.swid
              , OLD.beid
              , OLD.seen_grp
              , OLD.event
              , OLD.time;
*/
/*
  SELECT swerb_bes.seq INTO this_seq
    FROM swerb_bes
    WHERE swerb_bes.beid = OLD.beid
          AND swerb_bes.start IS NULL
    LIMIT 1;

  IF this_seq IS NOT NULL THEN
    UPDATE swerb_bes
      SET seq = this_seq
      WHERE swerb_bes.beid = OLD.beid;
  END IF;
*/

  -- 

  PERFORM 1
    FROM swerb_bes AS this_be
         JOIN swerb_departs_data AS this_sdd
              ON (this_sdd.did = this_be.did)
       , swerb_bes AS other_be
         JOIN swerb_departs_data AS other_sdd
              ON (other_sdd.did = other_be.did)
         JOIN swerb_data AS other_sd
              ON (other_sd.beid = other_be.beid)
    WHERE this_be.beid = OLD.beid
          AND other_sdd.date = this_sdd.date
          AND other_be.focal_grp = this_be.focal_grp
          AND this_be.start IS NULL
          AND this_be.seq < other_be.seq
          AND this_be.start > other_sd.time
    LIMIT 1;

  IF FOUND THEN
    RAISE EXCEPTION 'SWId %: BEId %: Time %: A SWERB_BES row (with SWERB_BES.BEId = ''%'') related to this SWERB_DATA row is sequenced before another SWERB_BES row having a NULL Start value, and for the same group/day, with a related SERB_DATA.Start that is before the first SWERB_BES row (with SWERB_BES.BEId = ''%'') Start value -- the bout of obserevation belonging to this waypoint record is sequenced before a second bout of observation but this bout of observervation starts (SWERB_BES.Start) after the second observatinal bout''s first waypoint''s timestamp (SWERB_DATA.Time).  This cannot be fixed automatically because the second bout of observation has a NULL SWERB_BES.Start.'
                  , NEW.swid
                  , NEW.beid
                  , NEW.time
                  , NEW.beid
                  , NEW.beid;
    RETURN NULL;
  END IF;

  -- Update SWERB_BES or tell it to resequence.

  IF OLD.event = 'bb_swerb_begin' THEN
    -- Begin rows update SWERB_BES.
    UPDATE swerb_bes
      SET start = NULL
      WHERE beid = OLD.beid;
  ELSIF OLD.event = 'bb_swerb_end' THEN
    -- End rows update SWERB_BES.
    UPDATE swerb_bes
      SET stop = NULL
      WHERE beid = OLD.beid;
  ELSE
    IF OLD.time IS NOT NULL THEN
      -- Resequence.
      PERFORM 1
        FROM swerb_bes
        WHERE swerb_bes.beid = OLD.beid
              AND swerb_bes.start IS NULL
        LIMIT 1;
      IF FOUND THEN
        -- Resequence.  This is only really needed if the deleted
        -- row is the earliest of all the rows in the bout of observation,
        -- but it's probably not worth trying to figure this out.
        UPDATE swerb_bes
          SET start = start                -- i.e. NULL
          WHERE swerb_bes.beid = OLD.beid;
      END IF;
    END IF;
  END IF;


  RETURN NULL;
  END;
$$;

SELECT 'swerb_data_commit_func' AS function;
CREATE OR REPLACE FUNCTION swerb_data_commit_func()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  -- Function for swerb_data insert and update fired upon transaction commit.
  --
  -- GPL_notice(`  --', `2012', `The Meme Factory, Inc.  http://www.meme.com/')
  --
  BEGIN

  -- Get the latest values of the row
  SELECT * INTO NEW FROM swerb_data WHERE swerb_data.swid = NEW.swid;
  IF NOT FOUND THEN
    -- Whatever row was inserted was subsequently deleted.
    -- Nothing to do.
    RETURN NULL;
  END IF;

  -- Check against SWERB_LOC_DATA and against the physical location
  -- where the group is placed.

  IF NEW.event = 'bb_swerb_water' THEN
    --  Water events must place the group at a watering hole.
    DECLARE
      do_check BOOLEAN := TRUE;

    BEGIN
      IF TG_OP = 'UPDATE' THEN
        -- no point in checking when the Event has not changed.
        do_check := NEW.event <> OLD.event;
      END IF;

      IF do_check THEN
        -- Must be at a watering hole, even if it's the unknown
        -- watering hole.
        PERFORM 1
          FROM swerb_loc_data
               JOIN swerb_gws ON (swerb_gws.loc = swerb_loc_data.loc)
          WHERE swerb_loc_data.swid = NEW.swid
                AND swerb_gws.type = 'bb_swerb_gws_water'
          LIMIT 1;

        IF NOT FOUND THEN
          RAISE EXCEPTION 'Error on SWERB_DATA commit: SWId %: BEId %; Seen_grp %: Event %: Time %: X %: Y %: When SWERB_DATA.Event is bb_swerb_water then there must be a related SWERB_GWS row with a Type of bb_swerb_gws_water'
                        , NEW.swid
                        , NEW.beid
                        , NEW.seen_grp
                        , NEW.event
                        , NEW.time
                        , ST_X(NEW.xyloc)
                        , ST_Y(NEW.xyloc);
          RETURN NULL;
        END IF;
      END IF;
    END;
  END IF; -- NEW.event = 'bb_swerb_water'

  -- Every bout of observation must have exactly one begin record.
  DECLARE
    c INTEGER;

  BEGIN
    SELECT INTO c COUNT(*)
      FROM swerb_data
      WHERE swerb_data.beid = NEW.beid
            AND swerb_data.event = 'bb_swerb_begin';

    IF c <> 1 THEN
      RAISE EXCEPTION 'Error on SWERB_DATA commit: Swid %: BEid %: Seen_grp %: Event %: Time %: Subgroup %: There must be exactly one begin row for this bout of observation, exactly one SWERB_DATA row with Event=bb_swerb_begin where BEId=%: % found'
                      , NEW.swid
                      , NEW.beid
                      , NEW.seen_grp
                      , NEW.event
                      , NEW.time
                      , NEW.subgroup
                      , NEW.beid
                      , c;
      RETURN NULL;
    END IF;
  END;

  -- Every bout of observation must have exactly one end record.
  DECLARE
    c INTEGER;

  BEGIN
    SELECT INTO c COUNT(*)
      FROM swerb_data
      WHERE swerb_data.beid = NEW.beid
            AND swerb_data.event = 'bb_swerb_end';

    IF c <> 1 THEN
      RAISE EXCEPTION 'Error on SWERB_DATA commit: Swid %: BEid %: Seen_grp %: Event %: Time %: Subgroup %: There must be exactly one end row for this bout of observation, exactly one SWERB_DATA row with Event=bb_swerb_end where BEId=%: % found'
                      , NEW.swid
                      , NEW.beid
                      , NEW.seen_grp
                      , NEW.event
                      , NEW.time
                      , NEW.subgroup
                      , NEW.beid
                      , c;
      RETURN NULL;
    END IF;
  END;

  --
  -- Begin rows must record a descent from a sleeping grove when
  -- the first observation of the group for the day.
  --

  IF NEW.event = 'bb_swerb_begin' THEN
    -- Begin rows must record a descent from a sleeping grove when
    -- the observation team's first observation of the group
    -- for the day and otherwise, ignoring those rows that are
    -- descents from the unknown grove,
    -- must not record descent from a sleeping grove, unless
    -- all of the descents from sleeping groves are of subgroups
    -- in which case there can be multiple descents.
    DECLARE
      do_check BOOLEAN := TRUE;

    BEGIN
      IF TG_OP = 'UPDATE' THEN
        do_check := OLD.event <> 'bb_swerb_begin';
      END IF;

      IF do_check THEN
        -- Is this the team's first observation of the group for the day?
        PERFORM 1
          FROM swerb_bes AS this_sbe
                JOIN swerb_bes AS other_sbe ON (other_sbe.did = this_sbe.did)
          WHERE this_sbe.beid = NEW.beid
                AND other_sbe.focal_grp = this_sbe.focal_grp
                AND other_sbe.seq < this_sbe.seq
          LIMIT 1;
        IF FOUND THEN
          -- This is not the team's first observation of the day.
          -- It must not be an ascent or descent, unless all
          -- the other sleeping is (either ascents or descents)
          -- done in subgroups.
          DECLARE
            this_adn adcodes.adn%TYPE;
            this_loc swerb_loc_data.loc%TYPE;

          BEGIN
            SELECT swerb_loc_data.loc, adcodes.adn
              INTO this_loc,           this_adn
              FROM swerb_loc_data
                   JOIN adcodes ON (adcodes.adcode = swerb_loc_data.adcode)
              WHERE swerb_loc_data.swid = NEW.swid
                    AND adcodes.adn <> 'bb_swerb_notad';

            IF FOUND THEN
              IF this_adn = 'bb_swerb_ascent' THEN
                RAISE EXCEPTION 'Error on SWERB_DATA commit: Swid %: BEid %: Seen_grp %: Event %: Time %: Subgroup %: SWERB_DATA begin rows cannot be related to SWERB_LOC_DATA rows representing ascent into a sleeping grove'
                              , NEW.swid
                              , NEW.beid
                              , NEW.seen_grp
                              , NEW.event
                              , NEW.time
                              , NEW.subgroup;
                RETURN NULL;
              END IF;

              -- This is a descent from a sleeping grove.
              IF NOT(NEW.subgroup AND this_loc = 'bb_swerb_unk_grove') THEN
                -- It's not a descent that should be ignored.

                -- Descents are only allowed
                -- if all the descents are for subgroups.
                PERFORM 1
                  FROM swerb_bes AS this_sbe
                       JOIN swerb_bes AS other_sbe
                            ON (other_sbe.did = this_sbe.did)
                       JOIN swerb_data
                            ON (swerb_data.beid = other_sbe.beid)
                       JOIN swerb_loc_data
                            ON (swerb_loc_data.swid = swerb_data.swid)
                      JOIN adcodes
                            ON (adcodes.adcode = swerb_loc_data.adcode)
                  WHERE this_sbe.beid = NEW.beid
                        AND other_sbe.focal_grp = this_sbe.focal_grp
                        AND NOT(swerb_data.subgroup)
                        AND adcodes.adn = 'bb_swerb_descent'
                  LIMIT 1;

                IF FOUND THEN
                  RAISE EXCEPTION 'Error on SWERB_DATA commit: Swid %: BEid %: Seen_grp %: Event %: Time %: Subgroup %: SWERB_DATA rows cannot be related to SWERB_LOC_DATA rows representing descent from sleeping groves unless the SWERB_DATA row begins an observation team''s observation of the group for the day or unless all of the descents are of subgroups'
                                , NEW.swid
                                , NEW.beid
                                , NEW.seen_grp
                                , NEW.event
                                , NEW.time
                                , NEW.subgroup;
                  RETURN NULL;
                END IF;
              END IF;
            END IF;
          END;
        ELSE
          -- This is the observation team's first observation of the
          -- group of the day.
          -- It must be a descent, unless it's of the unknown group.
          PERFORM 1
            FROM swerb_bes
            WHERE swerb_bes.beid = NEW.beid
                  AND swerb_bes.focal_grp = 'bb_unk_grp'
            LIMIT 1;
          IF NOT FOUND THEN
            -- Not exempted from needing a sleeping grove by way of being
            -- an observation of the unknown group.
            PERFORM 1
              FROM swerb_loc_data
                   JOIN adcodes ON (adcodes.adcode = swerb_loc_data.adcode)
              WHERE swerb_loc_data.swid = NEW.swid
                    AND adcodes.adn = 'bb_swerb_descent'
              LIMIT 1;
            IF NOT FOUND THEN
              RAISE EXCEPTION 'Error on SWERB_DATA commit: Swid %: BEid %: Seen_grp %: Event %: Time %: Subgroup %: Unless the focal group is the unknown group (bb_unk_grp), SWERB_DATA rows must be related to a SWERB_LOC_DATA rows representing descent into a sleeping grove when the SWERB_LOC_DATA row is the observation team''s first observation of the group for the day'
                            , NEW.swid
                            , NEW.beid
                            , NEW.seen_grp
                            , NEW.event
                            , NEW.time
                            , NEW.subgroup;
              RETURN NULL;
            END IF;
          END IF;
        END IF;
      END IF;
    END;
  END IF;

  --
  -- End rows must record an ascent into a sleeping grove when
  -- the last observation of the group for the day.
  --

  IF NEW.event = 'bb_swerb_end' THEN
    -- End rows must record an ascent into a sleeping grove when
    -- the observation team's last observation of the group
    -- for the day and otherwise
    -- must not record descent from a sleeping grove.
    DECLARE
      do_check BOOLEAN := TRUE;

    BEGIN
      IF TG_OP = 'UPDATE' THEN
        do_check := OLD.event <> 'bb_swerb_end';
      END IF;

      IF do_check THEN
        -- Is this the team's last observation of the group for the day?
        PERFORM 1
          FROM swerb_bes AS this_sbe
                JOIN swerb_bes AS other_sbe ON (other_sbe.did = this_sbe.did)
          WHERE this_sbe.beid = NEW.beid
                AND other_sbe.focal_grp = this_sbe.focal_grp
                AND other_sbe.stop > this_sbe.stop
          LIMIT 1;
        IF FOUND THEN
          -- This is not the team's last observation of the day.
          -- If it's not a subgroup that sleeps in the unknown grove
          -- it must not be an ascent or descent, unless all
          -- the other sleeping is (either ascents or descents)
          -- done in subgroups.
          DECLARE
            this_adn adcodes.adn%TYPE;
            this_loc swerb_loc_data.loc%TYPE;

          BEGIN
            SELECT swerb_loc_data.loc, adcodes.adn
              INTO this_loc,           this_adn
              FROM swerb_loc_data
                   JOIN adcodes ON (adcodes.adcode = swerb_loc_data.adcode)
              WHERE swerb_loc_data.swid = NEW.swid
                    AND adcodes.adn <> 'bb_swerb_notad';

            IF FOUND THEN
              IF this_adn = 'bb_swerb_descent' THEN
                RAISE EXCEPTION 'Error on SWERB_DATA commit: Swid %: BEid %: Seen_grp %: Event %: Time %: Subgroup %: SWERB_DATA end rows cannot be related to SWERB_LOC_DATA rows representing descent from a sleeping grove'
                              , NEW.swid
                              , NEW.beid
                              , NEW.seen_grp
                              , NEW.event
                              , NEW.time
                              , NEW.subgroup;
                RETURN NULL;
              END IF;

              -- This is an ascent into a sleeping grove.
              IF NOT(NEW.subgroup AND this_loc = 'bb_swerb_unk_grove') THEN
                -- Not a subgroup sleeping in the unknown grove.
                -- Check it.

                -- Ascent is only allowed
                -- if all the ascents are for subgroups.
                PERFORM 1
                  FROM swerb_bes AS this_sbe
                       JOIN swerb_bes AS other_sbe
                            ON (other_sbe.did = this_sbe.did)
                       JOIN swerb_data
                            ON (swerb_data.beid = other_sbe.beid)
                       JOIN swerb_loc_data
                            ON (swerb_loc_data.swid = swerb_data.swid)
                       JOIN adcodes
                            ON (adcodes.adcode = swerb_loc_data.adcode)
                  WHERE this_sbe.beid = NEW.beid
                        AND other_sbe.focal_grp = this_sbe.focal_grp
                        AND NOT(swerb_data.subgroup)
                        AND adcodes.adn = 'bb_swerb_ascent'
                  LIMIT 1;

                IF FOUND THEN
                  RAISE EXCEPTION 'Error on SWERB_DATA commit: Swid %: BEid %: Seen_grp %: Event %: Time %: Subgroup %: SWERB_DATA rows cannot be related to SWERB_LOC_DATA rows representing ascent into sleeping groves unless the SWERB_DATA row ends an observation team''s observation of the group for the day or unless all of the ascents are of subgroups'
                                , NEW.swid
                                , NEW.beid
                                , NEW.seen_grp
                                , NEW.event
                                , NEW.time
                                , NEW.subgroup;
                  RETURN NULL;
                END IF;
              END IF;
            END IF;
          END;
        ELSE
          -- This might be the observation team's last observation of the day.
          -- It's either (one of) the row(s) with the last Stop time
          -- or has a NULL Stop time.
          DECLARE
            this_is_ascent BOOLEAN;
            other_ascent_exists BOOLEAN;

            this_loc swerb_loc_data.loc%TYPE;

          BEGIN
            -- Does this bout of observation have an ascent?
            SELECT swerb_loc_data.loc
              INTO this_loc
              FROM swerb_loc_data
                   JOIN adcodes ON (adcodes.adcode = swerb_loc_data.adcode)
              WHERE swerb_loc_data.swid = NEW.swid
                    AND adcodes.adn = 'bb_swerb_ascent'
              LIMIT 1;
            this_is_ascent := FOUND;

            -- Are there any other observations that also have
            -- ascents?
            PERFORM 1
              FROM swerb_bes AS this_sbe
                   JOIN swerb_bes ON (swerb_bes.did = this_sbe.did)
                   JOIN swerb_data ON (swerb_data.beid = swerb_bes.beid)
                   JOIN swerb_loc_data ON (swerb_loc_data.swid = swerb_data.swid)
                   JOIN adcodes ON (adcodes.adcode = swerb_loc_data.adcode)
              WHERE this_sbe.beid = NEW.beid
                    AND swerb_bes.focal_grp = this_sbe.focal_grp
                    AND swerb_bes.beid <> this_sbe.beid
                    AND adcodes.adn = 'bb_swerb_ascent'
              LIMIT 1;
            other_ascent_exists := FOUND;

            IF this_is_ascent THEN
              IF other_ascent_exists THEN
                -- Can't have more than one bout of observation by a single
                -- team for a single group on a single day that is an ascent,
                -- ignoring subgroup ascents into the unknown grove,
                -- unless all the ascent are of subgroups.
                IF NOT(NEW.subgroup AND this_loc = 'bb_swerb_unk_grove') THEN
                  -- Not a subgroup ascent into the unkonwn grove.

                  PERFORM 1
                    FROM swerb_bes AS this_sbe
                         JOIN swerb_bes ON (swerb_bes.did = this_sbe.did)
                         JOIN swerb_data ON (swerb_data.beid = swerb_bes.beid)
                         JOIN swerb_loc_data
                              ON (swerb_loc_data.swid = swerb_data.swid)
                         JOIN adcodes ON (adcodes.adcode = swerb_loc_data.adcode)
                    WHERE this_sbe.beid = NEW.beid
                          AND swerb_bes.focal_grp = this_sbe.focal_grp
                          AND NOT(swerb_data.subgroup)
                          AND adcodes.adn = 'bb_swerb_ascent'
                    LIMIT 1;

                  IF FOUND THEN
                    RAISE EXCEPTION 'Error on SWERB_DATA commit: Swid %: BEid %: Seen_grp %: Event %: Time %: Subgroup %: Only one bout of observation of a group that meets the qualifications for being the last of the observation team''s for the day can have a related SWERB_LOC_DATA row recording ascent into a sleeping grove, unless all the ascents are of subgroups'
                                  , NEW.swid
                                  , NEW.beid
                                  , NEW.seen_grp
                                  , NEW.event
                                  , NEW.time
                                  , NEW.subgroup;
                    RETURN NULL;
                  END IF;
                END IF;
              END IF;
            ELSE
              -- This row does not have a related ascent.
              IF NOT other_ascent_exists THEN
                PERFORM 1
                  FROM swerb_bes
                  WHERE swerb_bes.beid = NEW.beid
                        AND swerb_bes.focal_grp = 'bb_unk_grp'
                  LIMIT 1;
                IF NOT FOUND THEN
                  -- When the focal is not the unknown group
                  -- there must be at least one ascent for the team/group/day.
                  RAISE EXCEPTION 'Error on SWERB_DATA commit: Swid %: BEid %: Seen_grp %: Event %: Time %: Subgroup %: Unless the focal group is the unknown group (bb_unk_grp), one bout of observation of a group that meets the qualifications for being the last of the observation team''s for the day must have a related SWERB_LOC_DATA row recording ascent into a sleeping grove'
                                , NEW.swid
                                , NEW.beid
                                , NEW.seen_grp
                                , NEW.event
                                , NEW.time
                                , NEW.subgroup;
                  RETURN NULL;
                END IF;
              END IF;
            END IF;
          END;
        END IF;
      END IF;
    END;
  END IF;

  IF TG_OP = 'UPDATE' THEN
    IF OLD.event = 'bb_swerb_begin'
       AND NEW.event <> 'bb_swerb_begin' THEN
      -- (Probably) got rid of a begin row.
      DECLARE
        begin_beid swerb_bes.beid%TYPE;

      BEGIN
        -- Find the start seq of the first observation of the group by the team.
        SELECT        other_sbe.beid
          INTO STRICT begin_beid
          FROM swerb_bes AS this_sbe
               JOIN swerb_bes AS other_sbe ON (other_sbe.did = this_sbe.did)
          WHERE this_sbe.beid = NEW.beid
                AND other_sbe.focal_grp = this_sbe.focal_grp
          ORDER BY other_sbe.seq
          LIMIT 1;

        -- The first begin of the observation team for the day
        -- must be a descent from a grove.
        PERFORM 1
          FROM swerb_data
               JOIN swerb_loc_data ON (swerb_loc_data.swid = swerb_data.swid)
               JOIN acodes ON (adcodes.adcode = swerb_loc_data.adcode)
        WHERE swerb_data.beid = begin_beid
              AND adcodes.adn = 'bb_swerb_descent'
        LIMIT 1;
        IF NOT FOUND THEN
          -- But the unknown group is exempt from sleeping in a grove.
          PERFORM 1
            FROM swerb_bes
            WHERE swerb_bes.beid = begin_beid
                  AND swerb_bes.focal_grp = 'bb_unk_grp'
            LIMIT 1;
          IF NOT FOUND THEN
            RAISE EXCEPTION 'Error on SWERB_DATA commit: Swid %: BEid %: New Seen_grp %: Old Event %: New Event %: New Time %: New Subgroup %: Unless the focal group is the unknown group (bb_unk_grp), the observation team''s first observation of the group for the day no longer begins with a  sleeping grove descent SWERB_LOC_DATA row'
                          , NEW.swid
                          , NEW.beid
                          , NEW.seen_grp
                          , OLD.event
                          , NEW.event
                          , NEW.time
                          , NEW.subgroup;
            RETURN NULL;
          END IF;
        END IF;
      END;
    END IF;

    IF OLD.event = 'bb_swerb_end'
       AND NEW.event <> 'bb_swerb_end' THEN
      -- (Probably) got rid of an end row.

      -- The unknown group is exempt from sleeping in groves.
      PERFORM 1
        FROM swerb_bes
        WHERE swerb_bes.beid = NEW.beid
              AND swerb_bes.focal_grp = 'bb_unk_grp'
        LIMIT 1;

      IF FOUND THEN
        -- The unknown group is not the focal.
        -- The last observation of the observation team at
        -- the end of the day must be an aescent from a grove.

        DECLARE
          end_time swerb_bes.stop%TYPE;

        BEGIN
          -- Find the stop time of the last observation of
          -- the group by the team.
          SELECT        other_sbe.stop
            INTO STRICT end_time
            FROM swerb_bes AS this_sbe
                 JOIN swerb_bes AS other_sbe ON (other_sbe.did = this_sbe.did)
            WHERE this_sbe.beid = NEW.beid
                  AND other_sbe.stop IS NOT NULL
                  AND other_sbe.focal_grp = this_sbe.focal_grp
            ORDER BY other_sbe.stop DESC
            LIMIT 1;

          -- Make sure there's an ascent with the stop time of the
          -- last observation, or with a NULL stop time.
          PERFORM 1
            FROM swerb_bes AS this_sbe
                 JOIN swerb_bes AS other_sbe ON (other_sbe.did = this_sbe.did)
                 JOIN swerb_data ON (swerb_data.beid = other_sbe.beid)
                 JOIN swerb_loc_data ON (swerb_loc_data.swid = swerb_data.swid)
                 JOIN acodes ON (adcodes.adcode = swerb_loc_data.adcode)
            WHERE this_sbe.beid = NEW.beid
                  AND other_sbe.focal_grp = this_sbe.focal_grp
                  AND (other_sbe.stop = end_time
                       OR other_sbe.stop IS NULL)
                  AND adcodes.adn = 'bb_swerb_ascent'
            LIMIT 1;
          IF NOT FOUND THEN
            RAISE EXCEPTION 'Error on SWERB_DATA commit: Swid %: BEid %: New Seen_grp %: Old Event %: New Event %: New Time %: New Subgroup %: The focal group is not the unknown group (bb_unk_grp) and the observation team''s last observation of the group for the day no longer has a sleeping grove ascent SWERB_LOC_DATA row'
                          , NEW.swid
                          , NEW.beid
                          , NEW.seen_grp
                          , OLD.event
                          , NEW.event
                          , NEW.time
                          , NEW.subgroup;
            RETURN NULL;
          END IF;
        END;
      END IF;
    END IF;
  END IF; -- TG_OP = 'UPDATE'

  RETURN NULL;
  END;
$$;


SELECT 'swerb_data_delete_commit_func' AS function;
CREATE OR REPLACE FUNCTION swerb_data_delete_commit_func()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  -- Function for swerb_data delete operations fired upon transaction commit.
  --
  -- GPL_notice(`  --', `2012', `The Meme Factory, Inc.  http://www.meme.com/')
  --
  DECLARE
    this_focal_grp swerb_bes.focal_grp%TYPE;

  BEGIN

  -- Are there still rows
  -- associated with this bout of observation?
  SELECT swerb_bes.focal_grp
    INTO this_focal_grp
    FROM swerb_bes
    WHERE swerb_bes.beid = OLD.beid;

  IF  FOUND THEN
    -- Yes, we need to check the delete didn't break things.
    -- (If no, then we can't check -- and we don't need to
    -- because the swerb_bes (etc.) triggers will check for us.)

    IF OLD.event = 'bb_swerb_begin' THEN
      -- Every bout of observation must have a begin record.
      DECLARE
        c INTEGER;

      BEGIN
        SELECT INTO c COUNT(*)
          FROM swerb_data
          WHERE swerb_data.beid = OLD.beid
                AND swerb_data.event = 'bb_swerb_begin';

        IF c <> 1 THEN
          RAISE EXCEPTION 'Error on SWERB_DATA delete commit: Swid %: BEid %: Seen_grp %: Event %: Time %: Subgroup %: Every bout of observation must have exactly one begin, exactly one SWERB_DATA row with Event=bb_swerb_begin where BEId=%: % found'
                          , OLD.swid
                          , OLD.beid
                          , OLD.seen_grp
                          , OLD.event
                          , OLD.time
                          , OLD.subgroup
                          , OLD.beid
                          , c;
          RETURN NULL;
        END IF;
      END;
    END IF;

    IF this_focal_grp <> 'bb_unk_grp' THEN
      -- Every day of an observation team's observeration of a group
      -- must begin with descent from a sleeping grove.  We need to know
      -- that we have not deleted this.

      DECLARE
        begin_beid swerb_bes.beid%TYPE;

      BEGIN
        -- Find the beid when the team started observing the group.
        SELECT swerb_bes.beid
          INTO STRICT begin_beid
          FROM swerb_bes
          WHERE swerb_bes.beid = OLD.beid
                AND swerb_bes.focal_grp = OLD.focal_grp
          ORDER BY swerb_bes.seq
          LIMIT 1;

        -- Make sure that the first beid has a descent.
        PERFORM 1
          FROM swerb_bes
               JOIN swerb_data
                    ON (swerb_data.beid = swerb_bes.beid)
               JOIN swerb_loc_data
                    ON (swerb_loc_data.swid = swerb_data.swid)
               JOIN adcodes
                    ON (adcodes.adcode = swerb_loc_data.adcode)
          WHERE swerb_bes.beid = begin_beid
                AND adcodes.adn = 'bb_swerb_descent'
          LIMIT 1;
        IF NOT FOUND THEN
          RAISE EXCEPTION 'Error on SWERB_DATA delete commit: Swid %: BEid %: Seen_grp %: Event %: Time %: Subgroup %: Cannot delete SWERB_DATA row: Unless the focal group is the unknown group (bb_unk_grp), every day of an observation team''s observation of a group must begin with descent from a sleeping grove'
                        , OLD.swid
                        , OLD.beid
                        , OLD.seen_grp
                        , OLD.event
                        , OLD.time
                        , OLD.subgroup;
          RETURN NULL;
        END IF;
      END;
    END IF;

    IF OLD.event = 'bb_swerb_end' THEN
      -- Every bout of observation must have exactly 1 end record.
      DECLARE
        c INTEGER;

      BEGIN
        SELECT INTO c COUNT(*)
          FROM swerb_data
          WHERE swerb_data.beid = OLD.beid
                AND swerb_data.event = 'bb_swerb_end';

        IF c <> 1 THEN
          RAISE EXCEPTION 'Error on SWERB_DATA delete commit: Swid %: BEid %: Seen_grp %: Event %: Time %: Subgroup %: Every bout of observation must have exactly one end, exactly one SWERB_DATA row with Event=bb_swerb_end where BEId=%: % found'
                          , OLD.swid
                          , OLD.beid
                          , OLD.seen_grp
                          , OLD.event
                          , OLD.time
                          , OLD.subgroup
                          , OLD.beid
                          , c;
          RETURN NULL;
        END IF;
      END;

      IF this_focal_grp <> 'bb_unk_grp' THEN
        -- Every day of an observation team's observeration of a group
        -- must end with ascent into a sleeping grove.  We need to know
        -- that we have not deleted this.

        DECLARE
          end_time swerb_bes.stop%TYPE;

        BEGIN
          -- Find the time when the team stopped observing the group.
          SELECT swerb_bes.stop
            INTO end_time
            FROM swerb_bes
            WHERE swerb_bes.beid = OLD.beid
                  AND swerb_bes.stop IS NOT NULL
                  AND swerb_bes.focal_grp = OLD.focal_grp
            ORDER BY swerb_bes.stop DESC
            LIMIT 1;

          -- See if the end row for the team/group/day with the
          -- time when the team stopped observation for the day,
          -- or with a NULL time, has an ascent.
          PERFORM 1
            FROM swerb_bes
                 JOIN swerb_data
                      ON (swerb_data.beid = swerb_bes.beid)
                 JOIN swerb_loc_data
                      ON (swerb_loc_data.swid = swerb_data.swid)
                 JOIN adcodes
                      ON (adcodes.adcode = swerb_loc_data.adcode)
            WHERE swerb_bes.beid = OLD.beid
                  AND swerb_bes.focal_grp = OLD.focal_grp
                  AND (swerb_bes.stop = end_time
                       OR swerb_bes.stop IS NULL)
                  AND adcodes.adn = 'bb_swerb_ascent'
            LIMIT 1;
          IF NOT FOUND THEN
            RAISE EXCEPTION 'Error on SWERB_DATA delete commit: Swid %: BEid %: Seen_grp %: Event %: Time %: Subgroup %: Cannot delete SWERB_DATA row: Every day of an observation team''s observation of a group must end with ascent into a sleeping grove'
                          , OLD.swid
                          , OLD.beid
                          , OLD.seen_grp
                          , OLD.event
                          , OLD.time
                          , OLD.subgroup;
            RETURN NULL;
          END IF;
        END;
      END IF;
    END IF;
  END IF;

  RETURN NULL;
  END;
$$;


SELECT 'swerb_data_trigger' AS trigger;
CREATE TRIGGER swerb_data_trigger
  AFTER INSERT OR UPDATE
  ON swerb_data FOR EACH ROW
  EXECUTE PROCEDURE swerb_data_func();

SELECT 'swerb_data_delete_trigger' AS trigger;
CREATE TRIGGER swerb_data_delete_trigger
  AFTER DELETE
  ON swerb_data FOR EACH ROW
  EXECUTE PROCEDURE swerb_data_delete_func();

SELECT 'swerb_data_commit_trigger' AS trigger;
CREATE CONSTRAINT TRIGGER swerb_data_commit_trigger
  AFTER INSERT OR UPDATE
  ON swerb_data
  DEFERRABLE INITIALLY DEFERRED
  FOR EACH ROW
  EXECUTE PROCEDURE swerb_data_commit_func();

SELECT 'swerb_data_delete_commit_trigger' AS trigger;
CREATE CONSTRAINT TRIGGER swerb_data_delete_commit_trigger
  AFTER DELETE
  ON swerb_data
  DEFERRABLE INITIALLY DEFERRED
  FOR EACH ROW
  EXECUTE PROCEDURE swerb_data_delete_commit_func();
