dnl Copyright (C) 2012, 2014 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2011, Karl O. Pinc
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl

dnl plpgsql fragment for checking whether there's a (non-)NULL Garmincode.
dnl
dnl Syntax: _check_garmincode(is)
dnl
dnl is    Either 'IS or 'IS NOT' to test for NULL.
dnl       (Case is significant.)
dnl
changequote({,})
define({_check_garmincode},{
      -- Check SWERB_DEPARTS_GPS.Garmincode
      PERFORM 1
        FROM swerb_departs_gps
        WHERE swerb_departs_gps.did = NEW.did
              AND swerb_departs_gps.garmincode $1 NULL
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION 'DId %: Date %: Cannot have a ifelse($1, `IS NOT', `non-')NULL SWERB_DEPARTS_GPS.Garmincode ifelse($1, `IS NOT', `before', `on or after') bb_subm_alt_date'
                      , NEW.did
                      , NEW.date;
        RETURN NULL;
      END IF;

      -- Check SWERB_DATA.Garmincode
      PERFORM 1
        FROM swerb_bes
             JOIN swerb_data ON (swerb_data.beid = swerb_bes.beid)
        WHERE swerb_bes.did = NEW.did
              AND swerb_data.garmincode $1 NULL
              ifelse($1, `IS', AND swerb_data.event <> 'bb_swerb_begin' AND swerb_data.event <> 'bb_swerb_end')
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION 'DId %: Date %: Cannot have a ifelse($1, `IS NOT', `non-')NULL SWERB_DATA.Garmincode ifelse($1, `IS NOT', `before', `on or after') bb_subm_alt_date`'ifelse($1, `IS', ` unless the SWERB_DATA.Event is bb_swerb_begin or bb_swerb_end')'
                      , NEW.did
                      , NEW.date;
        RETURN NULL;
      END IF;

      -- (No need to check SWERB_LOC_GPS.Garmincode because it's rows can't
      -- exist before bb_subm_alt_date.)
})dnl
changequote(`,')dnl


--  
-- swerb_departs_data
--
SELECT 'swerb_departs_data' AS table;

SELECT 'swerb_departs_data_func' AS function;
CREATE FUNCTION swerb_departs_data_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  BEGIN
  -- Function for swerb_departs_data insert and update triggers
  --
  -- Copyright (C) 2012, 2013 The Meme Factory, Inc.  http://www.meme.com/
  -- GPL_notice(`  --', `2011', `Karl O. Pinc <kop@meme.com>')

  IF TG_OP = 'UPDATE' THEN

    -- Can't change the id.
    cannot_change(`SWERB_DEPARTS_DATA', `DId')

    -- Can't have GPS information before GPS units were used.
    IF NEW.date < OLD.date
       AND NEW.date < 'bb_gps_start' THEN

      -- Can't have the GPS info in the departure record before bb_gps_start.
      PERFORM 1
        FROM swerb_departs_gps
        WHERE swerb_departs_gps.did = NEW.did
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION 'Did %: Date %: SWERB_DEPARTS_DATA cannot have an associated SWERB_DEPARTS_GPS row for dates earlier than bb_gps_start'
                      , NEW.Did
                      , NEW.Date;
        RETURN NULL;
      END IF;

      -- And can't have GPS info in the SWERB row before bb_gps_start.
      PERFORM 1
        FROM swerb_bes
             JOIN swerb_data ON (swerb_data.beid = swerb_bes.beid)
        WHERE swerb_bes.did = NEW.did
              AND swerb_data.xyloc IS NOT NULL
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION 'DId %: Date %: SWERB_DEPARTS_DATA cannot have an associcated SWERB_DATA row with a non-NULL XYLoc when the Date is before bb_gps_start'
                      , NEW.did
                      , NEW.date;
        RETURN NULL;
      END IF;
    END IF;

    -- Can't have quad information after quad data collection stopped.
    IF NEW.date <> OLD.date THEN
      IF NEW.date > OLD.date THEN
        PERFORM 1
          FROM swerb_bes
               JOIN swerb_data ON (swerb_data.beid = swerb_bes.beid)
          WHERE swerb_bes.did = NEW.did
                AND swerb_data.quad IS NOT NULL
          LIMIT 1;
        IF FOUND THEN
          RAISE EXCEPTION 'DId %: Date %: SWERB_DEPARTS_DATA cannot have an associcated SWERB_DATA row with a non-NULL Quad when the Date is after bb_quad_end'
                        , NEW.did
                        , NEW.date;
          RETURN NULL;
        END IF;

        -- The combination of time, date, and observer, when all are non-null,
        -- must be unique.  (Checked only on update since observers
        -- can't exist at the time of insert.)
        PERFORM 1
          FROM swerb_departs_data, swerb_observers
          WHERE swerb_observers.did = swerb_departs_data.did
                AND swerb_departs_data.date = NEW.date
                AND swerb_departs_data.time = NEW.time
                AND swerb_departs_data.did <> NEW.did
                AND swerb_observers.observer
                    IN (SELECT so.observer
                          FROM swerb_observers AS so
                          WHERE so.did = NEW.did)
          LIMIT 1;
        IF FOUND THEN
          RAISE EXCEPTION 'Did %: Date %: Time %: The combination of date, time, and observer, when all are non-NULL, must be unique'
                        , NEW.did
                        , NEW.date
                        , NEW.time;
          RETURN NULL;
        END IF;

        -- Can't observe a lone animal before its entrydate.
        PERFORM 1
          FROM swerb_bes
               JOIN swerb_data ON (swerb_data.beid = swerb_bes.beid)
               JOIN biograph ON (biograph.sname = swerb_data.lone_animal)
          WHERE biograph.entrydate > NEW.date
          LIMIT 1;
        IF FOUND THEN
          RAISE EXCEPTION integrity_constraint_violation USING
                MESSAGE = 'Error on ' || TG_OP || 'of SWERB_DEPARTS_DATA'
              , DETAIL = 'Key (DId) = (' || NEW.did
                           || '): Value(Date) = (' || NEW.date
                           || '): Lone animal cannot be observed before their '
                           || 'BIOGRAPH.Entrydate';
          RETURN NULL;
        END IF;

        -- Can't observe a lone male after the individual's statdate.
        PERFORM 1
          FROM swerb_bes
               JOIN swerb_data ON (swerb_data.beid = swerb_bes.beid)
               JOIN biograph ON (biograph.sname = swerb_data.lone_animal)
          WHERE biograph.statdate < NEW.date
          LIMIT 1;
        IF FOUND THEN
          RAISE EXCEPTION 'Did %: Date %: Time %: Lone males cannot be observed after their BIOGRAPH.Statdate'
                        , NEW.did
                        , NEW.date
                        , NEW.time;
          RETURN NULL;
        END IF;
      END IF;

    END IF;

    -- Must have a time when there's data from a GPS unit.
    IF NEW.time IS NULL THEN
      PERFORM 1
        FROM swerb_departs_gps
        WHERE swerb_departs_gps.did = NEW.did
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION 'Did %: Date %: SWERB_DEPARTS_DATA.Time cannot be NULL when there is a related SWERB_DEPARTS_GPS row'
                      , NEW.did
                      , NEW.date;
        RETURN NULL;
      END IF;
    END IF;

    -- Can't have sub-meter altitude accuracy before bb_subm_alt_date.
    IF NEW.date < 'bb_subm_alt_date' THEN
      PERFORM 1
        FROM swerb_departs_gps
        WHERE swerb_departs_gps.did = NEW.did
              AND swerb_departs_gps.altitude
                  <> TRUNC(swerb_departs_gps.altitude)
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION 'Did %: Date %: Cannot have sub-meter altitude measurements on SWERB_DEPARTS_GPs before bb_subm_alt_date'
                      , NEW.did
                      , NEW.date;
        RETURN NULL;
      END IF;

      PERFORM 1
        FROM swerb_bes
             JOIN swerb_data ON (swerb_data.beid = swerb_bes.beid)
        WHERE swerb_bes.did = NEW.did
              AND TRUNC(swerb_data.altitude) <> swerb_data.altitude
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION 'Did %: Date %: Cannot have sub-meter altitude measurements on SWERB_DATA before bb_subm_alt_date'
                      , NEW.did
                      , NEW.date;
        RETURN NULL;
      END IF;
    END IF;

    -- Departure time must be before start of bouts of observation.
    PERFORM 1
      FROM swerb_bes
      WHERE swerb_bes.did = NEW.did
            AND swerb_bes.start <= NEW.time
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION 'DId %: Date %: Time %: SWERB_DEPARTS_DATA.Time invalid -- must be before SWERB_BES.Start'
                    , NEW.did
                    , NEW.date
                    , NEW.time;
      RETURN NULL;
    END IF;

    -- SWERB_BES can have NULL Start values.
    -- Make sure that SWERB_DATA observations don't happen before departure.
    IF (OLD.time IS NULL AND NEW.time IS NOT NULL)
       OR OLD.time < NEW.time THEN
      PERFORM 1
        FROM swerb_bes
             JOIN swerb_data ON (swerb_data.beid = swerb_bes.beid)
        WHERE swerb_bes.did = NEW.did
              AND swerb_data.time < NEW.time
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION 'DId %: Date %: Time %: SWERB_DEPARTS_DATA.Time invalid -- must be before SWERB_DATA.Time'
                      , NEW.did
                      , NEW.date
                      , NEW.time;
        RETURN NULL;
      END IF;
    END IF;

    -- PDOP

    IF (NEW.date < 'bb_earliest_pdop'
        AND 'bb_earliest_pdop' <= OLD.date)
       OR (NEW.date > 'bb_latest_pdop'
           AND OLD.DATE <= 'bb_latest_pdop') THEN
      -- Be sure there are no related PDOP values.

      -- swerb_departs_gps
      PERFORM 1
        FROM swerb_departs_gps
        WHERE swerb_departs_gps.did = NEW.did
              AND swerb_departs_gps.pdop IS NOT NULL
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION 'DId %: Date %: Cannot have a non-NULL PDOP value on SWERB_DEPARTS_GPS before bb_earliest_pdop or after bb_latest_pdop'
                      , NEW.did
                      , NEW.date;
        RETURN NULL;
      END IF;

      -- swerb_data
      -- Can't have a PDOP before bb_earliest_pdop or after bb_latest_pdop.
      PERFORM 1
        FROM swerb_bes
             JOIN swerb_data ON (swerb_data.beid = swerb_bes.beid)
        WHERE swerb_bes.did = NEW.did
              AND swerb_data.pdop IS NOT NULL
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION 'DId %: Date %: Cannot have a non-NULL PDOP value on SWERB_DATA before bb_earliest_pdop or after bb_latest_pdop'
                      , NEW.did
                      , NEW.date;
        RETURN NULL;
      END IF;
    END IF;

    -- Can't have an accuracy before bb_earliest_accuracy.
    IF (NEW.date < 'bb_earliest_accuracy'
        AND 'bb_earliest_accuracy' <= OLD.DATE) THEN
      -- Be sure there are no related accuracy values.

      -- swerb_departs_gps
      PERFORM 1
        FROM swerb_departs_gps
        WHERE swerb_departs_gps.did = NEW.did
              AND swerb_departs_gps.accuracy IS NOT NULL
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION 'DId %: Date %: Cannot have a non-NULL Accuracy value on SWERB_DEPARTS_GPS before bb_earliest_accuracy'
                      , NEW.did
                      , NEW.date;
        RETURN NULL;
      END IF;

      -- swerb_data
      PERFORM 1
        FROM swerb_bes
             JOIN swerb_data ON (swerb_data.beid = swerb_bes.beid)
        WHERE swerb_bes.did = NEW.did
              AND swerb_data.accuracy IS NOT NULL
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION 'DId %: Date %: Cannot have a non-NULL Accuracy value on SWERB_DATA before bb_earliest_accuracy'
                      , NEW.did
                      , NEW.date;
        RETURN NULL;
      END IF;
    END IF;

    -- GPS_Datetime

    IF NEW.date < 'bb_subm_alt_date' THEN
      -- The GPS_Datetime must be NULL before bb_subm_alt_date.
      PERFORM 1
        FROM swerb_bes
             JOIN swerb_data ON (swerb_data.beid = swerb_bes.beid)
        WHERE swerb_bes.did = NEW.did
              AND swerb_data.gps_datetime IS NOT NULL
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION 'DId %: Date %: Cannot have a non-NULL SWERB_DATA.GPS_Datetime date value before bb_subm_alt_date'
                      , NEW.did
                      , NEW.date;
        RETURN NULL;
      END IF;
    END IF;

    -- The date must match the swerb_data.gps_datetime.
    PERFORM 1
      FROM swerb_bes
           JOIN swerb_data ON (swerb_data.beid = swerb_bes.beid)
      WHERE swerb_bes.did = NEW.did
            AND swerb_data.gps_datetime IS NOT NULL
            AND swerb_data.gps_datetime::DATE <> NEW.date
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION 'DId %: Date %: Cannot have a SWERB_DATA.GPS_Datetime date value that is different from the SWERB_DEPARTS_DATA.Date value'
                    , NEW.did
                    , NEW.date;
      RETURN NULL;
    END IF;

    -- The date must match the swerb_loc_gps.gps_datetime.
    PERFORM 1
      FROM swerb_bes
           JOIN swerb_data ON (swerb_data.beid = swerb_bes.beid)
           JOIN swerb_loc_gps ON (swerb_loc_gps.swid = swerb_data.swid)
      WHERE swerb_bes.did = NEW.did
            AND swerb_loc_gps.gps_datetime::DATE <> NEW.date
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION 'DId %: Date %: Cannot have a SWERB_LOC_GPS.GPS_Datetime date value that is different from the SWERB_DEPARTS_DATA.Date value'
                    , NEW.did
                    , NEW.date;
      RETURN NULL;
    END IF;

    -- Garmincode
    IF NEW.date < 'bb_subm_alt_date' THEN
      -- Can't have a non-NULL Garmincode before bb_subm_alt_date.
      _check_garmincode(`IS NOT')
    ELSE
      -- Can't have a NULL Garmincode on or after bb_subm_alt_date.
      _check_garmincode(`IS')
    END IF;
  END IF;  -- TG_OP = 'UPDATE'

  -- Can't be located at a place before the place exists.
  PERFORM 1
    FROM swerb_bes, swerb_data, swerb_loc_data, swerb_gws
    WHERE swerb_bes.did = NEW.did
          AND swerb_data.beid = swerb_bes.beid
          AND swerb_loc_data.swid = swerb_data.swid
          AND swerb_gws.loc = swerb_loc_data.loc
          AND NEW.date < swerb_gws.start
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'DId %: Date %: Cannot have a SWERB_DEPARTS_DATA.Date before a related SWERB_GWS.Start -- cannot be located at a place before the place exists'
                  , NEW.did
                  , NEW.date;
    RETURN NULL;
  END IF;

  -- Can't be located at a place after the place ceases to exist.
  PERFORM 1
    FROM swerb_bes, swerb_data, swerb_loc_data, swerb_gws
    WHERE swerb_bes.did = NEW.did
          AND swerb_data.beid = swerb_bes.beid
          AND swerb_loc_data.swid = swerb_data.swid
          AND swerb_gws.loc = swerb_loc_data.loc
          AND swerb_gws.finish < NEW.date
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'DId %: Date %: Cannot have a SWERB_DEPARTS_DATA.Date after a related SWERB_GWS.Finish -- cannot be located at a place after the place ceases to exist'
                  , NEW.did
                  , NEW.date;
    RETURN NULL;
  END IF;

  -- Can't have an associated Nearest_Loc before the Nearest_Loc
  -- begins to exist.
  PERFORM 1
    FROM swerb_bes, swerb_data, swerb_loc_data_confidences, swerb_gws
    WHERE swerb_bes.did = NEW.did
          AND swerb_data.beid = swerb_bes.beid
          AND swerb_loc_data_confidences.swid = swerb_data.swid
          AND swerb_gws.loc = swerb_loc_data_confidences.nearest_loc
          AND NEW.date < swerb_gws.start
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION integrity_constraint_violation USING
          MESSAGE = 'Error on ' || TG_OP || ' of SWERB_DEPARTS_DATA'
        , DETAIL = 'Key(DId) = (' || NEW.did
                     || '): Value (Date) = (' || NEW.date
                     || '): Cannot have a Nearest_Loc before it '
                     || 'begins to exist';
    RETURN NULL;
  END IF;

  -- Can't have an associated Nearest_Loc after the Nearest_Loc
  -- ceases to exist.
  PERFORM 1
    FROM swerb_bes, swerb_data, swerb_loc_data_confidences, swerb_gws
    WHERE swerb_bes.did = NEW.did
          AND swerb_data.beid = swerb_bes.beid
          AND swerb_loc_data_confidences.swid = swerb_data.swid
          AND swerb_gws.loc = swerb_loc_data_confidences.nearest_loc
          AND swerb_gws.finish < NEW.date
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION integrity_constraint_violation USING
          MESSAGE = 'Error on ' || TG_OP || ' of SWERB_DEPARTS_DATA'
        , DETAIL = 'Key(DId) = (' || NEW.did
                     || '): Value (Date) = (' || NEW.date
                     || '): Cannot have a Nearest_Loc after it '
                     || 'ceases to exist';
    RETURN NULL;
  END IF;

  RETURN NULL;
  END;
$$;


SELECT 'swerb_departs_data_trigger' AS trigger;
CREATE TRIGGER swerb_departs_data_trigger
  AFTER INSERT OR UPDATE
  ON swerb_departs_data FOR EACH ROW
  EXECUTE PROCEDURE swerb_departs_data_func();
