dnl Copyright (C) 2011, 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2011, Karl O. Pinc
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl


--  
-- swerb_departs_gps
--
SELECT 'swerb_departs_gps' AS table;

SELECT 'swerb_departs_gps_func' AS function;
CREATE FUNCTION swerb_departs_gps_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  BEGIN
  -- Function for swerb_departs_gps insert and update triggers
  --
  -- GPL_notice(`  --', `2011', `Karl O. Pinc <kop@meme.com>')

  IF TG_OP = 'UPDATE' THEN
    cannot_change(`SWERB_DEPARTS_GPS', `DId')
  END IF;

  -- Can't have GPS information before GPS units were used.
  PERFORM 1
    FROM swerb_departs_data
    WHERE swerb_departs_data.did = NEW.did
          AND swerb_departs_data.date < 'bb_gps_start'
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'Did %: Cannot have SWERB_DEPARTS_GPS rows for SWERB_DEPARTS_DATA.DATEs before bb_gps_start'
                  , NEW.did;
    RETURN NULL;
  END IF;

  -- GPS_UNITS related checks.

  -- Can't have departure data before the particular device was first used.
  PERFORM 1
    FROM swerb_departs_data
       , gps_units
    WHERE swerb_departs_data.did = NEW.did
          AND gps_units.gps = NEW.gps
          AND gps_units.start > swerb_departs_data.date
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'DId %: GPS %: Cannot use a GPS unit before it''s Start date'
                  , NEW.did
                  , NEW.gps;
    RETURN NULL;
  END IF;

  -- Can't have departure data after the particular device was last used.
  PERFORM 1
    FROM swerb_departs_data
       , gps_units
    WHERE swerb_departs_data.did = NEW.did
          AND gps_units.gps = NEW.gps
          AND gps_units.finish < swerb_departs_data.date
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'DId %: GPS %: Cannot use a GPS unit after it''s Finish date'
                  , NEW.did
                  , NEW.gps;
    RETURN NULL;
  END IF;

  -- Can't have Accuracy readings when the GPS unit reports it's error
  -- as a PDOP.
  IF NEW.accuracy IS NOT NULL THEN
    PERFORM 1
      FROM gps_units
      WHERE gps_units.gps = NEW.gps
            AND gps_units.errortype = 'bb_gpsu_pdop'
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION 'DId %: GPS %: Accuracy %: Cannot have a (non-NULL) Accuracy value when the GPS unit does not report error as an accuracy'
                    , NEW.did
                    , NEW.gps
                    , NEW.accuracy;
      RETURN NULL;
    END IF;
  END IF;

  -- Can't have PDOP readings when the GPS unit reports it's error
  -- as an accuracy.
  IF NEW.pdop IS NOT NULL THEN
    PERFORM 1
      FROM gps_units
      WHERE gps_units.gps = NEW.gps
            AND gps_units.errortype = 'bb_gpsu_accuracy'
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION 'DId %: GPS %: PDOP %: Cannot have a (non-NULL) PDOP value when the GPS unit does not report error as a PDOP'
                   , NEW.did
                   , NEW.gps
                   , NEW.pdop;
      RETURN NULL;
    END IF;
  END IF;

  -- Other checks.

  -- Can't have GPS data without a time.
  PERFORM 1
    FROM swerb_departs_data
    WHERE swerb_departs_data.did = NEW.did
          AND swerb_departs_data.time IS NULL
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'Did %: GPS %: Cannot have GPS data without a SWERB_DEPARTS_DATA.Time'
                  , NEW.did
                  , NEW.gps;
    RETURN NULL;
  END IF;

  -- PDOP values must be NULL for data collected
  -- before bb_earliest_pdop or after bb_latest_pdop.
  IF NEW.pdop IS NOT NULL THEN
    PERFORM 1
      FROM swerb_departs_data
      WHERE swerb_departs_data.did = NEW.did
            AND (swerb_departs_data.date < 'bb_earliest_pdop'
                 OR swerb_departs_data.date > 'bb_latest_pdop')
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION 'DId %: PDOP %: PDOP values must be NULL before bb_earliest_pdop and after bb_latest_pdop'
                    , NEW.did
                    , NEW.pdop;
      RETURN NULL;
    END IF;
  END IF;

  -- Accuracy values must be NULL for data collected before
  -- bb_earliest_accuracy.
  IF NEW.accuracy IS NOT NULL then
    PERFORM 1
      FROM swerb_departs_data
      WHERE swerb_departs_data.did = NEW.did
            AND swerb_departs_data.date < 'bb_earliest_accuracy'
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION 'DId %: Accuracy %: Accuracy values must be NULL before bb_earliest_accuracy'
                    , NEW.did
                    , NEW.accuracy;
      RETURN NULL;
    END IF;
  END IF;

  -- Can't have sub-meter accuracy before such was in the protocol.
  IF NEW.altitude <> TRUNC(NEW.altitude) THEN
    PERFORM 1
      FROM swerb_departs_data
      WHERE swerb_departs_data.did = NEW.did
            AND swerb_departs_data.date < 'bb_subm_alt_date'
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION 'Did %: Altitude %: Cannot have sub-meter altitude accuracy before bb_subm_alt_date'
                    , NEW.did
                    , NEW.altitude;
      RETURN NULL;
    END IF;
  END IF;

  -- The Garmincode must be NULL before bb_subm_alt_date and non-NULL
  -- on and after that date.
  PERFORM 1
    FROM swerb_departs_data
    WHERE swerb_departs_data.did = NEW.did
          AND ((swerb_departs_data.date < 'bb_subm_alt_date'
                AND NEW.garmincode IS NOT NULL)
                OR (swerb_departs_data.date >= 'bb_subm_alt_date'
                    AND NEW.garmincode IS NULL))
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'DId %: Garmincode %: The Garmincode must be NULL before bb_subm_alt_date and non-NULL on and after that date'
                  , NEW.did
                  , NEW.garmincode;
    RETURN NULL;
  END IF;

  RETURN NULL;
  END;
$$;


SELECT 'swerb_departs_gps_trigger' AS trigger;
CREATE TRIGGER swerb_departs_gps_trigger
  AFTER INSERT OR UPDATE
  ON swerb_departs_gps FOR EACH ROW
  EXECUTE PROCEDURE swerb_departs_gps_func();
