dnl Copyright (C) 2012, 2017 The Meme Factory, Inc., http://www.meme.com/
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl


dnl plpgsql fragment for BLAH
dnl
dnl Syntax: BLAH(ARG)
dnl
dnl ARG   description
dnl
changequote({,})
define({BLAH},{
})dnl
changequote(`,')dnl

--  
-- swerb_gw_loc_data
--
SELECT 'swerb_gw_loc_data' AS table;

SELECT 'swerb_gw_loc_data_func' AS function;
CREATE FUNCTION swerb_gw_loc_data_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  BEGIN
  -- Function for swerb_gw_loc_data insert and update triggers
  --
  -- GPL_notice(`  --', `2012, 2017',
                `The Meme Factory, Inc., http://www.meme.com/')

  -- Cannot locate a place before it exists.
  PERFORM 1
    FROM swerb_gws
    WHERE swerb_gws.loc = NEW.loc
          AND NEW.date < swerb_gws.start
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'SGWLid %: Date %: Time %: Quad %: x loc %: y loc %: Notes %: Cannot locate a place (a SWERB_GWS row) before the place exists (SWERB_GW_LOC_DATA.Date must be >= SWERB_GWS.Start)'
                  , NEW.sgwlid
                  , NEW.date
                  , NEW.time
                  , NEW.quad
                  , ST_X(NEW.xyloc)
                  , ST_Y(NEW.xyloc)
                  , NEW.notes;
    RETURN NULL;
  END IF;

  -- Cannot locate a place after it ceases to exist.
  PERFORM 1
    FROM swerb_gws
    WHERE swerb_gws.loc = NEW.loc
          AND NEW.date > swerb_gws.finish
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'SGWLid %: Date %: Time %: Quad %: x loc %: y loc %: Notes %: Cannot locate a place (a SWERB_GWS row) after the place ceases to exist (SWERB_GW_LOC_DATA.Date must be <= SWERB_GWS.Finish)'
                  , NEW.sgwlid
                  , NEW.date
                  , NEW.time
                  , NEW.quad
                  , ST_X(NEW.xyloc)
                  , ST_Y(NEW.xyloc)
                  , NEW.notes;
    RETURN NULL;
  END IF;

  IF NEW.gps IS NOT NULL THEN

    -- The date of the gps reading cannot be before the gps unit
    -- went into service.
    PERFORM 1
      FROM gps_units
      WHERE gps_units.gps = NEW.gps
            AND gps_units.start > NEW.date
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION 'SGWLid %: Date %: Time %: GPS %: x loc %: y loc %: Notes %: Cannot use a gps unit before it goes into service, GPS_UNITS.start must be <= SWERB_GW_LOC_DATA.date'
                    , NEW.sgwlid
                    , NEW.date
                    , NEW.time
                    , NEW.gps
                    , ST_X(NEW.xyloc)
                    , ST_Y(NEW.xyloc)
                    , NEW.notes;
      RETURN NULL;
    END IF;

    -- The date of the gps reading cannot be after the gps unit
    -- was removed from service.
    PERFORM 1
      FROM gps_units
      WHERE gps_units.gps = NEW.gps
            AND gps_units.finish < NEW.date
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION 'SGWLid %: Date %: Time %: GPS %: x loc %: y loc %: Notes %: Cannot use a gps unit after it was retired from service, GPS_UNITS.finish must be >= SWERB_GW_LOC_DATA.date'
                    , NEW.sgwlid
                    , NEW.date
                    , NEW.time
                    , NEW.gps
                    , ST_X(NEW.xyloc)
                    , ST_Y(NEW.xyloc)
                    , NEW.notes;
      RETURN NULL;
    END IF;

    -- Can't have Accuracy readings when the GPS unit reports it's error
    -- as a PDOP.
    IF NEW.accuracy IS NOT NULL THEN
      PERFORM 1
        FROM gps_units
        WHERE gps_units.gps = NEW.gps
              AND gps_units.errortype = 'bb_gpsu_pdop'
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION integrity_constraint_violation USING
              MESSAGE = 'Error on ' || TG_OP || ' of SWERB_GW_LOC_DATA'
            , DETAIL = 'Key (SGWLId) = (' || NEW.sgwlid
                         || '): Value (Date) = (' || NEW.date
                         || '): Value (Time) = (' || NEW.time
                         || '): Value (GPS) = (' || NEW.gps
                         || '): Value (ST_X(xyloc)) = (' || ST_X(NEW.xyloc)
                         || '): Value (ST_Y(xyloc)) = (' || ST_Y(NEW.xyloc)
                         || '): Value (Notes) = (' || NEW.notes
                         || '): Value (Accuracy) = (' || NEW.accuracy
                         || '): Value (PDOP) = (' || NEW.pdop
                         || '): Cannot have a (non-NULL) Accuracy value when '
                         || 'the GPS unit does not report error as an '
                         || 'accuracy';
        RETURN NULL;
      END IF;
    END IF;

    -- Can't have PDOP readings when the GPS unit reports it's error
    -- as an accuracy.
    IF NEW.pdop IS NOT NULL THEN
      PERFORM 1
        FROM gps_units
        WHERE gps_units.gps = NEW.gps
              AND gps_units.errortype = 'bb_gpsu_accuracy'
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION 'SGWLid %: Date %: Time %: GPS %: x loc %: y loc %: Notes %: Accuracy %: PDOP %: Cannot have a (non-NULL) PDOP value when the GPS unit does not report error as a PDOP'
                    , NEW.sgwlid
                    , NEW.date
                    , NEW.time
                    , NEW.gps
                    , ST_X(NEW.xyloc)
                    , ST_Y(NEW.xyloc)
                    , NEW.notes
                    , NEW.accuracy
                    , NEW.pdop;
        RETURN NULL;
      END IF;
    END IF;
  END IF;

  RETURN NULL;
  END;
$$;


SELECT 'swerb_gw_loc_data_trigger' AS trigger;
CREATE TRIGGER swerb_gw_loc_data_trigger
  AFTER INSERT OR UPDATE
  ON swerb_gw_loc_data FOR EACH ROW
  EXECUTE PROCEDURE swerb_gw_loc_data_func();
