dnl Copyright (C) 2017 Jake Gordon, <jacob.gordon@duke.edu>
dnl Copyright (C) 2011, 2012 The Meme Factory, Inc., http://www.meme.com/
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl


dnl plpgsql fragment for BLAH
dnl
dnl Syntax: BLAH(ARG)
dnl
dnl ARG   description
dnl
changequote({,})
define({BLAH},{
})dnl
changequote(`,')dnl

--  
-- swerb_gws
--
SELECT 'swerb_gws' AS table;

SELECT 'swerb_gws_func' AS function;
CREATE FUNCTION swerb_gws_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  BEGIN
  -- Function for swerb_gws insert and update triggers
  --
  -- GPL_notice(`  --', `2011, 2012', `The Meme Factory, Inc., http://www.meme.com/')

  IF TG_OP = 'UPDATE' THEN
    -- Cannot change the Loc (the id).  (It's just too annoying to check for.)
    cannot_change(`SWERB_GWS', `Loc')

    IF NEW.loc = 'bb_swerb_unk_grove' THEN
      -- Cannot change the unknown grove row.
      IF (NEW.loc <> OLD.loc
          OR NEW.descr <> OLD.descr)
         AND NOT(has_table_privilege('swerb_loc_statuses', 'trigger')) THEN
        RAISE EXCEPTION 'SWERB_LOC_STATUSES.Loc_Status: bb_swerb_certain: cannot_change_msg';
        RETURN NULL;
      END IF;

      -- No trees may be located in the unknown grove.
      PERFORM 1
        FROM trees
        WHERE trees.loc = 'bb_swerb_unk_grove'
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION 'Old Loc %: New Loc %: Trees may not be located in the unknown grove'
                      , OLD.loc
                      , NEW.loc;
        RETURN NULL;
      END IF;
    END IF;

    IF OLD.type = 'bb_swerb_gws_water'
       AND NEW.type <> 'bb_swerb_gws_water' THEN

      -- SWERB water events must be related to watering holes.
      PERFORM 1
        FROM swerb_loc_data
             JOIN swerb_data ON (swerb_data.swid = swerb_loc_data.swid)
        WHERE swerb_loc_data.loc = NEW.loc
              AND swerb_data.event = 'bb_swerb_water'
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION 'Loc %: Old Type %: New Type %: SWERB_DATA rows with an Event value of bb_swerb_water must be related to SWERB_GWS rows with a Type of bb_swerb_gws_water'
                      , NEW.loc
                      , OLD.type
                      , NEW.type;
        RETURN NULL;
      END IF;

--
-- We allow waterholes to be one of the landmarks recorded at ascent/descent
-- although it's not clear whether this is useful.
--
--    ELSIF OLD.type <> 'bb_swerb_gws_water'
--            AND NEW.type = 'bb_swerb_gws_water' THEN
--
--      -- Waterholes cannot be realted to ascent or descent locations.
--      PERFORM 1
--        FROM swerb_loc_data
--             JOIN adcodes ON (adcodes.adcode = swerb_loc_data.adcode)
--        WHERE swerb_loc_data.loc = NEW.loc
--              AND adcodes.adn <> 'bb_swerb_notad'
--        LIMIT 1;
--      IF FOUND THEN
--        RAISE EXCEPTION 'Loc %: Old Type %: New Type %: Waterholes cannot be related to ascent into or descent from a grove, the related ADCODES.ADN value (via SWERB_LOC_DATA) must be bb_swerb_notad'
--                      , NEW.loc
--                      , OLD.type
--                      , NEW.type;
--        RETURN NULL;
--      END IF;

    ELSIF OLD.type = 'bb_swerb_gws_grove'
          AND NEW.type <> 'bb_swerb_gws_grove' THEN
      -- Used to be a grove and now it's not.  Make sure that
      -- every ascent and descent record is related to a grove.
      PERFORM 1
        FROM swerb_loc_data
             JOIN adcodes ON (adcodes.adcode = swerb_loc_data.adcode)
        WHERE swerb_loc_data.loc = NEW.loc
              AND adcodes.adn <> 'bb_swerb_notad'
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION 'Loc %: Old Type %: New Type %: This location has recorded ascents and/or descents, cannot change type from bb_swerb_gws_grove: SWERB_LOC_DATA rows for ascent and descent must be related to SWERB_GWS rows with a Type of bb_swerb_gws_grove'
                      , NEW.loc
                      , OLD.type
                      , NEW.type;
        RETURN NULL;
      END IF;
    END IF;
  END IF; -- TG_OP = 'UPDATE'

  -- Test for "grove collision" due to the "probable" coding used
  -- during data collection.
  -- Do these tests on both insert and update in case the Type changes.

  IF NEW.type = 'bb_swerb_gws_grove' THEN
    IF UPPER(SUBSTRING(NEW.loc FROM 1 FOR 1)) = 'bb_swerb_probable_char' THEN
      -- Can't create a grove that begins with 'bb_swerb_probable_char' and
      -- the rest of the characters match an existing code.
      PERFORM 1
        FROM swerb_gws
        WHERE UPPER(swerb_gws.loc) = UPPER(SUBSTRING(NEW.loc FROM 2))
              AND swerb_gws.type = 'bb_swerb_gws_grove'
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION 'Loc %: Type %: Cannot create a grove Loc beginning with bb_swerb_probable_char when there is an existing SWERB_GWS.Loc for a grove which is identical except for the leading bb_swerb_probable_char character'
                      , NEW.loc
                      , NEW.type;
        RETURN NULL;
      END IF;

      IF NOT(has_table_privilege('swerb_gws', 'trigger')) THEN
        -- Only the administrator can create codes beginning with
        -- the letter bb_swerb_probable_char.
        RAISE EXCEPTION 'Loc %: Type %: Only the Babase administrator can create Loc codes for groves that begin with the letter bb_swerb_probable_char'
                      , NEW.loc
                      , NEw.type;
        RETURN NULL;
      END IF;
    ELSE
      -- Loc does not begin with 'bb_swerb_probable_char'.
      -- Can't create a grove that's the same as an existing grove that
      -- has it's initial bb_swerb_probable_char removed.
      PERFORM 1
        FROM swerb_gws
        WHERE UPPER(swerb_gws.loc) = 'bb_swerb_probable_char' || UPPER(NEW.loc)
              AND swerb_gws.type = 'bb_swerb_gws_grove'
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION 'Loc %: Type %: Cannot create a Loc value for a grove that is identical to the Loc value of an existing grove except with an initial bb_swerb_probable_char removed (P%)'
                      , NEW.loc
                      , NEW.type
                      , NEW.loc;
        RETURN NULL;
      END IF;
    END IF;
  END IF;

  -- The location must exist before SWERB rows can locate groups in the
  -- location.
  PERFORM 1
    FROM swerb_loc_data, swerb_data, swerb_bes, swerb_departs_data
    WHERE swerb_loc_data.loc = NEW.loc
          AND swerb_data.swid = swerb_loc_data.swid
          AND swerb_bes.beid = swerb_data.beid
          AND swerb_departs_data.did = swerb_bes.beid
          AND swerb_departs_data.date < NEW.start
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'Loc %: Type %: Start %: SWERB_GWS.Start must be after any related SWERB_DEPARTS_DATA.Date values'
                  , NEW.loc
                  , NEW.type
                  , NEW.start;
    RETURN NULL;
  END IF;

  -- The location must exist until after SWERB rows locate groups in the
  -- location.
  PERFORM 1
    FROM swerb_loc_data, swerb_data, swerb_bes, swerb_departs_data
    WHERE swerb_loc_data.loc = NEW.loc
          AND swerb_data.swid = swerb_loc_data.swid
          AND swerb_bes.beid = swerb_data.beid
          AND swerb_departs_data.did = swerb_bes.did
          AND NEW.finish < swerb_departs_data.date
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'Loc %: Type %: Finish %: SWERB_GWS.finish must be NULL or after any related SWERB_DEPARTS_DATA.Date values'
                  , NEW.loc
                  , NEW.type
                  , NEW.finish;
    RETURN NULL;
  END IF;

  -- The place must exist before it can be located on SWERB_GW_LOC_DATA.
  PERFORM 1
    FROM swerb_gw_loc_data
    WHERE swerb_gw_loc_data.loc = NEW.loc
          AND swerb_gw_loc_data.date < NEW.start
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'Loc %: Type %: Start %: SWERB_GWS.start must be before any related SWERB_GW_LOC_DATA.Date values'
                  , NEW.loc
                  , NEW.type
                  , NEW.start;
    RETURN NULL;
  END IF;

  -- The place must still exist when located on SWERB_GW_LOC_DATA.
  PERFORM 1
    FROM swerb_gw_loc_data
    WHERE swerb_gw_loc_data.loc = NEW.loc
          AND swerb_gw_loc_data.date > NEW.finish
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'Loc %: Type %: Finish %: SWERB_GWS.finish must be NULL or after any related SWERB_GW_LOC_DATA.Date values'
                  , NEW.loc
                  , NEW.type
                  , NEW.finish;
    RETURN NULL;
  END IF;

  RETURN NULL;
  END;
$$;


SELECT 'swerb_gws_delete_func' AS function;
CREATE FUNCTION swerb_gws_delete_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- Function for swerb_gws delete trigger
  --
  -- GPL_notice(`  --', `2011', `The Meme Factory, Inc., http://www.meme.com/')

  BEGIN

  IF OLD.loc = 'bb_swerb_unk_grove'
     AND NOT(has_table_privilege('swerb_gws', 'trigger')) THEN
    RAISE EXCEPTION 'SWERB_GWS.Loc: bb_swerb_unk_grove: cannot_delete_msg';
  END IF;

  RETURN NULL;
  END;
$$;

SELECT 'swerb_gws_commit_func' AS function;
CREATE OR REPLACE FUNCTION swerb_gws_commit_func()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  -- Function for swerb_gws insert and update fired upon transaction commit.
  --
  -- GPL_notice(`  --', `2017', `Jake Gordon, <jacob.gordon@duke.edu>')
  --
  BEGIN

  -- Get the latest values of the row
  SELECT * INTO NEW FROM swerb_gws WHERE swerb_gws.loc = NEW.loc;
  IF NOT FOUND THEN
    -- Whatever row was inserted was subsequently deleted.
    -- Nothing to do.
    RETURN NULL;
  END IF;

  -- Make sure all SWERB_LOC_DATA.Loc and
  -- SWERB_LOC_DATA_CONFIDENCES.Nearest_Loc pairs have the same type.
  --
  -- Check this on commit, in case multiple update commands are needed
  -- to change this. It also makes the error-checking WAY easier to
  -- write.
  PERFORM 1
    FROM swerb_loc_data
      JOIN swerb_loc_data_confidences
           ON swerb_loc_data_confidences.swid = swerb_loc_data.swid
      JOIN swerb_gws AS recorded_loc_gws
           ON recorded_loc_gws.loc = swerb_loc_data.loc
      JOIN swerb_gws AS nearest_loc_gws
           ON nearest_loc_gws.loc = swerb_loc_data_confidences.nearest_loc
    WHERE recorded_loc_gws.type <> nearest_loc_gws.type
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION integrity_constraint_violation USING
          MESSAGE = 'Error on SWERB_GWS ' || TG_OP || ' commit'
        , DETAIL = 'Cannot change Type if it causes a '
                     || 'SWERB_LOC_DATA.Loc - SWERB_LOC_DATA_CONFIDENCES'
                     || '.Nearest_Loc pair to have different Types';
    RETURN NULL;
  END IF;

  RETURN NULL;
  END;
$$;

SELECT 'swerb_gws_trigger' AS trigger;
CREATE TRIGGER swerb_gws_trigger
  AFTER INSERT OR UPDATE
  ON swerb_gws FOR EACH ROW
  EXECUTE PROCEDURE swerb_gws_func();

SELECT 'swerb_gws_delete_trigger' AS trigger;
CREATE TRIGGER swerb_gws_delete_trigger
  AFTER DELETE
  ON swerb_gws FOR EACH ROW
  EXECUTE PROCEDURE swerb_gws_delete_func();

SELECT 'swerb_gws_commit_trigger' AS trigger;
CREATE CONSTRAINT TRIGGER swerb_gws_commit_trigger
  AFTER INSERT OR UPDATE
  ON swerb_gws
  DEFERRABLE INITIALLY DEFERRED
  FOR EACH ROW
  EXECUTE PROCEDURE swerb_gws_commit_func();
