dnl Copyright (C) 2011-2013 The Meme Factory, Inc., http://www.meme.com/
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl


dnl plpgsql fragment for BLAH
dnl
dnl Syntax: BLAH(ARG)
dnl
dnl ARG   description
dnl
changequote({,})
define({BLAH},{
})dnl
changequote(`,')dnl

--  
-- swerb_loc_data
--
SELECT 'swerb_loc_data' AS table;

SELECT 'swerb_loc_data_func' AS function;
CREATE FUNCTION swerb_loc_data_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  AS $$
  -- Function for swerb_loc_data insert and update triggers
  --
  -- GPL_notice(`  --', `2011-2013', `The Meme Factory, Inc., http://www.meme.com/')

  DECLARE
    this_event swerb_data.event%TYPE;
    this_subgroup swerb_data.subgroup%TYPE;
    this_beid  swerb_data.beid%TYPE;
    this_adn   adcodes.adn%TYPE;
    this_date  swerb_departs_data.date%TYPE;
    this_time  swerb_data.time%TYPE;

  BEGIN

    IF TG_OP = 'UPDATE' THEN
      cannot_change(`SWERB_LOC_DATA', `SWId')

      -- Make sure that Loc is still the same SWERB_GWS.Type as this
      -- SWId's Nearest_Loc.
      IF NEW.loc <> OLD.loc  THEN
        PERFORM 1
          FROM swerb_loc_data_confidences
               JOIN swerb_gws AS nearest_loc_gws
                     ON nearest_loc_gws.loc = swerb_loc_data_confidences.nearest_loc
               JOIN swerb_gws AS recorded_loc_gws
      	             ON recorded_loc_gws.loc = NEW.loc
           WHERE swerb_loc_data_confidences.swid = NEW.swid
             AND recorded_loc_gws.type <> nearest_loc.type
           LIMIT 1;
         IF FOUND THEN
           RAISE EXCEPTION integrity_constraint_violation USING
                 MESSAGE = 'Error on ' || TG_OP || ' of SWERB_LOC_DATA'
               , DETAIL = 'Key (SWId) = (' || NEW.swid
                            || '): Value (Loc) = (' || NEW.loc
                            || '): Cannot change Loc to a one whose SWERB_GWS.'
                            || 'Type does not match that of this SWId''s '
                            || 'SWERB_LOC_DATA_CONFIDENCES.Nearest_Loc';
           RETURN NULL;
         END IF;
      END IF;


    END IF;

    -- Get the SWERB_DATA row content related to this row.
    SELECT swerb_data.event, swerb_data.beid, swerb_data.subgroup
      INTO this_event,       this_beid,       this_subgroup
      FROM swerb_data
      WHERE swerb_data.swid = NEW.swid;

    -- Get the ADCODES.ADN content related to this row.
    SELECT        adcodes.adn
      INTO STRICT this_adn
      FROM adcodes
      WHERE adcodes.adcode = NEW.adcode;


    IF this_event = 'bb_swerb_water' THEN
      -- The group is at a waterhole.

      -- The SWERB_GW_DATA row must be for a waterhole.
      PERFORM 1
        FROM swerb_gws
        WHERE swerb_gws.loc = NEW.loc
              AND swerb_gws.type <> 'bb_swerb_gws_water'
        LIMIT 1;

      IF FOUND THEN
        RAISE EXCEPTION integrity_constraint_violation USING
              MESSAGE = 'Error on ' || TG_OP || ' of SWERB_LOC_DATA'
            , DETAIL = 'Key (SWId) = (' || NEW.swid
                         || '): Value (Loc) = (' || NEW.loc
                         || '): Value (ADCode) = (' || NEW.adcode
                         || '): Value (Loc_Status) = (' || NEW.loc_status
                         || '): Value (ADTime) = (' || textualize(`NEW.adtime')
                         || '): Value (SWERB_DATA.BEId) = (' || this_beid
                         || '): Value (SWERB_DATA.Event) = (' || this_event
                         || '): SWERB_DATA rows with Event values of '
                         || 'bb_swerb_water must be related to '
                         || 'SWERB_GWS rows with Type values of '
                         || 'bb_swerb_gws_water';
        RETURN NULL;
      END IF;

      -- Drinking means there cannot be a recorded ascent or descent.
      -- (Given the test for ascent and decent below this is redundant.
      -- However, it's cheap to test and the error message is more specific
      -- so maybe it'll be helpful.)
      IF this_adn <> 'bb_swerb_notad' THEN
        RAISE EXCEPTION integrity_constraint_violation USING
              MESSAGE = 'Error on ' || TG_OP || ' of SWERB_LOC_DATA'
            , DETAIL = 'Key (SWId) = (' || NEW.swid
                         || '): Value (Loc) = (' || NEW.loc
                         || '): Value (ADCode) = (' || NEW.adcode
                         || '): Value (Loc_Status) = (' || NEW.loc_status
                         || '): Value (ADTime) = (' || textualize(`NEW.adtime')
                         || '): Value (SWERB_DATA.BEId) = (' || this_beid
                         || '): Value (SWERB_DATA.Event) = (' || this_event
                         || '): SWERB_DATA rows with Event values of '
                         || 'bb_swerb_water cannot be related to '
                         || 'SWERB_LOC_DATA rows unlees the '
                         || 'SWERB_LOC_DATA.ADCode value has a '
                         || 'ADCODES.ADN value of bb_swerb_notad';
        RETURN NULL;
      END IF;
    END IF;

    IF this_event = 'bb_swerb_begin' THEN
      -- A bout of observation of the group is beginning.

      -- Is this the first observation of the group for the day
      -- by the team?
      PERFORM 1
        FROM swerb_bes AS this_sbe
             JOIN swerb_bes AS other_sbe
                  ON (other_sbe.did = this_sbe.did)
        WHERE this_sbe.beid = this_beid
              AND other_sbe.focal_grp = this_sbe.focal_grp
              AND other_sbe.seq < this_sbe.seq
        LIMIT 1;
      IF NOT FOUND THEN
        -- This is the first observation of the group by the team for
        -- the day.

        IF this_adn <> 'bb_swerb_descent' THEN
          PERFORM 1
            FROM swerb_bes
            WHERE swerb_bes.beid = this_beid
                  AND swerb_bes.focal_grp = 'bb_unk_grp'
            LIMIT 1;
          IF NOT FOUND THEN
            -- The observation team's first observation of the
            -- group for the day, excpting the unknown group,
            -- must be of descent from the sleeping grove.
            RAISE EXCEPTION integrity_constraint_violation USING
              MESSAGE = 'Error on ' || TG_OP || ' of SWERB_LOC_DATA'
            , DETAIL = 'Key (SWId) = (' || NEW.swid
                         || '): Value (Loc) = (' || NEW.loc
                         || '): Value (ADCode) = (' || NEW.adcode
                         || '): Value (Loc_Status) = (' || NEW.loc_status
                         || '): Value (ADTime) = (' || textualize(`NEW.adtime')
                         || '): Value (SWERB_DATA.BEId) = (' || this_beid
                         || '): Value (SWERB_DATA.Event) = (' || this_event
                         || '): Unless the focal group is the unknown '
                         || 'group (bb_unk_grp) the beginning of an '
                         || 'observation team''s observation of a group '
                         || 'for a day must be related to a '
                         || 'SWERB_LOC_DATA row with an ADcode value '
                         || 'where ADCODES.ADcode = '
                         || '''bb_swerb_descent''';
            RETURN NULL;
          END IF;
        END IF;

      ELSE
        -- Not the team's first observation of the day.
        IF this_adn = 'bb_swerb_descent'
           AND NOT(this_subgroup AND NEW.loc = 'bb_swerb_unk_grove') THEN
          -- Only the observation team's first observation of the
          -- group for the day can be of descent from the sleeping grove,
          -- ignoring subgroup descent from the unknown grove,
          -- unless all are subgroups.

          PERFORM 1
            FROM swerb_bes AS this_sbe
                 JOIN swerb_bes AS other_sbe ON (other_sbe.did = this_sbe.did)
                 JOIN swerb_data ON (swerb_data.beid = other_sbe.beid)
                 JOIN swerb_loc_data ON (swerb_loc_data.swid = swerb_data.swid)
                 JOIN adcodes ON (adcodes.adcode = swerb_loc_data.adcode)
            WHERE this_sbe.beid = this_beid
                  AND other_sbe.focal_grp = this_sbe.focal_grp
                  AND NOT(swerb_data.subgroup)
                  AND adcodes.adn = 'bb_swerb_descent'
            LIMIT 1;

          IF FOUND THEN

            -- Get the SWERB_DATA row content related to this row.
            -- (It's convenient to have in a variable in case it's NULL.)
            SELECT swerb_data.time
              INTO this_time
              FROM swerb_data
              WHERE swerb_data.swid = NEW.swid;

            RAISE EXCEPTION integrity_constraint_violation USING
                  MESSAGE = 'Error on ' || TG_OP || ' of SWERB_LOC_DATA'
                , DETAIL = 'Key (SWId) = (' || NEW.swid
                           || '): Value (Loc) = (' || NEW.loc
                           || '): Value (ADCode) = (' || NEW.adcode
                           || '): Value (Loc_Status) = (' || NEW.loc_status
                           || '): Value (ADTime) = ('
                           || textualize(`NEW.adtime')
                           || '): Value (SWERB_DATA.BEId) = (' || this_beid
                           || '): Value (SWERB_DATA.Event) = (' || this_event
                           || '): Value (SWERB_DATA.Time) = ('
                           || textualize(`this_time')
                           || '): Value (SWERB_DEPARTS_DATA.Date) = ('
                           || (SELECT swerb_departs_data.date
                                 FROM swerb_bes
                                        JOIN swerb_departs_data
                                             ON (swerb_departs_data.did
                                                 = swerb_bes.did)
                                 WHERE swerb_bes.beid = this_beid
                                 LIMIT 1)

                           || '): Only the observation team''s daily '
                           || 'beginning of observation of a group can '
                           || 'be related to a SWERB_LOC_DATA row with '
                           || 'an ADcode value where ADCODES.ADcode = '
                           || '''bb_swerb_descent'' unless all '
                           || 'descents, ignoring those that are both '
                           || 'subgroups and descents from the unknown '
                           || 'grove, are of subgroups';
            RETURN NULL;
          END IF;
        END IF;
      END IF;
    END IF;

    IF this_adn = 'bb_swerb_ascent'
       OR this_adn = 'bb_swerb_descent' THEN
      -- Ascents and descents must locate the group at a sleeping grove.

      PERFORM 1
        FROM swerb_gws
        WHERE swerb_gws.loc = NEW.loc
              AND swerb_gws.type = 'bb_swerb_gws_grove'
        LIMIT 1;
      IF NOT FOUND THEN
        RAISE EXCEPTION integrity_constraint_violation USING
              MESSAGE = 'Error on ' || TG_OP || ' of SWERB_LOC_DATA'
            , DETAIL = 'Key (SWId) = (' || NEW.swid
                         || '): Value (Loc) = (' || NEW.loc
                         || '): Value (ADCode) = (' || NEW.adcode
                         || '): Value (Loc_Status) = (' || NEW.loc_status
                         || '): Value (ADTime) = (' || textualize(`NEW.adtime')
                         || '): Value (SWERB_DATA.BEId) = (' || this_beid
                         || '): Value (SWERB_DATA.Event) = (' || this_event
                         || '): Ascents into sleeping groves and '
                         || 'descents from sleeping groves must be '
                         || 'related to a SWERB_GWS row with a Type '
                         || 'value of bb_swerb_gws_grove';
        RETURN NULL;
      END IF;
    END IF;

    -- Get the date associated with this observation.

    SELECT INTO STRICT this_date swerb_departs_data.date
      FROM swerb_data
           JOIN swerb_bes ON (swerb_bes.beid = swerb_data.beid)
           JOIN swerb_departs_data ON (swerb_departs_data.did = swerb_bes.did)
      WHERE swerb_data.swid = NEW.swid;

    -- Cannot be located at a place after observations at that place ended.
    PERFORM 1
      FROM swerb_gws
      WHERE swerb_gws.loc = NEW.loc
            AND swerb_gws.finish < this_date
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION integrity_constraint_violation USING
              MESSAGE = 'Error on ' || TG_OP || ' of SWERB_LOC_DATA'
            , DETAIL = 'Key (SWId) = (' || NEW.swid
                         || '): Value (Loc) = (' || NEW.loc
                         || '): Value (ADCode) = (' || NEW.adcode
                         || '): Value (Loc_Status) = (' || NEW.loc_status
                         || '): Value (ADTime) = (' || textualize(`NEW.adtime')
                         || '): Value (SWERB_DATA.BEId) = (' || this_beid
                         || '): Value (SWERB_DATA.Event) = (' || this_event
                         || '): Value (SWERB_DEPARTS_DATA.Date) = (' || this_date
                         || '): SWERB_LOC_DATA cannot place a group at '
                         || 'a location after the location ceases to '
                         || 'exist, after the SWERB_GWS.Finish date';
      RETURN NULL;
    END IF;

    -- Cannot be located at a place before observations at that place began.
    PERFORM 1
      FROM swerb_gws
      WHERE swerb_gws.loc = NEW.loc
            AND this_date < swerb_gws.start
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION integrity_constraint_violation USING
              MESSAGE = 'Error on ' || TG_OP || ' of SWERB_LOC_DATA'
            , DETAIL = 'Key (SWId) = (' || NEW.swid
                         || '): Value (Loc) = (' || NEW.loc
                         || '): Value (ADCode) = (' || NEW.adcode
                         || '): Value (Loc_Status) = (' || NEW.loc_status
                         || '): Value (ADTime) = (' || textualize(`NEW.adtime')
                         || '): Value (SWERB_DATA.BEId) = (' || this_beid
                         || '): Value (SWERB_DATA.Event) = (' || this_event
                         || '): Value (SWERB_DEPARTS_DATA.Date) = (' || this_date
                         || '): SWERB_LOC_DATA cannot place a group at '
                         || 'a location before the location exists, '
                         || 'before the SWERB_GWS.Start date';
      RETURN NULL;
    END IF;

    RETURN NULL;
  END;
$$;


SELECT 'swerb_loc_data_commit_func' AS function;
CREATE OR REPLACE FUNCTION swerb_loc_data_commit_func()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  -- Function for swerb_loc_data insert and update operations fired
  -- upon transaction commit.
  --
  -- GPL_notice(`  --', `2012', `The Meme Factory, Inc.  http://www.meme.com/')
  --
  -- Check ADTime values.
  DECLARE
    this_beid  swerb_data.beid%TYPE;
    this_event swerb_data.event%TYPE;
    this_time  swerb_data.time %TYPE;
    this_subgroup swerb_data.subgroup%TYPE;
    this_adn   adcodes.adn%TYPE;
    this_adcode_time adcodes.time%TYPE;

  BEGIN

    -- Get the latest values of the row
    SELECT * INTO NEW FROM swerb_loc_data WHERE swerb_loc_data.swid = NEW.swid;
    IF NOT FOUND THEN
      -- Whatever row was inserted was subsequently deleted.
      -- Nothing to do.
      RETURN NULL;
    END IF;

    -- Get the ADCODES.ADN content related to this row.
    SELECT        adcodes.adn, adcodes.time
      INTO STRICT this_adn,    this_adcode_time
      FROM adcodes
      WHERE adcodes.adcode = NEW.adcode;

    -- Get the SWERB_DATA row content related to this row.
    SELECT swerb_data.event, swerb_data.beid, swerb_data.time, swerb_data.subgroup
      INTO this_event,       this_beid,       this_time,       this_subgroup
      FROM swerb_data
      WHERE swerb_data.swid = NEW.swid;

    IF this_event = 'bb_swerb_end' THEN
      -- A bout of observation is ending.

      -- Is this the last observation of the group of the day?
      PERFORM 1
        FROM swerb_bes AS this_sbe
             JOIN swerb_bes AS other_sbe
                  ON (other_sbe.did = this_sbe.did)
        WHERE this_sbe.beid = this_beid
              AND other_sbe.focal_grp = this_sbe.focal_grp
              AND other_sbe.stop > this_sbe.stop
       LIMIT 1;
      IF NOT FOUND THEN
        -- This meets the criteria for being the team's last bout of
        -- observation for the day; either (one of) the largest 
        -- SWERB_BES.Stop values or NULL.

        PERFORM 1
          FROM swerb_bes
          WHERE swerb_bes.beid = this_beid
                AND swerb_bes.focal_grp = 'bb_unk_grp'
          LIMIT 1;
        IF NOT FOUND THEN
          -- The unknown group is exempted from having to have
          -- a sleeping grove at the end of the day.
          -- But here the focal is a regular group.

          -- Could be the last observation of the day.
          -- One of the rows meeting the "last observation of the
          -- day" criteria must be an ascent.
          PERFORM 1
            FROM swerb_bes AS this_sbe
                 JOIN swerb_bes AS other_sbe
                      ON (other_sbe.did = this_sbe.did)
                 JOIN swerb_data
                      ON (swerb_data.beid = other_sbe.beid)
                 JOIN swerb_loc_data
                      ON (swerb_loc_data.swid = swerb_data.swid)
                 JOIN adcodes
                      ON (adcodes.adcode = swerb_loc_data.adcode)
            WHERE this_sbe.beid = this_beid
                  AND other_sbe.focal_grp = this_sbe.focal_grp
                  AND (other_sbe.stop = this_sbe.stop
                       OR other_sbe.stop IS NULL
                       OR this_sbe.stop IS NULL)
                  AND adcodes.adn = 'bb_swerb_ascent'
            LIMIT 1;

          IF NOT FOUND THEN
            -- One of the team's last observations of the group for the
            -- day must be of ascent into the sleeping grove.
            -- (This is the check that must not happen until transaction
            -- commit because otherwise the ascent can't always be last
            -- when uploaded.)

            RAISE EXCEPTION integrity_constraint_violation USING
                  MESSAGE = 'Error on commit after ' || TG_OP
                            || ' of SWERB_LOC_DATA'
                , DETAIL = 'Key (SWId) = (' || NEW.swid
                           || '): Value (Loc) = (' || NEW.loc
                           || '): Value (ADCode) = (' || NEW.adcode
                           || '): Value (Loc_Status) = (' || NEW.loc_status
                           || '): Value (ADTime) = ('
                           || textualize(`NEW.adtime')
                           || '): Value (SWERB_DATA.BEId) = (' || this_beid
                           || '): Value (SWERB_DATA.Event) = (' || this_event
                           || '): Value (SWERB_DATA.Time) = ('
                           || textualize(`this_time')
                           || '): Value (SWERB_DEPARTS_DATA.Date) = ('
                           || (SELECT swerb_departs_data.date
                                 FROM swerb_bes
                                        JOIN swerb_departs_data
                                             ON (swerb_departs_data.did
                                                 = swerb_bes.did)
                                 WHERE swerb_bes.beid = this_beid
                                 LIMIT 1)

                           || '): Unless the focal group is the '
                           || 'unknown group (bb_unk_grp) the '
                           || 'observation team''s daily end of '
                           || 'observation of a group must be related '
                           || 'to a SWERB_LOC_DATA row with an ADcode '
                           || 'value where ADCODES.ADcode = '
                           || '''bb_swerb_ascent''';

            RETURN NULL;
          END IF;
        END IF;

        -- If more than one ascent exists then they all (ignoring
        -- subgroups that ascend into the unknown grove) must be
        -- of subgroups.
        DECLARE
          c INT;
          this_did swerb_bes.did%TYPE;
          this_focal_grp swerb_bes.focal_grp%TYPE;

        BEGIN

          -- Get the did of the obs team's departure.
          SELECT        swerb_bes.did, swerb_bes.focal_grp
            INTO STRICT this_did,      this_focal_grp
            FROM swerb_bes
            WHERE swerb_bes.beid = this_beid;

          -- How many ascents (ignoring subgroups that ascend
          -- into the unknown grove) are there for
          -- the group/obs team/day?
          SELECT COUNT(*)
            INTO c
            FROM swerb_bes
                 JOIN swerb_data
                      ON (swerb_data.beid = swerb_bes.beid)
                 JOIN swerb_loc_data
                      ON (swerb_loc_data.swid = swerb_data.swid)
                 JOIN adcodes
                      ON (adcodes.adcode = swerb_loc_data.adcode)
            WHERE swerb_bes.did = this_did
                  AND swerb_bes.focal_grp = this_focal_grp
                  AND NOT (swerb_data.subgroup
                           AND swerb_loc_data.loc = 'bb_swerb_unk_grove')
                  AND adcodes.adn = 'bb_swerb_ascent';

          IF c > 1 THEN
            -- All the observations must be of subgroups.
            PERFORM 1
              FROM swerb_bes
                   JOIN swerb_data
                        ON (swerb_data.beid = swerb_bes.beid)
                   JOIN swerb_loc_data
                        ON (swerb_loc_data.swid = swerb_data.swid)
                   JOIN adcodes
                        ON (adcodes.adcode = swerb_loc_data.adcode)
              WHERE swerb_bes.did = this_did
                    AND swerb_bes.focal_grp = this_focal_grp
                    AND NOT(subgroup)
                    AND adcodes.adn = 'bb_swerb_ascent'
              LIMIT 1;

            IF FOUND THEN
              -- We have more than one ascent but they
              -- are not all subgroups.
              RAISE EXCEPTION integrity_constraint_violation USING
                  MESSAGE = 'Error on commit after ' || TG_OP
                            || ' of SWERB_LOC_DATA'
                , DETAIL = 'Key (SWId) = (' || NEW.swid
                           || '): Value (SWERB_BES.DId) = (' || this_did
                           || '): Value (SWERB_BES.Focal_Grp) = (' || this_focal_grp
                           || '): Value (Loc) = (' || NEW.loc
                           || '): Value (ADCode) = (' || NEW.adcode
                           || '): Value (Loc_Status) = (' || NEW.loc_status
                           || '): Value (ADTime) = (' || textualize(`NEW.adtime')
                           || '): Value (SWERB_DATA.BEId) = (' || this_beid
                           || '): Value (SWERB_DATA.Event) = (' || this_event
                           || '): The observation team''s daily end of '
                           || 'observation of a group must be related '
                           || 'to a single SWERB_LOC_DATA row with an '
                           || 'ADcode value where ADCODES.ADcode = '
                           || '''bb_swerb_ascent'', irnoring those that '
                           || 'are both subgroups and ascents into the '
                           || 'unknown grove, unless all of the ascents '
                           || 'are of subgroups';
              RETURN NULL;
            END IF;
          END IF;
        END;
      ELSE
        -- Not the team's last observation of the day.
        IF this_adn = 'bb_swerb_ascent'
           AND NOT(this_subgroup AND NEW.loc = 'bb_swerb_unk_grove') THEN
          -- Only the team's last observation of the group for the day
          -- can be of ascent into the sleeping grove, ignoring those
          -- that are both subgroups and ascents into the unknown
          -- grove, unless all of the ascents are of subgroups.
          PERFORM 1
            FROM swerb_bes AS this_sbe
                 JOIN swerb_bes AS other_sbe ON (other_sbe.did = this_sbe.did)
                 JOIN swerb_data ON (swerb_data.beid = other_sbe.beid)
                 JOIN swerb_loc_data ON (swerb_loc_data.swid = swerb_data.swid)
                 JOIN adcodes ON (adcodes.adcode = swerb_loc_data.adcode)
            WHERE this_sbe.beid = this_beid
                  AND other_sbe.focal_grp = this_sbe.focal_grp
                  AND NOT(swerb_data.subgroup)
                  AND adcodes.adn = 'bb_swerb_asscent'
            LIMIT 1;

          IF FOUND THEN
            RAISE EXCEPTION integrity_constraint_violation USING
                  MESSAGE = 'Error on commit after ' || TG_OP
                            || ' of SWERB_LOC_DATA'
                , DETAIL = 'Key (SWId) = (' || NEW.swid
                           || '): Value (Loc) = (' || NEW.loc
                           || '): Value (ADCode) = (' || NEW.adcode
                           || '): Value (Loc_Status) = (' || NEW.loc_status
                           || '): Value (ADTime) = (' || textualize(`NEW.adtime')
                           || '): Value (SWERB_DATA.BEId) = (' || this_beid
                           || '): Value (SWERB_DATA.Event) = (' || this_event
                           || '): Only the observation team''s daily '
                           || 'end of observation of a group can be '
                           || 'related to a SWERB_LOC_DATA row with an '
                           || 'ADcode value where ADCODES.ADcode = '
                           || '''bb_swerb_ascent'', ignoring those that '
                           || 'are both subgroups and ascent into the '
                           || 'unknown grove, unless all of the ascents '
                           || 'are of subgroups';
            RETURN NULL;
          END IF;
        END IF;
      END IF;
    END IF;

    -- The ADTime value must be NULL when ADCODES says so.
    -- (Check whether the ascent/descent itself is valid before
    -- checking A/D times.)

    IF this_adcode_time THEN
      -- The ADTime value must be non-NULL.
      IF NEW.adtime IS NULL THEN
        RAISE EXCEPTION integrity_constraint_violation USING
              MESSAGE = 'Error on commit after ' || TG_OP
                        || ' of SWERB_LOC_DATA'
            , DETAIL = 'Key (SWId) = (' || NEW.swid
                       || '): Value (Loc) = (' || NEW.loc
                       || '): Value (ADCode) = (' || NEW.adcode
                       || '): Value (Loc_Status) = (' || NEW.loc_status
                       || '): Value (ADTime) = ('
                       || textualize(`NEW.adtime')
                       || '): Value (SWERB_DATA.BEId) = (' || this_beid
                       || '): Value (SWERB_DATA.Event) = (' || this_event
                       || '): Value (SWERB_DATA.Time) = ('
                       || textualize(`this_time')
                       || '): Value (SWERB_DEPARTS_DATA.Date) = ('
                       || (SELECT swerb_departs_data.date
                             FROM swerb_bes
                                    JOIN swerb_departs_data
                                         ON (swerb_departs_data.did
                                             = swerb_bes.did)
                             WHERE swerb_bes.beid = this_beid
                             LIMIT 1)
                       || '): SWERB_LOC_DATA rows with this ADCode value must have non-NULL ADTime values'
            , HINT = 'Is the 2nd begin or end row missing from the GPS data?';
        RETURN NULL;
      END IF;
    ELSE
      -- this_adcode_time = FALSE
      IF NEW.adtime IS NOT NULL THEN
        RAISE EXCEPTION integrity_constraint_violation USING
              MESSAGE = 'Error on commit after ' || TG_OP
                        || ' of SWERB_LOC_DATA'
            , DETAIL = 'Key (SWId) = (' || NEW.swid
                       || '): Value (Loc) = (' || NEW.loc
                       || '): Value (ADCode) = (' || NEW.adcode
                       || '): Value (Loc_Status) = (' || NEW.loc_status
                       || '): Value (ADTime) = ('
                       || textualize(`NEW.adtime')
                       || '): Value (SWERB_DATA.BEId) = (' || this_beid
                       || '): Value (SWERB_DATA.Event) = (' || this_event
                       || '): Value (SWERB_DATA.Time) = ('
                       || textualize(`this_time')
                       || '): Value (SWERB_DEPARTS_DATA.Date) = ('
                       || (SELECT swerb_departs_data.date
                             FROM swerb_bes
                                    JOIN swerb_departs_data
                                         ON (swerb_departs_data.did
                                             = swerb_bes.did)
                             WHERE swerb_bes.beid = this_beid
                             LIMIT 1)
                       || '): SWERB_LOC_DATA rows with this ADCode '
                       || 'value must have NULL ADTime values';
        RETURN NULL;
      END IF;
    END IF;

    IF this_adn <> 'bb_swerb_notad' THEN
      -- Ascending into or descending from a sleeping grove.

      -- The ADTime cannot be earlier the beginning of the bout of observation.
      PERFORM 1
        FROM swerb_data
             JOIN swerb_bes ON (swerb_bes.beid = swerb_data.beid)
        WHERE swerb_data.swid = NEW.swid
              AND NEW.adtime < swerb_bes.start
        LIMIT 1;
      IF FOUND THEN
        RAISE EXCEPTION integrity_constraint_violation USING
              MESSAGE = 'Error on commit after ' || TG_OP
                        || ' of SWERB_LOC_DATA'
            , DETAIL = 'Key (SWId) = (' || NEW.swid
                       || '): Value (Loc) = (' || NEW.loc
                       || '): Value (ADCode) = (' || NEW.adcode
                       || '): Value (Loc_Status) = (' || NEW.loc_status
                       || '): Value (ADTime) = ('|| textualize(`NEW.adtime')
                       || '): ADtime cannot be earlier than the '
                       || 'beginning of the bout of observation, '
                       || 'earlier than the related SWERB_BES.Start';
        RETURN NULL;
      END IF;
    END IF;

    RETURN NULL;
  END;
$$;


SELECT 'swerb_loc_data_delete_commit_func' AS function;
CREATE OR REPLACE FUNCTION swerb_loc_data_delete_commit_func()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  -- Function for swerb_loc_data delete operations fired upon transaction commit.
  --
  -- GPL_notice(`  --', `2012', `The Meme Factory, Inc.  http://www.meme.com/')
  --
  -- Check that every days observeration a group begins and ends
  -- with a descent from/ascent into a sleeping grove.
  DECLARE
    this_adn adcodes.adn%TYPE;
    this_did swerb_bes.did%TYPE;
    this_focal_grp swerb_bes.focal_grp%TYPE;

  BEGIN

  SELECT INTO STRICT this_adn adcodes.adn
    FROM adcodes
    WHERE adcodes.adcode = OLD.adcode;

  IF this_adn = 'bb_swerb_notad' THEN
    -- The deleted row does not involve ascent or decent.

    -- Check to be sure that drinking events always have
    -- a related waterhole of some sort.
    PERFORM 1
      FROM swerb_data
      WHERE swerb_data.swid = OLD.swid
            AND swerb_data.event = 'bb_swerb_water'
      LIMIT 1;

    IF FOUND THEN
      -- This is a drinking event.
      -- There must be a related waterhole.
      RAISE EXCEPTION integrity_constraint_violation USING
            MESSAGE = 'Error on commit after ' || TG_OP
                      || ' from SWERB_LOC_DATA'
          , DETAIL = 'Key (SWId) = (' || OLD.swid
                      || '): Value (Loc) = (' || OLD.loc
                      || '): Value (ADCode) = (' || OLD.adcode
                      || '): SWERB_DATA rows with Event values of '
                      || 'bb_swerb_water must be related to '
                      || 'SWERB_LOC_DATA rows which place the drinking '
                      || 'event at a SWERB_GWS which represents a '
                      || 'waterhole -- has a Type of '
                      || 'bb_swerb_gws_water';
      RETURN NULL;
    END IF;

    -- Nothing more to do.
    RETURN NULL;
  END IF;

  -- Get the focal group and did.
  SELECT INTO this_focal_grp,      this_did
              swerb_bes.focal_grp, swerb_bes.did
    FROM swerb_data
         JOIN swerb_bes ON (swerb_bes.beid = swerb_data.beid)
    WHERE swerb_data.swid = OLD.swid
    LIMIT 1;
  -- Are there still rows for this group for this day?
  IF NOT FOUND THEN
    -- There might be rows left, but we can't check for them.
    -- Leave the work to other triggers.
    -- Nothing to do.
    RETURN NULL;
  END IF;

  IF this_focal_grp = 'bb_unk_grp' THEN
    -- The unknown group is exempt from sleeping in groves.
    -- Nothing to do.
    RETURN NULL;
  END IF;

  IF this_adn = 'bb_swerb_descent' THEN
    -- Every observation team's daily observation of a group must
    -- begin with a descent from a sleeping grove.

    DECLARE
      begin_beid swerb_bes.beid%TYPE;

    BEGIN
      -- Find the beid the observation team started observation of the
      -- group for the day.

      SELECT        swerb_bes.beid
        INTO STRICT begin_beid
        FROM swerb_bes
        WHERE swerb_bes.did = this_did
              AND swerb_bes.focal_grp = this_focal_grp
        ORDER BY swerb_bes.seq
        LIMIT 1;

      -- Be sure the first observation has a descent.
      PERFORM 1
        FROM swerb_data
             JOIN swerb_loc_data ON (swerb_loc_data.swid = swerb_data.swid)
             JOIN adcodes ON (adcodes.adn = swerb_loc_data.adcode)
        WHERE swerb_data.beid = begin_beid
              AND adcodes.adn = 'bb_swerb_descent'
        LIMIT 1;
      IF NOT FOUND THEN
        RAISE EXCEPTION integrity_constraint_violation USING
              MESSAGE = 'Error on commit after ' || TG_OP
                         || ' from SWERB_LOC_DATA'
            , DETAIL = 'Key (SWId) = (' || OLD.swid
                         || '): Value (Loc) = (' || OLD.loc
                         || '): Value (ADCode) = (' || OLD.adcode
                         || '): Value (SWERB_BES.DId) = (' || this_did
                         || '): Value (SWERB_BES.Focal_grp) = (' || this_focal_grp
                         || '): Cannot delete, unless the focal group '
                         || 'is the unknown group (bb_unk_grp) every '
                         || 'day of a group''s observation must begin '
                         || 'with a descent from a sleeping grove';
        RETURN NULL;
      END IF;
    END;
  END IF;

  IF this_adn = 'bb_swerb_ascent' THEN
    -- Every observation team's daily observation of a group must
    -- end with ascent into a sleeping grove.

    DECLARE
      end_time swerb_bes.stop%TYPE;

    BEGIN
      -- Find the time the observation team stopped observation of the
      -- group for the day.

      SELECT        swerb_bes.stop
        INTO STRICT end_time
        FROM swerb_bes
        WHERE swerb_bes.did = this_did
              AND swerb_bes.stop IS NOT NULL
              AND swerb_bes.focal_grp = this_focal_grp
        ORDER BY swerb_bes.stop DESC
        LIMIT 1;

      -- Is there an ascent into a sleeping grove on the time
      -- of last observation, or when end time is NULL?
      PERFORM 1
        FROM swerb_bes
             JOIN swerb_data ON (swerb_data.beid = swerb_bes.beid)
             JOIN swerb_loc_data ON (swerb_loc_data.swid = swerb_data.swid)
             JOIN adcodes ON (adcodes.adn = swerb_loc_data.adcode)
        WHERE swerb_bes.did = this_did
              AND (swerb_bes.stop = end_time
                   OR swerb_bes.stop IS NULL)
          AND swerb_bes.focal_grp = this_focal_grp
          AND adcodes.adn = 'bb_swerb_ascent'
        LIMIT 1;
      IF NOT FOUND THEN
        RAISE EXCEPTION integrity_constraint_violation USING
              MESSAGE = 'Error on commit after ' || TG_OP
                         || ' from SWERB_LOC_DATA'
            , DETAIL = 'Key (SWId) = (' || OLD.swid
                         || '): Value (Loc) = (' || OLD.loc
                         || '): Value (ADCode) = (' || OLD.adcode
                         || '): Value (SWERB_BES.DId) = (' || this_did
                         || '): Value (SWERB_BES.Focal_grp) = (' || this_focal_grp
                         || '): Cannot delete, unless the focal group '
                         || 'is the unknown group (bb_unk_grp) every '
                         || 'day of a group''s observation must end '
                         || 'with an ascent into a sleeping grove';
        RETURN NULL;
      END IF;
    END;
  END IF;

  RETURN NULL;
  END;
$$;


SELECT 'swerb_loc_data_trigger' AS trigger;
CREATE TRIGGER swerb_loc_data_trigger
  AFTER INSERT OR UPDATE
  ON swerb_loc_data FOR EACH ROW
  EXECUTE PROCEDURE swerb_loc_data_func();

SELECT 'swerb_loc_data_commit_trigger' AS trigger;
CREATE CONSTRAINT TRIGGER swerb_loc_data_commit_trigger
  AFTER INSERT OR UPDATE
  ON swerb_loc_data
  DEFERRABLE INITIALLY DEFERRED
  FOR EACH ROW
  EXECUTE PROCEDURE swerb_loc_data_commit_func();

SELECT 'swerb_loc_data_delete_commit_trigger' AS trigger;
CREATE CONSTRAINT TRIGGER swerb_loc_data_delete_commit_trigger
  AFTER DELETE
  ON swerb_loc_data
  DEFERRABLE INITIALLY DEFERRED
  FOR EACH ROW
  EXECUTE PROCEDURE swerb_loc_data_delete_commit_func();
