dnl Copyright (C) 2017 Jake Gordon, <jacob.gordon@duke.edu>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Jake Gordon
dnl

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl

--  
-- swerb_loc_data_confidences
--
SELECT 'swerb_loc_data_confidences' AS table;

SELECT 'swerb_loc_data_confidences_func' AS function;
CREATE FUNCTION swerb_loc_data_confidences_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  BEGIN
  -- Function for swerb_loc_data_confidences insert and update triggers
  --
  -- GPL_notice(`  --', `2017', `Jake Gordon, <jacob.gordon@duke.edu>')

  IF TG_OP = 'UPDATE' THEN
    cannot_change(`SWERB_LOC_DATA_CONFIDENCES', `SWId')
  END IF;

  -- Nearest_Loc must have existed when the observation was made.
  PERFORM 1
    FROM swerb_data
         JOIN swerb_bes
              ON swerb_bes.beid = swerb_data.beid
         JOIN swerb_departs_data
              ON swerb_departs_data.did = swerb_bes.did
         JOIN swerb_gws
              ON swerb_gws.loc = NEW.nearest_loc
    WHERE swerb_data.swid = NEW.swid
      AND (swerb_departs_data.date < swerb_gws.start
            OR swerb_departs_data.date > swerb_gws.finish)
    LIMIT 1;
  IF FOUND THEN
    -- The date of this observation is outside of this Loc's time of
    -- existence.
    RAISE EXCEPTION integrity_constraint_violation USING
          MESSAGE = 'Error on ' || TG_OP || ' of SWERB_LOC_DATA_CONFIDENCES'
        , DETAIL = 'Key (SWId) = (' || NEW.swid
                     || '): Value (Nearest_Loc) = (' || NEW.nearest_loc
                     || '): Nearest_Loc must have existed when the '
                     || 'observation was made'
        , HINT = 'Check this Loc''s Start and Finish in SWERB_GWS';
    RETURN NULL;
  END IF;

  -- The Nearest_Loc's "type" (grove, waterhole, etc.) must match that
  -- of the related SWERB_LOC_DATA.Loc.
  PERFORM 1
    FROM swerb_loc_data
         JOIN swerb_gws AS recorded_loc_gws
              ON recorded_loc_gws.loc = swerb_loc_data.loc
         JOIN swerb_gws AS nearest_loc_gws
              ON nearest_loc_gws.loc = NEW.nearest_loc
    WHERE swerb_loc_data.swid = NEW.swid
      AND recorded_loc_gws.type <> nearest_loc_gws.type
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION integrity_constraint_violation USING
          MESSAGE = 'Error on ' || TG_OP || ' of SWERB_LOC_DATA_CONFIDENCES'
        , DETAIL = 'Key (SWId) = (' || NEW.swid
                     || '): Value (Nearest_Loc) = (' || NEW.nearest_loc
                     || '): Nearest_Loc must have same SWERB_GWS.Type '
                     || 'as the related SWERB_LOC_DATA.Loc';
    RETURN NULL;
  END IF;


  RETURN NULL;
  END;
$$;


SELECT 'swerb_loc_data_confidences_delete_func' AS function;
CREATE FUNCTION swerb_loc_data_confidences_delete_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- Function for swerb_loc_data_confidences delete trigger
  --
  -- GPL_notice(`  --', `2017', `Jake Gordon, <jacob.gordon@duke.edu>')

  DECLARE

  BEGIN

  RETURN NULL;
  END;
$$;


SELECT 'swerb_loc_data_confidences_trigger' AS trigger;
CREATE TRIGGER swerb_loc_data_confidences_trigger
  AFTER INSERT OR UPDATE
  ON swerb_loc_data_confidences FOR EACH ROW
  EXECUTE PROCEDURE swerb_loc_data_confidences_func();

SELECT 'swerb_loc_data_confidences_delete_trigger' AS trigger;
CREATE TRIGGER swerb_loc_data_confidences_delete_trigger
  AFTER DELETE
  ON swerb_loc_data_confidences FOR EACH ROW
  EXECUTE PROCEDURE swerb_loc_data_confidences_delete_func();
