dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2011, Karl O. Pinc
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl


--  
-- swerb_observers
--
SELECT 'swerb_observers' AS table;

SELECT 'swerb_observers_func' AS function;
CREATE FUNCTION swerb_observers_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  BEGIN
  -- Function for swerb_observers insert and update triggers
  --
  -- Copyright (C) 2012 The Meme Factory, Inc.  http://www.meme.com/
  -- GPL_notice(`  --', `2011', `Karl O. Pinc <kop@meme.com>')

  -- Cannot change the id.
  IF TG_OP = 'UPDATE' THEN
    cannot_change(`OBSERVERS', `SWERBOId')
  END IF;

  -- The combination of date, time, and observer, when all are non-NULL
  -- must be unique.
  PERFORM 1
    FROM swerb_departs_data AS other_sdd
       , swerb_departs_data AS this_sdd
       , swerb_observers
    WHERE this_sdd.did = NEW.did
          AND other_sdd.date = this_sdd.date
          AND other_sdd.time = this_sdd.time
          AND other_sdd.did <> this_sdd.did
          AND swerb_observers.did = other_sdd.did
          AND swerb_observers.observer = NEW.observer
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'Did %: Observer %: The combination of swerb_departs_data.date, swerb_departs_data.time, and observer must be unique, when all are non-NULL'
                  , NEW.did
                  , NEW.observer;
    RETURN NULL;
  END IF;

  RETURN NULL;
  END;
$$;


SELECT 'swerb_observers_trigger' AS trigger;
CREATE TRIGGER swerb_observers_trigger
  AFTER INSERT OR UPDATE
  ON swerb_observers FOR EACH ROW
  EXECUTE PROCEDURE swerb_observers_func();
