dnl Copyright (C) 2012, 2017 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2010, 2011, Karl O. Pinc
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl


dnl plpgsql fragment for BLAH
dnl
dnl Syntax: BLAH(ARG)
dnl
dnl ARG   description
dnl
changequote({,})
define({BLAH},{
})dnl
changequote(`,')dnl

--  
-- teeth
--
SELECT 'teeth' AS table;

SELECT 'teeth_before_func' AS function;
CREATE FUNCTION teeth_before_func()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  BEGIN
  -- Function for teeth before insert trigger
  --
  -- GPL_notice(`  --', `2010', `Karl O. Pinc <kop@meme.com>')
  --
  -- Remarks:
  --  Exists to faciliate insert into the DENT_CODES view.
  --  Without this trigger the rule would be far too complicated
  --  to write.  The view should not insert into TEETH unless there
  --  is data for the row.

    IF NEW.tstate IS NULL THEN
      -- Silently ignore inserts when there's no Tstate.
      RETURN NULL;
    END IF;

    RETURN NEW;

  END;
$$;

SELECT 'teeth_func' AS function;
CREATE FUNCTION teeth_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  BEGIN
  -- Function for teeth insert and update triggers
  --
  -- Copyright (C) 2017 The Meme Factory, Inc., http://www.meme.com/
  -- GPL_notice(`  --', `2010', `Karl O. Pinc <kop@meme.com>')

  IF TG_OP = 'UPDATE' THEN
    cannot_change(`TEETH', `Teethid')
    cannot_change(`Teeth', `Dartid')

    IF NEW.tstate IS NULL THEN
      RAISE EXCEPTION integrity_constraint_violation USING
            MESSAGE = 'Error on ' || TG_OP || ' of TEETH'
          , DETAIL = 'Key (Teethid) = (' || OLD.teethid
                       || '): Value (Dartid) = (' || OLD.dartid
                       || '): Old Value (Tooth) = (' || OLD.tooth
                       || '): Old Value (Tstate) = (' || OLD.tstate
                       || '): TEETH.Tstate cannot be changed to NULL';
      RETURN NULL;
    END IF;
  END IF;

  -- Can have only one tooth in any given possible tooth site at any
  -- one time.
  PERFORM 1
    FROM toothcodes as thisloc
         , teeth as thattooth
         , toothcodes as thatloc
    WHERE thattooth.dartid = NEW.dartid
          AND thattooth.teethid <> NEW.teethid
          AND thatloc.tooth = thattooth.tooth
          AND thisloc.tooth = NEW.tooth
          AND thatloc.toothsite = thisloc.toothsite
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'TEETH.Teethid %: TEETH.Dartid %: TEETH.Tooth %: Cannot have more than one tooth in any given tooth site (TOOTHSITES) within the mouth, at any one time'
                    , NEW.teethid
                    , NEW.dartid
                    , NEW.tooth;
    RETURN NULL;
  END IF;

  RETURN NULL;
  END;
$$;


SELECT 'teeth_trigger' AS trigger;
CREATE TRIGGER teeth_trigger
  AFTER INSERT OR UPDATE
  ON teeth FOR EACH ROW
  EXECUTE PROCEDURE teeth_func();

SELECT 'teeth_before_trigger' AS trigger;
CREATE TRIGGER teeth_before_trigger
  BEFORE INSERT
  ON teeth FOR EACH ROW
  EXECUTE PROCEDURE teeth_before_func();
