dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2009, Karl O. Pinc
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl

--  
-- Tempmaxs
--
SELECT 'tempmaxs' AS table;

SELECT 'tempmaxs_func' AS function;
CREATE FUNCTION tempmaxs_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$

  BEGIN
  -- Function for update triggers
  --
  -- GPL_notice(`  --', `2009', `Karl O. Pinc <kop@meme.com>')

  -- Cannot change the WRID.
  IF TG_OP = 'UPDATE' THEN
    -- Cannot change the WRID.
    cannot_change(`TEMPMAXS', `WRid')
  END IF;

  -- The maximum temp must be more than the minimum temp when both are
  -- present.
  PERFORM 1
    FROM tempmins
    WHERE tempmins.wrid = NEW.wrid
          AND tempmins.tempmin > NEW.tempmax
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'WRid %: Bad maximum temperature %: Maxtemp less than TEMPMINS.Mintemp'
                  , NEW.wrid
                  , NEW.tempmax;
    RETURN NULL;
  END IF;

  RETURN NULL;
  END;
$$;


SELECT 'tempmaxs_trigger' AS trigger;
CREATE TRIGGER tempmaxs_trigger
  AFTER INSERT OR UPDATE
  ON tempmaxs FOR EACH ROW
  EXECUTE PROCEDURE tempmaxs_func();
