dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2010, 2011, Karl O. Pinc
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl
dnl Remarks:
dnl  This file is sort of special in that it defines the triggers for
dnl two tables, TESTES_ARC and TESTES_DIAM.
dnl

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl
include(`seqmacros.m4')dnl

dnl A m4 macro to produce SQL code that further restricts
dnl distinct Testsql values to Testside within Dartid.
dnl
dnl Syntax: _per_new_testside(t)
dnl
dnl t  The name of the table.  This exists so other
dnl    macros can alias TESTES need be.
dnl
changequote({,})
define({_per_new_testside},{dnl
$1.testside = NEW.testside})dnl
changequote(`,')dnl

dnl A m4 macro to produce SQL code that further restricts
dnl distinct Testsql values to Testside within Dartid.
dnl
dnl Syntax: _per_old_testside(t)
dnl
dnl t  The name of the table.  This exists so other
dnl    macros can alias TESTES need be.
dnl
changequote({,})
define({_per_old_testside},{dnl
$1.testside = OLD.testside})dnl
changequote(`,')dnl

dnl plpgsql fragment to ensure that Testwidth and Testlength
dnl have contiguous Testseq values, without interspersed NULL
dnl Testwidth and Testlength values.
dnl
dnl Syntax: _contiguous_testseq(table, col)
dnl
dnl table  The name of the testes table (with suffix).
dnl col    The column that can't be NULL (testwidth or testlength)
dnl
changequote({,})
define({_contiguous_testseq},{
  -- When the width (or length) is NULL then all the rows (for the
  -- same darting) with a higher sequence number should be NULL too.
  -- It's not clear to me how this plays out with automatic re-sequencing
  -- under all conditions.  If it causes a problem, too bad.  Delete
  -- the rows and re-insert them rather than try to resequence.
  IF NEW.$2 IS NULL THEN
    PERFORM 1
      FROM $1
      WHERE $1.dartid = NEW.dartid
            AND $1.testseq > NEW.testseq
            AND $1.$2 IS NOT NULL
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION '$1.Testesid %: $1.Dartid %: $1.Testseq %: $2 cannot be NULL if there are non-NULL $1 values with higher Testseq values'
                      , NEW.testesid
                      , NEW.dartid
                      , NEW.testseq;
      RETURN NULL;
    END IF;
  END IF;
})dnl
changequote(`,')dnl

dnl A m4 macro to create all the functions and triggers
dnl needed to support one o the testes tables.
dnl
dnl Syntax: _make_testes_triggers(suffix)
dnl
dnl Input:
dnl   suffix  The suffix at the end of the testes table's name.
dnl
changequote({,})
define({_make_testes_triggers},{dnl
--  
-- testes_$1
--
SELECT 'testes_$1' AS table;

SELECT 'testes_$1_before_insert_func' AS function;
CREATE FUNCTION testes_$1_before_insert_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  BEGIN
  -- Function for testes_$1 before insert trigger
  --
  -- GPL_notice(`  --', `2010', `Karl O. Pinc <kop@meme.com>')
  --
  -- Remarks:
  -- WARNING: See remarks in testes_$1_func().

    _init_seq_value(`TESTES_$1',
                    `Dartid',
                    `Testseq',
                    `_per_new_testside')

    RETURN NEW;
  END;
$$;

SELECT 'testes_$1_func' AS function;
CREATE FUNCTION testes_$1_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  BEGIN
  -- Function for testes_$1 insert and update triggers
  --
  -- GPL_notice(`  --', `2010, 2011', `Karl O. Pinc <kop@meme.com>')
  --
  -- Remarks:

  -- Must be male to have testes measurements.
  PERFORM 1
    FROM dartings
         JOIN biograph ON (biograph.sname = dartings.sname)
    WHERE dartings.dartid = NEW.dartid
          AND biograph.sex = 'bb_male'
    LIMIT 1;
  IF NOT FOUND THEN
    RAISE EXCEPTION 'TESTES_$1.Testesid %: TESTES_$1.Dartid %: Must be male to have a row in TESTES_$1'
                  , NEW.testesid
                  , NEW.dartid;
    RETURN NULL;
  END IF;

  -- Deal with sequence re-numbering.
  _seq_insert_update_func(`TESTES_$1',
                          `Testesid',
                          `Dartid',
                          `Testseq',
                          `_per_new_testside')

  -- Pick up the new sequence number in case it's changed.
  SELECT INTO NEW.testseq testes_$1.testseq
    FROM testes_$1
    WHERE testes_$1.testesid = NEW.testesid;

  -- At this point the current Testseq/Dartid/Testside should
  -- be unique.
  PERFORM 1
    FROM testes_$1
    WHERE testes_$1.dartid = NEW.dartid
          AND testes_$1.testesid <> NEW.testesid
          AND testes_$1.testside = NEW.testside
          AND testes_$1.testseq = NEW.testseq
    LIMIT 1;
  IF FOUND THEN
    RAISE EXCEPTION 'TESTES_$1.Testesid %: TESTES_$1.Dartid %: TESTES_$1.Testside %: TESTES_$1.Testseq %: The combination of Dartid, Testside, and Testseq must be unique'
                    , NEW.testesid
                    , NEW.dartid
                    , NEW.testside
                    , NEW.testseq;
    RETURN NULL;
  END IF;

  -- Meet minimum width and length requirements based on age.
  PERFORM 1
    FROM dartings
         JOIN biograph ON (biograph.sname = dartings.sname)
    WHERE dartings.dartid = NEW.dartid
          AND AGE(dartings.date, biograph.birth) >= 'bb_adult_testes_age'::INTERVAL
    LIMIT 1;
  IF FOUND THEN
    -- The individual was darted when adult.

    IF NEW.testlength < translit(bb_min_adult_$1_testlength, A-Z, a-z) THEN
      RAISE EXCEPTION 'TESTES_$1.Testesid %: TESTES_$1.dartid %: TESTES_$1.Testlength %: Individuals at least bb_adult_testes_age old when darted cannot have a Testlength less than translit(bb_min_adult_$1_testlength, A-Z, a-z)'
                    , NEW.testesid
                    , NEW.dartid
                    , NEW.testlength;
      RETURN NULL;
    END IF;

    IF NEW.testwidth < translit(bb_min_adult_$1_testwidth, A-Z, a-z) THEN
      RAISE EXCEPTION 'TESTES_$1.Testesid %: TESTES_$1.Dartid %: TESTES_$1.Testwidth %: Individuals at least bb_adult_testes_age old when darted cannot have a Testwidth less than translit(bb_min_adult_$1_testwidth, A-Z, a-z)'
                    , NEW.testesid
                    , NEW.dartid
                    , NEW.testwidth;
      RETURN NULL;
    END IF;
  END IF;

  -- Ensure that Testseq values remain contiguous, without
  -- interspersed NULLs.
  _contiguous_testseq(TESTES_$1, `Testwidth')
  _contiguous_testseq(TESTES_$1, `Testlength')

  RETURN NULL;
  END;
$$;


SELECT 'testes_$1_delete_func' AS function;
CREATE FUNCTION testes_$1_delete_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- Function for testes_$1 delete trigger
  --
  -- GPL_notice(`  --', `2010', `Karl O. Pinc <kop@meme.com>')
  --
  -- Remarks:
  -- WARNING: See remarks in testes_$1_func().

  BEGIN

    _fill_seq_gap(`TESTES_$1',
                  `Testesid',
                  `Dartid',
                  `Testseq',
                  `_per_old_testside')

  RETURN NULL;
  END;
$$;

SELECT 'testes_$1_before_insert_trigger' AS trigger;
CREATE TRIGGER testes_$1_before_insert_trigger
  BEFORE INSERT
  ON testes_$1 FOR EACH ROW
  EXECUTE PROCEDURE testes_$1_before_insert_func();

SELECT 'testes_$1_trigger' AS trigger;
CREATE TRIGGER testes_$1_trigger
  AFTER INSERT OR UPDATE
  ON testes_$1 FOR EACH ROW
  EXECUTE PROCEDURE testes_$1_func();

SELECT 'testes_$1_delete_trigger' AS trigger;
CREATE TRIGGER testes_$1_delete_trigger
  AFTER DELETE
  ON testes_$1 FOR EACH ROW
  EXECUTE PROCEDURE testes_$1_delete_func();
})dnl
changequote(`,')dnl

dnl
dnl Create triggers for the two testes tables.
dnl
_make_testes_triggers(`ARC')
_make_testes_triggers(`DIAM')
