dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2011, Karl O. Pinc
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl


changequote({,})
dnl
dnl Raise an exception noting that the user does not have permission
dnl
dnl syntax: _permission_exception
dnl
define({_permission_exception}, {dnl
      RAISE EXCEPTION 'TICKSTATUSES rows with a Tickstatus of bb_tickstatus_zero or bb_tickstatus_ticks cannot be changed by ordinary users';})
changequote(`,')


--  
-- tickstatuses
--
SELECT 'tickstatuses' AS table;

SELECT 'tickstatuses_func' AS function;
CREATE FUNCTION tickstatuses_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  BEGIN
  -- Function for tickstatuses insert and update triggers
  --
  -- GPL_notice(`  --', `2011', `Karl O. Pinc <kop@meme.com>')

  IF TG_OP = 'UPDATE' THEN

    -- Can't change the Tickstatuses column.
    cannot_change(`TICKSTATUSES', `Tickstatus')
  END IF;

  IF NOT(has_table_privilege('acts', 'trigger'))
     AND (NEW.tickstatus = bb_tickstatus_zero
          OR NEW.tickstatus = bb_tickstatus_ticks) THEN
    IF TG_OP = 'INSERT' THEN
      _permission_exception
      RETURN NULL;
    ELSEIF NEW <> OLD THEN
      _permission_exception
      RETURN NULL;
    END IF;
  END IF;

  RETURN NULL;
  END;
$$;


SELECT 'tickstatuses_delete_func' AS function;
CREATE FUNCTION tickstatuses_delete_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  BEGIN
  -- Function for tickstatuses delete triggers
  --
  -- GPL_notice(`  --', `2011', `Karl O. Pinc <kop@meme.com>')

  IF NOT(has_table_privilege('acts', 'trigger'))
     AND (OLD.tickstatus = bb_tickstatus_zero
          OR OLD.tickstatus = bb_tickstatus_ticks) THEN
    _permission_exception
    RETURN NULL;
  END IF;

  RETURN NULL;
  END;
$$;


SELECT 'tickstatuses_trigger' AS trigger;
CREATE TRIGGER tickstatuses_trigger
  AFTER INSERT OR UPDATE
  ON tickstatuses FOR EACH ROW
  EXECUTE PROCEDURE tickstatuses_func();

SELECT 'tickstatuses_delete_trigger' AS trigger;
CREATE TRIGGER tickstatuses_delete_trigger
  AFTER DELETE
  ON tickstatuses FOR EACH ROW
  EXECUTE PROCEDURE tickstatuses_delete_func();
