dnl Copyright (C) 2017 Jake Gordon <jacob.gordon@duke.edu>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl

dnl m4 includes
include(`copyright.m4')dnl
include(`constants.m4')dnl
include(`macros.m4')dnl

--
-- wbc_counts
--
SELECT 'wbc_counts' AS table;

SELECT 'wbc_counts_func' AS function;
CREATE FUNCTION wbc_counts_func ()
  RETURNS trigger
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$
  BEGIN
  -- Function for wbc_counts insert and update triggers
  --
  -- GPL_notice(`  --', `2017', `Jake Gordon <jacob.gordon@duke.edu>')

    -- Make sure that Slide_number does not exceed the number of blood
    -- smears in DART_SAMPLES for this Dartid.
    PERFORM 1
      FROM dart_samples
      WHERE dart_samples.dartid = NEW.dartid
        AND dart_samples.ds_type = bb_dst_bloodsmear
        AND dart_samples.num < NEW.slide_number
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION integrity_constraint_violation USING
            MESSAGE = 'Error on ' || TG_OP || ' of WBC_COUNTS'
          , DETAIL = 'Key (WCId) = (' || NEW.wcid
                       || '): Value (Dartid) = (' || NEW.dartid
                       || '): Value (Slide_number) = (' || NEW.slide_number
                       || '): Slide_number cannot exceed the number of '
                       || 'blood smears collected for this Dartid';
      RETURN NULL;
    END IF;

    -- Count_Date must be on or after a row's related DARTINGS.Date
    PERFORM 1
      FROM dartings
      WHERE dartings.dartid = NEW.dartid
        AND dartings.date > NEW.count_date
      LIMIT 1;
    IF FOUND THEN
      RAISE EXCEPTION integrity_constraint_violation USING
            MESSAGE = 'Error on ' || TG_OP || ' of WBC_COUNTS'
          , DETAIL = 'Key (WCId) = (' || NEW.wcid
                       || '): Value (Dartid) = (' || NEW.dartid
                       || '): Value (Count_Date) = (' || NEW.count_date
                       || '): WBC_COUNTS.Count_Date must be on or after '
                       || 'the row''s related DARTINGS.Date';
      RETURN NULL;
    END IF;

  RETURN NULL;
  END;
$$;


SELECT 'wbc_counts_trigger' AS trigger;
CREATE TRIGGER wbc_counts_trigger
  AFTER INSERT OR UPDATE
  ON wbc_counts FOR EACH ROW
  EXECUTE PROCEDURE wbc_counts_func();
