dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2007, 2008, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Handle inserts into the interact view.
dnl Karl O. Pinc <kop@meme.com>
dnl
dnl
dnl m4 includes
include(`copyright.m4')
include(`constants.m4')
include(`macros.m4')
dnl


CREATE OR REPLACE FUNCTION
  _interact_insert(this_row interact)
  RETURNS interact_data
  LANGUAGE plpgsql
  bb_function_set_search_path
  AS $$

  -- Handle inserts into the interact view.
  --
  -- GPL_notice(`  --', `2007, 2008', `Karl O. Pinc <kop@meme.com>')
  --
  -- Syntax:  _interact_insert(this_row)
  --
  -- Input:
  --   this_row  A the interact row to insert.
  --
  -- Returns:
  --   A interact_data row to insert.
  --
  -- Remarks:
  --   You'd think the either-data-or-computed-value code would
  -- be ripe for a macro but there's just too many parameters
  -- to make sense of the macro.


  DECLARE
    new_row interact_data%ROWTYPE;

  BEGIN

    -- Assign iid.
    IF this_row.iid IS NULL THEN
      new_row.iid := nextval('interact_data_iid_seq');
    ELSE
      new_row.iid = this_row.iid;
    END IF;

    -- Assign sid.
    new_row.sid := this_row.sid;

    -- Assign act.
    new_row.act := this_row.act;

    -- Assign date.
    IF this_row.date IS NOT NULL THEN
      IF this_row.jdate IS NOT NULL
         AND julian(this_row.date) <> this_row.jdate THEN
        -- Mismatch between supplied julian date and actual date.
        RAISE EXCEPTION 'INTERACT.Iid %: INTERACT.Date %: INTERACT.Jdate: Mismatch between Date and Jdate: Cannot insert into INTERACT_DATA'
                      , new_row.iid
                      , this_row.date
                      , this_row.jdate;
        RETURN new_row;
      END IF;
      new_row.date := this_row.date;
    ELSE
      IF this_row.jdate IS NOT NULL THEN
        -- Use the jdate when there's no date.
        new_row.date := julian_to(this_row.jdate);
      ELSE
        -- NULL date.  Bound to cause problems later.
        new_row.date := this_row.date;
      END IF;
    END IF;

    -- Assign start.
    IF this_row.start IS NOT NULL THEN
      IF this_row.startspm IS NOT NULL
         AND startspm(this_row.start) <> this_row.startspm THEN
        -- Mismatch between supplied startspm and actual start.
        RAISE EXCEPTION 'INTERACT.Iid %: INTERACT.Start %: INTERACT.Startspm: Mismatch between Start and Startspm: Cannot insert into INTERACT_DATA'
                      , new_row.iid
                      , this_row.start
                      , this_row.startspm;
        RETURN new_row;
      END IF;
      new_row.start := this_row.start;
    ELSE
      IF this_row.startspm IS NOT NULL THEN
        -- Use the startspm when there's no start.
        new_row.start := spm_to(this_row.startspm);
      ELSE
        -- NULL start.  Bound to cause problems later.
        new_row.start := this_row.start;
      END IF;
    END IF;

    -- Assign stop.
    IF this_row.stop IS NOT NULL THEN
      IF this_row.stopspm IS NOT NULL
         AND stopspm(this_row.stop) <> this_row.stopspm THEN
        -- Mismatch between supplied stopspm and actual stop.
        RAISE EXCEPTION 'INTERACT.Iid %: INTERACT.Stop %: INTERACT.Stopspm: Mismatch between Stop and Stopspm: Cannot insert into INTERACT_DATA'
                      , new_row.iid
                      , this_row.stop
                      , this_row.stopspm;
        RETURN new_row;
      END IF;
      new_row.stop := this_row.stop;
    ELSE
      IF this_row.stopspm IS NOT NULL THEN
        -- Use the stopspm when there's no stop.
        new_row.stop := spm_to(this_row.stopspm);
      ELSE
        -- NULL stop.  Bound to cause problems later.
        new_row.stop := this_row.stop;
      END IF;
    END IF;

    RETURN new_row;
  END;
$$;

grant_func_priv(`_interact_insert(interact)')
