dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Functions and triggers to allow insert, update, and delete into
dnl the interact_sorted view.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl
dnl
dnl m4 includes
include(`copyright.m4')
include(`constants.m4')
include(`macros.m4')
dnl

SELECT 'interact_sorted_insert_func()';
CREATE OR REPLACE FUNCTION interact_sorted_insert_func()
  RETURNS trigger
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- For triggers instead of insert into the interact_sorted view.
  --
  -- GPL_notice(`  --', `2012', `The Meme Factory, Inc.  http://www.meme.com/')
  --
  -- Remarks:
  --
  -- Bugs:

  DECLARE

  BEGIN
    INSERT INTO interact_data (iid
                             , sid
                             , act
                             , date
                             , start
                             , stop
                             , observer
                             , handwritten)
      VALUES (NEW.iid
            , NEW.sid
            , NEW.act
            , NEW.date
            , NEW.start
            , NEW.stop
            , NEW.observer
            , NEW.handwritten);

  RETURN NULL;
  END;
$$;

SELECT 'interact_sorted_update_func()';
CREATE OR REPLACE FUNCTION interact_sorted_update_func()
  RETURNS trigger
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- For triggers instead of update of the interact_sorted view.
  --
  -- GPL_notice(`  --', `2012', `The Meme Factory, Inc.  http://www.meme.com/')
  --
  -- Remarks:
  --
  -- Bugs:

  DECLARE

  BEGIN

    UPDATE interact_data
      SET iid = NEW.iid
        , sid = NEW.sid
        , act = NEW.act
        , date = NEW.date
        , start = NEW.start
        , stop = NEW.stop
        , observer = NEW.observer
        , handwritten = NEW.handwritten
      WHERE interact_data.iid = OLD.iid;

  RETURN NULL;
  END;
$$;

SELECT 'interact_sorted_delete_func()';
CREATE OR REPLACE FUNCTION interact_sorted_delete_func()
  RETURNS trigger
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- For triggers instead of delete of the interact_sorted view.
  --
  -- GPL_notice(`  --', `2012', `The Meme Factory, Inc.  http://www.meme.com/')
  --
  -- Remarks:
  --
  -- Bugs:

  DECLARE

  BEGIN

    DELETE FROM interact_data
      WHERE interact_data.iid = OLD.iid;

  RETURN NULL;
  END;
$$;


SELECT 'interact_sorted_insert_trigger' AS trigger;
CREATE TRIGGER interact_sorted_insert_trigger
  INSTEAD OF INSERT
  ON interact_sorted FOR EACH ROW
  EXECUTE PROCEDURE interact_sorted_insert_func();

SELECT 'interact_sorted_update_trigger' AS trigger;
CREATE TRIGGER interact_sorted_update_trigger
  INSTEAD OF UPDATE
  ON interact_sorted FOR EACH ROW
  EXECUTE PROCEDURE interact_sorted_update_func();

SELECT 'interact_sorted_delete_trigger' AS trigger;
CREATE TRIGGER interact_sorted_delete_trigger
  INSTEAD OF DELETE
  ON interact_sorted FOR EACH ROW
  EXECUTE PROCEDURE interact_sorted_delete_func();
