dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Functions and triggers to allow insert, update, and delete into
dnl the quads view.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl
dnl
dnl m4 includes
include(`copyright.m4')
include(`constants.m4')
include(`macros.m4')
dnl

SELECT 'quads_insert_func()';
CREATE OR REPLACE FUNCTION quads_insert_func()
  RETURNS trigger
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- For triggers instead of insert into the quads view.
  --
  -- GPL_notice(`  --', `2012', `The Meme Factory, Inc.  http://www.meme.com/')
  --
  -- Remarks:
  --
  -- Bugs:

  DECLARE

  BEGIN
    INSERT
      INTO quad_data (quad, xyloc, aerial)
      VALUES (NEW.quad
            , bb_makepoint(NEW.x, NEW.y)
            , NEW.aerial);

  RETURN NULL;
  END;
$$;

SELECT 'quads_update_func()';
CREATE OR REPLACE FUNCTION quads_update_func()
  RETURNS trigger
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- For triggers instead of update of the quads view.
  --
  -- GPL_notice(`  --', `2012', `The Meme Factory, Inc.  http://www.meme.com/')
  --
  -- Remarks:
  --
  -- Bugs:

  DECLARE

  BEGIN
    UPDATE quad_data
      SET quad = NEW.quad
        , xyloc = bb_makepoint(NEW.x, NEW.y)
        , aerial = NEW.aerial
      WHERE quad = OLD.quad;

  RETURN NULL;
  END;
$$;

SELECT 'quads_delete_func()';
CREATE OR REPLACE FUNCTION quads_delete_func()
  RETURNS trigger
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- For triggers instead of delete of the quads view.
  --
  -- GPL_notice(`  --', `2012', `The Meme Factory, Inc.  http://www.meme.com/')
  --
  -- Remarks:
  --
  -- Bugs:

  DECLARE

  BEGIN
    DELETE
      FROM quad_data
      WHERE quad = OLD.quad;

  RETURN NULL;
  END;
$$;


SELECT 'quads_insert_trigger' AS trigger;
CREATE TRIGGER quads_insert_trigger
  INSTEAD OF INSERT
  ON quads FOR EACH ROW
  EXECUTE PROCEDURE quads_insert_func();

SELECT 'quads_update_trigger' AS trigger;
CREATE TRIGGER quads_update_trigger
  INSTEAD OF UPDATE
  ON quads FOR EACH ROW
  EXECUTE PROCEDURE quads_update_func();

SELECT 'quads_delete_trigger' AS trigger;
CREATE TRIGGER quads_delete_trigger
  INSTEAD OF DELETE
  ON quads FOR EACH ROW
  EXECUTE PROCEDURE quads_delete_func();
