dnl Copyright (C) 2011, 2012, 2014 The Meme Factory, Inc., http://www.meme.com/
dnl Copyright (C) 2005-2011, Karl O. Pinc <kop@meme.com>
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl
dnl m4 includes.
include(`copyright.m4')
include(`constants.m4')

-- The rules used to support the views.

-- Don't forget that when we use conditional rules we need
-- an unconditional "INSTEAD" rule and make the conditonal
-- ones non-INSTEAD.




dnl sql fragment for making census_demog-like view rules
dnl
dnl Syntax: _m_census_demog(name)
dnl
dnl name   View name.
dnl
changequote({,})
define({_m_census_demog},{
--
-- $1
--

-- insert

-- Add default cenid
-- This does not work because we can't reference NEW.cenid,
-- postgresql lexically subsitutes the default value (the nextval())
-- which causes the demog insert to make a nextval call.
-- And if we use currval() all the time we don't allow
-- the user to specify his own cenid.
--ALTER TABLE $1
--  ALTER cenid SET DEFAULT nextval('census_cenid_seq');

-- 4 cases:
--  Demog data specified, cenid NULL
--  Demog data specified, cenid NOT NULL
--  No demog data specified, cenid NULL
--  No demog data specified, cenid NOT NULL
CREATE OR REPLACE RULE $1_insert
  AS ON insert
  TO $1
  DO INSTEAD NOTHING;
SELECT '$1_insert';

--  Demog data specified, cenid NULL
CREATE OR REPLACE RULE $1_insert_both_cenid
  AS ON insert
  TO $1
  WHERE NEW.cenid IS NULL
        AND (NEW.reference IS NOT NULL
             OR NEW.comment IS NOT NULL)
  DO ALSO (
    INSERT INTO census (cenid, sname, date, grp, status, cen)
           VALUES (  nextval('census_cenid_seq')
                   , NEW.sname
                   , NEW.date
                   , NEW.grp
                   , NEW.status
                   , NEW.cen);
    INSERT INTO demog (cenid, reference, comment)
           VALUES(  currval('census_cenid_seq')
                  , NEW.reference
                  , NEW.comment);
  );
SELECT '$1_insert_both_cenid';

--  Demog data specified, cenid NOT NULL
CREATE OR REPLACE RULE $1_insert_both_nocenid
  AS ON insert
  TO $1
  WHERE NEW.cenid IS NOT NULL
        AND (NEW.reference IS NOT NULL
             OR NEW.comment IS NOT NULL)
  DO ALSO (
    INSERT INTO census (cenid, sname, date, grp, status, cen)
           VALUES (  NEW.cenid
                   , NEW.sname
                   , NEW.date
                   , NEW.grp
                   , NEW.status
                   , NEW.cen);
    INSERT INTO demog (cenid, reference, comment)
           VALUES(  NEW.cenid
                  , NEW.reference
                  , NEW.comment);
  );
SELECT '$1_insert_both_nocenid';

--  No demog data specified, cenid NULL
CREATE OR REPLACE RULE $1_insert_one_nocenid
  AS ON insert
  TO $1
  WHERE NEW.cenid IS NULL
        AND NEW.reference IS NULL
        AND NEW.comment IS NULL 
  DO ALSO
    INSERT INTO census (cenid, sname, date, grp, status, cen)
           VALUES (  nextval('census_cenid_seq')
                   , NEW.sname
                   , NEW.date
                   , NEW.grp
                   , NEW.status
                   , NEW.cen);
SELECT '$1_insert_one_nocenid';

--  No demog data specified, cenid NOT NULL
CREATE OR REPLACE RULE $1_insert_one_cenid
  AS ON insert
  TO $1
  WHERE NEW.cenid IS NOT NULL
        AND NEW.reference IS NULL
        AND NEW.comment IS NULL 
  DO ALSO
    INSERT INTO census (cenid, sname, date, grp, status, cen)
           VALUES (  NEW.cenid
                   , NEW.sname
                   , NEW.date
                   , NEW.grp
                   , NEW.status
                   , NEW.cen);
SELECT '$1_insert_one_cenid';

-- update
--
-- 4 cases, in order:
--  Both old and new have demog row
--  Neither old nor new have demog row
--  New has demog row but not old
--  Old has demog row but not new
CREATE OR REPLACE RULE $1_update
  AS ON update
  TO $1
  DO INSTEAD NOTHING;
SELECT '$1_update';

CREATE OR REPLACE RULE $1_update_both
  AS ON update
  TO $1
  WHERE (OLD.reference IS NOT NULL
         OR OLD.comment IS NOT NULL)
        AND (NEW.reference IS NOT NULL
             OR NEW.comment IS NOT NULL)
  DO ALSO (
    UPDATE census
      SET cenid = NEW.cenid
        , sname = NEW.sname
        , date = NEW.date
        , grp = NEW.grp
        , status = NEW.status
        , cen = NEW.cen
      WHERE cenid = OLD.cenid;
    UPDATE demog
      SET cenid = NEW.cenid
        , reference = NEW.reference
        , comment = NEW.comment
      WHERE cenid = OLD.cenid;
  );
SELECT '$1_update_both';

CREATE OR REPLACE RULE $1_update_one
  AS ON update
  TO $1
  WHERE OLD.reference IS NULL
        AND NEW.reference IS NULL
        AND OLD.comment IS NULL
        AND NEW.comment IS NULL
  DO ALSO (
    UPDATE census
      SET cenid = NEW.cenid
        , sname = NEW.sname
        , date = NEW.date
        , grp = NEW.grp
        , status = NEW.status
        , cen = NEW.cen
      WHERE cenid = OLD.cenid;
  );
SELECT '$1_update_one';

-- Do demog last as this can alter census.
CREATE OR REPLACE RULE $1_update_create
  AS ON update
  TO $1
  WHERE OLD.reference IS NULL
        AND OLD.comment IS NULL
        AND (NEW.reference IS NOT NULL
             OR NEW.comment IS NOT NULL)
  DO ALSO (
    UPDATE census
      SET cenid = NEW.cenid
        , sname = NEW.sname
        , date = NEW.date
        , grp = NEW.grp
        , status = NEW.status
        , cen = NEW.cen
      WHERE cenid = OLD.cenid;
    INSERT INTO demog (cenid, reference, comment)
               VALUES (NEW.cenid, NEW.reference, NEW.comment);
  );
SELECT '$1_update_create';

-- Do demog last as this can alter census.
CREATE OR REPLACE RULE $1_update_destroy
  AS ON update
  TO $1
  WHERE (OLD.reference IS NOT NULL
         OR OLD.comment IS NOT NULL)
        AND NEW.reference IS NULL
        AND NEW.comment IS NULL
  DO ALSO (
    UPDATE census
      SET cenid = NEW.cenid
        , sname = NEW.sname
        , date = NEW.date
        , grp = NEW.grp
        , status = NEW.status
        , cen = NEW.cen
      WHERE cenid = OLD.cenid;
    DELETE FROM demog WHERE cenid = OLD.cenid;
  );
SELECT '$1_update_destroy';

-- delete
CREATE OR REPLACE RULE $1_delete
  AS ON delete
  TO $1
  DO INSTEAD (
    DELETE FROM demog WHERE cenid = OLD.cenid;
    DELETE FROM census
           WHERE cenid = OLD.cenid
                 AND NOT(census.cen);
  );
SELECT '$1_delete';

})dnl
changequote(`,')dnl

-- Make the census_demog-like views

-- census_demog
_m_census_demog(`census_demog')

-- census_demog_sorted
_m_census_demog(`census_demog_sorted')



dnl sql fragment for making demog_census-like view rules
dnl
dnl Syntax: _m_demog_census(name)
dnl
dnl name   View name.
dnl
changequote({,})
define({_m_demog_census},{
--
-- $1
--

-- insert
-- Don't set a default here, do it in code so that
-- we don't inadvertently change existing census rows
-- to our "default".  (I.E. We can tell the difference
-- as to whether or not the user supplied a value.)
--ALTER TABLE $1
--  ALTER cen SET DEFAULT FALSE;

-- 2 cases
--  cenid is null
--  cenid is not null

CREATE OR REPLACE RULE $1_insert
  AS ON insert
  TO $1
  DO INSTEAD NOTHING;
SELECT '$1_insert';

--  cenid is null
CREATE OR REPLACE RULE $1_insert_nocenid
  AS ON insert
  TO $1
  WHERE NEW.cenid IS NULL
  DO ALSO (
    -- Either the insert or the update gets executed.
    -- Update goes first so it can test for existance.
    UPDATE census
      SET status = CASE WHEN NEW.status IS NULL
                        THEN (SELECT c.status
                                FROM census AS c
                                WHERE c.sname = NEW.sname
                                      AND c.date = NEW.date
                                      AND c.grp = NEW.grp)
                        ELSE NEW.status
                   END
        , cen = CASE WHEN NEW.cen IS NULL
                     THEN (SELECT c.cen
                             FROM census AS c
                             WHERE c.sname = NEW.sname
                                   AND c.date = NEW.date
                                   AND c.grp = NEW.grp)
                     ELSE NEW.cen
                 END
      WHERE sname = NEW.sname
            AND date = NEW.date
            AND grp = NEW.grp;

    INSERT INTO census (cenid, sname, date, grp, status, cen)
      SELECT nextval('census_cenid_seq')
                   , NEW.sname
                   , NEW.date
                   , NEW.grp
                   , CASE WHEN NEW.status IS NULL
                          THEN 'bb_demog'
                          ELSE NEW.status
                     END
                   , CASE WHEN NEW.cen IS NULL
                          THEN false
                          ELSE NEW.cen
                     END
        WHERE NOT EXISTS (SELECT census.cenid
                          FROM census
                          WHERE census.sname = NEW.sname
                                AND census.date = NEW.date
                                AND census.grp = NEW.grp);

    INSERT INTO demog (cenid, reference, comment)
      SELECT census.cenid
           , NEW.reference
           , NEW.comment
        FROM census
        WHERE census.sname = NEW.sname
              AND census.date = NEW.date
              AND census.grp = NEW.grp;
  );
SELECT '$1_insert_nocenid';

--  cenid is not null
CREATE OR REPLACE RULE $1_insert_cenid
  AS ON insert
  TO $1
  WHERE NEW.cenid IS NOT NULL
  DO ALSO (
    -- Either the insert or the update gets executed.
    -- Update goes first so it can test for existance.
    UPDATE census
      SET sname = CASE WHEN NEW.sname IS NULL
                       THEN (SELECT c.sname
                               FROM census AS c
                               WHERE c.cenid = NEW.cenid)
                       ELSE NEW.sname
                   END
        , date = CASE WHEN NEW.date IS NULL
                      THEN (SELECT c.date
                              FROM census AS c
                              WHERE c.cenid = NEW.cenid)
                      ELSE NEW.date
                 END
        , grp = CASE WHEN NEW.grp IS NULL
                     THEN (SELECT c.grp
                             FROM census AS c
                             WHERE c.cenid = NEW.cenid)
                     ELSE NEW.grp
                END
        , status = CASE WHEN NEW.status IS NULL
                        THEN (SELECT c.status
                                FROM census AS c
                                WHERE c.cenid = NEW.cenid)
                        ELSE NEW.status
                   END
        , cen = CASE WHEN NEW.cen IS NULL
                        THEN (SELECT c.cen
                                FROM census AS c
                                WHERE c.cenid = NEW.cenid)
                     ELSE NEW.cen
                 END
      WHERE cenid = NEW.cenid;

    INSERT INTO census (cenid, sname, date, grp, status, cen)
      SELECT NEW.cenid
           , NEW.sname
           , NEW.date
           , NEW.grp
           , CASE WHEN NEW.status IS NULL
                  THEN 'bb_demog'
                  ELSE NEW.status
             END
           , CASE WHEN NEW.cen IS NULL
                  THEN false
                  ELSE NEW.cen
             END
        WHERE NOT EXISTS (SELECT census.cenid
                            FROM census
                            WHERE census.cenid = NEW.cenid);

    INSERT INTO demog (cenid, reference, comment)
           VALUES(  NEW.cenid
                  , NEW.reference
                  , NEW.comment);
  );
SELECT '$1_insert_cenid';

-- update
CREATE OR REPLACE RULE $1_update
  AS ON update
  TO $1
  DO INSTEAD (
    UPDATE census
      SET cenid = NEW.cenid
        , sname = NEW.sname
        , date = NEW.date
        , grp = NEW.grp
        , status = NEW.status
        , cen = NEW.cen
      WHERE cenid = OLD.cenid;

    UPDATE demog
      SET cenid = NEW.cenid
        , reference = NEW.reference
        , comment = NEW.comment
      WHERE cenid = OLD.cenid;
  );
SELECT '$1_update';

-- delete
CREATE OR REPLACE RULE $1_delete
  AS ON delete
  TO $1
  DO INSTEAD (
    DELETE FROM demog WHERE cenid = OLD.cenid;
    DELETE FROM census
           WHERE cenid = OLD.cenid
                 AND NOT(census.cen);
  );
SELECT '$1_delete';

})dnl
changequote(`,')dnl

-- Make the demog_census-like views

-- demog_census
_m_demog_census(`demog_census')

-- demog_census_sorted
_m_demog_census(`demog_census_sorted')



dnl  Make a macro because we've also got a sorted view.

dnl sql fragment for creating cycpoints_cycles-like view rules.
dnl
dnl Syntax: _m_cycpoints_cycles(name)
dnl
dnl name   Name of the view.
dnl
changequote({,})
define({_m_cycpoints_cycles},{
--
-- $1
--

-- insert
-- Doesn't work because we can't return a rowtype or RECORD type
-- -- and use that to insert into a bunch of columns.
-- -- The _$1_insert function takes care of inserting
-- -- into cycles and we insert it's return value into cycpoints.
-- CREATE OR REPLACE RULE $1_insert
--   AS ON insert
--   TO $1
--   DO INSTEAD
--     INSERT INTO cycpoints (cpid, date, edate, ldate, code, source)
--            SELECT _$1_insert(NEW.*);

ALTER TABLE $1
    ALTER cpid SET DEFAULT nextval('cycpoints_cpid_seq')
  , ALTER source SET DEFAULT 'bb_data';

CREATE OR REPLACE RULE $1_insert
  AS ON insert
  TO $1
  DO INSTEAD NOTHING;
SELECT '$1_insert';

-- 4 cases:
-- cid is NULL, sname is not
-- sname is NULL, cid is not
-- sname and cid are NULL (lump in with prior case)
-- sname is not NULL, cid is not NULL

-- cid is NULL, sname is not
CREATE OR REPLACE RULE $1_insert_sname
  AS ON insert
  TO $1
  WHERE NEW.cid IS NULL
        AND NEW.sname IS NOT NULL
  DO ALSO (
    -- The easy way to do this would be to always create the
    -- cycles row, and then use that and let the triggers delete
    -- it if necessary, but let's not.

    -- Create a cycles row if none exists for the female.
    -- (There's a bug here in that we get no error if there's no
    -- rows on BIOGRAPH, but I'm not going to worry about that.)
    INSERT INTO cycles (sname)
      SELECT NEW.sname
        FROM biograph
        WHERE NOT EXISTS (SELECT 1
                            FROM cycles AS thisf
                            WHERE thisf.sname = NEW.sname)
        LIMIT 1;

    -- We must find some (any) cid to go with the supplied sname.
    -- Use a subquery so if we find none then we get a NULL cid
    -- and so get an error.
    INSERT INTO cycpoints (cpid, cid, date, edate, ldate, source, code)
      SELECT NEW.cpid
           , (SELECT cycles.cid
                FROM cycles
                WHERE cycles.sname = NEW.sname
                LIMIT 1)
           , NEW.date
           , NEW.edate
           , NEW.ldate
           , NEW.source
           , NEw.code;
  );
SELECT '$1_insert_sname';

-- sname is NULL
-- (Regardless of whether cid is NULL or not
-- so that we can get an error message if both are NULL.)
CREATE OR REPLACE RULE $1_insert_cid
  AS ON insert
  TO $1
  WHERE NEW.sname IS NULL
  DO ALSO
    INSERT INTO cycpoints (cpid, cid, date, edate, ldate, source, code)
      VALUES(NEW.cpid
           , NEW.cid
           , NEW.date
           , NEW.edate
           , NEW.ldate
           , NEW.source
           , NEw.code);
SELECT '$1_insert_cid';

-- sname is not NULL, cid is not NULL
CREATE OR REPLACE RULE $1_insert_both
  AS ON insert
  TO $1
  WHERE NEW.cid IS NOT NULL
        AND NEW.sname IS NOT NULL
  DO ALSO (
    -- Triggers don't allow the sname to change, so this will
    -- ensure that the sname goes with the cid.
    UPDATE cycles
      SET sname = NEW.sname
      WHERE cid = NEW.cid;

    -- Put the new cycpoints in.
    INSERT INTO cycpoints (cpid, cid, date, edate, ldate, source, code)
      VALUES(NEW.cpid
           , NEW.cid
           , NEW.date
           , NEW.edate
           , NEW.ldate
           , NEW.source
           , NEw.code);
  );
SELECT '$1_insert_both';

-- update
--
-- Do cycpoints first as that might change cycles.
CREATE OR REPLACE RULE $1_update
  AS ON update
  TO $1
  DO INSTEAD (
    UPDATE cycpoints
      SET cpid = NEW.cpid
        , date = NEW.date
        , edate = NEW.edate
        , ldate = NEW.ldate
        , code = NEW.code
        , source = NEW.source
      WHERE cpid = OLD.cpid;
    UPDATE cycles
      SET cid = NEW.cid
        , sname = NEW.sname
        --, seq = NEW.seq
        --, series = NEW.series
      WHERE cid = OLD.cid;
  );
SELECT '$1_update';

-- delete
CREATE OR REPLACE RULE $1_delete
  AS ON delete
  TO $1
  DO INSTEAD (
    DELETE FROM cycpoints
           WHERE cpid = OLD.cpid;
  );
SELECT '$1_delete';
})dnl
changequote(`,')dnl

-- Make the view and it's sorted form.
_m_cycpoints_cycles(`cycpoints_cycles')

_m_cycpoints_cycles(`cycpoints_cycles_sorted')




dnl sql fragment for creating cycles_sexskins-like view rules.
dnl
dnl Syntax: _m_cycles_sexskins(name)
dnl
dnl name   Name of the view.
dnl
changequote({,})
define({_m_cycles_sexskins},{
--
-- $1
--

-- insert
-- Same strategy as cycpoints_cycles
ALTER TABLE $1
  ALTER sxid SET DEFAULT nextval('sexskins_sxid_seq');

CREATE OR REPLACE RULE $1_insert
  AS ON insert
  TO $1
  DO INSTEAD NOTHING;

SELECT '$1_insert';

-- 4 cases:
-- cid is NULL, sname is not
-- sname is NULL, cid is not
-- sname and cid are NULL (lump in with prior case)
-- sname is not NULL, cid is not NULL

-- cid is NULL, sname is not
--
-- On insert we can pick any of the female's cycles.
-- Use a subquery so that we get NULL and thus an error message
-- when a female with no cycles is chosen.
--
CREATE OR REPLACE RULE $1_insert_sname
  AS ON insert
  TO $1
  WHERE NEW.cid IS NULL
        AND NEW.sname IS NOT NULL
  DO ALSO
    INSERT INTO sexskins (sxid, cid, date, size)
    SELECT NEW.sxid
         , (SELECT cycles.cid
              FROM cycles
              WHERE cycles.sname = NEW.sname 
              LIMIT 1)
        , NEW.date
        , NEW.size;
SELECT '$1_insert_sname';

--
-- sname is NULL, cid is not
-- sname and cid are NULL (lump in with prior case to get an error)
--
CREATE OR REPLACE RULE $1_insert_cid
  AS ON insert
  TO $1
  WHERE NEW.sname IS NULL
  DO ALSO
    INSERT INTO sexskins (sxid, cid, date, size)
      VALUES (NEW.sxid, NEW.cid, NEW.date, NEW.size);
SELECT '$1_insert_cid';

--
-- sname is not NULL, cid is not NULL
--
CREATE OR REPLACE RULE $1_insert_both
  AS ON insert
  TO $1
  WHERE NEW.cid IS NOT NULL
        AND NEW.sname IS NOT NULL
  DO ALSO (
    -- Triggers don't allow the sname to change, so this will
    -- ensure that the sname goes with the cid.
    UPDATE cycles
      SET sname = NEW.sname
      WHERE cid = NEW.cid;

    INSERT INTO sexskins (sxid, cid, date, size)
      VALUES (NEW.sxid, NEW.cid, NEW.date, NEW.size);
  );

SELECT '$1_insert_both';

dnl Update rules are made in another macro.

-- delete
CREATE OR REPLACE RULE $1_delete
  AS ON delete
  TO $1
  DO INSTEAD (
    DELETE FROM sexskins
           WHERE sxid = OLD.sxid;
  );
SELECT '$1_delete';

})dnl
changequote(`,')dnl


dnl sql fragment for creating cycles_sexskins_update-like view rules.
dnl
dnl Syntax: _m_cycles_sexskins_update(name)
dnl
dnl name   Name of the view.
dnl
changequote({,})
define({_m_cycles_sexskins_update},{
--
-- update
--
CREATE OR REPLACE RULE $1_update_both
  AS ON update
  TO $1
  DO INSTEAD NOTHING;

-- 4 cases, in order:
--  Both old and new have sexskins row
--  Neither old nor new have sexskins row
--  New has sexskins row but not old
--  Old has sexskins row but not new

--  Both old and new have sexskins row
CREATE OR REPLACE RULE $1_update_both
  AS ON update
  TO $1
  WHERE (OLD.date IS NOT NULL
         OR OLD.size IS NOT NULL)
        AND (NEW.date IS NOT NULL
             OR NEW.size IS NOT NULL)
  DO ALSO (
    UPDATE cycles
      SET cid = NEW.cid
        , sname = NEW.sname
      WHERE cid = OLD.cid;

    UPDATE sexskins
      SET sxid = NEW.sxid
        , date = NEW.date
        , size = NEW.size
      WHERE sxid = OLD.sxid;
  );
SELECT '$1_update_both';

--  Neither old nor new have sexskins row
CREATE OR REPLACE RULE $1_update_one
  AS ON update
  TO $1
  WHERE OLD.date IS NULL
        AND NEW.date IS NULL
        AND OLD.size IS NULL
        AND NEW.size IS NULL
  DO ALSO (
    UPDATE cycles
      SET cid = NEW.cid
        , sname = NEW.sname
      WHERE cid = OLD.cid;
  );
SELECT '$1_update_one';

--  New has sexskins row but not old
CREATE OR REPLACE RULE $1_update_create
  AS ON update
  TO $1
  WHERE OLD.date IS NULL
        AND OLD.size IS NULL
        AND (NEW.date IS NOT NULL
             OR NEW.size IS NOT NULL)
  DO ALSO (
    UPDATE cycles
      SET cid = NEW.cid
        , sname = NEW.sname
      WHERE cid = OLD.cid;

    INSERT INTO sexskins (cid, sxid, date, size)
      SELECT NEW.cid
           , NEW.sxid
           , NEW.date
           , NEW.size;
  );
SELECT '$1_update_create';

--  Old has sexskins row but not new
CREATE OR REPLACE RULE $1_update_destroy
  AS ON update
  TO $1
  WHERE (OLD.date IS NOT NULL
         OR OLD.size IS NOT NULL)
        AND NEW.date IS NULL
        AND NEW.size IS NULL
  DO ALSO (
    UPDATE cycles
      SET cid = NEW.cid
        , sname = NEW.sname
      WHERE cid = OLD.cid;

    DELETE FROM sexskins WHERE sxid = OLD.sxid;
  );
SELECT '$1_update_destory';

})dnl
changequote(`,')dnl



-- Make the view and it's sorted form.
_m_cycles_sexskins(`cycles_sexskins')
_m_cycles_sexskins_update(`cycles_sexskins')

_m_cycles_sexskins(`cycles_sexskins_sorted')
_m_cycles_sexskins_update(`cycles_sexskins_sorted')



dnl sql fragment for creating sexskins_cycles_update-like view rules.
dnl
dnl Syntax: _m_sexskins_cycles_update(name)
dnl
dnl name   Name of the view.
dnl
changequote({,})
define({_m_sexskins_cycles_update},{

--
-- update
--
CREATE OR REPLACE RULE $1_update
  AS ON update
  TO $1
  DO INSTEAD (
    UPDATE cycles
      SET cid = NEW.cid
        , sname = NEW.sname
      WHERE cid = OLD.cid;

    UPDATE sexskins
      SET sxid = NEW.sxid
        , date = NEW.date
        , size = NEW.size
      WHERE sxid = OLD.sxid;
  );
SELECT '$1_update';

})dnl
changequote(`,')dnl

-- Make the view and it's sorted form.
_m_cycles_sexskins(`sexskins_cycles')
_m_sexskins_cycles_update(`sexskins_cycles')

_m_cycles_sexskins(`sexskins_cycles_sorted')
_m_sexskins_cycles_update(`sexskins_cycles_sorted')


--
-- pcskins_sorted
--

-- insert
ALTER TABLE pcskins_sorted
  ALTER pcsid SET DEFAULT nextval('pcskins_pcsid_seq');

CREATE OR REPLACE RULE pcskins_sorted_insert
  AS ON INSERT
  TO pcskins_sorted
  DO INSTEAD
    INSERT INTO pcskins (pcsid, sname, date, color)
      VALUES (NEW.pcsid, NEW.sname, NEW.date, NEW.color);

-- update
CREATE OR REPLACE RULE pcskins_sorted_update
  AS ON UPDATE
  TO pcskins_sorted
  DO INSTEAD
    UPDATE pcskins
      SET pcsid = NEW.pcsid
        , sname = NEW.sname
        , date = NEW.date
        , color = NEW.color
      WHERE pcsid = OLD.pcsid;

-- delete
CREATE OR REPLACE RULE pcskins_sorted_delete
  AS ON DELETE
  TO pcskins_sorted
  DO INSTEAD
    DELETE FROM pcskins
      WHERE pcsid = OLD.pcsid;

--
-- mtd_cycles
--

-- insert
ALTER TABLE mtd_cycles
    ALTER mcpid SET DEFAULT nextval('cycpoints_cpid_seq')
  , ALTER tcpid SET DEFAULT nextval('cycpoints_cpid_seq')
  , ALTER dcpid SET DEFAULT nextval('cycpoints_cpid_seq')
  , ALTER msource SET DEFAULT 'bb_data'
  , ALTER tsource SET DEFAULT 'bb_data'
  , ALTER dsource SET DEFAULT 'bb_data';

-- 2 cases
---  cid is null
-- cid is not null

CREATE OR REPLACE RULE mtd_cycles_insert
  AS ON INSERT
  TO mtd_cycles
  DO INSTEAD NOTHING;
SELECT 'mtd_cycles_insert';

-- cid is null
-- Sadly, for each cycpoints row we waste a cid because
-- we never know what cid the triggers will
-- assign to a cycpoint row.
CREATE OR REPLACE RULE mtd_cycles_insert_nocid
  AS ON INSERT
  TO mtd_cycles
  WHERE NEW.cid IS NULL
  DO ALSO (
    -- cycles for mdate
    INSERT INTO cycles (cid, sname)
      SELECT nextval('cycles_cid_seq'), NEW.sname
        WHERE NEW.mdate IS NOT NULL
              OR NEW.emdate IS NOT NULL
              OR NEW.lmdate IS NOT NULL;
    -- mdate
    INSERT INTO cycpoints (cpid, cid, code, date, edate, ldate, source)
      SELECT NEW.mcpid
           , currval('cycles_cid_seq')
           , 'bb_menses'
           , NEW.mdate
           , NEW.emdate
           , NEw.lmdate
           , NEW.msource
        WHERE NEW.mdate IS NOT NULL
              OR NEW.emdate IS NOT NULL
              OR NEW.lmdate IS NOT NULL;

    -- cycles for tdate
    INSERT INTO cycles (cid, sname)
      SELECT nextval('cycles_cid_seq'), NEW.sname
        WHERE NEW.tdate IS NOT NULL
              OR NEW.etdate IS NOT NULL
              OR NEW.ltdate IS NOT NULL;
    -- tdate
    INSERT INTO cycpoints (cpid, cid, code, date, edate, ldate, source)
      SELECT NEW.tcpid
           , currval('cycles_cid_seq')
           , 'bb_turgesence'
           , NEW.tdate
           , NEW.etdate
           , NEw.ltdate
           , NEW.tsource
        WHERE NEW.tdate IS NOT NULL
              OR NEW.etdate IS NOT NULL
              OR NEW.ltdate IS NOT NULL;

    -- cycles for ddate
    INSERT INTO cycles (cid, sname)
      SELECT nextval('cycles_cid_seq'), NEW.sname
        WHERE NEW.ddate IS NOT NULL
              OR NEW.eddate IS NOT NULL
              OR NEW.lddate IS NOT NULL;
    -- ddate
    INSERT INTO cycpoints (cpid, cid, code, date, edate, ldate, source)
      SELECT NEW.dcpid
           , currval('cycles_cid_seq')
           , 'bb_deturgesence'
           , NEW.ddate
           , NEW.eddate
           , NEw.lddate
           , NEW.dsource
        WHERE NEW.ddate IS NOT NULL
              OR NEW.eddate IS NOT NULL
              OR NEW.lddate IS NOT NULL;
  );
SELECT 'mtd_cycles_insert_nocid';

-- cid is not null
CREATE OR REPLACE RULE mtd_cycles_insert_cid
  AS ON INSERT
  TO mtd_cycles
  WHERE NEW.cid IS NOT NULL
  DO ALSO (
    -- cycles
    INSERT INTO cycles (cid, sname)
      VALUES (NEW.cid, NEW.sname);
    -- mdate
    INSERT INTO cycpoints (cpid, cid, code, date, edate, ldate, source)
      SELECT NEW.mcpid
           , NEW.cid
           , 'bb_menses'
           , NEW.mdate
           , NEW.emdate
           , NEw.lmdate
           , NEW.msource
        WHERE NEW.mdate IS NOT NULL
              OR NEW.emdate IS NOT NULL
              OR NEW.lmdate IS NOT NULL;
    -- tdate
    INSERT INTO cycpoints (cpid, cid, code, date, edate, ldate, source)
      SELECT NEW.tcpid
           , NEW.cid
           , 'bb_turgesence'
           , NEW.tdate
           , NEW.etdate
           , NEw.ltdate
           , NEW.tsource
        WHERE NEW.tdate IS NOT NULL
              OR NEW.etdate IS NOT NULL
              OR NEW.ltdate IS NOT NULL;
    -- mdate
    INSERT INTO cycpoints (cpid, cid, code, date, edate, ldate, source)
      SELECT NEW.dcpid
           , NEW.cid
           , 'bb_deturgesence'
           , NEW.ddate
           , NEW.eddate
           , NEw.lddate
           , NEW.dsource
        WHERE NEW.ddate IS NOT NULL
              OR NEW.eddate IS NOT NULL
              OR NEW.lddate IS NOT NULL;
  );
SELECT 'mtd_cycles_insert_cid';

-- update
-- Not allowed.

-- delete
-- Delete in reverse-time order.
CREATE OR REPLACE RULE mtd_cycles_delete
  AS ON DELETE
  TO mtd_cycles
  DO INSTEAD (
    DELETE
      FROM cycpoints
      WHERE cycpoints.cpid = OLD.dcpid;
    DELETE
      FROM cycpoints
      WHERE cycpoints.cpid = OLD.tcpid;
    DELETE
      FROM cycpoints
      WHERE cycpoints.cpid = OLD.mcpid;
    DELETE
      FROM cycles
      WHERE cycles.cid = OLD.cid;
  );
SELECT 'mtd_cycles_delete';





dnl sql fragment for creating mpi_events-like view rules.
dnl
dnl Syntax: _m_mpi_events(name)
dnl
dnl name   Name of the view.
dnl
changequote({,})
define({_m_mpi_events},{
--
-- $1
--

-- insert

ALTER TABLE $1
   ALTER actorid SET DEFAULT nextval('mpi_parts_mpipid_seq')
 , ALTER acteeid SET DEFAULT nextval('mpi_parts_mpipid_seq');

CREATE OR REPLACE RULE $1_insert
  AS ON INSERT
  TO $1
  DO INSTEAD NOTHING;
SELECT '$1_insert';

-- 4 cases
--  mpiid is null, mpidid is null
--  mpiid is null, mpidid is not null
--  mpiiid is not null, mpidid is null
--  mpiiid is not null, mpidid is not null

--  mpiid is null, mpidid is null
--  mpiid is null means make a new mpis row.
CREATE OR REPLACE RULE $1_insert_nompiid_nompidid
  AS ON INSERT
  TO $1
  WHERE NEW.mpiid IS NULL
        AND NEW.mpidid IS NULL
  DO ALSO (
    INSERT INTO mpis (mpiid, date, context_type, context)
      VALUES ( nextval('mpis_mpiid_seq')
             , NEW.date
             , NEW.context_type
             , NEW.context);
    INSERT INTO mpi_data (mpiid, mpidid, mpiact, seq, helped, active)
      VALUES ( currval('mpis_mpiid_seq')
             , nextval('mpi_data_mpidid_seq')
             , NEW.mpiact
             , NEW.seq
             , NEW.helped
             , NEW.active);
    INSERT INTO mpi_parts (mpipid, mpidid, role, sname, unksname)
      VALUES (NEW.actorid
            , currval('mpi_data_mpidid_seq')
            , 'bb_actor'
            , NEW.actor
            , NEW.unkactor);
    INSERT INTO mpi_parts (mpipid, mpidid, role, sname, unksname)
      VALUES (NEW.acteeid
            , currval('mpi_data_mpidid_seq')
            , 'bb_actee'
            , NEW.actee
            , NEW.unkactee);
  );
SELECT '$1_insert_nompiid_nompidid';

--  mpiid is null, mpidid is not null
--  mpiid is null means make a new mpis row.
CREATE OR REPLACE RULE $1_insert_nompiid_mpidid
  AS ON INSERT
  TO $1
  WHERE NEW.mpiid IS NULL
        AND NEW.mpidid IS NOT NULL
  DO ALSO (
    INSERT INTO mpis (mpiid, date, context_type, context)
      VALUES ( nextval('mpis_mpiid_seq')
             , NEW.date
             , NEW.context_type
             , NEW.context);
    INSERT INTO mpi_data (mpiid, mpidid, mpiact, seq, helped, active)
      VALUES ( currval('mpis_mpiid_seq')
             , NEW.mpidid
             , NEW.mpiact
             , NEW.seq
             , NEW.helped
             , NEW.active);
    INSERT INTO mpi_parts (mpipid, mpidid, role, sname, unksname)
      VALUES (NEW.actorid
            , NEW.mpidid
            , 'bb_actor'
            , NEW.actor
            , NEW.unkactor);
    INSERT INTO mpi_parts (mpipid, mpidid, role, sname, unksname)
      VALUES (NEW.acteeid
            , NEW.mpidid
            , 'bb_actee'
            , NEW.actee
            , NEW.unkactee);
  );
SELECT '$1_insert_nompiid_mpidid';

--  mpiid is not null, mpidid is null
--  Use existing mpis, ignore supplied date, context_type, and context.
CREATE OR REPLACE RULE $1_insert_mpiid_nompidid
  AS ON INSERT
  TO $1
  WHERE NEW.mpiid IS NOT NULL
        AND NEW.mpidid IS NULL
  DO ALSO (
    INSERT INTO mpi_data (mpiid, mpidid, mpiact, seq, helped, active)
      VALUES ( NEW.mpiid
             , nextval('mpi_data_mpidid_seq')
             , NEW.mpiact
             , NEW.seq
             , NEW.helped
             , NEW.active);
    INSERT INTO mpi_parts (mpipid, mpidid, role, sname, unksname)
      VALUES ( NEW.actorid
             , currval('mpi_data_mpidid_seq')
             , 'bb_actor'
             , NEW.actor
             , NEW.unkactor);
    INSERT INTO mpi_parts (mpipid, mpidid, role, sname, unksname)
      VALUES ( NEW.acteeid
             , currval('mpi_data_mpidid_seq')
             , 'bb_actee'
             , NEW.actee
             , NEW.unkactee);
  );
SELECT '$1_insert_mpiid_nompidid';

--  mpiid is not null, mpidid is not null
--  Use existing mpis, ignore supplied date, context_type, context.
CREATE OR REPLACE RULE $1_insert_mpiid_mpidid
  AS ON INSERT
  TO $1
  WHERE NEW.mpiid IS NOT NULL
        AND NEW.mpidid IS NOT NULL
  DO ALSO (
    INSERT INTO mpi_data (mpiid, mpidid, mpiact, seq, helped, active)
      VALUES ( NEW.mpiid
             , NEW.mpidid
             , NEW.mpiact
             , NEW.seq
             , NEW.helped
             , NEW.active);
    INSERT INTO mpi_parts (mpipid, mpidid, role, sname, unksname)
      VALUES ( NEW.actorid
             , NEW.mpidid
             , 'bb_actor'
             , NEW.actor
             , NEW.unkactor);
    INSERT INTO mpi_parts (mpipid, mpidid, role, sname, unksname)
      VALUES ( NEW.acteeid
             , NEW.mpidid
             , 'bb_actee'
             , NEW.actee
             , NEW.unkactee);
  );
SELECT '$1_insert_mpiid_mpidid';

--
-- update
--
CREATE OR REPLACE RULE $1_update
  AS ON UPDATE
  TO $1
  DO INSTEAD (
    UPDATE mpis
      SET mpiid = NEW.mpiid
        , date = NEW.date
        , context_type = NEW.context_type
        , context = NEW.context
      WHERE mpiid = OLD.mpiid;
    UPDATE mpi_data
      SET mpidid = NEW.mpidid
        , mpiact = NEW.mpiact
        , seq = NEW.seq
        , helped = NEW.helped
        , active = NEW.active
      WHERE mpidid = OLD.mpidid;
    UPDATE mpi_parts
      SET mpipid = NEW.actorid
        , mpidid = NEW.mpidid
        , sname = NEW.actor
        , unksname = NEW.unkactor
      WHERE mpipid = OLD.actorid;
    UPDATE mpi_parts
      SET mpipid = NEW.acteeid
        , mpidid = NEW.mpidid
        , sname = NEW.actee
        , unksname = NEW.actee
      WHERE mpipid = OLD.acteeid;
  );
SELECT '$1_update';

--
-- delete
--
CREATE OR REPLACE RULE $1_delete
  AS ON DELETE
  TO $1
  DO INSTEAD (
    DELETE FROM mpi_parts
      WHERE mpidid = OLD.mpidid;
    DELETE FROM mpi_data
      WHERE mpidid = OLD.mpidid;
    DELETE FROM mpis
      WHERE mpiid = OLD.mpiid
            AND NOT EXISTS (SELECT 1
                              FROM mpi_data
                              WHERE mpi_data.mpiid = OLD.mpiid);
  );
SELECT '$1_delete';

})dnl
changequote(`,')dnl

-- Make the view.
_m_mpi_events(`mpi_events')


dnl sql fragment for creating min_maxs-like view rules.
dnl
dnl Syntax: _m_min_maxs(name, wreadings_macro)
dnl
dnl name   Name of the view.
dnl wreadings_macro  Name of the macro used to make the wreadings
dnl                  insert rule.  This macro takes one argument,
dnl                  the name of the view.
dnl
changequote({,})
define({_m_min_maxs},{
--
-- $1
--

ALTER TABLE $1
   ALTER wrid SET DEFAULT nextval('wreadings_wrid_seq');

-- Insert
-- 5 rules.  A default for the view and one for each of the
-- tables in the view.  Note that because all the manipulation
-- happens in the parser you can't use NEW.wrid because that
-- will result in another call to nextval() for the key.
-- This means that inserting via the view must either always
-- compute a wrid automatically or never do so.  We choose
-- to always compute the wrid.
-- Note that rules are applied in alhpabetical order by rule name.
-- We want to make sure that the rule that inserts the wreadings
-- always gets fired first so that we can use currval().

-- The default rule that tell's postgres the view won't be updated.
CREATE OR REPLACE RULE $1_00
  AS ON INSERT
  TO $1
  DO INSTEAD NOTHING;
SELECT '$1_00';

--
-- Wreadings
--

-- Wreadings
CREATE OR REPLACE RULE $1_01_wreadings
  AS ON INSERT
  TO $1
  DO ALSO (
    INSERT INTO wreadings(wstation
                          , wrdaytime
                          , estdaytime
                          , wrperson
                          , wrnotes)
      VALUES(NEW.wstation
             , NEW.wrdaytime
             , NEW.estdaytime
             , NEW.wrperson
             , NEW.wrnotes);
);
SELECT '$1_01_wreadings';

-- -- Wreadings using the wrdaytime value.
-- CREATE OR REPLACE RULE $1_01_wreadings_wrdaytime
--   AS ON INSERT
--   TO $1
--   WHERE NEW.wrdaytime IS NOT NULL
--   DO ALSO (
--     INSERT INTO wreadings(wstation
--                           , wrdaytime
--                           , estdaytime
--                           , wrperson
--                           , wrnotes)
--       VALUES(NEW.wstation
--              , NEW.wrdaytime
--              , NEW.estdaytime
--              , NEW.wrperson
--              , NEW.wrnotes);
-- );
-- SELECT '$1_01_wreadings_wrdaytime';
-- 
-- CREATE OR REPLACE RULE $1_01_wreadings_wrday_wrtime
--   AS ON INSERT
--   TO $1
--   WHERE NEW.wrdaytime IS NULL
--   DO ALSO (
--     INSERT INTO wreadings(wstation
--                           , wrdaytime
--                           , estdaytime
--                           , wrperson
--                           , wrnotes)
--       VALUES(NEW.wstation
--              , CAST(NEW.wrdate || ' ' || NEW.wrtime AS timestamp)
--              , NEW.estdaytime
--              , NEW.wrperson
--              , NEW.wrnotes);
-- );
-- SELECT '$1_01_wreadings_wrday_wrtime';

-- Max
CREATE OR REPLACE RULE $1_02_tempmaxs
  AS ON INSERT
  TO $1
  WHERE NEW.tempmax IS NOT NULL
  DO ALSO (
    INSERT INTO tempmaxs(wrid, tempmax)
      VALUES(currval('wreadings_wrid_seq')
             , NEW.tempmax);
);
SELECT '$1_02_tempmaxs';

-- Min
CREATE OR REPLACE RULE $1_03_tempmins
  AS ON INSERT
  TO $1
  WHERE NEW.tempmin IS NOT NULL
  DO ALSO (
    INSERT INTO tempmins(wrid, tempmin)
      VALUES(currval('wreadings_wrid_seq')
             , NEW.tempmin);
);
SELECT '$1_03_tempmins';

-- Raingauges
CREATE OR REPLACE RULE $1_04_raingauges
  AS ON INSERT
  TO $1
  WHERE NEW.rain IS NOT NULL
        OR NEW.rgspan IS NOT NULL
        OR NEW.estrgspan IS NOT NULL
  DO ALSO (
    INSERT INTO raingauges(wrid, rgspan, estrgspan, rain)
      VALUES(currval('wreadings_wrid_seq')
             , NEW.rgspan
             , NEW.estrgspan
             , NEW.rain);
);
SELECT '$1_04_raingauges';

-- Update not allowed.

-- Delete
CREATE OR REPLACE RULE $1_delete
  AS ON DELETE
  TO $1
  DO INSTEAD (
    DELETE FROM raingauges
      WHERE raingauges.wrid = OLD.wrid;
    DELETE FROM tempmins
      WHERE tempmins.wrid = OLD.wrid;
    DELETE FROM tempmaxs
      WHERE tempmaxs.wrid = OLD.wrid;
    DELETE FROM wreadings
      WHERE wreadings.wrid = OLD.wrid;
);
})dnl
changequote(`,')dnl

-- Make the views.
_m_min_maxs(`min_maxs')
_m_min_maxs(`min_maxs_sorted')

--
-- Dartings
--

-- Dent_codes
SELECT 'dent_codes';

-- Insert
CREATE OR REPLACE RULE dent_codes_insert
  AS ON INSERT
  TO dent_codes
  DO INSTEAD (
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'rum3', NEW.rum3tstate, NEW.rum3tcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'rum2', NEW.rum2tstate, NEW.rum2tcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'rum1', NEW.rum1tstate, NEW.rum1tcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'rup2', NEW.rup2tstate, NEW.rup2tcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'rup1', NEW.rup1tstate, NEW.rup1tcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid
              , 'ruc'
              , NEW.ructstate
              , NEW.ructcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'rui2', NEW.rui2tstate, NEW.rui2tcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'rui1', NEW.rui1tstate, NEW.rui1tcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'lui1', NEW.lui1tstate, NEW.lui1tcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'lui2', NEW.lui2tstate, NEW.lui2tcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid
              , 'luc'
              , NEW.luctstate
              , NEW.luctcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'lup1', NEW.lup1tstate, NEW.lup1tcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'lup2', NEW.lup2tstate, NEW.lup2tcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'lum1', NEW.lum1tstate, NEW.lum1tcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'lum2', NEW.lum2tstate, NEW.lum2tcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'lum3', NEW.lum3tstate, NEW.lum3tcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'llm3', NEW.llm3tstate, NEW.llm3tcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'llm2', NEW.llm2tstate, NEW.llm2tcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'llm1', NEW.llm1tstate, NEW.llm1tcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'llp2', NEW.llp2tstate, NEW.llp2tcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'llp1', NEW.llp1tstate, NEW.llp1tcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid
              , 'llc'
              , NEW.llctstate
              , NEW.llctcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'lli2', NEW.lli2tstate, NEW.lli2tcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'lli1', NEW.lli1tstate, NEW.lli1tcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'rli1', NEW.rli1tstate, NEW.rli1tcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'rli2', NEW.rli2tstate, NEW.rli2tcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid
              , 'rlc'
              , NEW.rlctstate
              , NEW.rlctcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'rlp1', NEW.rlp1tstate, NEW.rlp1tcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'rlp2', NEW.rlp2tstate, NEW.rlp2tcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'rlm1', NEW.rlm1tstate, NEW.rlm1tcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'rlm2', NEW.rlm2tstate, NEW.rlm2tcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'rlm3', NEW.rlm3tstate, NEW.rlm3tcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'drum2', NEW.drum2tstate, NEW.drum2tcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'drum1', NEW.drum1tstate, NEW.drum1tcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid
              , 'druc'
              , NEW.dructstate
              , NEW.dructcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'drui2', NEW.drui2tstate, NEW.drui2tcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'drui1', NEW.drui1tstate, NEW.drui1tcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'dlui1', NEW.dlui1tstate, NEW.dlui1tcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'dlui2', NEW.dlui2tstate, NEW.dlui2tcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid
              , 'dluc'
              , NEW.dluctstate
              , NEW.dluctcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'dlum1', NEW.dlum1tstate, NEW.dlum1tcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'dlum2', NEW.dlum2tstate, NEW.dlum2tcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'dllm2', NEW.dllm2tstate, NEW.dllm2tcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'dllm1', NEW.dllm1tstate, NEW.dllm1tcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid
              , 'dllc'
              , NEW.dllctstate
              , NEW.dllctcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'dlli2', NEW.dlli2tstate, NEW.dlli2tcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'dlli1', NEW.dlli1tstate, NEW.dlli1tcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'drli1', NEW.drli1tstate, NEW.drli1tcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'drli2', NEW.drli2tstate, NEW.drli2tcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid
              , 'drlc'
              , NEW.drlctstate
              , NEW.drlctcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'drlm1', NEW.drlm1tstate, NEW.drlm1tcondition);
    INSERT INTO teeth (dartid, tooth, tstate, tcondition)
      VALUES (NEW.dartid, 'drlm2', NEW.drlm2tstate, NEW.drlm2tcondition);
);

-- Update not allowed.

-- Delete
CREATE OR REPLACE RULE dent_codes_delete
  AS ON DELETE
  TO dent_codes
  DO INSTEAD (
    DELETE FROM teeth
      WHERE teeth.dartid = OLD.dartid;
);

--
-- Defaults for when we use triggers instead of views.
--

-- interact
SELECT 'interact';
ALTER TABLE interact
  ALTER COLUMN iid
    SET DEFAULT nextval('interact_data_iid_seq'::regclass);

-- interact_sorted
SELECT 'interact_sorted';
ALTER TABLE interact_sorted
  ALTER COLUMN iid
    SET DEFAULT nextval('interact_data_iid_seq'::regclass);

--
-- SWERB
--

-- swerb_departs
SELECT 'swerb_departs';
ALTER TABLE swerb_departs
  ALTER COLUMN did
    SET DEFAULT nextval('swerb_departs_data_did_seq'::regclass);


-- swerb_data_xy
SELECT 'swerb_data_xy';
ALTER TABLE swerb_data_xy
  ALTER COLUMN swid 
    SET DEFAULT nextval('swerb_data_swid_seq'::regclass);


-- swerb_gw_loc_data_xy
SELECT 'swerb_gw_loc_data_xy';
ALTER TABLE swerb_gw_loc_data_xy
  ALTER COLUMN sgwlid 
    SET DEFAULT nextval('swerb_gw_loc_data_sgwlid_seq'::regclass);

