dnl Copyright (C) 2012 The Meme Factory, Inc., http://www.meme.com/
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Functions and triggers to allow insert, update, and delete into
dnl the swerb_data_xy view.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl
dnl
dnl m4 includes
include(`copyright.m4')
include(`constants.m4')
include(`macros.m4')
dnl

SELECT 'swerb_data_xy_insert_func()';
CREATE OR REPLACE FUNCTION swerb_data_xy_insert_func()
  RETURNS trigger
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- For triggers instead of insert into the swerb_data_xy view.
  --
  -- GPL_notice(`  --', `2012', `The Meme Factory, Inc.  http://www.meme.com/')
  --
  -- Remarks:
  --
  -- Bugs:

  DECLARE

  BEGIN
    INSERT
      INTO swerb_data (swid
                     , beid
                     , seen_grp
                     , event
                     , time
                     , quad
                     , xyloc
                     , altitude
                     , pdop
                     , accuracy
                     , ogdistance
                     , subgroup
                     , gps_datetime
                     , garmincode)
      VALUES (NEW.swid
            , NEW.beid
            , NEW.seen_grp
            , NEW.event
            , NEW.time
            , NEW.quad
            , bb_makepoint(NEW.x, NEW.y)
            , NEW.altitude
            , NEW.pdop
            , NEW.accuracy
            , NEW.ogdistance
            , NEW.subgroup
            , NEW.gps_datetime
            , NEW.garmincode);

  RETURN NULL;
  END;
$$;

SELECT 'swerb_data_xy_update_func()';
CREATE OR REPLACE FUNCTION swerb_data_xy_update_func()
  RETURNS trigger
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- For triggers instead of update of the swerb_data_xy view.
  --
  -- GPL_notice(`  --', `2012', `The Meme Factory, Inc.  http://www.meme.com/')
  --
  -- Remarks:
  --
  -- Bugs:

  DECLARE

  BEGIN
    UPDATE swerb_data
      SET swid = NEW.swid
        , beid = NEW.beid
        , seen_grp = NEW.seen_grp
        , event = NEW.event
        , time = NEW.time
        , quad = NEW.quad
        , xyloc = bb_makepoint(NEW.x, NEW.y)
        , altitude = NEW.altitude
        , pdop = NEW.pdop
        , accuracy = NEW.accuracy
        , ogdistance = NEW.ogdistance
        , subgroup = NEW.subgroup
        , gps_datetime = NEW.gps_datetime
        , garmincode = NEW.garmincode
      WHERE swid = OLD.swid;

  RETURN NULL;
  END;
$$;

SELECT 'swerb_data_xy_delete_func()';
CREATE OR REPLACE FUNCTION swerb_data_xy_delete_func()
  RETURNS trigger
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- For triggers instead of delete of the swerb_data_xy view.
  --
  -- GPL_notice(`  --', `2012', `The Meme Factory, Inc.  http://www.meme.com/')
  --
  -- Remarks:
  --
  -- Bugs:

  DECLARE

  BEGIN
    DELETE
      FROM swerb_data
      WHERE swid = OLD.swid;

  RETURN NULL;
  END;
$$;


SELECT 'swerb_data_xy_insert_trigger' AS trigger;
CREATE TRIGGER swerb_data_xy_insert_trigger
  INSTEAD OF INSERT
  ON swerb_data_xy FOR EACH ROW
  EXECUTE PROCEDURE swerb_data_xy_insert_func();

SELECT 'swerb_data_xy_update_trigger' AS trigger;
CREATE TRIGGER swerb_data_xy_update_trigger
  INSTEAD OF UPDATE
  ON swerb_data_xy FOR EACH ROW
  EXECUTE PROCEDURE swerb_data_xy_update_func();

SELECT 'swerb_data_xy_delete_trigger' AS trigger;
CREATE TRIGGER swerb_data_xy_delete_trigger
  INSTEAD OF DELETE
  ON swerb_data_xy FOR EACH ROW
  EXECUTE PROCEDURE swerb_data_xy_delete_func();
