dnl Copyright (C) 2012, 2013 The Meme Factory, Inc., http://www.meme.com/
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Functions and triggers to allow insert, update, and delete into
dnl the swerb_departs view.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl
dnl
dnl m4 includes
include(`copyright.m4')
include(`constants.m4')
include(`macros.m4')
dnl

SELECT 'swerb_departs_insert_func()';
CREATE OR REPLACE FUNCTION swerb_departs_insert_func()
  RETURNS trigger
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- For triggers instead of insert into the swerb_departs view.
  --
  -- GPL_notice(`  --', `2012', `The Meme Factory, Inc.  http://www.meme.com/')
  --
  -- Remarks:
  --
  -- Bugs:

  DECLARE

  BEGIN
    INSERT INTO swerb_departs_data (did, date, time)
      VALUES(NEW.did, NEW.date, NEW.time);

    IF (NEW.x IS NOT NULL
        OR NEW.y IS NOT NULL
        OR NEW.altitude IS NOT NULL
        OR NEW.pdop IS NOT NULL
        OR NEW.accuracy IS NOT NULL
        OR NEW.gps IS NOT NULL
        OR NEW.garmincode IS NOT NULL) THEN
      -- Some value(s) have been supplied for SWERB_DEPARTS_GPS.
      -- Insert them and see how the database likes it.
      INSERT INTO swerb_departs_gps (did
                                   , xyloc
                                   , altitude
                                   , pdop
                                   , accuracy
                                   , gps
                                   , garmincode)
        VALUES (NEW.did
              , bb_makepoint(NEW.x, NEW.y)
              , NEW.altitude
              , NEW.pdop
              , NEW.accuracy
              , NEW.gps
              , NEW.garmincode);
    END IF;

  RETURN NULL;
  END;
$$;

SELECT 'swerb_departs_update_func()';
CREATE OR REPLACE FUNCTION swerb_departs_update_func()
  RETURNS trigger
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- For triggers instead of update of the swerb_departs view.
  --
  -- GPL_notice(`  --', `2012', `The Meme Factory, Inc.  http://www.meme.com/')
  --
  -- Remarks:
  --
  -- Bugs:

  DECLARE

  BEGIN
    UPDATE swerb_departs_gps
      SET did = NEW.did
        , xyloc = bb_makepoint(NEW.x, NEW.y)
        , altitude = NEW.altitude
        , pdop = NEW.pdop
        , accuracy = NEW.accuracy
        , gps = NEW.gps
        , garmincode = NEW.garmincode
      WHERE did = OLD.did;

    UPDATE swerb_departs_data
      SET did = NEW.did
        , date = NEW.date
        , time = NEW.time
      WHERE did = OLD.did;

  RETURN NULL;
  END;
$$;

SELECT 'swerb_departs_delete_func()';
CREATE OR REPLACE FUNCTION swerb_departs_delete_func()
  RETURNS trigger
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- For triggers instead of delete of the swerb_departs view.
  --
  -- GPL_notice(`  --', `2012, 2013', `The Meme Factory, Inc.  http://www.meme.com/')
  --
  -- Remarks:
  --
  -- Bugs:

  DECLARE

  BEGIN

    -- swerb_loc_gps
    DELETE
      FROM swerb_loc_gps
      USING swerb_bes
              JOIN swerb_data ON (swerb_data.beid = swerb_bes.beid)
      WHERE swerb_bes.did = OLD.did
            AND swerb_loc_gps.swid = swerb_data.swid;

    -- swerb_loc_data
    DELETE
      FROM swerb_loc_data
      USING swerb_bes
              JOIN swerb_data ON (swerb_data.beid = swerb_bes.beid)
      WHERE swerb_bes.did = OLD.did
            AND swerb_loc_data.swid = swerb_data.swid;

    -- swerb_data
    DELETE
      FROM swerb_data
      USING swerb_bes
      WHERE swerb_bes.did = OLD.did
            AND swerb_data.beid = swerb_bes.beid;

    -- swerb_bes
    DELETE
      FROM swerb_bes
      WHERE swerb_bes.did = OLD.did;

    -- swerb_departs_gps
    DELETE
      FROM swerb_departs_gps
      WHERE did = OLD.did;

    -- swerb_observers
    DELETE
      FROM swerb_observers
      WHERE did = OLD.did;

    -- swerb_departs_data
    DELETE
      FROM swerb_departs_data
      WHERE did = OLD.did;

  RETURN NULL;
  END;
$$;


SELECT 'swerb_departs_insert_trigger' AS trigger;
CREATE TRIGGER swerb_departs_insert_trigger
  INSTEAD OF INSERT
  ON swerb_departs FOR EACH ROW
  EXECUTE PROCEDURE swerb_departs_insert_func();

SELECT 'swerb_departs_update_trigger' AS trigger;
CREATE TRIGGER swerb_departs_update_trigger
  INSTEAD OF UPDATE
  ON swerb_departs FOR EACH ROW
  EXECUTE PROCEDURE swerb_departs_update_func();

SELECT 'swerb_departs_delete_trigger' AS trigger;
CREATE TRIGGER swerb_departs_delete_trigger
  INSTEAD OF DELETE
  ON swerb_departs FOR EACH ROW
  EXECUTE PROCEDURE swerb_departs_delete_func();
