dnl Copyright (C) 2012, 2013 The Meme Factory, Inc., http://www.meme.com/
dnl
dnl    This file is part of Babase.
dnl
dnl    Babase is free software; you can redistribute it and/or modify
dnl    it under the terms of the GNU General Public License as published by
dnl    the Free Software Foundation; either version 3 of the License, or
dnl    (at your option) any later version.
dnl
dnl    This program is distributed in the hope that it will be useful,
dnl    but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl    GNU General Public License for more details.
dnl
dnl    You should have received a copy of the GNU General Public License
dnl    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl Functions and triggers to allow insert, update, and delete into
dnl the swerb_gw_loc_data_xy view.
dnl
dnl Karl O. Pinc <kop@meme.com>
dnl
dnl
dnl m4 includes
include(`copyright.m4')
include(`constants.m4')
include(`macros.m4')
dnl

SELECT 'swerb_gw_loc_data_xy_insert_func()';
CREATE OR REPLACE FUNCTION swerb_gw_loc_data_xy_insert_func()
  RETURNS trigger
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- For triggers instead of insert into the swerb_gw_loc_data_xy view.
  --
  -- GPL_notice(`  --', `2012, 2013', `The Meme Factory, Inc.  http://www.meme.com/')
  --
  -- Remarks:
  --
  -- Bugs:

  DECLARE

  BEGIN
    INSERT
      INTO swerb_gw_loc_data (sgwlid
                     , loc
                     , date
                     , time
                     , quad
                     , xysource
                     , xyloc
                     , altitude
                     , pdop
                     , accuracy
                     , gps
                     , notes)
      VALUES (NEW.sgwlid
            , NEW.loc
            , NEW.date
            , NEW.time
            , NEW.quad
            , NEW.xysource
            , bb_makepoint(NEW.x, NEW.y)
            , NEW.altitude
            , NEW.pdop
            , NEW.accuracy
            , NEW.gps
            , NEW.notes);

  RETURN NULL;
  END;
$$;

SELECT 'swerb_gw_loc_data_xy_update_func()';
CREATE OR REPLACE FUNCTION swerb_gw_loc_data_xy_update_func()
  RETURNS trigger
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- For triggers instead of update of the swerb_gw_loc_data_xy view.
  --
  -- GPL_notice(`  --', `2012, 2013', `The Meme Factory, Inc.  http://www.meme.com/')
  --
  -- Remarks:
  --
  -- Bugs:

  DECLARE

  BEGIN
    UPDATE swerb_gw_loc_data
      SET sgwlid = NEW.sgwlid
        , loc = NEW.loc
        , date = NEW.date
        , time = NEW.time
        , quad = NEW.quad
        , xysource = NEW.xysource
        , xyloc = bb_makepoint(NEW.x, NEW.y)
        , altitude = NEW.altitude
        , pdop = NEW.pdop
        , accuracy = NEW.accuracy
        , gps = NEW.gps
        , notes = NEW.notes
      WHERE sgwlid = OLD.sgwlid;

  RETURN NULL;
  END;
$$;

SELECT 'swerb_gw_loc_data_xy_delete_func()';
CREATE OR REPLACE FUNCTION swerb_gw_loc_data_xy_delete_func()
  RETURNS trigger
  LANGUAGE plpgsql
  SECURITY DEFINER
  bb_function_set_search_path
  AS $$
  -- For triggers instead of delete of the swerb_gw_loc_data_xy view.
  --
  -- GPL_notice(`  --', `2012', `The Meme Factory, Inc.  http://www.meme.com/')
  --
  -- Remarks:
  --
  -- Bugs:

  DECLARE

  BEGIN
    DELETE FROM swerb_gw_loc_data
      WHERE sgwlid = OLD.sgwlid;

  RETURN NULL;
  END;
$$;


SELECT 'swerb_gw_loc_data_xy_insert_trigger' AS trigger;
CREATE TRIGGER swerb_gw_loc_data_xy_insert_trigger
  INSTEAD OF INSERT
  ON swerb_gw_loc_data_xy FOR EACH ROW
  EXECUTE PROCEDURE swerb_gw_loc_data_xy_insert_func();

SELECT 'swerb_gw_loc_data_xy_update_trigger' AS trigger;
CREATE TRIGGER swerb_gw_loc_data_xy_update_trigger
  INSTEAD OF UPDATE
  ON swerb_gw_loc_data_xy FOR EACH ROW
  EXECUTE PROCEDURE swerb_gw_loc_data_xy_update_func();

SELECT 'swerb_gw_loc_data_xy_delete_trigger' AS trigger;
CREATE TRIGGER swerb_gw_loc_data_xy_delete_trigger
  INSTEAD OF DELETE
  ON swerb_gw_loc_data_xy FOR EACH ROW
  EXECUTE PROCEDURE swerb_gw_loc_data_xy_delete_func();
