<?php
/* Copyright (C) 2012, 2015 The Meme Factory, Inc.  http://www.meme.com/
 * Copyright (C) 2006-2011, Karl O. Pinc <kop@meme.com>
 *
 *    This file is part of Babase.
 *
 *    Babase is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Upload darting related data.
 *
 * Karl O. Pinc <kop@meme.com>
 *
 * Remarks:
 *  Does not do anything to protect against SQL injection attacks,
 * but this does not matter as anybody with a login has restricted
 * permissions anyhow.
 *
 * Bugs:
 *  Does two queries to check the name and sex instead of one.
 */

// Includes
require('constants.php.inc');
require('lib.php.inc');
require('lib_upload.php.inc');
require('lib_darting.php.inc');

//
// Constants
//

// Delimiter separating data elements in data file that's input.
define('DELIM', "\t");

// SQL stuff

// How a null value is encoded.
define('NULL', '');

// Use dollar quoting to avoid escape and single quote issues.
define('DQ', '$$');

// Tables in the input file.
define('DARTINGS', 'dartings'); // Dartings must be written first or last, depending
define('TEETH', 'teeth');       // Teeth column names have toothcodes in them

// Columns in the input file.
define('SNAME_COL', 1);  // Sorta a hack to hardcode, but ...
define('DARTDATE_COL', 3);  // Ditto.

// What sort of darting data are we uploading.
define('DARTLOG', 'log');
define('DARTMORPHO', 'morpho');
define('DARTPHYSIO', 'physio');
define('DARTSAMP', 'samp');
define('DARTTEETH', 'teeth');
define('DARTTESTESARC', 'testes_arc');
define('DARTTESTESDIAM', 'testes_diam');

//
// Functions.
// 

function upload_dartdata(&$dartdata_form, &$dbconn, $darttype) {
  /*
   * Upload the census data and show the result.
   *
   * Input:
   *  $dartdata_form  An instance of the Upload_form class.
   *  $dbconn  An instance of the Upload_connection class.
   *  $darttype  The type of darting data uploaded.
   *
   * Returns: n/a
   *
   * Side Effects:
   *   Outputs html.
   *
   * Remarks:
   *  One of the main pages of the program.
   *
   *  Mostly, as far as $darttype goes, we care if we're uploading
   * logistic data because that creates the DARTINGS row.
   */

  // Generic testes upload template.
  // The 'testes' table name is later replaced with the actual name of the
  // particular type of testes table.
  $testes_headers = array(array('table' => '',          'col' => ''     , 'hdr' => 'name')
                     , array('table' => ''       , 'col' => 'sname', 'hdr' => 'sname')
                     , array('table' => ''       , 'col' => ''     , 'hdr' => 'sex')
                     , array('table' => ''       , 'col' => ''     , 'hdr' => 'dartdate')
                     , array('table' => 'testes'                     , 'hdr' =>
                             array(array('col' => 'testlength', 'hdr' => 'ltesteslength')
                                   , array('col' => 'testwidth', 'hdr' => 'ltesteswidth')
                                   , array('col' => 'testside', 'hdr' => 'ltestesside')))
                     , array('table' => 'testes'                     , 'hdr' =>
                             array(array('col' => 'testlength', 'hdr' => 'rtesteslength')
                                   , array('col' => 'testwidth', 'hdr' => 'rtesteswidth')
                                   , array('col' => 'testside', 'hdr' => 'rtestesside')))
                     , array('table' => DARTINGS , 'col' => 'testesnotes' , 'hdr' => 'other_notes_measures'));

  // Get the uploaded darting data.
  $upload_object =& new Uploaded_data($dartdata_form);
  if (!$upload_object->upload()) {
    return;
  }
  
  $dartdata = $upload_object->data;

  //
  // Ignore trailing blank lines.
  //
  $lines = count($dartdata) - 1;
  for ($l = $lines; $l > 0; $l--) {
    if (trim($dartdata[$l]) == '') {
      $lines--;
    } else {
      break;
    }
  }

  //
  // Ignore leading blank lines.
  //
  $firstline = -1;
  for ($l = 0; $l <= $lines; $l++) {
    if (trim($dartdata[$l]) != '') {
      $firstline = $l;
      break;
    }
  }

  if ($firstline == -1) {
    $dartdata_form->show('<p class="error">No data in uploaded file</p>');
    return;
  }
  
  //
  // Check the column headings.
  //
  switch ($darttype) {
  case DARTLOG:
    $headers = array(array('table' => '',          'col' => ''     , 'hdr' => 'name')
                     , array('table' => DARTINGS,  'col' => 'sname', 'hdr' => 'sname')
                     , array('table' => ''      ,  'col' => '', 'hdr' => 'sex')
                     , array('table' => DARTINGS,  'col' => 'date', 'hdr' => 'dartdate')
                     , array('table' => DARTINGS,  'col' => 'darttime', 'hdr' => 'darttime')
                     , array('table' => DARTINGS,  'col' => 'downtime', 'hdr' => 'downtime')
                     , array('table' => DARTINGS,  'col' => 'pickuptime', 'hdr' => 'pickuptime')
                     , array('table' => DARTINGS,  'col' => 'drug', 'hdr' => 'dartdrug')
                     , array('table' => 'anesths',                  'hdr' =>
                             array(array('col' => 'drug', 'hdr' => 'extra_anesth')
                                   , array('col' => 'antime', 'hdr' => 'extra_anesth_time')
                                   , array('col' => 'anamount', 'hdr' => 'extra_anesth_amt')))
                     , array('table' => DARTINGS,  'col' => 'logisticnotes', 'hdr' => 'other_notes')
                     , array('table' => DARTINGS,  'col' => 'dartcomments', 'hdr' => 'comments'));
    break;
  case DARTMORPHO:
    $headers = array(array('table' => '',          'col' => ''     , 'hdr' => 'name')
                     , array('table' => ''       , 'col' => 'sname', 'hdr' => 'sname')
                     , array('table' => ''       , 'col' => ''     , 'hdr' => 'sex')
                     , array('table' => ''       , 'col' => ''     , 'hdr' => 'dartdate')
                     , array('table' => DARTINGS , 'col' => 'mass' , 'hdr' => 'bodymass')
                     , array('table' => 'crownrumps'               , 'hdr' =>
                             array(array('col' => 'crlength', 'hdr' => 'crownrump')
                                   , array('col' => 'crobserver', 'hdr' => 'crobserver')))
                     , array('table' => 'chests'                   , 'hdr' =>
                             array(array('col' => 'chcircum', 'hdr' => 'chestcircum')
                                   , array('col' => 'chunadjusted', 'hdr' => 'unadj_chestcircum')
                                   , array('col' => 'chobserver', 'hdr' => 'chobserver')))
                     , array('table' => 'ulnas'                     , 'hdr' =>
                             array(array('col' => 'ullength', 'hdr' => 'ulna')
                                   , array('col' => 'ulunadjusted', 'hdr' => 'unadj_ulna')
                                   , array('col' => 'ulobserver', 'hdr' => 'ulobserver')))
                     , array('table' => 'humeruses'                 , 'hdr' =>
                             array(array('col' => 'hulength', 'hdr' => 'humerus')
                                   , array('col' => 'huunadjusted', 'hdr' => 'unadj_humerus')
                                   , array('col' => 'huobserver', 'hdr' => 'huobserver')))
                     , array('table' => DARTINGS, 'col' => 'crnotes', 'hdr' => 'crnotes')
                     , array('table' => DARTINGS, 'col' => 'chnotes', 'hdr' => 'chnotes')
                     , array('table' => DARTINGS, 'col' => 'ulnotes', 'hdr' => 'ulnotes')
                     , array('table' => DARTINGS, 'col' => 'hunotes', 'hdr' => 'hunotes'));
   break;
 case DARTPHYSIO:
    $headers = array(array('table' => '',          'col' => ''     , 'hdr' => 'name')
                     , array('table' => ''       , 'col' => 'sname', 'hdr' => 'sname')
                     , array('table' => ''       , 'col' => ''     , 'hdr' => 'sex')
                     , array('table' => ''       , 'col' => ''     , 'hdr' => 'dartdate')
                     , array('table' => 'pcvs'                     , 'hdr' =>
                             array(array('col' => 'pcv', 'hdr' => 'hematocrit')))
                     , array('table' => 'bodytemps'                , 'hdr' =>
                             array(array('col' => 'btemp', 'hdr' => 'bodytemp')
                                   , array('col' => 'bttime', 'hdr' => 'bodytemptime')))
                     , array('table' => 'dphys'  , 'col' => 'pulse', 'hdr' => 'pulse')
                     , array('table' => 'dphys'  , 'col' => 'respiration', 'hdr' => 'respiration')
                     , array('table' => 'dphys'  , 'col' => 'ringnode', 'hdr' => 'r_inguinal_lymph')
                     , array('table' => 'dphys'  , 'col' => 'lingnode', 'hdr' => 'l_inguinal_lymph')
                     , array('table' => 'dphys'  , 'col' => 'raxnode', 'hdr' => 'r_axillary_lymph')
                     , array('table' => 'dphys'  , 'col' => 'laxnode', 'hdr' => 'l_axillary_lymph')
                     , array('table' => 'dphys'  , 'col' => 'rsubmandnode', 'hdr' => 'r_submandibular_lymph')
                     , array('table' => 'dphys'  , 'col' => 'lsubmandnode', 'hdr' => 'l_submandibular_lymph')
                     , array('table' => DARTINGS , 'col' => 'dphysnotes', 'hdr' => 'other_notes_measures')
                     , array('table' => DARTINGS , 'col' => 'pcvnotes', 'hdr' => 'pcvnotes')
                     , array('table' => DARTINGS , 'col' => 'bodytempnotes', 'hdr' => 'bodytempnotes'));

   break;
 case DARTSAMP:
    $headers = array(array('table' => '',          'col' => ''     , 'hdr' => 'name')
                     , array('table' => ''       , 'col' => 'sname', 'hdr' => 'sname')
                     , array('table' => ''       , 'col' => ''     , 'hdr' => 'sex')
                     , array('table' => ''       , 'col' => ''     , 'hdr' => 'dartdate')
                     , array('table' => 'dsamples', 'col' => 'bloodpaxgene', 'hdr' => 'bloodinpaxgene')
                     , array('table' => 'dsamples', 'col' => 'bloodpurpletops', 'hdr' => 'bloodinpurpletops')
                     , array('table' => 'dsamples', 'col' => 'bloodseptube', 'hdr' => 'bloodinseparatortube')
                     , array('table' => 'dsamples', 'col' => 'skinpunch', 'hdr' => 'skin_punch_rnal')
                     , array('table' => 'dsamples', 'col' => 'mouthphotos3', 'hdr' => 'mouth_photos_3')
                     , array('table' => 'dsamples', 'col' => 'lmandmold', 'hdr' => 'l_mandible_tooth_cast')
                     , array('table' => 'dsamples', 'col' => 'lmaxillamold', 'hdr' => 'l_maxilla_tooth_cast')
                     , array('table' => 'dsamples', 'col' => 'lm1m2siliconemold', 'hdr' => 'l_m1m2_silicone_cast')
                     , array('table' => 'dsamples', 'col' => 'hairsamples', 'hdr' => 'hair_samples')
                     , array('table' => 'dsamples', 'col' => 'vaginalswab', 'hdr' => 'vaginalswab')
                     , array('table' => 'dsamples', 'col' => 'bloodsmear', 'hdr' => 'bloodsmear')
                     , array('table' => 'dtcultures', 'col' => 'dtcskin_taken', 'hdr' => 'tissue_culture_skin')
                     , array('table' => 'dtcultures', 'col' => 'dtcskin_count', 'hdr' => 'tcskin_count')
                     , array('table' => 'dtcultures', 'col' => 'dtcblood_taken', 'hdr' => 'tissue_culture_blood')
                     , array('table' => 'dtcultures', 'col' => 'dtcblood_count', 'hdr' => 'tc_blood_count')
                     , array('table' => DARTINGS , 'col' => 'dsamplenotes' , 'hdr' => 'other_notes_measures'));

   break;
 case DARTTEETH:
    $headers = array(array('table' => '',          'col' => ''     , 'hdr' => 'name')
                     , array('table' => ''       , 'col' => 'sname', 'hdr' => 'sname')
                     , array('table' => ''       , 'col' => ''     , 'hdr' => 'sex')
                     , array('table' => ''       , 'col' => ''     , 'hdr' => 'dartdate')
                     , array('table' => TEETH                      , 'hdr' =>
                             // Key is data file column header, value is db column name.
                             array('tstate' => 'tstate'
                                   , 'tcondition' => 'tcondition'))
                     , array('table' => DARTINGS , 'col' => 'teethnotes', 'hdr' => 'notes')
                     , array('table' => DARTINGS , 'col' => 'caninenotes', 'hdr' => 'canine_notes'));

   break;
 case DARTTESTESARC:
   foreach ($testes_headers as $a) {
     if ($a['table'] == 'testes') {
       $a['table'] = 'testes_arc';
     }
     $headers[] = $a;
   }

   break;
 case DARTTESTESDIAM:
   foreach ($testes_headers as $a) {
     if ($a['table'] == 'testes') {
       $a['table'] = 'testes_diam';
     }
     $headers[] = $a;
   }

   break;
  }

  $row = explode(DELIM, rtrim($dartdata[$firstline]));

  // While we check keep track of what data's in what column.
  $tables = array(); // Table to insert into, indexed by column
  $cols = array();   // Column name to insert into, indexed by column
  $instances = array(); // Instance of row, for repeating data -- starts with 1
  $hdrs = array();   // Headings in spreadsheet, indexed by column
  $toothcodes = array();  // Toothcode values to insert.

  $instance_by_table = array(); // Instances keyed per table
  $rowpos = 0;
  $rowcount = count($row);
  foreach ($headers as $k => $v) {
    $table = $v['table'];
    $hdr = $v['hdr'];
    if (is_array($hdr)) {
      if ($table == TEETH) {
        // Columns in the teeth table are prefaced by the toothcode.
        // Go through each column in the input file testing if it's a column on teeth.
        $last_toothcode = '';
        if (array_key_exists($table, $instance_by_table)) {
          $instance = $instance_by_table[$table];
        } else {
          $instance = 0;
        }
        while (TRUE) {
          $this_hdr = $row[$rowpos];
          $this_col = strrchr($this_hdr, '_');
          if ($this_col === FALSE) {
            // The data file column header is not prefaced with a toothcode.
            // It must not be a column on teeth.
            break;
          }
          $this_col = substr($this_col, 1);
          $toothcode = substr($this_hdr, 0, strlen($this_hdr) - strlen($this_col) - 1);

          if ($toothcode != $last_toothcode) {
            // New toothcode, and presumeably a new set of columns.
            $instance = $instance + 1;
            $instance_by_table[$table] = $instance;
            $last_toothcode = $toothcode;
          }
          
          if (!array_key_exists($this_col, $hdr)) {
            $dartdata_form->show('<p class="error">Bad column heading: "'
                                 . htmlentities($row[$rowpos], ENT_QUOTES)
                                 . '" not a TEETH column</p>');
            return;
          }
              
          $tables[$rowpos] = $table;
          $cols[$rowpos] = $hdr[$this_col];
          $instances[$rowpos] = $instance;
          $hdrs[$rowpos] = $this_hdr;
          $toothcodes[$rowpos] = $toothcode;

          $rowpos = $rowpos + 1;
          if ($rowpos >= $rowcount) {
            // No more columns in input file.
            break;
          }
        }
      } else {
        // A "numbered" set of columns.
        if (array_key_exists($table, $instance_by_table)) {
          $instance = $instance_by_table[$table] + 1;
        } else {
          $instance = 1;
        }
        $suffix = 1;
        while ($row[$rowpos] == $hdr[0]['hdr'] . $suffix) {
          $instance_by_table[$table] = $instance;
          foreach ($hdr as $repk => $repv) {
            $this_col = $repv['col'];
            $this_hdr = $repv['hdr'];

            if ($rowpos >= $rowcount) {
              $dartdata_form->show('<p class="error">Too few columns: "'
                                   . htmlentities($this_hdr . $suffix, ENT_QUOTES)
                                   . '" missing</p>');

              return;
            }
            
            if ($this_hdr . $suffix != $row[$rowpos]) {
            $dartdata_form->show('<p class="error">Bad column heading: "'
                                 . htmlentities($row[$rowpos], ENT_QUOTES)
                                 . '" is misspelled or out of order: expecting "'
                                 . $this_hdr . $suffix
                                 . '"</p>');
              return;
            }
            $tables[$rowpos] = $table;
            $cols[$rowpos] = $this_col;
            $instances[$rowpos] = $instance;
            $hdrs[$rowpos] = $this_hdr;

            $rowpos += 1;
          }
          $instance += 1;
          $suffix += 1;
        }
      }
    } else {
      $col = $v['col'];
      if ($rowpos >= $rowcount) {
        $dartdata_form->show('<p class="error">Too few columns: "'
                             . htmlentities($hdr, ENT_QUOTES)
                             . '" missing</p>');
        return;
      }
      if ($hdr != $row[$rowpos]) {
        $dartdata_form->show('<p class="error">Bad column heading: "'
                            . htmlentities($row[$rowpos], ENT_QUOTES)
                            . '" is misspelled or out of order: expecting "'
                            . $hdr
                            . '"</p>');
        return;
      }
      $tables[$rowpos] = $table;
      $cols[$rowpos] = $col;
      $instances[$rowpos] = 1;
      $instance_by_table[$table] = 1;  // Not worth optimizing this out.
      $hdrs[$rowpos] = $hdr;

      $rowpos += 1;
    }
  }

  if ($rowpos < $rowcount) {
    $dartdata_form->show('<p class="error">Too many columns, extras after: "'
                         . htmlentities($hdrs[$rowpos - 1], ENT_QUOTES)
                         . '"</p>');
    return;
  }

  $last_col = $rowcount - 1;
  $firstline += 1;

  //
  // Connect to the db.
  //
  if (!$dbconn->connect()) {
    return;
  }

  // Begin a transaction.
  if (!$dbconn->begin(TRUE)) {
    return;
  }
  $faulting = FALSE;

  //
  // Go through all the data and load it.
  // Iterate over the lines of the uploaded file.
  //

  $last_sname = '';
  $last_dartdate = '';
  for ($line = $firstline; $line <= $lines; $line++) {

    //
    // Parse the line.
    //

    // Ensure that we can dollar quote everything in the row.
    if (strpos($dartdata[$line], DQ) != 0) {
      $dartdata_form->error($line + 1
                           , DQ . ' character sequence found on the line'
                           , $faulting
                           , 'These characters cannot be used in input to this program');
      $faulting = TRUE;
      continue; // to the next line of the input file
    }

    // Break each line into words.
    $row = explode(DELIM, rtrim($dartdata[$line]));

    for ($i = count($row); $i <= $last_col; $i++) {
      // Make sure that there are as many columns as we might reference.
      $row[$i] = '';
    }

    //
    // Construct the rows.
    //
    $data = array();  // Contains one row per table
    $name = '';
    $sex = '';
    if ($darttype == DARTLOG) {
      // We don't get the dartdate "specially" when uploading logistic data.
      $dartdate = $row[DARTDATE_COL]; // cheat
    } else {
      $dartdate = '';
    }
    $sname = $row[SNAME_COL]; // Cheat
    for ($i = 0; $i <= $last_col; $i++) {
      if ($row[$i] !== '') {
        // Data exists (empty string means NULL)
        $table = $tables[$i];
        $col = $cols[$i];
        if ($col === '' | $table === '') {
          // Save data with no table for special processing.
          switch ($hdrs[$i]) {
          case 'name':
            $name = $row[$i];
            break;
          case 'sex':
            $sex = $row[$i];
            break;
          case 'dartdate':
            $dartdate = $row[$i];
            break;
          }
        } else {
          if (!array_key_exists($table, $data)) {
            // Table not seen yet.
            $data[$table] = array();  // Contains one row per repeating instance
          }
          $instance = $instances[$i];
          if (!array_key_exists($instance, $data[$table])) {
            // Instance not seen yet.
            $data[$table][$instance] = array(); // Contains one row per db column
            if ($table == TEETH) {
              $data[$table][$instance]['tooth'] = $toothcodes[$i];
            }
          }
          
          $data[$table][$instance][$col] = $row[$i];
        }
      }
    }

    //
    // Basic validation
    //
    if (!validate_darting($dbconn
                          , $dartdata_form
                          , $line
                          , $dartdate
                          , $sname
                          , $name
                          , $sex
                          , $dartid
                          , $darttype != DARTLOG
                          , $faulting)) {
      $faulting = TRUE;
      continue; // to the next line of the input file.
    }

    //
    // Save the data.
    //
    $update_darting = '';
    foreach ($data as $table => $tablecontent) {
      foreach ($tablecontent as $instance => $datarow) {
        if ($darttype != DARTLOG & $table == DARTINGS) {
          // The dartings row already exists.
          // Write a query that checks that we're not overwriting
          // non-null data.
          $check_darting = "SELECT 1 FROM dartings WHERE dartid = $dartid AND (";
          $or = '';
          // Write an UPDATE statement.
          $update_darting = 'UPDATE dartings SET ';
          $comma = '';
          foreach ($datarow as $colname => $v) {
            if ($colname != 'sname') {
              // We don't want to update the sname but it's always
              // present in the uploaded file so we can find the right row.
              $check_darting .= $or . "$colname IS NOT NULL";
              $or = ' OR ';
              $update_darting .= $comma . $colname . ' = ' . DQ . $v . DQ;
              $comma = ', ';
            }
          }
          $check_darting .= ');';
          $update_darting .= " WHERE dartid = $dartid;";
        } else {
          // The row must be inserted.
          $insert = 'INSERT INTO ' . $table . '(';
          if ($table !== DARTINGS) {
            // The table needs a dartid column
            $collist = 'dartid';
            $values = $dartid;
            $comma = ', ';
          } else {
            $collist = '';
            $values = '';
            $comma = '';
          };
          foreach ($datarow as $colname => $v) {
            $collist .= $comma . $colname;
            $values .= $comma . DQ . $v . DQ;
            $comma = ', ';
          }
          $insert .= $collist . ') values (' . $values . ');';

          /* debug */
          //error_log($insert);
        
          // Put the row into the db.
          if (!$dbconn->sql($insert)) {
            $dartdata_form->error($line + 1
                                  ,  htmlentities($dbconn->last_error())
                                  , $faulting);
            $faulting = TRUE;
            if ($darttype == DARTLOG & $table === DARTINGS) {
              // When the dartings table is broken don't bother
              // trying to insert other rows.
              //goto nextinputline; // I hate php
              continue 3; // a lot
            }
          }
        }

        if ($darttype == DARTLOG & $table === DARTINGS) {
          // Get the dartid for use with the other tables.
          $dartid = $dbconn->sql_result("SELECT currval('dartings_dartid_seq');");
          if ($dartid === FALSE or $dartid === 0) {
            $dartdata_form->error($line + 1
                                 ,  htmlentities($dbconn->last_error())
                                 , $faulting);
            $faulting = TRUE;
            // goto nextinputline; // I hate php
            continue 3; // a lot
          }
        }
      }
    }
        
    if ($update_darting !== '') {
      // Check to make sure that we won't overwrite anything when
      // updating the dartings row.
      $r = $dbconn->sql_result($check_darting);
      if ($r === FALSE) {
        // Database error.
        $dartdata_form->error($line + 1
                              , htmlentities($dbconn->last_error())
                              , $faulting);
        $faulting = TRUE;
        continue; // to the next input line
      }

      if ($r !== 0) {
        // Some rows were found on dartings with non-NULL column values 
        // that the update statement would overwrite.
        $dartdata_form->error($line + 1
                              , 'Dartid '
                                . $dartid
                                . ': Cannot overwrite Non-NULL data'
                              , $faulting
                              , 'Likely upload of duplicate data');
        $faulting = TRUE;
        continue; // to the next input line
      }

      // Change the darting row last, now that all the other data is in place.
      if (!$dbconn->sql($update_darting)) {
        $dartdata_form->error($line + 1
                              ,  htmlentities($dbconn->last_error())
                              , $faulting);
        $faulting = TRUE;
        continue; // to the next input line
      }
    }

    // Last check to make sure we didn't upload duplicate rows.
    if ($sname == $last_sname and $dartdate == $last_dartdate) {
      $dartdata_form->error($line +1
                            , 'Line has the same sname and dartdate as the previous line'
                            , $faulting);
      $faulting = TRUE;
      continue; // to the next input line
    } else {
      $last_sname = $sname;
      $last_dartdate = $dartdate;
    }

    //  nextinputline:
  }

  //
  // Commit the transaction (or allow user to try again.
  //

  /* debug */
  //$dbconn->rollback('<p>Transaction deliberately rolled back.</p>');
  //return;

  if ($faulting) {
    $dbconn->rollback('');
    return;
  }

  if (!$dbconn->complete()) {
    return;
  }

  //
  // Everything seems to have worked.
  // Allow the user to do it again.
  //
  $upload_object->success();  // Remember uploaded file.
  $dartdata_form->show('<p class="success">Darting data successfully updated!</p>');

  return;
}

//
// Body of code
//

maintain_login_session();

// $darttype  What sort of darting data are we uploading.
$darttype = get_session_var('darttype', TRUE);
switch ($darttype) {
 case DARTLOG:
   $dt_desc = 'logistic';
   break;
 case DARTMORPHO:
   $dt_desc = 'morphology';
   break;
 case DARTPHYSIO:
   $dt_desc = 'physiological';
   break;
 case DARTSAMP:
   $dt_desc = 'sample';
   break;
 case DARTTEETH:
   $dt_desc = 'teeth';
   break;
 case DARTTESTESARC:
   $dt_desc = 'testes circumference';
   break;
 case DARTTESTESDIAM:
   $dt_desc = 'testes diameter';
   break;
 case '':
   ?>
   <html>
   <head>
     <title>updart -- Internal error</title>
   </head>
     <body>
      <p>Sorry, there's a bug in the system and it's lost track of
         what you're trying to do.  You will have to start over.
      </p>
      <p>We will see about fixing this after upgrading the OS from
      RedHat EL 4.
      </p>
     </body>
   </html>
   <?php
   error_log('The updart program has received an empty darttype value');
   exit(1);
 default:
   ?>
   <html>
   <head>
     <title>updart -- Internal error</title>
   </head>
     <body>
      <p><?php print "'$darttype': Invalid darttype value."; ?>
      </p>
     </body>
   </html>
   <?php
   error_log("'$darttype': The updart program has received an invalid darttype value");
   exit(1);
}

show_html_start("updart -- Update darting $dt_desc data");

$dt_title = ucfirst($dt_desc);
print "<h1>Update Darting $dt_title Data</h1>";

//
// Get the form data.
//

// $action
// What page we're in.
if (array_key_exists('action', $_REQUEST)) {
  $action = $_REQUEST['action'];
} else {
  $action = ACTION_NONE;
}

// $db
$db = get_session_var('db', TRUE);

// $user
$user = get_session_var('user', TRUE);

// $password
$password = get_session_var('password', FALSE);

//error_log('p: ' . crypt($password, substr('12345678901234',0, CRYPT_SALT_LENGTH)) . ': ' .  ($password === ''));

// Instantiate the upload form.
$dartdata_form = new Upload_form($db, $user, $password, "Darting $dt_desc");

//
// Process input values
//

//
// Dispatch on action
//
switch ($action) {
  case ACTION_NONE:
    $dartdata_form->show();
    break;

  case ACTION_UPLOAD:
    // Make a database connection.
    $dbconn =& new Upload_connection($db
                                           , $user
                                           , $password
                                           , $dartdata_form);
    // Do the deed.
    upload_dartdata($dartdata_form, $dbconn, $darttype);
    break;
}

// Finish the stock page.

show_html_finish();

return;  // End of program.
