<?php
/* Copyright (C) 2016 The Meme Factory, Inc.  http://www.meme.com/
 * Copyright (C) 2006, 2007, 2008, 2011, Karl O. Pinc
 *
 *    This file is part of Babase.
 *
 *    Babase is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Web interface for (w)diff
 *
 * Bugs:
 *  Does not detect bad return codes from cmp.
 *
 * Karl O. Pinc <kop@meme.com>
 */

// Includes
require('constants.php.inc');
require('lib.php.inc');

// Constants

// dtype values -- what sort of diff to do.
define('D_TABLE_CHANGED', 'table_changed');
define('D_TABLE', 'table');
define('D_WORD', 'word');
define('D_LINE', 'line');
define('D_FILE', 'file');

define('DTYPE_DEFAULT', D_TABLE_CHANGED);  // default $dtype value
          
// Constants for converting wdiff markup to html
define('DELETED_MARKUP', '<em class="deleted">');
define('ADDED_MARKUP', '<em class="added">');
define('CLOSE_MARKUP', '</em>');
define('WD_PATTERN', '/(\[-.*-])|(\{\+.*\+\})/');
define('WD_DELETED_START', '[-');
define('WD_DELETED_END', '-]');
define('WD_ADDED_START', '{+');
define('WD_ADDED_END', '+}');
define('WD_DELETED_START_LEN', '2');
define('WD_DELETED_END_LEN', '2');
define('WD_ADDED_START_LEN', '2');
define('WD_ADDED_END_LEN', '2');
$oldmarks = array(WD_DELETED_START
                  , WD_DELETED_END
                  , WD_ADDED_START
                  , WD_ADDED_END);
$newmarks = array( DELETED_MARKUP
                  , CLOSE_MARKUP
                  , ADDED_MARKUP
                  , CLOSE_MARKUP);

// Some constants for table diffs
define('DF_SAME', 's');
define('DF_DELETED', 'd');
define('DF_ADDED', 'a');

//
// Functions.
// 

function move_diff_file($dtype, $from_path, $to_path) {
  /*
   * Move a file we will diff, converting eol sequences if necessary.
   *
   * Input:
   * $dtype  Type of diff we will do
   * $from_path  Pathname to move.
   * $to_path    Place to move to.
   *
   * Returns: True on success, false otherwise.
   *
   * Side Effects: moves file, fiddles eol
   *
   * Bugs:
   */

  if ($dtype == D_FILE) {
    return move_uploaded_file($from_path, $to_path);
  } else {
    if (!is_uploaded_file($from_path)) {
      return FALSE;
    }
    return move_and_fix_eol($from_path, $to_path) == 0;
  }
}

function textual($f, $dtype, $letter) {
  /*
   * See if a file is ASCII text.
   *
   * Input:
   * $f       Path to file.
   * $dtype   Type of comparsion being done.
   * $letter  Letter identifying file A or B.
   *
   * Returns: True when file is text.
   *
   * Side Effects: Outputs html when non-textual.
   *
   * Bugs: It's probably a bad idea to have side effects.
   */

   exec("file -b $f", $result, $err);
   reset($result);
   $line = current($result);

   if (preg_match('/( |^)[Tt]ext(,?|$)/', $line) != 1) {
     print '<p class="error"> File ' . $letter . ' cannot be compared by ';
     switch ($dtype) {
       case D_TABLE_CHANGED:
       case D_TABLE:
       case D_WORD:
         print 'word';
         break;

       case D_LINE:
         print 'line';
         break;
     };
     print " because it is not text.  File $letter is: ";
     print htmlentities($line, ENT_QUOTES);
     print '</p>';

     return FALSE;
   };

   return TRUE;
};

function compare_files($dtype) {
  /*
   * Compare two files (if any have been uploaded).
   *
   * Input:
   * $dtype  The type of comparison to do.
   *
   * Returns: TRUE when there is HTML output, FALSE otherwise.
   *
   * Side Effects: Outputs html.
   *
   * Bugs:
   */

    global $newmarks, $oldmarks;

    //
    // Get the rest of the form data.
    //

    // Filea
    if (array_key_exists('filea', $_FILES)) {
      $filea = htmlentities($_FILES['filea']['name'], ENT_QUOTES);
      $filea_tmp = $_FILES['filea']['tmp_name'];
      $filea_err = $_FILES['filea']['error'];
    } else {
      $filea = '';
      $filea_tmp = '';
    }

    // Fileb
    if (array_key_exists('fileb', $_FILES)) {
      $fileb = htmlentities($_FILES['fileb']['name'], ENT_QUOTES);
      $fileb_tmp = $_FILES['fileb']['tmp_name'];
      $fileb_err = $_FILES['fileb']['error'];
    } else {
      $fileb = '';
      $fileb_tmp = '';
    }

    //
    // Set about comparing the files.
    //

    if ($filea_tmp == '' && $fileb_tmp == '') {
      // We've not yet gotten uploads from the user so we
      // don't need to compare anything.
      return FALSE;
    }

    if ($filea_err != UPLOAD_ERR_OK
        || $fileb_err != UPLOAD_ERR_OK) {
      // Some sort of error uploading.
      print explain_error("File A [Old File] ($filea)", $filea_err);
      print explain_error("File B [New File] ($fileb)", $fileb_err);
      return TRUE;
    }

    //
    // Process input values
    //
    switch ($dtype) {
     case D_TABLE_CHANGED:
     case D_TABLE:
       $command = 'wdiff -n';
       break;

     case D_WORD:
       $command = 'wdiff -n';
       break;

     case D_LINE:
       $command = 'diff -u';
       break;

     case D_FILE:
       $command = 'cmp';
       break;
    }

    // Compare the two files.
    $tmpa = '/var/tmp/FileA.' . getmypid();
    $tmpb = '/var/tmp/FileB.' . getmypid();
    if (!move_diff_file($dtype, $filea_tmp, $tmpa)
        || !move_diff_file($dtype, $fileb_tmp, $tmpb)) {
      print '<p class="error">Unexpected failure trying to move uploaded files.</p>';
      return TRUE;
    }

    print '<p>';
    print "File A [Old File]: " . $filea . "<br />";
    print "File B [New File]: " . $fileb;
    print '</p>';

    if ($dtype == D_TABLE_CHANGED || $dtype == D_TABLE
        || $dtype == D_WORD || $dtype == D_LINE) {
      // Make sure the files are text and can be compared.
      $aistext = textual($tmpa, $dtype, 'A');
      $bistext = textual($tmpb, $dtype, 'B');
      if (!($aistext && $bistext)) {
        return TRUE;
      };
    };   
            
    exec("$command $tmpa $tmpb", $result, $err);
    unlink($tmpa);
    unlink($tmpb);

    if ($err == 0) {

      // The files are identical.
        
      if (trim($filea) == trim($fileb)) {
        // File names are identical too!
        print '<p><em class="error">Uh, the same file was uploaded twice....</em><br />';
        print '(Or you uploaded 2 files having identical names <em>and</em> content.)</p>';
        print '<hr />';
      } else {
        print '<p class="success">The files are identical!';
        print '<em class="remark">(You should be so proud!)</em></p>';
      }

      return TRUE;
    };

    // The files are not identical.
          
    //
    // Display the result
    //
    switch ($dtype) {
      case D_FILE:
        reset($result);
        $line = current($result);
        print '<p class="error">';
        print htmlentities($line);
        print '</p>';

        break;
              
      case D_TABLE_CHANGED:
      case D_TABLE:
        // Tabular by word output
              
        /* Debug
        print '<hr />';
        print '<pre>';
        foreach ($result as $line) {
          print "$line\n";
        }
        print '</pre>';
        reset($result);
        */ // Debug

        print '<hr />';
              
        // Go through the result and display it line by line 
        // in a fixed-width font, and colorize it.
        print "<table class=\"comparison\">\n";
              
        $lineno = 1;
        foreach ($result as $line) {
          // Start a new row in the table.
          if ($dtype == D_TABLE
              OR preg_match(WD_PATTERN, $line)) {
            print '<tr><td>Line'
              . str_repeat('&nbsp;', 5 - strlen($lineno))
              . "$lineno</td>";

            if ($line != '') {
              $td_close = '';
              $words = split('[[:space:]]'
                             , htmlentities($line, ENT_QUOTES));

              // Go through the words of the line and pair
              // the changes.
              $paired_words = array();
              $next_pwi = 0;          // pwi is "paired_words index"
              $max_pwi = $next_pwi;
              $wtype = DF_SAME;
              $next_wtype = $wtype;
              foreach ($words as $word) {
                // Look at the word and see if it begins
                // an add or delete sequence.
                // Strip off beginning of sequence marks.
                $prefix = substr($word, 0, WD_DELETED_START_LEN);
                if ($prefix == WD_DELETED_START) {
                  // Start of delete sequence.
                  $next_wtype = DF_DELETED;
                  $word = substr($word, WD_DELETED_START_LEN);
                  $pwi_saved = $next_pwi;
                } elseif ($prefix == WD_ADDED_START) {
                  // Start of add sequence.
                  if ($wtype == DF_DELETED) {
                    // Add sequence follows delete sequence.
                    // Rewind to pair adds with deletes.
                    $next_pwi = $pwi_saved;
                  }
                  $word = substr($word, WD_ADDED_START_LEN);
                  $next_wtype = DF_ADDED;
                }

                // Remember characteristics of current word and
                // prep for next iteration.
                // We do this so early because when the word
                // ends an add or delete sequence that affects
                // both the markup of of the next word and it's
                // column.
                $pwi = $next_pwi;
                $wtype = $next_wtype;

                $next_pwi++;
                if ($next_pwi > $max_pwi) {
                  // Remember how far we got before the adds
                  // "rewound" back over deletes.
                  $max_pwi = $next_pwi;
                }

                // See if the word ends an add or delete sequence.
                // Remember seqeuence for future.
                // Strip off end of sequence marks.
                $suffix = substr($word, strlen($word)
                                             - WD_DELETED_END_LEN);
                if ($suffix == WD_DELETED_END) {
                  // Finish delete sequence.
                  $word = substr($word, 0, strlen($word)
                                                     - WD_DELETED_END_LEN);
                  $next_wtype = DF_SAME;
                } elseif ($suffix == WD_ADDED_END) {
                  // Finish add sequence.
                  $word = substr($word
                                      , 0
                                      , strlen($word)
                                        - WD_ADDED_END_LEN);
                  $next_wtype = DF_SAME;
                  // Make sure that ending an add resynronizes
                  // and regular data is not paired with deletes.
                  $next_pwi = $max_pwi;
                }

                // Put html markup on the word.
                if ($wtype == DF_DELETED) {
                  // In a delete sequence.
                  $word = DELETED_MARKUP . $word . CLOSE_MARKUP;
                } elseif ($wtype == DF_ADDED) {
                  // In a add sequence.
                  $word = ADDED_MARKUP . $word . CLOSE_MARKUP;
                }
                    
                // Save the word.
                if (array_key_exists($pwi, $paired_words)) {
                  // There's already something in the pair.
                  $paired_words[$pwi] .= " $word";
                } else {
                  $paired_words[$pwi] = $word;
                }
              }

              // Output the line.
              foreach ($paired_words as $td) {
                print "<td class=\"comparison\">$td</td>\n";
              }
            }
            print "</tr>\n";
          }
          $lineno++;
        }
        print "</table>\n";
              
        break;

      case D_WORD:

        print '<hr />';
              
        print '<p class="comparison">';
              
        $lineno = 1;
        foreach ($result as $line) {
          print str_repeat('&nbsp;', 5 - strlen($lineno))
            . $lineno
            . ': '
            . str_replace($oldmarks
                          , $newmarks
                          , htmlentities($line, ENT_QUOTES))
            . '<br />';
          print "\n";
          $lineno++;
        }
        print '</p>';
              
        break;
              
      case D_LINE:

        print '<hr />';

        print '<pre>';
        $colored = FALSE;

        foreach ($result as $line) {
          $c = substr($line, 0, 1);
          switch ($c) {
            case '+':
              print '</pre><pre class="added">';
              $colored = TRUE;
              break;
            case '-':
              print '</pre><pre class="deleted">';
              $colored = TRUE;
              break;
            default:
              if ($colored) {
                print '</pre><pre>';
                $colored = FALSE;
              }
              break;
          }
          print htmlentities($line, ENT_QUOTES);
          print "\n";
        }
        print '</pre>';
              
        break;
    }
          
    if ($dtype != D_FILE) {
      ?>
      <p>Color Key:<br />
      <em class="deleted">In File A [Old File] but not in File B.</em><br />
       <em class="added">In File B [New File] but not in File A.</em><br />
      </p>
      <?php
    }

    //          break;
    //
    //        default:
    // Some error running wdiff.
    //          print '<p class="error">Unexpected failure in wdiff command.  Error: '
    //            . $err
    //            . '</p>';
    //        
    //          print '</p>';

    return TRUE;
};

//
// Body of code
//

?><?xml version="1.0" encoding="UTF-8" standalone="no"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
  <head>
    <meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
    <title>wwwdiff -- A web based diff program</title>
    <link rel="stylesheet" href="/babase.css" type="text/css" />
  </head>
  <body>

<p class="navbar">
<a href="<?php print HOME_TARGET; ?>">HOME</a>
</p>

<h1>Compare Two Files</h1>

<?php

// Get some of the form data.

// Type of diff
if (array_key_exists('dtype', $_REQUEST)) {
  $dtype = $_REQUEST['dtype'];
} else {
  $dtype = DTYPE_DEFAULT;
}

// Compare the files (assuming anything was uploaded).
if (compare_files($dtype)) {
  print '<hr />';
};

//
// Ouput a form where the user can upload files to compare.
?>

<form action="" enctype="multipart/form-data" method="post">

<p>
<label for="filea_id">File A [Old File]</label>
<input name="filea"
       tabindex="1"
       id="filea_id"
       type="file"
       size="30" />
</p>

<p>
<label for="fileb_id">File B [New File]</label>
<input name="fileb"
       tabindex="2"
       id="fileb_id"
       type="file"
       size="30" />
</p>

<?php
// The kind of diff to do.
?>

<p>
<input name="dtype"
       tabindex="10"
       id="dtype-table-changed"
       type="radio"
       value="<?php print D_TABLE_CHANGED; ?>"
       <?php if ($dtype == D_TABLE_CHANGED) {
               print 'checked="checked"'; } ?>
               />&nbsp;<label for="dtype-table">Tabular by Word
                                &ndash;&ndash;
                                Suppress identical lines</label><br />

<input name="dtype"
       tabindex="11"
       id="dtype-table"
       type="radio"
       value="<?php print D_TABLE; ?>"
       <?php if ($dtype == D_TABLE) {
               print 'checked="checked"'; } ?>
       />&nbsp;<label for="dtype-table">Tabular by Word</label><br />

<input name="dtype"
       tabindex="12"
       id="dtype-word"
       type="radio"
       value="<?php print D_WORD; ?>"
       <?php if ($dtype == D_WORD) {
               print 'checked="checked"'; } ?>
       />&nbsp;<label for="dtype-word">By Word</label><br />

<input name="dtype"
       tabindex="13"
       id="dtype-line"
       type="radio"
       value="<?php print D_LINE; ?>"
       <?php if ($dtype == D_LINE) {
               print 'checked="checked"'; } ?>
       />&nbsp;<label for="dtype-line">By Line</label><br />

<input name="dtype"
       tabindex="14"
       id="dtype-file"
       type="radio"
       value="<?php print D_FILE; ?>"
       <?php if ($dtype == D_FILE) {
               print 'checked="checked"'; } ?>
       />&nbsp;<label for="dtype-file">By File</label>
</p>

<p>
<input value="Diff" tabindex="20" type="submit" />
</p>
</form>



<p class="navbar">
<a href="<?php print HOME_TARGET; ?>">HOME</a>
</p>

</body>
</html>
