<?php
/* Copyright (C) 2006-2011, Karl O. Pinc
 *
 *    This file is part of Babase.
 *
 *    Babase is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Functions shared by programs that upload darting data.
 *
 * Karl O. Pinc <kop@meme.com>
 */

  function validate_darting(&$dbconn
                            , &$form
                            , $line
                            , $dartdate
                            , $sname
                            , $name
                            , $sex
                            , &$dartid
                            , $get_dartid
                            , $faulting) {
  /*
   * Validate the "basic" information that identifies the darting.
   *
   * Input:
   *   $dbconn  An instance of the Upload_connection class.
   *   $form    An instance of the Upload_form class.
   *   $line    Line number - 1 of the data in the uploaded file.
   *   $dartdate  The date of the darting.
   *   $sname   Sname of individual darted.
   *   $name    Name of darted individual.
   *   $sex     Sex of darted individual.
   *   $get_dartid  Boolean.  TRUE when $dartid should be found and
   *                assigned.
   *   $faulting  Boolean.  TRUE when an error has already occured.
   *
   * Updates:
   *   $dartid  Dartid of the darting row during which the data was
   *            collected.
   *
   * Returns:
   *  A boolean.  True when the row validates.  False when there's been
   *              an error.
   *
   * Side Effects:
   *   Sets $dartid when requested.
   *   Uses/alters $dbconn.
   *   Alters $form on error.
   *   Outputs html on error.
   *  
   * Remarks:
   * The validated data is included on every row of the uploaded file.
   */

    //
    // Basic validation.
    //

    // Check (sname and) name.
    $real_name = $dbconn->sql_result('SELECT biograph.name'
                                     . ' FROM biograph'
                                     . " WHERE biograph.sname = '$sname';");
    if ($real_name === 0 or $real_name === FALSE) {
      $form->error($line + 1
                           , "Invalid SNAME '$sname'"
                           , $faulting
                           , '(or database error)');
      return FALSE;
    }

    if (strtoupper($real_name) !== strtoupper($name)) {
      $form->error($line + 1
                           , 'Name mismatch'
                           , $faulting
                           , "'$name' found in data but the Name of '$sname' is '$real_name'");
      return FALSE;
    }

    // Check sex.
    $sex_found = $dbconn->sql_result('SELECT 1'
                                     . ' FROM biograph'
                                     . " WHERE biograph.sname = '$sname'"
                                           . " AND biograph.sex = '$sex';");
    if ($sex_found === 0 or $sex_found === FALSE) {
      $form->error($line + 1
                           , "$sex: Incorrect Sex"
                           , $faulting);
      return FALSE;
    }

    if ($get_dartid) {
      // Check that the dartings row exists and get the dartid.
      $dartid = $dbconn->sql_result('SELECT dartings.dartid'
                                    . ' FROM dartings'
                                    . " WHERE dartings.sname = '$sname'"
                                           . ' AND dartings.date'
                                               . " = '$dartdate';");
      if ($dartid === 0 or $dartid === FALSE) {
        $form->error($line + 1
                              , 'No such darting'
                              , $faulting
                              , "Sname '$sname': Date '$dartdate'");
        return FALSE;
      }
    }

    return TRUE;
  }
?>
