# Copyright (C) 2015 Domen Kozar https://www.domenkozar.com
#
#    This file is part of Babase.
#
#    Babase is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
#
# Domen Kozar <domen@dev.si>

import os
from uuid import uuid4

from pyramid.config import Configurator
from sqlalchemy import engine_from_config

from csvdump.sql import DBSession


HERE = os.path.dirname(__file__)


def api_include(config):  # pragma: no cover
    config.add_route('download', '/download')
    config.add_route('dropdowns', '/dropdowns')


def main(global_config, **settings):  # pragma: no cover
    """ This function returns a Pyramid WSGI application.
    """
    config = Configurator(settings=settings)
    config.add_route('root', '/')
    config.add_static_view(name='static', path=os.path.join(HERE, 'static'))
    config.include(api_include, route_prefix='/api/v1/')
    config.scan()

    engine = engine_from_config(settings, 'sqlalchemy.')
    DBSession.configure(bind=engine)

    def get_uuid(request):
        return str(uuid4())

    config.add_request_method(get_uuid, 'id', reify=True)

    return config.make_wsgi_app()
