# Copyright (C) 2015 Domen Kozar https://www.domenkozar.com
#
#    This file is part of Babase.
#
#    Babase is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
#
# Domen Kozar <domen@dev.si>

import io
import csv

from sqlalchemy.sql import text, select, and_
from sqlalchemy.orm import scoped_session, sessionmaker
from zope.sqlalchemy import ZopeTransactionExtension


DBSession = scoped_session(sessionmaker(
    extension=ZopeTransactionExtension(),
    autocommit=True,
))

QUERY1 = """
    select analysis.analysis_id -- for internal use
       , analysis.description as study_label
            -- for column 1 display (Study)
       , cvterm.name as data_type_label
            -- for column 2 related stuff (Data Type)
    from analysis
           join cvterm on (cvterm.cvterm_id = analysis.type_id)
     where -- There is a related file of analysis results stored in the fs
       (select count(*)
          from analysisprop
                 join cvterm
                      on (cvterm.cvterm_id = analysisprop.type_id)
                 join cv on (cv.cv_id = cvterm.cv_id)
          where analysisprop.analysis_id = analysis.analysis_id
                and cvterm.name = 'pathname'
                and cv.name = 'babase_chado_terms'
       ) = 1
       and exists -- only functional genomic analysis
           (select 1
              from analysis_nd_experiment
               where analysis_nd_experiment.analysis_id
                    = analysis.analysis_id)
    order by lower(analysis.description)
"""

# Table options:
# stock_id              | integer               |
# babid                 | character varying(10) |
# ageclass              | text                  |
# sex                   | character(1)          |
# birth                 | date                  |
# bstatus               | numeric(2,1)          |
# collection_date       | date                  |
# age_sample_collection | numeric(5,3)          |
# entrydate             | date                  |
# entrytype             | character(1)          |
# depart_date           | date                  |
# depart_type           | smallint              |
# mature_date           | date                  |
# mature_date_status    | character(1)          |
# mom_id                | character varying(10) |

QUERY3 = """
    select {}
    from stock_phenotypes
    where stock_id in :stock_ids
"""


def query2(engine, sex=None, ageclass=None):
    """
    :param sex iterable: list of sex values to filter on
    :param ageclass iterable: lits of ageclass values to filter on
    """
    where = []
    if sex:
        sex = tuple(sex)
        where.append(text('sex in :sex'))
    if ageclass:
        ageclass = tuple(ageclass)
        where.append(text('ageclass in :ageclass'))
    query = select([text('stock_id')]).where(and_(*where))\
                                      .select_from(text('stock_phenotypes'))
    return query_db(engine, query, sex=sex, ageclass=ageclass)


def query_db(engine, query, **params):  # pragma: no cover
    """Query DB and fetch results"""
    if isinstance(query, str):
        query = text(query)
    return engine.execute(query, params).fetchall()


def resultproxy_to_csv(engine, columns, **params):
    """Query DB for stock_phenotypes and render results into CSV format"""
    query = QUERY3.format(', '.join(columns))
    output = io.StringIO()
    outcsv = csv.writer(output, dialect='excel-tab')

    # query db
    cursor = engine.execute(text(query), params)

    # dump column titles
    outcsv.writerow(cursor.keys())

    # dump rows
    outcsv.writerows(cursor.fetchall())
    return output
