# Copyright (C) 2015 Domen Kozar https://www.domenkozar.com
#
#    This file is part of Babase.
#
#    Babase is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
#
# Domen Kozar <domen@dev.si>

import mock

from csvdump.sql import query2, resultproxy_to_csv


@mock.patch('csvdump.sql.query_db')
class Testquery2:
    def test_simple(self, mock_query_db):
        result = query2(mock.sentinel.engine)
        mock_query_db.assert_called_with(
            mock.sentinel.engine,
            mock.ANY,
            sex=None,
            ageclass=None,
        )
        query = mock_query_db.mock_calls[0][1][1]
        # assert no filtering
        assert 'WHERE' not in str(query)
        assert 'SELECT stock_id' in str(query)
        assert result == mock_query_db()

    def test_with_kwargs(self, mock_query_db):
        result = query2(
            mock.sentinel.engine,
            sex=[mock.sentinel.sex],
            ageclass=[mock.sentinel.ageclass],
        )
        mock_query_db.assert_called_with(
            mock.sentinel.engine,
            mock.ANY,
            sex=(mock.sentinel.sex,),
            ageclass=(mock.sentinel.ageclass,),
        )
        query = mock_query_db.mock_calls[0][1][1]
        # assert filtering
        assert 'WHERE' in str(query)
        assert 'SELECT stock_id' in str(query)
        assert result == mock_query_db()


class Testresultproxy_to_csv:
    def test_no_results(self):
        mock_engine = mock.Mock()
        mock_engine.execute().fetchall.return_value = []
        mock_engine.execute().keys.return_value = []

        output = resultproxy_to_csv(
            mock_engine,
            ['foo', 'bar'],
        )
        assert output.getvalue().strip() == ""

    def test_results(self):
        mock_engine = mock.Mock()
        mock_engine.execute().fetchall.return_value = [(1, 2), (3, 4)]
        mock_engine.execute().keys.return_value = [
            'foo',
            'bar',
        ]

        output = resultproxy_to_csv(
            mock_engine,
            ['foo', 'bar'],
        )
        assert output.getvalue().strip() == "{}\t{}\r\n1\t2\r\n3\t4".format('foo', 'bar')
