# Copyright (C) 2015 The Meme Factory, Inc.  http://www.meme.com/
#
#    This file is part of Babase.
#
#    Babase is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
#
# Karl O. Pinc <kop@meme.com>

from pyramid.config import Configurator
from sqlalchemy import engine_from_config
import configparser

from .models import (
    DBSession,
    Base,
    )

def main(global_config, **settings):
    """ This function returns a Pyramid WSGI application.
    """
    engine = engine_from_config(settings, 'sqlalchemy.')
    DBSession.configure(bind=engine)
    Base.metadata.bind = engine
    
    # Get all the extra config sections
    # Make them dicts in a `chadodb` dict
    full_config = configparser.ConfigParser()
    full_config.read(global_config['__file__'])
    chadodbmap={}    
    for sec in full_config.sections():
        section_parts = sec.split(':')
        if section_parts[0] == 'chadodbmap':
            label = section_parts[1]
            chadodbmap[label] = {}
            for key in full_config[sec]:
                chadodbmap[label][key] = full_config[sec][key]
    settings['chadodbmap'] = chadodbmap

    config = Configurator(settings=settings)
    config.include('pyramid_mako')
    config.add_static_view(
        settings['media_location'], 'static', cache_max_age=3600)
    config.add_route('home', '/')
    config.add_route('vcfdl', '/vcfdl')
    config.scan()
    return config.make_wsgi_app()
