/* Copyright (C) 2015 Steve Piercy - Website Builder
 * http://www.stevepiercy.com/
 *
 *    This file is part of Babase.
 *
 *    Babase is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with Babase.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Steve Piercy
 */
var path = require('path');
var webpack = require('webpack');
var HtmlwebpackPlugin = require('html-webpack-plugin');
var merge = require('webpack-merge');
var Clean = require('clean-webpack-plugin');
var ExtractTextPlugin = require('extract-text-webpack-plugin');

var pkg = require('./package.json');

var TARGET = process.env.TARGET;
var ROOT_PATH = path.resolve(__dirname);

var common = {
  entry: path.resolve(ROOT_PATH, 'src/main'),
  resolve: {
    extensions: ['', '.js', '.jsx']
  },
  output: {
    path: path.resolve(ROOT_PATH, 'build'),
    filename: 'bundle.js'
  },
  plugins: [
    new HtmlwebpackPlugin({
      title: 'Amboseli Baboon Research Project - Genomics Data Search',
      template: 'src/index.html',
      inject: 'body'
    }),
    new HtmlwebpackPlugin({
      template: '../pyramid/VCFDump/vcfdump/templates/base.mako',
      inject: 'body',
      filename: 'base.mako'
    })
  ]
};

if(TARGET === 'start' || !TARGET) {
  module.exports = merge(common, {
    devtool: 'eval',
    module: {
      loaders: [
        {
          test: /\.css$/,
          loaders: ['style', 'css']
        },
        {
          test: /\.jsx?$/,
          loaders: ['react-hot', 'babel?stage=1'],
          include: path.resolve(ROOT_PATH, 'src')
        },
        {
          test: /\.less$/,
          loader: 'style!css!less',
          include: path.resolve(ROOT_PATH, 'src')
        }, {
          test: /\.(woff|woff2|ttf|eot|svg)(\?v=[0-9]\.[0-9]\.[0-9])?$/,
          loader: 'file?name=fonts/[name].[ext]'
        }
      ]
    },
    devServer: {
      colors: true,
      historyApiFallback: true,
      hot: true,
      inline: true,
      progress: true
    },
    plugins: [
      new webpack.HotModuleReplacementPlugin()
    ]
  });
}

if(TARGET === 'build') {
  module.exports = merge(common, {
    entry: {
      app: path.resolve(ROOT_PATH, 'src/main'),
      vendor: path.resolve(ROOT_PATH, 'src/vendor.js')
    },
    output: {
      path: path.resolve(ROOT_PATH, 'build'),
      filename: 'app.[chunkhash].js'
    },
    devtool: 'source-map',
    module: {
      loaders: [
        {
          test: /\.(js|jsx)$/,
          loaders: ['babel?stage=1'],
          include: path.resolve(ROOT_PATH, 'src')
        }, {
          test: /\.less$/,
          loader: ExtractTextPlugin.extract(
            // activate source maps via loader query
            'style',
            'css?sourceMap!' +
            'less?sourceMap'),
          include: path.resolve(ROOT_PATH, 'src')
        }, {
          test: /\.(woff|woff2|ttf|eot|svg)(\?v=[0-9]\.[0-9]\.[0-9])?$/,
          loader: 'file?name=fonts/[name].[ext]'
        }
      ]
    },
    plugins: [
      new ExtractTextPlugin('styles.css'),
      new Clean(['build']),
      new webpack.ProvidePlugin({
        $: 'jquery',
        jQuery: 'jquery'
      }),
      new webpack.optimize.CommonsChunkPlugin(
        'vendor',
        'vendor.[chunkhash].js'
      ),
//       TODO - for merging with development cycle in Phase 2
//       new webpack.DefinePlugin({
//         'process.env': {
//           // This affects react lib size
//           'NODE_ENV': JSON.stringify('production')
//         }
//       }),
      new webpack.optimize.UglifyJsPlugin({
        compress: {
          warnings: false
        }
      })
    ]
  });
}
