#!/bin/sh
# Copyright (C) 2015 The Meme Factory, Inc.  http://www.meme.com/
#
#   This file is part of Babase.
#
#   Babase is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with Babase.  If not, see <http://www.gnu.org/licenses/>.
#
# Karl O. Pinc <kop@meme.com>
#
# Syntax: receive-abrp-genomics-schema dbname
#
# Input:
#   dbname  Name of db which is the source of the data
#
# Remarks:
#   Receives a pg_dump -Fc of the schema on stdin.
#   See docbook docs for more info.
#
# Bugs:

include(`constants.m4')
include(`macros.m4')
editwarning()

# Parse command line.
export db="$1"

if [ "$db" = 'bb_live_db' ] ; then
  export targetdb=abrp_genomics
else
  export targetdb=abrp_genomics_test
fi

logger -t "$(basename $0)" -p bb_syslog_facility.INFO \
       "Starting replacement of abrp_genomics schema content in db $targetdb"

psql -q -U babase_admin $targetdb <<EOF

\set ON_ERROR_STOP
set client_min_messages to warning;

drop schema if exists abrp_genomics_new cascade;
EOF

pg_restore -U babase_admin -d $targetdb
export result=$?
if [ $result -ne 0 ] ; then
    logger -t "$(basename $0)" -p bb_syslog_facility.ERR -s \
           "Cannot restore abrp_genomics schema in db $targetdb"
    exit $result
fi

psql -q -U babase_admin $targetdb <<EOF

\set ON_ERROR_STOP
set client_min_messages to warning;

begin;
drop schema if exists abrp_genomics cascade;
alter schema abrp_genomics_new rename to abrp_genomics;
commit;
EOF
export result=$?
if [ $result -ne 0 ] ; then
    logger -t "$(basename $0)" -p bb_syslog_facility.ERR -s \
           "Cannot replace old abrp_genomics schema with new in db $targetdb"
    exit $result
fi

logger -t "$(basename $0)" -p bb_syslog_facility.INFO \
       "Replaced abrp_genomics schema content in db $targetdb"
